// maAudioPA.c

/*
 * MoreAmp PortAudio interface
 * Copyright (C) 2004-2014 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include "maConfig.h" // included by all moreamp files

#ifdef __cplusplus
extern "C"
{
#endif

#include "maAudioPA.h"

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>

#if defined(__MAOSX__)
 #include <CoreServices/CoreServices.h>
 #include <AudioUnit/AudioUnit.h>
 AudioDeviceID g_osxDevID = kAudioDeviceUnknown;
#endif

#include "portaudio.h"
#include "maPlayer.h"
#include "maUtils.h"
#include "maEqualizer.h"

int maPaInitialize();
int maPaTerminate();

// static void maPAFinishedCallbackOut(void *userData);

static int maPACallbackOut(const void *inputBuffer, void *outputBuffer,
						unsigned long framesPerBuffer,
						const PaStreamCallbackTimeInfo* timeInfo,
						PaStreamCallbackFlags statusFlags,
						void *userData);

static int maPACallbackIn(const void *inputBuffer, void *outputBuffer,
						unsigned long framesPerBuffer,
						const PaStreamCallbackTimeInfo* timeInfo,
						PaStreamCallbackFlags statusFlags,
						void *userData);

static int maPACallbackInOut(const void *inputBuffer, void *outputBuffer,
						unsigned long framesPerBuffer,
						const PaStreamCallbackTimeInfo* timeInfo,
						PaStreamCallbackFlags statusFlags,
						void *userData);

int l_pavolumer;
int l_pavolumel;
int l_insamplerate = 44100; // default
int l_paisinited = 0;
// int l_pafinished = 0;

PaStream *l_PaStreamOut = NULL;
PaStream *l_PaStreamIn = NULL;
PaStream *l_PaStreamInOut = NULL;

maFifo g_pafifoout;
maFifo *g_ppafifoout = NULL;;
char g_pafifooutbuf[PAFIFOOUTBUFSIZE];

#if !defined(NODECODEAUDIOIN)
 maFifo g_pafifoin;
 maFifo *g_ppafifoin = NULL;
 char g_pafifoinbuf[PAFIFOINBUFSIZE];
#endif

// frame buffer
char l_frameout[PAMAXFRAMESOUT * PABYTESPERFRAME];

maaudiohost **g_paudiohost;
int g_numaudiohosts = 0;
PaHostApiIndex g_audiohostin = 0;
PaHostApiIndex g_audiohostout = 0;
PaDeviceIndex g_audiodevicein = 0;
PaDeviceIndex g_audiodeviceout = 0;


int maPaInitialize()
{
	PaError err;
	machar mastr[MAXLOGLINELEN];
	machar mastr2[MAXLOGLINELEN];

	if(l_paisinited) // already done
		return 1;

	if((err = Pa_Initialize()) != paNoError)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maPaInitialize Pa_Initialize failed, %ls",
										os2ma(mastr2, (char *)Pa_GetErrorText(err)));
		maLog(mastr);
		return 1;
	}

	l_paisinited = 1;	

	return 0;

} // maPaInitialize


int maPaTerminate()
{
	PaError err;
	machar mastr2[MAXLOGLINELEN];
	machar mastr[MAXLOGLINELEN];

	if(l_paisinited)
	{
		if((err = Pa_Terminate()) != paNoError)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maPaTerminate failed, %ls",
									os2ma(mastr2, (char *)Pa_GetErrorText(err)));
			maLog(mastr);
			return 1;
		}
		l_paisinited = 0;
	}

	maFreeAudioHosts();

	return 0;

} // maPaTerminate


int maOpenPAStreamOut(PaHostApiIndex host, PaDeviceIndex device)
{
	void *pauserdata = NULL;
	const PaDeviceInfo* outputdeviceinfo;
	PaStreamParameters outputParameters;
	PaError err;
	machar mastr2[MAXLOGLINELEN];
	machar mastr[MAXLOGLINELEN];

    // we may want to close and reopen
	if(g_state.infiletype == kMAMAA || g_sets.fileout[kOUTSOUND].mode || l_insamplerate != g_state.insamplerate)
    {
#if !defined(NODECODEAUDIOIN)
		maClosePAStreamInOut();
#endif
		maClosePAStreamOut(1, 1); // wait and force
	}
    else
        if(l_PaStreamOut)
            return 0; // leave it open
	
	memset(l_frameout, 0, PAMAXFRAMESOUT * PABYTESPERFRAME);

	if((outputParameters.device = g_paudiohost[host]->paudiodevice[device]->devindex) < 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maOpenPAStreamOut bad device index host=%d device=%d",
																			host, device);
		maLog(mastr);
		return 1;
	}

	outputdeviceinfo = Pa_GetDeviceInfo(outputParameters.device);
	if((outputParameters.channelCount = outputdeviceinfo->maxOutputChannels) > 2)
		outputParameters.channelCount = 2; // basic stereo, not 5.1 or such
	outputParameters.sampleFormat = paInt16;
	outputParameters.suggestedLatency = outputdeviceinfo->defaultLowOutputLatency;
 	outputParameters.hostApiSpecificStreamInfo = NULL;

	g_state.outchannels = outputParameters.channelCount;
	g_state.outsamplerate = g_state.insamplerate;

	if((err = Pa_OpenStream(&l_PaStreamOut, NULL, &outputParameters, g_state.insamplerate,
			PAFRAMESPERBUFFEROUT, // paFramesPerBufferUnspecified,
			paClipOff | paDitherOff, maPACallbackOut, pauserdata)) != paNoError)
	{
		l_PaStreamOut = NULL;
		masprintf(mastr, MAXLOGLINELEN, L"maOpenPAStreamOut Pa_OpenStream failed, %ls",
										os2ma(mastr2, (char *)Pa_GetErrorText(err)));
		maLog(mastr);
		return 1;
	}

	/*
	if((err = Pa_SetStreamFinishedCallback(l_PaStreamOut, maPAFinishedCallbackOut)) != paNoError)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maOpenPAStreamOut Pa_SetStreamFinishedCallback failed, %ls",
										os2ma(mastr2, (char *)Pa_GetErrorText(err)));
		maLog(mastr);
	}
	*/

	l_insamplerate = g_state.insamplerate;
	
	g_state.iostreamopened = 1;
	
	return 0;

} // maOpenPAStreamOut


void maStopPAStreams()
{
	PaError err;
	machar mastr2[MAXLOGLINELEN];
	machar mastr[MAXLOGLINELEN];
	
	if(l_PaStreamOut && Pa_IsStreamActive(l_PaStreamOut) == 1)
		if((err = Pa_StopStream(l_PaStreamOut)) != paNoError)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maStopPAStreams Pa_StopStream Out failed, %ls",
								os2ma(mastr2, (char *)Pa_GetErrorText(err)));
			maLog(mastr);
		}

#if !defined(NODECODEAUDIOIN)
	if(l_PaStreamIn && Pa_IsStreamActive(l_PaStreamIn) == 1)
		if((err = Pa_StopStream(l_PaStreamIn)) != paNoError)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maStopPAStreams Pa_StopStream In failed, %ls",
								os2ma(mastr2, (char *)Pa_GetErrorText(err)));
			maLog(mastr);
		}

	if(l_PaStreamInOut && Pa_IsStreamActive(l_PaStreamInOut) == 1)
		if((err = Pa_StopStream(l_PaStreamInOut)) != paNoError)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maStopPAStreams Pa_StopStream InOut failed, %ls",
								os2ma(mastr2, (char *)Pa_GetErrorText(err)));
			maLog(mastr);
		}
#endif

} // maStopPAStreams


void maStartPAStreams()
{
	PaError err;
	machar mastr2[MAXLOGLINELEN];
	machar mastr[MAXLOGLINELEN];
	
	if(l_PaStreamOut && Pa_IsStreamActive(l_PaStreamOut) != 1)
		if((err = Pa_StartStream(l_PaStreamOut)) != paNoError)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maStartPAStreams Pa_StartStream failed, %ls",
								os2ma(mastr2, (char *)Pa_GetErrorText(err)));
			maLog(mastr);
		}

#if !defined(NODECODEAUDIOIN)
	if(l_PaStreamIn && Pa_IsStreamActive(l_PaStreamIn) != 1)
		if((err = Pa_StartStream(l_PaStreamIn)) != paNoError)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maStartPAStreams Pa_StartStream In failed, %ls",
								os2ma(mastr2, (char *)Pa_GetErrorText(err)));
			maLog(mastr);
		}

	if(l_PaStreamInOut && Pa_IsStreamActive(l_PaStreamInOut) != 1)
		if((err = Pa_StartStream(l_PaStreamInOut)) != paNoError)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maStartPAStreams Pa_StartStream InOut failed, %ls",
								os2ma(mastr2, (char *)Pa_GetErrorText(err)));
			maLog(mastr);
		}
#endif

} // maStartPAStreams


void maClosePAStreamOut(int wait, int force)
{
	PaError err;
	machar mastr2[MAXLOGLINELEN];
	machar mastr[MAXLOGLINELEN];

	if(!l_PaStreamOut)
		return;

	// we may not want to close it
	if(!force && g_state.infiletype != kMAMAA && g_sets.fileout[kOUTSOUND].mode == 0  && l_insamplerate == g_state.insamplerate)
		return;

	if(wait && Pa_IsStreamActive(l_PaStreamOut) == 1)
	{
		// l_pafinished = 0;
		if((err = Pa_StopStream(l_PaStreamOut)) != paNoError)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maClosePAStreamOut Pa_StopStream failed, %ls",
								os2ma(mastr2, (char *)Pa_GetErrorText(err)));
			maLog(mastr);
		}
		else // Pa_StopStream seems to return early, before all sound has played
			 // while(!l_pafinished)	// using maPAFinishedCallbackOut doesn't help,
			 //		maSleep(1);			// nor does returning paComplete from maPACallbackOut
			maSleep(60);				// resort to this	
	}

	if((err = Pa_CloseStream(l_PaStreamOut)) != paNoError)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maClosePAStreamOut Pa_CloseStream failed, %ls",
								os2ma(mastr2, (char *)Pa_GetErrorText(err)));
		maLog(mastr);
	}

	l_PaStreamOut = NULL;

} // maClosePAStreamOut


#if !defined(NODECODEAUDIOIN)
 int maOpenPAStreamIn()
 {
	void *pauserdata = NULL;
	const PaDeviceInfo* inputdeviceinfo;
	PaStreamParameters inputParameters;
	PaError err;
	machar mastr2[MAXLOGLINELEN];
	machar mastr[MAXLOGLINELEN];

    // we may want to close and reopen
    if(g_state.infiletype == kMAMAA || g_sets.fileout[kOUTSOUND].mode)
    {
 		maClosePAStreamIn();
		maClosePAStreamInOut();
		maClosePAStreamOut(0, 1); // don't wait, force
    }
    else
        if(l_PaStreamIn)
            return 1; // leave it open - we should never get here because we only use input with kMAMAA

	if((inputParameters.device = g_paudiohost[g_audiohostin]->paudiodevice[g_audiodevicein]->devindex) < 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maOpenPAStreamIn bad device index host=%d device=%d",
																		g_audiohostin, g_audiodevicein);
		maLog(mastr);
		return 1;
	}

	inputdeviceinfo = Pa_GetDeviceInfo(inputParameters.device);
	if((inputParameters.channelCount = inputdeviceinfo->maxInputChannels) > 2)
		inputParameters.channelCount = 2;	
	inputParameters.sampleFormat = paInt16;
	inputParameters.suggestedLatency = inputdeviceinfo->defaultLowInputLatency;
	inputParameters.hostApiSpecificStreamInfo = NULL;

    g_state.inchannels = inputParameters.channelCount;
	g_state.insamplerate = (int)inputdeviceinfo->defaultSampleRate;
	g_state.ininterleaved = 1;

	if((err = Pa_OpenStream(&l_PaStreamIn, &inputParameters, NULL, g_state.insamplerate,
				PAFRAMESPERBUFFERIN, // paFramesPerBufferUnspecified,
				paClipOff | paDitherOff, maPACallbackIn, pauserdata)) != paNoError)
	{
		l_PaStreamIn = NULL;
		masprintf(mastr, MAXLOGLINELEN, L"maOpenPAStreamIn Pa_OpenStream failed, %ls",
										os2ma(mastr2, (char *)Pa_GetErrorText(err)));
		maLog(mastr);
		return 1;
	}

	l_insamplerate = g_state.insamplerate;
	
	g_state.iostreamopened = 1;
	
	return 0;
	
 } // maOpenPAStreamIn


 void maClosePAStreamIn()
 {
	PaError err;
	machar mastr2[MAXLOGLINELEN];
	machar mastr[MAXLOGLINELEN];

	if(!l_PaStreamIn)
		return;

	if((err = Pa_CloseStream(l_PaStreamIn)) != paNoError)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maClosePAStreamIn Pa_CloseStream failed, %ls",
								os2ma(mastr2, (char *)Pa_GetErrorText(err)));
		maLog(mastr);
	}

	l_PaStreamIn = NULL;
	return;

 } // maClosePAStreamIn


 int maOpenPAStreamInOut(PaHostApiIndex host, PaDeviceIndex devicein, PaDeviceIndex deviceout)
 {
	void *pauserdata = NULL;
	const PaDeviceInfo* inputdeviceinfo;
	const PaDeviceInfo* outputdeviceinfo;
	PaStreamParameters inputParameters;
	PaStreamParameters outputParameters;
	PaError err;
	machar mastr2[MAXLOGLINELEN];
	machar mastr[MAXLOGLINELEN];

    // we may want to close and reopen
    if(g_state.infiletype == kMAMAA || g_sets.fileout[kOUTSOUND].mode)
    {
		maClosePAStreamIn();
		maClosePAStreamInOut();
		maClosePAStreamOut(0, 1); // don't wait, force
    }
    else
		if(l_PaStreamInOut)
			return 1; // leave it open - we should never get here because we only use input with kMAMAA

	if((inputParameters.device = g_paudiohost[host]->paudiodevice[devicein]->devindex) < 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maOpenPAStreamInOut bad input device index host=%d device=%d",
																			host, devicein);
		maLog(mastr);
		return 1;
	}
 
	inputdeviceinfo = Pa_GetDeviceInfo(inputParameters.device);
	if((inputParameters.channelCount = inputdeviceinfo->maxInputChannels) > 2)
		inputParameters.channelCount = 2;	
	inputParameters.sampleFormat = paInt16;
	inputParameters.suggestedLatency = inputdeviceinfo->defaultLowInputLatency;
	inputParameters.hostApiSpecificStreamInfo = NULL;

    g_state.inchannels = inputParameters.channelCount;
	g_state.insamplerate = (int)inputdeviceinfo->defaultSampleRate;
	g_state.ininterleaved = 1;

	memset(l_frameout, 0, PAMAXFRAMESOUT * PABYTESPERFRAME);

	if((outputParameters.device = g_paudiohost[host]->paudiodevice[deviceout]->devindex) < 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maOpenPAStreamInOut bad output device index host=%d device=%d",
																			host, deviceout);
		maLog(mastr);
		return 1;
	}

	outputdeviceinfo = Pa_GetDeviceInfo(outputParameters.device);
	if((outputParameters.channelCount = outputdeviceinfo->maxOutputChannels) > 2)
		outputParameters.channelCount = 2; // basic stereo, not 5.1 or such
	outputParameters.sampleFormat = paInt16;
	outputParameters.suggestedLatency = outputdeviceinfo->defaultLowOutputLatency;
 	outputParameters.hostApiSpecificStreamInfo = NULL;

	g_state.outchannels = outputParameters.channelCount;
	g_state.outsamplerate = g_state.insamplerate;

	if((err = Pa_OpenStream(&l_PaStreamInOut, &inputParameters, &outputParameters,
				g_state.insamplerate, // input sample rate, output gets the same rate
				PAFRAMESPERBUFFERINOUT, // paFramesPerBufferUnspecified,
				paClipOff | paDitherOff, maPACallbackInOut, pauserdata)) != paNoError)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maOpenPAStreamInOut Pa_OpenStream failed, %ls",
										os2ma(mastr2, (char *)Pa_GetErrorText(err)));
		maLog(mastr);
		l_PaStreamInOut = NULL;
		return 1;
	}

	l_insamplerate = g_state.insamplerate;
	
	g_state.iostreamopened = 1;
	
	return 0;

 } // maOpenPAStreamInOut


 void maClosePAStreamInOut()
 {
	PaError err;
	machar mastr2[MAXLOGLINELEN];
	machar mastr[MAXLOGLINELEN];

	if(!l_PaStreamInOut)
		return;

	if((err = Pa_CloseStream(l_PaStreamInOut)) != paNoError)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maClosePAStreamInOut Pa_CloseStream failed, %ls",
								os2ma(mastr2, (char *)Pa_GetErrorText(err)));
		maLog(mastr);
	}

	l_PaStreamInOut = NULL;

	return;

 } // maClosePAStreamInOut
#endif


void maOutputSetVolume(int volumeL, int volumeR)
{
	// this sets up integer multipliers used in ReduceVolume()
	l_pavolumer = volumeR;
	l_pavolumel = volumeL;

} // maOutputSetVolume


/*
static void maPAFinishedCallbackOut(void *userData)
{
	(void)userData;

	l_pafinished = 1;

} // streamFinishedCallback
*/


static int maPACallbackOut(	const void *inputBuffer,
							void *outputBuffer,
							unsigned long framesPerBuffer,
							const PaStreamCallbackTimeInfo* timeInfo,
							PaStreamCallbackFlags statusFlags,
							void *userData)
{
	unsigned long frames;
	unsigned long bytes;
	unsigned long bytesin;
	short *p, *r;
	unsigned long i;

	(void)inputBuffer; (void)timeInfo; (void)statusFlags; (void)userData;

	if(outputBuffer)
	{
		if(g_ppafifoout == NULL)
		{
			// maLog(L"maPACallbackOut null g_ppafifoout");
			g_state.stop |= STOP_ALL;
			return paContinue;
		}

		if((frames = framesPerBuffer) > PAMAXFRAMESOUT)
			frames = PAMAXFRAMESOUT;
			
		bytesin = PABYTESPERFRAME * frames;

		if((bytes = maFifoGetFilledSpace(g_ppafifoout)) < bytesin)
		{
			if(bytes > 0)
			{
				maFifoDraw(g_ppafifoout, l_frameout, bytes);
				MAFIFOSEMRELEASE(g_ppafifoout->semaphore);
				memset(l_frameout + bytes, 0, bytesin - bytes); // pad with silence
			}
			else // if(bytes <= 0)
			{
				memset(outputBuffer, 0, bytesin); // silence
				goto mapacallbackoutcontinue; // paComplete
			}
		}
		else
		{
			maFifoDraw(g_ppafifoout, l_frameout, bytesin);
			MAFIFOSEMRELEASE(g_ppafifoout->semaphore);
		}

		// immediate silence
		if((g_state.stop & STOP_ALL) || g_state.silence)
		{
			memset(outputBuffer, 0, bytesin);
			goto mapacallbackoutcontinue;
		}

		p = (short *)outputBuffer;
		r = (short *)&l_frameout;
	
		// we do the volume control here
		if(l_pavolumer == kVOLMAX && l_pavolumel == kVOLMAX) // no change in volume
			memcpy((char *)p,(char *)r, bytesin);
		else if(l_pavolumer == kVOLMAX) // reduce left volume only
		{
			for(i = 0; i < frames; i++)
			{
				*p++ = (short)((l_pavolumel * *r++) >> kVOLEXP);
				*p++ = *r++;
			}
		}
		else if(l_pavolumel == kVOLMAX) // reduce right volume only
		{
			for(i = 0; i < frames; i++)
			{
				*p++ = *r++;
				*p++ = (short)((l_pavolumer * *r++) >> kVOLEXP);
			}
		}
		else // reduce both left and right volumes
		{
			for(i = 0; i < frames; i++)
			{
				*p++ = (short)((l_pavolumel * *r++) >> kVOLEXP);
				*p++ = (short)((l_pavolumer * *r++) >> kVOLEXP);
			}
		}
		
	} // if outputBuffer
	
mapacallbackoutcontinue:
	return paContinue;

} // maPACallbackOut


#if !defined(NODECODEAUDIOIN)
 static int maPACallbackIn(	const void *inputBuffer,
							void *outputBuffer,
							unsigned long framesPerBuffer,
							const PaStreamCallbackTimeInfo* timeInfo,
							PaStreamCallbackFlags statusFlags,
							void *userData)
 {
	unsigned long frames;
	unsigned long bytes;
	unsigned long bytesin;

	(void)outputBuffer; (void)timeInfo; (void)statusFlags; (void)userData;

	if(g_ppafifoin == NULL)
	{
		// maLog(L"maPACallbackIn null g_ppafifoin");
		g_state.stop |= STOP_ALL;
		return paContinue;
	}

	if((frames = framesPerBuffer) > PAMAXFRAMESIN)
		frames = PAMAXFRAMESIN;
	else
		frames = framesPerBuffer;

	if(g_state.inchannels == 1)
		bytesin = PABYTESPERFRAMEMONO * frames;
	else
		bytesin = PABYTESPERFRAME * frames;
        
	if((bytes = maFifoGetFreeSpace(g_ppafifoin)) < bytesin) // do the best you can
	{
		if(bytes > 0)
		{
			maFifoStore(g_ppafifoin, (char *)inputBuffer, bytes);
			MAFIFOSEMRELEASE(g_ppafifoin->semaphore);
			return paContinue;
		}
	}
	else
		maFifoStore(g_ppafifoin, (char *)inputBuffer, bytesin);

	MAFIFOSEMRELEASE(g_ppafifoin->semaphore);
	return paContinue;

 } // maPACallbackIn


  static int maPACallbackInOut(const void *inputBuffer, void *outputBuffer,
						unsigned long framesPerBuffer,
						const PaStreamCallbackTimeInfo* timeInfo,
						PaStreamCallbackFlags statusFlags,
						void *userData)
 {
	int result;

	if(framesPerBuffer <= 0)
		return paContinue;

	if(inputBuffer)
		if((result = maPACallbackIn(inputBuffer, NULL,
						framesPerBuffer,
						NULL,
						statusFlags,
						userData)) != paContinue)
			return result;

	if(outputBuffer)
	{
		result = maPACallbackOut(NULL, outputBuffer,
						framesPerBuffer,
						timeInfo,
						statusFlags,
						userData);
		return result;
	}
	
	return paContinue;
	
 } // maPACallbackInOut
#endif


int maGetAudioInputIndex(int index)
{
	PaHostApiIndex host;
	PaDeviceIndex device;

	for(host = 0; host < g_numaudiohosts; host++)
		for(device = 0; device < g_paudiohost[host]->info->deviceCount; device++)
			if(g_paudiohost[host]->paudiodevice[device]->info->maxInputChannels > 0)
				if(g_paudiohost[host]->paudiodevice[device]->index == index)
					return device;

	return -1;

} // maGetAudioInputIndex


int maGetAudioOutputIndex(int index)
{
	PaHostApiIndex host;
	PaDeviceIndex device;

	for(host = 0; host < g_numaudiohosts; host++)
		for(device = 0; device < g_paudiohost[host]->info->deviceCount; device++)
			if(g_paudiohost[host]->paudiodevice[device]->info->maxOutputChannels > 0)
				if(g_paudiohost[host]->paudiodevice[device]->index == index)
					return device;

	return -1;

} // maGetAudioOutputIndex


int maFreeAudioHosts()
{
	PaHostApiIndex host;
	PaDeviceIndex device;

	if(g_paudiohost)
	{
		for(host = 0; host < g_numaudiohosts; host++)
	 	{
			for(device = 0; device < g_paudiohost[host]->ndevices; device++)
				if(g_paudiohost[host]->paudiodevice[device])
				{
					MAFREE(g_paudiohost[host]->paudiodevice[device]);
					g_paudiohost[host]->paudiodevice[device] = NULL;
				}

			if(g_paudiohost[host]->paudiodevice)
			{
				MAFREE(g_paudiohost[host]->paudiodevice);
				g_paudiohost[host]->paudiodevice = NULL;
			}

			if(g_paudiohost[host])
			{
				MAFREE(g_paudiohost[host]);
				g_paudiohost[host] = NULL;
			}
		}
		if(g_paudiohost)
		{
			MAFREE(g_paudiohost);
			g_paudiohost = NULL;
		}
	}

	g_numaudiohosts = 0;

	return 0;

} // maFreeAudioHosts


int maScanAudioDevices()
{
	PaHostApiIndex host;
	PaDeviceIndex device;
	int count;

	// restart portaudio to update the device list
	maPaTerminate(); // calls maFreeAudioHosts() -> g_numaudiohosts = 0;
	maPaInitialize();

	// get the audio hosts
	if((g_numaudiohosts = Pa_GetHostApiCount()) <= 0)
	{
		maLog(L"maScanAudioDevices failed on Pa_GetHostApiCount");
		maPaTerminate();
		return 0;
	}

	if((g_paudiohost = (maaudiohost **)MACALLOC(1, g_numaudiohosts * sizeof(maaudiohost *))) == NULL)
	{
		maLog(L"maScanAudioDevices MACALLOC failed");
		maPaTerminate();
		return 0;
	}

	count = 0;
	for (host = 0; host < g_numaudiohosts; host++)
	{
		if((g_paudiohost[host] = (maaudiohost *)MACALLOC(1, sizeof(maaudiohost))) == NULL)
		{
			maLog(L"maScanAudioDevices MACALLOC failed");
			maPaTerminate(); // maScanAudioDevices fail
			return 0;
		}

		if((g_paudiohost[host]->info = Pa_GetHostApiInfo(host)) == NULL)
		{
			maLog(L"maScanAudioDevices Pa_GetHostApiInfo failed");
			continue;
		}

		if((g_paudiohost[host]->ndevices = g_paudiohost[host]->info->deviceCount) > 0)
		{
			if((g_paudiohost[host]->paudiodevice = (maaudiodevice **)MACALLOC(1, g_paudiohost[host]->info->deviceCount * sizeof(maaudiodevice *))) == NULL)
			{
				maLog(L"maScanAudioDevices MACALLOC failed");
				maPaTerminate(); // maScanAudioDevices fail
				return 0;
			}

			for (device = 0; device < g_paudiohost[host]->info->deviceCount; device++)
			{
				if((g_paudiohost[host]->paudiodevice[device] = (maaudiodevice *)MACALLOC(1, sizeof(maaudiodevice))) == NULL)
				{
					maLog(L"maScanAudioDevices MACALLOC failed");
					maPaTerminate(); // maScanAudioDevices fail
					return 0;
				}

				g_paudiohost[host]->paudiodevice[device]->info = Pa_GetDeviceInfo(device);
				g_paudiohost[host]->paudiodevice[device]->index = -1;
				g_paudiohost[host]->paudiodevice[device]->devindex = Pa_HostApiDeviceIndexToDeviceIndex(host, device);

				count++;
			}
		}
 	}

	return count;

} // maScanAudioDevices


int maRescanAudioDevices(PaHostApiIndex hostin, PaHostApiIndex hostout)
{
	int lhost;
	int numdevices;


	if((numdevices = maScanAudioDevices()) <= 0) // failed or no devices
	{
		if(hostin >= 0)
		{
			g_audiohostin = 0;
			g_sets.audiohostinname[0] = 0;
		}
		if(hostout >= 0)
		{
			g_audiohostout = 0;
			g_sets.audiohostoutname[0] = 0;
		}
	}
	else
	{
		if(hostin >= 0)
  		{
          if((lhost = hostin) >= g_numaudiohosts)
                if((lhost = g_numaudiohosts - 1) < 0)
                    lhost = 0;
  			g_audiohostin = lhost;
			strcpy(g_sets.audiohostinname, g_paudiohost[lhost]->info->name);
		}
		if(hostout >= 0)
  		{
          if((lhost = hostout) >= g_numaudiohosts)
                if((lhost = g_numaudiohosts - 1) < 0)
                    lhost = 0;
  			g_audiohostout = lhost;
			strcpy(g_sets.audiohostoutname, g_paudiohost[lhost]->info->name);
		}
	}

	return numdevices;

} // maRescanAudioDevices


int maSetAudioInputDevice() // we expect to have updated g_audio* and g_sets before calling this
{
	PaHostApiIndex host;
	PaDeviceIndex device;
	int inindex; // 0-based index for popup

	host = g_audiohostin;
	
	// find the popup index of the input device name that matches the name saved in the settings
	for(device = 0, inindex = -1; device < g_paudiohost[host]->info->deviceCount; device++)
	{
		if(g_paudiohost[host]->paudiodevice[device]->info->maxInputChannels > 0)
		{
			inindex++;										// this device has an input
			if(!strcmp(g_paudiohost[host]->paudiodevice[device]->info->name, g_sets.audiodevinname))
				break;										// and it's the one in the settings
		}
	}
	if(inindex >= 0)										// we found inputs
	{
		if(device < g_paudiohost[host]->info->deviceCount)	// and a match in settings
			g_audiodevicein = device;
		else												// else
		{
			inindex = g_audiodevicein = 0;					// default to 0
			for(device = 0; device < g_paudiohost[host]->info->deviceCount; device++)
				if(g_paudiohost[host]->paudiodevice[device]->info->maxInputChannels > 0)
				{											// take the first input device name found 	
					strcpy(g_sets.audiodevinname, g_paudiohost[host]->paudiodevice[device]->info->name);
					g_audiodevicein = device;
					break;
				}
		}			
	}

	return inindex;

} // maSetAudioInputDevice


int maSetAudioOutputDevice() // we expect to have updated g_audio* and g_sets before calling this
{
	PaHostApiIndex host;
	PaDeviceIndex device;
	int outindex; // 0-based index for popup

	host = g_audiohostout;

	// find the popup index of the input device name that matches the name saved in the settings
	for(device = 0, outindex = -1; device < g_paudiohost[host]->info->deviceCount; device++)
	{
		if(g_paudiohost[host]->paudiodevice[device]->info->maxOutputChannels > 0)
		{
			outindex++;										// this device has an output
			if(!strcmp(g_paudiohost[host]->paudiodevice[device]->info->name, g_sets.audiodevoutname))
				break;										// and it's the one in the settings
		}
	}
	if(outindex >= 0)										// we found outputs
	{
		if(device < g_paudiohost[host]->info->deviceCount)	// and a match in settings
			g_audiodeviceout = device;
		else												// else
		{
			outindex = g_audiodeviceout = 0;				// default to 0
			for(device = 0; device < g_paudiohost[host]->info->deviceCount; device++)
				if(g_paudiohost[host]->paudiodevice[device]->info->maxOutputChannels > 0)
				{											// take the first output device name found 	
					strcpy(g_sets.audiodevoutname, g_paudiohost[host]->paudiodevice[device]->info->name);
					g_audiodeviceout = device;
					break;
				}
		}			
	}

	return outindex;

} // maSetAudioOutputDevice


#ifdef __cplusplus
}
#endif
// eof
