// maCdaUnix.c

/*
 * MoreAmp cd reading for Unix
 * Copyright (C) 2004-2014 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

// this code contains extracts from cdparanoia :
/*
cdparanoia release III alpha 9.8
cdparanoia (C) 1998 Monty <xiphmont@mit.edu>
Monty <monty@xiph.org>
http://www.xiph.org/paranoia/
 */

#include "maConfig.h" // included by all moreamp files

#if defined(__MAUNIX__) && !defined(MACDAOSX)

#ifndef MANOCDA

#ifdef __cplusplus
extern "C" {
#endif

#define MALOGcda(x) maLog(x)
//#define CDATRACE(x) maLog(x)

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <math.h>
#include <sys/ioctl.h>
#include <sys/stat.h>

 #if defined(USE_CDPARANOIA_SYSTEM)
  //#warning unix cda player with cdparanoia (system)
  #include "cdda_interface.h"
  #include "cdda_paranoia.h"
 #else
  //#warning unix cda player with cdparanoia
  #include "../libs/cdparanoia/interface/cdda_interface.h"
  #include "../libs/cdparanoia/paranoia/cdda_paranoia.h"
 #endif

 // compat for ioctl eject op
 #if defined(HAVE_LINUX_CDROM_H)
  #include <linux/cdrom.h>
 #elif defined(HAVE_LINUX_UCDROM_H)
  #include <linux/ucdrom.h>
 #elif defined(HAVE_SYS_CDIO_H)
  #include <sys/cdio.h>
 #elif defined(HAVE_IO_CAM_CDIO_H)
  #include <io/cam/cdrom.h>
 #endif
 #if defined(CDROMEJECT)
  #define MACDROMEJECT CDROMEJECT
 #elif defined(CDAUDIO_EJECT)
  #define MACDROMEJECT CDAUDIO_EJECT
 //#elif defined(...)
  //#define MACDROMEJECT ...
 #else
  #if !defined(__MACUNIX__)
   #warning defining MACDROMEJECT=0x5309
  #endif
  #define MACDROMEJECT 0x5309
 #endif

#include "maCda.h"
#include "maPlayer.h"
#include "maUtils.h"

// globals :
cdadrive g_cdadrive[CDAMAXNUMDRIVES + 1];	// info about each cd drive
int g_cdanumdrives = 0;						// highest index of drives opened
int g_cdaactive = 0;						// 1-based index of active drive
char *g_cdabuf = NULL;						// pointer to cd pcm bytes buffer
int g_cdabufblocks = 8; // max for cdparanoia ?	// MAFIFOBUFSIZE / CDABLOCKSIZE; // 11, size of cd pcm bytes buffer in blocks

#define MASTATIC
int CdaOpen(int drive);
int CdaClose(int drive);
MASTATIC void CdaFinish(void);
MASTATIC cdrom_drive *g_d=NULL;
MASTATIC cdrom_paranoia *g_p=NULL;


int CdaOpen(int drive)
{
	machar mastr[MAXLOGLINELEN];

#ifdef CDATRACE
 masprintf(mastr, MAXLOGLINELEN, L"CdaOpen_ drive=%d", drive);
 CDATRACE(mastr);
#endif

	if(g_d)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaOpen drive=%d already open", drive);
		MALOGcda(mastr);
		return 1;
	}

	// reset our drive table entries for this drive
	g_cdadrive[drive].exists = 0;
	g_cdadrive[drive].isready = 0;
	g_cdadrive[drive].isactive = 0;
	g_cdadrive[drive].tracklast = 0;
	fsstrcpy(g_cdadrive[drive].name, g_setscdrom.cdaname[drive]);

	// a name for the drive must have been set, e.g. /dev/cdrom
	if(!g_cdadrive[drive].name[0] || g_cdadrive[drive].name[0] == '(') // name not set
		return 1;
	
#if defined(__MACUNIX__) // if we have osx cdparanoia
	g_d = cdda_find_a_cdrom(0, NULL);
#else
	// find the drive by name
	g_d = cdda_identify(g_cdadrive[drive].name, 0, NULL);
#endif

	if(!g_d)
	{
#ifdef CDATRACE
		masprintf(mastr, MAXLOGLINELEN, L"CdaOpen cda_identify found no drive %d '%ls'", drive, fs2ma(mastr2, g_cdadrive[drive].name));
		CDATRACE(mastr);
#endif
		return 1;
	}

#ifdef CDATRACE
 masprintf(mastr, MAXLOGLINELEN, L"CdaOpen cda_identify found drive %d '%ls'", drive, fs2ma(mastr2, g_cdadrive[drive].name));
 CDATRACE(mastr);
#endif

	// try to open the drive, must have an audio cd in it
	switch(cdda_open(g_d))
	{
		case 0: // ok
		break;

		case -2:case -3:case -4:case -5:
			// maLog(L"Unable to open disc. Is there an audio CD in the drive?");

		case -6:
			// maLog(L"Cdparanoia could not find a way to read audio from this drive.");

		default:
			// maLog(L"Unable to open disc.");

			CdaFinish();
#ifdef CDATRACE
			masprintf(mastr, MAXLOGLINELEN, L"CdaOpen cdda_open found no audio cd in  drive %d '%ls'",
								 drive, fs2ma(mastr2, g_cdadrive[drive].name));
			CDATRACE(mastr);
#endif
			return 1;
	}

	// drive is opened

#ifdef CDATRACE
 masprintf(mastr, MAXLOGLINELEN, L"CdaOpen found a CD in drive %d '%ls'", drive, fs2ma(mastr2, g_cdadrive[drive].name));
 CDATRACE(mastr);
#endif
			
	g_cdadrive[drive].exists = 1;
	g_cdadrive[drive].isready = 1;
	g_cdadrive[drive].tracklast = g_d->tracks;

	return 0;

} // CdaOpen


int CdaClose(int drive)
{
	machar mastr[MAXLOGLINELEN];

#ifdef CDATRACE
	masprintf(mastr, MAXLOGLINELEN, L"CdaClose_ drive=%d", drive);
	CDATRACE(mastr);
#endif

	if(!g_d)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaClose drive=%d not open", drive);
		MALOGcda(mastr);
	}

	CdaFinish();

#ifdef CDATRACE
 CDATRACE("");
#endif

	return 0;

} // CdaClose


int CdaGetTrackStartEnd(int drive)
{
	machar mastr[MAXLOGLINELEN];

#ifdef CDATRACE
 masprintf(mastr, MAXLOGLINELEN, L"CdaGetTrackStartEnd_ drive=%d track=%d", drive, g_cdadrive[1].track);
 CDATRACE(mastr);
#endif

	if(drive < 1 || drive > CDAMAXNUMDRIVES)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaGetTrackStartEnd bad drive index=%d", drive);
		MALOGcda(mastr);
		return 1;
	}

	if(CdaOpen(drive))
	{
		MALOGcda(L"CdaGetTrackStartEnd CdaOpen failed");
		return 1;
	}

	g_cdadrive[drive].lbafirst = cdda_track_firstsector(g_d, g_cdadrive[drive].track);

	g_cdadrive[drive].lbalast = cdda_track_lastsector(g_d, g_cdadrive[drive].track);

#ifdef CDATRACE
 masprintf(mastr, MAXLOGLINELEN, L"CdaGetTrackStartEnd drive=%d track=%d first=%d", drive, 
				g_cdadrive[drive].track, g_cdadrive[drive].lbafirst);
 CDATRACE(mastr);
 masprintf(mastr, MAXLOGLINELEN, L"CdaGetTrackStartEnd drive=%d track=%d last=%d", drive, 
				g_cdadrive[drive].track, g_cdadrive[drive].lbalast);
 CDATRACE(mastr);
#endif
	
	if(CdaClose(drive))
	{
		MALOGcda(L"CdaGetTrackStartEnd CdaClose failed");
		return 1;
	}

	return 0;
	
} // CdaGetTrackStartEnd


int CdaReadTrack(int drive, long *pbytesread)
{
	long actualRead = 0;
	machar mastr[MAXLOGLINELEN];
	
	*pbytesread = 0;
	
	if(drive < 1 || drive > CDAMAXNUMDRIVES)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaReadTrack bad drive index=%d", drive);
		MALOGcda(mastr);
		return 1;
	}
	
	if(!g_cdabuf)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaReadTrack drive=%d NULL buf (lba=%d)", drive, g_cdadrive[drive].lba);
		MALOGcda(mastr);
		return 1;
	}

	if(!g_d)
	{
#ifdef CDATRACE
		CDATRACE("CdaReadTrack_ CdaOpen");
#endif
		if(CdaOpen(drive))
		{
			MALOGcda(L"CdaReadTrack CdaOpen failed");
			return 1;
		}
	}

	if(g_setscdrom.cdparanoia)
	{
		if(!g_p)
		{
#ifdef CDATRACE
			CDATRACE("CdaReadTrack_ CdaOpen (cdparanoia)");
#endif
			g_p = paranoia_init(g_d);

			if(!g_p)
			{
				MALOGcda(L"CdaReadTrack paranoia_init failed");
				return 1;
			}

				long first_sector = cdda_track_firstsector(g_d, g_cdadrive[drive].track);
				paranoia_seek(g_p, first_sector, SEEK_SET);			

				// full paranoia, but allow skipping
				paranoia_modeset(g_p, PARANOIA_MODE_FULL^PARANOIA_MODE_NEVERSKIP);
			}

			int b = g_cdabufblocks;
			char *c = g_cdabuf;

			while(b-- >0)
			{
				short *r = paranoia_read(g_p, NULL);

			if(r <= 0)
			{
				masprintf(mastr, MAXLOGLINELEN, L"CdaReadTrack paranoia_read failed drive=%d lba=%d lbasperread=%d",
					drive, g_cdadrive[drive].lba, g_cdabufblocks);
				MALOGcda(mastr);
				return 1;
			}
			else
			{
				memcpy(c, r, CDABLOCKSIZE);
				*pbytesread += CDABLOCKSIZE;
				c += CDABLOCKSIZE;
			}

		} // while

	} // g_setscdrom.cdparanoia
	else
	{
		actualRead = cdda_read(g_d, g_cdabuf, g_cdadrive[drive].lba, g_cdabufblocks);
		
		if(actualRead != g_cdabufblocks)
		{
			masprintf(mastr, MAXLOGLINELEN, L"CdaReadTrack drive=%d lba=%d lbasperread=%d actual=%ld",
					drive, g_cdadrive[drive].lba, g_cdabufblocks, actualRead);
			MALOGcda(mastr);
		}
			
		*pbytesread = g_cdabufblocks * CDABLOCKSIZE;

	} // !g_setscdrom.cdparanoia

		
	return 0;
	
} // CdaReadTrack


int CdaGetCdInfo(int drive, char *StrtTrk, char *EndTrk)
{
	int num;
	machar mastr[MAXLOGLINELEN];

#ifdef CDATRACE
 CDATRACE("CdaGetGetCDInfo_");
#endif
	
	if(drive < 1 || drive > CDAMAXNUMDRIVES)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaGetCdInfo bad drive index=%d", drive);
		MALOGcda(mastr);
		return 1;
	}
	
	if(!StrtTrk)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaGetCdInfo drive=%d null StrtTrk", drive);
		MALOGcda(mastr);
		return 1;
	}
	
	if(!EndTrk)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaGetCdInfo drive=%d null EndTrk", drive);
		MALOGcda(mastr);
		return 1;
	}

	if(CdaOpen(drive))
	{
		MALOGcda(L"CdaGetCdInfo CdaOpen failed");
		return 1;
	}

	if((num = g_cdadrive[drive].tracklast) <= 0)
	{
		MALOGcda(L"CdaGetCdInfo no tracks");
	}
	else
	{
		*StrtTrk = 1;
		*EndTrk = (machar)num;
	}

	if(CdaClose(drive))
	{
		MALOGcda(L"CdaGetCdInfo CdaClose failed");
		return 1;
	}

	return 0;
	
} // CdaGetCdInfo


int CdaSetSpindleSpeed(int drive, int speed)
{
	machar mastr[MAXLOGLINELEN];

	if(drive < 1 || drive > CDAMAXNUMDRIVES)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaSetSpindleSpeed bad drive index=%d speed=%d", drive, speed);
		MALOGcda(mastr);
		return 1;
	}
	
	// do nothing on unix ?
	{
#ifdef CDATRACE
		masprintf(mastr, MAXLOGLINELEN, L"CdaSetSpindleSpeed drive=%d speed=%d", drive, speed);
		CDATRACE(mastr);
#endif
	}
	return 0;
	
} // CdaSetSpindleSpeed



int CdaSetBlockSize(int drive, int blocksize)
{
	machar mastr[MAXLOGLINELEN];

	if(drive < 1 || drive > CDAMAXNUMDRIVES)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaSetBlockSize bad drive index=%d blocksize=%d", drive, blocksize);
		MALOGcda(mastr);
		return 1;
	}
	
	// do nothing on unix ?
	{
#ifdef CDATRACE
		masprintf(mastr, MAXLOGLINELEN, L"CdaSetBlockSize drive=%d blocksize=%d", drive, blocksize);
		CDATRACE(mastr);
#endif
	}

	return 0;
	
} // CdaSetBlockSize


int CdaCleanup(int drive)
{
	machar mastr[MAXLOGLINELEN];

#ifdef CDATRACE
 CDATRACE("CdaCleanup_");
#endif

	if(drive < 1 || drive > CDAMAXNUMDRIVES)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaCleanup bad drive index=%d", drive);
		MALOGcda(mastr);
		return 1;
	}
	
	// KillIO
	// ...
	
	/*
	if(CdaSetBlockSize(g_cdadrive[drive].driverRefNum, 512))
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaCleanup CdaSetBlockSize 512 failed";
		MALOGcda(mastr);
	}
	*/

	// GoodBye
	// ...

	//CR_DeInit();

	if(g_d)
	{
		if(CdaClose(drive))
		{
			MALOGcda(L"CdaCleanup CdaClose failed");
			return 1;
		}
	}

	return 0;
	
} // CdaCleanup


int CdaEject(int drive)
{
  int ret;
	machar mastr[MAXLOGLINELEN];

#ifdef CDATRACE
 CDATRACE("CdaEject_");
#endif

	if(drive < 1 || drive > CDAMAXNUMDRIVES)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaEject bad drive index=%d", drive);
		MALOGcda(mastr);
		return 1;
	}
	
	if(CdaOpen(drive))
	{
		MALOGcda(L"CdaEject CdaOpen failed");
		return 1;
	}

	if(!g_d || !g_d->ioctl_fd)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaEject d or ioctl null, drive=%d '%ls'",
					drive, g_cdadrive[drive].name);
		MALOGcda(mastr);
	}

	if((ret = ioctl(g_d->ioctl_fd, MACDROMEJECT, NULL)))
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaEject ioctl CDROEJECT ret=%d failed errno=%d drive=%d '%ls'",
					ret, errno, drive, g_cdadrive[drive].name);
		MALOGcda(mastr);
	}

	if(CdaClose(drive))
	{
		MALOGcda(L"CdaEject CdaClose failed");
		return 1;
	}

	return 0;

} // CdaEject


int CdaInitCDRom()
{
	int drive;

#ifdef CDATRACE
 CDATRACE("CdaInitCDRom");
#endif

	g_cdaactive = 0; // should this be here ?
	g_cdanumdrives = 0;

	for(drive = 1; drive <= CDAMAXNUMDRIVES; drive++)
	{
		g_cdadrive[drive].track = 0; // should this be here ?

		if(!CdaOpen(drive))
		{
			g_cdanumdrives = drive; // index of highest drive
		 	CdaClose(drive);
		}
	}

	if(g_cdanumdrives < 1)
	{
#ifdef CDATRACE
		CDATRACE("CdaInitCDRom no audio cd found");
#endif
		return 1;
	}

	return 0;	

} // CdaInitCDRom


MASTATIC void CdaFinish()
{
#ifdef CDATRACE
 CDATRACE("CdaFinish_");
#endif

	if(g_d)
	{
		cdda_close(g_d);
		g_d = NULL;
	}

	if(g_p)
	{
		paranoia_free(g_p);
		g_p = NULL;
	}

} // CdaFinish

#ifdef __cplusplus
}
#endif

#endif // MANOCDA

#endif // __MAUNIX__ // && !__MACUNIX__

//eof
