// maCdaWin.cpp

/*
 * MoreAmp cd reading for win32
 * Copyright (C) 2004-2014 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include "maConfig.h" // included by all moreamp files

#ifndef MANOCDA

#include <windows.h>
#include "../libs/libcdread/CDRip.h"

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "maCda.h"
#include "maPlayer.h"
#include "maUtils.h"

typedef int SInt16;

int IsAudioMediaPresent();
int GetNumTracks();
int GetTrackStartTime(const int nTrack);
int GetNextCDDriveLetter(int drive);
int CdaSetActiveDriveLetter(int driveletter);

cdadrive g_cdadrive[CDAMAXNUMDRIVES + 1] = {0};
char *g_cdabuf = NULL;

int g_cdabufblocks = 10;
LONG g_cdabufsize = 10 * CDABLOCKSIZE;

int g_cdaactive = 0; // 1-based drive index, 0 not used
int g_cdanumdrives = 0;

int g_lba = 0;
int g_cdexisinit = 0;
DWORD g_dwEndSector = 0;

int CdaInitCDRom()
{
	int numready = 0;
	int i;
	fschar inipath[MAX_PATH];
	machar mastr[MAXLOGLINELEN];
	machar mastr2[MAXLOGLINELEN];

	for(i = 1; i <= CDAMAXNUMDRIVES; i++)
	{
		g_cdadrive[i].exists = 0;
		g_cdadrive[i].isready = 0;
		g_cdadrive[i].isactive = 0;
		g_cdadrive[i].fsvolrefnum = 0;
		g_cdadrive[i].name[0] = fsT('\0');
		g_cdadrive[i].track = 0;
		g_cdadrive[i].tracklast = 0;
	}
	g_cdaactive = 0;

	fssprintf(inipath, MAX_PATH, fsS fsC fsT("macda.ini"), g_settingsdir, FILESEP); 
	if(!g_cdexisinit)
	{
		if(CR_Init((char *)inipath) == CDEX_ERROR)
		{
			masprintf(mastr, MAXLOGLINELEN, L"CdaInitCDRom CR_Init 1 '%ls' failed", fs2ma(mastr2, inipath)); 
			maLog(mastr);
			return 1;
		}
		g_cdexisinit = 1;
	}

	// try both transport layers if necessary :
	for(int j = 0; j < 2; j++)
	{
		if((g_cdanumdrives = CR_GetNumCDROM()) > 0)
		{
			/* g_sets.usentscsi = */ CR_GetTransportLayer();
			break;
		}

		if(j >= 2)
		{
			maLog(L"CdaInitCDRom no cdrom drive found");
			return 1;
		}

		// switch transport layer
		CR_SetTransportLayer(CR_GetTransportLayer() ? 0 : 1);

		CR_SaveSettings();

		// re-init for new transport layer
		if(CR_Init((char *)inipath) == CDEX_ERROR)
		{
			g_cdexisinit = 0;
			machar  mastr[MAXLOGLINELEN];
			masprintf(mastr, MAXLOGLINELEN, L"CdaInitCDRom CR_Init 2 '%ls' failed", fs2ma(mastr2, inipath)); 
			maLog(mastr);
			return 1;
		}
	}

	if(g_cdanumdrives > CDAMAXNUMDRIVES)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaInitCDRom found %d cdrom drives, limited to first %d", 
													g_cdanumdrives, CDAMAXNUMDRIVES);
		maLog(mastr);
		g_cdanumdrives = CDAMAXNUMDRIVES;
	}

	for(i = 1; i <= g_cdanumdrives; i++)
	{
		g_cdadrive[i].exists = 1;
		g_cdadrive[i].fsvolrefnum = GetNextCDDriveLetter(i); // e.g. 'D'
		char osstr[8];
		osstr[0] = (char)g_cdadrive[i].fsvolrefnum;
		osstr[1] = 0;
		os2fs(mastr2, osstr); // fschar and machar are the same on unicode win
		fssprintf(g_cdadrive[i].name, MAX_PATH, fsS fsT(":") fsC, mastr2, FILESEP);

		CR_SetActiveCDROM(i - 1);
		if(CR_IsUnitReady() == 0x01)
		{
			if(CR_ReadToc() == CDEX_ERROR)
				maLog(L"CdaInitCDRom CR_ReadToc failed");
			else if((g_cdadrive[i].tracklast = CR_GetNumTocEntries()) > 1)
			{
				g_cdadrive[i].isready = 1;
				numready++;
			}
		}
	}
		
	if(numready < 1)
	{
		//masprintf(mastr, MAXLOGLINELEN, L"%d ready of %d cdroms", numready, g_cdanumdrives);
		//maLog(mastr);
		return 1;
	}

	return 0;

} // CdaInitCDRom


void CdaDeInitCDRom()
{
	CR_DeInit(); // this function is cplusplus
}


int CdaCloseRipper();
int CdaCloseRipper()
{
	return CR_CloseRipper();

} // CdaCloseRipper



int CdaOpenRipper();
int CdaOpenRipper()
{
	CDROMPARAMS Param = {0};

	CR_GetCDROMParameters(&Param);

	Param.nNumReadSectors = g_cdabufblocks;
	//Param.DriveTable.DriveType = GENERIC;
	//Param.DriveTable.SetSpeed = SPEEDMMC; // x1 //SPEEDNONE;
	//Param.DriveTable.Endian = LITTLEENDIAN;
	//Param.DriveTable.bAtapi = 1;
	//Param.nAspiTimeOut = 3000;
	//Param.btTargetID = 0;
	//Param.btAdapterID = 2;
	//Param.btLunID = 0;
	Param.nSpeed = 1;
	Param.bLockDuringRead = 0;

	if(CR_SetCDROMParameters(&Param) == CDEX_ERROR)
	{
		maLog(L"CdaOpenRipper CR_SetCDROMParameters failed");
		return 1;
	}

	if(CR_ReadToc() == CDEX_ERROR)
	{
		maLog(L"CdaOpenRipper CR_ReadToc failed");
		return 1;
	}

	TOCENTRY myToc = CR_GetTocEntry(0);
	TOCENTRY myToc1 = CR_GetTocEntry(CR_GetNumTocEntries() - 1);

	DWORD dwStartSector = myToc.dwStartSector;
	g_dwEndSector = myToc1.dwStartSector;

	if(CR_OpenRipper(&g_cdabufsize, dwStartSector, g_dwEndSector - 1) == CDEX_ERROR)
	{
		maLog(L"CdaOpenRipper CR_OpenRipper failed");
		return 1;
	}

	g_lba = 0;

	return 0;

} // CdaOpenRipper


int CdaCleanup(int drive)
{
	machar mastr[MAXLOGLINELEN];

	if(drive < 1 || drive > CDAMAXNUMDRIVES)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaCleanup bad drive index=%d", drive);
		maLog(mastr);
		return 1;
	}
	
	// KillIO
	// ...
	
	/*
	if(CdaSetBlockSize(g_cdadrive[drive].driverRefNum, 512))
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaCleanup CdaSetBlockSize 512 failed");
		maLog(mastr);
	}
	*/

	// GoodBye
	// ...

	//CR_DeInit();

	return 0;
	
} // CdaCleanup


int CdaGetTrackStartEnd(int drive)
{
	int startsecs;
	int endsecs;
	machar mastr[MAXLOGLINELEN];

	if(drive < 1 || drive > CDAMAXNUMDRIVES)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaGetTrackStartEnd bad drive index=%d", drive);
		maLog(mastr);
		return 1;
	}
	
	startsecs = GetTrackStartTime(g_cdadrive[drive].track);
	
	endsecs = startsecs + GetTrackLength(g_cdadrive[drive].track);
	
	g_cdadrive[drive].lbafirst = /*-SILENTFRAMES + */ startsecs * CDAFRAMESPERSEC;

	g_cdadrive[drive].lbalast = /*-SILENTFRAMES + */ endsecs * CDAFRAMESPERSEC;

	return 0;
	
} // CdaGetTrackStartEnd


/*
int CdaGetBlockSize(int drive, int *blocksize)
{
	int err = 1;
	machar mastr[MAXLOGLINELEN];

	if(drive < 1 || drive > CDAMAXNUMDRIVES)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaGetBlockSize bad drive index=%d", drive);
		maLog(mastr);
		return 1;
	}
	
	if(!blocksize)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaGetBlockSize drive=%d null blocksize", drive);
		maLog(mastr);
		return 1;
	}
	
	if(err)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaGetBlockSize PBControl err=%d", err);
		maLog(mastr);
		//return err;
	}

	if(blocksize)
		*blocksize = CDABLOCKSIZE; //PB.cntrlParam.csParam[0];
	
	return 0;
	
} // CdaGetBlockSize
*/


int CdaSetBlockSize(int drive, int blocksize)
{
	machar mastr[MAXLOGLINELEN];

	if(drive < 1 || drive > CDAMAXNUMDRIVES)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaSetBlockSize bad drive index=%d blocksize=%d", drive, blocksize);
		maLog(mastr);
		return 1;
	}
	
	// do nothing on pc ?
	return 0;
	
} // CdaSetBlockSize


int CdaSetSpindleSpeed(int drive, int speed)
{
	machar mastr[MAXLOGLINELEN];
	
	if(drive < 1 || drive > CDAMAXNUMDRIVES)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaSetSpindleSpeed bad drive index=%d speed=%d", drive, speed);
		maLog(mastr);
		return 1;
	}
	
	// do nothing on pc ?
	return 0;
	
} // CdaSetSpindleSpeed


int CdaReadTrack(int drive, long *pbytesread)
{
	BOOL bAbort = FALSE;
	machar mastr[MAXLOGLINELEN];
	
	if(drive < 1 || drive > CDAMAXNUMDRIVES)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaReadTrack bad drive index=%d", drive);
		maLog(mastr);
		return 1;
	}
	
	if(!g_cdabuf)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaReadTrack drive=%d NULL buf (lba=%d)", drive, g_cdadrive[drive].lba);
		maLog(mastr);
		return 1;
	}
	
	// seek
	if(g_lba != (int)g_cdadrive[drive].lba)
	{
		CR_CloseRipper();
		if(CR_OpenRipper(&g_cdabufsize, g_cdadrive[drive].lba, g_dwEndSector - 1) == CDEX_ERROR)
		{
			masprintf(mastr, MAXLOGLINELEN, L"CdaReadTrack CR_OpenRipper failed lba=%d lbasperread=%d",
										g_cdadrive[drive].lba, g_cdabufblocks);
			maLog(mastr);
			return 1;
		}
		g_lba = g_cdadrive[drive].lba;
	}
	
	if(CR_RipChunk((unsigned char *)g_cdabuf, pbytesread, bAbort) == CDEX_ERROR)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaReadTrack CR_RipChunk failed lba=%d lbasperread=%d",
	 							g_cdadrive[drive].lba, g_cdabufblocks);
		maLog(mastr);
		return 1;
	}

	return 0;
	
} // CdaReadTrack


int CdaGetCdInfo(int drive, char *StrtTrk, char *EndTrk)
{
	int num;
	machar mastr[MAXLOGLINELEN];
	
	if(drive < 1 || drive > CDAMAXNUMDRIVES)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaGetCdInfo bad drive index=%d", drive);
		maLog(mastr);
		return 1;
	}
	
	if(!StrtTrk)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaGetCdInfo drive=%d null StrtTrk", drive);
		maLog(mastr);
		return 1;
	}
	
	if(!EndTrk)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaGetCdInfo drive=%d null EndTrk", drive);
		maLog(mastr);
		return 1;
	}

	if(!IsAudioMediaPresent())
	{
		maLog(L"CdaGetCdInfo no media present");
		return 1;
	}

	if((num = GetNumTracks()) <= 0)
	{
		maLog(L"CdaGetCdInfo no tracks");
		return 1;
	}

	*StrtTrk = 1;
	*EndTrk = (char)num;

	return 0;
	
} // CdaGetCdInfo


int CdaEject(int drive)
{
	machar mastr[MAXLOGLINELEN];
	
	if(drive < 1 || drive > CDAMAXNUMDRIVES)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaEject bad drive index=%d", drive);
		maLog(mastr);
		return 1;
	}
	
	if(CdaInitCDRom()) // CdaEject
	{
		maLog(L"CdaEject CdaInitCDRom failed");
		return 1;
	}

	/*
	int err = CdaSetBlockSize(drive, 512);
	if(err)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaEject CdaSetBlockSize err=%d", err);
		maLog(mastr);
	}
	*/

	CDROMPARAMS Param;

	CR_SetActiveCDROM(drive - 1);
	if(CR_IsUnitReady() == 0x01)
	{
		CR_GetCDROMParameters(&Param);
		if(CR_EjectCD(TRUE) != 0x01) // SS_COMP 0x01
		{
			masprintf(mastr, MAXLOGLINELEN, L"CdaEject CR_EjectCD failed drive=%d '%ls'",
												drive, Param.lpszCDROMID);
			maLog(mastr);
		}
	}

	return 0;

} // CdaEject


int GetNextCDDriveLetter(int drive)
{
	machar mastr[MAXLOGLINELEN];
	if(drive <= 0)
	{
		maLog(L"GetNextCDDriveLetter bad drive index");
		return 0;
	}
	
	BYTE i;
	char buf[4];
	UINT uDriveType;
	int j;

	for(i = 2; i < 26; i++)
	{
		sprintf(buf, "%c:\\", ('A' + i));
		os2ma(mastr, buf);
		uDriveType = GetDriveType(mastr);

		if(uDriveType == DRIVE_CDROM)
		{
			for(j = 1; j <= drive; j++)
				if(g_cdadrive[j].fsvolrefnum == 'A' + i)
					break;

			if(j > drive)
				return 'A' + i;
		}
	}

	return 0;

} // GetNextCDDriveLetter


int CdaSetActiveDriveLetter(int driveletter)
{
	int i;
	int drive;
	machar mastr[MAXLOGLINELEN];

	if(driveletter < 'D' || driveletter > 'Z')
	{
		masprintf(mastr, MAXLOGLINELEN,
		L"CdaSetActiveDriveLetter bad driveletter '%lc' (%d)",
													driveletter, driveletter);
		maLog(mastr);
		return 1;
	}
	
	if(CdaInitCDRom()) // CdaSetActiveDriveLetter
	{
		masprintf(mastr, MAXLOGLINELEN,
			L"CdaSetActiveDriveLetter CdaInitCDRom failed driveletter '%lc' (%d)",
													driveletter, driveletter);
		maLog(mastr);
		return 1;
	}
	
	for(drive = 1; drive <= g_cdanumdrives; drive++)
		if(driveletter == g_cdadrive[drive].fsvolrefnum)
			break;

	if(drive > g_cdanumdrives)
	{
		masprintf(mastr, MAXLOGLINELEN,
		L"CdaSetActiveDriveLetter '%lc:' not found", driveletter);
		maLog(mastr);
		return 1;
	}
	
	for(i = 1; i <= g_cdanumdrives; i++)
		g_cdadrive[i].isactive = 0;
	g_cdadrive[drive].isactive = 1;

	g_cdaactive = drive;

	CR_SetActiveCDROM(drive - 1);

	return 0;

} // CdaSetActiveDriveLetter


int IsAudioMediaPresent()
{
	return CR_IsUnitReady();

} // IsAudioMediaPresent


int GetNumTracks()
{
	return CR_GetNumTocEntries();
	
} // GetNumTracks


int GetTrackStartTime(const int nTrack)
{
	TOCENTRY toc = CR_GetTocEntry(nTrack - 1);

	return toc.dwStartSector / CDAFRAMESPERSEC;

 } // GetTrackStartTime


int GetTrackLength(const int nTrack)
{
	TOCENTRY toc = CR_GetTocEntry(nTrack - 1);
	TOCENTRY toc2 = CR_GetTocEntry(nTrack);

	return (toc2.dwStartSector - toc.dwStartSector) / CDAFRAMESPERSEC;

} // GetTrackLength

#ifdef __cplusplus
}
#endif

#endif // MANOCDA

//eof
