//  maConfig.h

/*
 * MoreAmp
 * Copyright (C) 2004-2014 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

// this file is included first by all moreamp ma*.c and ma*.cpp files

// Platform		Dev environment		defined in environment  defined here	config file

// Mac OS X		Apple XCode			__XCODE__				__MAOSX__		maConfigMacOSX.h
// Mac OS X		darwin bash/gcc		__MACNOTXCODE__			__MAOSX__		maConfigMacOSX.h		
// Mac OS X		darwin bash/gcc		__MACUNIX__				__MAUNIX__		maConfigUnix.h		
// unix/linux   bash/gcc			__UNIX__				__MAUNIX__		maConfigUnix.h
// Win32		VC++ Express 2008	_WIN32 _MSC_VER			__MAWIN__		maConfigWin32.h
//									MAWIN_APP

#ifndef __MACONFIG_H__
#define __MACONFIG_H__

// only one of these :
#if defined(__XCODE__) || defined(__MACNOTXCODE__) // set by XCode, resp configure.in
 #if !defined(__MACUNIX__)
  #define __MAOSX__
 #endif
 #ifdef __UNIX__ // gcc 4 now sets this?
  #undef __UNIX__
 #endif
#endif

#ifdef __UNIX__ // set with -D__UNIX__ in configure.in and by wxGTK and wxX11
 #define __MAUNIX__
#endif

#if defined(_WIN32) || defined(_MSC_VER) // set by MSVC++
 #if !defined(__MAWIN__)
  #define __MAWIN__
 #endif
#endif

#if !defined(__MAOSX__) && !defined(__MAUNIX__) && !defined(__MAWIN__)
 #if defined(MABUILDLIBS) // defined in lib files that include maConfig.h while building the lib
  #if defined(__MACOS__)
   #define __MAUNIX__ // __MAOSX__
  #else
   #define __MAUNIX__
  #endif
 #endif
#endif

// sanity check
#if !defined(__MAOSX__) && !defined(__MAUNIX__) && !defined(__MAWIN__)
 #error missing definition __MAOSX__ __MAUNIX__ or __MAWIN__
#endif

#if defined(__MAOSX__) && (defined(__MAUNIX__) || defined(__MAWIN__))
 #error multiple definition with __MAOSX__
#endif

#if defined(__MAUNIX__) && (defined(__MAOSX__) || defined(__MAWIN__))
 #error multiple definition with __MAUNIX__
#endif

#if defined(__MACUNIX__) && !defined(__MAUNIX__)
 #error __MAUNIX__ not defined with __MACUNIX__
#endif

#ifdef __MAOSX__
 #include "../maConfigMacOSX.h"
#else
 #ifdef __MAWIN__
  #include "../maConfigWin32.h"
 #else // __MAUNIX__
  #include "../maConfigUnix.h"
 #endif
#endif

// program name
#ifdef MOREAMP_NAME
 #undef MOREAMP_NAME
#endif
#if defined(__MAUNIX__)
 #define MOREAMP_NAME L"moreamp"
 #define MOREAMP_NAMEos "moreamp"
 #define MOREAMP_NAMEfs fsT("moreamp")
#else
 #define MOREAMP_NAME L"MoreAmp"
 #define MOREAMP_NAMEos "MoreAmp"
 #define MOREAMP_NAMEfs fsT("MoreAmp")
#endif

// moreamp dir name
#ifdef MOREAMP_DIRNAME
 #undef MOREAMP_DIRNAME
#endif
#if defined(__MAUNIX__) && !defined(__MACUNIX__)
 #define MOREAMP_DIRNAME fsT("moreamp")
#else
 #define MOREAMP_DIRNAME fsT("MoreAmp")
#endif

// version also needs updating in several places :
// in the MacOSX XCode projects and in mac/Info.plist.
// Also, moreamp_version_stringOS below is fgrep-ed by mamksrcgz.sh for making the tar archive
#define MOREAMP_VERSION_STRINGos "0.1.29"
#define MOREAMP_VERSION_STRING L"0.1.29"

#define MOREAMP_HOME_URL L"http://sourceforge.net/projects/moreamp/"

// As of MoreAmp version 0 1 29, portaudio is required on all platforms 
// AUDIO_PORTAUDIO is ignored
// USE_AUDIOIN is optional, defined or undef'ed in the maConfig*.h files
// AUDIO_PORTAUDIO_SYSTEM PORTAUDIO_WITH_JACK are also optional

/*
#if !defined(USE_AUDIOIN)
 #error USE_AUDIOIN is not defined
#endif
*/

#if defined(AUDIO_MAC_CORE)
 #error AUDIO_MAC_CORE is defined but no longer supported
#endif

// these are from config.h in the libportaudio project:
#define PA_USE_DS 1 
#define PA_USE_WDMKS 0
#define PA_USE_WASAPI 1 
#define PA_USE_WMME 1
// these are also in the libportaudio project:
// #undef PA_USE_ASIO // win
// #undef PA_USE_JACK // win
// #undef PA_DYNAMIC_JACK // win

#if defined(MAAUDIONONE) // was once used in testing
 #undef MAAUDIONONE
 #if defined(__MAWIN__)
  #pragma message("warning MAAUDIONONE is ignored")
 #else
  #warning MAAUDIONONE is ignored
 #endif
#endif

// decoders can be excluded :
#ifndef USE_AUDIOIN
 #define NODECODEAUDIOIN
#endif

#ifndef USE_AIF
 #define NODECODEAIF
 #define NODECODERAW
#endif

#ifndef USE_WAV
 #define NODECODEWAV
#endif

#ifndef USE_WMA
 #define NODECODEWMA
#endif

#ifndef USE_LIBMAD
 #define NODECODEMP3 // also mp1 and mp2
#endif

#ifndef USE_LIBVORBIS
 #define NODECODEOGG
#endif

#ifndef USE_LIBFAAD
 #define NODECODEAAC
#endif

#if !defined(USE_LIBFAAD) || !defined(USE_LIBMP4V2)
 #define NODECODEMP4
#endif

#ifndef USE_LIBFLAC
 #define NODECODEFLAC
#endif

// define MACDAOSX for mac unix build with OSX cd reader instead of without cdparanoia
#if defined(USE_CDREADER) && defined(__MACUNIX__)
 #define MACDAOSX
#endif

#if !defined(USE_CDREADER) || (defined(__MACUNIX__) && !defined(MACDAOSX)) // no cdparanoia on macunix
 #define MANOCDA
 #if defined(MACDAOSX)
  #undef MACDAOSX
 #endif
#endif

#if !defined(USE_LIBSOUNDTOUCH)
 #if defined(__MAWIN__)
  #pragma message("warning USE_LIBSOUNDTOUCH is not defined")
 #else
  #warning USE_LIBSOUNDTOUCH is not defined
 #endif
#endif

// use a player thread for sbsms effect
#if defined(USE_LIBSBSMS)
 #define MAPLAYSBSMS
#endif

// allow cd reader on mac unix gtk2 builds for partial testing
/*
#if defined(MANOCDA)
 #if defined(USE_CDREADER) && defined(__MACUNIX__) && (defined(USE_GUI_CARBON) || defined(USE_GUI_GTK2))
  #undef MANOCDA
 #endif
#endif
*/

#ifndef USE_LIBID3TAG
 #define NOID3TAG
#endif

// encoders must be specifically included :
// binaries of some encoders should not be distributed without sources :

// these four encoders are always legal
#ifdef USE_LIBVORBIS		// this is defined, or not, in file maConfigXXXX.h generated by configure
 #define ENCODEOGG		
#endif

#ifdef USE_LIBFLAC			// this is defined, or not, in file maConfigXXXX.h generated by configure
 #define ENCODEFLAC
#endif

#ifdef USE_WAV_ENC			// this is defined, or not, in file maConfigXXXX.h generated by configure
 #define ENCODEWAV
#endif

#ifdef USE_AIF_ENC			// this is defined, or not, in file maConfigXXXX.h generated by configure
 #define ENCODEAIF
 #define ENCODERAW
#endif

//ss #define BINARYDISTRIB //ss  
#ifdef BINARYDISTRIB
#else
 #ifdef USE_LIBLAME			// this is defined, or not, in file maConfigXXXX.h generated by configure
  #define ENCODEMP3
 #endif

 #ifdef USE_LIBFAAC			// this is defined, or not, in file maConfigXXXX.h generated by configure
  #define ENCODEAAC	
 #endif

 #ifdef USE_LIBMP4V2		// this is defined, or not, in file maConfigXXXX.h generated by configure
  #ifdef USE_LIBMP4V2_ENC	// this is defined, or not, in file maConfigXXXX.h generated by configure
   #define ENCODEMP4	
  #endif
 #endif

 #ifdef USE_LIBWMA_ENC		// probably never
  #define ENCODEWMA
 #endif
#endif // !BINARYDISTRIB

// OSX moreampdir location
#if defined(__MAOSX__)
 #define MAOSXHOME	// this puts the osx moreampdir in the users home dir, like on unix
#endif				// else the moreampdir is the dir holding the MoreAmp executable, like on win

#if defined(__MAOSX__)
 #if !defined(MAOSXHOME)
  #define MAOSXHOME // moreampdir must be in the user's home dir
 #endif
#endif
#define MAUSESHM // use shared memory

// kludges
#ifdef __MAWIN__
 #define LROUND(x) x
#else
 #define LROUND(x) lround(floor(x))
#endif

// override some maConfigMacOSX.h definitions (as set by 'configure')
#if defined(__MAOSX__)
 #if defined(MAOSXN_GUI) // in MoreAmpN.pbproj
  #if !defined(USE_GUI)
   #define USE_GUI 1
  #endif
  #if !defined(USE_GUI_CARBON)
   #define USE_GUI_CARBON
  #endif
  #if defined(USE_GUI_GTK2)
   #undef USE_GUI_GTK2
  #endif
  #if defined(USE_GUI_WX)
   #undef USE_GUI_WX
  #endif
 #endif
#endif

#if defined(MABUILDLIBS) // defined in libid3tag\file.c, which includes maConfig.h while building the lib
 #undef USE_GUI			 // to avoid wx includes
#endif

// gui
#if !defined(USE_GUI)
 #define MAGUINONE
 #define NOTEXTVIEWER
 #if defined(USE_TEXTVIEWER)
  #undef USE_TEXTVIEWER
 #endif
#elif defined(USE_GUI_CARBON)
 #define MAGUICARBON
#elif defined(USE_GUI_GTK2)
 #define MAGUIGTK2
#elif defined(USE_GUI_WX) // wxWidgets-2.8.12 is the supported version
 #include "wx/platform.h" // for unicode
 #include "wx/version.h"
 #ifndef wxMAJOR_VERSION
  #error wxMAJOR_VERSION undefined
 #endif
 #ifndef wxMINOR_VERSION
  #error wxMINOR_VERSION undefined
 #endif
 #if (wxMAJOR_VERSION != 2)
   #error wxMAJOR_VERSION is not 2
 #endif
 #if (wxMINOR_VERSION == 4)
  #define MAGUIWX 4
  #warning wxMINOR_VERSION is 4, no longer supported
 #elif(wxMINOR_VERSION == 6)
  #define MAGUIWX 6 // no longer tested
  // #warning wxMINOR_VERSION is 6
 #elif(wxMINOR_VERSION == 8)
  #define MAGUIWX 8
  // #warning wxMINOR_VERSION is 8
 #elif(wxMINOR_VERSION == 9)
  #define MAGUIWX 9
  // #warning wxMINOR_VERSION is 9
 #else
  #error wxMINOR_VERSION is not 4 or 6 or 8 or 9, e.g 2.9
 #endif
 #if defined(__WXDEBUG__)
  // #warning __WXDEBUG__
 #endif
 // wx must have been built with unicode support
 #if !defined(wxUSE_UNICODE) || (wxUSE_UNICODE == 0)
  #error wxUSE_UNICODE not set, wxWidgets needs to be built with --enable-unicode
 #endif
#else
 #error USE_GUI defined without USE_GUI_CARBON or USE_GUI_GTK2 or USE_GUI_WX
#endif

#if defined(MAGUICARBON)
 #if !defined(__MAOSX__)
  #error USE_GUI_CARBON is only implemented on Mac OS X
 #endif
#elif defined(MAGUIGTK2)
 #if defined(__MAUNIX__)
  #define MAGUIGTK2
 #else
  #error USE_GUI_GTK2 is only implemented on Unix/Linux/GTK+2
 #endif
#endif

// custom buttons under wx
#if defined(USE_GUI_WX) && !defined(USE_WXBUTTONS)
 #define MAUSEMABUTTON 1
#endif

// custom sliders under wx
#if defined(USE_GUI_WX) && !defined(USE_WXSLIDERS)
 #define MAUSEMASLIDER 1
#endif

// custom checkboxes under wx
#if defined(USE_GUI_WX) && !defined(USE_WXCHECKBOXES)
 #define MAUSEMACHECKBOX 1
#endif

 // allow use of keyboard shortcuts
#if defined(USE_KEYBOARD)
 #define MAKEYBOARD
#endif

// mousewheel volume control
#if defined(USE_MOUSEWHEEL)
 #define MAMOUSEWHEEL
#endif

#if defined(USE_SMALLFONT) || (defined(__MAOSX__) && defined(__WXMAC__))  || (defined(__MACUNIX__) && defined(__WXGTK__))
 #if defined(__MACUNIX__) && defined(__WXGTK__) // wxGTK-2.8.12 SetFont() seems to do nothing 
  #define MASMALLERFONT 8
 #else
  #define MASMALLERFONT 11
 #endif
#endif

// text viewer (for readme, keybindings, log, catalog)
#if !defined(USE_TEXTVIEWER)
 #define NOTEXTVIEWER
#endif

#if defined(__MAOSX__)
 #if defined(MA_CPU_PPC) || defined(MA_CPU_PPC64)
  #ifndef WORDS_BIGENDIAN
   #error WORDS_BIGENDIAN not defined
  #endif
 #endif
 #if defined(MA_CPU_I86)
  #ifdef WORDS_BIGENDIAN
   #error WORDS_BIGENDIAN defined
  #endif
 #endif
#endif

#ifdef __MAWIN__
 #ifdef WORDS_BIGENDIAN
  #error WORDS_BIGENDIAN defined
 #endif
#endif

#define MASETLOCALE

// mem test
// #define MAMEMTRACE
#if defined(MAMEMTRACE)
 #include <string.h>
 #ifdef __cplusplus
 extern "C"
 {
 #endif
 extern int g_nmamallocs;
 extern int g_nmacallocs;
 extern int g_ndupallocs;
 extern int g_nmafrees;
 extern int g_ndupfrees;
 void *mamalloc(size_t a);
 void *macalloc(size_t a, size_t b);
 void mafree(void *a);
 void madupfree(void *a);
 #define MAMALLOC mamalloc
 #define MACALLOC macalloc
 #define MAREALLOC realloc
 #define MAFREE(x) mafree((void *)x)
 #define MADUPFREE madupfree
 #ifdef __cplusplus
 }
 #endif
#else
 #define MAMALLOC malloc
 #define MACALLOC calloc
 #define MAREALLOC realloc
 #define MAFREE(x) free((void *)x)
 #define MADUPFREE(x) free((void *)x)
#endif

// these are defined by configure and are redefined in some lib includes
// we don't use them so we undef them to avoid compiler warnings
#ifdef PACKAGE_BUGREPORT
 #undef PACKAGE_BUGREPORT
#endif
#ifdef PACKAGE_NAME
 #undef PACKAGE_NAME
#endif
#ifdef PACKAGE_STRING
 #undef PACKAGE_STRING
#endif
#ifdef PACKAGE_TARNAME
 #undef PACKAGE_TARNAME
#endif
#ifdef PACKAGE_VERSION
 #undef PACKAGE_VERSION
#endif
#ifdef PACKAGE_BUGREPORT
 #undef PACKAGE_BUGREPORT
#endif
#ifdef PACKAGE_NAME
 #undef PACKAGE_NAME
#endif
#ifdef PACKAGE_STRING
 #undef PACKAGE_STRING
#endif
#ifdef PACKAGE_TARNAME
 #undef PACKAGE_TARNAME
#endif
#ifdef PACKAGE_VERSION
 #undef PACKAGE_VERSION
#endif

#endif // __MACONFIG_H__
