// maDefs.h

/*
 * MoreAmp
 * Copyright (C) 2004-2014 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef __MADEFS_H__
#define __MADEFS_H__

#include "maTypes.h"

#define MAX_PATH 260							// max length of file paths in chars
#define SONGSLOADMAX 128						// max songs loaded on vdisk or in memory
#define MAVDISKTMP fsT("mavdisktmp")
#define MAPLAYLISTSDIR fsT("maplaylists")
#define MASETTINGSDIR fsT("masettings")
#define MARECORDSDIR fsT("marecords")
#define MAFILEOUTDIR fsT("mafileout")
#define MASETSDEFAULT fsT("madefault")
#define PLAYLISTTMP fsT("maplaylisttmpf")
#define PLAYLISTDROP fsT("madrop")
#define PLAYLISTCDA fsT("macda")
#define PLAYLISTAUDIOIN fsT("maaudioin")

#define NAMELENMAX 255
#define MARECORD unsigned short // info saved for each song in a list (has played or not)
#define RECENTPLAYLIST fsT("marecentlistname.txt")
#define MALOG fsT("maLog")

#define MASONGSLOADEDTITLE L"Songs Loaded"
#define MASONGSLOADED L"Songs loaded"
#define MANOSONGSLOADED L"(No songs loaded)"
#define MASONGINFOTITLE L"Song Info"
#define MASONGINFO L"Song info"
#define MANOSONGSELECTED L"(No song selected)"
#define MADIRECTMODE L"(Direct mode - songs are not loaded)"

#define CDAMAXNUMDRIVES 16
#define MAREADME L"README"
#define MAREADMEFILE fsT("README")
#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
 #define MASETSCDROMFILE fsT("masetscdrom")	// cdrom settings file name
#endif
#if defined(MAKEYBOARD)
 #define MAKEYBINDINGSFILE fsT("makeybindings") // keyboard bindings file name
#endif

// button labels
#if defined(MAGUICARBON)
 #define LABEL_QUIT L""
 #define LABEL_PREV L""
 #define LABEL_PLAY L""
 #define LABEL_PAUSE L""
 #define LABEL_STOP L""
 #define LABEL_NEXT L""
#elif defined(MAGUIGTK2)
 #define LABEL_QUIT L"   '" // these go under buttons
 #define LABEL_PREV L"   '"
 #define LABEL_PLAY L"   '"
 #define LABEL_PAUSE L"   '"
 #define LABEL_STOP L"   '"
 #define LABEL_NEXT L"   '"
 #define LABEL_B_QUIT L"Quit" // these go inside buttons
 #define LABEL_B_PREV L"Prev"
 #define LABEL_B_PLAY L"Play"
 #define LABEL_B_PAUSE L"Pause"
 #define LABEL_B_STOP L"Stop"
 #define LABEL_B_NEXT L"Next"
 #define LABEL_INFO L"Info"
 #define LABEL_LIST L"List"
 #define LABEL_LOAD L"Load"
 #define LABEL_SONG L"Song" 
 #define LABEL_MINUS L"Minus"
 #define LABEL_PLUS L"Plus"
 #define LABEL_MID L"Mid"
 #define LABEL_COPY L"Copy"
 #define LABEL_PASTE L"Paste"
 #define LABEL_RESET L"Reset"
 #define LABEL_SHIFTL L"Shift L"
 #define LABEL_SHIFTR L"Shift R"
 #define LABEL_INVERT L"Invert"
 #define LABEL_OK L"Ok"
 #define LABEL_CANCEL L"Cancel"
 #define LABEL_EDIT MASTREDIT // L""Edit"
#else // wx
 #define LABEL_QUIT L"Quit" // these go under buttons
 #define LABEL_PREV L"Prev"
 #define LABEL_PLAY L"Play"
 #define LABEL_PAUSE L"Pause"
 #define LABEL_STOP L"Stop"
 #define LABEL_NEXT L"Next"
#endif

// if you log this many errors something is wrong
#define MALOGMAXDEF 1000

// if you log errors this fast something is wrong
#define MALOGEVENTSDEF 100
#define MALOGPERIODDEF 3 // seconds

// values for logflags to stop or limit logging :
#define kMALOGOFF			1 // stop logging
#define kMALOGMAX			2 // popup a warning and stop logging if MALOGMAXDEF log events occur
#define kMALOGRATE			4 // popup a warning if more than MALOGEVENTSDEF occur per MALOGPERIODDEF
#define kMALOGNOTPLAYABLE	8 // log when FileIsPlayable fails

// file extensions
#define EXTMP1 fsT(".mp1")
#define EXTMP2 fsT(".mp2")
#define EXTMP3 fsT(".mp3")
#define EXTOGG fsT(".ogg")
#define EXTWMA fsT(".wma")

#define EXTAIF fsT(".aif")
#define EXTAIFF fsT(".aiff")
#define EXTWAV fsT(".wav")

#define EXTRAW fsT(".raw") // RAW PCM data, 16-bit signed, little-endian, 44100

/* these could be added to maPlayAif
#define EXTAU fsT(".au")		// Sun/NeXT AU format (big endian)
#define EXTPAF fsT(".paf")		// Ensoniq PARIS file format
#define EXTSVX fsT(".svx")		// Amiga IFF / SVX8 / SV16 format
#define EXTNIST fsT(".nist")	// Sphere NIST format
#define EXTVOC fsT(".voc")		// VOC fil
#define EXTIRCAM fsT(".ircam")	// Berkeley/IRCAM/CARL
#define EXTW64 fsT(".w64")		// Sonic Foundry's 64 bit RIFF/WAV
#define EXTMAT4 fsT(".mat4")	// Matlab (tm) V4.2 / GNU Octave 2.0
#define EXTMAT5 fsT(".mat5")	// Matlab (tm) V5.0 / GNU Octave 2.1
#define EXTPVF fsT(".pvf")		// Portable Voice Format
#define EXTXI fsT(".xi")		// Fasttracker 2 Extended Instrument
#define EXTHTK fsT(".htk")		// HMM Tool Kit format
*/

#define EXTAAC fsT(".aac")
#define EXTM2A fsT(".m2a")
#define EXTM4A fsT(".m4a")
#define EXTMP4 fsT(".mp4")
#define EXTFLA fsT(".fla")
#define EXTFLAC fsT(".flac")
#define EXTMAA fsT(".maa") // audio input flag file
#define EXTCDA fsT(".cda")
#define EXTMAF fsT(".maf") // ma playlist
#define EXTMAS fsT(".mas") // ma settings
#define EXTMAR fsT(".mar") // ma records (song records)
#define EXTM3U fsT(".m3u") // standard playlist
#define EXTTXT fsT(".txt") // ma log, catalog

// input files (order matches g_extstr[] in maPlayer.c)
enum
{
	kMAMP1 = 0,
	kMAMP2,
	kMAMP3,
	kMAOGG,
	kMAWMA,

	kMAAIF,
	kMAAIFF,
	kMAWAV,
	kMARAW,
	
	/* these could be added to maPlayAif
	kMAAU,		// Sun/NeXT AU format (big endian)
	kMAPAF,		// Ensoniq PARIS file format
	kMASVX,		// Amiga IFF / SVX8 / SV16 format
	kMANIST,	// Sphere NIST format
	kMAVOC,		// VOC fil
	kMAIRCAM,   // Berkeley/IRCAM/CARL
	kMAW64,		// Sonic Foundry's 64 bit RIFF/WAV
	kMAMAT4,	// Matlab (tm) V4.2 / GNU Octave 2.0
	kMAMAT5,	// Matlab (tm) V5.0 / GNU Octave 2.1
	kMAPVF,		// Portable Voice Format
	kMAXI,		// Fasttracker 2 Extended Instrument
	kMAHTK,		// HMM Tool Kit format
	*/
	
	kMAAAC,
	kMAM2A,
	kMAM4A,
	kMAMP4,
	kMAFLA,
	kMAFLAC,
	kMAMAA,		// audio input flag file
	
	// files from here on are 'not playable'
	kMAMAF,		// ma playlist file
	kMAMAS,		// ma settings file
	kMAMAR,		// ma records file
	kMAM3U,		// playlist file
	kMACDA,		// keep this here (next-to-last)
	kMALAST
};

#define kMAFIRSTPLAYBLE 0 // kMAMP1
#define kMALASTPLAYBLE (kMAMAF - 1) // not kMACDA

// outputs (order matches g_extstrout[] in maPlayer.c)
enum
{
	kOUTSOUND = 0,
#ifdef ENCODEMP3
	kOUTMP3,
#endif
#ifdef ENCODEOGG
	kOUTOGG,
#endif
#ifdef ENCODEFLAC
	kOUTFLAC,
#endif
#ifdef ENCODEAAC
	kOUTAAC,
#endif
#ifdef ENCODEMP4
	kOUTMP4,
#endif
#ifdef ENCODEWMA
	kOUTWMA,
#endif
#ifdef ENCODEWAV
	kOUTWAV,
#endif
#ifdef ENCODEAIF
	kOUTAIF,
#endif
#ifdef ENCODERAW
	kOUTRAW,
#endif
	kOUTLASTE
};

// #define MASTRSOUND L"Sound")
#define MASTRSOUNDfs fsT("Sound")
#define kOUTFIRST (kOUTSOUND + 1) // kOUTMP3
#define kOUTLAST (kOUTLASTE - 1) // kOUTRAW
#define kOUTMAX 10 // at least kOUTLASTE. This affects settings version.

// faac encoder options
#define kACCADTS	1
#define kACCMPEG2   2 // else MPEG4
#define kACCMAIN	4
#define kACCLTP		8 // else LC // MAIN 0, LOW 1, SSR 2, LTP 3
#define kACCTNS		16
#define kACCM4A		32 // make .m4a instead of .mp4

#if defined(__MAOSX__)
  #define FILESEP fsT('/')
  #define VOLSEP fsT('/')
#endif

#if defined(__MAWIN__)
 #define FILESEP fsT('\\')
 #define VOLSEP fsT(':')
#endif

#if defined(__MAUNIX__)
  #define FILESEP fsT('/')
  #define VOLSEP fsT('/')
#endif

#define MADNUMBANDS 10									// mad mp3 equalizer bands (used as preamp)

#define EQBANDS31 31									// larger number of pcm equalizer bands (sliders)
#define EQBANDS10 10									// lesser number of pcm equalizer bands

#define EQSLIDERFIRST 0									// first pcm equalizer slider index
#define EQSLIDERLAST (EQBANDS31 - 1)					// last  pcm equalizer slider index
#define EQ2SLIDERFIRST (EQSLIDERLAST + 1)				// first pcm equalizer2 slider index
#define EQ2SLIDERLAST (EQ2SLIDERFIRST + EQBANDS31 - 1)	// last  pcm equalizer2 slider index
#define EQNAMELEN 32									// max length equalizer preset name in chars
#define EQNUMPRESETS 16									// sets of pcm equalizer slider values

#define SLIDERINCR 1									// slider increment value when pushing the + button
#define SLIDERDECR 1									// slider decrement value when pushing the - button

// listbox display modes
#define LBOXPLAYERINFO	 2
#define LBOXPLAYLIST	 4
#define LBOXSONGSLOADED	 8
#define LBOXSONGINFO	16
#define LBOXSHOWINDEX   32
#define LBOXSHOWDIR		64
#define LBOXSHOWPATH   128

// slider ranges. Some code depends on MIN values being 0.
#define POSMIN 0				// position slider range 0 to 1000
#define POSMAX 1000

#define VOLUMEMIN 0				// volume slider range
// volume attenuation
#define VOLUMEMAX 1000
#define VOLUMESCALE VOLUMEMAX
#define kVOLMAX 32767
#define kVOLEXP 15

#define SEPARATIONMIN 0			// separation slider range 0 to 100
#define SEPARATIONMAX 100
#define SEPARATIONCALE SEPARATIONMAX

#define PREAMPMIN DBSLIDERMIN	// preamp slider range 0 to 64
#define PREAMPMAX DBSLIDERMAX
#define PREAMPMINDB (-16)		// preamp slider db range -16 to +16
#define PREAMPMAXDB 16

#define EFFECTSMIN 0			// speed and tempo effects sliders range 0 to 100
#define EFFECTSMAX 100
#define EFFECTSZERO (EFFECTSMIN + ((EFFECTSMAX - EFFECTSMIN)/ 2) -24) // speed and tempo effects sliders zero (no effect) position

#define EFFECTSPITCHMIN 0		// pitch effect slider range 0 to 48
#define EFFECTSPITCHMAX 48
#define EFFECTSPITCHZERO (EFFECTSPITCHMIN + ((EFFECTSPITCHMAX - EFFECTSPITCHMIN)/ 2)) // pitch effect slider zero (no effect) position

#define EFFECTSPITCH2MIN 0		// pitch2 effect slider range 0 to 100
#define EFFECTSPITCH2MAX 100
#define EFFECTSPITCH2ZERO (EFFECTSPITCH2MIN + ((EFFECTSPITCH2MAX - EFFECTSPITCH2MIN)/ 2)) // pitch2 effect slider zero (no effect) position

#define DBSLIDERMIN 0			// dbsliders have a symmetric dB scale (eq and preamp sliders)
#define DBSLIDERMAX 64			// db slider range 0 to 64
#define DBRANGE 16				// db slider abs dB range (-16 to +16)

//constants
//							 0 to EQBANDS31
#define PREAMPSLIDER		-1
#define SEPARATIONSLIDER	-2
#define VOLUMESLIDERS		-3
#define EFFECTSSLIDERS		-4
#define VOLUMESLIDERRIGHT	-5
#define VOLUMESLIDERLEFT	-6
#define EFFECTSSLIDERPITCH  -7
#define EFFECTSSLIDERPITCH2 -8
#define EFFECTSSLIDERTEMPO  -9
#define EFFECTSSLIDERSPEED  -10
#define EQSLIDERS			-11
#define EQ2SLIDERS			-12
#define ALLSLIDERS			-13

// effects switches
#define EFFECTSON			 1
#define EFFECTSPITCH		 2
#define EFFECTSTEMPO		 4
#define EFFECTSSPEED		 8
#define EFFECTSSBSMS		 16 // High quality sbsms, else SoundTouch

// values for usemem
#define BUFUSEMEM		0
#define BUFUSEVDISK		1
#define BUFUSEDIRECT	2
#define BUFUSEMAX		3

// songmem
#define KILOBYTE 1024		// 1 KB
#define MEGABYTE 1048576	// 1 MB

//(2047 * MEGABYTE is close to max positive int value, 2048 goes negative)
#define SONGMEMMAX 2047		// 2047 MB maximum

#define SONGMEMDEF 16		// default
#define SONGMEMMIN 16		// minimum

// sound output
#define MAOUTCHANNELS   2   // number of output channels
#define MAOUTBITS	   16   // bits per mono sample
#define MAOUTBYTES		4   // bytes per stereo sample
#define MAOUTRATE   44100   // not always

// equalizer flags
#define MAEQACTIVE		1 // equalizer is on
//						2 // unused
#define MAEQ31BANDS		4 // 31 bands, else 10
#define MAEQEXTRA		8 // filter twice, else once
#define MASEPARATELR   16 // separate left/right equalizers
#define MAEQZOOM	   32 // zoom mode, else offset
#define MAEQLINK	   64 // link sliders

#define FILEOUTBITRATEDEF 8 // 128 

// 2 near-best quality, not too slow; 5 good quality, fast; 7 ok quality, really fast
#define FILEOUTQUALITYDEF 5
#define FILEOUTQUALITYMP3DEF 2
#define FILEOUTQUALITYOGGDEF 5 // not used
#define FILEOUTQUALITYAACDEF 3
#define FILEOUTQUALITYMP4DEF 3

#define FILEOUTMODEDEF 0 // Stereo

#define FILEOUTSAMPLERATEDEF 7 // 44100

#define NUMWINCONF 2 // alternate window configurations

#define VERSIONSTRINGSIZE 16

#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
 #define MACDANAMEDEF "/dev/cdrom" // default unix cdrom drive name
#endif

// g_cdabufblocks = MAFIFOBUFSIZE / CDABLOCKSIZE s.b. a mult of 4, CDABLOCKSIZE = 2352
#define MAFIFOBUFSIZE 18816 // CDABLOCKSIZE * 8 = 18816 bytes, or 4704 stereo frames, or 107 ms at 44100 

// values for g_state.stop
#define STOP_ALL		0x01	// set by Stop button or some error condition for immediate stop
#define STOP_REQUEST	0x02	// set in RunPlay to normally exit RunLoop, 
#define STOP_DECODER	0x04	// set by Previous, Next, double-clicks in lists, index prompt, and
								// Viewer Play button, or a decoder error condition, to abort a song currently playing
#define STOP_APPQUIT	0x08	// set in DoAppQuit

#define MSPERSEC 1000		// milliseconds per second, or MAD_UNITS_MILLISECONDS
#define DUMMYOUTFILEP ((FILE *)-123)
#define NOTIMESTR L"--:--:--.-"
#define ZEROMSSTR L"00:00:00.0"
#if defined(MAGUICARBON)
 #define LISTLINELEN 256	// max listbox line length, must be <= MAXLOGLINELEN - 2
#else
 #define LISTLINELEN 512
#endif
#define MINSPERHOUR 60		// minutes per hour
#define SECSPERMIN 60		// seconds per minute
#define SECSPERHOUR 3660	// seconds per hour
#define MSPERDS 100			// milliseconds per decisecond, or MAD_UNITS_DECISECONDS
#define MSPERMIN 60000		// milliseconds per minute
#define MSPERHOUR 3600000	// milliseconds per hour
#define MAXLOGLINELEN (MAX_PATH + MAX_PATH)
#define MAXTITLELEN (MAX_PATH + MAX_PATH)
#if defined(__MAWIN__)
 #define MAEOL L"\r\n" // 0x0d0a lf
 #define FSEOL fsT("\r\n") // 0x0d0a lf
#else
 #define MAEOL L"\n" // 0x0a lf
 #define FSEOL fsT("\n") // 0x0a lf
#endif

#define AUDIOINPUTSTR L"Audio Input Device"
#define AUDIOOUTPUTSTR L"Audio Output Device"
#define AUDIOHOSTINSTR L"Audio Input Host API"
#define AUDIOHOSTOUTSTR L"Audio Output Host API"

// gui
// for mamessageBox
#define maYES                   0x00000002
#define maOK                    0x00000004
#define maNO                    0x00000008
#define maYES_NO                (maYES | maNO)
#define maCANCEL                0x00000010
#define maICON_EXCLAMATION      0x00000100
#define maICON_HAND             0x00000200
#define maICON_QUESTION         0x00000400
#define maICON_INFORMATION      0x00000800

// for maFileDialog
enum
{
    maOPEN              = 0x0001,
    maSAVE              = 0x0002,
    maOVERWRITE_PROMPT  = 0x0004,
    maHIDE_READONLY     = 0x0008,
    maFILE_MUST_EXIST   = 0x0010,
    maMULTIPLE          = 0x0020,
    maCHANGE_DIR        = 0x0040
};

// windows enum
enum // ID_WINDOW  all windows
{
	WINDOWIDINVALID = 0,	// 0
	WINDOWSPLASH,			// 1 WINDOWFIRST // WINDOWSPLASH unused
	WINDOWAPP,				// 2
	WINDOWLIST,				// 3
	WINDOWEQ,				// 4
	WINDOWEQ2,				// 5
	WINDOWEFFECTS,			// 6
	WINDOWABOUT,			// 7
	WINDOWREADME,			// 8
	WINDOWCATALOG,			// 9
	WINDOWPLAYLIST,			// 10
	WINDOWLOG,				// 11
	WINDOWKEYBINDINGS,		// 12
	WINDOWADVANCED,			// 13
	WINDOWAUDIOCD,			// 14
	WINDOWPROMPTNUM,		// 15
	WINDOWFILEOUT			// 16 WINDOWLAST
};


#define WINDOWFIRST WINDOWSPLASH
#define WINDOWLAST WINDOWFILEOUT
#define NUMWINDOWS (1 + WINDOWLAST - WINDOWFIRST + 1) // 17, windowid == 0 is invalid
#define MAXOPENRECENT 10

// window menus enum
enum // ID_MENU
{
#if defined(MAGUICARBON)
	// these id values must match those set manually in the IB nib files
	MENUMOREAMP,	// 0
	MENUFILES,		// 1
	MENUMACONTROLS,	// 2
	MENUMASETS,		// 3
	MENUTOOLS,		// 4
	MENUSHOW,		// 5
	MENUWINDOW,		// 6
	MENUHELP		// 7
#else // wx, gtk
	MENUFILES,		// 0
	MENUMACONTROLS,
	MENUMASETS,
	MENUTOOLS,
	MENUSHOW,
	MENUHELP,
	MENUTITLEINFO,
	MENUOPENRECENT	// MENULAST
#endif
};

#if defined(MAGUIWX)
 # define MENULAST MENUOPENRECENT
#else // not used
#endif

#if defined(MAGUICARBON)
 // these id values must match those set manually in the IB nib files
 #define ID0 0 // changing this or defines below breaks MAGUICARBON IB nib files
#else // wx and others
 #define ID0 6000 // >wxID_HIGHEST
#endif

enum {
 ID_CHK_ADDMODE = (ID0 + 10), // ID_CHECKBOX_FIRST
 ID_CHK_ALTSIZE,
 ID_CHK_RANDOM,
 ID_CHK_REPTPLAYED,
 ID_CHK_LOOPLOADED,
 ID_CHK_LOOPLIST,
 ID_CHK_EQWINDOW,
 ID_CHK_LISTWINDOW,
 ID_CHK_BEG,
 ID_CHK_END,
 ID_CHK_MONOLEFT,
 ID_CHK_HOSTINENABLE,
 ID_CHK_MONORIGHT,
 ID_CHK_INPUTMONITOR,
 ID_CHK_SINGLEPLAY,
 ID_CHK_EQUALIZER,
 ID_CHK_EQUALIZERB,
 ID_CHK_LIMITER,
 ID_CHK_EXITAFTER, // 28 was ID_CHK_EQEXTRA
 ID_CHK_EQEXTRAB,
 ID_CHK_EFFECTS,
 ID_CHK_TITLEINFO,
 ID_CHK_INVERTRIGHT,
 ID_CHK_EQ31BANDS, // not used
 ID_CHK_EQ31BANDSB,
 ID_CHK_REPTSONG,
 ID_CHK_MONODIFF,
 ID_CHK_BALANCE,
 ID_CHK_EQZOOM,
 ID_CHK_EQLINK,
 ID_CHK_SEPARATELR,
 ID_CHK_ADTS,
 ID_CHK_M4A,
 ID_CHK_PITCH,
 ID_CHK_TEMPO,
 ID_CHK_SPEED, // 45
 ID_CHK_EFFECTSB, // 46
 ID_CHK_CLOSEREOPEN, // 47
#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
  ID_CHK_CDPARANOIA,
#endif
 ID_CHK_USEM3U, // 48
 ID_CHK_SHOWINDEX, // 49
 ID_CHK_SHOWDIR, // 50
 ID_CHK_SHOWPATH, // 51
 ID_CHK_SBSMS, // 52
 ID_CHK_LOGNOTPLAYABLE // ID_CHECKBOX_LAST // 53
};

#define ID1 (ID0 + 100)
enum {
 ID_BTN_PREVIOUS = (ID1 + 1), // ID_BTN_FIRST
 ID_BTN_PLAY,
 ID_BTN_QUIT,
 ID_BTN_PAUSE,
 ID_BTN_STOP,
 ID_BTN_NEXT,
 ID_BTN_LOADED,
 ID_BTN_PLAYLIST,
 ID_BTN_INFO,
 ID_BTN_SONGINFO,
 ID_BTN_EQMINUS,
 ID_BTN_EQPLUS,
 ID_BTN_EQMID,
 ID_BTN_EQCOPY,
 ID_BTN_EQPASTE,
 ID_BTN_EQRESTORE,
 ID_BTN_EQSHIFTL,
 ID_BTN_EQSHIFTR,
 ID_BTN_EQINVERT,
 ID_BTN_EQ2MID,
 ID_BTN_EQ2COPY,
 ID_BTN_EQ2PASTE,
 ID_BTN_EQ2RESTORE,
 ID_BTN_FILEOUTDIR,
#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
  ID_BTN_AUDIOCDSET,
  ID_BTN_AUDIOCDCLEAR,
  ID_BTN_AUDIOCDSAVE,
  ID_BTN_AUDIOCDCLOSE,
#endif
 ID_BTN_VIEWERFIND,
 ID_BTN_VIEWERPLAY,
 ID_BTN_EDITSAVE,
 ID_BTN_PROMPTNUMCANCEL,
 ID_BTN_PROMPTNUMOK // ID_BTN_LAST
};

#define ID2 (ID1 + 100)
#define ID_LIST_CTRL			(ID2 + 1)

#define ID3 (ID2 + 100)
enum {
 ID_POPUP_OUTPUT = (ID3 + 1), // ID_POPUP_FIRST
 ID_POPUP_BUFFER,
 ID_POPUP_BITRATE,
 ID_POPUP_QUALITY,
 ID_POPUP_MODE,
 ID_POPUP_SAMPLERATE,
 ID_POPUP_AUDIOOUTPUT,
 ID_POPUP_AUDIOINPUT,
 ID_POPUP_AUDIOHOSTIN,
 ID_POPUP_AUDIOHOSTOUT,
#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX) 
  ID_POPUP_EQPRESET,
  ID_POPUP_AUDIOCD // ID_POPUP_LAST
#else
  ID_POPUP_EQPRESET // ID_POPUP_LAST
#endif
};

#define ID4 (ID3 + 100)
#define ID_EDITTEXT_PROMPTNUM	(ID4 + 1)
#define ID_SCROLLBAR_VIEWERV	(ID4 + 2)
#define ID_TEXTVIEW_ABOUT		(ID4 + 3)
#define ID_EDITTEXT_VIEWERVIEW	(ID4 + 4)
#define ID_EDITTEXT_EQ			(ID4 + 5)
#define ID_EDITTEXT_VIEWERFIND  (ID4 + 6)
#define ID_EDITTEXT_ABOUT		(ID4 + 7)

#define ID5 (ID4 + 100)
enum {
 ID_SPIN_BEG = (ID5 + 1), // ID_SPIN_FIRST 
 ID_SPIN_END,
 ID_SPIN_BUFFER,
 ID_SPIN_PREA,
 ID_SPIN_SEP,
 ID_SPIN_VOLR,
 ID_SPIN_VOLL,
 ID_SPIN_EQOFFSET,
 ID_SPIN_EQ0,
 ID_SPIN_EQ1,
 ID_SPIN_EQ2,
 ID_SPIN_EQ3,
 ID_SPIN_EQ4,
 ID_SPIN_EQ5,
 ID_SPIN_EQ6,
 ID_SPIN_EQ7,
 ID_SPIN_EQ8,
 ID_SPIN_EQ9,
 ID_SPIN_EQ10,
 ID_SPIN_EQ11,
 ID_SPIN_EQ12,
 ID_SPIN_EQ13,
 ID_SPIN_EQ14,
 ID_SPIN_EQ15,
 ID_SPIN_EQ16,
 ID_SPIN_EQ17,
 ID_SPIN_EQ18,
 ID_SPIN_EQ19,
 ID_SPIN_EQ20,
 ID_SPIN_EQ21,
 ID_SPIN_EQ22,
 ID_SPIN_EQ23,
 ID_SPIN_EQ24,
 ID_SPIN_EQ25,
 ID_SPIN_EQ26,
 ID_SPIN_EQ27,
 ID_SPIN_EQ28,
 ID_SPIN_EQ29,
 ID_SPIN_EQ30,
 ID_SPIN_EQ2_0,
 ID_SPIN_EQ2_1,
 ID_SPIN_EQ2_2,
 ID_SPIN_EQ2_3,
 ID_SPIN_EQ2_4,
 ID_SPIN_EQ2_5,
 ID_SPIN_EQ2_6,
 ID_SPIN_EQ2_7,
 ID_SPIN_EQ2_8,
 ID_SPIN_EQ2_9,
 ID_SPIN_EQ2_10,
 ID_SPIN_EQ2_11,
 ID_SPIN_EQ2_12,
 ID_SPIN_EQ2_13,
 ID_SPIN_EQ2_14,
 ID_SPIN_EQ2_15,
 ID_SPIN_EQ2_16,
 ID_SPIN_EQ2_17,
 ID_SPIN_EQ2_18,
 ID_SPIN_EQ2_19,
 ID_SPIN_EQ2_20,
 ID_SPIN_EQ2_21,
 ID_SPIN_EQ2_22,
 ID_SPIN_EQ2_23,
 ID_SPIN_EQ2_24,
 ID_SPIN_EQ2_25,
 ID_SPIN_EQ2_26,
 ID_SPIN_EQ2_27,
 ID_SPIN_EQ2_28,
 ID_SPIN_EQ2_29,
 ID_SPIN_EQ2_30,
 ID_SPIN_PITCH,
 ID_SPIN_PITCH2,
 ID_SPIN_TEMPO,
 ID_SPIN_SPEED // ID_SPIN_LAST
};

#define ID6 (ID5 + 100)
enum {
 ID_SLIDER_VOLUMEL = (ID6 + 1), // ID_SLIDER_FIRST // slider ids need to be contiguous
 ID_SLIDER_VOLUMER,
 ID_SLIDER_VOLUMESEP,
 ID_SLIDER_PREAMP,
 ID_SLIDER_POSITION,
 ID_SLIDER_EQ0, // ID_SLIDER_EQ_FIRST
 ID_SLIDER_EQ1,
 ID_SLIDER_EQ2,
 ID_SLIDER_EQ3,
 ID_SLIDER_EQ4,
 ID_SLIDER_EQ5,
 ID_SLIDER_EQ6,
 ID_SLIDER_EQ7,
 ID_SLIDER_EQ8,
 ID_SLIDER_EQ9,
 ID_SLIDER_EQ10,
 ID_SLIDER_EQ11,
 ID_SLIDER_EQ12,
 ID_SLIDER_EQ13,
 ID_SLIDER_EQ14,
 ID_SLIDER_EQ15,
 ID_SLIDER_EQ16,
 ID_SLIDER_EQ17,
 ID_SLIDER_EQ18,
 ID_SLIDER_EQ19,
 ID_SLIDER_EQ20,
 ID_SLIDER_EQ21,
 ID_SLIDER_EQ22,
 ID_SLIDER_EQ23,
 ID_SLIDER_EQ24,
 ID_SLIDER_EQ25,
 ID_SLIDER_EQ26,
 ID_SLIDER_EQ27,
 ID_SLIDER_EQ28,
 ID_SLIDER_EQ29,
 ID_SLIDER_EQ30,
 ID_SLIDER_EQ2_0, // ID_SLIDER_EQ2_FIRST
 ID_SLIDER_EQ2_1,
 ID_SLIDER_EQ2_2,
 ID_SLIDER_EQ2_3,
 ID_SLIDER_EQ2_4,
 ID_SLIDER_EQ2_5,
 ID_SLIDER_EQ2_6,
 ID_SLIDER_EQ2_7,
 ID_SLIDER_EQ2_8,
 ID_SLIDER_EQ2_9,
 ID_SLIDER_EQ2_10,
 ID_SLIDER_EQ2_11,
 ID_SLIDER_EQ2_12,
 ID_SLIDER_EQ2_13,
 ID_SLIDER_EQ2_14,
 ID_SLIDER_EQ2_15,
 ID_SLIDER_EQ2_16,
 ID_SLIDER_EQ2_17,
 ID_SLIDER_EQ2_18,
 ID_SLIDER_EQ2_19,
 ID_SLIDER_EQ2_20,
 ID_SLIDER_EQ2_21,
 ID_SLIDER_EQ2_22,
 ID_SLIDER_EQ2_23,
 ID_SLIDER_EQ2_24,
 ID_SLIDER_EQ2_25,
 ID_SLIDER_EQ2_26,
 ID_SLIDER_EQ2_27,
 ID_SLIDER_EQ2_28,
 ID_SLIDER_EQ2_29,
 ID_SLIDER_EQ2_30, // ID_SLIDER_EQ2_LAST
 ID_SLIDER_PITCH,
 ID_SLIDER_PITCH2,
 ID_SLIDER_TEMPO,
 ID_SLIDER_SPEED // ID_SLIDER_LAST
};

#define ID7 (ID6 + 100)
enum {
 ID_TEXT_TIMEPOS = ID7 + 1, // ID_TEXT_FIRST
 ID_TEXT_TIMEBEG,
 ID_TEXT_TIMEEND,
 ID_TEXT_ENCODING_MINBITRATE, // not used
 ID_TEXT_BUTTON_QUIT,
 ID_TEXT_BUTTON_PREVIOUS,
 ID_TEXT_BUTTON_PLAY,
 ID_TEXT_BUTTON_PAUSE,
 ID_TEXT_BUTTON_STOP, 
 ID_TEXT_BUTTON_NEXT, // ID_TEXT_BUTTON_LAST
 ID_TEXT_VOLUME,
 ID_TEXT_PREAMP,
 ID_TEXT_SEP,
 ID_TEXT_M,
 ID_TEXT_S,
 ID_TEXT_PLUS, // not used
 ID_TEXT_OUTPUT,
 ID_TEXT_BUFFER,
 ID_TEXT_100VOLL,
 ID_TEXT_100VOLR,
 ID_TEXT_100SEP,
 ID_TEXT_0,
 ID_TEXT_L2, // not used
 ID_TEXT_R2, // not used
 ID_TEXT_PREAMP2, // not used
 ID_TEXT_END, // not used
 ID_TEXT_BEG,
 ID_TEXT_PITCH0,
 ID_TEXT_PITCH02,
 ID_TEXT_TEMPO0,
 ID_TEXT_SPEED0,
 ID_TEXT_EQBUTTON_ZOOM,
 ID_TEXT_EQBUTTON_MID,
 ID_TEXT_EQBUTTON_COPY,
 ID_TEXT_EQBUTTON_PASTE,
 ID_TEXT_EQBUTTON_RESET,
 ID_TEXT_EQBUTTON_SHIFTL,
 ID_TEXT_EQBUTTON_SHIFTR,
 ID_TEXT_EQBUTTON_INVERT,
 ID_TEXT_EQPRESET,
 ID_TEXT_EQEDIT,
 ID_TEXT_EQOFFSET,
 ID_TEXT_EQOFFSETVAL,
 ID_TEXT_EQBAND0,
 ID_TEXT_EQBAND1,
 ID_TEXT_EQBAND2,
 ID_TEXT_EQBAND3,
 ID_TEXT_EQBAND4,
 ID_TEXT_EQBAND5,
 ID_TEXT_EQBAND6,
 ID_TEXT_EQBAND7,
 ID_TEXT_EQBAND8,
 ID_TEXT_EQBAND9,
 ID_TEXT_EQBAND10,
 ID_TEXT_EQBAND11,
 ID_TEXT_EQBAND12,
 ID_TEXT_EQBAND13,
 ID_TEXT_EQBAND14,
 ID_TEXT_EQBAND15,
 ID_TEXT_EQBAND16,
 ID_TEXT_EQBAND17,
 ID_TEXT_EQBAND18,
 ID_TEXT_EQBAND19,
 ID_TEXT_EQBAND20,
 ID_TEXT_EQBAND21,
 ID_TEXT_EQBAND22,
 ID_TEXT_EQBAND23,
 ID_TEXT_EQBAND24,
 ID_TEXT_EQBAND25,
 ID_TEXT_EQBAND26,
 ID_TEXT_EQBAND27,
 ID_TEXT_EQBAND28,
 ID_TEXT_EQBAND29,
 ID_TEXT_EQBAND30,
 ID_TEXT_EQ2BAND0,
 ID_TEXT_EQ2BAND1,
 ID_TEXT_EQ2BAND2,
 ID_TEXT_EQ2BAND3,
 ID_TEXT_EQ2BAND4,
 ID_TEXT_EQ2BAND5,
 ID_TEXT_EQ2BAND6,
 ID_TEXT_EQ2BAND7,
 ID_TEXT_EQ2BAND8,
 ID_TEXT_EQ2BAND9,
 ID_TEXT_EQ2BAND10,
 ID_TEXT_EQ2BAND11,
 ID_TEXT_EQ2BAND12,
 ID_TEXT_EQ2BAND13,
 ID_TEXT_EQ2BAND14,
 ID_TEXT_EQ2BAND15,
 ID_TEXT_EQ2BAND16,
 ID_TEXT_EQ2BAND17,
 ID_TEXT_EQ2BAND18,
 ID_TEXT_EQ2BAND19,
 ID_TEXT_EQ2BAND20,
 ID_TEXT_EQ2BAND21,
 ID_TEXT_EQ2BAND22,
 ID_TEXT_EQ2BAND23,
 ID_TEXT_EQ2BAND24,
 ID_TEXT_EQ2BAND25,
 ID_TEXT_EQ2BAND26,
 ID_TEXT_EQ2BAND27,
 ID_TEXT_EQ2BAND28,
 ID_TEXT_EQ2BAND29,
 ID_TEXT_EQ2BAND30,
 ID_TEXT_EQ0,
 ID_TEXT_EQ1,
 ID_TEXT_EQ2,
 ID_TEXT_EQ3,
 ID_TEXT_EQ4,
 ID_TEXT_EQ5,
 ID_TEXT_EQ6,
 ID_TEXT_EQ7,
 ID_TEXT_EQ8,
 ID_TEXT_EQ9,
 ID_TEXT_EQ10,
 ID_TEXT_EQ11,
 ID_TEXT_EQ12,
 ID_TEXT_EQ13,
 ID_TEXT_EQ14,
 ID_TEXT_EQ15,
 ID_TEXT_EQ16,
 ID_TEXT_EQ17,
 ID_TEXT_EQ18,
 ID_TEXT_EQ19,
 ID_TEXT_EQ20,
 ID_TEXT_EQ21,
 ID_TEXT_EQ22,
 ID_TEXT_EQ23,
 ID_TEXT_EQ24,
 ID_TEXT_EQ25,
 ID_TEXT_EQ26,
 ID_TEXT_EQ27,
 ID_TEXT_EQ28,
 ID_TEXT_EQ29,
 ID_TEXT_EQ30,
 ID_TEXT_EQ2_0,
 ID_TEXT_EQ2_1,
 ID_TEXT_EQ2_2,
 ID_TEXT_EQ2_3,
 ID_TEXT_EQ2_4,
 ID_TEXT_EQ2_5,
 ID_TEXT_EQ2_6,
 ID_TEXT_EQ2_7,
 ID_TEXT_EQ2_8,
 ID_TEXT_EQ2_9,
 ID_TEXT_EQ2_10,
 ID_TEXT_EQ2_11,
 ID_TEXT_EQ2_12,
 ID_TEXT_EQ2_13,
 ID_TEXT_EQ2_14,
 ID_TEXT_EQ2_15,
 ID_TEXT_EQ2_16,
 ID_TEXT_EQ2_17,
 ID_TEXT_EQ2_18,
 ID_TEXT_EQ2_19,
 ID_TEXT_EQ2_20,
 ID_TEXT_EQ2_21,
 ID_TEXT_EQ2_22,
 ID_TEXT_EQ2_23,
 ID_TEXT_EQ2_24,
 ID_TEXT_EQ2_25,
 ID_TEXT_EQ2_26,
 ID_TEXT_EQ2_27,
 ID_TEXT_EQ2_28,
 ID_TEXT_EQ2_29,
 ID_TEXT_EQ2_30,
 ID_TEXT_KEYEFECTS, // ID_TEXT_KEYEFFECTS_FIRST
 ID_TEXT_KEYPITCHSET,
 ID_TEXT_KEYPITCHUP,
 ID_TEXT_KEYPITCHDOWN,
 ID_TEXT_KEYPITCH2UP,
 ID_TEXT_KEYPITCH2DOWN,
 ID_TEXT_KEYPITCHRESET,
 ID_TEXT_KEYTEMPOSET,
 ID_TEXT_KEYTEMPOUP,
 ID_TEXT_KEYTEMPODOWN,
 ID_TEXT_KEYTEMPORESET,
 ID_TEXT_KEYSPEEDSET,
 ID_TEXT_KEYSPEEDUP,
 ID_TEXT_KEYSPEEDDOWN,
 ID_TEXT_KEYSPEEDRESET, // ID_TEXT_KEYEFFECTS_LAST
 ID_TEXT_FILEOUTDIR,
 ID_TEXT_VIEWERFOUND,
 ID_TEXT_VIEWERPLAY,
 ID_TEXT_AUDIOOUTPUT,
 ID_TEXT_AUDIOINPUT,
 ID_TEXT_AUDIOHOSTIN,
 ID_TEXT_AUDIOHOSTOUT,
 ID_TEXT_PROMPTNUM // ID_TEXT_LAST
};

#define ID_SLIDER_EQ2_FIRST ID_SLIDER_EQ2_0
#define ID_SLIDER_EQ2_LAST ID_SLIDER_EQ2_30

#define ID_SPIN_FIRST ID_SPIN_BEG
#define ID_SPIN_LAST ID_SPIN_SPEED

#define ID_BTN_FIRST ID_BTN_PREVIOUS
#define ID_BTN_LAST ID_BTN_PROMPTNUMOK

#define ID_POPUP_FIRST ID_POPUP_OUTPUT
#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
 #define ID_POPUP_LAST ID_POPUP_AUDIOCD
#else
 #define ID_POPUP_LAST ID_POPUP_EQPRESET
#endif

#define ID_CHECKBOX_FIRST ID_CHK_ADDMODE
#define ID_CHECKBOX_LAST ID_CHK_LOGNOTPLAYABLE 

#define ID_SLIDER_FIRST ID_SLIDER_VOLUMEL
#define ID_SLIDER_LAST ID_SLIDER_SPEED

#define ID_SLIDER_EQ_FIRST ID_SLIDER_EQ0
#define ID_SLIDER_EQ_LAST ID_SLIDER_EQ30

#define ID_TEXT_KEYEFFECTS_FIRST ID_TEXT_KEYEFECTS
#define ID_TEXT_KEYEFFECTS_LAST ID_TEXT_KEYSPEEDRESET

#define ID_TEXT_BUTTON_FIRST ID_TEXT_BUTTON_QUIT
#define ID_TEXT_BUTTON_LAST ID_TEXT_BUTTON_NEXT

#define ID_TEXT_FIRST ID_TEXT_TIMEPOS
#define ID_TEXT_LAST ID_TEXT_PROMPTNUM

// menu items
#define ID_MENU_FIRST ID_MENU_ABOUT

#if !defined(NOTEXTVIEWER)
 #define ID_MENU_LAST ID_MENU_README
#else
 #define ID_MENU_LAST ID_MENU_FACTORYSIZE
 #define ID_MENU_NONE 0
 #define ID_MENU_EDITKEYBINDINGS ID_MENU_NONE
 #define ID_MENU_VIEWCATALOG ID_MENU_NONE
 #define ID_MENU_README ID_MENU_NONE
 #define ID_MENU_VIEWLOG ID_MENU_NONE
#endif
enum
{
	// MoreAmp menu
	ID_MENU_ABOUT = 1, // ID_MENU_FIRST
	// File menu
	ID_MENU_OPENLIST,
	ID_MENU_MAKELIST,
	ID_MENU_SAVELISTAS,
	ID_MENU_DELETELIST,
	ID_MENU_PLAYFILE,
	ID_MENU_OPENRECENT,
	ID_SUBMENU_OPENRECENT1,
	ID_SUBMENU_OPENRECENT2,
	ID_SUBMENU_OPENRECENT3,
	ID_SUBMENU_OPENRECENT4,
	ID_SUBMENU_OPENRECENT5,
	ID_SUBMENU_OPENRECENT6,
	ID_SUBMENU_OPENRECENT7,
	ID_SUBMENU_OPENRECENT8,
	ID_SUBMENU_OPENRECENT9,
	ID_SUBMENU_OPENRECENT10, // exactly MAXOPENRECENT submenus
	ID_MENU_QUIT,
	// Controls menu
	ID_MENU_STOP,
	ID_MENU_PAUSE,
	ID_MENU_PREVIOUS,
	ID_MENU_PLAY,
	ID_MENU_NEXT,
	ID_MENU_PLAYCD,
	ID_MENU_EJECTCD,
	ID_MENU_AUDIOIN,
	// Settings menu
	ID_MENU_SAVEMASETS,
	ID_MENU_SETSAVED,
	ID_MENU_SETDEFAULT,
	ID_MENU_SETFACTORY,
	ID_MENU_SETCOPY,
	ID_MENU_SAVEMASETSDEFAULT,
#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
	ID_MENU_SETAUDIOCD,
#endif
	ID_MENU_EFFECTS,
	ID_MENU_RANDOM,
	ID_MENU_REPTPLAYED,
	ID_MENU_LOOPLOADED,
	ID_MENU_LOOPLIST,
	ID_MENU_EQUALIZER,
	ID_MENU_EQ31BANDS,
	ID_MENU_EQEXTRA,
	ID_MENU_MONO,
	ID_MENU_INVERTRIGHT,
	ID_MENU_SINGLEPLAY,
	ID_MENU_INPUTMONITOR,
	ID_MENU_TITLEINFO,
	ID_SUBMENU_TITLEINFO1,
	ID_SUBMENU_TITLEINFO2,
	ID_SUBMENU_TITLEINFO3,
	ID_MENU_AUTOSTART,
	ID_MENU_ERRLOGGING,
	ID_MENU_SETVDISK,
	ID_MENU_MULTIPLEINST,
#if defined(MAKEYBOARD)
	ID_MENU_KEYBOARD,
#endif
#if defined(MAMOUSEWHEEL)
	ID_MENU_MOUSEWHEEL,
#endif
	ID_MENU_ADVANCED,
	// Tools menu
	ID_MENU_ZEROLIST,
	ID_MENU_MAKECATALOG,
#if !defined(NOTEXTVIEWER)
	ID_MENU_VIEWCATALOG,
	ID_MENU_VIEWPLAYLIST,
	ID_MENU_VIEWLOG,
#endif
	ID_MENU_CLEARLOG,
#if !defined(NOTEXTVIEWER)
 #if defined(MAKEYBOARD)
	ID_MENU_EDITKEYBINDINGS,
	ID_MENU_BLANKKEYBINDINGS,
	ID_MENU_FACTORYKEYBINDINGS,
 #endif
#endif
	ID_MENU_RESCANAUDIODEVICES,
	// Show menu
	ID_MENU_SHOWLIST,
	ID_MENU_SHOWEQ,
	ID_MENU_SHOWEFFECTS,
	ID_MENU_SHOWFILEOUT,
	ID_MENU_ALTSIZE,
	ID_MENU_SAVEDSIZE,
	ID_MENU_DEFAULTSIZE,
	ID_MENU_FACTORYSIZE, // ID_MENU_LAST
	// Help menu
#if !defined(NOTEXTVIEWER)
	ID_MENU_README, // ID_MENU_LAST
#endif

	ID_TIMER_CHANGEPLAYLIST,
	ID_TIMER_PLAYPLAYLIST,
	ID_TIMER_RUN,
	ID_TIMER_PLAY,
	ID_TIMER_STOP
#if defined(MAKEYBOARD)
	,ID_TIMER_KBFOCUS
#endif
#if defined(__WXMAC__)
	,ID_TIMER_LEFTUP
#endif
};

#define MAINDEXSTRLEN 5 // L"12345 ----------------------------------------------------"
#define MACATALOGSTRING L"----------------------------------------------------"
#define MACATALOGERRSTRING L"- - - - - ------------------------------------------"
#define MALOADINGSTRING L"Loading..."

#endif // __MADEFS_H__

//eof
