// maEffects.cpp

/*
 * MoreAmp
 * Copyright (C) 2004-2014 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

// this code contains extracts from SoundStretch :
//  SoundTouch audio processing library
//  Copyright (c) Olli Parviainen

#include "maConfig.h" // included by all moreamp files

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <memory.h>
#include <stdexcept>
#include <limits.h>
#include <math.h>

#include "maTypes.h"
#include "maDefs.h"
#include "maPlayer.h"
#include "maUtils.h"
#include "maEffects.h"

#if defined(USE_LIBSOUNDTOUCH)
 #if defined(__XCODE__) // mac xcode seems to get assert.h somewhere else
  #include <assert.h>
  #if !defined(assert)
   #define assert(e) ((void)0) // and we don't care about asserts
  #endif
 #endif
 #include "SoundTouch.h"
 using namespace soundtouch;
 #if defined(SOUNDTOUCH_FLOAT_SAMPLES)
  #undef SOUNDTOUCH_FLOAT_SAMPLES
 #endif
 #if !defined(SOUNDTOUCH_INTEGER_SAMPLES)
  #define SOUNDTOUCH_INTEGER_SAMPLES
 #endif
 #if defined(SAMPLETYPE) //typedef short MASAMPLETYPE
  #define MASAMPLETYPE SAMPLETYPE;
 #else
  typedef short MASAMPLETYPE;
 #endif
#else
 #if defined(__MAWIN__)
  #pragma message("warning USE_LIBSOUNDTOUCH is not defined")
 #else
  #warning USE_LIBSOUNDTOUCH is not defined
 #endif
#endif // !USE_LIBSOUNDTOUCH

// RunParameters.h
////////////////////////////////////////////////////////////////////////////////
///
/// A class for parsing the 'soundstretch' application command line parameters
///
/// Author        : Copyright (c) Olli Parviainen
/// Author e-mail : oparviai @ iki.fi
/// SoundTouch WWW: http://www.iki.fi/oparviai/soundtouch
///
////////////////////////////////////////////////////////////////////////////////
//
// Last changed  : $Date: 2004/10/30 16:10:16 $
// File revision : $Revision: 1.5 $
//
// $Id: RunParameters.h,v 1.5 2004/10/30 16:10:16 Olli Exp $
//
////////////////////////////////////////////////////////////////////////////////
#ifndef RUNPARAMETERS_H
#define RUNPARAMETERS_H

/// Parses command line parameters into program parameters
class RunParameters
{
private:
//	void throwIllegalParamExp(const string &str) const;
//	void throwLicense() const;
//	void parseSwitchParam(const string &str);
//	float parseSwitchValue(const string &str) const;

public:
	void checkLimits();
	float tempoDelta;
	float pitchDelta;
	float rateDelta;
	int   quick;
	int   noAntiAlias;
	float goalBPM;
	BOOL  detectBPM;

	RunParameters();
};

#endif


// RunParameters.cpp
////////////////////////////////////////////////////////////////////////////////
///
/// A class for parsing the 'soundstretch' application command line parameters
///
/// Author        : Copyright (c) Olli Parviainen
/// Author e-mail : oparviai @ iki.fi
/// SoundTouch WWW: http://www.iki.fi/oparviai/soundtouch
///
////////////////////////////////////////////////////////////////////////////////
//
// Last changed  : $Date: 2004/10/30 16:10:16 $
// File revision : $Revision: 1.6 $
//
// $Id: RunParameters.cpp,v 1.6 2004/10/30 16:10:16 Olli Exp $
//
////////////////////////////////////////////////////////////////////////////////

// Program usage instructions 
/*
static const machar whatText[] = 
	"This application processes WAV audio files by modifying the sound tempo,\n"
	"pitch and playback rate properties independently from each other.\n"
	"\n";

static const machar usage[] = 
	"Usage :\n"
	"	soundstretch infile.wav outfile.wav [switches]\n\n"
	"Available switches are:\n"
	"  -tempo=n : Change sound tempo by n percents  (n=-95..+5000 %)\n"
	"  -pitch=n : Change sound pitch by n semitones (n=-60..+60 semitones)\n"
	"  -rate=n  : Change sound rate by n percents   (n=-95..+5000 %)\n"
	"  -bpm=n   : Detect the BPM rate of sound and adjust tempo to meet 'n' BPMs.\n"
	"             If '=n' is omitted, just detects the BPM rate.\n"
	"  -quick   : Use quicker tempo change algorithm (gain speed, lose quality)\n"
	"  -naa     : Don't use anti-alias filtering (gain speed, lose quality)\n"
	"  -license : Display the program license text (LGPL)\n";
*/

// Constructor
RunParameters::RunParameters()
{
	quick = 0;
	noAntiAlias = 0;
	goalBPM = 0;
	detectBPM = FALSE;
	
	tempoDelta = 0; // -95..+5000 %
	pitchDelta = 0; // -60..+60 semitones
	rateDelta = 0; // -95..+5000 %
			
	checkLimits();
}


// Checks parameter limits
void RunParameters::checkLimits()
{
	if(tempoDelta < -95.0f) 
        tempoDelta = -95.0f;
	else if(tempoDelta > 5000.0f) 
        tempoDelta = 5000.0f;

	if(pitchDelta < -60.0f) 
        pitchDelta = -60.0f;
	else if(pitchDelta > 60.0f) 
        pitchDelta = 60.0f;

	if(rateDelta < -95.0f) 
        rateDelta = -95.0f;
	else if(rateDelta > 5000.0f) 
        rateDelta = 5000.0f;
}


// main.cpp
////////////////////////////////////////////////////////////////////////////////
///
/// SoundStretch main routine.
///
/// Author        : Copyright (c) Olli Parviainen
/// Author e-mail : oparviai @ iki.fi
/// SoundTouch WWW: http://www.iki.fi/oparviai/soundtouch
///
////////////////////////////////////////////////////////////////////////////////
//
// Last changed  : $Date: 2004/10/30 16:10:16 $
// File revision : $Revision: 1.19 $
//
// $Id: main.cpp,v 1.19 2004/10/30 16:10:16 Olli Exp $
//
////////////////////////////////////////////////////////////////////////////////

// Processing chunk size in 'samples'
#define FXBUFF_SIZE (MAFIFOBUFSIZE / sizeof(MASAMPLETYPE))

// Sets the 'SoundTouch' object up according to input sound format
static void setupSoundTouch(SoundTouch *pSoundTouch, RunParameters *params)
{
	pSoundTouch->setSampleRate(g_state.insamplerate);
	pSoundTouch->setChannels(2);

	pSoundTouch->setTempoChange(params->tempoDelta);
	pSoundTouch->setPitchSemiTones(params->pitchDelta);
	pSoundTouch->setRateChange(params->rateDelta);

	pSoundTouch->setSetting(SETTING_USE_QUICKSEEK, params->quick);
	pSoundTouch->setSetting(SETTING_USE_AA_FILTER, !params->noAntiAlias);

} // setupSoundTouch


#ifdef __cplusplus
 extern "C"
 {
#endif

RunParameters *g_fxparams = NULL;
SoundTouch *g_pSoundTouch = NULL;

float effectsval(int effect);
float effectsval(int effect)
{
	switch(effect) // assuming EFFECTSMIN, EFFECTSPITCHMIN are zero
	{
		case EFFECTSTEMPO :
		return (((float)g_sets.temposlider / EFFECTSMAX) * 200.0f) - 52.0f; // -52 to 148
		
		case EFFECTSPITCH :
		return (g_sets.pitchslider - EFFECTSPITCHMAX / 2) + .02f * (g_sets.pitchslider2 - 50.0f); // -25 to +25
		
		case EFFECTSSPEED :
		return (((float)g_sets.speedslider / EFFECTSMAX) * 200.0f) - 52.0f; // -52 to 148
	}
		
	return 0.0f;

} // effectsval


int l_setupeffectsflagSoundTouch = 0;
int SetupEffectsSoundTouch();
int SetupEffectsSoundTouch()
{
	if(l_setupeffectsflagSoundTouch)
		return 1;
		
	l_setupeffectsflagSoundTouch = 1;
	
	if(!AnyEffectOn())
		goto setupexitSoundTouch; // Effects off or none enabled
		
	if(!g_pSoundTouch)
	{
		if((g_pSoundTouch = new SoundTouch()) == NULL)
		{
			maLog(L"SetupEffects new SoundTouch failed");
			goto setupexitSoundTouch;
		}
	}
	
	if(!g_fxparams)
	{
		if((g_fxparams = new RunParameters()) == NULL)
		{
			maLog(L"SetupEffects new RunParameters failed");
			goto setupexitSoundTouch;
		}
	}
	
	g_fxparams->tempoDelta  = (g_sets.effects & EFFECTSTEMPO) ? effectsval(EFFECTSTEMPO) : 0.0f;
	g_fxparams->pitchDelta  = (g_sets.effects & EFFECTSPITCH) ? effectsval(EFFECTSPITCH) : 0.0f;
	g_fxparams->rateDelta   = (g_sets.effects & EFFECTSSPEED) ? effectsval(EFFECTSSPEED) : 0.0f;

	g_fxparams->quick		= 0;
	g_fxparams->noAntiAlias = 0;
	
	g_fxparams->checkLimits();

	setupSoundTouch(g_pSoundTouch, g_fxparams);

setupexitSoundTouch:
	l_setupeffectsflagSoundTouch = 0;
	
	return 0;
	
} // SetupEffectsSoundTouch


#define FXBUFFBYTES ((long)(FXBUFF_SIZE * sizeof(MASAMPLETYPE)))
MASAMPLETYPE l_fxbuffer[FXBUFF_SIZE];
int ApplyEffectsSoundTouch(char *indata, unsigned long inlen, char *outdata, unsigned long *poutlen)
{
	int numsamples;
    long numbytes;
	long numInBytes;
    long numOutBytes;
	char *p;
	char *q;

	numOutBytes = *poutlen;
	*poutlen = 0;
	
	if(!outdata || (inlen && !indata))
	{
		maLog(L"ApplyEffects bad params");
		return 1;
	}
		
	q = outdata;
	
	if(inlen > 0)
	{
		// Setup the 'SoundTouch' object for processing the sound
		if(!g_pSoundTouch || g_effectschange)
		{
			if(SetupEffectsSoundTouch())
				return 1;
			g_effectschange = 0;
		}
		
		// Process the sound

		// soundtouch routines work with 'Samples'
		// a 'Sample' is a frame, i.e two SAMPLETYPEs for stereo, or one for mono
		// FXBUFF_SIZE is in SAMPLETYPEs

		p = indata;
		numInBytes = inlen;

		while(numInBytes > 0 && numOutBytes > 0)
		{
			if((numbytes = numInBytes) > FXBUFFBYTES)
				numbytes = FXBUFFBYTES;

			numsamples = numbytes / (sizeof(MASAMPLETYPE) * 2);

			// Feed the samples into SoundTouch processor
			g_pSoundTouch->putSamples((MASAMPLETYPE *)p, numsamples);

			numInBytes -= numbytes;
			p += numbytes;

			// Read ready samples from SoundTouch processor
			// NOTES:
			// - 'receiveSamples' doesn't necessarily return any samples at all
			//   during some rounds!
			// - On the other hand, during some round 'receiveSamples' may have more
			//   ready samples than would fit into 'sampleBuffer', and for this reason 
			//   the 'receiveSamples' call is iterated for as many times as it
			//   outputs samples.
			while((numsamples = g_pSoundTouch->receiveSamples(l_fxbuffer, FXBUFF_SIZE / 2)) > 0)
			{
				if((numbytes = numsamples * sizeof(MASAMPLETYPE) * 2) > numOutBytes)
				{
					numbytes = numOutBytes;
					maLog(L"ApplyEffects fxbuffer overrun");
				}
				if(numbytes > 0)
				{
					memcpy(q, l_fxbuffer, numbytes);
					q += numbytes;
					*poutlen += numbytes;
					numOutBytes -= numbytes;
				}
			}
		}
	} // inlen > 0
	else if(inlen == 0 && g_pSoundTouch) // last, called at end of input file
	{
		// Now the input is processed, yet 'flush' few last samples that are
		// hiding in the SoundTouch's internal processing pipeline.
		g_pSoundTouch->flush();
		while((numsamples = g_pSoundTouch->receiveSamples((MASAMPLETYPE *)q, FXBUFF_SIZE / 2)) > 0)
		{
			if((numbytes = numsamples * sizeof(MASAMPLETYPE) * 2) > numOutBytes)
			{
				numbytes = numOutBytes;
				maLog(L"ApplyEffects fxbuffer overrun");
			}
			memcpy(q, l_fxbuffer, numbytes);
			q += numbytes;
			*poutlen += numbytes;
			numOutBytes -= numbytes;
		}
	} // last
	
	return 0;

} // ApplyEffectsSountTouch


void CleanupEffectsSoundTouch()
{
	if(g_pSoundTouch)
	{
		delete g_pSoundTouch;
		g_pSoundTouch = NULL;
	}
	
	if(g_fxparams)
	{
		delete g_fxparams;
		g_fxparams = NULL;
	}

	if(g_fxoutbuffer)
	{
		MAFREE(g_fxoutbuffer);
		g_fxoutbuffer = NULL;
	}

} // CleanupEffectsSoundTouch

#ifdef __cplusplus
 }
#endif


#ifdef __cplusplus
 extern "C"
 {
#endif

int ApplyEffects(char *indata, unsigned long inlen, char *outdata, unsigned long *poutlen)
{
#if defined(USE_LIBSOUNDTOUCH)
 #if defined(MAPLAYSBSMS)
	if(g_state.pdecoder != (threadparametertype)maPlaySbsms)
 #else
	if(!(g_sets.effects & EFFECTSSBSMS))
 #endif
		return ApplyEffectsSoundTouch(indata, inlen, outdata, poutlen);
#endif

#if defined(USE_LIBSBSMS)&& !defined(MAPLAYSBSMS)
	if(g_sets.effects & EFFECTSSBSMS)
	{
		if(inlen == 0) // no flush for sbsms
		{
			*poutlen = 0;
			return 0;
		}
		return ApplyEffectsSBSMS(indata, inlen, outdata, poutlen);
	}
#endif

	return 1;

} // ApplyEffects


void CleanupEffects() // called from AppQuit
{
#if defined(USE_LIBSOUNDTOUCH)
	CleanupEffectsSoundTouch();
#endif
#if defined(USE_LIBSBSMS)
	CleanupEffectsSBSMS();
#endif
} // CleanupEffects

#ifdef __cplusplus
 }
#endif

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// adapted from Audacity: A Digital Audio Editor SBSMSEffect.cpp Clayton Otey

#include "maConfig.h" // included by all moreamp files

#if defined(USE_LIBSBSMS)
 #include <stdlib.h>

 #include "maTypes.h"
 #include "maAudioPA.h"
 #include "maPlayer.h"
 typedef maint64 sampleCount;
 #include "masbsms.h"
 using namespace _sbsms_;

 class EffectSBSMS
 {
	public:
		bool Process(char *indata, unsigned long inlen, char *outdata, unsigned long *poutlen);
		void setParameters(double rateStart, double rateEnd, double pitchStart, double pitchEnd,
						SlideType rateSlideType, SlideType pitchSlideType,
						bool bLinkRatePitch, bool bRateReferenceInput, bool bPitchReferenceInput);

	private:
		double rateStart, rateEnd, pitchStart, pitchEnd;
		bool bLinkRatePitch, bRateReferenceInput, bPitchReferenceInput;
		SlideType rateSlideType;
		SlideType pitchSlideType;
 };


 #include <math.h>

 enum {
  SBSMSOutBlockSize = 512
 };

 class ResampleBuf
 {
  public:
	ResampleBuf()
	{
		processed = 0;
		buf = NULL;
		quality = NULL;
		iface = NULL;
		sbsms = NULL;
		resampler = NULL;
		SBSMSBuf = NULL;
	}

	~ResampleBuf()
	{
		/*
		if(buf)
			free(buf);
		if(leftTrack)
			free(leftTrack);
		if(rightTrack)
			free(rightTrack);
		if(SBSMSBuf)
			free(SBSMSBuf);
		*/

		if(quality)
			delete quality;
		if(sbsms)
			delete sbsms;
		if(iface)
			delete iface;
		if(resampler)
			delete resampler;
	}

	bool bPitch;
	audio *buf;
	double ratio;
	sampleCount processed;
	sampleCount blockSize;
	sampleCount SBSMSBlockSize;
	sampleCount offset;
	sampleCount end;
	float *leftTrack;
	float *rightTrack;
	SBSMS *sbsms;
	SBSMSInterface *iface;
	audio *SBSMSBuf;

	// Not required by callbacks, but makes for easier cleanup
	Resampler *resampler;
	SBSMSQuality *quality;
 };

 class SBSMSEffectInterface : public SBSMSInterfaceSliding {
 public:
   SBSMSEffectInterface(Resampler *resampler,
                        Slide *rateSlide, Slide *pitchSlide,
                        bool bReferenceInput,
                        long samples, long preSamples,
                        SBSMSQuality *quality) 
      : SBSMSInterfaceSliding(rateSlide,pitchSlide,bReferenceInput,samples,preSamples,quality)
   {
      this->resampler = resampler;
   }
   virtual ~SBSMSEffectInterface() {}

   long samples(audio *buf, long n) { 
      return resampler->read(buf, n);
   }

 protected:
   Resampler *resampler;
 };

 long resampleCB(void *cb_data, SBSMSFrame *data);

 short *l_pindata;
 short *l_poutdata;
 long l_blockSize;
 EffectSBSMS *l_pEffectSBSMS = NULL;

 long resampleCB(void *cb_data, SBSMSFrame *data) // Resampler callback to get effect input pcm data
 {
	ResampleBuf *r = (ResampleBuf*)cb_data;
   
	maint64 blockSize = l_blockSize;

	// Adjust the block size if it is the final block
	if (r->offset + blockSize > r->end)
		blockSize = r->end - r->offset;

	// Get the input pcm samples into the sbsms buffers.
	for(unsigned int i = 0; i < blockSize; i++)
	{
		r->buf[i][0] = (float)*l_pindata++;
		r->buf[i][1] = (float)*l_pindata++;
	}

	data->buf = r->buf;
	data->size = (long)blockSize;
	if(r->bPitch)
	{
		float t0 = (float)(r->processed) / r->iface->getSamplesToInput();
		float t1 = (float)(r->processed + blockSize) / r->iface->getSamplesToInput();
		data->ratio0 = r->iface->getStretch(t0);
		data->ratio1 = r->iface->getStretch(t1);
	}
	else
	{
		data->ratio0 = (float)r->ratio;
		data->ratio1 = (float)r->ratio;
	}

	r->processed += blockSize;
	r->offset += blockSize;

	return (long)blockSize; // return value is ignored by caller ?

 } // resampleCB


 long postResampleCB(void *cb_data, SBSMSFrame *data)
 {
	ResampleBuf *r = (ResampleBuf*) cb_data;
	long sampleCount = r->sbsms->read(r->iface, r->SBSMSBuf, (long)r->SBSMSBlockSize);
	data->buf = r->SBSMSBuf;
	data->size = sampleCount;
	data->ratio0 = (float)(1.0 / r->ratio);
	data->ratio1 = (float)(1.0 / r->ratio);
	return sampleCount;
 }

 void EffectSBSMS :: setParameters(double rateStart, double rateEnd, double pitchStart, double pitchEnd,
                                  SlideType rateSlideType, SlideType pitchSlideType,
                                  bool bLinkRatePitch, bool bRateReferenceInput, bool bPitchReferenceInput)
 {
   this->rateStart = rateStart;
   this->rateEnd = rateEnd;
   this->pitchStart = pitchStart;
   this->pitchEnd = pitchEnd;
   this->bLinkRatePitch = bLinkRatePitch;
   this->rateSlideType = rateSlideType;
   this->pitchSlideType = pitchSlideType;
   this->bRateReferenceInput = bRateReferenceInput;
   this->bPitchReferenceInput = bPitchReferenceInput;
 }

 /* TimeWarper *createTimeWarper(double t0, double t1, double duration, 
                             double rateStart, double rateEnd, SlideType rateSlideType)
 {
   TimeWarper *warper = NULL;
   if (rateStart == rateEnd || rateSlideType == SlideConstant) {
      warper = new LinearTimeWarper(t0, t0, t1, t0+duration);
   } else if(rateSlideType == SlideLinearInputRate) {
      warper = new LinearInputRateTimeWarper(t0, t1, rateStart, rateEnd);
   } else if(rateSlideType == SlideLinearOutputRate) {
      warper = new LinearOutputRateTimeWarper(t0, t1, rateStart, rateEnd);
   } else if(rateSlideType == SlideLinearInputStretch) {
      warper = new LinearInputStretchTimeWarper(t0, t1, rateStart, rateEnd);
   } else if(rateSlideType == SlideLinearOutputStretch) {
      warper = new LinearOutputStretchTimeWarper(t0, t1, rateStart, rateEnd);
   } else if(rateSlideType == SlideGeometricInput) {
      warper = new GeometricInputTimeWarper(t0, t1, rateStart, rateEnd);
   } else if(rateSlideType == SlideGeometricOutput) {                
      warper = new GeometricOutputTimeWarper(t0, t1, rateStart, rateEnd);
   }
   return warper;
 }
 */

 #define SBSMSCHUNKS 1
 #define SBSMSTOTALSAMPLES ((inlen / SBSMSCHUNKS) / PABYTESPERFRAME) // number of frames passed in call to Process()
 #define SBSMSMAXBLOCKSIZE ((inlen / SBSMSCHUNKS) / PABYTESPERFRAME) // blocksize for callback
 
 audio *l_SBSMSBufa = NULL;
 audio *l_bufa = NULL;
 float *l_leftTrackf = NULL;
 float *l_rightTrackf = NULL;
 void mafreeSBSMSbufs();
 void mafreeSBSMSbufs()
 {
	if(l_bufa)
	{
		free(l_bufa);
		l_bufa = NULL;
	}
	if(l_leftTrackf)
	{
		free(l_leftTrackf);
		l_leftTrackf = NULL;
	}
	if(l_rightTrackf)
	{
		free(l_rightTrackf);
		l_rightTrackf = NULL;
	}
	if(l_SBSMSBufa)
	{
		free(l_SBSMSBufa);
		l_SBSMSBufa = NULL;
	}
 } // mafreeSBSMSbufs


 bool EffectSBSMS::Process(char *indata, unsigned long inlen, char *outdata, unsigned long *poutlen)
 {
	long pos = 1;
   
	Slide rateSlide(rateSlideType, (float)rateStart, (float)rateEnd);
	Slide pitchSlide(pitchSlideType, (float)pitchStart, (float)pitchEnd);
	// mTotalStretch = rateSlide.getTotalStretch(); // 1.0

	l_pindata = (short *)indata;
	l_poutdata = (short *)outdata;

	for(int j = 0; j < SBSMSCHUNKS; j++)
	{
		if(g_state.stop & (STOP_ALL | STOP_DECODER))
			break;

		sampleCount end = SBSMSTOTALSAMPLES;
		l_blockSize = SBSMSMAXBLOCKSIZE;

		// SBSMS has a fixed 44100 sample rate - we just convert to its sample rate and then convert back
		float srTrack = (float)g_state.insamplerate;
		float srProcess = (float)(bLinkRatePitch ? srTrack : 44100.0);

		// the resampler needs a callback to supply its samples
		ResampleBuf rb;
		sampleCount maxBlockSize = SBSMSMAXBLOCKSIZE;
		rb.blockSize = maxBlockSize;

		if(l_bufa == NULL)
			if((l_bufa = (audio*)calloc((size_t)rb.blockSize, sizeof(audio))) == NULL)
				goto Processerrorexit;
		rb.buf = l_bufa;

		if(l_leftTrackf == NULL)
			if((l_leftTrackf = (float*)calloc((size_t)maxBlockSize, sizeof(float))) == NULL)
				goto Processerrorexit;
		rb.leftTrack = l_leftTrackf;

		if(l_rightTrackf == NULL)
			if((l_rightTrackf = (float*)calloc((size_t)maxBlockSize, sizeof(float))) == NULL)
				goto Processerrorexit;
		rb.rightTrack = l_rightTrackf;
 
		// Samples in
		sampleCount samplesIn = end;
            
		// Samples for SBSMS to process after resampling
		sampleCount samplesToProcess = (sampleCount)((float)samplesIn * (srProcess / srTrack));
		SlideType outSlideType;
		SBSMSResampleCB outResampleCB;
		sampleCount processPresamples = 0;

		if(bLinkRatePitch)
		{
			rb.bPitch = true;
			outSlideType = rateSlideType;
			outResampleCB = resampleCB;
			rb.offset = 0;
			rb.end = end;
			rb.iface = new SBSMSInterfaceSliding(&rateSlide, &pitchSlide, bPitchReferenceInput,
														samplesToProcess, 0, NULL);              
		}
		else
		{
			rb.bPitch = false;
			outSlideType = (srProcess==srTrack?SlideIdentity:SlideConstant);
			outResampleCB = postResampleCB;
			rb.ratio = srProcess / srTrack;
			rb.quality = new SBSMSQuality(&SBSMSQualityStandard);
			rb.resampler = new Resampler(resampleCB, &rb, srProcess == srTrack ? SlideIdentity : SlideConstant);
			rb.sbsms = new SBSMS(2, rb.quality, true);
			rb.SBSMSBlockSize = rb.sbsms->getInputFrameSize();

			if(l_SBSMSBufa == NULL)
				if((l_SBSMSBufa = (audio*)calloc((size_t)rb.SBSMSBlockSize, sizeof(audio))) == NULL)
					goto Processerrorexit;
			rb.SBSMSBuf = l_SBSMSBufa;

			processPresamples = 0;
			rb.offset = 0;
			rb.end = end;
			rb.iface = new SBSMSEffectInterface(rb.resampler, &rateSlide, &pitchSlide, bPitchReferenceInput,
											(long)samplesToProcess, (long)processPresamples, rb.quality);
		}

		Resampler resampler(outResampleCB, &rb, outSlideType);
            
		audio outBuf[SBSMSOutBlockSize];
           
		// Samples in output after SBSMS
		sampleCount samplesToOutput = rb.iface->getSamplesToOutput();

		// Samples in output after resampling back
		sampleCount samplesOut = (sampleCount)((float)samplesToOutput * (srTrack / srProcess));

		pos = 1;
		long outputCount = -1;
		while((pos += outputCount) < samplesOut && outputCount)
		{
			if(g_state.stop & (STOP_ALL | STOP_DECODER))
				break;
			long frames;
			if(pos + SBSMSOutBlockSize > samplesOut)
				frames = (long)(samplesOut - pos);
			else
				frames = SBSMSOutBlockSize;
			outputCount = resampler.read(outBuf, frames);
			
#if defined(MAPLAYSBSMS)
			if(g_state.pdecoder == (threadparametertype)maPlaySbsms)
			{
				short pcmbuffer[SBSMSOutBlockSize * 2];
				l_poutdata = pcmbuffer;
				for(long i = 0; i < outputCount; i++)
				{
					// *l_poutdata++ = (short)lrintf(outBuf[i][0]);
					int j;
					if((j = lrintf(outBuf[i][0])) < -32768)
						*l_poutdata++ = -32768;
					else if(j > 32767)
						*l_poutdata++ = 32767;
					else 
						*l_poutdata++ = (short)j;
					// *l_poutdata++ = (short)lrintf(outBuf[i][1]);
					if((j = lrintf(outBuf[i][1])) < -32768)
						*l_poutdata++ = -32768;
					else if(j > 32767)
						*l_poutdata++ = 32767;
					else 
						*l_poutdata++ = (short)j;
				}
				// pass data on to output processing chain
				if(maOutChain1((char *)pcmbuffer, outputCount * PABYTESPERFRAME) != 0)
				{
					machar mastr[MAXLOGLINELEN];
					masprintf(mastr, MAXLOGLINELEN,
						L"EffectSBSMS::Process maOutChain1 failed : len=%d g_state.stop=x%x",
												outputCount * PABYTESPERFRAME, g_state.stop);
					maLog(mastr);
					return 1;
				}
			}
			else
#endif // MAPLAYSBSMS
			{
				for(long i = 0; i < outputCount; i++)
				{
					*l_poutdata++ = (short)outBuf[i][0];
					*l_poutdata++ = (short)outBuf[i][1];
				}
			}
			g_state.position = (mauint64)(LROUND(((double)pos * g_state.duration) / samplesOut));
		}
	}

	*poutlen += (pos * PABYTESPERFRAME);
	mafreeSBSMSbufs();
	return true;

Processerrorexit:
	mafreeSBSMSbufs();
	maLog(L"EffectSBSMS::Process calloc failed");
	*poutlen = 0;
	return false;

 } // EffectSBSMS::Process


 #ifdef __cplusplus
  extern "C"
  {
 #endif
  float effectsval(int effect);
  int l_setupeffectsflagSBSMS = 0;
  int SetupEffectsSBSMS();
  int SetupEffectsSBSMS()
  {
	double semitones;
	double pitchStart;
	double pitchEnd;
	double rateStart;
	double rateEnd;
	double speed;

	if(l_setupeffectsflagSBSMS)
		return 1;
		
	l_setupeffectsflagSBSMS = 1;
	
  #if !defined(MAPLAYSBSMS)
	if(AnyEffectOn() != EFFECTSSBSMS)
		goto setupexitSBSMS; // Effects off or none enabled or not sbsms
  #endif

	// 0 - 48 pitch sliders, -25 - +25 semitones to ratio
	semitones  = (g_sets.effects & EFFECTSPITCH) ? (double)effectsval(EFFECTSPITCH) : 0.0;
	pitchStart =  1.0 + ((100.0 * (pow(2.0, semitones / 12.0) - 1.0))) / 100.0;
	
	// 0 - 100 tempo slider to .48 to 2.48 ratio
	rateStart = (g_sets.effects & EFFECTSTEMPO) ? ((double)g_sets.temposlider / 50.0) + 0.48 : 1.0;

	// 0 - 100 speed slider, .48 to 2.48 ratio
	speed = (g_sets.effects & EFFECTSSPEED) ? ((double)g_sets.speedslider / 50.0) + 0.48 : 1.0;

	pitchStart *= speed;
	rateStart *= speed;
	pitchEnd = pitchStart;
	rateEnd = rateStart;

	if(l_pEffectSBSMS)
	{
		delete(l_pEffectSBSMS);
		l_pEffectSBSMS = NULL;
	}
	if((l_pEffectSBSMS = new EffectSBSMS()) == NULL)
		return 1;

	l_pEffectSBSMS->setParameters(rateStart, rateEnd,
								pitchStart, pitchEnd,
								SlideLinearOutputRate,
								SlideLinearOutputRate,
								false, // bLinkRatePitch
								false,
								false);

  #if !defined(MAPLAYSBSMS)
setupexitSBSMS:
  #endif
	l_setupeffectsflagSBSMS = 0; // ok
	
	return 0;
	
  } // SetupEffectsSBSMS


  #if !defined(MAPLAYSBSMS)
   int ApplyEffectsSBSMS(char *indata, unsigned long inlen, char *outdata, unsigned long *poutlen)
   {
	if(poutlen)
		*poutlen = 0;

	if(inlen > 0)
	{
		// Setup the sbsms object for processing the sound
		if(/* !g_pSBSMS || */ g_effectschange)
		{
			if(SetupEffectsSBSMS())
				return 1;
			g_effectschange = 0;
		}

		if(l_pEffectSBSMS)
			l_pEffectSBSMS->Process(indata, inlen, outdata, poutlen);
	}

	return 0; // ok

   } // ApplyEffectsSBSMS
  #endif // !MAPLAYSBSMS


  void CleanupEffectsSBSMS()
  {
	mafreeSBSMSbufs();
	if(l_pEffectSBSMS)
	{
		delete l_pEffectSBSMS;
		l_pEffectSBSMS = NULL;
	}
  } // CleanupEffectsSBSMS

 #ifdef __cplusplus
 }
 #endif


#if defined(MAPLAYSBSMS)
 #ifdef __cplusplus
 extern "C"
 {
 #endif


 // This is called when "Effects" and "High quality" are on.
 void maPlaySbsms()
 {
	char *outdata = NULL;
	unsigned long inlen;
	unsigned long outlen;
	unsigned long *poutlen = &outlen;
	threadparametertype parameter;
	int posbegon = g_posbegon;
	int posendon = g_posendon;
	int reptsong = g_reptsong;

	if(!g_state.inpath)
	{
		maLog(L"maPlaySbsms error null filename");
		goto maplaysbsmsexit;
	}

	/*
	if(g_state.infiletype == kMAMAA)
	{
		maLog(L"maPlaySbsms High quality effects not implemented for Audio In");
		goto maplaysbsmsexit;
	}

	if(g_state.infiletype == kMACDA)
	{
		maLog(L"maPlaySbsms High quality effects not implemented for CD");
		goto maplaysbsmsexit;
	}
	*/

	g_state.pcm = g_state.pcmpos = NULL;

	g_state.loadingsbsms = 1; //flag for app title

	g_state.size = mafilereadraw(NULL, g_state.inpath, 1);

	// set a suitable buffer size for decoded data
	g_state.pcmmax *= (size_t)g_state.size;
	
	// decode the whole song into sbsms pcm buf
	parameter = g_state.pdecoder;
	g_state.pdecoder = (threadparametertype)maPlaySbsms;
	g_state.pcmnbytes = 1; // non-zero, used as flag in maOutChain2
	g_posbegon = 0;
	g_posendon = 0;
	g_reptsong = 0;
	parameter(); // maPlayXXX
	g_state.loadingsbsms = 2; // done flag for app title
	g_posbegon = posbegon;
	g_posendon = posendon;
	g_reptsong = reptsong;

	inlen = (unsigned long)g_state.pcmnbytes;

	// format of this decoder output
 #ifdef WORDS_BIGENDIAN
	g_state.inwordsbigendian = 1;
 #else
	g_state.inwordsbigendian = 0;
 #endif
	g_state.ininterleaved = 1;
	g_state.dumpoff = 1;
	g_state.psf = NULL;
	g_state.position = 0;
	g_state.inchannels = 2; 

 	if(maIOStart() < 0)
	{
		maLog(L"maPlaySbsms maIOStart failed");
		goto maplaysbsmsexit;
	}

	g_state.duration = (mauint64)(LROUND((250.0 * (double)inlen) / g_state.insamplerate));
	// if(!g_posendon || g_posend > g_state.duration)
		g_posend = g_state.duration;

	for(; ;) // repeat loop
	{
		if(SetupEffectsSBSMS())
		{
			maLog(L"maPlaySbsms SetupEffectsSBSMS failed");
			goto maplaysbsmsexit;
		}

		g_state.pcmnbytes = 0;

		// call sbsms Process loop, this plays the whole song
		if(l_pEffectSBSMS)
			l_pEffectSBSMS->Process(g_state.pcm, inlen, outdata, poutlen);

		if(g_state.stop || g_loadedquit || !g_reptsong)
			break;

	} // repeat loop

	// finish

	maOutChainFlush();

maplaysbsmsexit:
	g_state.loadingsbsms = 0;
	if(g_state.pcm)
	{
		MAFREE(g_state.pcm);
		g_state.pcm = NULL;
	}

 } // maPlaySbsms

 #ifdef __cplusplus
 }
 #endif
#endif // !MAPLAYSBSMS

#endif // USE_LIBSBSMS
