// maGUInone.cpp

/*
 * MoreAmp
 * Copyright (C) 2004-2014 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include "maConfig.h" // included by all moreamp files

// This file contains the code to run without a gui and stubs for gui functions.

#if defined(MAGUINONE)

// #define MATRACEGUINONE

#ifdef __MAOSX__
 #error __MAOSX__ is defined. NoGUI builds on OS X should be __MACUNIX__
 #error use `./configure --with-macunix --without-gui ... `
 // and __MAUNIX__ is defined whenever __MACUNIX__ is defined
#endif

#include <stdio.h>
#include <string.h>

#ifndef __MAWIN__
 #include <unistd.h>
#endif

#ifdef __MAUNIX__
 #include <signal.h>
#endif

#if defined(__MAWIN__) // have stdin
 #include <windows.h>
#endif

#if defined(__MAUNIX__) // have stdin
 #include <errno.h>
 #include <termios.h>
 #include <sys/time.h>
 #include <stdlib.h>
 #include <sys/stat.h>
#endif

#include "maGUI.h"
#include "maUtils.h"
#include "maPlayer.h"
#include "maSongList.h"
#include "maTextView.h"

#ifdef __cplusplus
extern "C"
{
#endif

#if defined(__MAUNIX__) || defined(__MAWIN__) 
 int l_timerrunloop = 0;
#endif

int l_timerplay = 0;
int l_timerchangeplaylist = 0;
int l_timerplayplaylist = 0;
int l_appquitted = 0;
int l_prompting = 0;

fschar l_fschar[STRLEN16];
fschar *maGetChar();

void maGuiLog(machar *str);

#if defined(MATRACEGUINONE)
 #define maTrace maGuiLog
#endif


void maGuiLog(machar *mastr2)
{
#if defined(__MAUNIX__)
	fschar fsstr[MAX_PATH];
	ma2fs(fsstr, mastr2);
	printf("%s\n", fsstr);
#else
	maprintf(L"%ls%ls", mastr2, g_maeol);
#endif
	fflush(stdout);
}


#if defined(__MAUNIX__)
 void onsigint(int sig);
 void onsigint(int sig)
 {
	DoAppQuit();
 }


 struct termios ttybu;
 int l_ttyinit = 0;
 void mattyexit(void);
 void mattyexit(void)
 {
	if(l_ttyinit)
	{
		tcsetattr(0, TCSANOW, &ttybu);
		l_ttyinit = 0;
	}
 }


 int mattyinit();
 int mattyinit()
 {
	struct termios tty;
	
	if(!isatty(0)) // piped input
		return 1;

	// if() // started in background e.g. 'moreampnogui &') // FIXME
	// {
	//		return 2; // else tcsetattr causes a program exit
	// }
	
	if(tcgetattr(0, &tty))
	{
		printf("mattyinit tcgetattr errno=%d\n", errno);
		return 3;
	}

	ttybu = tty;
	tty.c_iflag &= ~(IGNBRK | BRKINT | PARMRK | ISTRIP | INLCR | IGNCR | ICRNL | IXON);
	tty.c_oflag |= OPOST;
	tty.c_lflag &= ~(ECHO | ECHONL | ICANON | IEXTEN);
	tty.c_cflag &= ~(CSIZE | PARENB);
	tty.c_cflag |= CS8;
	tty.c_cc[VMIN] = 1;
	tty.c_cc[VTIME] = 0;
	
	if(tcsetattr(0, TCSANOW, &tty))
	{
		printf("mattyinit tcsetattr errno=%d\n", errno);
		return 4;
	}

	atexit(mattyexit);
	l_ttyinit = 1;
	
	return 0;
	
 } // mattyinit


 #define MAEOF WEOF
 #define magetchar getwchar
 fschar *maGetChar() // async stdin read
 {
	int i;
	int err;
	maint d;
	fd_set files;

	if(!l_ttyinit)
	{
		mattyinit();
		if(!l_ttyinit)
			return NULL;
	}
	
    FD_ZERO(&files);
    FD_SET(STDIN_FILENO, &files);

    timeval timeout = {0, 0};
	i = 0;
    l_fschar[0] = l_fschar[1] = l_fschar[2] = l_fschar[3] = l_fschar[4] = fsT('\0');
	while((err = select(1, &files, NULL, NULL, &timeout)) != 0)
	{
		if(FD_ISSET(STDIN_FILENO, &files))
		{
			if(err < 0) // error
			{
				printf("maGetChar select errno=%d\n", errno);
				return NULL;
			}
			d = magetchar(); // int to char (bad) or win_t to wchar_t
			if(d == MAEOF) // error or eof
			{
				if(ferror(stdin))
				{
					if((err = errno) == EILSEQ) // 92 == EILSEQ
					{
						printf("maGetChar errno=%d=EILSEQ\n", err);
						printf("Your terminal is probably not configured for Unicode\n");
						printf("See 'No gui' in the README\n");
						printf("Quitting %s\n", MOREAMP_NAMEos);
						DoAppQuit();
					}
					else
						printf("maGetChar errno=%d\n", err);
				}
				clearerr(stdin);
				return NULL;
			}
			else if(d != (maint)MAEOL[0])
			{
				if(i < 4)
					l_fschar[i++] = d;
			}
		}
	}	
	if(i > 0 && i <= 4) 
		return l_fschar;

	return NULL;

 } // maGetChar unix
#endif // __MAUNIX__


#if defined(__MAWIN__) // have stdin
 fschar *maGetChar()
 {
	HANDLE console;
	INPUT_RECORD input;
	DWORD count;

	console = GetStdHandle(STD_INPUT_HANDLE);

	do
	{
		if(GetNumberOfConsoleInputEvents(console, &count) == 0) // broken in wine?
		{
			printf("maGetChar GetNumberOfConsoleInputEvents error\n");
			return NULL;
		}
		count = 0;
		GetNumberOfConsoleInputEvents(console, &count);
		
		if(count == 0)
			return NULL;

		if(ReadConsoleInput(console, &input, 1, &count) == 0 || count != 1)
		{
			printf("maGetChar ReadConsoleInput error\n");
			return NULL;
		}
	} while(input.EventType != KEY_EVENT || !input.Event.KeyEvent.bKeyDown ||
			input.Event.KeyEvent.uChar.UnicodeChar == 0);

    l_fschar[1] = l_fschar[2] = l_fschar[3] = l_fschar[4] = fsT('\0');
	l_fschar[0] = input.Event.KeyEvent.uChar.UnicodeChar;

	return l_fschar;
	
 } // maGetChar win
#endif


#if defined(__MAWIN__)
 int wmain(int argc, wchar_t *argv[]) // MAGUINONE
#else
 int main(int argc, char *argv[]) // MAGUINONE
#endif
{
	g_argc = argc;
	g_argv = argv;

 #if defined(__MAWIN__)
	if(argc == 2 && !fsstrcmp(argv[1], fsT("maclearlocks"))) // run this in case we have a stuck lock
 #else
	if(argc == 2 && !strcmp(argv[1], "maclearlocks")) // run this in case we have a stuck lock
 #endif
	{
		InstanceClearAll();
		printf("cleared locks\n");
		return 0;
	}

 #if defined(__MAWIN__)
	if(argc == 2 && !fsstrcmp(argv[1], fsT("malistlocks"))) // run this to see locks
 #else
	if(argc == 2 && !strcmp(argv[1], "malistlocks")) // run this to see locks
 #endif
	{
		fssprintf(g_logpath, MAX_PATH, fsT("malistlocks.txt"));
		InstanceListAll();
		printf("listed locks\n");
		return 0;
	}

#if defined(MAKEYBOARD)
	fschar *fskey;
#endif
#if defined(__MAUNIX__) // have stdin
	// mattyinit();
#endif

#if defined(__MAUNIX__)
	struct sigaction sa;
	sa.sa_handler = onsigint;
	sigemptyset(&sa.sa_mask);
	sa.sa_flags = 0;
	sigaction(SIGINT, &sa, 0); // SIGKILL
#endif
	
	mastrcpy(&g_abouttext[mastrlen(g_abouttext) - STRLEN64], L" No GUI"); 

	if(!OnInit_())		
		return 1; // failed

	// main loop
#if defined(__MAUNIX__) || defined(__MAWIN__)
	if(g_argc <= 1) // no args
	{
		if(l_timerplay)
		{
			l_timerplay = 0;
			DoPlay();
		}
	}
	else if(g_argc > 1) // args
	{
		if(l_timerplay)
		{
			l_timerplay = 0;
			DoPlay();
		}
		else if(l_timerchangeplaylist)
		{
			l_timerchangeplaylist = 0;
			ChangePlaylist(g_changeplaylist);
		}	
		else if(l_timerplayplaylist)
		{
			l_timerplayplaylist = 0;
			PlayPlaylistB(g_newplaylist);
		}	
	}

	for (; ;)
	{
		if(l_timerrunloop)
		{
 #if defined(MAKEYBOARD)
			if(!l_prompting && (fskey = maGetChar()) != NULL)
				maOnKey(fskey);
			maSleep(l_timerrunloop);
 #endif
			if(l_timerplay)
			{
				l_timerplay = 0;
				DoPlay();
			}
			l_timerrunloop = RunLoop();
		}
		else
		{
			if(l_appquitted)
				break;
 #if defined(MAKEYBOARD)
			if(!l_prompting && (fskey = maGetChar()) > 0)
				maOnKey(fskey);
 #endif
			if(l_timerplay)
			{
				l_timerplay = 0;
				DoPlay();
			}
			else if(l_timerchangeplaylist)
			{
				l_timerchangeplaylist = 0;
				ChangePlaylist(g_changeplaylist);
			}	
			else if(l_timerplayplaylist)
			{
				l_timerplayplaylist = 0;
				PlayPlaylistB(g_newplaylist);
			}	
			maSleep(1000);
		}
	}
#endif // defined(__MAUNIX__) || defined(__MAWIN__)
	
	return 0;
	
} // main


void StaticTextSetText(int windowid, int statictext, machar *text)
{
#ifdef MATRACEGUINONE
	maTrace(L"StaticTextSetText");
#endif

	if(windowid == WINDOWPROMPTNUM)
	{
#if defined(__MAUNIX__)
		fschar fsstr[MAX_PATH];
		ma2fs(fsstr, text);
		printf("%s\n", fsstr);
#else
		maprintf(L"%ls%ls", text, g_maeol);
#endif
		fflush(stdout);
	}
}


int	AppWindowMake()
{
#ifdef MATRACEGUINONE
	maTrace(L"AppWindowMake");
#endif
	g_w[WINDOWAPP] = (void *)-1;
	return 0;	
} // AppWindowMake


void guiAppYield() 
{
#ifdef MATRACEGUINONE
	maTrace(L"guiAppYield");
#endif
} // guiAppYield


void CheckBoxDisable(int windowid, int checkbox)
{
#ifdef MATRACEGUINONE
	maTrace(L"CheckBoxDisable");
#endif
}

void CheckBoxEnable(int windowid, int checkbox)
{

}


int CheckBoxGetValue(int windowid, int checkbox)
{
#ifdef MATRACEGUINONE
	maTrace(L"CheckBoxGetValue");
#endif
return 0;
}


void CheckBoxHide(int windowid, int checkbox)
{
#ifdef MATRACEGUINONE
	maTrace(L"CheckBoxHide");
#endif
}


void CheckBoxSetDescriptor(int windowid, int checkbox, machar *desc)
{
#ifdef MATRACEGUINONE
	maTrace(L"CheckBoxSetDescriptor");
#endif
}


void MenuSetLabel(int item, machar *label)
{
#ifdef MATRACEGUINONE
	maTrace(L"MenuSetLabel");
#endif
}


void MenuGetLabel(int item, machar *label)
{
#ifdef MATRACEGUINONE
	maTrace(L"MenuGetLabel");
#endif
}


void UpdateSubMenusRecent()
{
#ifdef MATRACEGUINONE
	maTrace(L"UpdateSubMenusRecent");
#endif
}


/*
void CheckBoxSetFocus(int windowid, int checkbox)
{
#ifdef MATRACEGUINONE
	maTrace(L"CheckBoxSetFocus");
#endif
}
*/


void CheckBoxSetValue(int windowid, int checkbox, int value)
{
#ifdef MATRACEGUINONE
	maTrace(L"CheckBoxSetValue");
#endif
}


void CheckBoxShow(int windowid, int checkbox)
{
#ifdef MATRACEGUINONE
	maTrace(L"CheckBoxShow");
#endif
}


void DisplayGetSize(int *x, int *y)
{
#ifdef MATRACEGUINONE
	maTrace(L"DisplayGetSize");
#endif
}


void DoTimerRunLoop(int delay)
{
#ifdef MATRACEGUINONE
	maTrace(L"DoTimerRunLoop");
#endif

#if defined(__MAUNIX__)  
	maSleep(delay);
	l_timerrunloop = delay;
#endif

#if defined(__MAWIN__)  
	maSleep(delay);
	l_timerrunloop = delay;
#endif
}


void DoTimerPlay(int delay)
{
#ifdef MATRACEGUINONE
	maTrace(L"DoTimerPlay");
#endif
	maSleep(delay);
	l_timerplay = 1;
}


/* void DoTimerStop(int delay)
{
#ifdef MATRACEGUINONE
	maTrace(L"DoTimerStop");
#endif
	maSleep(delay);
	DoStop();
}
*/

void DoTimerChangePlaylist(int delay)
{
#ifdef MATRACEGUINONE
	maTrace(L"DoTimerChangePlaylist");
#endif
	maSleep(delay);
	l_timerchangeplaylist = 1;
}


void DoTimerPlayPlaylist(int delay)
{
#ifdef MATRACEGUINONE
	maTrace(L"DoTimerPlayPlaylist");
#endif
	maSleep(delay);
	l_timerplayplaylist = 1;
}


void EditTextEnable(int windowid, int edittext)
{
#ifdef MATRACEGUINONE
	maTrace(L"EditTextEnable");
#endif
}


void EditTextDisable(int windowid, int edittext)
{
#ifdef MATRACEGUINONE
	maTrace(L"EditTextDisable");
#endif
}


void EditTextSetText(int windowid, int edittext, machar *text)
{
#ifdef MATRACEGUINONE
	maTrace(L"EditTextSetText");
#endif
}


void EditTextGetText(int windowid, int edittext, machar *outBuffer, int inBufferLength, long *outTextLength, long from, long to)
{
#ifdef MATRACEGUINONE
	maTrace(L"EditTextGetText");
#endif

	if(windowid == WINDOWPROMPTNUM)
	{
		machar buf[MAPROMPTMAX + 2];
	
#if defined(__MAUNIX__)
		mattyexit(); // disable tty, reenable stdin
					 // then first call to maGetChar calls mattyinit	
#endif

		if(fgetws(buf, MAPROMPTMAX, stdin))
		{
			mastrncpy(outBuffer, buf, MAPROMPTMAX);
			outBuffer[MAPROMPTMAX] = L'\0';
			*outTextLength = (long)mastrlen(outBuffer);
		}
		else
		{
			outBuffer[0] = L'\0';
			*outTextLength = 0;
		}
	}
	
} // EditTextGetText


#if !defined(NOTEXTVIEWER)
 void EditTextGetSelection(int windowid, int edittext, long *start, long *end)
 {
 #ifdef MATRACEGUINONE
	maTrace(L"EditTextGetSelection");
 #endif
 }


 void EditTextSetSelection(int windowid, int edittext, long start, long end)
 {
 #ifdef MATRACEGUINONE
	maTrace(L"EditTextGetSelection");
 #endif
 }


 void EditTextSetEditable(int windowid, int edittext, int editable)
 {
 #ifdef MATRACEGUINONE
	maTrace(L"EditTextSetEditable");
 #endif
 }


 void EditTextSetFocus(int windowid, int edittext)
 {
 #ifdef MATRACEGUINONE
	maTrace(L"EditTextSetFocus");
 #endif
 }


 /* int EditTextLoadFile(int windowid, int edittext, fschar *fpath)
 {
 #ifdef MATRACEGUINONE
	maTrace(L"EditTextLoadFile");
 #endif
	return -1; // not implemented
 }


 int EditTextSaveFile(int windowid, int edittext, fschar *fpath)
 {
 #ifdef MATRACEGUINONE
	maTrace(L"EditTextSaveFile");
 #endif
	return -1; // not implemented
 }
 */

 int EditTextGetSize(int windowid, int edittext, size_t *pfsize)
 {
 #ifdef MATRACEGUIGTK
	maTrace(L"EditTextGetSize");
 #endif
	return -1; // not implemented
 }


 void EditTextShowPosition(int windowid, int edittext, long pos)
 {
 #ifdef MATRACEGUINONE
	maTrace(L"EditTextShowPosition");
 #endif
	// not implemented
 }
#endif

void PushButtonGetDescriptor(int windowid, int edittext, machar *text)
{
#ifdef MATRACEGUINONE
	maTrace(L"PushButtonGetDescriptor");
#endif
}

/*
void SplashWindowMake()
{
#ifdef MATRACEGUINONE
	maTrace(L"SplashWindowMake");
#endif
	g_w[WINDOWSPLASH] = (void *)-1;
}
*/


void EqualizerWindowMake()
{
#ifdef MATRACEGUINONE
	maTrace(L"EqualizerWindowMake");
#endif
	g_w[WINDOWEQ] = (void *)-1;
} // EqualizerWindowMake


void Equalizer2WindowMake()
{
#ifdef MATRACEGUINONE
	maTrace(L"Equalizer2WindowMake");
#endif
	g_w[WINDOWEQ2] = (void *)-1;
} // EqualizerWindowMake


void EffectsWindowMake()
{
#ifdef MATRACEGUINONE
	maTrace(L"EffectsWindowMake");
#endif
	g_w[WINDOWEFFECTS] = (void *)-1;
} // EffectsWindowMake


#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
void AudioCDWindowMake()
{
#ifdef MATRACEGUINONE
	maTrace(L"AudioCDWindowMake");
#endif
	g_w[WINDOWAUDIOCD] = (void *)-1;
} // AudioCDWindowMake
#endif


void AdvancedWindowMake()
{
#ifdef MATRACEGUINONE
	maTrace(L"AdvancedWindowMake");
#endif
	g_w[WINDOWADVANCED] = (void *)-1;
} // AdvancedWindowMake


void FileOutWindowMake()
{
#ifdef MATRACEGUINONE
	maTrace(L"FileOutWindowMake");
#endif
	g_w[WINDOWFILEOUT] = (void *)-1;
} // FileOutWindowMake


void PromptNumWindowMake()
{
#ifdef MATRACEGUINONE
	maTrace(L"PromptNumWindowMake");
#endif

	g_w[WINDOWPROMPTNUM] = (void *)-1;
	
} // PromptNumWindowMake


void ListAppend(int windowid, int list, machar *text)
{
#ifdef MATRACEGUINONE
	maTrace(L"ListAppend");
#endif
#if defined(__MAUNIX__)
	fschar fsstr[MAX_PATH];
	ma2fs(fsstr, text);
	printf("%s\n", fsstr);
#else
	maprintf(L"%ls%ls", text, g_maeol);
#endif
	fflush(stdout);
	
} // ListAppend


void ListClear(int windowid, int list)
{
#ifdef MATRACEGUINONE
	maTrace(L"ListClear");
#endif
}


int ListGetRowSelected(int windowid, int list)
{
#ifdef MATRACEGUINONE
	maTrace(L"ListGetRowSelected");
#endif
	return -1; // none selected
}


void ListEnsureVisible(int windowid, int list, int row)
{
#ifdef MATRACEGUINONE
	maTrace(L"ListEnsureVisible");
#endif
}


void ListSelect(int windowid, int list, int row)
{
#ifdef MATRACEGUINONE
	maTrace(L"ListSelect");
#endif
}


void ListGetText(int windowid, int list, int row, machar *text)
{
#ifdef MATRACEGUINONE
	maTrace(L"ListGetText");
#endif
}


/* not used void ListSetText(int windowid, int list, int row, machar *text)
{
#ifdef MATRACEGUINONE
	maTrace(L"ListSetText");
#endif
}
*/


void ListSetHeader(int windowid, int list, int col, machar *header)
{
#ifdef MATRACEGUINONE
	maTrace(L"ListSetHeader");
#endif
#if defined(__MAUNIX__)
	fschar fsstr[MAX_PATH];
	ma2fs(fsstr, header);
	printf("%s\n", fsstr);
#else
	maprintf(L"%ls%ls", header, g_maeol);
#endif
	fflush(stdout);
}


void ListSetColumnWidth()
{
#ifdef MATRACEGUINONE
	maTrace(L"ListSetColumnWidth");
#endif
}


void ListFreeze()
{
#ifdef MATRACEGUINONE
	maTrace(L"ListFreeze");
#endif
}


void ListThaw()
{
#ifdef MATRACEGUINONE
	maTrace(L"ListThaw");
#endif
}


void ListWindowMake()
{
#ifdef MATRACEGUINONE
	maTrace(L"ListWindowMake");
#endif
	g_w[WINDOWLIST] = (void *)-1;
} // ListWindowMake


/* int MenuIsChecked(int menu, int item)
{
#ifdef MATRACEGUINONE
	maTrace(L"MenuIsChecked");
#endif
	return 0;
}
*/


void MenuEnable(int menu, int item)
{
#ifdef MATRACEGUINONE
	maTrace(L"MenuEnable");
#endif
}


void MenuDisable(int menu, int item)
{
#ifdef MATRACEGUINONE
	maTrace(L"MenuDisable");
#endif
}


void MenuSetChecked(int menu, int item, int value)
{
#ifdef MATRACEGUINONE
	maTrace(L"MenuSetChecked");
#endif
}


void PopupButtonEnable(int windowid, int popup)
{
#ifdef MATRACEGUINONE
	maTrace(L"PopupButtonEnable");
#endif
}


void PopupButtonDisable(int windowid, int popup)
{
#ifdef MATRACEGUINONE
	maTrace(L"PopupButtonDisable");
#endif
}


void PopupButtonClear(int windowid, int popup)
{
#ifdef MATRACEGUINONE
	maTrace(L"PopupButtonClear");
#endif
}


void PopupButtonAppend(int windowid, int popup, machar *text)
{
#ifdef MATRACEGUINONE
	maTrace(L"PopupButtonAppend");
#endif
}


void PopupButtonSetValue(int windowid, int popup, int value)
{
#ifdef MATRACEGUINONE
	maTrace(L"PopupButtonSetValue");
#endif
}


void PopupButtonSetText(int windowid, int popup, int item, machar *text)
{
#ifdef MATRACEGUINONE
	maTrace(L"PopupButtonSetText");
#endif
}


#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
void PopupButtonGetSelection(int windowid, int popup, int *item)
{
#ifdef MATRACEGUINONE
	maTrace(L"PopupButtonGetSelection");
#endif
}


void PopupButtonSetSelection(int windowid, int popup, int item)
{
#ifdef MATRACEGUINONE
	maTrace(L"PopupButtonSetSelection");
#endif
}
#endif


/* not used void SpinButtonEnable(int windowid, int button)
{
#ifdef MATRACEGUINONE
	maTrace(L"SpinButtonEnable");
#endif
}


void SpinButtonDisable(int windowid, int button)
{
#ifdef MATRACEGUINONE
	maTrace(L"SpinButtonDisable");
#endif
}
*/


void PushButtonPaintUp(int windowid, int button)
{
#ifdef MATRACEGUINONE
	maTrace(L"PushButtonPaintUp");
#endif
}


void PushButtonPaintDown(int windowid, int button)
{
#ifdef MATRACEGUINONE
	maTrace(L"PushButtonPaintDown");
#endif
}


void PushButtonEnable(int windowid, int button)
{
#ifdef MATRACEGUINONE
	maTrace(L"PushButtonEnable");
#endif
}


void PushButtonDisable(int windowid, int button)
{
#ifdef MATRACEGUINONE
	maTrace(L"PushButtonDisable");
#endif
}


void PushButtonSetDescriptor(int windowid, int button, machar *desc)
{
#ifdef MATRACEGUINONE
	maTrace(L"PushButtonSetDescriptor");
#endif
}


int HSliderGetValue(int windowid, int slider)
{
#ifdef MATRACEGUINONE
	maTrace(L"HSliderGetValue");
#endif
	return 0;
}


void HSliderSetValue(int windowid, int slider, int value)
{
#ifdef MATRACEGUINONE
	maTrace(L"HSliderSetValue");
#endif
}


int SliderGetValue(int windowid, int slider)
{
#ifdef MATRACEGUINONE
	maTrace(L"SliderGetValue");
#endif
	return 0;
}


void SliderSetValue(int windowid, int slider, int value)
{
#ifdef MATRACEGUINONE
	maTrace(L"SliderSetValue");
#endif
}


void SliderEnable(int windowid, int slider)
{
#ifdef MATRACEGUINONE
	maTrace(L"SliderEnable");
#endif
}


void SliderDisable(int windowid, int slider)
{
#ifdef MATRACEGUINONE
	maTrace(L"SliderDisable");
#endif
}


void StaticTextSetValue(int windowid, int statictext, int value)
{
#ifdef MATRACEGUINONE
	maTrace(L"StaticTextSetValue");
#endif
}


void WindowClose(int windowid)
{
#ifdef MATRACEGUINONE
	maTrace(L"WindowClose");
#endif
}


void WindowDestroy(int windowid)
{
#ifdef MATRACEGUINONE
	maTrace(L"WindowDestroy");
#endif
}


void WindowGetCoordinates(int windowid, int *xpos, int *ypos, int *xsize, int *ysize)
{
#ifdef MATRACEGUINONE
	maTrace(L"WindowGetCoordinates");
#endif
} // WindowGetCoordinates


void WindowItemHide(int windowid, int item)
{
#ifdef MATRACEGUINONE
	maTrace(L"WindowItemHide");
#endif
} // WindowItemHide


void WindowItemShow(int windowid, int item)
{
#ifdef MATRACEGUINONE
	maTrace(L"WindowItemShow");
#endif
} // WindowItemShow


int WindowIsShown(int windowid)
{
#ifdef MATRACEGUINONE
	maTrace(L"WindowIsShown");
#endif
	return 1;
}


/*
void WindowWidgetSetSize(int windowid, int widget, int xsize, int ysize)
{
#ifdef MATRACEGUINONE
	maTrace(L"WindowWidgetSetSize");
#endif
}
*/


void WindowSetSize(int windowid, int xsize, int ysize)
{
#ifdef MATRACEGUINONE
	maTrace(L"WindowSetSize");
#endif
}


void WindowMove(int windowid, int xpos, int ypos)
 {
 #ifdef MATRACEGUINONE
	maTrace(L"WindowMove");
 #endif
 }


 void WindowShow(int windowid)
 {
 #ifdef MATRACEGUINONE
	maTrace(L"WindowShow");
 #endif
 }


 void WindowSetTitle(int windowid, machar *title)
 {
 #ifdef MATRACEGUINONE
	machar mastr[MAXLOGLINELEN];
	masprintf(mastr, MAXLOGLINELEN, L"maGUI WindowSetTitle %d '%ls'", windowid, title);
	maTrace(mastr);
 #endif
 #if defined(__MAUNIX__)
	fschar fsstr[MAX_PATH];
	ma2fs(fsstr, title);
	printf("%s\n", fsstr);
 #else
	maprintf(L"%ls%ls", title, g_maeol);
 #endif
	fflush(stdout);
 }


 void WindowActivate(int windowid)
 {
 #ifdef MATRACEGUINONE
	maTrace(L"WindowActivate");
 #endif
 }


 int maDirSelector(fschar *dirpath, machar *prompt)
 {
 #if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
 #endif
	machar mastr[MAXLOGLINELEN];
 #ifdef MATRACEGUINONE
	maTrace(L"maDirSelector");
 #endif
	masprintf(mastr, MAXLOGLINELEN, L"maGUI maDirSelector '%ls' '%ls'", prompt, fs2ma(mastr2, dirpath));
	maGuiLog(mastr);
	return 1; // cancelled
 }


 int maFileDialog(fschar *selectedpath, int windowid, machar *prompt, fschar *defdir, machar *deffile, machar *wildcard, int flags)
 {
 #if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
 #endif
	machar mastr[MAXLOGLINELEN];
 #ifdef MATRACEGUINONE
	maTrace(L"maFileDialog");
 #endif
	masprintf(mastr, MAXLOGLINELEN, L"maGUI maFileDialog '%ls' '%ls'", prompt, fs2ma(mastr2, defdir));
	maGuiLog(mastr);
	return 1; // cancelled
 }


 void maFileDialogMFree(maArray *parray)
 {
 #ifdef MATRACEGUINONE
	maTrace(L"maFileDialogMFree");
 #endif
 }


 int maFileDialogM(maArray *parray, int windowid, machar *prompt, fschar *defdir, machar *deffile, machar *wildcard, int flags)
 {
 #ifdef MATRACEGUINONE
	maTrace(L"maFileDialogM");
 #endif
	return 1; // cancelled
	
 } //maFileDialogM


 int maMessageBox(machar *s1, machar *s2, int flags)
 {
	machar mastr[MAXLOGLINELEN];
	
	masprintf(mastr, MAXLOGLINELEN, L"maGUI maMessageBox %ls %ls", s1, s2);
 #ifdef MATRACEGUINONE
	maTrace(mastr);
 #endif
	maGuiLog(mastr);
	return maNO;
	
 } // maMessageBox


 int ViewerWindowMake(int windowid, machar *wintitle)
 {
 #ifdef MATRACEGUINONE
	maTrace(L"ViewerWindowMake");
 #endif
 #if defined(MAKEYBOARD)
	size_t len;
	machar *p0;
	fschar *path;

 	switch(windowid)
	{
		case WINDOWKEYBINDINGS:
			path = g_keybindingspath;
		break;
		case WINDOWLOG:
			path = g_logpath;
		break;
		case WINDOWREADME:
			path = g_readmepath;
		break;
		case WINDOWCATALOG:
			path = g_catalogpath;
		break;
		default:
			return 1;
		break;
	}
	
 	if(path[0] != fsT('\0') && maFileExists(path))
	{
		len = mafileread(&p0, path);
		if(len <= 0 || p0 == NULL)
		{			
			if(p0 != NULL)
				MAFREE(p0);
			maLog(L"ViewerWindowMake mafileread failed");
			return 1;
		}

  #if defined(__MAUNIX__)
		char *p1;
		if((p1 = (char *)MAMALLOC((len + 8) * 4)) == NULL)
		{
			MAFREE(p0);
			maLog(L"ViewerWindowMake MAMALLOC failed");
			return 1;
		}
		ma2fs(p1, p0);
		MAFREE(p0);
		printf("%s\n\n", p1);
		MAFREE(p1);
  #else
		maprintf(L"%ls%ls%ls", p0, g_maeol, g_maeol);
  #endif
		fflush(stdout);
	}
 
	return 0;
 
 } // ViewerWindowMake
#endif


void AboutWindowMake()
{
 #ifdef MATRACEGUINONE
	maTrace(L"AboutWindowMake");
 #endif
	maprintf(L"%ls%ls%ls", g_abouttext, g_maeol, g_maeol);
	fflush(stdout);

	return;
 
} // AboutWindowMake


void maGUIfree()
{
}


void maGUIQuit()
{
	l_appquitted = 1;
} // maGUIQuit


int maModKeyDown()
{
	return 0;
}

#ifdef __cplusplus
}
#endif

#endif // MAGUINONE

// eof

