// maMemFile.c

/*
 * MoreAmp
 * Copyright (C) 2004-2014 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

// maMemFile is not mmap()

#include "maConfig.h" // included by all moreamp files

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdio.h>
#include <errno.h>
#include <string.h>

#ifdef __MAUNIX__
 #include <unistd.h>
#endif

#ifdef __MAWIN__
 #include <io.h>
 /*
 #if defined(_MSC_VER)
  #if _MSC_VER <= 1600
   #define fseeko fseek
   #define ftello ftell
  #endif
 #endif
 */
#ifndef fseeko
  #define fseeko _fseeki64
 #endif
 #ifndef ftello
  #define ftello _ftelli64
 #endif
#endif

#include <sys/stat.h>

#if defined(__MAOSX__)
 #include <unistd.h>
#endif

#include "maPlayer.h"
#include "maUtils.h"
#include "maMemFile.h"

#define MAXMEMFP 8 // seven available, see memfdtofp() and memfptofd()

typedef struct memFILE
{
	int errorind;
	int isopen;
	char *songbeg;
	char *songpos;
	char *songend;
} memFILE;

void memShowErr(const machar * msg);

int g_noamem = 1;
int g_nobmem = 1;
int g_noimem = 1;
memFILE g_memfp[MAXMEMFP]; // element 0 unused because 0 == INVALID_HANDLE_VALUE

#ifdef __MAWIN__
 #ifndef MAX_PATH
  #define MAX_PATH 260
 #endif
#endif


int mbmfseeko(FILE *stream, maoff_t64 offset, int whence)
{
	maoff_t64 loffset;
	
	if(g_nobmem)
		return fseeko(stream, offset, whence);
	else
	{
		loffset = offset;
		return memfseeko(stream, loffset, whence);
	}
}


maoff_t64 mbmftello(FILE *stream)
{	
	if(g_nobmem)
		return ftello(stream);
	else
		return memftello(stream);
}


size_t mbmfread(void *ptr, size_t size, size_t nmemb, FILE *stream)
{
	if(g_nobmem)
		return fread(ptr, size, nmemb, stream);

	return memfread(ptr, size, nmemb, stream);
}


/*
int mbmlseek(int fd, off_t offset, int whence)
{
	if(g_nobmem)
		return MALSEEK(fd, offset, whence);

	return memlseek(fd, offset, whence);
}


int mbmfclose(FILE *stream)
{
	if(g_nobmem)
		return fclose(stream);

	return memfclose(stream);
}


long int mbmftell(FILE *stream)
{
	if(g_nobmem)
		return ftell(stream);
		
	return memftell(stream);
}


void mbmrewind(FILE *stream)
{
	if(g_nobmem)
		rewind(stream);
	else
		memrewind(stream);
}


int mbmread(int fd, char *ptr, int size)
{
	if(g_nobmem)
		return MAREAD(fd, ptr, size);
		
	return memread(fd, ptr, size);
}


// int	stat __P((const machar *, struct stat *));
int mbmstat(const fschar *filename, struct stat *buf)
{
	if(g_nobmem)
		return stat((char *)filename, buf);
	else
	{
		int i;
		for(i = 0; i < g_loadednum; i++)
		{
			if(!fsstrcmp(filename, g_loadedpath[i]))
				break;
		}
		
		if(g_loadednum <= 0 || i >= g_loadednum)
			return -1;
			
		return memfstat(i + 1, buf);	
	}
}


// int	fstat __P((int, struct stat *));
int mbmfstat(int j, struct stat *buf)
{
	if(g_nobmem)
		return fstat(j, buf);
		
	return memfstat(j, buf);
}


// int	 feof(FILE *);
int mbmfeof(FILE *stream)
{
	if(g_nobmem)
		return feof(stream);
		
	return memfeof(stream);
}


// int	 ferror(FILE *);
int mbmferror(FILE *stream)
{
	if(g_nobmem)
		return ferror(stream);
		
	return memferror(stream);
}


void mbmclearerr(FILE *stream)
{
	if(g_nobmem)
		clearerr(stream);
	else
		memclearerr(stream);
}


int mbmfgetpos(FILE *stream, fpos_t *pos)
{
	if(g_nobmem)
		return fgetpos(stream, pos);
		
	return memfgetpos(stream, pos);
}


int mbmfsetpos(FILE *stream, const fpos_t *pos)
{
	if(g_nobmem)
		return fsetpos(stream, pos);
		
	return memfsetpos(stream, pos);
}
*/


// mim* functions use g_noimem (called from libid3tag)
FILE *mimfopen(const fschar *filename, fschar *mode)
{
	// this is only called from libid3tag/file.c with "r+b" or "rb"
	// libid3tag is not modified for unicode
	if(g_noimem)
	{
		if(mode[1] == '+')
			return fsfopen(filename, (const fschar *)fsT("r+b"));
		else
			return fsfopen(filename, (const fschar *)fsT("rb"));
	}
		
	return memfopen(filename);
}


int mimfseek(FILE *stream, long offset, int whence)
{
	if(g_noimem)
		return fseek(stream, offset, whence);
		
	return memfseek(stream, offset, whence);
}


size_t mimfread(void *ptr, size_t size, size_t nmemb, FILE *stream)
{
	if(g_noimem)
		return fread(ptr, size, nmemb, stream);
		
	return memfread(ptr, size, nmemb, stream);
}


int mimfclose(FILE *stream)
{
	if(g_noimem)
		return fclose(stream);
		
	return memfclose(stream);
}


long int mimftell(FILE *stream)
{
	if(g_noimem)
		return ftell(stream);
		
	return memftell(stream);
}


void mimrewind(FILE *stream)
{
	if(g_noimem)
		rewind(stream);
	else
		memrewind(stream);
}



int mimfgetpos(FILE *stream, fpos_t *pos)
{
	if(g_noimem)
		return fgetpos(stream, pos);
		
	return memfgetpos(stream, pos);
}


int mimfsetpos(FILE *stream, const fpos_t *pos)
{
	if(g_noimem)
		return fsetpos(stream, pos);
		
	return memfsetpos(stream, pos);
}


void mimclearerr(FILE *stream)
{
	if(g_noimem)
		clearerr(stream);
	else
		memclearerr(stream);
}


/*
int mimlseek(int stream, off_t offset, int whence)
{
	if(g_noimem)
		return MALSEEK(stream, offset, whence);
		
	return memlseek(stream, offset, whence);
}


int mimread(int fd, char *ptr, int size)
{
	if(g_noimem)
		return MAREAD(fd, ptr, size);
		
	return memread(fd, ptr, size);
}


// int	stat __P((const machar *, struct stat *));
int mimstat(const fschar *filename, struct stat *buf)
{
	if(g_noimem)
		return stat((char *)filename, buf);
	else
	{
		int i;
		for(i = 0; i < g_loadednum; i++)
		{
			if(!fsstrcmp(filename, g_loadedpath[i]))
				break;
		}
		
		if(g_loadednum <= 0 || i >= g_loadednum)
			return -1;
			
		return memfstat(i + 1, buf);	
	}
}


// int	fstat __P((int, struct stat *));
int mimfstat(int j, struct stat *buf)
{
	if(g_noimem)
		return fstat(j, buf);
		
	return memfstat(j, buf);
}


// int	 feof(FILE *);
int mimfeof(FILE *stream)
{
	if(g_noimem)
		return feof(stream);
		
	return memfeof(stream);
}


int mimferror(FILE *stream)
{
	if(g_noimem)
		return ferror(stream);
		
	return memferror(stream);
}
*/


// mem* functions operate on files in memory
void memShowErr(const machar * msg)
{
	maLog(msg);
	
} // memShowErr


int memfptofd(FILE *fp)
{
	// return (int)fp; // avoid warning C4311: 'type cast' : pointer truncation from 'FILE *' to 'int'
	if(fp == (FILE *)1) return 1;
	if(fp == (FILE *)2) return 2;
	if(fp == (FILE *)3) return 3;
	if(fp == (FILE *)4) return 4;
	if(fp == (FILE *)5) return 5;
	if(fp == (FILE *)6) return 6;
	if(fp == (FILE *)7) return 7;
	if(fp == (FILE *)8) return 8; // MAXMEMFP
	memShowErr(L"memfptofd file pointer out of range");
	return 0;
}


FILE *memfdtofp(int fd)
{
	// return (FILE *)fd; // avoid warning C4312: 'type cast' : conversion from 'int' to 'FILE *' of greater size
	switch(fd)
	{
		case 1: return (FILE *)1;
		case 2: return (FILE *)2;
		case 3: return (FILE *)3;
		case 4: return (FILE *)4;
		case 5: return (FILE *)5;
		case 6: return (FILE *)6;
		case 7: return (FILE *)7;
		case 8: return (FILE *)8; // MAXMEMFP
		default:
		break;
	}
	memShowErr(L"memfdtofp file descriptor out of range");
	return (FILE *)0;
}


FILE *memfopen(const fschar *filename) // okret == FILE *, else NULL
{
	return maCreateFile((fschar *)filename, 0);
}


int memopen(fschar *filename) // okret == fd, else -1
{
	FILE *fp;

	if(((fp = maCreateFile(filename, 0)) != 0))
		return memfptofd(fp);
	
	return -1;
}


int memfclose(FILE *stream) // okret == 0, else -1
{
	if(stream < (FILE *)1 || stream >= (FILE *)MAXMEMFP)
		return -1;

	g_memfp[memfptofd(stream)].isopen = 0;

	return 0;
}


int memclose(int fd) // okret == 0, else -1
{
	if(fd < 1 || fd >= MAXMEMFP)
		return -1;

	g_memfp[fd].isopen = 0;

	return 0;
}


// fseek() returns zero if it is successful and returns a nonzero value if it fails
int memfseek(FILE *stream, long offset, int whence) // okret == 0
{
	if(stream < (FILE *)1 || stream >= (FILE *)MAXMEMFP)
		return -1;
		
	if(maSetFilePointer(stream, offset, whence, 0) >= 0)
		return 0;

	return -1;
}

// fseeko() returns zero if it is successful and returns a nonzero value if it fails
int memfseeko(FILE *stream, maoff_t64 offset, int whence) // okret == 0 
{
	if(stream < (FILE *)1 || stream >= (FILE *)MAXMEMFP)
		return -1;
		
	if(maSetFilePointero(stream, offset, whence, 0) >= 0)
		return 0;

	return -1;
}

// lseek() returns the new file position indicator value.
int memlseek(int fd, off_t offset, int whence)
{
	return maSetFilePointer(memfdtofp(fd), (int)offset, whence, 0);
}


// ftell(), when successful, returns the current file position indicator value.
// If it fails, ftell() returns -1L and sets the global variable errno to a nonzero value
long int memftell(FILE *stream) // ret == file pos
{
	long pos;
	int j;

	if(stream < (FILE *)1 || stream >= (FILE *)MAXMEMFP)
		return -1L;
	
	j = memfptofd(stream);
	if((pos = (long int)(g_memfp[j].songpos - g_memfp[j].songbeg)) >= 0 &&
				 pos <= (g_memfp[j].songend - g_memfp[j].songbeg))
		return pos;
		
	return -1L;
}


long int memftello(FILE *stream) // ret == file pos
{
	long int pos;
	int j;

	if(stream < (FILE *)1 || stream >= (FILE *)MAXMEMFP)
		return -1L;
	
	j = memfptofd(stream);
		
	if((pos = (long int)(g_memfp[j].songpos - g_memfp[j].songbeg)) >= 0 &&
			 pos <= (g_memfp[j].songend - g_memfp[j].songbeg))
		return pos;
		
	return -1L;
}


long int memtelli(int fd) // ret == file pos
{
	long int pos;

	if(fd < 1 || fd >= MAXMEMFP)
		return -1L;
		
	if((pos = (long int)(g_memfp[fd].songpos - g_memfp[fd].songbeg)) >= 0 &&
				 pos <= (g_memfp[fd].songend - g_memfp[fd].songbeg))
		return pos;
		
	return -1L;
}


// fread() returns the number of items read successfully
size_t memfread(void *ptr, size_t size, size_t nmemb, FILE *stream)
{
	unsigned long bytesread;
	int j = memfptofd(stream);
	size_t nm;
	char *pos;
	
	if(j < 1 || j >= MAXMEMFP || g_memfp[j].songpos < g_memfp[j].songbeg)
	{
		return 0;
	}
		
	//g_memfp[j].errorind = 0;
	
	if((bytesread = (unsigned long)(size * nmemb)) < 1)
	{
		g_memfp[j].errorind = 1;
		return 0;
	}
	
	if((pos = g_memfp[j].songpos + bytesread) <= g_memfp[j].songend)
	{
		memcpy(ptr, g_memfp[j].songpos, bytesread);
		g_memfp[j].songpos = pos;			
		return nmemb;
	}

	// past eof :
	 
	bytesread = (unsigned long)(g_memfp[j].songend - g_memfp[j].songpos);
		
	nm = bytesread / size;
	bytesread = (long)(size * nm);
	memcpy(ptr, g_memfp[j].songpos, bytesread);
	g_memfp[j].songpos = g_memfp[j].songend;				
	return nm;
	
} // memfread


// read() returns the number of bytes actually read from the file.
// In case of an error a value of negative one is returned and errno is set
int memread(int j, char *ptr, int size)
{
	int bytesread;

	if(j < 1 || j >= MAXMEMFP)
		return -1;
	
	if((bytesread = size) < 0)
		return -1;
	
	if(g_memfp[j].songpos >= g_memfp[j].songend) // eof
		return 0;
		
	if(bytesread > g_memfp[j].songend - g_memfp[j].songpos) // eof
		bytesread = (int)(g_memfp[j].songend - g_memfp[j].songpos);
	
	if(g_memfp[j].songpos < g_memfp[j].songbeg)
		return -1;

	memcpy(ptr, g_memfp[j].songpos, (size_t)bytesread);
	g_memfp[j].songpos += bytesread;
				
	return bytesread;
}


void memrewind(FILE *stream)
{
	int j;

	if(stream < (FILE *)1 || stream >= (FILE *)MAXMEMFP)
		return;

	j = memfptofd(stream);

	g_memfp[j].songpos = g_memfp[j].songbeg;
}


// If it is successful, fstat() returns zero.
// If it encounters an error, fstat() returns -1 and sets errno
int memfstat(int j, struct stat *buf) // okret = 0, else -1 and errno
{
	if(j < 1 || j >= MAXMEMFP)
		return -1;
		
	buf->st_mode = S_IFREG;
	buf->st_size =
#if defined(__MAWIN__)
					(_off_t)
#endif
							(g_memfp[j].songend - g_memfp[j].songbeg);

	return 0;
}


// int feof(FILE *stream) tests the end-of-file indicator for the stream
//     pointed to by stream, returning non-zero if it is set.
int memfeof(FILE *stream)
{
	int j = memfptofd(stream);

	if(j < 1 || j >= MAXMEMFP || g_memfp[j].songpos >= g_memfp[j].songend)
		return 1;
	 
	return 0;
}


// int ferror(FILE *stream) tests the error indicator for the stream pointed to
//     by stream, returning non-zero if it is set.  The error indicator can only
//     be reset by the clearerr() function.
int memferror(FILE *stream)
{
	if(stream < (FILE *)1 || stream >= (FILE *)MAXMEMFP)
		return -1;
		
	return g_memfp[memfptofd(stream)].errorind;
}


// fileno() examines the argument stream and returns its integer descriptor.
int memfileno(FILE *stream)
{
	if(stream < (FILE *)1 || stream >= (FILE *)MAXMEMFP)
		return 1;
		
	return memfptofd(stream);
}


void memclearerr(FILE *stream)
{
	if(stream < (FILE *)1 || stream >= (FILE *)MAXMEMFP)
		return ;
		
	g_memfp[memfptofd(stream)].errorind = 0;
}


/* some compilers have any of these in unistd.h:
typedef struct
{
 __off_t __pos;
 __mbstate_t __state;
} _G_fpos_t;
typedef _G_fpos_t fpos_t;
typedef __off_t off_t;
typedef __off64_t off_t;
typedef long long fpos_t; // 64 bit file sizes & positions
typedef unsigned long fpos_t;
*/

// fgetpos() returns zero when successful and returns a nonzero value when it fails
int memfgetpos(FILE *stream, fpos_t *pos)
{
	long lpos;

	if(stream < (FILE *)1 || stream >= (FILE *)MAXMEMFP)
		return -1; // error
	 
	if((lpos = memtelli(memfptofd(stream))) < 0)
	 	return -1;
	 
	*((off_t *)pos) = (off_t)lpos;
	 
	return 0;
}


// fsetpos() returns zero if it is successful and returns a nonzero value if it fails
int memfsetpos(FILE *stream, const fpos_t *pos)
{	
	if(stream < (FILE *)1 || stream >= (FILE *)MAXMEMFP || !pos)
		return -1; // error
		
	if(memfseek(stream, (long)(*((off_t *)pos)), SEEK_SET))
    	return -1;

	return 0;
}


// pc functions :

FILE *maCreateFile(fschar *path, int nomem)
{
	FILE *filep;
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
#endif
	machar mastr[MAXLOGLINELEN];

	if(!nomem)
	{
		int i, j;
		
		for(i = 0; i < g_loadednum; i++)
		{
			if(!fsstrcmp(path, g_loadedpath[i]))
				break;
		}
		
		if(g_loadednum <= 0 || i >= g_loadednum)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maCreateFile not loaded : '%ls'", fs2ma(mastr2, path));
			memShowErr(mastr);
			return 0;
		}
		
		if(g_songmemptr == NULL)
		{
			memShowErr(L"maCreateFile null g_songmemptr");
			return 0;
		}
		
		for(j = 1; j < MAXMEMFP; j++)
			if(!g_memfp[j].isopen)
				break;
		
		if(j >= MAXMEMFP)
		{
			memShowErr(L"maCreateFile failed, no more FILE handles available");
			return 0;
		}
		
		g_memfp[j].errorind = 0;
		g_memfp[j].isopen = 1;
		g_memfp[j].songbeg = g_memfp[j].songpos = g_songmemptr[i];
		g_memfp[j].songend = g_songmemptr[i + 1];

		return memfdtofp(j);
		
	} // USEMEM
		
	if(!path || (path[0] == fsT('\0')))
	{
		memShowErr(L"maCreateFile null path (CREATE_ALWAYS)");
		return 0;
	}
		
	if((filep = mafopen(path, fsT("rb"))) == NULL) // read only
	{
		masprintf(mastr, MAXLOGLINELEN, L"maCreateFile mafopen '%ls'", fs2ma(mastr2, path));
		memShowErr(mastr);
		return 0;
	}
		
#ifdef ASYNCREAD
	g_ar_offset = 0;
#endif

	return filep;		
	
} // maCreateFile


// called from maPlayMp3 only
int maReadFile(FILE *file, unsigned char *buffer, int bytes, int *pbytes, int nomem)
{
	int bytesread;
	machar mastr[MAXLOGLINELEN];

	if(nomem)
	{	
		bytesread = (int)fread(buffer, 1, (size_t)bytes, file);			
	}
	else
	{
		int j = memfptofd(file);
		if(j < 1 || j >= MAXMEMFP)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maReadFile bad memfile pointer=%d", j);
			memShowErr(mastr);
			*pbytes = 0;
			return 0; // error
		}

		if((bytesread = bytes) > (g_memfp[j].songend - g_memfp[j].songpos))
					  bytesread = (int)(g_memfp[j].songend - g_memfp[j].songpos);
			
		memcpy(buffer, g_memfp[j].songpos, (unsigned long)bytesread);
		g_memfp[j].songpos += bytesread;
	}
	
	*pbytes = bytesread;
	
	return 1; // true == success
		
} // maReadFile


int maSetFilePointer(FILE *file, long position, int method, int nomem)
{	
	machar mastr[MAXLOGLINELEN];
	
	if(nomem)
	{
		// SEEK_SET 0, SEEK_CUR 1, SEEK_END 2	
		if(fseek(file, position, method) != 0)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maSetFilePointer fseek file=x%x method=%d position=%d",
														memfptofd(file), method, position);
			memShowErr(mastr);
			return -1;
		}
	
#ifdef ASYNCREAD
		g_ar_offset = position;
#endif // ASYNCREAD
	
		return ftell(file); // ok		
	}
	else
	{
		int j = memfptofd(file);
		if(j < 1 || j >= MAXMEMFP)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maSetFilePointer (g_usesongmem) bad memfile pointer=%d", memfptofd(file));
			memShowErr(mastr);
			return -1;
		}

		switch(method)
		{
			case SEEK_SET :
				g_memfp[j].songpos = g_memfp[j].songbeg + position;
			break;	

			case SEEK_CUR :
				g_memfp[j].songpos += position;
			break;	

			case SEEK_END : // position s.b. <= 0
				g_memfp[j].songpos = g_memfp[j].songend + position;
			break;
			
			default:
				memShowErr(L"maSetFilePointer fseek bad method");
				return -1;
				
		} // switch

		if(g_memfp[j].songpos > g_memfp[j].songend)
			g_memfp[j].songpos = g_memfp[j].songend - 1;
		if(g_memfp[j].songpos < g_memfp[j].songbeg)
			g_memfp[j].songpos = g_memfp[j].songbeg;

		return (int)(g_memfp[j].songpos - g_memfp[j].songbeg);

	} // if(!nomem)
	
} // maSetFilePointer


int maSetFilePointero(FILE *file, maoff_t64 position, int method, int nomem)
{	
	machar mastr[MAXLOGLINELEN];
	
	if(nomem)
	{
		// SEEK_SET 0, SEEK_CUR 1, SEEK_END 2	
		if(fseeko(file, position, method) != 0)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maSetFilePointer fseek file=x%x method=%d position=%d",
														memfptofd(file), method, position);
			memShowErr(mastr);
			return -1;
		}
	
#ifdef ASYNCREAD
		g_ar_offset = position;
#endif // ASYNCREAD
	
		return ftell(file); // ok		
	}
	else
	{
		int j = memfptofd(file);
		if(j < 1 || j >= MAXMEMFP)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maSetFilePointer (g_usesongmem) bad memfile pointer=%d", memfptofd(file));
			memShowErr(mastr);
			return -1;
		}

		switch(method)
		{
			case SEEK_SET :
				g_memfp[j].songpos = g_memfp[j].songbeg + position;
			break;	

			case SEEK_CUR :
				g_memfp[j].songpos += position;
			break;	

			case SEEK_END : // position s.b. <= 0
				g_memfp[j].songpos = g_memfp[j].songend + position;
			break;
			
			default:
				memShowErr(L"maSetFilePointer fseek bad method");
				return -1;
				
		} // switch

		if(g_memfp[j].songpos  > g_memfp[j].songend)
			g_memfp[j].songpos  = g_memfp[j].songend - 1;
		if(g_memfp[j].songpos  < g_memfp[j].songbeg)
			g_memfp[j].songpos  = g_memfp[j].songbeg;

		return (g_memfp[j].songpos - g_memfp[j].songbeg >= 0) ? 0 : -1;

	} // if(!nomem)
	
} // maSetFilePointero


int maGetFileSize(FILE *file, int nomem)
{
	int size;
	machar mastr[MAXLOGLINELEN];
	
	if(nomem)
	{	
		int current;
	
		if((current = ftell(file)) == -1)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maGetFileSize ftell errno=%d", errno);
			memShowErr(mastr);
			return 0;
		}

		fseek(file, 0, SEEK_END);
		if((size = ftell(file)) == -1)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maGetFileSize ftell errno=%d", errno);
			memShowErr(mastr);
			return 0;
		}
		fseek(file, current, SEEK_SET);
	
		return size;		
	}
	else
	{	
		int current;
	
		if((current = memftell(file)) == -1)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maGetFileSize memftell failed file=0x%p", file);
			memShowErr(mastr);
			return 0;
		}

		memfseek(file, 0, SEEK_END);
		size = memftell(file);
		memfseek(file, current, SEEK_SET);
	
		return size;
		
	} // if(!nomem)
	
} // maGetFileSize


#ifdef __cplusplus
}
#endif

// eof
