// maPlayAac.c

/*
 * MoreAmp
 * Copyright (C) 2004-2014 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include "maConfig.h" // included by all moreamp files

#ifndef NODECODEAAC

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#ifdef __MAWIN__
 #include <stdlib.h>
 #include <limits.h>
 #include <time.h>
#endif

#ifdef __MAUNIX__
 #include "time.h"
#endif

#if defined(__MAOSX__)
 #include <MacTypes.h>
#endif

#include "maTypes.h"
#include "maDefs.h"
#include "maPlayer.h"
#include "maUtils.h"
#include "maPlayAac.h"
#include "maEqualizer.h"
#include "maMemFile.h"
#include "maAudioPA.h"
#include "faad.h"

#define MAX_CHANNELS 6 /* make this higher to support files with more channels */

typedef struct
{
	long bytes_into_buffer;
	long bytes_consumed;
	long file_offset;
	long fileread;
	unsigned char *buffer;
	int at_eof;
	FILE *infilefp;
} aac_buffer;

typedef struct
{
	faacDecHandle hcodec;
	aac_buffer b;
	faacDecFrameInfo frameInfo;
	faacDecConfigurationPtr config;
	MASTATE *pstate;
	unsigned char old_format;
} AacContext;

int aaccurrentpossec();
int AacOpenFile(AacContext *paac);
void AacCloseFile(AacContext *paac);

void aacadvance_buffer(aac_buffer *b, int bytes);
int aac_fill_buffer(AacContext *paac);
int aacadts_parse(AacContext *paac, int *bitrate, float *length);

AacContext g_aaccontext;
AacContext *g_paacpcontext;

maint64 g_aacseek = -1;


void aacseek(mauint64 pos)
{
	if(g_paacpcontext == NULL || g_aacseek >= 0 || pos < 0)
		return;

	g_aacseek = pos;	

} // aacseek


/* int aacseek_(mauint64 position) // need to translate ms to frame then seek to frame...
{
	if(g_paacpcontext == NULL || position < 0)
		return -1;

	if(g_paacpcontext->pstate->nomem)
		return fseek(g_paacpcontext->b.infilefp, (long)position, SEEK_SET);
		
	return memfseek(g_paacpcontext->b.infilefp, (long)position, SEEK_SET);

} // aacseek_
*/


void maPlayAac()
{
	void *sample_buffer;
	machar mastr2[MAXLOGLINELEN];
#if !defined(__MAWIN__)
	machar mastr3[MAXLOGLINELEN];
#endif
	machar mastr[MAXLOGLINELEN];

	if(!g_state.inpath)
	{
		maLog(L"maPlayAac error null filename");
		return;
	}
	
	if(g_paacpcontext)
		maLog(L"maPlayAac unclosed context");
	
	// format of this decoder output
#ifdef WORDS_BIGENDIAN
	g_state.inwordsbigendian = 1;
#else
	g_state.inwordsbigendian = 0;
#endif
	g_state.ininterleaved = 1;

	// open
	g_paacpcontext = &g_aaccontext;
	memset(g_paacpcontext, 0, sizeof(AacContext));
	g_paacpcontext->pstate = &g_state;
	
	if(AacOpenFile(g_paacpcontext))
	{
		masprintf(mastr, MAXLOGLINELEN, L"maPlayAac AacOpenFile failed on '%ls'",
									fs2ma(mastr2, g_paacpcontext->pstate->inpath));
		maLog(mastr);
		if(g_paacpcontext->b.infilefp)
		{
			if(g_paacpcontext->pstate->nomem)
				fclose(g_paacpcontext->b.infilefp);
			else
				memfclose(g_paacpcontext->b.infilefp);			
			g_paacpcontext->b.infilefp = NULL;
		}
		g_paacpcontext = NULL;
		return;
	}
		
	if(maIOStart() < 0)
	{
		maLog(L"maPlayAac maIOStart failed");
		if(g_paacpcontext)
		{
			AacCloseFile(g_paacpcontext);
			g_paacpcontext = NULL;
		}
		return;
	}

	if(!g_posendon || g_posend > g_state.duration)
		g_posend = g_state.duration;
	
	// aac decode loop
	for (; ;)
	{	
		if(g_state.stop & (STOP_ALL | STOP_DECODER))
			break;
				
		if(g_aacseek >= 0)
		{
			// aacseek_(g_aacseek);		
			g_aacseek = -1;
		}

		sample_buffer = faacDecDecode(g_paacpcontext->hcodec, &g_paacpcontext->frameInfo,
							g_paacpcontext->b.buffer, g_paacpcontext->b.bytes_into_buffer);
	
		if(g_paacpcontext->frameInfo.error == 12) // Invalid number of channels, "12" is hardcoded in decoder.c
		{
			g_paacpcontext->config->useOldADTSFormat = g_paacpcontext->old_format ? 0 : 1;
			sample_buffer = faacDecDecode(g_paacpcontext->hcodec, &g_paacpcontext->frameInfo,
							g_paacpcontext->b.buffer, g_paacpcontext->b.bytes_into_buffer);
		}
		if(g_paacpcontext->frameInfo.error > 0)
		{
			masprintf(mastr, MAXLOGLINELEN, L"faacDecDecode failed : %s '%ls'",
			os2ma(mastr2, faacDecGetErrorMessage(g_paacpcontext->frameInfo.error)),
			fs2ma(mastr3, g_paacpcontext->pstate->inpath));
			maLog(mastr);
			break;
		}

		/* update buffer indices */
		aacadvance_buffer(&g_paacpcontext->b, g_paacpcontext->frameInfo.bytesconsumed);

		if((g_paacpcontext->frameInfo.error == 0) && (g_paacpcontext->frameInfo.samples > 0))
		{
			g_state.inchannels = g_paacpcontext->frameInfo.channels;

			// pass data on to output processing chain
			if(maOutChain1((char *)sample_buffer, g_paacpcontext->frameInfo.samples << 1) != 0)
				break;
		}

		/* fill buffer */
		aac_fill_buffer(g_paacpcontext);

		if(g_paacpcontext->b.bytes_into_buffer == 0)
			sample_buffer = NULL; /* to make sure it stops now */

		g_state.position = (mauint64)(LROUND(((double)g_paacpcontext->b.file_offset * g_state.duration) /
																	g_paacpcontext->b.fileread)); 
		// NULL at end of stream
		if(sample_buffer == NULL)
		{
			if(g_state.stop || g_loadedquit || !g_reptsong)
				break;
			AacCloseFile(g_paacpcontext);
			if(AacOpenFile(g_paacpcontext))
			{
				masprintf(mastr, MAXLOGLINELEN, L"maPlayAac AacOpenFile Repeat failed on '%ls'",
									fs2ma(mastr2, g_paacpcontext->pstate->inpath));
				maLog(mastr);
				break;
			}
		}

	} // while 1
	
	// song is finished

	g_aacseek = -1;
	
	if(g_paacpcontext)
	{
		AacCloseFile(g_paacpcontext);
		g_paacpcontext = NULL;
	}

	maOutChainFlush();

} // maPlayAac


int aac_fill_buffer(AacContext *paac)
{
	size_t bread;

	if(paac->b.bytes_consumed > 0)
	{
		if(paac->b.bytes_into_buffer)
			memmove((void*)paac->b.buffer, (void*)(paac->b.buffer + paac->b.bytes_consumed),
											paac->b.bytes_into_buffer * sizeof(unsigned char));

		if(!paac->b.at_eof)
		{
			if(paac->pstate->nomem)
				bread = fread((void*)(paac->b.buffer + paac->b.bytes_into_buffer), 1,
													paac->b.bytes_consumed, paac->b.infilefp);
			else
				bread = memfread((void*)(paac->b.buffer + paac->b.bytes_into_buffer), 1,
													paac->b.bytes_consumed, paac->b.infilefp);

			if((long)bread != paac->b.bytes_consumed)
				paac->b.at_eof = 1;

			paac->b.bytes_into_buffer += (long)bread;
		}

		paac->b.bytes_consumed = 0;

		if(paac->b.bytes_into_buffer > 3)
			if(memcmp(paac->b.buffer, "TAG", 3) == 0)
				paac->b.bytes_into_buffer = 0;

		if(paac->b.bytes_into_buffer > 11)
			if(memcmp(paac->b.buffer, "LYRICSBEGIN", 11) == 0)
				paac->b.bytes_into_buffer = 0;

		if(paac->b.bytes_into_buffer > 8)
			if(memcmp(paac->b.buffer, "APETAGEX", 8) == 0)
				paac->b.bytes_into_buffer = 0;
	}

	return 1;
}

void aacadvance_buffer(aac_buffer *b, int bytes)
{
	b->file_offset += bytes;
	b->bytes_consumed = bytes;
	b->bytes_into_buffer -= bytes;
}

static int adts_sample_rates[] =
		{96000,88200,64000,48000,44100,32000,24000,22050,16000,12000,11025,8000,7350,0,0,0};

int aacadts_parse(AacContext *paac, int *bitrate, float *length)
{
	int frames, frame_length;
	int t_framelength = 0;
	int samplerate = 44100;
	float frames_per_sec, bytes_per_frame;
	aac_buffer *b = &paac->b;
	
	/* Read all frames to ensure correct time and bitrate */
	for(frames = 0; /* */; frames++)
	{
		aac_fill_buffer(paac);

		if(b->bytes_into_buffer > 7)
		{
			/* check syncword */
			if(!((b->buffer[0] == 0xFF)&&((b->buffer[1] & 0xF6) == 0xF0)))
				break;

			if(frames == 0)
				samplerate = adts_sample_rates[(b->buffer[2]&0x3c)>>2];

			frame_length = ((((unsigned int)b->buffer[3] & 0x3)) << 11)
				| (((unsigned int)b->buffer[4]) << 3) | (b->buffer[5] >> 5);

			t_framelength += frame_length;

			if(frame_length > b->bytes_into_buffer)
				break;

			aacadvance_buffer(b, frame_length);
		}
		else
		{
			break;
		}
	}

	frames_per_sec = (float)samplerate / 1024.0f;
	if(frames != 0)
		bytes_per_frame = (float)t_framelength / (float)(frames*1000);
	else
		bytes_per_frame = 0;
	*bitrate = (int)(8. * bytes_per_frame * frames_per_sec + 0.5);
	if(frames_per_sec != 0)
		*length = (float)frames / frames_per_sec;
	else
		*length = 1;

	return 0;
}


int AacOpenFile(AacContext *paac)
{
	int lnomem;
	int object_type = LC;
	int def_srate = 44100;
	int downMatrix = 0;
	int outputFormat = FAAD_FMT_16BIT;
	machar mastr[MAXLOGLINELEN];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif

	int tagsize;
	unsigned long samplerate;
	unsigned char channels;
	size_t bread;
	int bitrate = 0;
	float length = 0;

	if(paac == NULL)
	{
		maLog(L"AacOpenFile null context pointer");
		return 1;
	}

	paac->old_format = 0;

	lnomem = ((g_sets.buffertype == BUFUSEMEM) && !paac->pstate->nomem) ? 0 : 1;
	
	if(lnomem)
	{
		if((paac->b.infilefp = mafopen(paac->pstate->inpath, fsT("rb"))) == NULL)
		{
			masprintf(mastr, MAXLOGLINELEN, L"AacOpenFile mafopen failed on '%ls'", fs2ma(mastr2, paac->pstate->inpath));
			maLog(mastr);
			return 1;
		}
	}
	else
	{
		if((paac->b.infilefp = memfopen(paac->pstate->inpath)) == NULL)
		{
			masprintf(mastr, MAXLOGLINELEN, L"AacOpenFile memfopen failed on '%ls'", fs2ma(mastr2, paac->pstate->inpath));
			maLog(mastr);
			return 1;
		}
	}

	// unsigned long cap = faacDecGetCapabilities();

	// if(FileNameHasExtension(paac->pstate->inpath, EXTM4A))
	// 	old_format = 0;
		
	if(lnomem)
	{
		fseek(paac->b.infilefp, 0, SEEK_END);
		paac->b.fileread = ftell(paac->b.infilefp);
		fseek(paac->b.infilefp, 0, SEEK_SET);
	}
	else
	{
		memfseek(paac->b.infilefp, 0, SEEK_END);
		paac->b.fileread = memftell(paac->b.infilefp);
		memfseek(paac->b.infilefp, 0, SEEK_SET);
	}

	if((paac->b.buffer = (unsigned char*)MAMALLOC(FAAD_MIN_STREAMSIZE * MAX_CHANNELS)) == 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"AacOpenFile Memory allocation error");
		maLog(mastr);
		return 0;
	}
		
	memset(paac->b.buffer, 0, FAAD_MIN_STREAMSIZE * MAX_CHANNELS);

	if(lnomem)
		bread = fread(paac->b.buffer, 1, FAAD_MIN_STREAMSIZE * MAX_CHANNELS, paac->b.infilefp);
	else
		bread = memfread(paac->b.buffer, 1, FAAD_MIN_STREAMSIZE * MAX_CHANNELS, paac->b.infilefp);
		
	paac->b.bytes_into_buffer = (long)bread;
	paac->b.bytes_consumed = 0;
	paac->b.file_offset = 0;

	if(bread != FAAD_MIN_STREAMSIZE * MAX_CHANNELS)
		paac->b.at_eof = 1;

	tagsize = 0;
	if(!memcmp(paac->b.buffer, "ID3", 3))
	{
		/* high bit is not used */
		tagsize = (paac->b.buffer[6] << 21) | (paac->b.buffer[7] << 14) |
						(paac->b.buffer[8] <<  7) | (paac->b.buffer[9] <<  0);

		tagsize += 10;
		aacadvance_buffer(&paac->b, tagsize);
		aac_fill_buffer(paac);
	}

	paac->hcodec = faacDecOpen();

	/* Set the default object type and samplerate */
	/* This is useful for RAW AAC files */
	paac->config = faacDecGetCurrentConfiguration(paac->hcodec);
	if(def_srate)
		paac->config->defSampleRate = def_srate;
	paac->config->defObjectType = (unsigned char)object_type;
	paac->config->outputFormat = (unsigned char)outputFormat;
	paac->config->downMatrix = (unsigned char)downMatrix;
	paac->config->useOldADTSFormat = paac->old_format;
	paac->config->dontUpSampleImplicitSBR = 1;
	faacDecSetConfiguration(paac->hcodec, paac->config);

	/* get AAC infos for printing */
	if((paac->b.buffer[0] == 0xFF) && ((paac->b.buffer[1] & 0xF6) == 0xF0))
	{
		aacadts_parse(paac, &bitrate, &length);
		
		if(lnomem)
		{
			fseek(paac->b.infilefp, tagsize, SEEK_SET);
			bread = fread(paac->b.buffer, 1, FAAD_MIN_STREAMSIZE * MAX_CHANNELS, paac->b.infilefp);
		}
		else
		{
			memfseek(paac->b.infilefp, tagsize, SEEK_SET);
			bread = memfread(paac->b.buffer, 1, FAAD_MIN_STREAMSIZE * MAX_CHANNELS, paac->b.infilefp);
		}
			
		if(bread != FAAD_MIN_STREAMSIZE * MAX_CHANNELS)
			paac->b.at_eof = 1;
		else
			paac->b.at_eof = 0;
				
		paac->b.bytes_into_buffer = (long)bread;
		paac->b.bytes_consumed = 0;
		paac->b.file_offset = tagsize;
	}
	else if(memcmp(paac->b.buffer, "ADIF", 4) == 0)
	{
		int skip_size = (paac->b.buffer[4] & 0x80) ? 9 : 0;
		bitrate = ((unsigned int)(paac->b.buffer[4 + skip_size] & 0x0F)<<19) |
			((unsigned int)paac->b.buffer[5 + skip_size]<<11) |
			((unsigned int)paac->b.buffer[6 + skip_size]<<3) |
			((unsigned int)paac->b.buffer[7 + skip_size] & 0xE0);

		length = (float)paac->b.fileread;
		if(length != 0)
			length = (float)(((double)length * 8) / bitrate + 0.5);

		bitrate = (int)(LROUND(((double)bitrate/1000.0 + 0.5)));
	}
	else // raw, no adts headers
		// length = 60; // 1.00 minute arbitrary for display
		length = (float)(paac->b.fileread / 12991); // approx value for files ecoded at q=100
	
	paac->pstate->duration = (mauint64)(LROUND(length * 1000));
	
	aac_fill_buffer(paac);
	
	if((bread = faacDecInit(paac->hcodec, paac->b.buffer,
					paac->b.bytes_into_buffer, &samplerate, &channels)) < 0)
	{
		/* If some error initializing occured, skip the file */
		masprintf(mastr, MAXLOGLINELEN, L"AacOpenFile error initializing decoder library.");
		maLog(mastr);
		if(paac->b.buffer)
			MAFREE(paac->b.buffer);
		faacDecClose(paac->hcodec);
		goto aacopenerrexit;
	}
	
	aacadvance_buffer(&paac->b, (int)bread);
	aac_fill_buffer(paac);
	paac->pstate->insamplerate = samplerate;

	paac->pstate->position = 0;

	return 0;

aacopenerrexit:
		if(lnomem)
			fclose(paac->b.infilefp);
		else
			memfclose(paac->b.infilefp);
					
		paac->b.infilefp = NULL;
		
		return 1;

} // AacOpenFile


void AacCloseFile(AacContext *paac)
{
	if(!paac)
	{
		maLog(L"AacCloseFile null context pointer");
		return;
	}

	// close paac->hcodec
	if(paac->hcodec)
	{
		faacDecClose(paac->hcodec);
		paac->hcodec = NULL;
	}
		
	// close file
	if(paac->b.infilefp)
	{
		if(paac->pstate->nomem)
			fclose(paac->b.infilefp);
		else
			memfclose(paac->b.infilefp);
					
		paac->b.infilefp = NULL;
	}
	
	if(paac->b.buffer)
	{
		MAFREE(paac->b.buffer);
		paac->b.buffer = NULL;
	}

} // AacCloseFile


// void DumpTagsAac(MASTATE *pstate, FILE *dumpfp, int nomem);

int DumpInfoAac(MASTATE *pstate, machar **p2, machar **p3, size_t *len3)
{
	machar tstr[STRLEN32];
	machar lstr[STRLEN512];
	
	AacContext aaccontext;
	int object_type = LC;
	int def_srate = 44100;
	int downMatrix = 0;
	int outputFormat = FAAD_FMT_16BIT;
	machar mastr[MAXLOGLINELEN];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif

	int tagsize;
	unsigned long samplerate;
	unsigned char channels;
	size_t bread;
	int bitrate = 0;
	float length = 0;

	// open
	memset(&aaccontext, 0, sizeof(AacContext));
	aaccontext.pstate = pstate;
	
	if(pstate->nomem)
	{
		if((aaccontext.b.infilefp = mafopen(aaccontext.pstate->inpath, fsT("rb"))) == NULL)
		{
			masprintf(mastr, MAXLOGLINELEN, L"DumpInfoAac mafopen failed on '%ls'",
											fs2ma(mastr2, aaccontext.pstate->inpath));
			maLog(mastr);
			return 1;
		}
	}
	else
	{
		if((aaccontext.b.infilefp = memfopen(aaccontext.pstate->inpath)) == NULL)
		{
			masprintf(mastr, MAXLOGLINELEN, L"DumpInfoAac memfopen failed on '%ls'",
											fs2ma(mastr2, aaccontext.pstate->inpath));
			maLog(mastr);
			return 1;
		}
	}

	// if(FileNameHasExtension(aaccontext.pstate->inpath, EXTM4A))
	// 	old_format = 0;
		
	if(pstate->nomem)
	{
		fseek(aaccontext.b.infilefp, 0, SEEK_END);
		aaccontext.b.fileread = ftell(aaccontext.b.infilefp);
		fseek(aaccontext.b.infilefp, 0, SEEK_SET);
	}
	else
	{
		memfseek(aaccontext.b.infilefp, 0, SEEK_END);
		aaccontext.b.fileread = memftell(aaccontext.b.infilefp);
		memfseek(aaccontext.b.infilefp, 0, SEEK_SET);
	}

	if((aaccontext.b.buffer = (unsigned char*)MAMALLOC(FAAD_MIN_STREAMSIZE * MAX_CHANNELS)) == 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"DumpInfoAac Memory allocation error");
		maLog(mastr);
		return 0;
	}
		
	memset(aaccontext.b.buffer, 0, FAAD_MIN_STREAMSIZE * MAX_CHANNELS);

	if(pstate->nomem)
		bread = fread(aaccontext.b.buffer, 1, FAAD_MIN_STREAMSIZE * MAX_CHANNELS, aaccontext.b.infilefp);
	else
		bread = memfread(aaccontext.b.buffer, 1, FAAD_MIN_STREAMSIZE * MAX_CHANNELS, aaccontext.b.infilefp);
		
	aaccontext.b.bytes_into_buffer = (long)bread;
	aaccontext.b.bytes_consumed = 0;
	aaccontext.b.file_offset = 0;

	if(bread != FAAD_MIN_STREAMSIZE * MAX_CHANNELS)
		aaccontext.b.at_eof = 1;

	tagsize = 0;
	if(!memcmp(aaccontext.b.buffer, "ID3", 3))
	{
		/* high bit is not used */
		tagsize = (aaccontext.b.buffer[6] << 21) | (aaccontext.b.buffer[7] << 14) |
						(aaccontext.b.buffer[8] <<  7) | (aaccontext.b.buffer[9] <<  0);

		tagsize += 10;
		aacadvance_buffer(&aaccontext.b, tagsize);
		aac_fill_buffer(&aaccontext);
	}

	aaccontext.hcodec = faacDecOpen();

	aaccontext.config = faacDecGetCurrentConfiguration(aaccontext.hcodec);
	if(def_srate)
		aaccontext.config->defSampleRate = def_srate;
	aaccontext.config->defObjectType = (unsigned char)object_type;
	aaccontext.config->outputFormat = (unsigned char)outputFormat;
	aaccontext.config->downMatrix = (unsigned char)downMatrix;
	// aaccontext.config->useOldADTSFormat = (unsigned char)old_format;
	aaccontext.config->dontUpSampleImplicitSBR = 1;
	faacDecSetConfiguration(aaccontext.hcodec, aaccontext.config);

	/* get AAC infos for printing */
	if((aaccontext.b.buffer[0] == 0xFF) && ((aaccontext.b.buffer[1] & 0xF6) == 0xF0))
	{
		aacadts_parse(&aaccontext, &bitrate, &length);
		
		if(pstate->nomem)
		{
			fseek(aaccontext.b.infilefp, tagsize, SEEK_SET);
			bread = fread(aaccontext.b.buffer, 1, FAAD_MIN_STREAMSIZE * MAX_CHANNELS, aaccontext.b.infilefp);
		}
		else
		{
			memfseek(aaccontext.b.infilefp, tagsize, SEEK_SET);
			bread = memfread(aaccontext.b.buffer, 1, FAAD_MIN_STREAMSIZE * MAX_CHANNELS, aaccontext.b.infilefp);
		}
			
		if(bread != FAAD_MIN_STREAMSIZE * MAX_CHANNELS)
			aaccontext.b.at_eof = 1;
		else
			aaccontext.b.at_eof = 0;
				
		aaccontext.b.bytes_into_buffer = (long)bread;
		aaccontext.b.bytes_consumed = 0;
		aaccontext.b.file_offset = tagsize;
	}
	else if(memcmp(aaccontext.b.buffer, "ADIF", 4) == 0)
	{
		int skip_size = (aaccontext.b.buffer[4] & 0x80) ? 9 : 0;
		bitrate = ((unsigned int)(aaccontext.b.buffer[4 + skip_size] & 0x0F)<<19) |
			((unsigned int)aaccontext.b.buffer[5 + skip_size]<<11) |
			((unsigned int)aaccontext.b.buffer[6 + skip_size]<<3) |
			((unsigned int)aaccontext.b.buffer[7 + skip_size] & 0xE0);

		if((length = (float)aaccontext.b.fileread) != 0)
			length = (float)(((double)length * 8) / bitrate + 0.5);

		bitrate = (int)(LROUND(((double)bitrate/1000.0 + 0.5)));
	}
	else // raw, no adts headers
	{
		// length = 60; // 1.00 minute arbitrary for display
		length = (float)(aaccontext.b.fileread / 12991); // approx value for files encoded at q=100
	}
	
	pstate->duration = (mauint64)(LROUND(MSPERSEC * length));
	
	aac_fill_buffer(&aaccontext);
	
	if((bread = faacDecInit(aaccontext.hcodec, aaccontext.b.buffer,
					aaccontext.b.bytes_into_buffer, &samplerate, &channels)) < 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"DumpInfoAac error initializing decoder library.");
		maLog(mastr);
	}
	
	if(aaccontext.b.buffer)
		MAFREE(aaccontext.b.buffer);
		
	faacDecClose(aaccontext.hcodec);

	if(pstate->nomem)
		fclose(aaccontext.b.infilefp);
	else
		memfclose(aaccontext.b.infilefp);

	if(pstate->duration < MSPERSEC)
		masprintf(lstr, STRLEN512, L"Duration : %d ms.", (int)pstate->duration);
	else
		masprintf(lstr, STRLEN512, L"Duration : %ls", MillisecondsToStrHMSD(tstr, pstate->duration));		
	DumpString(pstate->dumpfp, lstr, p2, p3, len3);
	
	masprintf(lstr, STRLEN512, L"Channels : %d", channels);
	DumpString(pstate->dumpfp, lstr, p2, p3, len3);
	
	masprintf(lstr, STRLEN512, L"Bitrate : %d", bitrate * 1000);
	DumpString(pstate->dumpfp, lstr, p2, p3, len3);
	
	masprintf(lstr, STRLEN512, L"Samplerate : %lu", samplerate);
	DumpString(pstate->dumpfp, lstr, p2, p3, len3);
	
	// DumpTagsAac(songfpath, *dumpfp, pstate->nomem);

	return 0;
	
} // DumpInfoAac


#ifdef __cplusplus
}
#endif

#endif

// eof
