// maPlayCda.c

/*
 * MoreAmp
 * Copyright (C) 2004-2014 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include "maConfig.h" // included by all moreamp files

#ifndef MANOCDA

#ifdef __MAWIN__
 int CdaCloseRipper();
#endif

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#ifdef __MAWIN__
 #include <stdlib.h>
 #include <limits.h>
 #include <time.h>
 #define noErr 0
 #define paramErr -17
 int CdaOpenRipper();
#endif

#if defined(__MAOSX__) || defined(__MACUNIX__)
 #include "Gestalt.h"
 #include <CarbonEvents.h>
#endif

#ifdef __MAUNIX__
 #define noErr 0
 #define paramErr -17 // ?
#endif

#ifdef __MAWIN__
 typedef int SInt16;
#endif

#include "maCda.h"
#include "maPlayCda.h"

#include "maPlayer.h"
#include "maUtils.h"
#include "maEqualizer.h"
#include "maMemFile.h"
#include "maAudioPA.h"

extern char *g_cdabuf;

int BCDtoHex(char SrcByte);
char HextoBCD(int SrcInt);
void lba_to_msf(long lba, unsigned char *M, unsigned char *S, unsigned char *F);
int msf_to_frames(int minutes, int seconds, int frames);
int msf_to_lba(int minutes, int seconds, int frames);
int CdaDecodeStream();
int CdaDoOpen(fschar *playpath);


int BCDtoHex(char SrcByte)
{
	int x, y;

    x = SrcByte % 16;
    y = SrcByte / 16;
    y = y * 10;
    return x + y;
}


char HextoBCD(int SrcInt)
{
	int x, y;
	char z;

	z = 0;
	x = SrcInt % 256;
	y = x;
	if(y >= 10)
	{
        z = z + 16;
        do
		{
			y = y - 10;
            if(y >= 10)
				z = z + 16;
        } while(y >= 10);
	}
	
    return (char)(y + z);

} // HextoBCD


void lba_to_msf(long lba, unsigned char *M, unsigned char *S, unsigned char *F)
{
  if(lba >= -150)
  {
    *M = (unsigned char)((lba + 150) / (60 * 75));
    lba -= (*M) * 60 * 75;
    *S = (unsigned char)((lba + 150) / 75);
    lba -= (*S) * 75;
    *F = (unsigned char)((lba + 150));
  }
  else
  {
    *M = (unsigned char)((lba + 450150) / (60 * 75));
    lba -= (*M) * 60 * 75;
    *S = (unsigned char)((lba + 450150) / 75);
    lba -= (*S) * 75;
    *F = (unsigned char)(lba + 450150);
  }
  
} // lba_to_msf


int msf_to_frames(int minutes, int seconds, int frames)
{
  return minutes * 4500 + seconds * 75 + frames;
}


int msf_to_lba(int minutes, int seconds, int frames)
{
  if(msf_to_frames(minutes, seconds, frames) > 150)
    return msf_to_frames(minutes, seconds, frames) - 150;
   
  return 0;
}


// our implementations of win and osx cd readers return data at
// the beginning of a track that we don't want to hear. This may
// or may not be a bug, but for now just apply a temp fix

#if defined(__MAWIN__)
 // on win silence the first 8820 frames (about 200 ms)
 #define SILENTFRAMES 8820
#elif defined(__MAOSX__) || defined(__MACUNIX__)
 // on osx silence first 128 frames (about 2.9 ms)
 #define SILENTFRAMES 128
#else
 #define SILENTFRAMES 0
#endif

#define SILENTBYTES (SILENTFRAMES * 4)

int l_silentbytes = 0;
maint64 l_cdaseek = -1;
void cdaseek(mauint64 pos)
{
	if(l_cdaseek >= 0 || pos < 0)
		return;
		
	l_cdaseek = pos;
	
} // cdaseek


void maPlayCda()
{
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];

	if(!g_state.inpath)
	{
		maLog(L"maPlayCda error null filename");
		return;
	}
	
	// format of this decoder output
#if defined(WORDS_BIGENDIAN) && !defined(__MAOSX__)
	g_state.inwordsbigendian = 1; // bigendian ppc, e.g. linux on mac, but not osx
#else
	g_state.inwordsbigendian = 0;
#endif
	g_state.ininterleaved = 1;
	g_state.inchannels = 2;

	// open input
	if(CdaOpenFile(g_state.inpath))
	{
		masprintf(mastr, MAXLOGLINELEN, L"maPlayCda CdaOpenFile failed on '%ls'", fs2ma(mastr2, g_state.inpath));
		maLog(mastr);
		CdaCloseFile(g_cdaactive);
		return;
	}
	
 	g_state.insamplerate = MAOUTRATE;
	
	if(maIOStart() < 0)
	{
		CdaCloseFile(g_cdaactive);
		maLog(L"maPlayCda maIOStart failed");
		return;
	}

	if(!g_posendon || g_posend > g_state.duration)
		g_posend = g_state.duration;

	// cd loop
	for (; ;)
	{	
		if(g_state.stop & (STOP_ALL | STOP_DECODER))
			break;
				
		if(l_cdaseek >= 0)
		{
			g_cdadrive[g_cdaactive].lba = (unsigned int)(LROUND(g_cdadrive[g_cdaactive].lbafirst +
								(((double)l_cdaseek * CDAFRAMESPERSECL) / MSPERSEC)));
			l_cdaseek = -1;
		}

		if(CdaDecodeStream())
			break;
	
		// update position indicator
		g_state.position = (mauint64)(LROUND((((double)g_cdadrive[g_cdaactive].lba -
						g_cdadrive[g_cdaactive].lbafirst) / CDAFRAMESPERSECL) * MSPERSEC));

	} // while 1

	// finish

	l_cdaseek = -1;
	
	CdaCloseFile(g_cdaactive);
	
	maOutChainFlush();

} // maPlayCda


int CdaDecodeStream()
{
	int alldone;
	long pcmblocksize;
	int err;
	int drive;
	machar mastr[MAXLOGLINELEN];

	if(!g_cdabuf)
	{
		maLog(L"CdaDecodeStream null g_pcdadrive or buf");
		return 1;
	}

	// Get block from CD
	alldone = 0;
	drive = g_cdaactive;

	if((err = CdaReadTrack(drive, &pcmblocksize)) != noErr)
	{	
		// workaround, ignore this error close to end of last track
		if(err == paramErr && g_cdadrive[drive].track == g_cdadrive[drive].tracklast &&
							   g_cdadrive[drive].lbalast - g_cdadrive[drive].lba < 200) 
		{
			pcmblocksize = 0;
			alldone = 1; // all done
		}
		else
		{
			masprintf(mastr, MAXLOGLINELEN,
				L"CdaDecodeStream CdaReadTrack error=%d drive=%d track=%d tracklast=%d lba=%d numblocks=%d lbalast=%d",
				err, drive, g_cdadrive[drive].track, g_cdadrive[drive].tracklast, g_cdadrive[drive].lba,
														g_cdabufblocks, g_cdadrive[drive].lbalast);
			maLog(mastr);
			pcmblocksize = 0;
			return 1; // err
		}
	}
		
	if(g_cdadrive[drive].lba == g_cdadrive[drive].lbafirst)
		l_silentbytes = SILENTBYTES;

	if(l_silentbytes > 0)
	{
		if(l_silentbytes <= pcmblocksize)
		{
			memset((unsigned char*)g_cdabuf, 0, l_silentbytes);
			l_silentbytes = 0;
		}
		else
		{
			memset((unsigned char*)g_cdabuf, 0, pcmblocksize);
			l_silentbytes -= pcmblocksize;
		}
	}
	
	if((g_cdadrive[drive].lba += g_cdabufblocks) >= g_cdadrive[drive].lbalast)
		alldone = 1;

	if(pcmblocksize > 0)
	{		
		// pass data on to output processing chain
		if(maOutChain1((char *)g_cdabuf, pcmblocksize) != 0)
		{
			masprintf(mastr, MAXLOGLINELEN, L"CdaDecodeStream maOutChain1 failed : len=%ld g_state.stop=x%x",
												pcmblocksize, g_state.stop);
			maLog(mastr);
			return 1;
		}

	} //if(pcmblocksize > 0)

	if(!alldone)
		return 0;

	// song is finished
	if(!g_state.stop && !g_loadedquit && g_reptsong)
	{
		g_cdadrive[g_cdaactive].lba = g_cdadrive[g_cdaactive].lbafirst;
		return 0;
	}
	
	return 1;

} // CdaDecodeStream


int CdaDoOpen(fschar *playpath)
{	
#if !defined(__MAOSX__) && !defined(MACDAOSX)
	char trkfirstBCD, trklastBCD;
#endif
	fschar lplaypath[MAX_PATH];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];
	
	if(!playpath || (playpath[0] == fsT('\0')))
	{
		maLog(L"CdaDoOpen null playpath");
		return 1;
	}

#if defined(__MAWIN__)
	if(CdaSetActiveDriveLetter((int)playpath[0]))
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaDoOpen CdaSetActiveDriveLetter failed '%ls'", fs2ma(mastr2, playpath));
		maLog(mastr);
		g_cdaactive = 0;
		return 1;
	}
#else
	if(!(g_cdaactive = IsAudioCd(playpath)))
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaDoOpen IsAudioCd '%ls' not found", fs2ma(mastr2, playpath));
		maLog(mastr);
		return 1;
	}
#endif
	
	// on imac internal cd system hangs after eject and reinsert cd if not reset to 512
	if(CdaSetBlockSize(g_cdaactive, CDABLOCKSIZE))
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaDoOpen CdaSetBlockSize failed '%ls'", fs2ma(mastr2, playpath));
		maLog(mastr);
		goto cdaerrret;
	}
	
	if(CdaSetSpindleSpeed(g_cdaactive, 0)) // 0 for normal, else max
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaDoOpen CdaSetSpindleSpeed failed '%ls'", fs2ma(mastr2, playpath));
		maLog(mastr);
		goto cdaerrret;
	}
	
#if !defined(__MAOSX__) && !defined(MACDAOSX)
	if(CdaGetCdInfo(g_cdaactive, &trkfirstBCD, &trklastBCD))
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaDoOpen CdaGetCdInfo failed '%ls'", fs2ma(mastr2, playpath));
		maLog(mastr);
		goto cdaerrret;
	}
	
	g_cdadrive[g_cdaactive].tracklast = BCDtoHex(trklastBCD);
#endif

	fsstrcpy(g_state.inpath, playpath);
	
#if defined(__MAWIN__)
	fsstrcpy(lplaypath, playpath); // e.g. "D:/Track05.cda"
	lplaypath[fsstrlen(playpath) - 4 ] = fsT('\0');
	g_cdadrive[g_cdaactive].track = fsatoi(&lplaypath[fsstrlen(lplaypath) - 2 ]);
#endif

 #if defined(__MAOSX__)
	fsstrcpy(lplaypath, playpath); // e.g. "Audio CD/Track05.cda"
	lplaypath[fsstrlen(playpath) - 4 ] = fsT('\0');
	g_cdadrive[g_cdaactive].track = fsatoi(&lplaypath[fsstrlen(lplaypath) - 2 ]);
 #endif
 
 #if defined(__MAUNIX__)
	fsstrcpy(lplaypath, playpath); // e.g. "/dev/cdrom/Track05.cda"
	lplaypath[fsstrlen(playpath) - 4 ] = fsT('\0');
	g_cdadrive[g_cdaactive].track = fsatoi(&lplaypath[fsstrlen(lplaypath) - 2 ]);
 #endif

	if(g_cdadrive[g_cdaactive].track < 1 || g_cdadrive[g_cdaactive].track > 99) // never happens
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaDoOpen bad track num %d '%ls' '%ls'",
					g_cdadrive[g_cdaactive].track, &lplaypath[5], fs2ma(mastr2, playpath));
		maLog(mastr);
		goto cdaerrret;
	}
	
	// get starting lba for the track to play
	if(CdaGetTrackStartEnd(g_cdaactive))
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaDoOpen bad start lba for track %d '%ls'", g_cdadrive[g_cdaactive].track, fs2ma(mastr2, playpath));
		maLog(mastr);
		goto cdaerrret;
	}
	
	g_cdadrive[g_cdaactive].lba = g_cdadrive[g_cdaactive].lbafirst;
		
	// length of track in blocks
	g_cdadrive[g_cdaactive].lbasintrack = 1 + g_cdadrive[g_cdaactive].lbalast - g_cdadrive[g_cdaactive].lbafirst;

	if(g_cdabuf)
		MAFREE(g_cdabuf);
	if((g_cdabuf = (char *)MAMALLOC(CDABLOCKSIZE * g_cdabufblocks)) == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaDoOpen MAMALLOC failed for %d bytes", CDABLOCKSIZE * g_cdabufblocks);
		maLog(mastr);
		goto cdaerrret;
	}

	return 0; // ok
	
cdaerrret:
		if(g_cdaactive)
			CdaCleanup(g_cdaactive);
		g_cdaactive = 0;
		return 1;
		
} // CdaDoOpen


int CdaOpenFile(fschar *filename)
{
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];
	
	if(CdaDoOpen(filename))
	{
		masprintf(mastr, MAXLOGLINELEN, L"CdaOpenFile CdaDoOpen failed on '%ls'", fs2ma(mastr2, filename));
		maLog(mastr);
		return 1;
	}
	
	g_cdadrive[g_cdaactive].durationsecs = g_cdadrive[g_cdaactive].lbasintrack / CDAFRAMESPERSECL;
			
	g_state.position = 0;
	g_state.duration = (mauint64)g_cdadrive[g_cdaactive].durationsecs * 1000;

#ifdef __MAWIN__
	if(CdaOpenRipper())
	{
		g_cdaactive = 0;
		maLog(L"CdaOpenFile CdaOpenRipper failed");
		return 1;
	}
#endif

#if defined(__MAOSX__) || defined(MACDAOSX)
	if(CdaPBOpenFile(g_cdaactive))
	{
		g_cdaactive = 0;
		masprintf(mastr, MAXLOGLINELEN, L"CdaOpenFile CdaPBOpenFile failed on '%ls'", fs2ma(mastr2, filename));
		maLog(mastr);
		return 1;
	}
#endif

	return 0; // ok
	
} // CdaOpenFile


void CdaCloseFile(int drive)
{
#ifdef __MAWIN__
	CdaCloseRipper();
#endif

#if defined(__MAOSX__) || defined(MACDAOSX)
	CdaPBCloseFile(drive);
#endif

	CdaCleanup(drive);
	if(g_cdabuf)
	{
		MAFREE(g_cdabuf);
		g_cdabuf = NULL;	
	}
	
} // CdaCloseFile


// stubs just for testing unix builds on osx
#if defined(__MACUNIX__)
 #include "../libs/cdparanoia/interface/cdda_interface.h"
 #include "../libs/cdparanoia/paranoia/cdda_paranoia.h"

 int cdda_close(cdrom_drive *d)
 {
	return 0;
 }

 cdrom_drive *cdda_find_a_cdrom(int messagedest, char **message)
 {
	return NULL;
 }
				  
 cdrom_drive *cdda_identify(const char *device, int messagedest, char **message)
 {
	return NULL;
 }
				  
 int cdda_open(cdrom_drive *d)
 {
	return 0;
 }

 long cdda_read(cdrom_drive *d, void *buffer, long beginsector, long sectors)
 {
	return 0;
 }

 long cdda_track_firstsector(cdrom_drive *d,int track)
 {
	return 0;
 }

 long cdda_track_lastsector(cdrom_drive *d,int track)
 {
	return 0;
 }

 void paranoia_free(cdrom_paranoia *p)
 {
 }

 cdrom_paranoia *paranoia_init(cdrom_drive *d)
 {
	return NULL;
 }

 void paranoia_modeset(cdrom_paranoia *p,int mode)
 {
 }

 int16_t *paranoia_read(cdrom_paranoia *p,void(*callback)(long,int))
 {
	return NULL;
 }

 long paranoia_seek(cdrom_paranoia *p,long seek,int mode)
 {
	return 0;
 }

#endif // __MACUNIX__


#ifdef __cplusplus
}
#endif

#else
 #ifdef __MAWIN__
  #pragma warning( disable : 4206 ) // nonstandard extension used : translation unit is empty
 #endif
#endif // MANOCDA

// eof
