// maPlayFlac.c

/*
 * MoreAmp
 * Copyright (C) 2004-2014 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include "maConfig.h" // included by all moreamp files

#ifndef NODECODEFLAC

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <limits.h>

#include "FLAC/assert.h"

#ifdef __MAWIN__
 #include <stdlib.h>
 #include <math.h>
 #include <limits.h>
 #include <time.h>
#endif

#if defined(__MAOSX__)
 #include <MacTypes.h>
#endif

#include "maTypes.h"
#include "maDefs.h"
#include "maPlayer.h"
#include "maPlayFlac.h"
#include "maEqualizer.h"
#include "maMemFile.h"
#include "maPlayer.h"
#include "maUtils.h"
#include "maAudioPA.h"
#include "FLAC/all.h"
// #include "share/utf8.h"

#if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT < 8
 #error "MoreAmp needs FLAC_API_VERSION_CURRENT greater than 8 "
#endif

unsigned long prng(unsigned long ulstate);

typedef enum
{
 FMT_U8, FMT_S8, FMT_U16_LE, FMT_U16_BE, FMT_U16_NE, FMT_S16_LE, FMT_S16_BE, FMT_S16_NE
}
AFormat;

typedef struct {
	FLAC__bool abort_flag;
	FLAC__bool eof;
	mauint64 total_samples;
	unsigned bits_per_sample;
	unsigned channels;
	unsigned sample_rate;
	mauint64 length_in_msec;
} file_info_struct;

typedef struct
{
	fschar *infilepath;
	int nomem; // file is not in mem, or don't use file in mem
	FLAC__StreamDecoder *decoder;
	FLAC__StreamDecoderState state;
	FLAC__bool eof;
	mauint64 total_samples;
	unsigned bits_per_sample;
	unsigned channels;
	unsigned sample_rate;
	mauint64 length_in_msec;
	mauint64 current_sample;
	AFormat sample_format;
	unsigned sample_format_bytes_per_sample;
	// FLAC__bool has_replaygain;
	// double replay_scale;
	FLAC__bool is_playing;
	maint64 flacseek;
	FILE *infilefp;
} FlacContext;

#ifdef min
 #undef min
#endif
#define min(x,y) ((x)<(y)?(x):(y))

#define FLAC_PLUGIN__MAX_SUPPORTED_CHANNELS 2

typedef struct {
	FLAC__int32 error[3];
	FLAC__int32 random;
} dither_state;

int FlacOpenFile(FlacContext **ppflac);
void FlacCloseFile(FlacContext *pflac);
FLAC__bool masafe_decoder_init(const fschar *filename, FLAC__StreamDecoder *decoder, FlacContext *pflac);
void masafe_decoder_delete(FLAC__StreamDecoder *decoder);
void masafe_decoder_finish(FLAC__StreamDecoder *decoder);
FLAC__StreamDecoderReadStatus maread_callback(const FLAC__StreamDecoder *decoder, FLAC__byte buffer[], size_t *bytes, void *client_data);
FLAC__StreamDecoderSeekStatus maseek_callback(const FLAC__StreamDecoder *decoder, FLAC__uint64 absolute_byte_offset, void *client_data);
FLAC__StreamDecoderTellStatus matell_callback(const FLAC__StreamDecoder *decoder, FLAC__uint64 *absolute_byte_offset, void *client_data);
FLAC__StreamDecoderLengthStatus malength_callback(const FLAC__StreamDecoder *decoder, FLAC__uint64 *stream_length, void *client_data);
FLAC__bool maeof_callback(const FLAC__StreamDecoder *decoder, void *client_data);
FLAC__StreamDecoderWriteStatus mawrite_callback(const FLAC__StreamDecoder *decoder, const FLAC__Frame *frame, const FLAC__int32 * const buffer[], void *client_data);
void /* FLAC__StreamDecoderMetadataCallback */ mametadata_callback(const FLAC__StreamDecoder *decoder, const FLAC__StreamMetadata *metadata, void *client_data);
void /* FLAC__StreamDecoderErrorCallback */ maerror_callback(const FLAC__StreamDecoder *decoder, FLAC__StreamDecoderErrorStatus status, void *client_data);
size_t mapack_pcm_signed_little_endian(FLAC__byte *data, const FLAC__int32 * const input[], unsigned wide_samples, unsigned channels, unsigned source_bps, unsigned target_bps);
static FLAC__int32 malinear_dither(unsigned source_bps, unsigned target_bps, FLAC__int32 sample, dither_state *dither, const FLAC__int32 MIN, const FLAC__int32 MAX);

FlacContext l_FlacContext;
FlacContext *l_pflac;

FLAC__int32 l_reservoir[FLAC__MAX_BLOCK_SIZE * 2 * FLAC_PLUGIN__MAX_SUPPORTED_CHANNELS];


void flacseek(mauint64 posms)
{
	if(posms < 0 || !l_pflac)
		return;

	if(l_pflac->flacseek >= 0)
		return;

	l_pflac->eof = false;

	l_pflac->flacseek = posms;	

	while(l_pflac && l_pflac->flacseek != -1)
		maSleep(10);

} // flacseek

static FLAC__bool is_big_endian_host_;


void maPlayFlac() // (with code from flac 1.3.0 plugin_xmms plugin.c)
{
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];
	
	if(!g_state.inpath)
	{
		maLog(L"maPlayFlac error null filename");
		return;
	}
	
	if(l_pflac)
	{
		maLog(L"maPlayFlac unclosed context");
		return;
	}

	// open
	l_pflac = &l_FlacContext;
	memset(l_pflac, 0, sizeof(FlacContext));
	l_pflac->nomem = g_state.nomem;
	l_pflac->is_playing = false;
	l_pflac->eof = false;
	// l_pflac->has_replaygain = false;
	l_pflac->current_sample = 0;
	l_pflac->flacseek = -1;

	// format of this decoder output
#ifdef WORDS_BIGENDIAN
	g_state.inwordsbigendian = 1;
#else
	g_state.inwordsbigendian = 0;
#endif
	g_state.ininterleaved = 1;

	l_pflac->infilepath = g_state.inpath;
	
	g_noamem = l_pflac->nomem;

	if(FlacOpenFile(&l_pflac))
	{
		masprintf(mastr, MAXLOGLINELEN, L"maPlayFlac FlacOpenFile failed on '%ls'", fs2ma(mastr2, g_state.inpath));
		maLog(mastr);
		l_pflac = NULL;
		return;
	}
		
	g_state.insamplerate = l_pflac->sample_rate;
	g_state.duration = (mauint64)l_pflac->length_in_msec;
	g_state.position = 0;
	g_state.inchannels = l_pflac->channels;

	if(g_state.inchannels == 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maPlayFlac FlacOpenFile null channels  on '%ls'",
															fs2ma(mastr2, g_state.inpath));
		maLog(mastr);
		FlacCloseFile(l_pflac);
		l_pflac = NULL;
		return;
	}

	if(g_state.insamplerate == 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maPlayFlac FlacOpenFile null sample rate on '%ls'",
																fs2ma(mastr2, g_state.inpath));
		maLog(mastr);
		FlacCloseFile(l_pflac);
		l_pflac = NULL;
		return;
	}

	if(g_state.duration == 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maPlayFlac FlacOpenFile 0 duration  on '%ls'",
																fs2ma(mastr2, g_state.inpath));
		maLog(mastr);
		FlacCloseFile(l_pflac);
		l_pflac = NULL;
		return;
	}


	if(l_pflac->bits_per_sample == 8)
	{
		l_pflac->sample_format = FMT_U8;
		l_pflac->sample_format_bytes_per_sample = 1;
	}
	else if(l_pflac->bits_per_sample == 16 || l_pflac->bits_per_sample == 24) // dither_24_to_16
	{
		l_pflac->sample_format = (is_big_endian_host_) ? FMT_S16_BE : FMT_S16_LE;
		l_pflac->sample_format_bytes_per_sample = 2;
	}
	else
	{
		masafe_decoder_finish(l_pflac->decoder);
		FlacCloseFile(l_pflac);
		l_pflac = NULL;
		return;
	}

 	if(maIOStart() < 0)
	{
		FlacCloseFile(l_pflac);
		l_pflac = NULL;
		maLog(L"maPlayFlac maIOStart failed");
		return;
	}

	if(!g_posendon || g_posend > g_state.duration)
		g_posend = g_state.duration;

	l_pflac->is_playing = true;

	while(l_pflac->is_playing)
	{
		if(g_state.stop & (STOP_ALL | STOP_DECODER))
			break;

		if(!l_pflac->eof && l_pflac->flacseek == -1)
		{
			if(FLAC__stream_decoder_get_state(l_pflac->decoder) == FLAC__STREAM_DECODER_END_OF_STREAM)
			{
				l_pflac->eof = true;
				goto playflacexit;
			}

			if(!FLAC__stream_decoder_process_single(l_pflac->decoder))
			{
				l_pflac->eof = true;
				goto playflacexit; // fatal decoder error
			}
		}
	
		if(l_pflac->flacseek != -1)
		{
			const double distance = (double)l_pflac->flacseek / (double)l_pflac->length_in_msec;
			FLAC__uint64 target_sample = (FLAC__uint64)(distance * (double)l_pflac->total_samples);
			if(l_pflac->total_samples > 0 && target_sample >= l_pflac->total_samples)
				target_sample = l_pflac->total_samples - 1;
			if(FLAC__stream_decoder_seek_absolute(l_pflac->decoder, target_sample))
				l_pflac->eof = false; // success
			else if(FLAC__stream_decoder_get_state(l_pflac->decoder) == FLAC__STREAM_DECODER_SEEK_ERROR)
			{
				FLAC__stream_decoder_flush(l_pflac->decoder);
				l_pflac->eof = false;
			}
			l_pflac->flacseek = -1;
		}

		// the current position in msec
		g_state.position = (mauint64)(LROUND(((double)l_pflac->current_sample * g_state.duration) / l_pflac->total_samples));

		continue;

		// song is finished	
playflacexit:
		if(g_state.stop || g_loadedquit || !g_reptsong)
			break;

		l_pflac->eof = false;
		l_pflac->flacseek = g_posbeg;

	} // while(l_pflac->is_playing)

	masafe_decoder_finish(l_pflac->decoder);

	l_pflac->is_playing = false;

	// finish

	l_pflac->flacseek = -1;

	if(l_pflac)
	{
		FlacCloseFile(l_pflac);
		l_pflac = NULL;
	}

	maOutChainFlush();

} // maPlayFlac


int FlacOpenFile(FlacContext **ppflac)
{
	FlacContext *pflac;
	machar mastr[MAXLOGLINELEN];

	if(ppflac == NULL || (pflac = *ppflac) == NULL)
	{
		maLog(L"FlacOpenFile null context pointer");
		return 1;
	}

	if(pflac->nomem)
	{
		if((pflac->infilefp = mafopen(pflac->infilepath, fsT("rb"))) == NULL)
		{
			masprintf(mastr, MAXLOGLINELEN, L"FlacOpenFile mafopen failed on '%ls'", pflac->infilepath);
			maLog(mastr);
			return 1;
		}
	}
	else
	{
		if((pflac->infilefp = memfopen(pflac->infilepath)) == NULL)
		{
			masprintf(mastr, MAXLOGLINELEN, L"FlacOpenFile memfopen failed on '%ls'", pflac->infilepath);
			maLog(mastr);
			return 1;
		}
	}

	if((pflac->decoder = FLAC__stream_decoder_new()) == 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"FlacOpenFile FLAC__stream_decoder_new failed on '%ls'", pflac->infilepath);
		maLog(mastr);
		return 1;
	}

	pflac->eof = false;

	if(!masafe_decoder_init(pflac->infilepath, pflac->decoder, pflac))
	{
		masprintf(mastr, MAXLOGLINELEN, L"FlacOpenFile safe_decoder_init_ failed on '%ls'", pflac->infilepath);
		maLog(mastr);
		return 1;
	}

	return 0; // ok

} // FlacOpenFile


void FlacCloseFile(FlacContext *pflac)
{
	if(!pflac)
	{
		maLog(L"FlacCloseFile null context pointer");
		return;
	}

	masafe_decoder_finish(pflac->decoder);

	// close pflac->decoder
	if(pflac->decoder)
	{
		masafe_decoder_delete(pflac->decoder);
		pflac->decoder = 0;
	}
		
	// close file
	if(pflac->infilefp)
	{
		if(pflac->nomem)
			fclose(pflac->infilefp);
		else
			memfclose(pflac->infilefp);
					
		pflac->infilefp = NULL;
	}
	
} // FlacCloseFile


void masafe_decoder_finish(FLAC__StreamDecoder *decoder)
{
	if(decoder && FLAC__stream_decoder_get_state(decoder) != FLAC__STREAM_DECODER_UNINITIALIZED)
		(void)FLAC__stream_decoder_finish(decoder);
}


FLAC__bool masafe_decoder_init(const fschar *filename, FLAC__StreamDecoder *decoder, FlacContext *pflac)
{
	(void)filename;
	if(decoder == 0)
		return false;

	masafe_decoder_finish(decoder);

	FLAC__stream_decoder_set_md5_checking(decoder, false);
	FLAC__stream_decoder_set_metadata_ignore_all(decoder);
	FLAC__stream_decoder_set_metadata_respond(decoder, FLAC__METADATA_TYPE_STREAMINFO);
	FLAC__stream_decoder_set_metadata_respond(decoder, FLAC__METADATA_TYPE_VORBIS_COMMENT);

	if(FLAC__stream_decoder_init_stream(
					decoder,
					maread_callback,
					maseek_callback,
					matell_callback,
					malength_callback,
					maeof_callback,
					mawrite_callback,
					mametadata_callback,
					maerror_callback,
					(void *)pflac
			) != FLAC__STREAM_DECODER_INIT_STATUS_OK)
		return false;

	if(!FLAC__stream_decoder_process_until_end_of_metadata(decoder))
		return false;

	return true; // ok

} // masafe_decoder_init


FLAC__StreamDecoderReadStatus maread_callback(const FLAC__StreamDecoder *decoder, FLAC__byte buffer[], size_t *bytes, void *client_data)
{
	int err;
	FlacContext *pflac = (FlacContext *)client_data;
	(void)decoder;

	if(*bytes > 0)
	{
		if(pflac->nomem)
			*bytes = fread(buffer, sizeof(FLAC__byte), *bytes, pflac->infilefp);
		else
			*bytes = memfread(buffer, sizeof(FLAC__byte), *bytes, pflac->infilefp);

		if(pflac->nomem)
			err = ferror(pflac->infilefp);
		else
			err = memferror(pflac->infilefp);

		if(err)
		{
			g_state.stop |= STOP_DECODER;
			return FLAC__STREAM_DECODER_READ_STATUS_ABORT;
		}

		if(*bytes == 0)
			return FLAC__STREAM_DECODER_READ_STATUS_END_OF_STREAM;
	
		return FLAC__STREAM_DECODER_READ_STATUS_CONTINUE;
	}

	g_state.stop |= STOP_DECODER;
	return FLAC__STREAM_DECODER_READ_STATUS_ABORT; /* abort to avoid a deadlock */

} // maread_callback


FLAC__StreamDecoderSeekStatus maseek_callback(const FLAC__StreamDecoder *decoder, FLAC__uint64 absolute_byte_offset, void *client_data)
{
	FlacContext *pflac = (FlacContext *)client_data;
	(void)decoder;

		if(pflac->nomem)
		{
			if(fseeko(pflac->infilefp, (off_t)absolute_byte_offset, SEEK_SET) < 0)
				return FLAC__STREAM_DECODER_SEEK_STATUS_ERROR;
		} 
		else
		{
			if(memfseeko(pflac->infilefp, (off_t)absolute_byte_offset, SEEK_SET) < 0)
				return FLAC__STREAM_DECODER_SEEK_STATUS_ERROR;
		} 

		return FLAC__STREAM_DECODER_SEEK_STATUS_OK;

} // maseek_callback


FLAC__StreamDecoderTellStatus matell_callback(const FLAC__StreamDecoder *decoder, FLAC__uint64 *absolute_byte_offset, void *client_data)
{
	maoff_t64  pos;
	FlacContext *pflac = (FlacContext *)client_data;
	(void)decoder;

		if(pflac->nomem)
		{
			if((pos = ftello(pflac->infilefp)) < 0)
				return FLAC__STREAM_DECODER_TELL_STATUS_ERROR;
		} 
		else
		{
			if((pos = memftello(pflac->infilefp)) < 0)
				return FLAC__STREAM_DECODER_TELL_STATUS_ERROR;
		} 

	*absolute_byte_offset = (FLAC__uint64)pos;

	return FLAC__STREAM_DECODER_TELL_STATUS_OK;

} // matell_callback


FLAC__StreamDecoderLengthStatus malength_callback(const FLAC__StreamDecoder *decoder, FLAC__uint64 *stream_length, void *client_data)
{
	FlacContext *pflac = (FlacContext *)client_data;
	struct stat filestats;
	
	(void)decoder;

	if(pflac->nomem)
	{
		if(fstat(MAFILENO(pflac->infilefp), &filestats) != 0)
			return FLAC__STREAM_DECODER_LENGTH_STATUS_ERROR;
	} 
	else
	{
		if(memfstat(memfileno(pflac->infilefp), &filestats) != 0)
			return FLAC__STREAM_DECODER_LENGTH_STATUS_ERROR;
	} 

	*stream_length = (FLAC__uint64)filestats.st_size;

	return FLAC__STREAM_DECODER_LENGTH_STATUS_OK;

} // malength_callback


FLAC__bool maeof_callback(const FLAC__StreamDecoder *decoder, void *client_data)
{
	FlacContext *pflac = (FlacContext *)client_data;

	(void)decoder;

	if(pflac->nomem)
		return feof(((FlacContext *)client_data)->infilefp)? true : false;
	else
		return memfeof(((FlacContext *)client_data)->infilefp)? true : false;

} // maeof_callback

 
#define MAFLACCOMPRESSION 12 // only used to guess at duration from filesize
void /* FLAC__StreamDecoderMetadataCallback */ mametadata_callback(const FLAC__StreamDecoder *decoder, const FLAC__StreamMetadata *metadata, void *client_data)
{
	/*
	file_info_struct *file_info = (file_info_struct *)client_data;
	( void )decoder;
	if(metadata->type == FLAC__METADATA_TYPE_STREAMINFO)
	{
		FLAC__ASSERT(metadata->data.stream_info.total_samples < 0x100000000); /w* this plugin can only handle < 4 gigasamples *w/
		file_info->total_samples = (unsigned)(metadata->data.stream_info.total_samples & 0xffffffff);
		file_info->bits_per_sample = metadata->data.stream_info.bits_per_sample;
		file_info->channels = metadata->data.stream_info.channels;
		file_info->sample_rate = metadata->data.stream_info.sample_rate;
	*w/

/w*
#ifdef FLAC__DO_DITHER
		if(file_info->bits_per_sample == 8)
		{
			file_info->sample_format = FMT_S8;
		}
		else if(file_info->bits_per_sample == 16 || file_info->bits_per_sample == 24)
		{
			file_info->sample_format = FMT_S16_LE;
		}
		else
		{
			/w* @@@ need some error here like wa2: MessageBox(mod_.hMainWindow, "ERROR: plugin can only handle 8/16/24-bit samples\n", "ERROR: plugin can only handle 8/16/24-bit samples", 0); *w/
			file_info->abort_flag = true;
			return;
		}
#else
		if(file_info->bits_per_sample == 8)
		{
			file_info->sample_format = FMT_S8;
		}
		else if(file_info->bits_per_sample == 16)
		{
			file_info->sample_format = FMT_S16_LE;
		}
		else
		{
			/w* @@@ need some error here like wa2: MessageBox(mod_.hMainWindow, "ERROR: plugin can only handle 8/16-bit samples\n", "ERROR: plugin can only handle 8/16-bit samples", 0); *w/
			file_info->abort_flag = true;
			return;
		}
#endif
 *w/
		// file_info->length_in_msec = file_info->total_samples * 10 / (file_info->sample_rate / 100);
		file_info->length_in_msec = (unsigned int)(((double)file_info->total_samples *
											(double)1000.0) / (double)file_info->sample_rate);
	}
	*/

	//stream_data_struct *stream_data = (stream_data_struct *)client_data;
	FlacContext *pflac = (FlacContext *)client_data;

	(void)decoder;

	if(metadata->type == FLAC__METADATA_TYPE_STREAMINFO)
	{
		if((pflac->total_samples = metadata->data.stream_info.total_samples) == 0) // guess from filesize
		{
			g_state.size = maGetFileSize(pflac->infilefp, g_state.nomem);
			pflac->total_samples = MAFLACCOMPRESSION * g_state.size / (metadata->data.stream_info.channels * metadata->data.stream_info.bits_per_sample);
		}
		pflac->bits_per_sample = metadata->data.stream_info.bits_per_sample;
		pflac->channels = metadata->data.stream_info.channels;
		pflac->sample_rate = metadata->data.stream_info.sample_rate;
		pflac->length_in_msec = (FLAC__uint64)((double)pflac->total_samples / (double)pflac->sample_rate * 1000.0 + 0.5);
	}
	/*
	else if(metadata->type == FLAC__METADATA_TYPE_VORBIS_COMMENT)
	{
		
		double reference, gain, peak;
		if(grabbag__replaygain_load_from_vorbiscomment(metadata, flac_cfg.output.replaygain.album_mode, / *strict=* /false, &reference, &gain, &peak))
		{
			pflac->has_replaygain = true;
			pflac->replay_scale = grabbag__replaygain_compute_scale_factor(peak, gain, (double)flac_cfg.output.replaygain.preamp, / *prevent_clipping=* /!flac_cfg.output.replaygain.hard_limit);
		}
	}
	*/

} // mametadata_callback


void /* FLAC__StreamDecoderErrorCallback */ maerror_callback(const FLAC__StreamDecoder *decoder, FLAC__StreamDecoderErrorStatus status, void *client_data)
{
	FlacContext *pflac = (FlacContext *)client_data;

	(void)decoder;

	if(status != FLAC__STREAM_DECODER_ERROR_STATUS_LOST_SYNC)
	{
		g_state.stop |= STOP_DECODER;
		pflac->is_playing = false;
	}
	/*
	DecoderSession *decoder_session = (DecoderSession*)client_data;
	(void)decoder;
	if(!decoder_session->error_callback_suppress_messages)
		stats_print_name(1, decoder_session->inbasefilename);
		flac__utils_printf(stderr, 1, "*** Got error code %d:%s\n", status, FLAC__StreamDecoderErrorStatusString[status]);
	if(!decoder_session->continue_through_decode_errors) {
		/w* if we got a sync error while looking for metadata, either it's not a FLAC file (more likely) or the file is corrupted *w/
		if(
			!decoder_session->error_callback_suppress_messages &&
			status == FLAC__STREAM_DECODER_ERROR_STATUS_LOST_SYNC &&
			FLAC__stream_decoder_get_state(decoder) == FLAC__STREAM_DECODER_SEARCH_FOR_METADATA
		) {
			flac__utils_printf(stderr, 1,
				"\n"
				"The input file is either not a FLAC file or is corrupted.  If you are\n"
				"convinced it is a FLAC file, you can rerun the same command and add the\n"
				"-F parameter to try and recover as much as possible from the file.\n"
			);
			decoder_session->error_callback_suppress_messages = true;
		}
		else if(status == FLAC__STREAM_DECODER_ERROR_STATUS_UNPARSEABLE_STREAM)
			decoder_session->aborting_due_to_unparseable = true;
		decoder_session->abort_flag = true;
	}
	*/

} // maerror_callback


void masafe_decoder_delete(FLAC__StreamDecoder *decoder)
{
	if(decoder)
	{
		masafe_decoder_finish(decoder);
		FLAC__stream_decoder_delete(decoder);
	}
}


FLAC__StreamDecoderWriteStatus mawrite_callback(const FLAC__StreamDecoder *decoder, const FLAC__Frame *frame, const FLAC__int32 *const buffer[], void *client_data)
{
	FlacContext *pflac = (FlacContext *)client_data;
	const unsigned channels = pflac->channels;
	const unsigned wide_samples = frame->header.blocksize;
	unsigned wide_sample, offset_sample, channel;
	unsigned int wide_samples_in_reservoir = 0;
	machar mastr2[MAXLOGLINELEN];

	(void)decoder;

	if(g_state.stop & (STOP_ALL | STOP_DECODER))
		return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;

	if(pflac->channels < 1 || pflac->channels > 2 || (pflac->bits_per_sample != 16 && pflac->bits_per_sample != 24))
	{
		maLog(L"maPlayFlac failed, only 16-bit or 24-bit mono or stereo is supported, on file:");
		masprintf(mastr2, MAXLOGLINELEN, L"'%ls'", fs2ma(mastr2, g_state.inpath));
		maLog(mastr2);
		pflac->is_playing = false;
		g_state.stop |= STOP_DECODER;
		return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;
	}

	// first sample of frame for displaying position
	pflac->current_sample = (unsigned)frame->header.number.sample_number;

	for(offset_sample = wide_samples_in_reservoir * channels, wide_sample = 0; wide_sample < wide_samples; wide_sample++)
		for(channel = 0; channel < channels; channel++, offset_sample++)
			l_reservoir[offset_sample] = buffer[channel][wide_sample];

	mapack_pcm_signed_little_endian((FLAC__byte *)l_reservoir,
														buffer,
														wide_samples,
														channels,
														pflac->bits_per_sample,
														pflac->sample_format_bytes_per_sample * 8);

	if(maOutChain1((char *)l_reservoir, wide_samples * pflac->channels * pflac->sample_format_bytes_per_sample) != 0)
	{
		maLog(L"maPlayFlac mawrite_callback maOutChain1 failed");
		pflac->is_playing = false;
		g_state.stop |= STOP_DECODER;
		return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;
	}

	return FLAC__STREAM_DECODER_WRITE_STATUS_CONTINUE;

} // mawrite_callback


static FLAC__int32 malinear_dither(unsigned source_bps, unsigned target_bps, FLAC__int32 sample, dither_state *dither, const FLAC__int32 MIN, const FLAC__int32 MAX)
{
	unsigned scalebits;
	FLAC__int32 output, mask, random;

	FLAC__ASSERT(source_bps < 32);
	FLAC__ASSERT(target_bps <= 24);
	FLAC__ASSERT(target_bps <= source_bps);

	/* noise shape */
	sample += dither->error[0] - dither->error[1] + dither->error[2];

	dither->error[2] = dither->error[1];
	dither->error[1] = dither->error[0] / 2;

	/* bias */
	output = sample + (1L << (source_bps - target_bps - 1));

	scalebits = source_bps - target_bps;
	mask = (1L << scalebits) - 1;

	/* dither */
	random = (FLAC__int32)prng(dither->random);
	output += (random & mask) - (dither->random & mask);

	dither->random = random;

	/* clip */
	if(output > MAX) {
		output = MAX;

		if(sample > MAX)
			sample = MAX;
	}
	else if(output < MIN) {
		output = MIN;

		if(sample < MIN)
			sample = MIN;
	}

	/* quantize */
	output &= ~mask;

	/* error feedback */
	dither->error[0] = sample - output;

	/* scale */
	return output >> scalebits;

} // malinear_dither


size_t mapack_pcm_signed_little_endian(	FLAC__byte *data,
										const FLAC__int32 * const input[],
										unsigned wide_samples,
										unsigned channels,
										unsigned source_bps,
										unsigned target_bps)
{
	static dither_state dither[FLAC_PLUGIN__MAX_SUPPORTED_CHANNELS];
	FLAC__byte * const start = data;
	FLAC__int32 sample;
	const FLAC__int32 *input_;
	unsigned samples, channel;
	const unsigned bytes_per_sample = target_bps / 8;
	const unsigned incr = bytes_per_sample * channels;

	FLAC__ASSERT(channels > 0 && channels <= FLAC_PLUGIN__MAX_SUPPORTED_CHANNELS);
	FLAC__ASSERT(source_bps < 32);
	FLAC__ASSERT(target_bps <= 24);
	FLAC__ASSERT(target_bps <= source_bps);
	FLAC__ASSERT((source_bps & 7) == 0);
	FLAC__ASSERT((target_bps & 7) == 0);

	if(source_bps != target_bps) {
		const FLAC__int32 MIN = -(1L << (source_bps - 1));
		const FLAC__int32 MAX = ~MIN; /*(1L << (source_bps-1)) - 1 */

		for(channel = 0; channel < channels; channel++) {
			
			samples = wide_samples;
			data = start + bytes_per_sample * channel;
			input_ = input[channel];

			while(samples--)
			{
				sample = malinear_dither(source_bps, target_bps, *input_++, &dither[channel], MIN, MAX);

				switch(target_bps)
				{
					case 8:
						data[0] = (FLAC__byte)(sample ^ 0x80);
						break;
					case 24:
						data[2] = (FLAC__byte)(sample >> 16);
						/* fall through */
					case 16:
						data[1] = (FLAC__byte)(sample >> 8);
						data[0] = (FLAC__byte)sample;
				}

				data += incr;
			}
		}
	}
	else
	{
		for(channel = 0; channel < channels; channel++) {
			samples = wide_samples;
			data = start + bytes_per_sample * channel;
			input_ = input[channel];

			while(samples--)
			{
				sample = *input_++;

				switch(target_bps)
				{
					case 8:
						data[0] = (FLAC__byte)(sample ^ 0x80);
						break;
					case 24:
						data[2] = (FLAC__byte)(sample >> 16);
						/* fall through */
					case 16:
						data[1] = (FLAC__byte)(sample >> 8);
						data[0] = (FLAC__byte)sample;
				}

				data += incr;
			}
		}
	}

	return wide_samples * channels * (target_bps/8);

} // mapack_pcm_signed_little_endian


#include "FLAC/metadata.h"
size_t /* FLAC__IOCallback_Read */ maiofread_callback(void *ptr, size_t size, size_t nmemb, FLAC__IOHandle handle);
int /* FLAC__IOCallback_Seek */ maiofseeko_callback(FLAC__IOHandle handle, FLAC__int64 offset, int whence);
FLAC__int64 /* FLAC__IOCallback_Tell */ maioftello_callback(FLAC__IOHandle handle);
FLAC__bool mado_major_operation_on_file(MASTATE *pstate, machar **p2, machar **p3, size_t *len3);
FLAC__bool mado_major_operation__list(MASTATE *pstate, FLAC__Metadata_Chain *chain, machar **p2, machar **p3, size_t *len3);

void write_vc_field(machar *fp, const FLAC__StreamMetadata_VorbisComment_Entry *entry, FLAC__bool raw);

int DumpInfoFlac(MASTATE *pstate, machar **p2, machar **p3, size_t *len3)
{
	int ret;
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];	
	g_nobmem = pstate->nomem;
	
	ret =  mado_major_operation_on_file(pstate, p2, p3, len3);
	
	if(!ret)
	{
		DumpString(pstate->dumpfp, L"flac comments:", p2, p3, len3);
		masprintf(mastr, MAXLOGLINELEN, L"DumpTagsFlac do_major_operation_on_file failed on '%ls'", fs2ma(mastr2, pstate->inpath));
		maLog(mastr);
		return 1;		
	}

	return 0;
	
} // DumpInfoFlac


void write_vc_field(machar *fp, const FLAC__StreamMetadata_VorbisComment_Entry *entry, FLAC__bool raw)
{
	char str[STRLEN512];
	machar mastr2[STRLEN512];
	
	if(0 != entry->entry)
	{
		if(!raw)
		{
			/*
			 * WATCHOUT: comments that contain an embedded null will
			 * be truncated by utf_decode().
			 */
			/* machar *converted;

			if(0) // utf8_decode // maconv_utf82charset((const char *)entry->entry, &converted) >= 0)
			{
				mastrcat(fp, converted);
				MAFREE(converted);
			}
			else
			{
			*/
				strncpy(str, (const char *)entry->entry, entry->length); 
				str[entry->length] = 0;
				os2ma(mastr2, str);
				mastr2[entry->length] = L'\0';
				mastrcat(fp, mastr2);
			// }
		}
		else
		{
			mastrncpy(mastr2, (const machar *)entry->entry, entry->length);
			mastr2[entry->length] = L'\0';
			mastrcat(fp, mastr2);
		}
	}

} // write_vc_field


void write_metadata(MASTATE *pstate, /* FILE *fp, */ FLAC__StreamMetadata *block, unsigned block_number, FLAC__bool raw,
						machar **p2, machar **p3, size_t *len3)
{
	unsigned i, j;
	double duration;
	mauint64 total_samples;
	FILE *fp = pstate->dumpfp;
	machar tstr[STRLEN32];
	machar mastr2[STRLEN512];
	machar mastr3[STRLEN512];

	i = 0;
	if(block_number == 0) // do duration here
	{
		if((total_samples = block->data.stream_info.total_samples) == 0) // guess from filesize
		{
			pstate->size = maGetFileSize(pstate->fp, pstate->nomem);
			total_samples = MAFLACCOMPRESSION * pstate->size /
					(block->data.stream_info.channels * block->data.stream_info.bits_per_sample);
			i = 1; // guess flag
		}
		if((duration = (double)(MSPERSEC * total_samples / block->data.stream_info.sample_rate)) < MSPERSEC)
			masprintf(mastr2, STRLEN512, L"Duration : %d ms.", (int)duration);
		else
			masprintf(mastr2, STRLEN512, L"Duration : %ls", MillisecondsToStrHMSD(tstr, (mauint64)duration));
		if(i)
			mastrcat(mastr2, L" (guess from file size)");
		DumpString(fp, mastr2, p2, p3, len3);
	
		DumpString(fp, L"flac comments:", p2, p3, len3);	
	}
	
	masprintf(mastr2, STRLEN512, L"METADATA block #%u", block_number);
	DumpString(fp, mastr2, p2, p3, len3);
	masprintf(mastr2, STRLEN512, L"  type: %u (%ls)", (unsigned)block->type, block->type < FLAC__METADATA_TYPE_UNDEFINED?
		os2ma(mastr3, (char *)FLAC__MetadataTypeString[block->type]) : L"UNKNOWN");
	DumpString(fp, mastr2, p2, p3, len3);
	masprintf(mastr2, STRLEN512, L"  is last: %ls", block->is_last? L"true" : L"false");
	DumpString(fp, mastr2, p2, p3, len3);
	masprintf(mastr2, STRLEN512, L"  length: %u", block->length);
	DumpString(fp, mastr2, p2, p3, len3);
	
	switch(block->type)
	{
		case FLAC__METADATA_TYPE_STREAMINFO:
			masprintf(mastr2, STRLEN512, L"  minumum blocksize: %u samples", block->data.stream_info.min_blocksize);
			DumpString(fp, mastr2, p2, p3, len3);
			masprintf(mastr2, STRLEN512, L"  maximum blocksize: %u samples", block->data.stream_info.max_blocksize);
			DumpString(fp, mastr2, p2, p3, len3);
			masprintf(mastr2, STRLEN512, L"  minimum framesize: %u bytes", block->data.stream_info.min_framesize);
			DumpString(fp, mastr2, p2, p3, len3);
			masprintf(mastr2, STRLEN512, L"  maximum framesize: %u bytes", block->data.stream_info.max_framesize);
			DumpString(fp, mastr2, p2, p3, len3);
			masprintf(mastr2, STRLEN512, L"  sample_rate: %u Hz", block->data.stream_info.sample_rate);
			DumpString(fp, mastr2, p2, p3, len3);
			masprintf(mastr2, STRLEN512, L"  channels: %u", block->data.stream_info.channels);
			DumpString(fp, mastr2, p2, p3, len3);
			masprintf(mastr2, STRLEN512, L"  bits-per-sample: %u", block->data.stream_info.bits_per_sample);
			DumpString(fp, mastr2, p2, p3, len3);
#ifdef _MSC_VER
			masprintf(mastr2, STRLEN512, L"  total samples: %I64u", block->data.stream_info.total_samples);
#else
			masprintf(mastr2, STRLEN512, L"  total samples: %llu", block->data.stream_info.total_samples);
#endif
			DumpString(fp, mastr2, p2, p3, len3);
			masprintf(mastr2, STRLEN512, L"  MD5 signature: ");
			for(i = 0; i < 16; i++)
				masprintf(mastr2, STRLEN512, L"%ls%02x", mastr2, (unsigned)block->data.stream_info.md5sum[i]);
			DumpString(fp, mastr2, p2, p3, len3);
		break;
		
		case FLAC__METADATA_TYPE_PADDING:
		break;
		
		case FLAC__METADATA_TYPE_APPLICATION:
			masprintf(mastr2, STRLEN512, L"  application ID: ");
			DumpString(fp, mastr2, p2, p3, len3);
			for(i = 0; i < 4; i++)
				masprintf(mastr2, STRLEN512, L"%02x ", block->data.application.id[i]);
			DumpString(fp, mastr2, p2, p3, len3);
			masprintf(mastr2, STRLEN512, L"  data contents:");
			DumpString(fp, mastr2, p2, p3, len3);
			if(0 != block->data.application.data)
			{
				masprintf(mastr2, STRLEN512, L"  application data length : %u", block->length - FLAC__STREAM_METADATA_HEADER_LENGTH);
				DumpString(fp, mastr2, p2, p3, len3);
			}
		break;
		
		case FLAC__METADATA_TYPE_SEEKTABLE:
			masprintf(mastr2, STRLEN512, L"  seek points: %u", block->data.seek_table.num_points);
			DumpString(fp, mastr2, p2, p3, len3);
			for(i = 0; i < block->data.seek_table.num_points; i++)
			{
				if(block->data.seek_table.points[i].sample_number != FLAC__STREAM_METADATA_SEEKPOINT_PLACEHOLDER)
				{
#ifdef _MSC_VER
					masprintf(mastr2, STRLEN512, L"    point %u: sample_number=%I64u, stream_offset=%I64u, frame_samples=%u", i, block->data.seek_table.points[i].sample_number, block->data.seek_table.points[i].stream_offset, block->data.seek_table.points[i].frame_samples);
#else
					masprintf(mastr2, STRLEN512, L"    point %u: sample_number=%llu, stream_offset=%llu, frame_samples=%u", i, block->data.seek_table.points[i].sample_number, block->data.seek_table.points[i].stream_offset, block->data.seek_table.points[i].frame_samples);
#endif
					DumpString(fp, mastr2, p2, p3, len3);
				}
				else
				{
					masprintf(mastr2, STRLEN512, L"    point %u: PLACEHOLDER", i);
					DumpString(fp, mastr2, p2, p3, len3);
				}
			}
		break;
		
		case FLAC__METADATA_TYPE_VORBIS_COMMENT:
			masprintf(mastr2, STRLEN512, L"  vendor string: ");
			write_vc_field(mastr2, &block->data.vorbis_comment.vendor_string, raw);
			DumpString(fp, mastr2, p2, p3, len3);
			masprintf(mastr2, STRLEN512, L"  comments: %u", block->data.vorbis_comment.num_comments);
			DumpString(fp, mastr2, p2, p3, len3);
			for(i = 0; i < block->data.vorbis_comment.num_comments; i++)
			{
				masprintf(mastr2, STRLEN512, L"    comment[%u]: ", i);
				write_vc_field(mastr2,&block->data.vorbis_comment.comments[i], raw);
				DumpString(fp, mastr2, p2, p3, len3);
			}
		break;
			
		case FLAC__METADATA_TYPE_CUESHEET:
			masprintf(mastr2, STRLEN512, L"  media catalog number: %ls", block->data.cue_sheet.media_catalog_number);
			DumpString(fp, mastr2, p2, p3, len3);
#ifdef _MSC_VER
			masprintf(mastr2, STRLEN512, L"  lead-in: %I64u", block->data.cue_sheet.lead_in);
#else
			masprintf(mastr2, STRLEN512, L"  lead-in: %llu", block->data.cue_sheet.lead_in);
#endif
			DumpString(fp, mastr2, p2, p3, len3);
			masprintf(mastr2, STRLEN512, L"  is CD: %ls", block->data.cue_sheet.is_cd? "true":"false");
			DumpString(fp, mastr2, p2, p3, len3);
			masprintf(mastr2, STRLEN512, L"  number of tracks: %u", block->data.cue_sheet.num_tracks);
			for(i = 0; i < block->data.cue_sheet.num_tracks; i++)
			{
				const FLAC__StreamMetadata_CueSheet_Track *track = block->data.cue_sheet.tracks+i;
				const FLAC__bool is_last = (i == block->data.cue_sheet.num_tracks-1);
				const FLAC__bool is_leadout = is_last && track->num_indices == 0;
				masprintf(mastr2, STRLEN512, L"    track[%u]", i);
				DumpString(fp, mastr2, p2, p3, len3);
#ifdef _MSC_VER
				masprintf(mastr2, STRLEN512, L"      offset: %I64u", track->offset);
#else
				masprintf(mastr2, STRLEN512, L"      offset: %llu", track->offset);
#endif
				DumpString(fp, mastr2, p2, p3, len3);
				if(is_last)
				{
					masprintf(mastr2, STRLEN512, L"      number: %u (%ls)", (unsigned)track->number, is_leadout? "LEAD-OUT" : "INVALID");
					DumpString(fp, mastr2, p2, p3, len3);
				}
				else
				{
					masprintf(mastr2, STRLEN512, L"      number: %u", (unsigned)track->number);
					DumpString(fp, mastr2, p2, p3, len3);
				}
				if(!is_leadout)
				{
					masprintf(mastr2, STRLEN512, L"      ISRC: %ls", track->isrc);
					DumpString(fp, mastr2, p2, p3, len3);
					masprintf(mastr2, STRLEN512, L"      type: %ls", track->type == 1? L"DATA" : L"AUDIO");
					DumpString(fp, mastr2, p2, p3, len3);
					masprintf(mastr2, STRLEN512, L"      pre-emphasis: %ls", track->pre_emphasis? L"true" : L"false");
					DumpString(fp, mastr2, p2, p3, len3);
					masprintf(mastr2, STRLEN512, L"      number of index points: %u", track->num_indices);
					DumpString(fp, mastr2, p2, p3, len3);
					for(j = 0; j < track->num_indices; j++)
					{
						const FLAC__StreamMetadata_CueSheet_Index *index = track->indices+j;
						masprintf(mastr2, STRLEN512, L"        index[%u]", j);
						DumpString(fp, mastr2, p2, p3, len3);
#ifdef _MSC_VER
						masprintf(mastr2, STRLEN512, L"          offset: %I64u", index->offset);
#else
						masprintf(mastr2, STRLEN512, L"          offset: %llu", index->offset);
#endif
						DumpString(fp, mastr2, p2, p3, len3);
						masprintf(mastr2, STRLEN512, L"          number: %u", (unsigned)index->number);
						DumpString(fp, mastr2, p2, p3, len3);
					}
				}
			}
		break;
		
		default:
			masprintf(mastr2, STRLEN512, L"  data contents length = %u:", block->length);
			DumpString(fp, mastr2, p2, p3, len3);
		break;
	}

} // write_metadata


FLAC__bool do_major_operation__list(MASTATE *pstate, FLAC__Metadata_Chain *chain, machar **p2, machar **p3, size_t *len3)
{
	FLAC__Metadata_Iterator *iterator = FLAC__metadata_iterator_new();
	FLAC__StreamMetadata *block;
	FLAC__bool ok = true;
	unsigned block_number;
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];

	if(0 == iterator)
	{
		maLog(L"Flac do_major_operation__list out of memory allocating iterator");
		return false;
	}

	FLAC__metadata_iterator_init(iterator, chain);

	block_number = 0;
	do
	{
		block = FLAC__metadata_iterator_get_block(iterator);
		ok &= (0 != block);
		if(!ok)
		{
			masprintf(mastr, MAXLOGLINELEN, L"Flac do_major_operation__list couldn't get block from chain for '%ls'",
										fs2ma(mastr2, (fschar *)pstate->inpath));
			maLog(mastr);
		}
		else
			write_metadata(pstate, /* pstate->dumpfp, */ block, block_number, 0, p2, p3, len3);
			
		block_number++;

	} while(ok && FLAC__metadata_iterator_next(iterator));

	FLAC__metadata_iterator_delete(iterator);

	return ok;
	
} // do_major_operation__list


size_t /* FLAC__IOCallback_Read */ maiofread_callback(void *ptr, size_t size, size_t nmemb, FLAC__IOHandle handle)
{
	return mbmfread(ptr, size, nmemb, handle);
}


int /* FLAC__IOCallback_Seek */ maiofseeko_callback(FLAC__IOHandle handle, FLAC__int64 offset, int whence)
{
	 return mbmfseeko(handle, offset, whence);
}


FLAC__int64 /* FLAC__IOCallback_Tell */ maioftello_callback(FLAC__IOHandle handle)
{
	return (FLAC__int64)mbmftello(handle);
}


FLAC__bool mado_major_operation_on_file(MASTATE *pstate, machar **p2, machar **p3, size_t *len3)
{
	FLAC__bool ok;
	FLAC__Metadata_Chain *chain = FLAC__metadata_chain_new();
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];	
	FLAC__IOCallbacks callbacks;

	if(0 == chain)
	{
		maLog(L"Flac do_major_operation_on_file out of memory allocating chain");
		return false;
	}
	
	callbacks.read = maiofread_callback;
	callbacks.write = NULL;
	callbacks.seek = maiofseeko_callback;
	callbacks.tell = maioftello_callback;
	callbacks.eof = NULL;
	callbacks.close = NULL;

	if(!FLAC__metadata_chain_read_with_callbacks(chain, pstate->fp, callbacks))
	{
		masprintf(mastr, MAXLOGLINELEN, L"Flac error reading metadata '%ls'",
												fs2ma(mastr2, pstate->inpath));
		maLog(mastr);
		FLAC__metadata_chain_delete(chain);
		return false;
	}

	ok = do_major_operation__list(pstate, chain, p2, p3, len3);

	FLAC__metadata_chain_delete(chain);

 	return ok;

} // mado_major_operation_on_file

#ifdef __cplusplus
}
#endif

#endif // !NODECODEFLAC

// eof
