// maPlayOgg.c

/*
 * MoreAmp
 * Copyright (C) 2004-2014 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include "maConfig.h" // included by all moreamp files

#ifndef NODECODEOGG

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#ifdef __MAWIN__
 #include <wtypes.h>
 #include <limits.h>
 #include <time.h>
#endif

#define OV_EXCLUDE_STATIC_CALLBACKS // avoid warnings
#include "ogg/ogg.h" // libs/libogg/include/ogg/os_types may need __MACOS__or __MACOSX__ or __MAWIN__ or __BEOS__ etc.
#include "vorbis/codec.h"
#include "vorbis/vorbisfile.h"

#include "maPlayer.h"
#include "maUtils.h"
#include "maPlayOgg.h"
#include "maAudioPA.h"

typedef struct
{
	// int m_iFileLength_Secs;
	mauint64 m_iFileLength_ms; // pcm duration
	int m_iBitRate_Kbs;
	int m_iFreq_Hz;
	int m_bStereo;
	int m_b16bit;
} OggFileInfo;

typedef struct
{
	FILE *m_hFile;
	OggFileInfo m_FileInfo;
	int oggInstance;
	OggVorbis_File vf;
	int current_section;
	MASTATE *pstate;
} OggContext;

int OggOpenFile(OggContext *pOggContext);
void OggDecodeStream();
void OggCloseFile(OggContext *pOggContext);
int OggGetPCMBlock(OggContext *pOggContext, void *pBlock, int *pdwBlockSize);

OggContext g_OggContext;
OggContext *g_pOggContext;

// from vorbisfile.c :
static int _memfseek64_wrap(FILE *f,ogg_int64_t off,int whence);
static int _memfseek64_wrap(FILE *f,ogg_int64_t off,int whence){
  if(f==NULL)return(-1);
  return memfseek(f,(int)off,whence);
}

int ov_memopen(FILE *f, OggVorbis_File *vf, char *initial, long ibytes);
int ov_memopen(FILE *f, OggVorbis_File *vf, char *initial, long ibytes)
{
	ov_callbacks callbacks = {
		(size_t (*)(void *, size_t, size_t, void *))  memfread,
		(int (*)(void *, ogg_int64_t, int))			  _memfseek64_wrap,
		(int (*)(void *))                             memfclose,
		(long (*)(void *))                            memftell
	};

  return ov_open_callbacks((void *)f, vf, initial, ibytes, callbacks);
}

int ov_memtest(FILE *f, OggVorbis_File *vf, char *initial, long ibytes);
int ov_memtest(FILE *f, OggVorbis_File *vf, char *initial, long ibytes){
	ov_callbacks callbacks = {
		(size_t (*)(void *, size_t, size_t, void *))  memfread,
		(int (*)(void *, ogg_int64_t, int))           _memfseek64_wrap,
		(int (*)(void *))                             memfclose,
		(long (*)(void *))                            memftell
	};

  return ov_test_callbacks((void *)f, vf, initial, ibytes, callbacks);
}


ogg_int64_t l_oggseek = -1;
void oggseek(mauint64 pos, mauint64 siz)
{
	if(!g_pOggContext || l_oggseek >= 0 || pos < 0 || siz <= 0)
		return;

	if(g_pOggContext->vf.pcmlengths)
		l_oggseek = (ogg_int64_t)(((double)pos / (double)siz) * (double)ov_pcm_total(&g_pOggContext->vf, -1));	

} // oggseek


void maPlayOgg()
{
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];

	if(!g_state.inpath)
	{
		maLog(L"maPlayOgg error null filename");
		return;
	}
	
	// format of this decoder output
#ifdef WORDS_BIGENDIAN
	g_state.inwordsbigendian = 1;
#else
	g_state.inwordsbigendian = 0;
#endif
	g_state.ininterleaved = 1;
	
	g_pOggContext = &g_OggContext;
	memset(g_pOggContext, 0, sizeof(OggContext));
	g_OggContext.pstate = &g_state;
	
	// open input
	if(OggOpenFile(g_pOggContext))
	{
		if(g_state.nomem)
		{
			if(g_state.fp)
				fclose(g_state.fp);
		}
		else
			memfclose(g_state.fp);
		g_pOggContext = NULL;
		masprintf(mastr, MAXLOGLINELEN, L"maPlayOgg OggOpenFile failed on '%ls'",
			fs2ma(mastr2, g_state.inpath));
		maLog(mastr);
		return;
	}
		
	g_state.position = 0;
	
 	if(maIOStart() < 0)
	{
		OggCloseFile(g_pOggContext);
		g_pOggContext = NULL;
		maLog(L"maPlayOgg maIOStart failed");
		return;
	}

	if(!g_posendon || g_posend > g_state.duration)
		g_posend = g_state.duration;

	// ogg decode loop
	for (; ;)
	{	
		if(g_state.stop & (STOP_ALL | STOP_DECODER))
			break;
				
		if(!g_pOggContext)
			break;

		if(l_oggseek >= 0)
		{
			ov_pcm_seek(&g_pOggContext->vf, l_oggseek);
			g_state.position = (mauint64)(LROUND(ov_time_tell(&g_pOggContext->vf) * MSPERSEC));	
			l_oggseek = -1;
		}

		OggDecodeStream();
	
		if(g_pOggContext)
			g_state.position = (mauint64)(LROUND(ov_time_tell(&g_pOggContext->vf) * MSPERSEC));

		if(!g_pOggContext) // OggDecodeStream sets to NULL at end of stream
			break;

	} // while 1

	// finish

	l_oggseek = -1;

	if(g_pOggContext)
	{
		OggCloseFile(g_pOggContext);
		g_pOggContext = NULL;
	}

	maOutChainFlush();

} // maPlayOgg


#define CPC_OUTPUTBLOCKSIZE 0x1000 // Block

void OggDecodeStream()
{
	machar pcmbuffer[CPC_OUTPUTBLOCKSIZE];
	int moredata;
	int pcmblocksize = CPC_OUTPUTBLOCKSIZE;
	machar mastr[MAXLOGLINELEN];

	g_state.inchannels = g_pOggContext->m_FileInfo.m_bStereo == 0 ? 1 : 2;
	
	// Get block from Codec
	moredata = OggGetPCMBlock(g_pOggContext, pcmbuffer, &pcmblocksize);
		
	if(pcmblocksize > 0)
	{
		// pass data on to output processing chain
		if(maOutChain1((char *)pcmbuffer, pcmblocksize) != 0)
		{
			masprintf(mastr, MAXLOGLINELEN,
				L"OggDecodeStream maOutChain1 failed : len=%d g_state.stop=x%x",
																pcmblocksize, g_state.stop);
			maLog(mastr);
			goto oggdecodereturn;
		}

	} //if(pcmblocksize > 0)

	if(moredata != 0)
		return;

	// song is finished
	if(!g_state.stop && !g_loadedquit && g_reptsong)
	{
		l_oggseek = g_posbeg;
		return;
	}

oggdecodereturn:
	OggCloseFile(g_pOggContext);
	g_pOggContext = NULL;

} // OggDecodeStream



double ov_timems_total(OggVorbis_File *vf, int i);
double ov_timems_total(OggVorbis_File *vf, int i)
{
  if(vf->ready_state < OPENED)
	  return(OV_EINVAL);
  if(!vf->seekable || i >= vf->links)
	  return(OV_EINVAL);
  if(i < 0)
  {
    double acc = 0;
    int i;
    for(i = 0; i < vf->links; i++)
      acc += ov_time_total(vf, i);
    return(acc * MSPERSEC);
  }
  else
    return((double)((MSPERSEC * vf->pcmlengths[i * 2 + 1])) / vf->vi[i].rate);

} // ov_timems_total


int OggOpenFile(OggContext *pOggContext)
{
	vorbis_info *pInfo = NULL;
	int ret;
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];
	
	// If a stream is open close it
	if(pOggContext->m_hFile != NULL)
	{
		ov_clear(&pOggContext->vf);
		pOggContext->m_hFile = NULL;
	}

	// Open the new stream
	if(pOggContext->pstate->nomem)
	{
		pOggContext->m_hFile = mafopen(pOggContext->pstate->inpath, fsT("rb"));
		if(pOggContext->m_hFile == NULL)
		{
			masprintf(mastr, MAXLOGLINELEN, L"OggOpenFile mafopen failed on '%ls'",
				fs2ma(mastr2, pOggContext->pstate->inpath));
			maLog(mastr);
			return 1;
		}
	}
	else
	{
		pOggContext->m_hFile = memfopen(pOggContext->pstate->inpath);
		if(pOggContext->m_hFile == NULL)
		{
			masprintf(mastr, MAXLOGLINELEN, L"OggOpenFile memfopen failed on '%ls'",
				fs2ma(mastr2, pOggContext->pstate->inpath));
			maLog(mastr);
			return 1;
		}
	}
	
	memset(&pOggContext->vf, 0, sizeof(pOggContext->vf));

	if(pOggContext->pstate->nomem)
		ret = ov_open(pOggContext->m_hFile, &pOggContext->vf, NULL, 0);
	else
		ret = ov_memopen(pOggContext->m_hFile, &pOggContext->vf, NULL, 0);

	if(ret < 0) 
	{
		maLog(L"OggOpenFile input does not appear to be an Ogg bitstream.");
		if(pOggContext->pstate->nomem)
			fclose(pOggContext->m_hFile);
		else
			memfclose(pOggContext->m_hFile);		
		pOggContext->m_hFile = NULL;
		return 1;
	}

	if(ov_streams(&pOggContext->vf) != 1)
	{
		ov_clear(&pOggContext->vf);
		if(pOggContext->pstate->nomem)
			fclose(pOggContext->m_hFile);
		else
			memfclose(pOggContext->m_hFile);		
		pOggContext->m_hFile = NULL;
		return 1;
	}

	pInfo = ov_info(&pOggContext->vf, -1);

	if(pInfo == NULL)
	{
		maLog(L"OggOpenFile unable to get ogg info.");
		ov_clear(&pOggContext->vf);
		if(pOggContext->pstate->nomem)
			fclose(pOggContext->m_hFile);
		else
			memfclose(pOggContext->m_hFile);		
		pOggContext->m_hFile = NULL;
		return 1;
	}

	if(pInfo->channels > 2)
	{
		ov_clear(&pOggContext->vf);
		if(pOggContext->pstate->nomem)
			fclose(pOggContext->m_hFile);
		else
			memfclose(pOggContext->m_hFile);		
		pOggContext->m_hFile = NULL;
		return 1;
	}

	// Load m_FileInfo
	pOggContext->m_FileInfo.m_b16bit = 1;
	// pOggContext->m_FileInfo.m_iFileLength_Secs = (int)ov_time_total(&pOggContext->vf, -1);
	pOggContext->m_FileInfo.m_iFileLength_ms = (mauint64)ov_timems_total(&pOggContext->vf, -1);
	pOggContext->m_FileInfo.m_bStereo = (pInfo->channels == 2);
	pOggContext->pstate->insamplerate =
				pOggContext->m_FileInfo.m_iBitRate_Kbs = ov_bitrate(&pOggContext->vf, -1);
	pOggContext->m_FileInfo.m_iBitRate_Kbs /= 1000;
	pOggContext->pstate->insamplerate = pOggContext->m_FileInfo.m_iFreq_Hz = pInfo->rate;
	// pOggContext->pstate->duration = (mauint64)pOggContext->m_FileInfo.m_iFileLength_Secs * 1000;
	pOggContext->pstate->duration = (mauint64)pOggContext->m_FileInfo.m_iFileLength_ms ;

	return 0;
	
} // OggOpenFile


void OggCloseFile(OggContext *pOggContext)
{
	if(pOggContext)
	{
		ov_clear(&pOggContext->vf);
		if(pOggContext->m_hFile)
		 	pOggContext->m_hFile = NULL;
	}
	
} // OggCloseFile


int OggGetPCMBlock(OggContext *pOggContext, void *pBlock, int *pBlockSize)
{
	int ret;
	int bytes_to_read = *pBlockSize;

	while(bytes_to_read > 0) 
    { 
		ret = ov_read(&pOggContext->vf, (char *)pBlock + (*pBlockSize - bytes_to_read), bytes_to_read,

#ifdef WORDS_BIGENDIAN
			1, // big endian
#else
			0, // little endian
#endif
			2, 1, &pOggContext->current_section);

		if(ret == 0)
		{ 
			*pBlockSize = 0;
			return 0;
			break; 
		} else if(ret > 0) 
			bytes_to_read -= ret; 
	} 
	
	return 1; // more data to come
	
} // OggGetPCMBlock


int DumpInfoOgg(MASTATE *pstate, machar **p2, machar **p3, size_t *len3)
{
	// int intLength;
	char **ptr;
	OggContext lOggContext = {NULL};
	vorbis_info *vi;
	machar tstr[STRLEN32];
	machar mastr2[MAXLOGLINELEN];	
	machar mastr[MAXLOGLINELEN];
	
	lOggContext.pstate = pstate;
	
	if(OggOpenFile(&lOggContext))
	{
		OggCloseFile(&lOggContext);
		masprintf(mastr, MAXLOGLINELEN, L"DumpInfoOgg OggOpenFile failed on '%ls'",
			fs2ma(mastr2, pstate->inpath));
		maLog(mastr);
		return 1;		
	}
	
	// intLength = (int)(LROUND(ov_time_total(&lOggContext.vf, -1))); // in seconds
		
	// if((pstate->duration = (mauint64)(intLength * MSPERSEC)) < MSPERSEC)
	if(pstate->duration < MSPERSEC)
		// masprintf(mastr, STRLEN512, L"Duration : < 1s.", (int)pstate->duration);
		masprintf(mastr, STRLEN512, L"Duration : %d ms.", (int)pstate->duration);
	else
		masprintf(mastr, STRLEN512, L"Duration : %ls", MillisecondsToStrHMSD(tstr, pstate->duration));
	DumpString(pstate->dumpfp, mastr, p2, p3, len3);

	ptr = ov_comment(&lOggContext.vf, -1)->user_comments;
	vi = ov_info(&lOggContext.vf, -1);

	masprintf(mastr, MAXLOGLINELEN, L"Channels=%d", vi->channels);
	DumpString(pstate->dumpfp, mastr, p2, p3, len3);
	
	masprintf(mastr, MAXLOGLINELEN, L"Samplerate=%ld Hz", vi->rate);
	DumpString(pstate->dumpfp, mastr, p2, p3, len3);
	
	if(vi->bitrate_upper > 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"Bitrate_upper=%ld bps", vi->bitrate_upper);
		DumpString(pstate->dumpfp, mastr, p2, p3, len3);
	}
	if(vi->bitrate_nominal > 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"Bitrate_nominal=%ld bps", vi->bitrate_nominal);
		DumpString(pstate->dumpfp, mastr, p2, p3, len3);
	}
	if(vi->bitrate_lower > 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"Bitrate_lower=%ld bps", vi->bitrate_lower);
		DumpString(pstate->dumpfp, mastr, p2, p3, len3);
	}
	if(vi->bitrate_window > 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"Bitrate_window=%ld bps", vi->bitrate_window);
		DumpString(pstate->dumpfp, mastr, p2, p3, len3);
	}

	masprintf(mastr, MAXLOGLINELEN, L"Decoded length=%ld samples", (long)ov_pcm_total(&lOggContext.vf, -1));
	DumpString(pstate->dumpfp, mastr, p2, p3, len3);

	DumpString(pstate->dumpfp, L"ogg tags:", p2, p3, len3);	

	while(*ptr)
	{
		masprintf(mastr, MAXLOGLINELEN, L"%ls", os2ma(mastr2, *ptr));
		DumpString(pstate->dumpfp, mastr, p2, p3, len3);
		++ptr;
	}

	masprintf(mastr, MAXLOGLINELEN, L"Encoded by: %ls", os2ma(mastr2, ov_comment(&lOggContext.vf, -1)->vendor));
	DumpString(pstate->dumpfp, mastr, p2, p3, len3);
	
	if(lOggContext.m_hFile != NULL)
		ov_clear(&lOggContext.vf);
	
	return 0;

} // DumpInfoOgg

#ifdef __cplusplus
}
#endif

#endif // NODECODEOGG

// eof
