// maPlayer.h

/*
 * MoreAmp
 * Copyright (C) 2004-2014 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef __MAPLAYER_H__
#define __MAPLAYER_H__

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdio.h>

#if defined(__MAWIN__)
 #include "Windows.h"
#endif

#if defined(__MAOSX__)
 #include <MacTypes.h>
#endif

#if defined(__MACUNIX__)
 #include <sys/types.h>
#endif

#if defined(__MAUNIX__)
 #include <pthread.h>
#endif

#include "maTypes.h"
#include "maDefs.h"
#include "maUtils.h"

typedef struct
{
	unsigned long size;
	unsigned long contains;
	unsigned long start;
	char *storage;
	void *semaphore;
#if defined(__MAWIN__)
	CRITICAL_SECTION lock;
#else
	pthread_mutex_t lock;
#endif
} maFifo;

typedef struct
{
	unsigned long size;
	unsigned long contains;
	unsigned long start;
	char *storage;
	void *semaphore;
} maFifoB;

void PlayThread(threadparametertype parameter);
typedef struct
{
	int infiletype;					// kMAXXX enumeration value, e.g. kMAMP3
	FILE *fp;						// songfile path file pointer
	fschar inpath[MAX_PATH];		// songfile path
	int nomem;						// don't use song mem
	threadparametertype pdecoder;	// one for each infiletype e.g. maPlayMp3
	// void *playersem;				// unused
	mauint64 size;					// songfile size in bytes
	mauint64 duration;				// total playing time in ms
	int bitrate;					// average bitrate in kbps
	mauint64 position;				// current playing position in ms
	double mpegattenuation;			// mp1 mp2 mp3 attenuation when limiter is on (gain 0.0 to 1.0)
	int stop;						// stop flag
	int running;					// player is rnning
	int paused;						// player is paused
	int scanning;					// player is scanning
	int silence;					// flag to silence the output
	int insamplerate;				// songfile samplerate
	int outsamplerate;				// output samplerate
	int inchannels;					// songfile number of channels
	int outchannels;				// output number of channels
	int inwordsbigendian;			// songfile decoded data
	int ininterleaved;				// songfile decoded data
	int iostreamopened;				// an input or output stream has been opened
	FILE *dumpfp;					// dump song info file pointer
	int dumpoff;					// flag for dump song info
	void *fsf;						// aif SNDFILE *
	void *psf;						// aif SF_PRIVATE *
	size_t pcmnbytes;				// number of bytes of pcm data to dump to effects buffer instead of to next output chain
	size_t pcmmax;					// size of effects pcm buffer to alloc
	char *pcm;						// where to dump effects pcm data
	char *pcmpos;					// current position of the effects pcm data
	int loadingsbsms;				// non-zero while decoding a song for sbsms
} MASTATE;

// settings
typedef struct
{
	int eqslider[EQBANDS31];
	int eqoffset;				// if adding members to this struct
	fschar eqname[EQNAMELEN];	// keep eqname last
} EQUPRESET;

typedef struct
{
	/* don't use VBR
	int encoding;
	int minbitrate;
	int maxbitrate;
	int vbrquality;
	*/
	int type; // kMAMP3, kMAOGG, etc.
	int bitrate;
	int quality;
	int mode;
	int samplerate;
	fschar outdir[MAX_PATH];
} FILEOUT;								

typedef struct
{
	int ison;				// window is on screen
	int xpos[NUMWINCONF];   // coordinates
	int ypos[NUMWINCONF];
	int xsize[NUMWINCONF];
	int ysize[NUMWINCONF];
} MAWINDOW;

typedef struct
{
	int autostart;					// start player when app is launched
	int random;						// get songs from playlist randomly
	int reptplayed;					// allow to repeat a song when in random mode 
	int looploaded;					// loop on songs that have been loaded in mem or vdisk
	int looplist;					// loop on playlist else stop when all have played 
	int inputmonitor;				// listen to audio input
	int effects; 					// effects active
	int limiter; 					// mad limiter
	int balance;					// free or joined left and right volume controls
	int monodiff;					// L-R in mono; reverse L,R in stereo
	int invertright;				// invert right channel
	int eqflags;					// equalizer flags
	int eqpreset;					// eq preset index
	int volumesliderleft; 			// volume L value
	int volumesliderright; 			// volume R value
	int separationslider; 			// separation value (stereo) 
	int panslider;					// pan value (mono)
	int preampslider;   			// preamp value (mad)
	EQUPRESET eqpresets[EQNUMPRESETS];  // equalizer values
	int pitchslider;				// pitch effect value
	int pitchslider2;				// pitch effect value (fine)
	int temposlider;				// tempo effect value
	int speedslider;				// speed effect value
	fschar vdiskname[MAX_PATH]; 	// virtual disk name
	int buffertype;					// type of buffering for songs (mem, vdisk, direct)
	int output;						// output to use from Output popup
	int buffersize;					// max mem or vdisk used, in Mbytes
	int titleinfo;					// put info in window's title bar
	int winconf;					// player window configuration
	int singleplay;					// stop afer every song
	FILEOUT fileout[kOUTMAX];		// file output parameters	
	int logflags;					// stop or limit logging
	int logmax;						// max log entries per session
	int logevents;					// max log entries per fast log period
	int logperiod;					// fast log period
	int multipleinst;				// multiple instances
	int keyboard;					// use keyboard
	int listcolxsize[NUMWINCONF][4];// h scrollable if wider than list window 
	int mousewheel;					// use mousewheel
	int usem3u;						// use m3u files
	int addmode;					// add new files to existing madrop entries
	EQUPRESET eq2presets[EQNUMPRESETS]; // equalizer 2 values
	MAWINDOW mawin[NUMWINDOWS];		// all windows
	int listboxmode;				// show list window | which to show
	int exitafter;					// exit after list has played (if no looplist)
	int future1;					// for future use before having to update settings version
	int future2;
	int future3;
	int future4;
	int future5;
	int future6;
	int future7;
	// new values in this settings version 0 1 29 :
	int hostinenable;
	char audiohostinname[MAX_PATH];
	char audiohostoutname[MAX_PATH];
	char audiodevinname[MAX_PATH];
	char audiodevoutname[MAX_PATH];
	char futurepath[MAX_PATH];
	char settingsversion[VERSIONSTRINGSIZE]; // first version to use this settings format, keep this here (last)
} MASETS;

#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
 typedef struct
 {
	int cdparanoia;									// cdparanoia active
	int item;										// last item selected to set
	char cdaname[CDAMAXNUMDRIVES + 1][STRLEN32];	// cdrom device names, e.g. '/dev/cdrom'
 } MASETSCDROM;
 extern MASETSCDROM g_setscdrom;
#endif

#define MACLOSESOUNDDEF 0

#if defined(MAMOUSEWHEEL)
 #if defined(MAGUIGTK2)
  #define MAWHEELVOLUMEDELTA (VOLUMEMAX / 20) // 100)
 #else
  #define MAWHEELVOLUMEDELTA (VOLUMEMAX / 20)
 #endif
#endif

#if defined(MAKEYBOARD)
 #define MANUMKEYS 69
 #define MAKEYDELIM L'\"'
 #define MAKEYVOLUMEDELTA (VOLUMEMAX / 20)
 typedef struct
 {
	union
	{
		machar makey[5]; // the key pressed
		wchar_t pad[5]; // for constant struct size
	} keyx;
	machar *label;		// label for internal use
	machar *labeld;		// default label for user
	void (*function)(); // the function called
	int menuid;			// menu ids for displaying shortcuts
	int textid;			// text ids for displaying shortcuts
 } MAKEYBINDINGS;
 extern MAKEYBINDINGS g_keybindings[];
#endif

#define MAWXK_UP 1				// 317 enums, not char values
#define MAWXK_DOWN 2			// 319
#define MAWXK_RETURN 3			// 13
#define MAWXK_RETURN_NUMPAD 4	// 3
#define MAWXK_ESCAPE 5			// 27

#define MAPROMPTMAX 62
#define USEM3UDEF 0			// default off
#define EXITAFTERDEF 0		// default off
#define ADDMODEDEF 0		// default off
#define CHECKENABLEDEF 0	// default off

extern int g_scanstop;
extern int g_scanfiles;
extern MASTATE g_state;
extern MASETS g_sets;					// loaded from settings section of settings file
extern MASETS g_setsbu;					// copy of g_sets, updated when settings are loaded or saved
extern int g_k[kOUTMAX];				// indices into g_sets.fileout
extern int g_droplaunch;
extern int g_loadedquit;
extern mauint64 g_posend;
extern int g_posbegon;
extern int g_posendon;
extern mauint64 g_posbeg;
extern int g_reptsong;
extern fschar g_playlistcda[];			// full path and name of the cda songs list file in playlists dir
extern fschar g_playlistdrop[];			// full path and name of the dropped songs list file in playlists dir
#if !defined(NODECODEAUDIOIN)
 extern fschar g_playlistaudioin[];		// full path and name of the audio input list file in playlists dir
#endif
extern fschar g_playlisttemp[];			// full path and name of the temp playlist file in playlists dir
extern machar g_prognametitle[];		// 'MoreAmp', 'MoreAmp2'..., or 'moreamp', 'moreamp2'
extern fschar *g_extstr[];
extern fschar g_playlistsdir[];			// full path and name of dir to contain playlists
extern fschar g_playlistpath[];			// full path and name of a playlist file in playlists dir
extern fschar g_catalogpath[];			// full path and name of a catalog file in 'moreamp' dir
extern machar *g_playlist0;				// playlist in mem from mafileread
extern fschar g_settingsdir[];			// full path and name of dir to contain settings
extern fschar g_recordsdir[];			// full path and name of dir to contain records
extern fschar g_fileoutdir[];			// full path and name of dir to contain output song files
extern fschar g_settingspath[];			// full path and name of a settings file
extern fschar g_settingsdefpath[];		// full path and name of the default settings file
extern fschar g_settingsdrop[];			// full path and name of the madrop settings file
#if defined(MAKEYBOARD)
 extern fschar g_keybindingspath[];		// full path and name of the key bindings file
#endif

extern fschar g_loadedname[][MAX_PATH];	// names of songs loaded e.g. 'Blue Suede Shoes.mp3'
extern fschar g_loadedpath[][MAX_PATH];	// full path and name of songs in g_loadedname
extern int g_loadednum;	// number of songs loaded on vdisk or in mem, 0 <= g_loadednum < SONGSLOADMAX, or 1 in Direct mode
extern int g_loadedcurrent; // 0-based index of song loaded currently playing, 0 <= g_loadedcurrent < g_loadednum

extern FILE *g_tempfilefp;
extern fschar g_tempfile[];
extern char *g_songmemptr[SONGSLOADMAX + 1];
extern char *g_mafifobuf;

extern int g_argc;
#if defined(MAGUIWX) || defined(__MAWIN__)
 extern machar **g_argv; // wchar_t
#else
 extern char **g_argv;
#endif
extern fschar g_madir[]; // full path and name of 'moreamp' dir
extern fschar g_logpath[];
extern fschar *g_extstrout[];
extern machar g_abouttext[];
extern fschar g_readmepath[];
extern size_t g_loadedmemsize;
extern char *g_memhandle;
extern int g_effectschange;
extern char *g_fxoutbuffer;
extern int g_bufready;
extern int g_listselect;
extern int g_songsnum; // number of songs in the playlist
extern void *g_w[];
extern void *g_menubar;
extern int g_thisinstance;
#if defined(MAKEYBOARD)
 extern int g_maOnKeySkip;
#endif
extern fschar g_fseol[];
extern machar g_maeol[];
extern int g_cataloguing;
extern machar g_logcmdclear;
extern maenvchar *g_macharset;
extern maenvchar g_charset[]; // suitable as parameter to iconv_open()
#if defined(MAGUICARBON)
 extern maenvchar *g_macharsetkb;
 extern maenvchar g_charsetkb[]; // suitable as parameter to iconv_open()
#endif

// #define MAPLAYERSEMWAIT(x) maSemaphoreWait(x)
// #define MAPLAYERSEMRELEASE(x) maSemaphorePost(x)

#define MAFIFOSEMWAIT(x) maSemaphoreWait(x)
#define MAFIFOSEMRELEASE(x) maSemaphorePost(x)

#if defined(__MAWIN__)
 #define MAFIFOLOCKWAIT(x) EnterCriticalSection(&x)
 #define MAFIFOLOCKRELEASE(x) LeaveCriticalSection(&x)
#else
 #define MAFIFOLOCKWAIT(x) maMutexLock(&x)
 #define MAFIFOLOCKRELEASE(x) maMutexUnlock(&x)
#endif

unsigned long maFifoGetFilledSpace(maFifo* pfifo);
unsigned long maFifoDraw(maFifo* pfifo, char* whereTo, unsigned long requested);
unsigned long maFifoStore(maFifo* pfifo, char* whereFrom, unsigned long requested);
unsigned long maFifoGetFreeSpace(maFifo* pfifo);
int maIOStart();
void maIOEnd();
void maWaitForFifosEmpty(int max);
void maOutChainFlush();
int maOutChain1(char *buf, unsigned long len);
void DumpString(FILE *fp, machar *p, machar **p2, machar **p3, size_t *len3);
int MadDoParamChange();
void DumpTagsID3(MASTATE *pstate, machar **p2, machar **p3, size_t *len3);
void UpdateListWindowList(int newcd);
#if defined(MASETLOCALE)
 int masetlocaledata();
#endif
int OpenPlaylist(int skipsettings);
int ChangePlaylist(fschar *playlist);
int DeletePlaylist(fschar *playlist);
int UpdateRecent(fschar *item, int del);
int GetRecent(fschar items[][MAX_PATH]);
void SetAppWindowTitle(machar *msg);
int PlayPlaylist(fschar *playlist);
int PlayPlaylistB(fschar *playlist);
int LoadDefaultSettings(MASETS *psets);
void SetPause(int pause);
void DoMakeList();
void DoOpenList();
void DoDeleteList();
void DoSetVDisk();
void DoMultipleInst(int checked);
#if defined(MAKEYBOARD)
 void DoKeyboard(int checked);
#endif
#if defined(MAMOUSEWHEEL)
 void DoMouseWheel(int checked);
#endif
void DoPlayFile();
void DoOpenRecent(int index);
void DoSetsSaved();
void DoEqWindow();
void DoRandom(int checked);
void DoReptPlayed(int checked);
void DoLoopLoaded(int checked);
void DoLoopList(int checked);
void DoErrLogging(int checked);
void DoSinglePlay(int checked);
void DoInMonitor(int checked);
void DoEqualizer(int checked);
void DoEqExtra(int checked);
void DoEffects(int checked);
void DoMono(int checked);
void DoInvertRight(int checked);
void DoEq31Bands(int checked);
void DoViewCatalog(int silent);
#if defined(MAKEYBOARD)
 void DoEditKeyBindings();
 void DoBlankKeyBindings();
 void DoFactoryKeyBindings();
 void UpdateMenuKbLabels();
 void UpdateButtonLabels();
#endif
void SetGUIPopupButtonsAudioOut();
void SetGUIPopupButtonsAudioIn();
void DoRescanAudioDevices();
void DoAltSize(int checked, int kbcommand);
void DoFactorySize();
void DoSavedSize();
void DoDefaultSize();

signed long RunLoop();
void UpdateMenus();
void UpdateDisplay();
#if defined(__MAOSX__) || defined(MACDAOSX)
 void InitAppleEvents(void *app);
#endif
int OnInit_();
int ClearInitLock();
void DoTitleInfo(int menu);
void DoHostInEnable(int checked);
void DoInputMonitor(int checked); 
void DoListWindow(int checked); 
void DoSeparateLR(int checked);
void DoReptSong(int checked); 
void DoBalance(int checked);
void DoMonoLeft(int checked);
void DoMonoRight(int checked);
void DoMonoDiff(int checked);
void UpdateMonoDiff();
void DoLimiter(int checked);
void DoEqZoom(int checked);
void DoEqLink(int checked);
void DoPitch(int checked);
void DoTempo(int checked);
void DoSpeed(int checked);
void DoSBSMS(int checked);
#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
 void DoCdParanoia(int checked);
 int LoadSettingsCDRom();
#endif
#if defined(MAKEYBOARD)
 int LoadKeyBindings(int showerr);
#endif
void DoBoxBeg(int checked);
void DoBoxEnd(int checked);
#if defined(ENCODEMP4)
 void DoBoxM4A(int checked);
#endif
#if defined(ENCODEAAC)
 void DoBoxADTS(int checked);
#endif
void DoCloseReopen(int checked);
void DoLogNotPlayable(int checked);
void DoUseM3u(int checked);
void DoExitAfter(int checked);
void DoAppQuit();
void DoStop();
void DoPause();
void DoPrevious();
void DoPlay();
void DoNext();
void DoPlayCD();
void DoMakeCatalog();
void DoSave();
void DoEjectCD();
void DoAudioIn();
void DoSaveListAs();
void OnButtonSong();
void OnButtonList();
void OnButtonInfo();
void DoZeroList();
#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
 void OnButtonAudioCDSet();
 void OnButtonAudioCDClear();
 void OnButtonAudioCDSave();
#endif
void DoAsDefaults();
void DoSetsFactory();
void DoSetsCopy();
void DoSetsDefault();
void OnButtonEqPlus();
void OnButtonEqMinus();
void OnButtonEqCopy();
void OnButtonEqPaste();
void OnButtonEqRestore();
void OnButtonEqShiftL();
void OnButtonEqShiftR();
void OnButtonEqInvert();
void OnButtonEqMid();
void OnButtonEq2Copy();
void OnButtonEq2Paste();
void OnButtonEq2Restore();
void OnButtonEq2Mid();
void OnButtonLoaded();
void OnButtonFileOutDir();
void DoPromptIndexOk();
int maOnKey(fschar *fskey);
void OnSliderPosition(int value);
void OnSliderVolumeR(int value);
void OnSliderVolumeL(int value);
void OnSliderSeparation(int value);
void OnSliderPreamp(int value);
void OnSliderEq(int slider, int value, int setpos);
void OnSliderEq2(int slider, int value, int setpos);
void OnEditTextEq();
void OnSliderPitch(int value);
void OnSliderPitch2(int value);
void OnSliderTempo(int value);
void OnSliderSpeed(int value);
void OnPopupMem();
void OnPopupVDisk();
void OnPopupDirect();
void OnPopupOutput(int output);
void OnPopupBuffer(int buffer);
void OnPopupAudioOutput(int value);
void OnPopupAudioHostOut(int value);
void OnPopupAudioInput(int value);
void OnPopupAudioHostIn(int value);
void OnPopupEqPreset(int preset);
void OnPopupBitRate(int bitrate);
void OnPopupQuality(int quality);
void OnPopupMode(int mode);
void OnPopupSampleRate(int samplerate);

#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
 void OnPopupAudioCD(int preset);
#endif
void OnLittleArrowBuffer(int value);
void OnLittleArrowVolumeL(int value);
void OnLittleArrowVolumeR(int value);
void OnLittleArrowSeparation(int value);
void OnLittleArrowPreamp(int value);
void OnLittleArrowBegin(int value);
void OnLittleArrowEnd(int value);
void OnLittleArrowEqOffset(int value);
void OnLittleArrowEq(int band, int value);
void OnLittleArrowEq2Offset(int value);
void OnLittleArrowEq2(int band, int value);
void OnLittleArrowPitch(int value);
void OnLittleArrowPitch2(int value);
void OnLittleArrowTempo(int value);
void OnLittleArrowSpeed(int value);

// void DoWindowSplash(int show);
void DoWindow(int windowid, int show);
void ToggleWindow(int windowid);

void DoWindowMoved(int windowid);
void DoWindowResized(int windowid);

void DoShowIndex(int checked);
void DoShowDir(int checked);
void DoShowPath(int checked);
void DoAddMode(int checked);

void OnListWindowSingleClick(int row);
void OnListWindowDoubleClick();

int InstanceClearAll();
int InstanceListAll();
int InstanceClearThis();
#if defined(MAUSESHM)
 extern void *g_pmashm;
#endif

void maLog(const machar *str);
void DoAppWindowDrop();
void AppWindowMakeA();
int SaveSettings();
int SaveRecords();
void ChangeVolume(int value);
int volumevalL();
int volumevalR();
void OnKeyButtonPrevious();
void OnKeyButtonNext();
void ResetAppTitle();
// int WindowIdFromRef(void *ref);
int AnyEffectOn();

#ifdef __cplusplus
}
#endif

#endif 	// __MAPLAYER_H__

//eof
