// maSongList.c

/*
 * MoreAmp
 * Copyright (C) 2004-2014 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include "maConfig.h" // included by all moreamp files

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>

#ifdef __MAOSX__
 #include <Files.h>
 #include <sys/stat.h>
 #include <dirent.h>
#endif

#ifdef __MAUNIX__
 #include <unistd.h>
 #include <sys/stat.h>
 #include <sys/param.h>
 #include <dirent.h>
#endif

#if defined(__MAOSX__)
 #include <AEDataModel.h>
 #include <AppleEvents.h>
 #include <CarbonEvents.h>
#endif

#ifdef __MAWIN__
 #include <windows.h>
#endif

// Application includes
#include "maTypes.h"
#include "maDefs.h"
#include "maGUI.h"
#include "maPlayer.h"
#include "maUtils.h"
#include "maSongList.h"
#include "maMemFile.h"

#ifndef MANOCDA
 #include "maCda.h"
 #include "maPlayCda.h"
#endif

#ifdef __cplusplus
 extern "C"
 {
#endif

int maPlaylistSkipItem(fschar *item);
int maPlaylistAddItems(FILE *fp, int nitems, fschar **pitems, int *pntracks, int *pnfiles, int *pnlists, int *pnslfiles, fschar *singlelist);
int maPlaylistResolveItem(FILE *fp, fschar *p, int *pntracks, int *pnfiles, int *pnlists, int *pnslfiles, fschar *singlelist);
int maPlaylistResolveFolder(FILE *fp, fschar *dirname, int *pntracks, int *pnfiles, int *pnlists, int *pnslfiles, fschar *singlelist);
int maPlaylistResolveFile(FILE *fp, fschar *outfname, int *pntracks, int *pnfiles, int *pnlists, int *pnslfiles, fschar *singlelist);
int maPlaylistAddFromPlaylist(FILE *fp, fschar *infname, int *pnslfiles);
int maPlaylistAddFromPlaylistm3u(FILE *fp, fschar *infname, int *pnfiles, int *pnslfiles);
// These functions are used to resolve songfile paths from items (songfiles, links,
// dirs, cd, playlists), passed as command line arguments or dragged-and-dropped,
// into a new playlist.
//
// A cd generates a list of tracks.
//
// If passed items resolve to a single file and that file is a playlist then it becomes the
// 'new' playlist, otherwise passed playlists' contents are added to the new playlist.
//
// args to these functions :
// 
// fp        : FILE* to the new playlist being written, e.g. g_playlisttemp
// nitems	 : the number of items passed or dropped
// pitems	 : an array of pointers to the items passed or dropped
// pntracks  : int* for the number of cd tracks resolved
// pnfiles   : int* for the number of songfiles resolved (not songfiles from playlists)
// pnlists   : int* for the number of playlists resolved
// pnslfiles : int* for the number of songfiles from playlists
// singlelist: full path of the playlist that is a single resolved file
//


int l_onlym3uindir = 1; // process only m3u files in a dir
fschar g_changeplaylist[MAX_PATH];
fschar g_newplaylist[MAX_PATH];

int maPlaylistAddItems(FILE *fp, int nitems, fschar **pitems, int *pntracks, int *pnfiles, int *pnlists, int *pnslfiles, fschar *singlelist)
{
	int i = 0;
	int n = nitems;
	fschar **p = pitems;
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];
	
	while(n-- > 0 && p && *p)
	{
		if(maPlaylistResolveItem(fp, *p, pntracks, pnfiles, pnlists, pnslfiles, singlelist))
		{
			masprintf(mastr, MAXLOGLINELEN, L"maPlaylistAddItems maPlaylistResolveItem failed on '%ls'", fs2ma(mastr2, *p));
			maLog(mastr);
		}
		else
			i++;
		p++;
	}
	
	return (i == 0) ? 1 : 0;
	
} // maPlaylistAddItems


#if defined(__MAOSX__)
 fschar l_rootvolumepath[MAX_PATH] = {fsT('\0')};


 int maPlaylistSkipItem(fschar *item)
 {
	if(fsstrstr(item, fsT(".Spotlight")) || fsstrstr(item,  fsT(".Trashes")) || fsstrstr(item,  fsT(".fseventsd"))) // system files/dirs
		return 1;
		
	return 0;
	
 } // maPlaylistSkipItem __MAOSX__


 int maPlaylistResolveItem(FILE *fp, fschar *p, int *pntracks, int *pnfiles, int *pnlists, int *pnslfiles, fschar *singlelist)
 {
	OSStatus err;
	FSRef theRef;
	Boolean isDirectory;
	Boolean targetIsFolder;
	Boolean wasAliased;
	fschar name[MAX_PATH];
	fschar bsdfname[MAX_PATH + MAX_PATH];
	fschar outfname[MAX_PATH];
	machar mastr2[MAXLOGLINELEN];
	machar mastr[MAXLOGLINELEN];
	
	if(g_scanstop)
		return 1;
		
	if(!p)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveItem null parameter p");
		maLog(mastr);
		return 1;
	}

	if(fsstrlen(p) >= MAX_PATH)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveItem file path > max %d : %zu", MAX_PATH - 1, fsstrlen(p));
		maLog(mastr);
		if(fsstrlen(p) < MAXLOGLINELEN)
			maLog(fs2ma(mastr2, p));
		else
		{
			fsstrncpy(name, p, MAX_PATH - 1);
			name[MAX_PATH - 1] = fsT('\0');
			maLog(fs2ma(mastr2, name));
			fsstrncpy(name, p + MAX_PATH, MAX_PATH - 1);
			name[MAX_PATH - 1] = fsT('\0');
			maLog(fs2ma(mastr2, name));
		}
		return 1;
	}

 #if !defined(MANOCDA)
	int drive;
	int i;
	if((drive = IsAudioCd(p)))
	{
		for(i = 1; i <= g_cdadrive[drive].tracklast; i++)
		{
			if(fsfprintf(fp, fsS fsC fsT("Track%02d") fsS fsT("\n"), p, FILESEP, i, EXTCDA) <= 0)
			{
				masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveItem fsfprintf errno=%d", errno);
				maLog(mastr);
				return 1;
			}
			(*pntracks)++;
		}
		return 0;
	}
 #endif

	fsstrcpy(bsdfname, p);
	
	// all our paths need to start from "/Volumes/VolumeName/"
	if(l_rootvolumepath[0] == fsT('\0'))
	{
		if(maGetRootVolumeName(name))
		{
			fsstrcpy(l_rootvolumepath,  fsT("/Volumes/Macintosh HD"));
			masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveItem maGetRootVolumeName failed, using '%ls'", fs2ma(mastr2, l_rootvolumepath));
			maLog(mastr);
		}
		else
			fssprintf(l_rootvolumepath, MAX_PATH, fsT("/Volumes/") fsS, name);
	}
	
	if(fsstrncmp(bsdfname, fsT("/Volumes/"), 9))
	{
		fssprintf(name, MAX_PATH, fsS fsS, l_rootvolumepath, bsdfname);
		fsstrcpy(bsdfname, name);
	}

	err = FSPathMakeRef((const UInt8 *)bsdfname, &theRef, &isDirectory);
	if(err)
	{
		/* if(err == fnfErr) // -43
			masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveItem File not found '%ls'", bsdfname);
		else */
		if(err != fnfErr)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveItem FSPathMakeRef err=%d '%ls'", (int)err, fs2ma(mastr2, bsdfname));
			maLog(mastr);
		}
		return 1;
	}
	
	if((err = FSResolveAliasFile(&theRef, true, &targetIsFolder, &wasAliased)))
	{
		if(err != mapReadErr && err != fnfErr) // -199 'map inconsistent with operation'
		{
			masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveItem ResolveAliasFile err=%d '%ls'", (int)err, fs2ma(mastr2, p));
			maLog(mastr);
		}
		return 1;
	}
	
	if(wasAliased) // get the target name
	{
		err = FSRefMakePath(&theRef, (UInt8 *)bsdfname, sizeof(bsdfname));
		if(err)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveItem FSRefMakePath err=%d '%ls'", (int)err, fs2ma(mastr2, bsdfname));
			maLog(mastr);
			return 1;
		}
		
		if(fsstrlen(bsdfname) >= MAX_PATH)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveItem filepath > max %d : %zu", MAX_PATH - 1, fsstrlen(bsdfname));
			maLog(mastr);
			fsstrncpy(name, bsdfname, MAX_PATH - 1);
			name[MAX_PATH - 1] = fsT('\0');
			maLog(fs2ma(mastr2, name));
			return 1;
		}		
		
		if(fsstrncmp(bsdfname, fsT("/Volumes/"), fsstrlen(fsT("/Volumes/"))))
			fssprintf(outfname, MAX_PATH, fsS fsS, l_rootvolumepath, bsdfname);
		else
			fsstrcpy(outfname, bsdfname);
		
 #if !defined(MANOCDA)
		if((drive = IsAudioCd(outfname)))
		{
			for(i = 1; i <= g_cdadrive[drive].tracklast; i++)
			{
				if(fsfprintf(fp, fsS fsC fsT("Track%02d") fsS fsT("\n"), outfname, FILESEP, i, EXTCDA) <= 0)
				{
					masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveItem fprintf errno=%d", errno);
					maLog(mastr);
					return 1;
				}
				(*pntracks)++;
			}
			return 0;
		}
 #endif
	}
	else
		fsstrcpy(outfname, bsdfname);

	if(targetIsFolder)
	{
		if(maPlaylistResolveFolder(fp, outfname, pntracks, pnfiles, pnlists, pnslfiles, singlelist))
		{
			masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveItem maPlaylistResolveFolder failed on '%ls'", fs2ma(mastr2, outfname));
			maLog(mastr);
			return 1;
		}
		return 0;
	}
	
	// not a mac audio cd, not an alias and not a folder, must be a file
	return maPlaylistResolveFile(fp, outfname, pntracks, pnfiles, pnlists, pnslfiles, singlelist);

  } // maPlaylistResolveItem // __MAOSX__
 
 
 int maPlaylistResolveFolder(FILE *fp, fschar *dirname, int *pntracks, int *pnfiles, int *pnlists, int *pnslfiles, fschar *singlelist)
 {
	fschar *p;
	int n;
	fschar ldirname[STRLEN256];
	DIR *dir;
	struct dirent *dt;
	fschar file[MAX_PATH];
	machar mastr2[MAXLOGLINELEN];
	machar mastr[MAXLOGLINELEN];
	
	if(g_scanstop)
		return 1;

	fsstrcpy(ldirname, dirname);

	n = *pnfiles;

	if(g_sets.usem3u && l_onlym3uindir) // scan for and use only m3u
	{
		if((dir = opendir((char *)ldirname)) == NULL)
		{
			if(!maPlaylistSkipItem(ldirname))
			{
				masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveFolder 1 opendir failed on '%ls'", fs2ma(mastr2, ldirname));
				maLog(mastr);
			}
			return 0;
		}
		p = NULL;
		while((dt = readdir(dir)))
		{
			if(FileNameHasExtension(dt->d_name, EXTM3U))
			{
				fssprintf(file, MAX_PATH, fsS fsC fsS, ldirname, FILESEP, dt->d_name);
				p = file;
				maPlaylistResolveItem(fp, p, pntracks, pnfiles, pnlists, pnslfiles, singlelist);
				// break;
			}
		}
		if(closedir(dir) == -1)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveFolder closedir errno=%d on '%ls'", errno, fs2ma(mastr2, ldirname));
			maLog(mastr);
		}
		if(p)
			return 0;
	}
	
	if((dir = opendir((char *)ldirname)) == NULL)
	{
		if(!maPlaylistSkipItem(ldirname))
		{
			masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveFolder 2 opendir failed on '%ls'", fs2ma(mastr2, ldirname));
			maLog(mastr);
		}
		return 0;
	}

	while((dt = readdir(dir)))
	{
 		if(	fsstrcmp(dt->d_name, fsT(".")) &&
			fsstrcmp(dt->d_name, fsT( "..")) &&
			fsstrcmp(dt->d_name, fsT(".DS_Store")) &&
			fsstrcmp(dt->d_name, fsT("Icon\015")) &&
			fsstrcmp(dt->d_name, fsT("Icon")))
		{
			if(g_sets.usem3u || !FileNameHasExtension(dt->d_name, EXTM3U))
			{
				fssprintf(file, MAX_PATH, fsS fsC fsS, ldirname, FILESEP, dt->d_name);
				p = file;
				maPlaylistResolveItem(fp, p, pntracks, pnfiles, pnlists, pnslfiles, singlelist);
			}
		}
	}
	n = *pnfiles - n;
	
	if(closedir(dir) == -1)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveFolder closedir errno=%d on '%ls'", errno, fs2ma(mastr2, ldirname));
		maLog(mastr);
	}

 #if !defined(MANOCDA)
	size_t i;
	if(n <= 0) // 'empty' dir is possibly a cd where the os did not enumerate the tracks
	{
		int drive;
		if((drive = IsAudioCd(dirname)))
		{
			for(i = 1; i <= (size_t)g_cdadrive[drive].tracklast; i++)
			{
				if(fsfprintf(fp, fsS fsC fsT("Track%02d") fsS fsT("\n"), dirname, FILESEP, (int)i, EXTCDA) <= 0)
				{
					masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveFolder fsfprintf errno=%d'", errno);
					maLog(mastr);
					return 1;
				}
				(*pntracks)++;
			}
		}
	}
 #endif
			
	return 0;

 } // maPlaylistResolveFolder __MAOSX__
#endif // __MAOSX__


#if defined(__MAUNIX__)
 int maPlaylistSkipItem(fschar *item)
 {
	// if(fsstrstr(item, ".Spotlight") || fsstrstr(item, ".Trashes") || fsstrstr(item, ".fseventsd")) // system files/dirs
	//	return 1;
		
	return 0;
	
 } // maPlaylistSkipItem __MAUNIX__


 int maPlaylistResolveItem(FILE *fp, fschar *p, int *pntracks, int *pnfiles, int *pnlists, int *pnslfiles, fschar *singlelist)
 {
	struct stat finfo;
	fschar linkpath[PATH_MAX];
	machar mastr2[MAXLOGLINELEN];
	machar mastr[MAXLOGLINELEN];

	if(g_scanstop)
		return 1;
		
 #if !defined(MANOCDA)
	int drive;
	int i;
	if((drive = IsAudioCd(p)))
	{
		for(i = 1; i <= g_cdadrive[drive].tracklast; i++)
		{
			if(fsfprintf(fp, fsS fsT("/Track%02d") fsS fsT("\n"), g_cdadrive[drive].name, i, EXTCDA) <= 0)
			{
				masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveItem fsfprintf errno=%d", errno);
				maLog(mastr);
				return 1;
			}
			(*pntracks)++;
		}
		return 0;
	}
 #endif

	if(lstat(p, &finfo))
	{
		/* if(errno == ENOENT) // 2
			masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveItem File not found '%ls'", fs2ma(mastr2, p));
		else */
		if(errno != ENOENT)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveItem lstat failed on '%ls' errno=%d", fs2ma(mastr2, p), errno);
			maLog(mastr);
		}
		return 1;
	}

	switch(finfo.st_mode & S_IFMT)
	{
		case S_IFREG : // regular file
		return maPlaylistResolveFile(fp, p, pntracks, pnfiles, pnlists, pnslfiles, singlelist);
			
		case S_IFDIR : // directory
			if(maPlaylistResolveFolder(fp, p, pntracks, pnfiles, pnlists, pnslfiles, singlelist))
			{
				masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveItem maPlaylistResolveFolder failed on '%ls'", fs2ma(mastr2, p));
				maLog(mastr);
				return 1;
			}
		return 0;
		
		case S_IFLNK : // symbolic link
			if(realpath((const fschar *)p, linkpath) == NULL)
			{
				masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveItem realpath failed on '%ls' errno=%d", fs2ma(mastr2, p), errno);
				maLog(mastr);
				return 1;
			}
		return maPlaylistResolveItem(fp, linkpath, pntracks, pnfiles, pnlists, pnslfiles, singlelist);

		default:
		break;
	}
		
	return -1; // other

 } // maPlaylistResolveItem // __MAUNIX__
 
 
 int maPlaylistResolveFolder(FILE *fp, fschar *dirname, int *pntracks, int *pnfiles, int *pnlists, int *pnslfiles, fschar *singlelist)
 {
	fschar *p;
	int n;
	fschar *ldirname = dirname;	
	DIR *dir;
	struct dirent *dt;
	fschar file[MAX_PATH];
	machar mastr2[MAXLOGLINELEN];
	machar mastr[MAXLOGLINELEN];

	if(g_scanstop)
		return 1;

	n = *pnfiles;

	if(g_sets.usem3u && l_onlym3uindir) // scan for and use only m3u
	{
		if((dir = opendir(ldirname)) == NULL)
		{
			if(!maPlaylistSkipItem(ldirname))
			{
				masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveFolder 3 opendir failed on '%ls'", fs2ma(mastr2, ldirname));
				maLog(mastr);
			}
			return 0;
		}
		p = NULL;
		while((dt = readdir(dir)))
		{
			if(FileNameHasExtension(dt->d_name, EXTM3U))
			{
				fssprintf(file, MAX_PATH, fsS fsC fsS, ldirname, FILESEP, dt->d_name);
				p = file;
				maPlaylistResolveItem(fp, p, pntracks, pnfiles, pnlists, pnslfiles, singlelist);
				// break;
			}
		}
		if(closedir(dir) == -1)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveFolder closedir errno=%d on '%ls'", errno, fs2ma(mastr2, ldirname));
			maLog(mastr);
		}
		if(p)
			return 0;
	}

	if((dir = opendir((char *)ldirname)) == NULL)
	{
		if(!maPlaylistSkipItem(ldirname))
		{
			masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveFolder 4 opendir failed on '%ls'", fs2ma(mastr2, ldirname));
			maLog(mastr);
		}
		return 0;
	}
	
	while((dt = readdir(dir)))
	{
 		if(fsstrcmp(dt->d_name, ".") && fsstrcmp(dt->d_name, ".."))
		{
			if(g_sets.usem3u || !FileNameHasExtension(dt->d_name, EXTM3U))
			{
				fssprintf(file, MAX_PATH, fsS fsC fsS, ldirname, FILESEP, dt->d_name);
				p = file;
				maPlaylistResolveItem(fp, p, pntracks, pnfiles, pnlists, pnslfiles, singlelist);
			}
		}
	}
	n = *pnfiles - n;
	
	if(closedir(dir) == -1)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveFolder closedir errno=%d on '%ls'", errno, fs2ma(mastr2, ldirname));
		maLog(mastr);
	}
			
	return 0;

 } // maPlaylistResolveFolder // __MAUNIX__
#endif


#ifdef __MAWIN__
 #define EXTWINLNK fsT(".lnk")
 int maPlaylistResolveItem(FILE *fp, fschar *p, int *pntracks, int *pnfiles, int *pnlists, int *pnslfiles, fschar *singlelist) // __MAWIN__
 {
	fschar outfname[MAX_PATH];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];

	int sz = maGetLongPathName(p, outfname, MAX_PATH);
	if(!sz)
		fsstrcpy(outfname, p);

	if(g_scanstop)
		return 1;
		
	if(FileNameHasExtension(outfname, EXTWINLNK)) // e.g ...tune.mp3.lnk
	{
		if(maWinResolveShortcut(outfname))	
		{
			/* masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveItem maWinResolveShortcut failed on '%ls'", fs2ma(mastr2, outfname));
			maLog(mastr); */
			return 1;
		}
	}

	if(maDirExists(outfname))
	{
		if(maPlaylistResolveFolder(fp, outfname, pntracks, pnfiles, pnlists, pnslfiles, singlelist))
		{
			masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveItem maPlaylistResolveFolder failed on '%ls'", fs2ma(mastr2, outfname));
			maLog(mastr);
			return 1;
		}
		return 0;
	}
	
	// maybe an audio cd e.g. "D:/", in case maDirExists did not catch a cd
	if(fsstrlen(outfname) == 3 && IsAudioCd(outfname))
	{
		if(maPlaylistResolveFolder(fp, outfname, pntracks, pnfiles, pnlists, pnslfiles, singlelist))
		{
			masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveItem maPlaylistResolveFolder failed on '%ls'", fs2ma(mastr2, outfname));
			maLog(mastr);
			return 1;
		}
		return 0;
	}

	// not a link and not a folder and not an audio cd, must be a file
	return maPlaylistResolveFile(fp, outfname, pntracks, pnfiles, pnlists, pnslfiles, singlelist);

 } // maPlaylistResolveItem // __MAWIN__


 int maPlaylistResolveFolder(FILE *fp, fschar *dirname, int *pntracks, int *pnfiles, int *pnlists, int *pnslfiles, fschar *singlelist)
 {
	HANDLE hFind;
	WIN32_FIND_DATA FindData;
	fschar dirwild[MAX_PATH];
	fschar file[MAX_PATH];
 #if !defined(MANOCDA)
	int i;
	int drive;
 #endif
	
	if(g_scanstop)
		return 1;

 #if !defined(MANOCDA)
	if(fsstrlen(dirname) == 3 && dirname[1] == fsT(':') && dirname[2] == fsT('\\')) // e.g. 'D:\"
	{
		fschar fname[MAX_PATH];
		machar mastr2[MAXLOGLINELEN];
		machar mastr[MAXLOGLINELEN];
		
		fssprintf(fname, MAX_PATH, fsS fsT("Track01") fsS, dirname, EXTCDA);
		if((drive = IsAudioCd(fname)) != 0)
		{
			for(i = 1; i <= g_cdadrive[drive].tracklast; i++)
			{
				masprintf(mastr2, MAX_PATH, fsS fsT("Track%02d") fsS fsT("\n"), dirname, i, EXTCDA);
				if(mafilewrite(0, mastr2, fp) == 0)
				{
					masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveFolder fsfprintf errno=%d'", errno);
					maLog(mastr);
					return 1;
				}
				(*pntracks)++;
			}
			return 0;
		}
	}
 #endif

	fssprintf(dirwild, MAX_PATH, fsS fsC fsT("*"), dirname, FILESEP);

	if(g_sets.usem3u && l_onlym3uindir) // scan for and use only m3u
	{
		hFind = FindFirstFile((const fschar *)dirwild, &FindData);
		if(hFind == INVALID_HANDLE_VALUE)
			return 1;
		file[0] = 0;
		for (; ;)
		{
			if(FileNameHasExtension(FindData.cFileName, EXTM3U))
			{
				fssprintf(file, MAX_PATH, fsS fsC fsS, dirname, FILESEP, FindData.cFileName);
				maPlaylistResolveItem(fp, file, pntracks, pnfiles, pnlists, pnslfiles, singlelist);
				//FindClose(hFind);
				// return 0;
			}
			if(!FindNextFile(hFind, &FindData))
				break;
		}
		FindClose(hFind);
		if(file[0])
			return 0;
	}
	
	hFind = FindFirstFile(dirwild, &FindData);
	if(hFind == INVALID_HANDLE_VALUE)
		return 1;
	for (; ;)
	{
		if(fsstrcmp(FindData.cFileName, fsT(".")) && fsstrcmp(FindData.cFileName, fsT("..")))
		{
			if(g_sets.usem3u || !FileNameHasExtension(FindData.cFileName, EXTM3U))
			{
				fssprintf(file, MAX_PATH, fsS fsC fsS, dirname, FILESEP, FindData.cFileName);
				maPlaylistResolveItem(fp, file, pntracks, pnfiles, pnlists, pnslfiles, singlelist);
			}
		}
		if(!FindNextFile(hFind, &FindData))
			break;
	}
	FindClose(hFind);

	return 0;

 } // maPlaylistResolveFolder __MAWIN__
 #endif
		

int maPlaylistResolveFile(FILE *fp, fschar *outfname, int *pntracks, int *pnfiles, int *pnlists, int *pnslfiles, fschar *singlelist)
{
#if defined(__MAOSX__) || defined(__MACUNIX__)
	fschar fsstr[MAX_PATH];
#endif	
	machar mastr[MAXLOGLINELEN];
	machar mastr2[MAXLOGLINELEN];

	if(g_scanstop)
		return 1;

	if((++g_scanfiles % 500) == 0) 
	{
		masprintf(mastr, MAXLOGLINELEN, L"MoreAmp scanning... %d", g_scanfiles);
		WindowSetTitle(WINDOWAPP, mastr);
		AppYield();
	}
	
	if(FileNameHasExtension(outfname, EXTMAF)) // a moreamp playlist
	{
		if(fsstrcmp(outfname, g_playlistcda) && fsstrcmp(outfname, g_playlistdrop) &&
			fsstrcmp(outfname, g_playlisttemp))
		{
			if((*pnlists)++ == 0)
				fsstrcpy(singlelist, outfname); // one single playlist resolved to this
				
			if(maPlaylistAddFromPlaylist(fp, outfname, pnslfiles))
			{
				masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveItem maPlaylistAddFromPlaylist failed on '%ls'", fs2ma(mastr2, outfname));
				maLog(mastr);
				return 1;
			}
		}
	}
	else if(FileNameHasExtension(outfname, EXTM3U)) // a standard m3u playlist (not an Extended M3u playlist)
	{
		if(maPlaylistAddFromPlaylistm3u(fp, outfname, pnfiles, pnslfiles))
		{
			masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveItem maPlaylistAddFromPlaylistm3u failed on '%ls'", fs2ma(mastr2, outfname));
			maLog(mastr);
			return 1;
		}
	}
	else if(FileIsPlayable(outfname)) // a song file or audio cd track
	{
#if !defined(MANOCDA)
 #if defined(__MAWIN__) // a cd is a dir with tracks as files
		if(IsAudioCd(outfname))
			(*pntracks)++;
		else
 #endif
#endif
		(*pnfiles)++;

#if defined(__MAOSX__) || defined(__MACUNIX__)
		// on mac outfname is	1. from DoArgsOpenDocOSX UTF-8-MAC
		// outfname was made by FSRefMakePath, UTF-8 decomposed e.g. ê is 65 cc 82
		//						2. from PlaylistMake idem
		//						3. from PlaySongFiles idem (with fn_str().data() on wx)
		fsstrcpy(fsstr, outfname);
		StrUTF8MACtoUTF8(fsstr); // now UTF-8 precomposed e.g. ê is now 0xc3aa
		if(fsfprintf(fp, fsS fsS, fsstr, g_fseol) <= 0)
#elif defined(__MAWIN__)
		masprintf(mastr2, MAXLOGLINELEN, L"%ls%ls", (machar *)outfname, g_maeol);
		if(mafilewrite(0, mastr2, fp) == 0) // on win unicode outfname is wchar_t local encoded
#else
		if(fsfprintf(fp, fsS fsS, outfname, g_fseol) <= 0) // on non-mac unix outfname is already utf8
#endif
		{
			masprintf(mastr, MAXLOGLINELEN, L"maPlaylistResolveItem fsfprintf errno=%d", errno);
			maLog(mastr);
			return 1;
		}
	}

	return 0;

} // maPlaylistResolveFile


int maPlaylistAddFromPlaylist(FILE *fp, fschar *infname, int *pnslfiles)
{
	size_t ret0;
	machar *p0;
	machar *p4;
	machar *p5;
	fschar item[MAX_PATH];
	machar maitem[MAX_PATH];
 	machar mastr[MAXLOGLINELEN];
	
	if(g_scanstop)
		return 1;

	ret0 = mafileread(&p0, infname);
	if(ret0 <= 0 || p0 == NULL)
	{			
		if(p0 != NULL)
			MAFREE(p0);
		maLog(L"maPlaylistAddFromPlaylist mafileread failed");
		return 1;
	}

	p4 = p0;
	while((p5 = memgets(maitem, MAX_PATH, p4)) != NULL)
	{
		ma2fsfn(item, maitem);
		p4 = p5;
		if(g_scanstop)
		{
			MAFREE(p0);
			return 1;
		}
		
		// skip lines with '#' or files that are not playable or don't exist
		if(item[0] != fsT('#'))
		{
			if(FileIsPlayable(item)) // && SongFileExists(item))
			{
				(*pnslfiles)++;
						
				if(fsfprintf(fp, fsS fsS, item, g_fseol) <= 0)
				{
					masprintf(mastr, MAXLOGLINELEN, L"maPlaylistAddFromPlaylist fsfprintf errno=%d", errno);
					maLog(mastr);
					MAFREE(p0);
					return 1;
				}				
			}
		} // if !#
		
	} // while
			
	MAFREE(p0);
	
	return 0;
	
} // maPlaylistAddFromPlaylist


int maPlaylistAddFromPlaylistm3u(FILE *fp, fschar *infname, int *pnfiles, int *pnslfiles)
{
	size_t ret0;
	machar *p0;
	machar *p4;
	machar *p5;
	fschar *p, *p2;
	int ntracks = 0;
	int nlists = 0;
	fschar item[MAX_PATH];
	machar maitem[MAX_PATH];
	fschar m3udir[MAX_PATH];
	fschar m3uitem[MAX_PATH];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];
	
	if(g_scanstop)
		return 1;

	// the dir holding the m3u file
	fsstrcpy(m3udir, infname);
	p = m3udir;
	p2 = NULL;
	while((p = fsstrchr(p, FILESEP)) != NULL)
		p2 = p++;
	if(p2 == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maPlaylistAddFromPlaylistm3u failed to get m3u dir from '%ls'", fs2ma(mastr2, infname));
		maLog(mastr);
		return 1;
	}
	*p2 = 0;
	
	ret0 = mafileread(&p0, infname);
	if(ret0 <= 0 || p0 == NULL)
	{			
		if(p0 != NULL)
			MAFREE(p0);
		maLog(L"maPlaylistAddFromPlaylistm3u mafileread failed");
		return 1;
	}

	p4 = p0;
	while((p5 = memgets(maitem, MAX_PATH, p4)) != NULL)
	{
		ma2fsfn(item, maitem);
		p4 = p5;
		if(g_scanstop)
		{
			MAFREE(p0);
			return 1;
		}

		// skip lines with '#' or files that are not playable or don't exist
		if(item[0] != fsT('#'))
		{
			if(maIsFullPath(item))
				p = item;
			else
			{
				fssprintf(m3uitem, MAX_PATH, fsS fsC fsS, m3udir, FILESEP, item);
				p = m3uitem;
			}
			
			if(maPlaylistResolveItem(fp, p, &ntracks, pnfiles, &nlists, pnslfiles, fsT("")))
			{
				masprintf(mastr, MAXLOGLINELEN, L"maPlaylistAddFromPlaylistm3u maPlaylistResolveItem failed on '%ls'", p);
				maLog(mastr);
			}
		}
		
	} // while
			
	MAFREE(p0);
	
	return 0;
	
} // maPlaylistAddFromPlaylistm3u


int maPlaylistMake(int nitems, fschar **pitems, fschar *newplaylist)
{
	int iscda = 0;
	int isfile = 0;
	int islist = 0;
	int isrefile = 0;
	int empty = 0;
	int concat = 0;
	FILE *fp;
	machar extmaf[STRLEN64];
	fschar name[MAX_PATH];
	fschar selectedpath[MAX_PATH];
	fschar lnewplaylist[MAX_PATH];
	fschar fsstr[MAX_PATH];
	machar mastr2[MAXLOGLINELEN];
	machar mastr3[MAXLOGLINELEN];
	machar mastr[MAXLOGLINELEN];

	if(g_state.scanning == 1) // == 2 if called from drop event
		return 1;

	g_state.scanning = 1;
	g_scanstop = 0;
	g_scanfiles = 0;

	if(nitems == 1 && FileNameHasExtension(*pitems, EXTMAF)) // one single playlist
	{
		if(fsstrlen(*pitems) >= MAX_PATH)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maPlaylistMake file path > max %d : %zu", MAX_PATH - 1, fsstrlen(*pitems));
			maLog(mastr);
			fsstrncpy(name, *pitems, MAX_PATH - 1);
			name[MAX_PATH - 1] = fsT('\0');
			maLog(fs2ma(mastr2, name));
			goto maplaylistmakeerrret;
		}
		fsstrcpy(newplaylist, *pitems); // no item to resolve or new list to build
		goto maplaylistmakeokret;
	}
	
	// open a file for writing a temp playlist
	if((fp = mafopen(g_playlisttemp, FSWBMODE)) == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maPlaylistMake mafopen failed on '%ls'", g_playlisttemp);
		maLog(mastr);
		goto maplaylistmakeerrret;
	}

#if defined(__MAOSX__) || defined(__MAUNIX__)
	machmod(g_playlisttemp, ACCESSPERMS); // 0777
#endif

	if(maPlaylistAddItems(fp, nitems, pitems, &iscda, &isfile, &islist, &isrefile, lnewplaylist))
	{
		fclose(fp);
		masprintf(mastr, MAXLOGLINELEN, L"maPlaylistMake maPlaylistAddItems failed on '%ls'", pitems[0]);
		maLog(mastr);
		goto maplaylistmakeerrret;
	}

	fclose(fp);
	
	if(g_scanstop)
		goto maplaylistmakeerrret;

	if(!iscda && !isfile && !islist && !isrefile)
	{
#if !defined(NODECODEAUDIOIN)
		if(nitems == 1 && !FileNameHasExtension(*pitems, EXTMAA))
#endif
		{
			if(g_sets.addmode)
				maLog(L"maPlaylistMake no new items for the playlist");
			else
				maLog(L"maPlaylistMake no items for a new playlist");
			goto maplaylistmakeerrret;
		}
	}
	
	if(!iscda && !isfile && islist == 1 && isrefile) // one single playlist resolved to lnewplaylist
	{
		if(newplaylist)
			fsstrcpy(newplaylist, lnewplaylist);
		goto maplaylistmakeokret;
	}
	
	if(newplaylist && newplaylist[0]) // prompt for new playlist name
	{
		masprintf(extmaf, STRLEN64, L"%ls file (%ls)|%ls", g_prognametitle,
					fs2ma(mastr2, g_extstr[kMAMAF]), fs2ma(mastr2, g_extstr[kMAMAF]));
		if(FilenameFromPathname(newplaylist, name))
		{
			masprintf(mastr, MAXLOGLINELEN, L"maPlaylistMake failed to get filename from '%ls'",
						fs2ma(mastr2, newplaylist));
			maLog(mastr);
			newplaylist[0] = fsT('\0');
		}					
		if(maFileDialog(selectedpath, WINDOWAPP,
				L"Enter a new playlist name or select a list to overwrite", g_playlistsdir,
				fs2ma(mastr2, name), // deffile
				extmaf, maSAVE | maOVERWRITE_PROMPT))
			goto maplaylistmakeerrret;
		
		// unless user selected PLAYLISTTMP, check to overwrite
		if(fsstrcasecmp(g_playlisttemp, selectedpath)) 
		{
			// append extension if missing
			if(!FileNameHasExtension(selectedpath, g_extstr[kMAMAF]))
			{
				fsstrcat(selectedpath, g_extstr[kMAMAF]);
	
				// overwrite ? (maOVERWRITE_PROMPT missed the name that user typed in without an extension)
				if(maFileExists(selectedpath))
				{
#if defined(__MAOSX__) || defined(__MACUNIX__)
					strcpy(fsstr, selectedpath);
					StrUTF8MACtoUTF8(fsstr);
#else
					fsstrcpy(fsstr, selectedpath);
#endif
					masprintf(mastr, MAXLOGLINELEN,
						L"This playlist already exists and will be overwritten%ls%ls'%ls'",
						g_maeol, g_maeol, fs2ma(mastr2, fsstr));
					if(maMessageBox(mastr, L"CONTINUE ?", maYES_NO | maICON_QUESTION) != maYES)
						goto maplaylistmakeerrret;
				}
			}
		}
		fsstrcpy(lnewplaylist, selectedpath);
	}
	
#ifndef MANOCDA
	else if(iscda && !isfile)
		fsstrcpy(lnewplaylist, g_playlistcda);
#endif
	else
	{
		fsstrcpy(lnewplaylist, g_playlistdrop);
		if(g_sets.addmode)
			concat = 1; // do not clear current list before adding dropped files
	}
		
	// we are stopped when doing this but lnewplaylist may still be open
	if(!fsstrcasecmp(lnewplaylist, g_playlistpath))
	{		
		if(g_playlist0)
		{
			MAFREE(g_playlist0);
			g_playlist0 = NULL;
		}	
	}
		
	// unless user selected PLAYLISTTMP, rename it
	if(fsstrcasecmp(g_playlisttemp, selectedpath)) 
	{
		// delete lnewplaylist if it already exists
		if(maFileExists(lnewplaylist))
		{
			if((fp = mafopen(lnewplaylist, FSRBMODE)) == NULL)
			{
				masprintf(mastr, MAXLOGLINELEN, L"maPlaylistMake mafopen failed on '%ls'", lnewplaylist);
				maLog(mastr);
			}
			else if(maGetFileSize(fp, 1) == 0)
				empty = 1;
			fclose(fp);

			if(!concat && maunlink(lnewplaylist))
			{
				masprintf(mastr, MAXLOGLINELEN, L"maPlaylistMake maunlink errno=%d '%ls'", errno, lnewplaylist);
				maLog(mastr);
				goto maplaylistmakeerrret;
			}		
		}
		else
		{
			concat = 0; // no file to concat to
			empty = 1;
		}
		
		if(empty || !concat)
		{
			if((fp = mafopen(lnewplaylist, FSWBMODE)) == NULL)
			{
				masprintf(mastr, MAXLOGLINELEN, L"maPlaylistMake mafopen failed on '%ls'", lnewplaylist);
				maLog(mastr);
			}
			else
			{
				// BOM
				fwrite("\xef\xbb\xbf", 3, 1, fp); // BOM_UTF-8

				// header, e.g. "MoreAmp playlist created Sun Dec 19 18:05:56 2010"
				masprintf(mastr2, MAXLOGLINELEN, L"# %ls playlist", MOREAMP_NAME);
				if(maGetLocalDateTimeString(mastr3))
				{
					masprintf(mastr, MAXLOGLINELEN, L" created %ls%ls%ls", mastr3, g_maeol, g_maeol);
					mastrcat(mastr2, mastr);
				}
				else
				{
					mastrcat(mastr2, g_maeol);
					mastrcat(mastr2, g_maeol);
				}
				mafilewrite(0, mastr2, fp);
				fclose(fp);
				concat = 1;
			}
		}

		// copy file playlisttemp to new playlist
		if(maFileCopy(g_playlisttemp, lnewplaylist, concat) != 0)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maPlaylistMake error copying '%ls' to '%ls'", g_playlisttemp, lnewplaylist);
			maLog(mastr);
			goto maplaylistmakeerrret;
		}

		// delete temp playlist
		if(maunlink(g_playlisttemp))
		{
			masprintf(mastr, MAXLOGLINELEN, L"maPlaylistMake maunlink errno=%d '%ls'", errno, g_playlisttemp);
			maLog(mastr);
		}
	}
	
	if(newplaylist)
		fsstrcpy(newplaylist, lnewplaylist);
		
	if(g_scanstop)
		goto maplaylistmakeerrret;

maplaylistmakeokret:
	if(!g_scanstop)
		g_state.scanning = 0;
	return 0;

maplaylistmakeerrret:
	if(!g_scanstop)
		g_state.scanning = 0;
	return 1;
	
} // maPlaylistMake	


int SavePlaylistAs()
{
	int err;
	fschar name[MAX_PATH];
	fschar selectedpath[MAX_PATH];
	fschar newsettingsfile[MAX_PATH];
	fschar newrecordsfile[MAX_PATH];
	machar extmaf[STRLEN64];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];

	// prompt for file name to save to
	masprintf(extmaf, STRLEN64, L"%ls file (%ls)|%ls",
		g_prognametitle, fs2ma(mastr2, g_extstr[kMAMAF]), fs2ma(mastr2, g_extstr[kMAMAF]));
	if(maFileDialog(selectedpath, WINDOWAPP,
			L"Enter a new playlist name or select a list to overwrite", g_playlistsdir, L"",
				extmaf, maSAVE | maOVERWRITE_PROMPT))
		return 1;

	// append extension if missing
	if(!FileNameHasExtension(selectedpath, g_extstr[kMAMAF]))
	{
		fsstrcat(selectedpath, g_extstr[kMAMAF]);
	
		// overwrite ? (maOVERWRITE_PROMPT missed the name that user typed in without an extension)
		if(maFileExists(selectedpath))
		{
			masprintf(mastr, MAXLOGLINELEN, L"This playlist already exists and will be overwritten'%ls%ls%ls'",
				g_maeol, g_maeol, fs2ma(mastr2, selectedpath));
			if(maMessageBox(mastr, L"CONTINUE ?", maYES_NO | maICON_QUESTION) != maYES)
				return 1;
		}
	}
	
	// if the playlist selected is the current playlist, just save settings
	if(!fsstrcasecmp(selectedpath, g_playlistpath)) 
	{
		memcpy(&g_setsbu, &g_sets, sizeof(MASETS));
		SaveSettings();
		SaveRecords();
		return 0;
	}
	
	if(g_playlist0)
	{
		MAFREE(g_playlist0);
		g_playlist0 = NULL;
	}	
	
	// delete file name selected if it already exists
	if(maFileExists(selectedpath))
	{
		if(maunlink(selectedpath))
		{
			masprintf(mastr, MAXLOGLINELEN, L"SavePlaylistAs failed to delete '%ls'", fs2ma(mastr2, selectedpath));
			maLog(mastr);
			return 1;
		}		
	}
		
	// copy current g_playlistpath to file name selected from prompt
	if((err = maFileCopy(g_playlistpath, selectedpath, 0)) != 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"%d SavePlaylistAs copying '%ls' to '%ls'", err, g_playlistpath, fs2ma(mastr2, selectedpath));
		maLog(mastr);
		return 1;
	}

	// copy saved settings file to new settings file
	if(FilenameFromPathname(selectedpath, name))
	{	
		masprintf(mastr, MAXLOGLINELEN, L"SavePlaylistAs failed to get filename from '%ls'", fs2ma(mastr2, selectedpath));
		maLog(mastr);
		return 1;
	}
	FileNameRemoveExtension(name);
	fssprintf(newsettingsfile, MAX_PATH, fsS fsC fsS fsS, g_settingsdir, FILESEP, name, EXTMAS);
	if(maFileExists(newsettingsfile))
		maunlink(newsettingsfile);
	if(maFileExists(g_settingspath))
	{
		if((err = maFileCopy(g_settingspath, newsettingsfile, 0)) != 0)
		{
			masprintf(mastr, MAXLOGLINELEN, L"SavePlaylistAs error %d copying '%ls' to '%ls'", err, g_settingspath, newsettingsfile);
			maLog(mastr);
			return 1;
		}
	}
	
	// copy saved records file to new records file
	if(FilenameFromPathname(selectedpath, name))
	{	
		masprintf(mastr, MAXLOGLINELEN, L"SavePlaylistAs failed to get filename from '%ls'", fs2ma(mastr2, selectedpath));
		maLog(mastr);
		return 1;
	}
	FileNameRemoveExtension(name);
	fssprintf(newrecordsfile, MAX_PATH, fsS fsC fsS fsS, g_recordsdir, FILESEP, name, EXTMAR);
	if(maFileExists(newrecordsfile))
		maunlink(newrecordsfile);
	if(maFileExists(g_recordsdir))
	{
		if((err = maFileCopy(g_recordsdir, newrecordsfile, 0)) != 0)
		{
			masprintf(mastr, MAXLOGLINELEN, L"SavePlaylistAs error %d copying '%ls' to '%ls'", err, g_recordsdir, newrecordsfile);
			maLog(mastr);
			return 1;
		}
	}
	
	// save current active settings to new settings file
	fsstrcpy(name, g_settingspath);
	fsstrcpy(g_settingspath, newsettingsfile);
	memcpy(&g_setsbu, &g_sets, sizeof(MASETS));
	SaveSettings();
	SaveRecords();
	fsstrcpy(g_settingspath, name);
	
	// new playlist becomes current playlist
	if(ChangePlaylist(selectedpath))
		return 1;

	UpdateRecent(g_playlistpath, 0);

	SetAppWindowTitle(L"Stopped");
	
	return 0;

} // SavePlaylistAs


#if defined(__MAOSX__)
 // make a playlist from files, dirs or links passed in an AEvent to the program
 void DoArgsOpenDocOSX(void *inAppleEvent)
 {
	int nitems;
	int arraysize;
	int i;
	int err = 0;
	fschar **p;
	fschar *q;
	fschar **pitems = NULL;
	fschar *items = NULL; // null separated items
	fschar name[MAX_PATH];
	fschar fname[MAX_PATH + MAX_PATH];
	SInt32 numDocs;
	AEDescList docList;
	AEDescList *pdocList = NULL;
	AEKeyword theKey;
	DescType theType;
	FSRef theRef;
	Size theSize;
	machar mastr2[MAXLOGLINELEN];
	machar mastr[MAXLOGLINELEN];

	if(g_state.scanning)
		return;
		
	g_state.scanning = 2;
	g_scanstop = 0;
	g_scanfiles = 0;

	// create the app window
	if(!g_w[WINDOWAPP])
		DoAppWindowDrop();
		
	// get CDRom drives
 #if !defined(MANOCDA)
	CdaInitCDRom(); // DoArgsOpenDocOSX
 #endif

	// build an array of numDocs null spearated items
	pdocList = &docList;
	if(AEGetParamDesc((const AppleEvent *)inAppleEvent, keyDirectObject, typeWildCard, pdocList))
	{
		maLog(L"DoArgsOpenDocOSX AEGetParamDesc failed");
		pdocList = NULL;
		goto errorret;
	}
	
	if(g_scanstop)
		goto errorret;
		
	if(AECountItems(pdocList, &numDocs))
	{
		maLog(L"DoArgsOpenDocOSX AECountItems failed");
		goto errorret;
	}
				
	// get arraysize needed for items
	arraysize = numDocs;
	for(i = 1; i <= numDocs; i++)
	{
		if(g_scanstop)
			goto errorret;
			
		if(AEGetNthPtr(pdocList, i, typeFSRef, &theKey, &theType,
							(Ptr)&theRef, sizeof(FSRef), &theSize) != noErr)
		{
			maLog(L"DoArgsOpenDocOSX AEGetNthPtr failed");
			goto errorret;
		}
		
		if(FSRefMakePath(&theRef, (UInt8 *)fname, sizeof(fname)) != noErr)
		{
			maLog(L"DoArgsOpenDocOSX FSRefMakePath failed");
			goto errorret;
		}

		if(fsstrlen(fname) >= MAX_PATH)
		{
			masprintf(mastr, MAXLOGLINELEN, L"AEGetParamDesc file path > max %d : %zu", MAX_PATH - 1, fsstrlen(fname));
			maLog(mastr);
			fsstrncpy(name, fname, MAX_PATH - 1);
			mastr[MAX_PATH - 1] = fsT('\0');
			maLog(fs2ma(mastr2, name));
			err = 1;
			goto errorret;
		}

		arraysize += fsstrlen(fname);
	} // for i
		
	nitems = numDocs;
	
	// allocate array members
	if((pitems = (fschar **)MAMALLOC(numDocs * sizeof(fschar *))) == NULL)
	{
		maLog(L"DoArgsOpenDocOSX MAMALLOC failed");
		goto errorret;
	}
	
	if((items = (fschar *)MAMALLOC(arraysize)) == NULL)
	{
		maLog(L"DoArgsOpenDocOSX MAMALLOC failed");
		goto errorret;
	}
	memset(items, 0, arraysize);
		
	// fill in array
	p = pitems;
	q = items;
	for(i = 1; i <= numDocs; i++, p++)
	{
		if(g_scanstop)
			goto errorret;

		if(AEGetNthPtr(pdocList, i, typeFSRef, &theKey, &theType,
							(Ptr)&theRef, sizeof(FSRef), &theSize))
		{
			maLog(L"DoArgsOpenDocOSX AEGetNthPtr failed");
			goto errorret;
		}
		
		if(FSRefMakePath(&theRef, (UInt8 *)fname, sizeof(fname)) != noErr)
		{
			maLog(L"DoArgsOpenDocOSX AEGetNthPtr failed");
			goto errorret;
		}

		if(fsstrlen(fname) >= MAX_PATH)
		{
			masprintf(mastr, MAXLOGLINELEN, L"DoArgsOpenDocOSX filepath > max %d : %zu", MAX_PATH - 1, fsstrlen(fname));
			maLog(mastr);
			fsstrncpy(name, fname, MAX_PATH - 1);
			mastr[MAX_PATH - 1] = fsT('\0');
			maLog(fs2ma(mastr2, name));
			goto errorret;
		}		

		*p = q;
		fsstrcpy(q, fname);
		q += fsstrlen(fname) + 1;
	}
		
	if(pdocList)
	{
		AEDisposeDesc(pdocList);
		pdocList = NULL;
	}
		
	// make a new playlist, either madrop or macda
	fname[0] = fsT('\0');
	err = maPlaylistMake(nitems, pitems, fname); // DoArgsOpenDocOSX

errorret:
	if(items)
		MAFREE(items);
	if(pitems)
		MAFREE(pitems);
	if(pdocList)
	{
		AEDisposeDesc(pdocList);
		pdocList = NULL;
	}

	if(!g_scanstop)
		g_state.scanning = 0;
	
	if(!err && !g_scanstop)
		PlayPlaylist(fname);
	else if(!g_w[WINDOWAPP])
	{
		LoadDefaultSettings(&g_sets);
		AppWindowMakeA();
	}
	 
 } // DoArgsOpenDocOSX // __MAOSX__
#endif


 int DoArgsOpenDoc()
 {
	int err = 0;
#if defined(MAGUIWX)
		int i;
		fschar **p;
		fschar **pitems;
#endif
	fschar newplaylist[MAX_PATH];

	if(g_state.scanning)
		return 1;

	g_state.scanning = 2;
	g_scanstop = 0;
	g_scanfiles = 0;

	// create the app window
	if(!g_w[WINDOWAPP])
		DoAppWindowDrop();

	// make a new playlist, either madrop or macda
	newplaylist[0] = fsT('\0');
	if(g_argc > 1)
	{
#if defined(MAGUIWX)
		// with wx g_argv is machar**, we want fschar**
		if((pitems = (fschar **)MAMALLOC((g_argc - 1) * sizeof(fschar *))) == NULL)
			maLog(L"DoArgsOpenDoc MAMALLOC failed");
		else
		{
			p = pitems;
			for(i = 1; i < g_argc; i++, p++)
			{
				if((*p = (fschar *)MAMALLOC(MAX_PATH)) == NULL)
				{
					maLog(L"DoArgsOpenDoc MAMALLOC failed");
					break;
				}
				else
					ma2fsfn(*p, g_argv[i]);
			}
		}
		if(pitems && pitems[0])
			err = maPlaylistMake(g_argc - 1, pitems, newplaylist); // DoArgsOpenDoc
		if(pitems)
		{
			for(i = 0; i < g_argc - 1; i++)
				if(pitems[i])
					MAFREE(pitems[i]);
			MAFREE(pitems);
		}
#else
		err = maPlaylistMake(g_argc - 1, &g_argv[1], newplaylist); // DoArgsOpenDoc
#endif
	}
	if(!g_scanstop)
		g_state.scanning = 0;

	if(!err && !g_scanstop)
		PlayPlaylist(newplaylist);
	else if(!g_w[WINDOWAPP])
	{
		LoadDefaultSettings(&g_sets);
		AppWindowMakeA();
	}			
	else
		ResetAppTitle();
	
	return 0;
	
 } // DoArgsOpenDoc


int DoPassedArgs()
{
#if defined(__MAOSX__) && !defined(MAGUIWX)
	EventTypeSpec eventspec[] = { { kCoreEventClass, kAEOpenDocuments } };
	// EventRecord event; EventAvail deprecated
	// return EventAvail(highLevelEventMask, &event) && event.message == kCoreEventClass &&
	//	*(OSType *)&event.where == kAEOpenDocuments; // DoArgsOpenDocOSX() gets called
	if(AcquireFirstMatchingEventInQueue(GetMainEventQueue(), 1, eventspec, kEventQueueOptionsNone))
		return 1; // DoArgsOpenDocOSX() gets called
	return 0; // no passed args
#else
	if(g_argc <= 1) // no passed args
		return 0;
	DoArgsOpenDoc();
	return 1;
#endif
}

#ifdef __cplusplus
 }
#endif

// eof
