// maTextView.c

/*
 * MoreAmp
 * Copyright (C) 2004-2014 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include "maConfig.h" // included by all moreamp files

#if !defined(NOTEXTVIEWER)

#ifdef __cplusplus
extern "C"
{
#endif

#include <string.h>
#include <errno.h>

#include "maTypes.h"
#include "maDefs.h"
#include "maGUI.h"
#include "maPlayer.h"
#include "maUtils.h"
#include "maMemFile.h"
#include "maTextView.h"

maviewer g_viewerReadme;
maviewer g_viewerCatalog;
maviewer g_viewerPlaylist;
maviewer g_viewerLog;
#if defined(MAKEYBOARD)
 maviewer g_viewerKeyBindings;
#endif
int l_playindex = 0;

int ViewerIsBusy(int windowid);


int ViewerIsBusy(int windowid)
{
	maviewer *v;
	
	if(g_w[windowid] == NULL)
		return 0;
		
	if((v = Viewer(windowid)) == NULL)
	{
		maLog(L"ViewerIsBusy Viewer failed");
		return 0;
	}
	
	return(v->busy);

} // ViewerIsBusy


int ViewerDisable(int windowid, maviewer *v)
{
	if(v->busy)
		return 1;

	v->busy = 1;

	PushButtonDisable(windowid, ID_BTN_VIEWERFIND);
	PushButtonDisable(windowid, ID_BTN_EDITSAVE);
		
	return 0;

} // ViewerDisable


void ViewerEnable(int windowid, maviewer *v)
{
	PushButtonEnable(windowid, ID_BTN_EDITSAVE);
	if(!v->editable)
		PushButtonEnable(windowid, ID_BTN_VIEWERFIND);
	
	v->busy = 0;
		
} // ViewerEnable


maviewer *Viewer(int windowid)
{
	switch(windowid)
	{
		case WINDOWREADME:
		return &g_viewerReadme;
		case WINDOWCATALOG:
		return &g_viewerCatalog;
		case WINDOWPLAYLIST:
		return &g_viewerPlaylist;
		case WINDOWLOG:
		return &g_viewerLog;
#if defined(MAKEYBOARD)
		case WINDOWKEYBINDINGS:
		return &g_viewerKeyBindings;
#endif
		default:
			maLog(L"Viewer invalid windowid");
	}
	return NULL;

} // Viewer


void ViewerOnWindowClosing(int windowid)
{
	long findlen;
	maviewer *v;
	machar findstr[MAFINDSTRMAX];
	
	if((v = Viewer(windowid)) == NULL)
	{
		maLog(L"ViewerOnWindowClosing Viewer failed");
		return;
	}
	
	// save findtext
	if(g_w[windowid])
	{
		EditTextGetText(windowid, ID_EDITTEXT_VIEWERFIND, findstr, MAFINDSTRMAX - 1, &findlen, 0, -1);
		if(findlen > 0)
		{
			if(findlen >= MAFINDSTRMAX)
				findlen = MAFINDSTRMAX - 1;
			findstr[findlen] = L'\0';
			mastrcpy(v->findstr, findstr); 
			v->findlen = findlen; 
		}
	}
	else
		maLog(L"ViewerOnWindowClosing null window");

	while(ViewerIsBusy(windowid))
		maSleep(500);

	ViewerFree(v);
	v->editable = 0;
	
	// window specific stuff
	switch(windowid)
	{
		case WINDOWCATALOG:
			if(!g_cataloguing)
				if(!(g_state.stop & STOP_APPQUIT))
					MenuEnable(MENUTOOLS, ID_MENU_MAKECATALOG);
		break;
		default:
		break;
	}
	
} // ViewerOnWindowClosing


void ViewerFree(maviewer *v)
{
	if(v && v->p0)
	{
		if(v->p0 != v->mastr)
			MAFREE(v->p0);
		v->p0 = NULL;
	}
}


// copy the text file it into our own buffer v->p0 and also load it into the viewer object
int ViewerLoad(int windowid, maviewer *v)
{
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif

	if(v->p0 && v->p0 != v->mastr)
		maLog(L"ViewerLoad warning p0 non null on entry"); 

	v->nchars0 = 0;
	
	if(!maFileExists(v->fpath))
	{
		if(windowid == WINDOWLOG)
			masprintf(v->mastr, MAXLOGLINELEN, L"NO ERRORS = no log file '%ls'", fs2ma(mastr2, v->fpath));
		else
			masprintf(v->mastr, MAXLOGLINELEN, L"File '%ls' not found", fs2ma(mastr2, v->fpath));
		v->p0 = v->mastr;
	}
	else
	{
		v->p0 = NULL;
		/*
		if((v->nchars0 = mafileread(&(v->p0), v->fpath)) < 0 || v->p0 == NULL)
		{
			masprintf(v->mastr, MAXLOGLINELEN, L"ViewerLoad mafileread failed on '%ls'", fs2ma(mastr2, v->fpath));
			maLog(v->mastr);
			v->p0 = v->mastr;
		}
		*/
		v->nchars0 = (long)mafileread(&(v->p0), v->fpath);
	}

	// put text v->p0 into the viewer object
	if(v->p0)
	{
		EditTextSetText(windowid, ID_EDITTEXT_VIEWERVIEW, v->p0);
		if(windowid == WINDOWCATALOG && v->nchars0 > 0)
			ViewerCheckCatalogMem(v->p0, v->nchars0);
	}
	
	v->editable = 0;
	EditTextSetEditable(windowid, ID_EDITTEXT_VIEWERVIEW, 0);
	v->hits = 0;
	v->ip = -1;

	return 0;

} // ViewerLoad


int ViewerInit(int windowid, maviewer **pv)
{
	fschar *p;
	fschar name[MAX_PATH];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];
	
	if(pv == NULL)
	{
		maLog(L"ViewerInit error null viewer");
		return 1;
	}
	
	if((*pv = Viewer(windowid)) == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"ViewerInit Viewer failed on window %d", windowid);
		maLog(mastr);
		return 1;
	}

	ViewerDisable(windowid, *pv);
	
	switch(windowid)
	{
		case WINDOWREADME:
			fsstrcpy(g_viewerReadme.fpath, g_readmepath);
		break;
		
		case WINDOWCATALOG:
			if(FilenameFromPathname(g_playlistpath, name))
			{
				masprintf(mastr, MAXLOGLINELEN, L"ViewerInit FilenameFromPathname failed on '%ls'",
																	fs2ma(mastr2, g_playlistpath));
				maLog(mastr);
				ViewerEnable(windowid, *pv);
				return 1;
			}
			if((p = fsstrstr(name, g_extstr[kMAMAF])) != NULL)
				*p = fsT('\0');
			fssprintf(g_viewerCatalog.fpath, MAX_PATH, fsS fsC fsS fsT(".txt"), g_madir, FILESEP, name);
		break;
		
		case WINDOWPLAYLIST:
			 fsstrcpy(g_viewerPlaylist.fpath, g_playlistpath);
		break;
		
		case WINDOWLOG:
			fsstrcpy(g_viewerLog.fpath, g_logpath);
		break;
		
#if defined(MAKEYBOARD)
		case WINDOWKEYBINDINGS:
			 fsstrcpy(g_viewerKeyBindings.fpath, g_keybindingspath);
		break;
#endif

		default:
			masprintf(mastr, MAXLOGLINELEN, L"ViewerInit bad window %d", windowid);
			maLog(mastr);
			ViewerEnable(windowid, *pv);
		return 1;
	}

	ViewerSetFindText(windowid, (*pv)->findstr);
	
	ViewerLoad(windowid, *pv);
	
	EditTextSetFocus(windowid, ID_EDITTEXT_VIEWERFIND);
	
	ViewerEnable(windowid, *pv);

	return 0;

} // ViewerInit


void ViewerSetFindText(int windowid, machar *text)
{
	EditTextSetText(windowid, ID_EDITTEXT_VIEWERFIND, text);	
	StaticTextSetText(windowid, ID_TEXT_VIEWERFOUND, MASTRMINUS);
	
} // ViewerSetFindText


void ViewerSetPlayIndex()
{
	maviewer *v;
	machar mastr[MAXLOGLINELEN];

	v = &g_viewerCatalog;

	if(v->nchars0 <= 0)
		return;

	ViewerGetCatalogIndex(v->p0 + v->ip, v->p0 + v->nchars0, &l_playindex);
	
	if(l_playindex > 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"%d", l_playindex);
		StaticTextSetText(WINDOWCATALOG, ID_TEXT_VIEWERPLAY, (machar *)mastr);
	}

} // ViewerSetPlayIndex


void ViewerCheckCatalogMem(machar *pmem, long nchars0)
{
	machar *p;
	machar mastr[MAXLOGLINELEN];
	
	if(g_songsnum <= 0)
		return;
	
	if(pmem == NULL || *pmem == 0)
	{
		maLog(L"ViewerCheckCatalogMem null pmem");
		return;
	}

	masprintf(mastr, MAXLOGLINELEN, L"%05d ", g_songsnum); // 5 is MAINDEXSTRLEN 
	mastrcat(mastr, MACATALOGSTRING);
	if((p = mastrstr(pmem, mastr)) != NULL) // have it
	{
		p += mastrlen(MACATALOGSTRING);
		if(p >= pmem + nchars0)
			return;
		// masprintf(mastr, MAXLOGLINELEN, L"%05d "MACATALOGSTRING, g_songsnum + 1); // 5 is MAINDEXSTRLEN 
		masprintf(mastr, MAXLOGLINELEN, L"%05d ", g_songsnum + 1); // 5 is MAINDEXSTRLEN 
		mastrcat(mastr, MACATALOGSTRING);
		if(mastrstr(p, mastr) == NULL) // no higher index string
			return; // ok
	}
	
	masprintf(mastr, MAXLOGLINELEN, L"Out of date catalog file%ls%ls"
			L"Make a new one with the Make Catalog menu item%ls%ls"
			L"or edit it so that the number of entries matches the playlist.",
			g_maeol, g_maeol, g_maeol, g_maeol);
	ShowErr(mastr);

} // ViewerCheckCatalogMem


// "12345 MACATALOGSTRING"
// "0123456789..."
 void ViewerGetCatalogIndex(machar *p1, machar *last, int *index)
 {
	machar indexstr[MAINDEXSTRLEN + 2];
	machar *p;
	
	if(g_songsnum <= 0)
	{
		*index = 0;
		return;
	}

	if(p1 + MAINDEXSTRLEN + 2 + mastrlen(MACATALOGSTRING) > last)
	{
		*index = g_songsnum;
		return;
	}

	if((p = mastrstr(p1 + MAINDEXSTRLEN + 2, MACATALOGSTRING)) != NULL) // have next index
	{
		mastrncpy(indexstr, p - MAINDEXSTRLEN - 1, MAINDEXSTRLEN);
		indexstr[MAINDEXSTRLEN] = L'\0';
		*index = maatoi(indexstr) - 1;
	}
	else // must be last one
		*index = g_songsnum;
 } // ViewerGetCatalogIndex


void ViewerOnLeftClick(int windowid)
{
	machar findstr[MAFINDSTRMAX];
	long findlen;
	long start;
	long end;
	maviewer *v;

	if((v = Viewer(windowid)) == NULL)
	{
		maLog(L"ViewerOnLeftClick Viewer failed");
		return;
	}

	if(v->editable)
		return;
	
	if(ViewerDisable(windowid, v))
		return;
	
	EditTextGetSelection(windowid, ID_EDITTEXT_VIEWERVIEW, &start, &end);

	if(start > end ) // user selected right to left, swap
	{
		findlen = start;
		start = end;
		end = findlen;
	}

	v->ip = end;

	if(end > start)
	{
		if((end - start) >= MAFINDSTRMAX)
			end = start + MAFINDSTRMAX - 1;
		EditTextGetText(windowid, ID_EDITTEXT_VIEWERVIEW, findstr, MAFINDSTRMAX - 1, &findlen, start, end);
		if(findlen >= MAFINDSTRMAX)
			findlen = MAFINDSTRMAX - 1;
		findstr[findlen] =  L'\0';
		ViewerSetFindText(windowid, findstr);
		mastrcpy(v->findstr, findstr);
		v->findlen = findlen;
		ViewerCountHits(windowid, v);
	}

	if(windowid == WINDOWCATALOG)
		ViewerSetPlayIndex();

	ViewerEnable(windowid, v);
	
} // ViewerOnLeftClick


void ViewerOnRightClick(int windowid)
{
	// right-clicking in the viewer text starts (continues) a search.
	ViewerOnButtonFind(windowid);

} // ViewerOnRightClick


void ViewerOnButtonEditSave(int windowid)
{
	int ret = 0;
	FILE *fp;	
	maviewer *v;
	machar *p0 = NULL;
	size_t nchars;
	long nchars0;
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];
	
	if((v = Viewer(windowid)) == NULL)
	{
		maLog(L"ViewerOnButtonEditSave Viewer failed");
		return;
	}

	if(ViewerDisable(windowid, v))
		return;
		
	PushButtonGetDescriptor(windowid, ID_BTN_EDITSAVE, mastr);
	if(!mastr[0])
	{
		maLog(L"ViewerOnButtonEditSave PushButtonGetDescriptor failed");
		ViewerEnable(windowid, v);
		return;
	}
	if(mastrcmp(mastr, MASTRSAVE)) // enter edit mode
	{
		PushButtonDisable(windowid, ID_BTN_VIEWERFIND);
		EditTextDisable(windowid, ID_EDITTEXT_VIEWERFIND);
		PushButtonSetDescriptor(windowid, ID_BTN_EDITSAVE, MASTRSAVE);
		if(windowid == WINDOWCATALOG)
			MenuDisable(MENUTOOLS, ID_MENU_MAKECATALOG);

		EditTextSetEditable(windowid, ID_EDITTEXT_VIEWERVIEW, 1); // set edit mode
		v->editable = 1;

		EditTextSetFocus(windowid, ID_EDITTEXT_VIEWERVIEW);
	}
	else // save and exit edit mode
	{
		if(windowid == WINDOWPLAYLIST && g_state.running)
		{
			masprintf(mastr, MAXLOGLINELEN, L"The playlist cannot be saved while playing.%ls%ls"
				L"Either :%ls%ls"
				L"- Stop the player and then Save,%ls%ls"
				L"or%ls%ls"
				L"- Close the Playlist window to cancel the changes.",
				g_maeol, g_maeol, g_maeol, g_maeol, g_maeol, g_maeol, g_maeol, g_maeol);
			ShowErr(mastr);
			ViewerEnable(windowid, v);
			return;
		}
		
		v->editable = 0;
		EditTextSetEditable(windowid, ID_EDITTEXT_VIEWERVIEW, 0);
		
		ret = 0;
		if(EditTextGetSize(windowid, ID_EDITTEXT_VIEWERVIEW, &nchars))
		{
			maLog(L"ViewerOnButtonEditSave EditTextGetsize failed");
			ret = 1;
		}
		else
		{
			if((p0 = MAMALLOC((nchars + 2) * sizeof(machar))) == NULL)
			{
				maLog(L"ViewerOnButtonEditSave MAMALLOC failed");
				ret = 1;
			}
			else
			{
				EditTextGetText(windowid, ID_EDITTEXT_VIEWERVIEW, p0, (int)(nchars + 1), &nchars0, 0, (long)nchars);
				if(nchars0 < 0)
				{
					maLog(L"ViewerOnButtonEditSave EditTextGetText failed");
					ret = 1;
				}
				else
				{
#if defined(__MAWIN__)
					if((fp = mafopen(v->fpath, fsT("wt"))) == NULL) // text mode so we always write crlf's
#else
					if((fp = mafopen(v->fpath, fsT("wb"))) == NULL) 
#endif
					{
						masprintf(mastr, MAXLOGLINELEN, L"ViewerOnButtonEditSave mafopen failed on '%ls'", fs2ma(mastr2, v->fpath));
						maLog(mastr);
						ret = 1;
					}
					else
					{
						if(nchars0 > 0)
						{
							// BOM
							fwrite("\xef\xbb\xbf", 3, 1, fp); // BOM_UTF-8

							if((nchars = mafilewrite(nchars0, p0, fp)) == 0)
							{
								masprintf(mastr, MAXLOGLINELEN, L"ViewerOnButtonEditSave mafilewrite failed '%ls'", fs2ma(mastr2, v->fpath));
								maLog(mastr);
								ret = 1;
							}
						}
						if(fclose(fp) != 0)
						{
							masprintf(mastr, MAXLOGLINELEN, L"ViewerOnButtonEditSave fclose errno=%d '%ls'", errno, fs2ma(mastr2, v->fpath));
							maLog(mastr);
						}
					}
				}
				MAFREE(p0);
			}
		}

		// reload the edited file
		if(ret == 0)
		{
#if defined(MAKEYBOARD)
			if(windowid == WINDOWKEYBINDINGS)
			{
				if((LoadKeyBindings(1)) == 1)
					maLog(L"ViewerOnButtonEditSave LoadKeyBindings failed");
			}
#endif
			ViewerFree(v);
			if((ret = ViewerLoad(windowid, v)) != 0) // redundant but saves code
			{
				masprintf(mastr, MAXLOGLINELEN, L"ViewerOnButtonEditSave ViewerLoad failed on '%ls'", fs2ma(mastr2, v->fpath));
				maLog(mastr);
			}
			else if(windowid == WINDOWPLAYLIST) // reopen the modified playlist
			{
				ViewerEnable(windowid, v);
				if(OpenPlaylist(WINDOWPLAYLIST)) // skipsettings
				{
					masprintf(mastr, MAXLOGLINELEN, L"ViewerOnButtonEditSave OpenPlaylist failed on '%ls'", fs2ma(mastr2, v->fpath));
					maLog(mastr);
				}
				else
					if(g_sets.mawin[WINDOWLIST].ison && ((g_sets.listboxmode & LBOXPLAYERINFO) | (g_sets.listboxmode & LBOXPLAYLIST)))
						UpdateListWindowList(0); // for updated play list
			}
		}
		
		PushButtonSetDescriptor(windowid, ID_BTN_EDITSAVE, MASTREDIT);
		StaticTextSetText(windowid, ID_TEXT_VIEWERFOUND, MASTRMINUS);
		v->hits = 0;
		if(windowid == WINDOWCATALOG)
			MenuEnable(MENUTOOLS, ID_MENU_MAKECATALOG);
		EditTextEnable(windowid, ID_EDITTEXT_VIEWERFIND);
		PushButtonEnable(windowid, ID_BTN_VIEWERFIND);
	}
	
	ViewerEnable(windowid, v);
	
} // ViewerOnButtonEditSave


void ViewerCountHits(int windowid, maviewer *v)
{
	machar *p;
	machar foundstr[STRLEN256];
		
	// count the hits
	p = v->p0;
	v->hits = 0;
	while((p = mastrcasestr(p, v->findstr)) != NULL)
	{
		v->hits++;
		p += v->findlen;
	}

	if(v->hits <= 0)
	{
		StaticTextSetText(windowid, ID_TEXT_VIEWERFOUND, MASTRZERO);
		return;
	}
	masprintf(foundstr, STRLEN256, L"%d", v->hits);
	StaticTextSetText(windowid, ID_TEXT_VIEWERFOUND, foundstr);

} // ViewerCountHits


void ViewerOnButtonFind(int windowid)
{
	machar *p1;
	machar *p2;
	machar findstr[STRLEN256];
	long findlen;
	maviewer *v;

	if((v = Viewer(windowid)) == NULL)
	{
		maLog(L"ViewerOnButtonFind Viewer failed");
		return;
	}

	if(v->editable)
		return;
	
	if(ViewerDisable(windowid, v))
		return;
		
	// the text in the find edit box is the text to find
	EditTextGetText(windowid, ID_EDITTEXT_VIEWERFIND, findstr, MAFINDSTRMAX - 1, &findlen, 0, -1);
	if(findlen < 1)
	{
		ViewerEnable(windowid, v);
		return;
	}
	
	if(findlen >= MAFINDSTRMAX)
		findlen = MAFINDSTRMAX - 1;
	findstr[findlen] = L'\0';
	ViewerSetFindText(windowid, findstr);

	// if find string has changed, recount hits
	if(v->hits <= 0 || mastrcasecmp(findstr, v->findstr))
	{
		mastrcpy(v->findstr, findstr);
		v->findlen = findlen;
		ViewerCountHits(windowid, v);
	}

	if(v->ip < 0)
	{
		v->ip = 0;
		v->start = v->end = 0;
#if defined(MAGUIGTK2)
		if(v->hits > 0)
			EditTextSetSelection(windowid, ID_EDITTEXT_VIEWERVIEW, v->start, v->end); // scroll to top
#endif
	}
		
	if(v->hits <= 0)
	{
		ViewerEnable(windowid, v);
		return;
	}
	
	// look for a hit starting at current ip
	if(((p1 = v->p0 + v->ip) < v->p0 + v->nchars0) && (p2 = mastrcasestr(p1, v->findstr)) != NULL)
	{
		// save the position of the hit, set ip at the end of the  hit
		v->start = (long)(p2 - v->p0);
		v->end = v->start + findlen;
		v->ip = v->end;

		// show the hit in the viewer
		EditTextSetSelection(windowid, ID_EDITTEXT_VIEWERVIEW, v->start, v->end); 
		
		// update the 'found' string with the number of hit
		masprintf(findstr, MAFINDSTRMAX - 1, L"%d", v->hits);
		StaticTextSetText(windowid, ID_TEXT_VIEWERFOUND, findstr);
		
		// on wxMac need to toggle focus to make selection visible in TextCtrl if focus is elsewhere :
#if defined(MAGUIWX)
		if(EditTextHasFocus(windowid, ID_EDITTEXT_VIEWERFIND))
		{
			EditTextSetFocus(windowid, ID_EDITTEXT_VIEWERVIEW);
			EditTextSetFocus(windowid, ID_EDITTEXT_VIEWERFIND);
		}
#endif		
		// set the vplay button to the index near the hit
		if(windowid == WINDOWCATALOG)
			ViewerSetPlayIndex();
	}
	else // no hit
	{
		StaticTextSetText(windowid, ID_TEXT_VIEWERFOUND, MASTRBOTTOM);
		// reset search
		v->ip = -1;
	}
	
	ViewerEnable(windowid, v);
	
} // ViewerOnButtonFind


void ViewerOnButtonPlay(int windowid)
{
	machar mastr[MAXLOGLINELEN];

	PushButtonDisable(windowid, ID_BTN_VIEWERPLAY);

	if(l_playindex > 0)
	{
		if(l_playindex <= g_songsnum) // 1-based index of song to play
		{
			g_droplaunch = l_playindex + 1; // 0-based row to play + 2

			if(g_state.paused)
			{
				g_state.silence = 1;
				SetPause(0);
			}

			if(g_state.running)
			{
				g_loadedcurrent = g_loadednum = 0;
				g_loadedquit = 1; // exit RunLoop
			}
			else
				DoTimerPlay(100); // ViewerOnButtonPlay
		}
		else
		{
			masprintf(mastr, MAXLOGLINELEN, L"Cannot play %d", l_playindex);
			StaticTextSetText(windowid, ID_TEXT_VIEWERPLAY, mastr);
		}
	}
	else
		StaticTextSetText(windowid, ID_TEXT_VIEWERPLAY, L"Nothing to play.");

	PushButtonEnable(windowid, ID_BTN_VIEWERPLAY);

} // ViewerOnButtonPlay


#ifdef __cplusplus
}
#endif

#endif 	// NOTEXTVIEWER

// eof
