// maTextView.h

/*
 * MoreAmp
 * Copyright (C) 2004-2014 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef __MATEXTVIEW_H__
#define __MATEXTVIEW_H__

#if !defined(NOTEXTVIEWER)

#if defined(MAGUICARBON)
 #include <Events.h>
#endif

#ifdef __cplusplus
extern "C"
{
#endif

#define MAFINDSTRMAX STRLEN256
#define MASTRSAVE L"Save"
#define MASTREDIT L"Edit"
#define MASTRMINUS L"-"
#define MASTRBOTTOM L"At bottom."
#define MASTRZERO L"0"

typedef struct
{
	int initialized;				// has been initialized
	void *text;						// text widget with text to view
	void *entry;					// entry widget for text to find
#if defined(MAGUIGTK2)
	void *buffer;
#endif
	machar *p0;						// our copy of the viewer text in machars
	long ip;						// insertion point (on win, counts crlf)
	long start;						// selection start (on win, counts crlf) 
	long end;						// selection  end (on win, counts crlf)
	long nchars0;					// numbers of characters (machars, not bytes)
	long findlen;					// length of text to find
	int hits;						// number of hits
	int busy;						// set while viewer state is subject to change (loading, saving, etc.)
	int editable;					// edit mode on or off
	fschar fpath[MAX_PATH];			// file to view
	machar findstr[MAFINDSTRMAX];	// text to find
	machar mastr[MAXLOGLINELEN];		// error msg, p0 sometimes set to this
#if defined(MAGUICARBON)
	int OffsetsValid;
	TXNOffset StartOffset;
	TXNOffset EndOffset;
	ControlRef findedit;
	TXNObject TXNobj;
	ControlRef scrollbar;
	SInt32 scrollCache;
	SInt32 scrollCacheS;
#endif
} maviewer;

extern maviewer g_viewerReadme;
extern maviewer g_viewerCatalog;
extern maviewer g_viewerPlaylist;
extern maviewer g_viewerLog;
#if defined(MAKEYBOARD)
 extern  maviewer g_viewerKeyBindings;
#endif
extern int l_playindex;

int ViewerDisable(int windowid, maviewer *v);
void ViewerEnable(int windowid, maviewer *v);
maviewer *Viewer(int windowid);
void ViewerOnWindowClosing(int windowid);
void ViewerFree(maviewer *v);
int ViewerLoad(int windowid, maviewer *v);
int ViewerInit(int windowid, maviewer **pv);
void ViewerSetFindText(int windowid, machar *text);
void ViewerSetPlayIndex();
void ViewerCheckCatalogMem(machar *pmem, long nchars0);
void ViewerGetCatalogIndex(machar *pmem, machar *last, int *index);
void ViewerOnLeftClick(int windowid);
void ViewerOnRightClick(int windowid);
void ViewerOnButtonEditSave(int windowid);
void ViewerCountHits(int windowid, maviewer *v);
void ViewerOnButtonFind(int windowid);
void ViewerOnButtonPlay(int windowid);

#ifdef __cplusplus
}
#endif

#endif 	// NOTEXTVIEWER

#endif 	// __MATEXTVIEW_H__

//eof
