// maTypes.h

/*
 * MoreAmp
 * Copyright (C) 2004-2014 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/* 
Platform dependent definitions, types, macros, functions, etc.

				unix-unicode				win-unicode
				mac-unicode

// chars for filepaths and system commands
fschar			char (1)					wchar_t	(2)

fsT(a)			a							L ** a

fsC				"%c"						L"%c"

fsS				"%s"						L"%s"


// chars for internal processing
machar		wchar_t (4)						wchar_t	(2)


Text files editable by the user, playlists (.maf), catalogs, keybindings, README (.txt)
are encoded UTF-8 with a BOM header (0xefbbbf).

Files not editable by the user, settings files (.mas) and records files (.mar), are binary.
Text within these settings file (eqnames, outputdirs, vdiskname) is UTF-8.

keyboard/display	MACROMAN	ISO8859-1	UTF-8-MAC	UTF-8	
example	ê			0x90		0xea		0x65cc82	0xc3aa	// type kb characters '^' then 'e'
example	é			0x8e		0xe9		0x65cc81	0xc3a9	// type kb character 'é'
*/

#ifndef __MATYPES_H__
#define __MATYPES_H__

#ifdef __cplusplus
extern "C"
{
#endif

#include "maConfig.h" // included by all moreamp files

#if defined(__MAUNIX__) && !defined(__MACUNIX__)
 #if !defined(__USE_ISOC95)
  #define __USE_ISOC95 // for swprintf and fwprintf
 #endif
#endif
#include <stdio.h>
#include <wchar.h>

#define STRLEN16 16
#define STRLEN32 32
#define STRLEN64 64
#define STRLEN128 128
#define STRLEN256 256
#define STRLEN512 512
#define STRLEN1024 1024
#define STRLEN8192 8192


#if defined(__MAWIN__) // unicode win
 typedef wchar_t mauchar;
 typedef wchar_t fschar;
 #define fseof WEOF
 #define masprintf _snwprintf
 #define mavsprint _vswprintf
 #define masnprintf _snwprintf
 #define fssprintf _snwprintf
 #define fsstrcpy wcscpy
 #define fsC L"%c"
 #define fsS L"%s"
 #define fsT(a) L ## a
 #define fsstrstr wcsstr
 #define fsstrchr wcschr
 #define fsstrcasecmp mastrcasecmp
 #define fsstrrchr wcsrchr
 #define fsstrcmp wcscmp
 #define fsstrlen wcslen
 #define fsstrncpy wcsncpy
 #define fsstrcat wcscat
 #define fsfprintf fwprintf
 #define fsgetcwd _wgetcwd
 #define fsstat _wstat
 #define fsstatstruct _stat
 #define osunlink _unlink
 #define fsunlink _wunlink
 #define fsstrncmp wcsncmp
 #define os2fs os2ma
 #define os2fsma(a, b) b 
 #define fsfopen(a, b) _wfopen(a, b)
 #define fsatoi _wtoi
 #define maatoi _wtoi
 #define ossprintf _snprintf
 #if defined(MASETLOCALE)
  #define MASETLOCALEWIN
  #define masetlocale _wsetlocale
  #define maenvchar wchar_t
  #define magetenv _wgetenv
  #define maenvS L"%ls"
  #define maenvT(a) L ## a
  #define maenvstrcpy wcscpy
  // #define maenvstrcmp wcscmp
  // #define maenvstrstr wcsstr
  #define maenv2fs ma2fs
  // #define maenv2os ma2os
 #endif
 #define fs2ma(a, b) b
 #define ma2fs(a, b) b
#else // unicode not win
 #define masprintf swprintf
 #define mavsprint vsprintf
 #define masnprintf swprintf
 typedef char mauchar;
 typedef char fschar;
 #define fseof EOF
 #define fssprintf snprintf
 #define fsstrcpy strcpy
 #define fsC "%c"
 #define fsS "%s"
 #define fsT(a) a
 #define fsstrstr fsstrstrfn
 #define fsstrchr strchr
 #define fsstrcasecmp strcasecmp
 #define fsstrrchr strrchr
 #define fsstrcmp strcmp
 #define fsstrlen strlen
 #define fsstrncpy strncpy
 #define fsstrcat strcat
 #define fsfprintf fprintf
 #define fsstat stat
 #define fsstatstruct stat
 #define osunlink unlink
 #define fsunlink unlink
 #define fsstrncmp strncmp
 #define os2fs(a, b) b
 #define os2fsma os2ma 
 #define fsfopen(a, b) fopen(a, b)
 #define fsatoi atoi
 #define maatoi maatoifn
 #define ossprintf snprintf
 #if defined(MASETLOCALE)
  #define masetlocale setlocale
  #define maenvchar char
  #define magetenv(a) getenv(a)
  #define maenvS "%s"
  #define maenvT(a) a
  #define maenvstrcpy strcpy
  // #define maenvstrcmp strcmp
  // #define maenvstrstr strstr
  #define maenv2fs os2fs
  // #define maenv2os(a, b) a
 #endif
 #define fs2ma(a, b) fs2mafn(a, b)
 #define ma2fs(a, b) ma2fsfn(a, b)
#endif // unicode not win
 
typedef wchar_t machar;
typedef wint_t maint;
#define os2ma(a, b) os2mafn(a, b)
#define mapopen _wpopen
#define maeof WEOF
#define matolower towlower
#define mastrcpy wcscpy
#define mastrlen wcslen
#define mastrstr wcsstr
#define mastrcmp wcscmp
#define maprintf wprintf
#define mafprintf fwprintf
#define mastrncpy wcsncpy
#define mastrrchr wcsrchr
#define mastrchr wcschr
#define mastrncmp wcsncmp
#define mastrcat wcscat
#define mafgets fgetws
// #define mafgetc fgetwc
// #define maungetc ungetwc
#define FSRBMODE fsT("rb")
#define FSWBMODE fsT("wb")
#define FSABMODE fsT("ab")

#if defined(__MAWIN__) 
 #define MAOPEN _wopen
 #define MACLOSE _close
 #define MAREAD _read
 #define MALSEEK _lseek
 #define MALSEEKI64 _lseeki64
 #define MAFSTAT _wstat
 #define MAFSTATI64 _wstati64
 #define MAFILENO _fileno
#else
 #define MAOPEN open
 #define MACLOSE close
 #define MAREAD read
 #define MALSEEK lseek
 #define MALSEEKI64 _lseeki64
 #define MAFSTAT fstat
 #define MAFSTATI64 _fstati64
 #define MAFILENO fileno
#endif

#ifdef __MAWIN__
 #include <io.h>
 /*
 #if defined(_MSC_VER)
  #if _MSC_VER <= 1600
   #define fseeko fseek
   #define ftello ftell
  #endif
 #endif
 */
 #ifndef fseeko
  #define fseeko _fseeki64
 #endif
 #ifndef ftello
  #define ftello _ftelli64
 #endif
#endif

#if (defined(__MAWIN__) || defined(__VISUALC__) && defined(__WIN32__))
 #define malonglong_t __int64
#elif defined(__BORLANDC__) && defined(__WIN32__) && (__BORLANDC__ >= 0x520)
 #define malonglong_t __int64
#elif (defined(__WATCOMC__) && (defined(__WIN32__) || defined(__DOS__) || defined(__OS2__)))
 #define malonglong_t __int64
#elif defined(__DIGITALMARS__)
 #define malonglong_t __int64
#elif defined(__MINGW32__)
 #define malonglong_t long long
#elif defined(__MWERKS__)
 #if __option(longlong)
  #define malonglong_t long long
 #else
  #error "The 64 bit integer support in CodeWarrior has been disabled."
  #error "See the documentation on the 'longlong' pragma."
 #endif
#elif defined(__WXPALMOS__)
 #define malonglong_t int64_t
#elif defined(__VISAGECPP__) && __IBMCPP__ >= 400
 #define malonglong_t long long
#elif (defined(SIZEOF_LONG_LONG) && SIZEOF_LONG_LONG >= 8)  || \
        defined(__GNUC__) || \
        defined(__CYGWIN__) || \
        defined(__WXMICROWIN__) || \
        (defined(__DJGPP__) && __DJGPP__ >= 2)
 #define malonglong_t long long
#elif defined(SIZEOF_LONG) && (SIZEOF_LONG == 8)
 #define malonglong_t long
#endif

typedef malonglong_t maint64;
typedef unsigned malonglong_t mauint64;

// these should be included in all the cases above
#ifdef __MAWIN__
 typedef __int64 maoff_t64;
 #define MAUINT64MAX 0xFFFFFFFFFFFFFFFF
#else
 #include <stdint.h>
 typedef uint64_t maoff_t64;
 #define MAUINT64MAX UINT64_MAX
#endif

#ifdef __cplusplus
}
#endif
#endif // __MATYPES_H__

// eof
