// maUtils.h

/*
 * MoreAmp
 * Copyright (C) 2004-2014 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef __MAUTILS_H__
#define __MAUTILS_H__

#ifdef __cplusplus
 extern "C"
 {
#endif

#include <wchar.h>
#if !defined(__MAWIN__)
 #include <stdint.h>
#endif

#include "maTypes.h"

void ShowErr(const machar *msg);

double SliderDBFromPos(int value);
int SliderPosFromDB(double db);

int SongIsInMemory(fschar *filepath);
int PlaylistIsNoRecords();
int PlaylistIsCda();
int PlaylistIsDrop();
#if !defined(NODECODEAUDIOIN)
 int PlaylistIsAudioIn();
#endif
int SongFileExists(fschar *songfile);
int PathnameFromFilename(fschar *filename, fschar *pathname);
int FilenameFromPathname(fschar *pathname, fschar *filename);
int FileNameRemoveExtension(fschar *filename);
int FileNameHasExtension(fschar *fname, const fschar *ext);
int FileNameHasExtensionPlayable(fschar *fname);
int FileIsPlayable(fschar *filepath);
int maCreateDir(fschar *from);

int maFileExists(fschar *filepath);
int maDirExists(fschar *dirpath);
int maRenameDir(fschar *fromdirpath, fschar *todirpath);
int maClearDir(fschar *dirpath);
int maFileCopy(fschar *frompath, fschar *topath, int concat);
int maFileConcat(fschar *frompath, fschar *topath);

int IsAudioCd(fschar *fname);

machar *SecondsToStrHMS(machar *strhms, int xs);
machar *MillisecondsToStrHMS(machar *strhms, mauint64 xms);
machar *MillisecondsToStrHMSD(machar *strhms, mauint64 xms);
int StrHMSToMilliSeconds(machar *strhms);

FILE *mafopen(fschar *filename, fschar *mode);

#if defined(ENCODEMP4) || defined(ENCODEFLAC)
 FILE *mafopenansi(char *filename, char *mode);
#endif	
int machmod(fschar *filename, int mode);
int mamkdir(fschar *filename);
int maunlink(fschar *filename);
// fschar *mafgets(fschar *s, int n, FILE *fp);
machar *memgets(machar *s, int n, machar *p);

int mastrcasecmp(machar *s1, machar *s2);
int mastrncasecmp(machar *s1, machar *s2, size_t n);
machar *mastrcasestr(machar *s1, machar *s2);

#ifdef __MAWIN__
 int maWinResolveShortcut(fschar *pathname);
 int maGetLongPathName(fschar *infname, fschar *outfname, int len);
#endif

#if defined(__MACUNIX__)
 #include <sys/types.h>
#endif

int maNamedSemaphoreOpen(void **ppsem, fschar *name);
int maNamedSemaphoreWait(void *psem);
int maNamedSemaphorePost(void *psem);
//int maNamedSemaphoreUnlink(void *psem, fschar *name);
#if defined(MAUSESHM) || defined(MAUSESHF)
 typedef struct
 {
	fschar name[32];
	void *psem;
 } MALOCK;
 int maNamedSemaphoreLock(MALOCK *lock);
 int maNamedSemaphoreUnlock(MALOCK *lock);
 // int maNamedSemaphoreClose(MALOCK *lock);
#endif

int maSemaphoreNew(void **ppsem);
int maSemaphoreWait(void *psem);
int maSemaphorePost(void *psem);
int maSemaphoreDelete(void *psem);

#if defined(__MAUNIX__) || defined(__MAOSX__)
 int maMutexNew(pthread_mutex_t *plock); // ppmutex
 int maMutexLock(pthread_mutex_t *plock);
 int maMutexUnlock(pthread_mutex_t *plock);
 int maMutexDelete(pthread_mutex_t *plock);
#endif

void ThreadYield();
void AppYield();
int maSleep(unsigned int millis);

#if defined(__MAOSX__)
 int maGetRootVolumeName(fschar *name);
#endif

typedef void *(*threadfunctiontype)(void *);
typedef void (*threadparametertype)(void);

int maThreadCreate(threadfunctiontype threadfunction, threadparametertype threadparameter);

#if defined(MAUSESHM) || defined(MAUSESHF)
 void *mashmopen(size_t shmsize, fschar *keyfile);
 int mashmclose(void *pmashm);
 int mashmread(char *ptr, size_t size, size_t nmemb, void *pmashm);
 int mashmwrite(char *ptr, size_t size, size_t nmemb, void *pmashm);
 int mashmdelete(void *pmashm);
#endif

int magetpid();
int maNumProcesses(int *pid);

fschar *magetmoreampdir(fschar *dir);
fschar *magetcwd(fschar *dir, int size);
fschar *magetbindir(fschar *dir);
// int masetcwd(fschar *dir);
#if defined(__MAWIN__)
 int maIsWin9x();
 int maIsWin7();
#endif
	 
unsigned long maGetLocalTimeMillis();
machar *maGetLocalDateTimeString(machar *retstring);

int matimer_set_timeout(long ms, void *callback);
// int matimer_unset_timeout();

#if defined(MAMEMTRACE)
 extern int g_nmamallocs;
 extern int g_nmacallocs;
 extern int g_nmafrees;
 void *mamalloc(size_t a);
 void *macalloc(size_t a, size_t b);
 void mafree(void *a);
 void maTrace(machar *str);
#endif

int string2int(machar *digit, int *pval);
int maIsFullPath(fschar *path);
size_t mastrlen_utf8(const char *_s);
machar *fs2mafn(machar *mastr, fschar *p);
fschar *fsstrstrfn(fschar *fsstr, fschar *p);
machar *os2mafn(machar *mastr, char *p);
int maatoifn(machar *s);
#if defined(MAGUICARBON)
 size_t StrCHARSETKBtoUTF8(char *strconvert);
#endif
#if defined(MAGUINONE) && defined(__MACUNIX__) // osxnogui
  uint32_t ma_keyval_to_unicode(uint32_t keyval); // guint32 gdk_keyval_to_unicode(guint keyval);
  int ma_unichar_to_utf8(uint32_t c, char *outbuf); // int g_unichar_to_utf8(gunichar c, gchar *outbuf);
#endif
maenvchar *magetcharset();
#define UTF8_IGNORE_ERROR		0x01
#define UTF8_SKIP_BOM			0x02
size_t utf8_to_wclocal(const char *in, size_t ninbytes, wchar_t *out, size_t noutchars, int flags);
size_t wclocal_to_utf8(const wchar_t *in, size_t ninchars, char *out, size_t noutbytes );
#if defined(MAPLAYSBSMS)
 size_t mafilereadraw(void **pmem, fschar *filepath, int sizeonly);
#endif
size_t mafileread(machar **pmastr, fschar *filepath);
size_t mafilewrite(size_t nchars, machar *mastr, FILE *fp);
fschar *ma2fsfn(fschar *fsstr, machar *p);
#if defined(MAGUIGTK2)
 size_t offset_wchar_to_utf8(wchar_t *mastr, size_t maoffset);
 size_t offset_utf8(char *mastr, size_t maoffset);
#endif
#if !defined(__MAWIN__)
 size_t StrCHARSETtoUTF8(fschar *strconvert);
 size_t StrISO88591toUTF8(char *strconvert);
#endif
#if !defined(MASETLOCALEWIN)
 #if defined(MAGUICARBON)
  size_t StrMACHARSETKBtoUTF8(char *strconvert);
 #endif
 size_t StrMACHARSETtoUTF8(char *strconvert);
#endif
#if defined(__MAOSX__) || defined(__MACUNIX__)
 // size_t StrCHARSETKBtoStrCHARSET(char *strconvert);
 size_t StrCHARSETtoUTF8MAC(fschar *strconvert);
 size_t StrUTF8MACtoUTF8(fschar *strconvert);
 size_t StrUTF8toUTF8MAC(fschar *strconvert);
 #if defined(MAGUICARBON)
  // size_t StrUTF8MACtoCHARSET(fschar *strconvert);
  size_t StrUTF8toCHARSET(fschar *strconvert);
 #endif
 size_t StrUTF8toUTF16LE(fschar *strconvert);
 // size_t StrISO88591toCHARSET(fschar *strconvert);
 // size_t StrUTF8toMACROMAN(char *strconvert);
#endif

#if defined(__MAWIN__)
 // #define lrint(x) malrint(x)
 // long int malrint(double flt);
 #define lrintf(x) malrintf(x)
 long int malrintf(float flt); 
#endif

#ifdef __cplusplus
 }
#endif

#endif // __MAUTILS_H__

// eof
