// maWriteMp4.c

/*
 * MoreAmp
 * Copyright (C) 2004-2014 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

 // with code from faac 1.25 frontend
 
#include "maConfig.h" // included by all moreamp files

// warnings not fixed
#ifdef __MAWIN__
 #pragma warning( disable : 4142 ) // benign redefinition of type
#endif

#ifdef ENCODEMP4

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#include <errno.h>

#ifdef __MAOSX__
 #include <sys/stat.h>
#endif

#ifdef __MAWIN__
 #include <wtypes.h>
 #include <mp4ff.h>

 typedef unsigned long u_int32_t;
 typedef unsigned __int64 u_int64_t;
 typedef __int64 int64_t;
 typedef unsigned short u_int16_t;
 typedef unsigned char u_int8_t;
 typedef char int8_t;
#endif

#include "faac.h"

#define __SYSTEMS_H__
#include "mp4.h"
#undef __SYSTEMS_H__
 
#include "maTypes.h"
#include "maDefs.h"
#include "maWriteMp4.h"
#include "maPlayer.h"
#include "maUtils.h"

#define DEFAULT_TNS 0

u_int64_t l_wmp4total_samples;
u_int64_t l_wmp4encoded_samples;
MP4FileHandle g_MP4hFile;
MP4TrackId l_wmp4track;
unsigned int l_wmp4frameSize;
unsigned int l_wmp4delay_samples;
faacEncHandle l_wmp4hEncoder = NULL;
unsigned long l_wmp4samplesInput, l_wmp4maxBytesOutput;
unsigned char *l_wmp4bitbuf = NULL;
short *l_wmp4pcmbuf= NULL;
short *l_wmp4end;

int *mp4mkChanMap(int channels, int center, int lf);

int OpenWriteMp4()
{
	int channels = 2;	
	faacEncConfigurationPtr myFormat;
	unsigned int mpegVersion = g_sets.fileout[g_k[kOUTMP4]].mode & kACCMPEG2 ? MPEG2 : MPEG4;
	unsigned int objectType = g_sets.fileout[g_k[kOUTMP4]].mode & kACCLTP ? LTP : LOW;
	unsigned int useMidSide = 1;
	unsigned int useTns = g_sets.fileout[g_k[kOUTMP4]].mode & kACCTNS ? 1 : 0; // 0=DEFAULT_TNS;
	unsigned int useAdts = 0;
	unsigned long quantqual;
	unsigned long samplerate;
	int chanC = 3;
	int chanLF = 4;
	int shortctl = SHORTCTL_NORMAL;
	int *chanmap;
	u_int8_t *ASC = 0;
	u_int32_t ASCLength = 0;
    char *faac_id_string;
    char *faac_copyright_string;
	machar mastr[MAXLOGLINELEN];
	
	samplerate = g_state.insamplerate;
	
	switch(g_sets.fileout[g_k[kOUTMP4]].quality)
	{
		case 0  : quantqual = 130; break;
		case 1  : quantqual = 120; break;
		case 2  : quantqual = 110; break;
		case 4  : quantqual =  90; break;
		case 5  : quantqual =  80; break;
		case 6  : quantqual =  70; break;
		default : quantqual = 100; break;
	}

	g_MP4hFile = MP4_INVALID_FILE_HANDLE;
	l_wmp4track = 0;
	l_wmp4total_samples = 0;
	l_wmp4encoded_samples = 0;

	// open the encoder library
	l_wmp4hEncoder = faacEncOpen(samplerate, channels, &l_wmp4samplesInput, &l_wmp4maxBytesOutput);

	if(l_wmp4hEncoder == NULL)
	{
		maLog(L"OpenWriteMp4 faacEncOpen failed");
		return 1;
	}
	
	l_wmp4frameSize = l_wmp4samplesInput / channels;
	l_wmp4delay_samples = l_wmp4frameSize; // encoder delay 1024 samples

	l_wmp4pcmbuf = (short *)MAMALLOC(2 * l_wmp4samplesInput * sizeof(short));
	
	if(l_wmp4pcmbuf == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"OpenWriteMp4 MAMALLOC failed for %d bytes", (int)(2 * l_wmp4samplesInput * sizeof(short)));
		maLog(mastr);
		return 1;
	}

	l_wmp4bitbuf = (unsigned char*)MAMALLOC(l_wmp4maxBytesOutput * sizeof(unsigned char));
	
	if(l_wmp4bitbuf == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"OpenWriteMp4 MAMALLOC failed for %d bytes", (int)(l_wmp4maxBytesOutput * sizeof(unsigned char)));
		maLog(mastr);
		return 1;
	}

	chanmap = mp4mkChanMap(channels, chanC, chanLF);

	if(chanmap)
	{
		masprintf(mastr, MAXLOGLINELEN, L"OpenWriteMp4 remapping input channels: Center=%d, LFE=%d",  chanC, chanLF);
		maLog(mastr);
	}

	// put the options in the configuration struct
	myFormat = faacEncGetCurrentConfiguration(l_wmp4hEncoder);
	myFormat->aacObjectType = objectType;
	myFormat->mpegVersion = mpegVersion;
	myFormat->useTns = useTns;
	
	switch(shortctl)
	{
		case SHORTCTL_NOSHORT:
			masprintf(mastr, MAXLOGLINELEN, L"OpenWriteMp4 disabling short blocks");
			maLog(mastr);
			myFormat->shortctl = shortctl;
		break;
		case SHORTCTL_NOLONG:
			masprintf(mastr, MAXLOGLINELEN, L"OpenWriteMp4 disabling long blocks");
			maLog(mastr);
			myFormat->shortctl = shortctl;
		break;
	}
	
	if(channels >= 6)
		myFormat->useLfe = 1;

	myFormat->allowMidside = useMidSide;

	myFormat->bandWidth = samplerate / 2; // cutOff;

	myFormat->quantqual = quantqual;

	myFormat->outputFormat = useAdts;
	myFormat->inputFormat = FAAC_INPUT_16BIT;

	if(!faacEncSetConfiguration(l_wmp4hEncoder, myFormat))
	{
		masprintf(mastr, MAXLOGLINELEN, L"OpenWriteMp4 unsupported output format!");
		maLog(mastr);
		return 1;
	}

	// initialize MP4 creation
	
#if defined(MP4_CREATE_EXTENSIBLE_FORMAT)
	g_MP4hFile = MP4Create((char *)g_tempfile, 0, 0); 
#else
	g_MP4hFile = MP4Create((char *)g_tempfile, 0, 0, 0);
#endif

	g_tempfilefp = (FILE *)g_MP4hFile;
		
	if(g_MP4hFile == MP4_INVALID_FILE_HANDLE)
	{
		g_tempfilefp = DUMMYOUTFILEP;
		masprintf(mastr, MAXLOGLINELEN, L"OpenWriteMp4 MP4Create failed on '%ls'", g_tempfile);
		maLog(mastr);
		return 1;
	}

#if defined(__MAOSX__) || defined(__MAUNIX__)
	machmod(g_tempfile, 0777); // ACCESSPERMS is defined in sys/stat.h
#endif

	MP4SetTimeScale(g_MP4hFile, 90000);
	l_wmp4track = MP4AddAudioTrack(g_MP4hFile, samplerate, MP4_INVALID_DURATION, MP4_MPEG4_AUDIO_TYPE);
	MP4SetAudioProfileLevel(g_MP4hFile, 0x0F);
	faacEncGetDecoderSpecificInfo(l_wmp4hEncoder, &ASC, (unsigned long *)(void *)&ASCLength);
	MP4SetTrackESConfiguration(g_MP4hFile, l_wmp4track, ASC, ASCLength);
	free(ASC);
	 
    // insert comment into output file
	if(faacEncGetVersion(&faac_id_string, &faac_copyright_string) == FAAC_CFG_VERSION)
    {
		sprintf((char *)mastr, "%s-%s / faac %s", MOREAMP_NAMEos, MOREAMP_VERSION_STRINGos, faac_id_string);
		MP4SetMetadataComment(g_MP4hFile, (char *)mastr);
    }
    else
		maLog(L"OpenWriteMp4 faacEncGetVersion failed");
		
	l_wmp4end = l_wmp4pcmbuf;

	return 0;

} // OpenWriteMp4


int WriteMp4(void* pcmblock, unsigned long len)
{
	unsigned long bytesToFile;
	unsigned long samplesToEncode;
	short *pcm;
	int res;
	machar mastr[MAXLOGLINELEN];
	
	samplesToEncode = len / sizeof(short);
	pcm = (short *)pcmblock;
	
	l_wmp4total_samples += samplesToEncode;
	
	samplesToEncode = len / sizeof(short);
	pcm = (short *)pcmblock;
	
	// encoding loop
	while(!(g_state.stop & STOP_ALL) && samplesToEncode >= l_wmp4samplesInput)
	{
		// concat pcm to any previous incomplete block
		memcpy(l_wmp4end, pcm, l_wmp4samplesInput * sizeof(short));
		
		// encode exactly l_wmp4samplesInput samples
		res = faacEncEncode(l_wmp4hEncoder, (int32_t *)l_wmp4pcmbuf, l_wmp4samplesInput, l_wmp4bitbuf, l_wmp4maxBytesOutput);

		if(res < 0)
		{
			masprintf(mastr, MAXLOGLINELEN, L"WriteMp4 faacEncEncode() failed");
			maLog(mastr);
			return 1;
		}

		bytesToFile = res;

		// write to output file
		if(bytesToFile > 0)
		{
			u_int64_t samples_left = l_wmp4total_samples - l_wmp4encoded_samples + l_wmp4delay_samples;
			MP4Duration dur = samples_left > l_wmp4frameSize ? l_wmp4frameSize : samples_left;
			MP4Duration ofs = l_wmp4encoded_samples > 0 ? 0 : l_wmp4delay_samples;

			MP4WriteSample(g_MP4hFile, l_wmp4track, l_wmp4bitbuf, bytesToFile, dur, ofs, 1);

			l_wmp4encoded_samples += dur;

		} // if(bytesToFile > 0)
				
		// shift incomplete block
		if(l_wmp4end != l_wmp4pcmbuf)
			memmove(l_wmp4pcmbuf, l_wmp4pcmbuf + l_wmp4samplesInput, (l_wmp4end - l_wmp4pcmbuf) * sizeof(short));

		// adjust pointers
		samplesToEncode -= l_wmp4samplesInput;
		pcm += l_wmp4samplesInput;
		
	} // while
	
	// save incomplete block
	if(samplesToEncode > 0)
	{
		memcpy(l_wmp4end, pcm, samplesToEncode * sizeof(short));
		l_wmp4end += samplesToEncode;
	}
	
	// until saved block is big enough to encode
	if(l_wmp4end >= l_wmp4pcmbuf + l_wmp4samplesInput)
	{
		// encode exactly l_wmp4samplesInput samples
		res = faacEncEncode(l_wmp4hEncoder, (int32_t *)l_wmp4pcmbuf, l_wmp4samplesInput, l_wmp4bitbuf, l_wmp4maxBytesOutput);

		if(res < 0)
		{
			masprintf(mastr, MAXLOGLINELEN, L"WriteMp4 faacEncEncode() failed");
			maLog(mastr);
			return 1;
		}

		bytesToFile = res;

		// write to output file
		if(bytesToFile > 0)
		{
			u_int64_t samples_left = l_wmp4total_samples - l_wmp4encoded_samples + l_wmp4delay_samples;
			MP4Duration dur = samples_left > l_wmp4frameSize ? l_wmp4frameSize : samples_left;
			MP4Duration ofs = l_wmp4encoded_samples > 0 ? 0 : l_wmp4delay_samples;

			MP4WriteSample(g_MP4hFile, l_wmp4track, l_wmp4bitbuf, bytesToFile, dur, ofs, 1);

			l_wmp4encoded_samples += dur;
		} // if(bytesToFile > 0)
		
		l_wmp4end -= l_wmp4samplesInput;
		
		// shift incomplete block
		if(l_wmp4end != l_wmp4pcmbuf)
			memmove(l_wmp4pcmbuf, l_wmp4pcmbuf + l_wmp4samplesInput, (l_wmp4end - l_wmp4pcmbuf) * sizeof(short));
	}
	
	return 0;

} // WriteMp4


int CloseWriteMp4()
{
	unsigned long bytesToFile;
	int32_t dummy;
	int i;
	
	/*
	int res;
	machar mastr[MAXLOGLINELEN];
	// pad to 0 and encode last saved blocks
	if(l_wmp4end != l_wmp4pcmbuf)
	{
		memset(l_wmp4end, 0, (l_wmp4end - l_wmp4pcmbuf) * sizeof(short));
		
		// encode exactly l_wmp4samplesInput samples
		res = faacEncEncode(l_wmp4hEncoder, (int32_t *)l_wmp4pcmbuf, l_wmp4samplesInput, l_wmp4bitbuf, l_wmp4maxBytesOutput);

		if(res < 0)
		{
			masprintf(mastr, MAXLOGLINELEN, L"CloseWriteMp4 faacEncEncode() failed";
			maLog(mastr);
			return 1;
		}

		bytesToFile = res;
		
		// write to output file
		if(bytesToFile > 0)
		{
			u_int64_t samples_left = l_wmp4total_samples - l_wmp4encoded_samples + l_wmp4delay_samples;
			MP4Duration dur = samples_left > l_wmp4frameSize ? l_wmp4frameSize : samples_left;
			MP4Duration ofs = l_wmp4encoded_samples > 0 ? 0 : l_wmp4delay_samples;

			MP4WriteSample(g_MP4hFile, l_wmp4track, l_wmp4bitbuf, bytesToFile, dur, ofs, 1);

			l_wmp4encoded_samples += dur;
		} // if(bytesToFile > 0)
	}
	*/

	// final calls with 0 bytes
	if(l_wmp4hEncoder && l_wmp4bitbuf)
	{
		i = 64; // arbitrary
		while(i-- > 0)
		{
			if((bytesToFile = faacEncEncode(l_wmp4hEncoder, &dummy, 0, l_wmp4bitbuf, l_wmp4maxBytesOutput)) > 0)
			{
				u_int64_t samples_left = l_wmp4total_samples - l_wmp4encoded_samples + l_wmp4delay_samples;
				MP4Duration dur = samples_left > l_wmp4frameSize ? l_wmp4frameSize : samples_left;
				MP4Duration ofs = l_wmp4encoded_samples > 0 ? 0 : l_wmp4delay_samples;
				MP4WriteSample(g_MP4hFile, l_wmp4track, l_wmp4bitbuf, bytesToFile, dur, ofs, 1);
			}
			
			if(bytesToFile <= 0)
				break;
		}
	}
	
	// clean up
	MP4Close(g_MP4hFile);
	g_tempfilefp = DUMMYOUTFILEP;
		
	if(l_wmp4hEncoder)
	{
		faacEncClose(l_wmp4hEncoder);
		l_wmp4hEncoder = NULL;
	}
	
	if(l_wmp4pcmbuf)
	{
		MAFREE(l_wmp4pcmbuf);
		l_wmp4pcmbuf = NULL;
	}
		
	if(l_wmp4bitbuf)
	{
		MAFREE(l_wmp4bitbuf);
		l_wmp4bitbuf = NULL;
	}
		
	return 0;

} // CloseWriteMp4


/////////////
// more from faac/frontend
int *mp4mkChanMap(int channels, int center, int lf)
{
	int *map;
	int inpos;
	int outpos;

	if(!center && !lf)
		return NULL;

	if(channels < 3)
		return NULL;

	if(lf > 0)
		lf--;
	else
		lf = channels - 1; // default MP4 position

	if(center > 0)
		center--;
	else
		center = 0; // default MP4 position

	map = (int *)MAMALLOC(channels * sizeof(map[0]));
	memset(map, 0, channels * sizeof(map[0]));

	outpos = 0;
	if((center >= 0) && (center < channels))
		map[outpos++] = center;

	inpos = 0;
	for(; outpos < (channels - 1); inpos++)
	{
		if(inpos == center)
			continue;
		if(inpos == lf)
			continue;

		map[outpos++] = inpos;
	}
	if(outpos < channels)
	{
		if((lf >= 0) && (lf < channels))
			map[outpos] = lf;
		else
			map[outpos] = inpos;
	}

	return map;
}

#ifdef __cplusplus
}
#endif
#else
 #ifdef __MAWIN__
  #pragma warning( disable : 4206 ) // nonstandard extension used : translation unit is empty
 #endif
#endif

// eof
