// maWriteWav.c

/*
 * MoreAmp
 * Copyright (C) 2004-2014 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

// extracts from madplay

#include "maConfig.h" // included by all moreamp files

#ifdef ENCODEWAV // or #ifdef ENCODERAW

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#include <errno.h>

#ifdef __MAOSX__
 #include <sys/stat.h>
#endif

#include "maTypes.h"
#include "maDefs.h"
#include "maWriteWav.h"
#include "maPlayer.h"
#include "maUtils.h"

#if !defined(WAVE_FORMAT_PCM)
  #define WAVE_FORMAT_PCM 0x0001
#endif

static unsigned long riff_len, data_len;
static long data_chunk;

#define UNKNOWN_LENGTH "\xff\xff\xff\xff"

int patch_int32w(long address, unsigned long num);
void int16w(unsigned char *ptr, unsigned int num);
void int32w(unsigned char *ptr, unsigned long num);

int OpenWriteWav(int raw)
{
	unsigned int block_al, bitdepth;
	unsigned long bytes_ps;
	unsigned char chunk[8 + 18];
	machar mastr[MAXLOGLINELEN];

	if(raw)
		return 0; // no header

	// RIFF header and (WAVE) data type identifier
	if(fwrite("RIFF" UNKNOWN_LENGTH "WAVE", 4 + 4 + 4, 1, g_tempfilefp) != 1)
	{
		masprintf(mastr, MAXLOGLINELEN, L"OpenWriteWav fwrite '%ls'", g_tempfile);
		maLog(mastr);
		return 1;
	}

	riff_len = 4;
	data_chunk = -1;

	// Format chunk
	bitdepth = MAOUTBITS;
	block_al = MAOUTCHANNELS * ((bitdepth + 7) / 8);
	bytes_ps = g_state.insamplerate * block_al;

	memcpy(&chunk[0], (unsigned char *)"fmt ", 4);	// chunkID
	int32w(&chunk[4], 16);							// chunkSize

	int16w(&chunk[8], WAVE_FORMAT_PCM);				// wFormatTag
	int16w(&chunk[10], MAOUTCHANNELS);				// wChannels
	int32w(&chunk[12], g_state.insamplerate);		// dwSamplesPerSec
	int32w(&chunk[16], bytes_ps);					// dwAvgBytesPerSec
	int16w(&chunk[20], block_al);					// wBlockAlign

	// PCM-format-specific
	int16w(&chunk[22], bitdepth);					// wBitsPerSample

	if(fwrite(chunk, 8 + 16, 1, g_tempfilefp) != 1)
	{
		masprintf(mastr, MAXLOGLINELEN, L"OpenWriteWav fwrite '%ls'", g_tempfile);
		maLog(mastr);
		return 1;
	}

	riff_len += 8 + 16;

	// Data chunk
	data_chunk = ftell(g_tempfilefp);

	if(fwrite("data" UNKNOWN_LENGTH, 8 + 0, 1, g_tempfilefp) != 1)
	{
		masprintf(mastr, MAXLOGLINELEN, L"OpenWriteWav fwrite '%ls'", g_tempfile);
		maLog(mastr);
		return 1;
	}

	riff_len += 8 + 0;
	data_len = 0;

	return 0;

} // OpenWriteWav


int WriteWav(void* pcmblock, unsigned long len)
{
	unsigned char *data = (unsigned char *)pcmblock;
	machar mastr[MAXLOGLINELEN];
	
#ifdef WORDS_BIGENDIAN
	// wav is little-endian, pcmblock is big-endian
	unsigned char d;
	unsigned int i;
	for(i = 0; i < len; i += 2)
	{
		d = data[i];
		data[i] = data[i + 1];
		data[i + 1] = d;	
	}
#endif

	if(fwrite(data, len, 1, g_tempfilefp) != 1)
	{
		masprintf(mastr, MAXLOGLINELEN, L"WriteWav fwrite error");
		maLog(mastr);
		return 1;
	}

	data_len += len;
	riff_len += len;
	
	return 0;

} // WriteWav


int CloseWriteWav()
{
	int result = 0;
	machar mastr[MAXLOGLINELEN];

	if(data_len & 1)
	{
		if(fputc(0, g_tempfilefp) == EOF && result == 0)
		{
			masprintf(mastr, MAXLOGLINELEN, L"CloseWriteWav fputc '%ls'", g_tempfile);
			maLog(mastr);
			return 1;
		}
		
		++riff_len;
	}

	if(result == 0 &&
			data_chunk != -1 && patch_int32w(data_chunk + 4, data_len) == -1)
		result = 1;

	if(result == 0)
		patch_int32w(4, riff_len);

	return 0;

} // CloseWriteWav


void int16w(unsigned char *ptr, unsigned int num)
{
	ptr[0] = (unsigned char)(num >> 0);
	ptr[1] = (unsigned char)(num >> 8);
}

void int32w(unsigned char *ptr, unsigned long num)
{
	ptr[0] = (unsigned char)(num >>	0);
	ptr[1] = (unsigned char)(num >>	8);
	ptr[2] = (unsigned char)(num >> 16);
	ptr[3] = (unsigned char)(num >> 24);
}

int patch_int32w(long address, unsigned long num)
{
	machar mastr[MAXLOGLINELEN];
	
	unsigned char dword[4];

	if(fseek(g_tempfilefp, address, SEEK_SET) != 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"patch_int32w fseek '%ls'", g_tempfile);
		maLog(mastr);
		return 1;
	}

	int32w(dword, num);

	if(fwrite(dword, sizeof(dword), 1, g_tempfilefp) != 1)
	{
		masprintf(mastr, MAXLOGLINELEN, L"patch_int32w fwrite '%ls'", g_tempfile);
		maLog(mastr);
		return 1;
	}

	if(fseek(g_tempfilefp, 0, SEEK_END) != 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"patch_int32w fseek '%ls'", g_tempfile);
		maLog(mastr);
		return 1;
	}

	return 0;

} // patch_int32w

#ifdef __cplusplus
}
#endif

#endif

// eof
