// maGUIwx.cpp

/*
 * MoreAmp
 * Copyright (C) 2004-2014 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
#include "maConfig.h" // included by all moreamp files

// File maGUInone.cpp contains the code to run without a gui and stubs for gui functions.
// File maGUIOSX.cpp  contains the code to run with the osx (Carbon, Interface Builder) gui.

#ifdef MAGUIWX

// #define MATRACEGUIWX

// This MAGUIWX section, and file maControls.cpp,
// contain the GUI (wxWidget) dependent code.

// Platform:	gui:	defined:
// Mac OS X		WXMAC	MAGUIWX __WXMAC__
// Mac OS X		WXGTK	MAGUIWX __WXGTK__
// Mac OS X		WXX11	MAGUIWX __WXX11__
// Win32		WXMSW	MAGUIWX __WXMSW__
// unix/linux	WXGTK	MAGUIWX __WXGTK__
// unix/linux	WXX11	MAGUIWX __WXX11__

// On unix and Mac, for a list of your wx configurations run `wx-config --list` :
// and you get a printout like this :
//
//    Default config is mac-unicode-release-static-2.8
//
//  Default config will be used for output
//
//  Alternate matches:
//    gtk-ansi-release-static-2.6
//    gtk-ansi-release-static-2.8
//    gtk2-ansi-release-static-2.6
//    gtk2-ansi-release-static-2.8
//    gtk2-unicode-release-static-2.8
//    mac-ansi-debug-static-2.6
//    mac-ansi-debug-static-2.8
//    mac-ansi-release-static-2.6
//    mac-ansi-release-static-2.8
//    mac-unicode-debug-static-2.8
//    x11univ-ansi-release-static-2.8
//
//  Also available in /usr/local:
//    osx_carbon-ansi-static-2.9

// To build moreamp on an osx or unix/linux system with multiple wxWidgets installations :

// `./configure` with no options will build moreamp with the wx 'Default config'.

// The wx "Default config" is the one that link /usr/local/bin/wx-config points to, e.g.
// command `ls -l /usr/local/bin/wx-config` prints :
// /usr/local/bin/wx-config -> /usr/local/lib/wx/config/mac-unicode-release-static-2.8

// You can select which wx config is the "Default config" by changing the `/usr/local/bin/wx-config` link :
// `ln -sfv /usr/local/lib/wx/config/gtk2-unicode-release-static-2.8 /usr/local/bin/wx-config`
// Or by rerunning `make install` from the wx build dir (osx-build, gtk2-build...) that you want to be the default.
// Or you can specify which config to use on the moreamp `./configure` line, using these three options :
// --with-wxtoolkit=[mac gtk gtk2 wxselected], defaults to "wxselected" if omitted.
// --with-unicode[yes no wxselected], defaults to "wxselected" if omitted.
// --with-wxversion=[2.6 2.8 wxselected], defaults to "wxselected" if omitted.
// "wxselected" means use the value from the wx 'Default config'.

// e.g. `./configure --with-wxtoolkit=gtk2 --with-unicode --with-wxversion=2.8
// This command selects the wx config "gtk2-unicode-release-static-2.8"
// This command does not alter the `/usr/local/bin/wx-config` link.

// Note that `./configure` has no option to select debug/release or shared/static.
// If you need to build with wx debug or with shared libs, change the wx Default config
// by changing the /usr/local/bin/wx-config link.

// Also note that the `./configure` option `--with-gui=wx` is the default.

// For a list of the moreamp `./configure` options, run `./configure --help`.

// On Windows systems, the VC++ Express 2008 project looks for a wxWidgets installation
// at C:\wx. So if you have built wx at C:\wxWidgets-2.8.12, you need to rename the it
// to 'C:\wx'.

#include <string.h>
#include <errno.h>
#include <stdio.h>

#ifdef __MAOSX__
 #include <unistd.h>
 #include <sys/stat.h>
#endif

#ifdef __MAUNIX__
 #include <unistd.h>
 #include <stdlib.h>
 #include <sys/stat.h>
 #include "wx/wx.h"
 #pragma GCC diagnostic ignored "-Wwrite-strings" // for (char *) <> (const char *) warnings
#else
 #include "wx/wxprec.h"
 #ifndef WX_PRECOMP
  #include "wx/wx.h"
 #endif
#endif

#if defined(__MAOSX__) && defined(__WXMAC__)
 // typically defined in /usr/local/lib/wx/include/*-*-*-*-?.*/wx/setup.h
 // e.g. /usr/local/lib/wx/include/mac-unicode-release-static-2.8/wx/setup.h
 #if !defined(wxUSE_SYSTEM_OPTIONS)
  #warning wxUSE_SYSTEM_OPTIONS not defined in /usr/local/lib/wx/include/*-*-*-*-2.*/wx/setup.h
 #endif

 #if (MAGUIWX == 6) && (wxMINOR_VERSION == 6) // wx-2.6
  #if !defined(wxMAC_TEXTCONTROL_USE_MLTE)
   #define wxMAC_TEXTCONTROL_USE_MLTE "mac.textcontrol-use-mlte"
  #endif
 #else // wx-2.8+
  #if defined(wxMAC_TEXTCONTROL_USE_MLTE)
   // #warning wxMAC_TEXTCONTROL_USE_MLTE was defined // so what
  #endif
 #endif
#endif

#if defined(__MAOSX__) && defined(__WXMAC__) && (MAGUIWX >= 8) // wx-2.8+
 #include "wx/sysopt.h" // for wxSystemOptions::SetOption
#endif

#if defined(__MAOSX__)
 #include "MacWindows.h"
#endif

#ifdef __MAWIN__
 #include <SHLOBJ.H>
 #include <SHLWAPI.H>
#endif

#include "wx/image.h"
#include "wx/filename.h"
#include "wx/utils.h" // for wxGetHomeDir
#include "wx/file.h"
#include "wx/dir.h"
#include "wx/colour.h"
#include "wx/event.h"
#include "wx/process.h"
#include "wx/spinbutt.h"
#include "wx/listctrl.h"
#include "wx/settings.h" // for fonts

#define MADRAGANDDROP
#ifdef MADRAGANDDROP
 #if wxUSE_DRAG_AND_DROP
  #include "wx/dnd.h"
 #else
  #warning no wxUSE_DRAG_AND_DROP
  #undef MADRAGANDDROP
 #endif
#endif

#if !defined(MASETFONT)
 #if defined(MASMALLERFONT)
  #include "wx/font.h"
  extern wxFont g_mafont;
  #define MASETFONT(f) f->SetFont(g_mafont)
 #else
  #define MASETFONT(f)
 #endif
#endif

/*
 // #define MAINTL
 #if defined(MAINTL)
  #if !defined(wxUSE_INTL)
   #error MAINTL defined and wxUSE_INTL not defined
  #endif
  #include <locale.h>
  #include "wx/intl.h" // is already included by wx.h
 #endif
*/

// Application includes
#include "maTypes.h"
#include "maDefs.h"
#include "maPlayer.h"
#include "maEqualizer.h"
#include "maEffects.h"
#include "maControls.h"
#include "maGUI.h"
#include "maMemFile.h"
#include "maSongList.h"
#include "maUtils.h"
#include "maTextView.h"

#ifndef MANOCDA
 #include "maCda.h"
 #include "maPlayCda.h"
#endif

#if defined(__MAOSX__)
 #include "maAudioPA.h"
#endif

#ifdef __MAWIN__
 #include "maAudioPA.h"
#endif

#ifdef __MAUNIX__
 #include "maAudioPA.h"
#endif

#define XBASE 2
#define YBASE 2
#define MABUTTONW 50
#define MABUTTONH 48
#define MABUTTONSIZE wxSize(MABUTTONW, MABUTTONH)

#if defined(__WXMAC__)
 #define MENUBARH 0		// mac has a menu bar at the top of the screen instead of in each window
#endif

#ifdef __MAWIN__		// MSW has menus in each window
 #define MENUBARH 19
#endif

#if defined(__WXX11__)  // X11 has menus in each window
 #define MENUBARH 19
#endif

#if defined(__WXGTK__)  // GTK has menus in each window
 #if defined(__MACUNIX__)
  #define MENUBARH 19
 #else
  #define MENUBARH 23
 #endif
#endif

#if defined(MAUSEMABUTTON)
 #define MABUTTON maButton
#else
 #define MABUTTON wxButton
#endif

#if defined(MAUSEMACHECKBOX)
 #define MACHECKBOX maCheckBox
#else
 #define MACHECKBOX wxCheckBox // have large labels on mac...
#endif

#if defined(MAUSEMASLIDER)
 #define MASLIDER maSlider
#else
 #define MASLIDER wxSlider // vertical sliders are upside down...
#endif


// wx versions and platforms don't all behave the same
#if defined(MAKEYBOARD)
 #ifdef __cplusplus
  extern "C"
  {
 #endif
 void maTrace(machar *str);
 #ifdef __cplusplus
  }
 #endif

 #if defined(USE_WXGTK)
  // #define MANOWXPANEL // ?
 #endif
#endif // MAKEYBOARD

#if defined(MANOWXPANEL)
 #define MPANEL this
#else
 #define MPANEL m_panel
#endif

void *l_theMenu[MENULAST + 1];
#if defined(__MAWIN__) && defined(MAMOUSEWHEEL)
	MACHECKBOX *l_ChkAltSize;
#endif

#if !defined(NOTEXTVIEWER)
 class MyViewerTextCtrl;
 class MyFindTextCtrl;
#endif

class MyApp : public wxApp
{
public:
/*
#if defined(MAINTL)
	void InitLang(const wxString &lang); 
#endif
*/
	bool OnInit();
#if defined(MAKEYBOARD)
	void maOnChar(wxKeyEvent &event);
	void maOnClick(wxMouseEvent &event);
#endif
#if defined(MAMOUSEWHEEL)
	void OnMouseWheel(wxMouseEvent &event);
#endif
	void OnGUICloseRequest(wxCloseEvent &event);
	void OnGUISized(wxSizeEvent &event);
	void OnGUIMoved(wxMoveEvent &event);
/*
#if defined(MAINTL)
	wxLocale *mLocale;
#endif
*/
};

DECLARE_APP(MyApp)


/*
class MyFrameSplash : public wxFrame
{
public:
	MyFrameSplash(wxWindowID id, const wxString& title, const wxPoint& pos = wxDefaultPosition,
		const wxSize& size = wxDefaultSize, long style = wxDEFAULT_FRAME_STYLE);
	
	virtual ~MyFrameSplash();

	wxStaticText *m_splashTextCtrl;

private:
	wxPanel *m_panel;

	// DECLARE_EVENT_TABLE()

}; // class MyFrameSplash
*/

class MyFrameApp : public wxFrame
{	
public:
	MyFrameApp(wxWindowID id, const wxString& title, const wxPoint& pos = wxDefaultPosition,
		const wxSize& size = wxDefaultSize, long style = wxDEFAULT_FRAME_STYLE);

	virtual ~MyFrameApp();

	void OnGUICheckAltSize(wxCommandEvent &event);
	void OnGUICheckRandom(wxCommandEvent &event);
	void OnGUICheckReptPlayed(wxCommandEvent &event);
	void OnGUICheckLoopLoaded(wxCommandEvent &event);
	void OnGUICheckLoopList(wxCommandEvent &event);
	void OnGUICheckEqWindow(wxCommandEvent &event);
	void OnGUICheckListWindow(wxCommandEvent &event);
	void OnGUICheckBoxBeg(wxCommandEvent &event);
	void OnGUICheckBoxEnd(wxCommandEvent &event);
	void OnGUICheckMonoLeft(wxCommandEvent &event);
	void OnGUICheckMonoRight(wxCommandEvent &event);
	// void OnGUICheckUseM3u(wxCommandEvent &event);
	void OnGUICheckSinglePlay(wxCommandEvent &event);
	void OnGUICheckEqualizer(wxCommandEvent &event);
	void OnGUICheckEffects(wxCommandEvent &event);
	void OnGUICheckLimiter(wxCommandEvent &event);
	// void OnGUICheckEqExtra(wxCommandEvent &event);
	void OnGUICheckTitleInfo(wxCommandEvent &event);
	void OnGUICheckHostInEnable(wxCommandEvent &event);

	void OnGUICheckInvertRight(wxCommandEvent &event);
	// void OnGUICheckEq31Bands(wxCommandEvent &event);
	void OnGUICheckReptSong(wxCommandEvent &event);
	void OnGUICheckMonoDiff(wxCommandEvent &event);
	void OnGUICheckBalance(wxCommandEvent &event);

	void OnGUIButtonQuit(wxCommandEvent &event);
	void OnGUIButtonStop(wxCommandEvent &event);
	void OnGUIButtonPrevious(wxCommandEvent &event);
	void OnGUIButtonPlay(wxCommandEvent &event);
	void OnGUIButtonPause(wxCommandEvent &event);
	void OnGUIButtonNext(wxCommandEvent &event);
	void OnGUIButtonCatalog(wxCommandEvent &event);
	
	void OnGUISliderVolumeL(wxCommandEvent &event);
	void OnGUISliderVolumeR(wxCommandEvent &event);
	void OnGUISliderSeparation(wxCommandEvent &event);
	void OnGUISliderPreamp(wxCommandEvent &event);
	void OnGUISliderPos(wxCommandEvent &event);
	
	void OnGUIPopupOutput(wxCommandEvent &event);
	void OnGUIPopupBuffer(wxCommandEvent &event);
	void OnGUIPopupAudioHostIn(wxCommandEvent &event);
	void OnGUIPopupAudioHostOut(wxCommandEvent &event);
	void OnGUIPopupAudioOutput(wxCommandEvent &event);
	void OnGUIPopupAudioInput(wxCommandEvent &event);

	void OnGUISpinUpMem(wxSpinEvent &event);
	void OnGUISpinUpPreamp(wxSpinEvent &event);
	void OnGUISpinUpSep(wxSpinEvent &event);
	void OnGUISpinUpVolR(wxSpinEvent &event);
	void OnGUISpinUpVolL(wxSpinEvent &event);
	void OnGUISpinUpBeg(wxSpinEvent &event);
	void OnGUISpinUpEnd(wxSpinEvent &event);

	void OnGUISpinDownBuffer(wxSpinEvent &event);
	void OnGUISpinDownPreamp(wxSpinEvent &event);
	void OnGUISpinDownSep(wxSpinEvent &event);
	void OnGUISpinDownVolR(wxSpinEvent &event);
	void OnGUISpinDownVolL(wxSpinEvent &event);
	void OnGUISpinDownBeg(wxSpinEvent &event);
	void OnGUISpinDownEnd(wxSpinEvent &event);

	void MakeButtons();
	
#if defined(MAUSEMABUTTON)
	MABUTTON *MakeButton(char const **foreground,
						char const **disabled,
						char const **foremask, int id, bool processdownevents);
#else
	MABUTTON *MakeButton(int id, machar *label);
#endif
		
	void MakeMenus();
	void MakeSubMenusRecent();
	void OnAbout(wxCommandEvent &event);
	void OnGUIMenuQuit(wxCommandEvent &event);
	void OnGUIMenuOpenList(wxCommandEvent &event);
	void OnGUIMenuMakeList(wxCommandEvent &event);
	void OnGUIMenuSaveListAs(wxCommandEvent &event);
	void OnGUIMenuDeleteList(wxCommandEvent &event);
	void OnGUIMenuPlayFile(wxCommandEvent &event);
	void OnGUIMenuOpenRecent(wxCommandEvent &event);
	void OnGUIMenuSetVDisk(wxCommandEvent &event);
	void OnGUIMenuMultipleInst(wxCommandEvent &event);
#if defined(MAKEYBOARD)
	void OnGUIMenuKeyboard(wxCommandEvent &event);
 #if !defined(NOTEXTVIEWER)
	void OnGUIMenuEditKeyBindings(wxCommandEvent &event);
	void OnGUIMenuBlankKeyBindings(wxCommandEvent &event);
	void OnGUIMenuFactoryKeyBindings(wxCommandEvent &event);
 #endif
#endif
	void OnGUIMenuRescanAudioDevices(wxCommandEvent &event);
#if defined(MAMOUSEWHEEL)
	void OnGUIMenuMouseWheel(wxCommandEvent &event);
#endif
	void OnGUIMenuAdvanced(wxCommandEvent &event);
#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
	void OnGUIMenuSetsAudioCD(wxCommandEvent &event);
#endif
	void OnGUIMenuSetsFactory(wxCommandEvent &event);
	void OnGUIMenuSetsDefault(wxCommandEvent &event);
	void OnGUIMenuSetsSaved(wxCommandEvent &event);
	void OnGUIMenuSetsCopy(wxCommandEvent &event);
	void OnGUIMenuStop(wxCommandEvent &event);
	void OnGUIMenuPause(wxCommandEvent &event);
	void OnGUIMenuPrevious(wxCommandEvent &event);
	void OnGUIMenuPlay(wxCommandEvent &event);
	void OnGUIMenuNext(wxCommandEvent &event);
	void OnGUIMenuPlayCD(wxCommandEvent &event);
	void OnGUIMenuTitleInfo(wxCommandEvent &event);
	void OnGUIMenuRandom(wxCommandEvent &event);
	void OnGUIMenuReptPlayed(wxCommandEvent &event);
	void OnGUIMenuLoopLoaded(wxCommandEvent &event);
	void OnGUIMenuLoopList(wxCommandEvent &event);
	void OnGUIMenuAutoStart(wxCommandEvent &event);
	void OnGUIMenuErrLogging(wxCommandEvent &event);
	void OnGUIMenuSinglePlay(wxCommandEvent &event);
	void OnGUIMenuInMonitor(wxCommandEvent &event);
	void OnGUIMenuEqualizer(wxCommandEvent &event);
	void OnGUIMenuEffects(wxCommandEvent &event);
	void OnGUIMenuEqExtra(wxCommandEvent &event);
	void OnGUIMenuMono(wxCommandEvent &event);
	void OnGUIMenuInvertRight(wxCommandEvent &event);
	void OnGUIMenuEq31Bands(wxCommandEvent &event);
	void OnGUIMenuSetPlayer(wxCommandEvent &event);
	void OnGUIMenuSaveSettings(wxCommandEvent &event);
	void OnGUIMenuSaveSettingsDefault(wxCommandEvent &event);
	void OnGUIMenuZeroList(wxCommandEvent &event);
	void OnGUIMenuEjectCD(wxCommandEvent &event);
	void OnGUIMenuAudioIn(wxCommandEvent &event);
	void OnGUIMenuMakeCatalog(wxCommandEvent &event);
#if !defined(NOTEXTVIEWER)
	void OnGUIMenuReadme(wxCommandEvent &event);
	void OnGUIMenuViewCatalog(wxCommandEvent &event);
	void OnGUIMenuViewPlaylist(wxCommandEvent &event);
	void OnGUIMenuViewLog(wxCommandEvent &event);
#endif
	void OnGUIMenuClearLog(wxCommandEvent &event);
	void OnGUIMenuShowList(wxCommandEvent &event);
	void OnGUIMenuShowEq(wxCommandEvent &event);
	void OnGUIMenuShowEffects(wxCommandEvent &event);
	void OnGUIMenuShowFileOut(wxCommandEvent &event);
	void OnGUIMenuAltSize(wxCommandEvent &event);
	void OnGUIMenuFactorySize(wxCommandEvent &event);
	void OnGUIMenuSavedSize(wxCommandEvent &event);
	void OnGUIMenuDefaultSize(wxCommandEvent &event);

	void OnGUICloseRequest(wxCloseEvent &event);

	wxTimer *m_timerchangeplaylist;
	wxTimer *m_timerplayplaylist;
	void OnTimerChangePlaylist(wxTimerEvent &event);
	void OnTimerPlayPlaylist(wxTimerEvent &event);

	void OnTimerRunLoop(wxTimerEvent &event);
	wxTimer *m_timerrunloop;

	void OnTimerPlay(wxTimerEvent &event);
	wxTimer *m_timerplay;

	// void OnTimerStop(wxTimerEvent &event);
	// wxTimer *m_timerstop;

#if !defined(MANOWXPANEL)
	wxPanel *m_panel;
#endif
	
	MASLIDER *m_sliderPos;
	wxComboBox *m_ComboOutput;
	wxComboBox *m_ComboBuffer;
	wxBoxSizer *m_sizerFrame;
	wxBoxSizer *m_sizerTop;

	wxComboBox *m_ComboAudioOutput;
	wxComboBox *m_ComboAudioInput;
	wxComboBox *m_ComboAudioHostIn;
	wxComboBox *m_ComboAudioHostOut;

	wxImage *m_upPattern;
	wxImage *m_downPattern;
	wxImage *m_hilitePattern;

	int m_ButtonPos;
	int m_buttonTop;

	DECLARE_EVENT_TABLE()
	
}; // class MyFrameApp


class MyFrameList : public wxFrame
{
public:
	MyFrameList(wxWindowID id, const wxString& title, const wxPoint& pos = wxDefaultPosition,
		const wxSize& size = wxDefaultSize, long style = wxDEFAULT_FRAME_STYLE);

	virtual ~MyFrameList();

	void OnGUIButtonLoaded(wxCommandEvent &event);
	void OnGUIButtonList(wxCommandEvent &event);
	void OnGUIButtonInfo(wxCommandEvent &event);
	void OnGUIButtonSong(wxCommandEvent &event);
	void OnSize(wxSizeEvent &event);
	void OnGUICheckShowIndex(wxCommandEvent &event);	
	void OnGUICheckShowDir(wxCommandEvent &event);	
	void OnGUICheckShowPath(wxCommandEvent &event);
	void OnGUICheckAddMode(wxCommandEvent &event);
	void OnGUIListSelected(wxListEvent &event);	
	void OnGUIListActivated(wxListEvent &event);
	void GetColumn0Width();
	void OnGUIColumnEndDrag(wxListEvent &event);
	void SetColumn0Width(int width);
	
	wxListCtrl *m_list;
	
	void MakeButtonsList();

#if defined(MAKEYBOARD)
	wxCheckBox *m_cbdummy;
#endif
	
#if defined(MAUSEMABUTTON)
	MABUTTON *MakeButtonList(char const **foreground,
						char const **disabled,
						char const **foremask, int id, bool processdownevents);		
#else
	MABUTTON *MakeButtonList(int id, machar *label);
#endif

#if !defined(MANOWXPANEL)
	wxPanel *m_panel;
#endif

private:
	wxImage *m_upPattern;
	wxImage *m_downPattern;
	wxImage *m_hilitePattern;
	int m_ButtonPos;
	int m_buttonTop;
	
	DECLARE_EVENT_TABLE()
	
}; // class MyFrameList


class MyFrameEq : public wxFrame
{
public:
	MyFrameEq( wxWindowID id, const wxString& title, const wxPoint& pos = wxDefaultPosition,
		const wxSize& size = wxDefaultSize, long style = wxDEFAULT_FRAME_STYLE);

	virtual ~MyFrameEq();

	void OnGUICheckEqZoom(wxCommandEvent &event);
	void OnGUICheckEqLink(wxCommandEvent &event);
	void OnGUICheckEqualizerB(wxCommandEvent &event);
	void OnGUICheckEq31BandsB(wxCommandEvent &event);
	void OnGUICheckEqExtraB(wxCommandEvent &event);
	void OnGUICheckSeparateLR(wxCommandEvent &event);
	void OnGUIButtonEqMinus(wxCommandEvent &event);
	void OnGUIButtonEqPlus(wxCommandEvent &event);
	void OnGUIButtonEqMid(wxCommandEvent &event);
	void OnGUIButtonEqCopy(wxCommandEvent &event);
	void OnGUIButtonEqPaste(wxCommandEvent &event);
	void OnGUIButtonEqRestore(wxCommandEvent &event);
	void OnGUIButtonEqShiftL(wxCommandEvent &event);
	void OnGUIButtonEqShiftR(wxCommandEvent &event);
	void OnGUIButtonEqInvert(wxCommandEvent &event);
	void OnGUIPopupEqPreset(wxCommandEvent &event);
	void OnGUIEditTextEq(wxCommandEvent &event);
	void OnGUISliderEq(wxCommandEvent &event);
	void OnGUISpinUpEqOffset(wxSpinEvent &event);
	void OnGUISpinUpEq(wxSpinEvent &event);
	void OnGUISpinDownEqOffset(wxSpinEvent &event);
	void OnGUISpinDownEq(wxSpinEvent &event);
	void MakeButtonsEq();
	
#if defined(MAUSEMABUTTON)
	MABUTTON *MakeButtonEq(char const **foreground,
						char const **disabled,
						char const **foremask, int id, bool processdownevents);		
#else
	MABUTTON *MakeButtonEq(int id, machar *label);
#endif
	
	wxTextCtrl *m_edittexteq;
#if defined(MAKEYBOARD)
	wxTextCtrl *m_dummytexteq;
#endif

#if !defined(MANOWXPANEL)
	wxPanel *m_panel;
#endif

private:
	wxComboBox *m_ComboPreset;
	int m_numslidersEq;

	wxImage *m_upPattern;
	wxImage *m_downPattern;
	wxImage *m_hilitePattern;
	int m_ButtonPos;
	int m_buttonTop;
	
	DECLARE_EVENT_TABLE()

}; // class MyFrameEq


class MyFrameEq2 : public wxFrame
{
public:
	MyFrameEq2( wxWindowID id, const wxString& title, const wxPoint& pos = wxDefaultPosition,
		const wxSize& size = wxDefaultSize, long style = wxDEFAULT_FRAME_STYLE);

	virtual ~MyFrameEq2();

	void OnGUIButtonEqMid(wxCommandEvent &event);
	void OnGUIButtonEqCopy(wxCommandEvent &event);
	void OnGUIButtonEqPaste(wxCommandEvent &event);
	void OnGUIButtonEqRestore(wxCommandEvent &event);
	void OnGUISliderEq(wxCommandEvent &event);
	void OnGUISpinUpEqOffset(wxSpinEvent &event);
	void OnGUISpinUpEq(wxSpinEvent &event);
	void OnGUISpinDownEqOffset(wxSpinEvent &event);
	void OnGUISpinDownEq(wxSpinEvent &event);
	void MakeButtonsEq();
	
#if defined(MAUSEMABUTTON)
	MABUTTON *MakeButtonEq(char const **foreground,
	 					char const **disabled,
	 					char const **foremask, int id, bool processdownevents);		
#else
	MABUTTON *MakeButtonEq(int id, machar *label);
#endif
	
 #if defined(MAKEYBOARD)
	wxTextCtrl *m_dummytexteq;
 #endif

#if !defined(MANOWXPANEL)
	wxPanel *m_panel;
#endif

private:
	int m_numslidersEq;

	wxImage *m_upPattern;
	wxImage *m_downPattern;
	wxImage *m_hilitePattern;
	
	int m_ButtonPos;
	int m_buttonTop;
	
	DECLARE_EVENT_TABLE()

}; // class MyFrameEq2


class MyFrameEffects : public wxFrame
{
public:
	MyFrameEffects( wxWindowID id, const wxString& title, const wxPoint& pos = wxDefaultPosition,
		const wxSize& size = wxDefaultSize, long style = wxDEFAULT_FRAME_STYLE);

	virtual ~MyFrameEffects();

	void OnGUISliderPitch(wxCommandEvent &event);
	void OnGUISliderPitch2(wxCommandEvent &event);
	void OnGUISliderTempo(wxCommandEvent &event);
	void OnGUISliderSpeed(wxCommandEvent &event);
	void OnGUISpinDownPitch(wxSpinEvent &event);
	void OnGUISpinDownPitch2(wxSpinEvent &event);
	void OnGUISpinDownTempo(wxSpinEvent &event);
	void OnGUISpinDownSpeed(wxSpinEvent &event);
	void OnGUISpinUpPitch(wxSpinEvent &event);
	void OnGUISpinUpPitch2(wxSpinEvent &event);
	void OnGUISpinUpTempo(wxSpinEvent &event);
	void OnGUISpinUpSpeed(wxSpinEvent &event);
	void OnGUICheckPitch(wxCommandEvent &event);
	void OnGUICheckTempo(wxCommandEvent &event);
	void OnGUICheckSpeed(wxCommandEvent &event);
	void OnGUICheckEffectsB(wxCommandEvent &event);
	void OnGUICheckSBSMS(wxCommandEvent &event);

	MACHECKBOX *m_checkbox;

private:
#if !defined(MANOWXPANEL)
	wxPanel *m_panel;
#endif

	DECLARE_EVENT_TABLE()

}; // class MyFrameEffects


#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
class MyFrameAudioCD : public wxFrame
{
public:
	MyFrameAudioCD( wxWindowID id, const wxString& title, const wxPoint& pos = wxDefaultPosition,
		const wxSize& size = wxDefaultSize, long style = wxDEFAULT_FRAME_STYLE);

	virtual ~MyFrameAudioCD();

	void OnGUIPopupAudioCD(wxCommandEvent &event);
	void OnGUIButtonAudioCDSet(wxCommandEvent &event);
	void OnGUIButtonAudioCDClear(wxCommandEvent &event);
	void OnGUIButtonAudioCDSave(wxCommandEvent &event);
	void OnGUICheckCdParanoia(wxCommandEvent &event);
	void OnGUIButtonClose(wxCommandEvent &event);

private:
	wxPanel *m_panel;
	wxComboBox *m_ComboAudioCD;

	DECLARE_EVENT_TABLE()

}; // class MyFrameAudioCD
#endif


class MyFrameFileOut : public wxFrame
{
public:
	MyFrameFileOut( wxWindowID id, const wxString& title, const wxPoint& pos = wxDefaultPosition,
		const wxSize& size = wxDefaultSize, long style = wxDEFAULT_FRAME_STYLE);

	virtual ~MyFrameFileOut();
	
	void OnGUIPopupBitRate(wxCommandEvent &event);
	void OnGUIPopupQuality(wxCommandEvent &event);
	void OnGUIPopupMode(wxCommandEvent &event);
	void OnGUIPopupSampleRate(wxCommandEvent &event);
	void OnGUIButtonFileOutDir(wxCommandEvent &event);
#if defined(ENCODEAAC)
	void OnGUICheckBoxADTS(wxCommandEvent &event);
#endif
#if defined(ENCODEMP4)
	void OnGUICheckBoxM4A(wxCommandEvent &event);
#endif

private:
	wxPanel *m_panel;	
	wxComboBox *m_ComboBitRate;
	wxComboBox *m_ComboQuality;
	wxComboBox *m_ComboMode;
	wxComboBox *m_ComboSampleRate;

	DECLARE_EVENT_TABLE()

}; // class MyFrameFileOut


class MyFrameAbout : public wxFrame
{
public:
	MyFrameAbout( wxWindowID id, const wxString& title, const wxPoint& pos = wxDefaultPosition,
		const wxSize& size = wxDefaultSize, long style = wxDEFAULT_FRAME_STYLE);
	
	virtual ~MyFrameAbout();

	void OnGUIButtonClose(wxCommandEvent &event);

	wxTextCtrl *m_aboutTextCtrl;
	wxTextCtrl *m_urlTextCtrl;
	wxButton *m_closebutton;

private:
	int m_winw;
	int m_winy;
	wxPanel *m_panel;
	
	DECLARE_EVENT_TABLE()

}; // class MyFrameAbout


#if !defined(NOTEXTVIEWER)
 class MyFrameViewer : public wxFrame
 {
 public:
	MyFrameViewer(int windowid, const wxString& title, const wxPoint& pos = wxDefaultPosition,
		const wxSize& size = wxDefaultSize, long style = wxDEFAULT_FRAME_STYLE);
	
	virtual ~MyFrameViewer();
	
	void OnGUIButtonFind(wxCommandEvent &event);
	void OnGUIButtonVPlay(wxCommandEvent &event);
	void OnGUIButtonEditSave(wxCommandEvent &event);
	void OnTimerLeftUp(wxTimerEvent &event);
	int get_windowid();
	void OnGUIButtonClose(wxCommandEvent &event);

	MyViewerTextCtrl *m_viewerTextCtrl;
	MyViewerTextCtrl *get_viewerTextCtrl();
 #if defined(__WXMAC__)
	wxTimer *m_timerleftup;
 #endif
	MyFindTextCtrl *m_findTextCtrl;
	wxStaticText *m_foundStaticText;
	wxStaticText *m_playStaticText;
	wxButton *m_findbutton;
	wxButton *m_playbutton;
	wxButton *m_editbutton;
	wxButton *m_closebutton;

 private:
	int m_windowid;
	int m_winw;
	int m_winy;
	wxPanel *m_panel;
	
	DECLARE_EVENT_TABLE()

 }; // class MyFrameViewer


 class MyViewerTextCtrl : public wxTextCtrl
 {
 public:
	MyViewerTextCtrl(wxWindow *parent, wxWindowID id, const wxString &value,
						const wxPoint &pos, const wxSize &size, int style)
		: wxTextCtrl(parent, id, value, pos, size, style)
	{ }

	void OnGUIMouse(wxMouseEvent &event);
	
 private:

	DECLARE_EVENT_TABLE()
 };


 class MyFindTextCtrl : public wxTextCtrl
 {
 public:
	MyFindTextCtrl(wxWindow *parent, wxWindowID id, const wxString &value,
				const wxPoint &pos, const wxSize &size)
		: wxTextCtrl(parent, id, value, pos, size)
	{ }

	void OnGUIChar(wxKeyEvent &event);
	void OnGUIMouse(wxMouseEvent &event);
	
 private:

	DECLARE_EVENT_TABLE()
 };
#endif


class MyFramePromptNum : public wxFrame
{
public:
	MyFramePromptNum( wxWindowID id, const wxString& title, const wxPoint& pos = wxDefaultPosition,
		const wxSize& size = wxDefaultSize, long style = wxDEFAULT_FRAME_STYLE); // wxCAPTION | wxCLIP_CHILDREN);

	virtual ~MyFramePromptNum();

	void OnGUIButtonPromptNumOk(wxCommandEvent &event);
	void OnGUIButtonPromptNumCancel(wxCommandEvent &event);
	void OnGUIEditTextPromptNum(wxCommandEvent &event);
	
	wxStaticText *m_textpromptnum;

#if !defined(MANOWXPANEL)
	wxPanel *m_panel;
#endif

	wxTextCtrl *m_edittextpromptnum;

private:
	int m_winx;
	int m_winy;
	int m_winw;
	wxButton *m_buttonOk;
	wxButton *m_buttonCancel;
	
	DECLARE_EVENT_TABLE()

}; // class MyFramePromptNum


class MyFrameAdvanced : public wxFrame
{
public:
	MyFrameAdvanced( wxWindowID id, const wxString& title, const wxPoint& pos = wxDefaultPosition,
		const wxSize& size = wxDefaultSize, long style = wxDEFAULT_FRAME_STYLE);
		
	virtual ~MyFrameAdvanced();
	
	void OnGUICheckCloseReopen(wxCommandEvent &event);
	void OnGUICheckInputMonitor(wxCommandEvent &event);
	void OnGUICheckLogNotPlayable(wxCommandEvent &event);
	void OnGUICheckUseM3u(wxCommandEvent &event);
	void OnGUICheckExitAfter(wxCommandEvent &event);

	void OnGUIButtonClose(wxCommandEvent &event);
	
	wxStaticText *m_textclosereopen;
	wxButton *m_closebutton;

private:
	wxPanel *m_panel;

	DECLARE_EVENT_TABLE()

}; // class MyFrameAdvanced


#if defined(MADRAGANDDROP)
 class DnDFile: public wxFileDropTarget
 {
 public:
	DnDFile() {}

	virtual bool OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames);

 private:

 };


 bool DnDFile::OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames) // unicode
 {
	int i;
	int nitems;
	fschar **p;
	fschar **pitems;
	fschar playlist[MAX_PATH];
	wxArrayString s = filenames;

	(void)x; (void)y;

	if(g_state.scanning)
		return TRUE;

	g_state.scanning = 2;
	g_scanstop = 0;
	g_scanfiles = 0;

	s.Sort(); // s.Sort(CompareNoCaseFileName)
	nitems = (int)s.GetCount();

	if((pitems = (fschar **)MAMALLOC(nitems * sizeof(fschar *))) == NULL)
	{
		maLog(L"OnDropFiles MAMALLOC failed");
		return TRUE;
	}
		
	p = pitems;
	for(i = 0; i < nitems; i++, p++)
	{

		if((*p = (fschar *)MAMALLOC(MAX_PATH)) == NULL)
			maLog(L"OnDropFiles MAMALLOC failed");
		else
  #if defined(__MAWIN__)
		fsstrcpy(*p, (fschar *)s[i].fn_str());
  #else
		fsstrcpy(*p, (fschar *)(s)[i].fn_str().data()); // on mac this is UTF-8-MAC
  #endif
	}

	// write a new playlist
	playlist[0] = fsT('\0');
	if(maPlaylistMake(nitems, pitems, playlist)) // OnDropFiles
	{
		p = pitems;
		for(i = 0; i < nitems; i++, p++)
			if(*p)
				MAFREE(*p);
		MAFREE(pitems);
		if(!g_scanstop)
			maLog(L"OnDropFiles maPlaylistMake failed");
		return TRUE;
	}
	p = pitems;
	for(i = 0; i < nitems; i++, p++)
		if(*p)
			MAFREE(*p);
	MAFREE(pitems);

	PlayPlaylist(playlist);	

	return TRUE;

 } // OnDropFiles
#endif // MADRAGANDDROP


#if !defined(NOTEXTVIEWER)
 void MyViewerTextCtrl::OnGUIMouse(wxMouseEvent& ev)
 {
	int windowid;
	maviewer *v;
	
	if(ev.Moving() || ev.Dragging())
    {
		ev.Skip();
		return;
	}

	windowid = ((MyFrameViewer *)((this->GetParent())->GetParent()))->get_windowid();
	
	if((v = Viewer(windowid)) == 0)
	{
		maLog(L"OnGUIMouse Viewer failed");
		ev.Skip();
		return;
	}

	if(v->editable)
	{
		ev.Skip(); // don't handle anyhting here while editing (allow edit popup menu)
		return;
	}

 #if defined(__WXMSW__) || defined(__WXGTK__) || defined(__WXX11__)
	if(!(ev.RightDown() || ev.RightDClick())) // avoid copy/paste popup
		ev.Skip();
	
	if(ev.LeftUp())
		ViewerOnLeftClick(((MyFrameViewer *)((this->GetParent())->GetParent()))->get_windowid());
	else if(ev.RightUp())
		ViewerOnRightClick(((MyFrameViewer *)((this->GetParent())->GetParent()))->get_windowid());

 #elif defined(__WXMAC__) // editor eats LeftUp events
	if(!(ev.RightDown() || ev.RightDClick())) // avoid copy/paste popup NOT ON 10.6 ?
		ev.Skip();
		
	if(ev.LeftDown()) // this is also the first of a double click // we don't get the new insertion point yet
		((MyFrameViewer *)g_w[((MyFrameViewer *)((this->GetParent())->GetParent()))->get_windowid()])->
						m_timerleftup->Start(200, TRUE);
	/*
	else if(ev.LeftUp()) // we don't get LeftUp if we've done a Skip on LeftDown (which we did)
	{
	}
	*/
	else if(ev.LeftDClick()) // restart the timer (started by LeftDown) so we have time to get a possible tripple-click selection
	{
		((MyFrameViewer *)g_w[((MyFrameViewer *)((this->GetParent())->GetParent()))->get_windowid()])->
						m_timerleftup->Start(200, TRUE);
	}
	else if(ev.RightUp())
		ViewerOnRightClick(((MyFrameViewer *)((this->GetParent())->GetParent()))->get_windowid());
 #else
  #error __WXMSW__ or __WXMAC__ or __WXGTK__ or __WXX11__ not defined
 #endif
 }


 void MyFindTextCtrl::OnGUIChar(wxKeyEvent &event)
 {
	wxCommandEvent dummy;
	int code = event.GetKeyCode();
	
	switch(code)
	{
		case 10:
		case 13:
			((MyFrameViewer *)((this->GetParent())->GetParent()))->OnGUIButtonFind(dummy);
		break;
		case 27: // escape
			WindowActivate(WINDOWAPP);
		break;
		default:
			event.Skip();
		break;
	}
 }


 void MyFindTextCtrl::OnGUIMouse(wxMouseEvent& ev)
 {
	wxCommandEvent dummy;

 #if defined(__WXMSW__)
	if(!ev.RightDown()) // avoid copy/paste popup
		ev.Skip();
 #else
	ev.Skip();
 #endif
 }
#endif


MyApp *g_theApp;

#if defined(MASMALLERFONT)
 wxFont g_mafont;
#endif

#if !defined(__WXMSW__) && !defined(__WXX11__)
 #include "MoreAmpIcon.xpm"
#endif

#if (MAGUIWX == 4) // wx-2.4.2
 wxApp *wxCreateApp(); // prototype for IMPLEMENT_APP
#endif

IMPLEMENT_APP(MyApp)

#if defined(__MAOSX__)
 #if (MAGUIWX == 9)
 #else
  WindowRef l_wrefwapp = NULL;
 #endif
#endif


int	AppWindowMake()
{
	wxPoint pos; 
	wxSize  siz;
	
	pos = wxPoint(g_sets.mawin[WINDOWAPP].xpos[g_sets.winconf], g_sets.mawin[WINDOWAPP].ypos[g_sets.winconf]);
	siz = wxSize(g_sets.mawin[WINDOWAPP].xsize[g_sets.winconf], g_sets.mawin[WINDOWAPP].ysize[g_sets.winconf]);
	
	MyFrameApp *frame = new MyFrameApp(WINDOWAPP, g_prognametitle, pos, siz);

	g_w[WINDOWAPP] = frame;

#if !defined(__WXX11__)
 #if defined(__WXMSW__)
    frame->SetIcon(wxICON(APP_ICON));
 #else
    frame->SetIcon(wxICON(MoreAmpIcon));
 #endif
#endif

#if defined(__MAOSX__) && !defined(__WXGTK__)
 #if (MAGUIWX == 9)
 #else
	l_wrefwapp = (WindowRef)frame->MacGetWindowRef();
 #endif
#endif

#if defined(__MAOSX__) && defined(__WXMAC__)
 #if (MAGUIWX == 9)
 #else
	HISize min, max;
	GetWindowResizeLimits(l_wrefwapp, &min, &max);
	min.width = 20;
	min.height = 20;
	SetWindowResizeLimits(l_wrefwapp, &min, &max);
 #endif
#endif

	frame->Move(pos.x, pos.y); // need this here
	frame->Show();
	g_theApp->SetTopWindow(frame);
	
#if defined(MAKEYBOARD)
	UpdateMenuKbLabels();
	UpdateButtonLabels();
#endif

#ifdef MADRAGANDDROP
	frame->SetDropTarget(new DnDFile()); // does not work in main window on mac
#endif

	return 0;
	
} // AppWindowMake


/*
#if defined(MAINTL)
 wxChar Internat_mDecimalSeparator;
 wxString Internat_forbid;
 wxArrayString Internat_exclude;
 void Internat_Init() // Internat::Init()
 {
	// Save decimal point character
	struct lconv * localeInfo = localeconv();
	if(localeInfo)
		Internat_mDecimalSeparator = wxString(localeInfo->decimal_point, wxConvLocal).GetChar(0);

	// wxLogDebug(wxT("Decimal separator set to '%c'"), mDecimalSeparator);

	// Setup list of characters that aren't allowed in file names
	wxFileName tmpFile;
	Internat_forbid = tmpFile.GetForbiddenChars();
	for(unsigned int i = 0; i < Internat_forbid.Length(); i++)
		Internat_exclude.Add(Internat_forbid.Mid(i, 1));
 }


 WX_DECLARE_STRING_HASH_MAP(wxString, LangHash);


 void GetLanguages(wxArrayString &langCodes, wxArrayString &langNames)
 {
   wxArrayString tempNames;
   wxArrayString tempCodes;
   LangHash localLanguageName;
   LangHash reverseHash;
   LangHash tempHash;

   // MM: Use only ASCII characters here to avoid problems with
   //     charset conversion on Linux platforms
   localLanguageName[wxT("ar")] = wxT("Arabic");
   localLanguageName[wxT("bg")] = wxT("Balgarski");
   localLanguageName[wxT("bs")] = wxT("Bosnian");
   localLanguageName[wxT("my")] = wxT("Burmese");
   localLanguageName[wxT("ca")] = wxT("Catalan");
   localLanguageName[wxT("cs")] = wxT("Czech");
   localLanguageName[wxT("da")] = wxT("Dansk");
   localLanguageName[wxT("de")] = wxT("Deutsch");
   localLanguageName[wxT("el")] = wxT("Ellinika");
   localLanguageName[wxT("en")] = wxT("English");
   localLanguageName[wxT("es")] = wxT("Espanol");
   localLanguageName[wxT("eu")] = wxT("Euskara");
   localLanguageName[wxT("fi")] = wxT("Suomi");
   localLanguageName[wxT("fr")] = wxT("Francais");
   localLanguageName[wxT("ga")] = wxT("Gaeilge");
   localLanguageName[wxT("id")] = wxT("Bahasa Indonesia"); // aka Indonesian
   localLanguageName[wxT("it")] = wxT("Italiano");
   localLanguageName[wxT("ja")] = wxT("Nihongo");
   localLanguageName[wxT("ka")] = wxT("Georgian");
   localLanguageName[wxT("km")] = wxT("Khmer");
   localLanguageName[wxT("lo")] = wxT("Lao"); // aka Laothian
   localLanguageName[wxT("lt")] = wxT("Lietuviu");
   localLanguageName[wxT("he")] = wxT("Hebrew");
   localLanguageName[wxT("hi")] = wxT("Hindi");
   localLanguageName[wxT("hu")] = wxT("Magyar");
   localLanguageName[wxT("mk")] = wxT("Makedonski");
   localLanguageName[wxT("nl")] = wxT("Nederlands");
   localLanguageName[wxT("nb")] = wxT("Norsk");
   localLanguageName[wxT("pl")] = wxT("Polski");
   localLanguageName[wxT("pt")] = wxT("Portugues");
   localLanguageName[wxT("pt_BR")] = wxT("Portugues (Brasil)");
   localLanguageName[wxT("ru")] = wxT("Russky");
   localLanguageName[wxT("sl")] = wxT("Slovenscina");
   localLanguageName[wxT("sv")] = wxT("Svenska");
   localLanguageName[wxT("tr")] = wxT("Turkce");
   localLanguageName[wxT("uk")] = wxT("Ukrainska");
   localLanguageName[wxT("ca@valencia")] = wxT("Valencian (southern Catalan)");
   localLanguageName[wxT("vi")] = wxT("Vietnamese");
   localLanguageName[wxT("zh")] = wxT("Chinese (Simplified)");
   localLanguageName[wxT("zh_TW")] = wxT("Chinese (Traditional)");

	/w* wxArrayString PathList = wxGetApp().PathList;
	// wxGetApp().AddUniquePathToPathList(wxString::Format(wxT("%s/share/locale"),
                                                   wxT(INSTALL_PREFIX)), PathList);
	*w/
	int i;
	for(i=wxLANGUAGE_UNKNOWN; i<wxLANGUAGE_USER_DEFINED;i++)
	{
      const wxLanguageInfo *info = wxLocale::GetLanguageInfo(i);

      if(!info)
         continue;

      wxString fullCode = info->CanonicalName;
      wxString code = fullCode.Left(2);
      wxString name = info->Description;

      // Logic: Languages codes are sometimes hierarchical, with a
      // general language code and then a subheading.  For example,
      // zh_TW for Traditional Chinese and zh_CN for Simplified
      // Chinese - but just zh for Chinese in general.  First we look
      // for the full code, like zh_TW.  If that doesn't exist, we
      // look for a code corresponding to the first two letters.
      // Note that if the language for a fullCode exists but we only
      // have a name for the short code, we will use the short code's
      // name but associate it with the full code.  This allows someone
      // to drop in a new language and still get reasonable behavior.

      if(fullCode.Length() < 2)
         continue;

      if(localLanguageName[code] != wxT("")) {
         name = localLanguageName[code];
      }
      if(localLanguageName[fullCode] != wxT("")) {
         name = localLanguageName[fullCode];
      }

      /w* if(TranslationExists(PathList, fullCode)) {
         code = fullCode;
      }
	*w/
	
      if(tempHash[code] != wxT(""))
         continue;

      /w* if(TranslationExists(PathList, code) || code==wxT("en"))
	  {
         tempCodes.Add(code);
         tempNames.Add(name);
         tempHash[code] = name;

         /w* for debugging *w/
         wxLogDebug(wxT("code=%s name=%s fullCode=%s name=%s -> %s\n"),
                code.c_str(), localLanguageName[code].c_str(),
                fullCode.c_str(), localLanguageName[fullCode].c_str(),
                name.c_str());
      }
	*w/
   }


   // Adding language
   {
      wxString code;
      wxString name;
      code = wxT("en-simple");
      name = wxT("Simplified");
      /w* if(TranslationExists(PathList, code) )
	  {
         tempCodes.Add(code);
         tempNames.Add(name);
         tempHash[code] = name;

         /w* for debugging
         printf(wxT("code=%s name=%s fullCode=%s name=%s -> %s\n"),
                code.c_str(), localLanguageName[code].c_str(),
                fullCode.c_str(), localLanguageName[fullCode].c_str(),
                name.c_str());
         *w/
      }
	  *w/
   }


   // Sort

   unsigned int j;
   for(j=0; j<tempNames.GetCount(); j++)
      reverseHash[tempNames[j]] = tempCodes[j];

   tempNames.Sort();

   // Add system language
   langNames.Add(wxT("System"));
   langCodes.Add(wxT(""));

   for(j=0; j<tempNames.GetCount(); j++) {
      langNames.Add(tempNames[j]);
      langCodes.Add(reverseHash[tempNames[j]]);
   }
   
 } // GetLanguages


 wxString GetSystemLanguageCode()
 {
	wxArrayString langCodes;
	wxArrayString langNames;

	GetLanguages(langCodes, langNames);
	int sysLang = wxLocale::GetSystemLanguage();
	const wxLanguageInfo *info = wxLocale::GetLanguageInfo(sysLang);
   
	if(info)
	{
		wxString fullCode = info->CanonicalName;
		if(fullCode.Length() < 2)
		return wxT("en");

		wxString code = fullCode.Left(2);
		unsigned int i;

		for(i=0; i<langCodes.GetCount(); i++)
		{
			if(langCodes[i] == fullCode)
				return fullCode;
			if(langCodes[i] == code)
				return code;
		}
	}
	return wxT("en");
 
 } // GetSystemLanguageCode


 /w*
 void MyApp::~MyApp()
 {
 #if defined(MAINTL)
	if(mLocale)
		delete mLocale;
 #endif
 }
*w/


 void MyApp::InitLang( const wxString & lang )
 {
   if(mLocale)
      delete mLocale;

   if(lang != wxT("en"))
   {
      wxLogNull nolog;

// LL: I do not know why loading translations fail on the Mac if LANG is not
//     set, but for some reason it does.  So wrap the creation of wxLocale
//     with the default translation.
 #if defined(__WXMAC__)
      wxString oldval;
      bool existed;
      
      existed = wxGetEnv(wxT("LANG"), &oldval);
      wxSetEnv(wxT("LANG"), wxT("en_US"));
 #endif

      wxLocale *Locale = mLocale = new wxLocale(wxT(""), lang, wxT(""), true, true);
 #if defined(__WXMAC__)
      if(existed) {
         wxSetEnv(wxT("LANG"), oldval);
      }
      else {
         wxUnsetEnv(wxT("LANG"));
      }
 #endif

	/w* for(unsigned int i=0; i<PathList.GetCount(); i++)
         mLocale->AddCatalogLookupPathPrefix(PathList[i]);
	*w/
#ifdef MOREAMP_NAME
      mLocale->AddCatalog(wxT(MOREAMP_NAMEos));
#else
      mLocale->AddCatalog(wxT("moreamp"));
#endif
   } else
      mLocale = NULL;

   // Initialize internationalisation (number formats etc.)
   //
   // This must go _after_ creating the wxLocale instance because
   // creating the wxLocale instance sets the application-wide locale.
   Internat_Init(); // Internat::Init();
   
 } // InitLang
#endif // MAINTL
*/


bool MyApp::OnInit() // called by the wx framework when starting the app
{
	g_theApp = this;
	g_argc = g_theApp->argc;
	g_argv = g_theApp->argv;

#if defined(__MAOSX__) && defined(__WXMAC__) && (MAGUIWX >= 8) // wx-2.8+
	// otherwise uses "native", which has problems with up/down arrow keys
 #if wxUSE_SYSTEM_OPTIONS 
	wxSystemOptions::SetOption(wxT("mac.listctrl.always_use_generic"), 1);
 #endif
#endif

#if defined(__MAOSX__)
	InitAppleEvents((void *)this);
#endif

/*
#if defined(MAINTL)
	wxString lang = GetSystemLanguageCode();
	mLocale = NULL;
	g_theApp->InitLang(lang);
#endif // MAINTL
*/
	masprintf(&g_abouttext[mastrlen(g_abouttext) - STRLEN64], STRLEN64,
#if defined(__MAOSX__) && defined(__WXMAC__) && (MAGUIWX == 6) && (wxMINOR_VERSION == 6)
				L" wxWidgets-%d.%d.%d (unicode, modified)",  // for wxMAC_TEXTCONTROL_USE_MLTE above
#else
				L" wxWidgets-%d.%d.%d (unicode, as released)",
#endif
				wxMAJOR_VERSION, wxMINOR_VERSION, wxRELEASE_NUMBER);

#if defined(MASMALLERFONT)
 #if (MAGUIWX == 4)
	g_mafont = wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT);
 #else
	g_mafont = wxSystemSettingsNative::GetFont(wxSYS_DEFAULT_GUI_FONT);
 #endif
	if(g_mafont.GetPointSize() > MASMALLERFONT)
		g_mafont.SetPointSize(MASMALLERFONT);
#endif									  

#if defined(MAKEYBOARD)
	g_theApp->Connect(wxID_ANY, wxEVT_CHAR,
		(wxObjectEventFunction)(wxEventFunction)(wxCharEventFunction)&MyApp::maOnChar, NULL);
	g_theApp->Connect(wxID_ANY, wxEVT_LEFT_UP,
		(wxObjectEventFunction)(wxEventFunction)(wxMouseEventFunction)&MyApp::maOnClick, NULL);
#endif

#if defined(MAMOUSEWHEEL)
	g_theApp->Connect(wxID_ANY, wxEVT_MOUSEWHEEL,
		(wxObjectEventFunction)(wxEventFunction)(wxMouseEventFunction)&MyApp::OnMouseWheel, NULL);
#endif

	return OnInit_() ? TRUE : FALSE;

} // OnInit


#if defined(MAKEYBOARD)
 void MyApp::maOnChar(wxKeyEvent &event)
 {
	void *p;
	void *parent;
	int keyval;
	fschar fskey[STRLEN16] = {fsT('\0')};
	
	if(g_sets.keyboard && event.GetEventType() == wxEVT_CHAR)
	{
		parent = p = event.GetEventObject();
		while(p && (p = ((wxWindow *)p)->GetParent()) != 0)
			parent = p;
		
		// accept only from these windows
		if((g_w[WINDOWAPP] && g_w[WINDOWAPP] == parent) ||
		   (g_w[WINDOWLIST] && g_w[WINDOWLIST] == parent) ||
		   ((g_w[WINDOWEQ] && g_w[WINDOWEQ] == parent) && ((MyFrameEq *)parent)->m_edittexteq != event.GetEventObject()) ||
		   (g_w[WINDOWEQ2] && g_w[WINDOWEQ2] == parent) ||
		   (g_w[WINDOWEFFECTS] && g_w[WINDOWEFFECTS] == parent) ||
		   (g_w[WINDOWPROMPTNUM] && g_w[WINDOWPROMPTNUM] == parent))
		{
			if((keyval = event.GetKeyCode()) == WXK_DOWN)
				fskey[4] = MAWXK_DOWN;
			else if(keyval == WXK_UP) 
				fskey[4] = MAWXK_UP;
			else if(keyval == WXK_RETURN)
				fskey[4] = MAWXK_RETURN;
			else if(keyval == WXK_ESCAPE)
				fskey[4] = MAWXK_ESCAPE;
			else
				if(keyval == 0) // WXK_NONE since 2.9.2
					return; // not a char but a function key we are not interested in

#if defined(__WXMAC__)
			// On Mac we do the encoding to UTF-8 here		
			wxString s = wxT(" ");
			if((s[0] = event.GetUnicodeKey()) != 0)
			{
				const char* v = s.utf8_str().data();
				unsigned int i;
				for (i = 0; i <= strlen(v); i++)
					fskey[i] = v[i];
			}
			else
#endif
			// On WXGTK2, GetUnicodeKey returns capitalized chars, e.g. 'E' for 'e'.
			// Also, on WXGTK2 builds we don't get composed chars
			// e.g for '' ('^' key then 'e' key) we get 'E' (or 'e' from GetKeyCode).
			// So we do the conversion to UTF8 based on g_charsetkb (in maOnKey).
		
			// On __MAWIN__ we don't need to convert to UTF-8
		 
				fskey[0] = keyval;

			if(!maOnKey(fskey))
				return;
		}
	}

	event.Skip(); // not a char event, pass it on

 } // maOnChar


 void MyApp::maOnClick(wxMouseEvent &event)
 {
	void *p;
	void *parent;
	
	if(g_sets.keyboard && event.GetEventType() == wxEVT_LEFT_UP)
	{
		parent = p = event.GetEventObject();
		while(p && (p = ((wxWindow *)p)->GetParent()) != 0)
			parent = p;
		
		// accept only from these windows
		if(g_w[WINDOWEQ] && g_w[WINDOWEQ] == parent && // in WINDOWEQ outside m_edittexteq
			event.GetEventObject() != ((MyFrameEq *)parent)->m_edittexteq &&
			wxFindFocusDescendant((wxWindow *)parent) == ((MyFrameEq *)parent)->m_edittexteq)
		{
			// move the kb focus from m_edittexteq
			((MyFrameEq *)parent)->m_dummytexteq->SetFocus();
			return;
		}

		if(g_w[WINDOWLIST] && g_w[WINDOWLIST] == parent && // in WINDOWLIST outside m_list
			event.GetEventObject() != ((MyFrameList *)parent)->m_list &&
			wxFindFocusDescendant((wxWindow *)parent) == ((MyFrameList *)parent)->m_list
 #if defined(__WXMAC__)				
			&& wxFindFocusDescendant((wxWindow *)parent) != event.GetEventObject()
 #endif			
				)
		{
			// move the kb focus from m_list
			((MyFrameList *)parent)->m_cbdummy->SetFocus();
 #if !defined(__WXGTK__)				
			return; // on WXGTK2 if we return now we lose list window checkbox clicks
 #endif			
		}
	}

	event.Skip();

 } // maOnClick
#endif // MAKEYBOARD


#if defined(MAMOUSEWHEEL)
 void MyApp::OnMouseWheel(wxMouseEvent &event)
 {
	int wheelRotation;
	void *p;
	void *parent;
	
	if(g_sets.mousewheel && event.GetEventType() == wxEVT_MOUSEWHEEL)
	{
		parent = p = event.GetEventObject();
		while(p && (p = ((wxWindow *)p)->GetParent()) != 0)
			parent = p;
		
		// accept only from these windows
		if(((g_w[WINDOWAPP] && g_w[WINDOWAPP] == parent) ||
		
			(g_w[WINDOWLIST] && g_w[WINDOWLIST] == parent && // in WINDOWLIST outside m_list
				event.GetEventObject() != ((MyFrameList *)parent)->m_list
 #if defined(__WXMAC__)				
			&& wxFindFocusDescendant((wxWindow *)parent) != event.GetEventObject()
 #endif			
				) ||
				
			(g_w[WINDOWEQ] && g_w[WINDOWEQ] == parent) ||
			
			(g_w[WINDOWEQ2] && g_w[WINDOWEQ2] == parent) ||
			
			(g_w[WINDOWEFFECTS] && g_w[WINDOWEFFECTS] == parent)))
		{
			if((wheelRotation = event.GetWheelRotation()) > 0)
				ChangeVolume(MAWHEELVOLUMEDELTA);
			else if(wheelRotation < 0)
				ChangeVolume(-MAWHEELVOLUMEDELTA);
			return;
		}
	}

	event.Skip();

 } // OnMouseWheel
#endif


void MyApp::OnGUICloseRequest(wxCloseEvent &event) // user clicked a window's close icon X
{
	int windowid = event.GetId();
	
	if(windowid < WINDOWFIRST || windowid > WINDOWLAST || !g_w[windowid])
		return;

	DoWindow(windowid, 0);

} // OnGUICloseRequest


void MyApp::OnGUISized(wxSizeEvent &event)
{
	int dummy;
	int xpos;
	int ypos;
	int wnew;
	int hnew;
	int xdelta;
	int ydelta;
	int windowid = event.GetId();
	
	if(windowid < WINDOWFIRST || windowid > WINDOWLAST || !g_w[windowid])
		return;

	((wxFrame *)g_w[windowid])->OnSize(event);
	
	switch(windowid)
	{
#if !defined(NOTEXTVIEWER)
		case WINDOWREADME:
		case WINDOWCATALOG:
		case WINDOWPLAYLIST:
		case WINDOWLOG:
 #if defined(MAKEYBOARD)
		case WINDOWKEYBINDINGS:
 #endif
			WindowGetCoordinates(windowid, &dummy, &dummy, &wnew, &hnew);
			xdelta = wnew - g_sets.mawin[windowid].xsize[g_sets.winconf];
			ydelta = hnew - g_sets.mawin[windowid].ysize[g_sets.winconf];
			// move buttons, static text and findtext
			((MyFrameViewer *)g_w[windowid])->m_findTextCtrl->GetPosition(&xpos, &ypos);
			((MyFrameViewer *)g_w[windowid])->m_findTextCtrl->Move(xpos, ypos + ydelta);
			((MyFrameViewer *)g_w[windowid])->m_findbutton->GetPosition(&xpos, &ypos);
			((MyFrameViewer *)g_w[windowid])->m_findbutton->Move(xpos, ypos + ydelta);
			if(windowid == WINDOWCATALOG)
			{
				((MyFrameViewer *)g_w[windowid])->m_playbutton->GetPosition(&xpos, &ypos);
				((MyFrameViewer *)g_w[windowid])->m_playbutton->Move(xpos, ypos + ydelta);
				((MyFrameViewer *)g_w[windowid])->m_playStaticText->GetPosition(&xpos, &ypos);
				((MyFrameViewer *)g_w[windowid])->m_playStaticText->Move(xpos, ypos + ydelta);
			}
			((MyFrameViewer *)g_w[windowid])->m_foundStaticText->GetPosition(&xpos, &ypos);
			((MyFrameViewer *)g_w[windowid])->m_foundStaticText->Move(xpos, ypos + ydelta);
			((MyFrameViewer *)g_w[windowid])->m_editbutton->GetPosition(&xpos, &ypos);
			((MyFrameViewer *)g_w[windowid])->m_editbutton->Move(xpos + xdelta, ypos + ydelta);
			((MyFrameViewer *)g_w[windowid])->m_closebutton->GetPosition(&xpos, &ypos);
			((MyFrameViewer *)g_w[windowid])->m_closebutton->Move(xpos + xdelta, ypos + ydelta);
			// resize edittext and findtext
			((MyFrameViewer *)g_w[windowid])->m_findTextCtrl->GetSize(&xpos, &ypos);
			((MyFrameViewer *)g_w[windowid])->m_findTextCtrl->SetSize(xpos + xdelta, ypos);
			((MyFrameViewer *)g_w[windowid])->m_viewerTextCtrl->GetSize(&xpos, &ypos);
			((MyFrameViewer *)g_w[windowid])->m_viewerTextCtrl->SetSize(xpos + xdelta, ypos + ydelta);
		break;
#endif
		case WINDOWABOUT:	
			WindowGetCoordinates(windowid, &dummy, &dummy, &wnew, &hnew);
			xdelta = wnew - g_sets.mawin[WINDOWABOUT].xsize[g_sets.winconf];
			ydelta = hnew - g_sets.mawin[WINDOWABOUT].ysize[g_sets.winconf];
			// move urltext and button
			((MyFrameAbout *)g_w[WINDOWABOUT])->m_urlTextCtrl->GetPosition(&xpos, &ypos);
			((MyFrameAbout *)g_w[WINDOWABOUT])->m_urlTextCtrl->Move(xpos, ypos + ydelta);
			((MyFrameAbout *)g_w[WINDOWABOUT])->m_closebutton->GetPosition(&xpos, &ypos);
			((MyFrameAbout *)g_w[WINDOWABOUT])->m_closebutton->Move(xpos + xdelta / 2, ypos + ydelta);
			// resize abouttext and urltext
			((MyFrameAbout *)g_w[WINDOWABOUT])->m_urlTextCtrl->GetSize(&xpos, &ypos);
			((MyFrameAbout *)g_w[WINDOWABOUT])->m_urlTextCtrl->SetSize(xpos + xdelta, ypos);
			((MyFrameAbout *)g_w[WINDOWABOUT])->m_aboutTextCtrl->GetSize(&xpos, &ypos);
			((MyFrameAbout *)g_w[WINDOWABOUT])->m_aboutTextCtrl->SetSize(xpos + xdelta, ypos + ydelta);
		break;

		case WINDOWADVANCED:	
			WindowGetCoordinates(windowid, &dummy, &dummy, &wnew, &hnew);
			xdelta = wnew - g_sets.mawin[WINDOWADVANCED].xsize[g_sets.winconf];
			ydelta = hnew - g_sets.mawin[WINDOWADVANCED].ysize[g_sets.winconf];
			// move close button
			((MyFrameAdvanced *)g_w[WINDOWADVANCED])->m_closebutton->GetPosition(&xpos, &ypos);
			((MyFrameAdvanced *)g_w[WINDOWADVANCED])->m_closebutton->Move(xpos + xdelta / 2, ypos + ydelta);
		break;

		default:
		break;		
	}

	DoWindowResized(windowid);

} // OnGUISized


void MyApp::OnGUIMoved(wxMoveEvent &event)
{
	int windowid = event.GetId();
	
	if(windowid < WINDOWFIRST || windowid > WINDOWLAST || !g_w[windowid])
		return;
	
	DoWindowMoved(windowid);

} // OnGUIMoved


void MyFrameAbout::OnGUIButtonClose(wxCommandEvent & WXUNUSED(event))
{
	DoWindow(WINDOWABOUT, 0);

} // OnGUIButtonClose


#if !defined(NOTEXTVIEWER)
 void MyFrameViewer::OnGUIButtonClose(wxCommandEvent & WXUNUSED(event))
 {
	DoWindow(this->get_windowid(), 0);

 } // OnGUIButtonClose
#endif


void MyFrameAdvanced::OnGUIButtonClose(wxCommandEvent & WXUNUSED(event))
{
	DoWindow(WINDOWADVANCED, 0);
	
} // OnGUIButtonClose


// control sizes
#define EDSIZ wxSize(72, 23)			// edit boxes

#define SPSIZ wxSize(16, 24)			// spin controls

#if defined(MAKEYBOARD)
 #define SLSIZ wxSize(XSIZEDEF - 12, 20) // pos slider
#else
 #define SLSIZ wxSize(XSIZEDEF - 26, 20)
#endif

#define CBYSIZ 12						// checkboxes
#if !defined(MAUSEMACHECKBOX) && defined(__MAOSX__) && defined(__WXMAC__)
 #define CBXSIZ 140						// for checkboxes with large font
 #define CBSIZ (wxSize(CBXSIZ, CBYSIZ))
 #define CBSIZ2 (wxSize(CBXSIZ + 20, CBYSIZ))
 #define CBSIZ3 (wxSize(CBXSIZ - 30, CBYSIZ))
#else
 #define CBXSIZ 80
 #define CBSIZ (wxSize(CBXSIZ, CBYSIZ))
 #define CBSIZ2 (wxSize(CBXSIZ + 20, CBYSIZ))
 #define CBSIZ3 (wxSize(50, CBYSIZ))
#endif
#define CBSIZ4 (wxSize(CBYSIZ + 2, CBYSIZ))
#define CBSIZ5 (wxSize(CBYSIZ + 2 + 40, CBYSIZ))


/*
MyFrameSplash::MyFrameSplash(wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& siz, long style)
			: wxFrame((wxWindow *)NULL, id, title, pos, siz, style)
{
	m_panel = new wxPanel(this, -1, wxDefaultPosition, siz, wxCLIP_CHILDREN | wxNO_BORDER);
	m_splashTextCtrl = new wxStaticText(m_panel, -1, wxT("Loading..."), wxPoint(20, -1), wxSize(-1, -1));
	MASETFONT(m_splashTextCtrl);
	// m_splashTextCtrl->SetFocus();
	
} // MyFrameSplash


MyFrameSplash::~MyFrameSplash()
{
}
*/

MyFrameApp::MyFrameApp(wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& siz, long style)
					: wxFrame((wxWindow *)NULL, id, title, pos, siz, style)
{
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
#endif
	m_timerchangeplaylist = new wxTimer(this, ID_TIMER_CHANGEPLAYLIST);
	m_timerplayplaylist = new wxTimer(this, ID_TIMER_PLAYPLAYLIST);
	m_timerrunloop = new wxTimer(this, ID_TIMER_RUN);
	m_timerplay = new wxTimer(this, ID_TIMER_PLAY);
	// m_timerstop = new wxTimer(this, ID_TIMER_STOP);
	
	MakeMenus();
	
#if !defined(MANOWXPANEL)
	m_panel = new wxPanel(this, -1, wxDefaultPosition, siz, wxCLIP_CHILDREN | wxNO_BORDER);
#endif

#if defined(MAKEYBOARD)
	// having a wxListCtrl makes kb focus work...
	new wxListCtrl(MPANEL, -1, wxPoint(0, 0), wxSize(1, 1),
		wxLC_REPORT | wxLC_ALIGN_LEFT | wxLC_SINGLE_SEL | wxLC_HRULES | wxLC_NO_SORT_HEADER);
#endif

	#define BUTTONTEXTY (YBASE + MABUTTONH)

	// Alt size checkbox
#if defined(__MAWIN__) && defined(MAMOUSEWHEEL)
	l_ChkAltSize =
#endif
	new MACHECKBOX(MPANEL, ID_CHK_ALTSIZE, wxT(""), wxPoint(XBASE + 1, YBASE + MABUTTONH), CBSIZ4, 0); // in front "Quit"

	// player buttons
	m_ButtonPos = XBASE;
	m_buttonTop = YBASE;
	MakeButtons();

	// position slider
#if defined(MAKEYBOARD)
	#define SLIDERPOSX (XBASE + 4)
#else
	#define SLIDERPOSX (XBASE + 12)
#endif
	#define SLIDERPOSY (YBASE + 62)
	m_sliderPos = new MASLIDER(MPANEL, ID_SLIDER_POSITION, 0, POSMIN, POSMAX, wxPoint(SLIDERPOSX, SLIDERPOSY), SLSIZ);

	// repeat controls
	wxStaticText *t;
	t = new wxStaticText(MPANEL, ID_TEXT_TIMEPOS, ZEROMSSTR, wxPoint(XBASE + 6, YBASE + 90 - 5));
	MASETFONT(t);
	StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEPOS, NOTIMESTR);
	new MACHECKBOX(MPANEL, ID_CHK_REPTSONG, wxT("Repeat"), wxPoint(XBASE + 6, YBASE + 90 + 9), CBSIZ, 1); // red
		
	wxSpinButton *s = new wxSpinButton(MPANEL, ID_SPIN_BEG, wxPoint(XBASE + 98 + 2, YBASE + 90 - 4), SPSIZ, wxSP_VERTICAL | wxSP_WRAP);
	s->SetRange(INT_MIN, INT_MAX);
	s->SetValue(0);

	t = new wxStaticText(MPANEL, ID_TEXT_TIMEBEG, ZEROMSSTR, wxPoint(XBASE + 98 + 20, YBASE + 90 - 5));
	MASETFONT(t);
	StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEBEG, NOTIMESTR);
	new MACHECKBOX(MPANEL, ID_CHK_BEG, wxT("Begin"), wxPoint(XBASE + 98 + 20, YBASE + 90 + 9), CBSIZ, 1); // red

	s = new wxSpinButton(MPANEL, ID_SPIN_END, wxPoint(XBASE + 200 + 2, YBASE + 90 - 4), SPSIZ, wxSP_VERTICAL | wxSP_WRAP);
	s->SetRange(INT_MIN, INT_MAX);
	s->SetValue(0);

	t = new wxStaticText(MPANEL, ID_TEXT_TIMEEND, ZEROMSSTR, wxPoint(XBASE + 200 + 20, YBASE + 90 - 5));
	MASETFONT(t);
	StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEEND, NOTIMESTR);
	new MACHECKBOX(MPANEL, ID_CHK_END, wxT("End"), wxPoint(XBASE + 200 + 20, YBASE + 90 + 9), CBSIZ, 1); // red

	// Volume, Separation, Preamp controls
	#define VOLY  (YBASE + 132)
	#define VOLX  (XBASE + 2)
	#define VOLLX (VOLX  + 0)
	#define VOLRX (VOLX  + 31)
	#define SEPX  (VOLX  + 63)
	#define PREAX (VOLX  + 93)
	
	new MACHECKBOX(MPANEL, ID_CHK_BALANCE, wxT("Balance"), wxPoint(VOLLX + 4, YBASE + 120), CBSIZ);

	t = new wxStaticText(MPANEL, ID_TEXT_VOLUME, wxT("Volume"), wxPoint(VOLLX + 4, YBASE + 140));      
	MASETFONT(t);

	new MASLIDER(MPANEL, ID_SLIDER_VOLUMER, (VOLUMEMAX - VOLUMEMIN) / 2, VOLUMEMIN, VOLUMEMAX,
		wxPoint(VOLRX, VOLY + 21), wxSize(20, 92), wxSL_VERTICAL); // | wxSL_AUTOTICKS | wxSL_LABELS | wxSL_BOTTOM);

	t = new wxStaticText(MPANEL, ID_TEXT_100VOLR,wxT("100"), wxPoint(VOLRX - 2, VOLY + 114));
	MASETFONT(t);

	s = new wxSpinButton(MPANEL, ID_SPIN_VOLR, wxPoint(VOLRX + 2, VOLY + 127), SPSIZ, wxSP_VERTICAL | wxSP_WRAP);
	s->SetRange(INT_MIN, INT_MAX);
	s->SetValue(0);

	new MASLIDER(MPANEL, ID_SLIDER_VOLUMEL, (VOLUMEMAX - VOLUMEMIN) / 2, VOLUMEMIN, VOLUMEMAX,
		wxPoint(VOLLX, VOLY + 21), wxSize(20, 92), wxSL_VERTICAL); // | wxSL_AUTOTICKS | wxSL_LABELS | wxSL_BOTTOM);

	t = new wxStaticText(MPANEL, ID_TEXT_100VOLL,wxT("100"), wxPoint(VOLLX - 2, VOLY + 114));
	MASETFONT(t);

	s = new wxSpinButton(MPANEL, ID_SPIN_VOLL, wxPoint(VOLLX + 2, VOLY + 127), SPSIZ, wxSP_VERTICAL | wxSP_WRAP);
	s->SetRange(INT_MIN, INT_MAX);
	s->SetValue(0);

	t = new wxStaticText(MPANEL, ID_TEXT_SEP, wxT("Sep"), wxPoint(SEPX, YBASE + 140));
	MASETFONT(t);

	t = new wxStaticText(MPANEL, ID_TEXT_S, wxT("s"), wxPoint(SEPX - 7, VOLY + 21));
	MASETFONT(t);
	new MASLIDER(MPANEL, ID_SLIDER_VOLUMESEP, (SEPARATIONMAX - SEPARATIONMIN) / 2, SEPARATIONMIN, SEPARATIONMAX,
		wxPoint(SEPX, VOLY + 21), wxSize(20, 92), wxSL_VERTICAL); // | wxSL_AUTOTICKS | wxSL_LABELS | wxSL_BOTTOM);

	t = new wxStaticText(MPANEL, ID_TEXT_M, wxT("m"), wxPoint(SEPX - 7, VOLY + 99));
	MASETFONT(t);

	t = new wxStaticText(MPANEL, ID_TEXT_100SEP, wxT("100"), wxPoint(SEPX, VOLY + 114));
	MASETFONT(t);

	s = new wxSpinButton(MPANEL, ID_SPIN_SEP, wxPoint(SEPX + 2, VOLY + 127), SPSIZ, wxSP_VERTICAL | wxSP_WRAP);
	s->SetRange(INT_MIN, INT_MAX);
	s->SetValue(0);

	t = new wxStaticText(MPANEL, ID_TEXT_PREAMP, wxT("Preamp"), wxPoint(PREAX - 2, YBASE + 140));
	MASETFONT(t);

	new MASLIDER(MPANEL, ID_SLIDER_PREAMP, (PREAMPMAX - PREAMPMAX) / 2, PREAMPMIN, PREAMPMAX,
			wxPoint(PREAX, VOLY + 21), wxSize(20, 92), wxSL_VERTICAL); // | wxSL_AUTOTICKS | wxSL_LABELS | wxSL_BOTTOM);

	new MACHECKBOX(MPANEL, ID_CHK_LIMITER, wxT("Limit"), wxPoint(PREAX - 2, YBASE + 120), CBSIZ5);

	t = new wxStaticText(MPANEL, ID_TEXT_0, wxT("-00.0"), wxPoint(PREAX, VOLY + 114));
	MASETFONT(t);
	s = new wxSpinButton(MPANEL, ID_SPIN_PREA, wxPoint(PREAX + 2, VOLY + 127), SPSIZ, wxSP_VERTICAL | wxSP_WRAP);
	s->SetRange(INT_MIN, INT_MAX);
	s->SetValue(0);

	// more checkboxes
	new MACHECKBOX(MPANEL, ID_CHK_EQWINDOW, wxT("EqWindow"), wxPoint(XBASE + 144, YBASE + 120), CBSIZ);
	new MACHECKBOX(MPANEL, ID_CHK_EQUALIZER, wxT("Equalizer"), wxPoint(XBASE + 144, YBASE + 141), CBSIZ);
	// new MACHECKBOX(MPANEL, ID_CHK_EQ31BANDS, wxT("Eq31Bands"), wxPoint(XBASE + 144, YBASE + 162), CBSIZ);
	// new MACHECKBOX(MPANEL, ID_CHK_EQEXTRA, wxT("EqExtra"), wxPoint(XBASE + 144, YBASE + 183), CBSIZ);
	new MACHECKBOX(MPANEL, ID_CHK_EFFECTS, wxT("Effects"), wxPoint(XBASE + 144, YBASE + 162), CBSIZ, 1); // red

	int cbypos = YBASE + 99;
	
	new MACHECKBOX(MPANEL, ID_CHK_LISTWINDOW, wxT("ListWindow"), wxPoint(XBASE + 220, cbypos += 21), CBSIZ2);
	new MACHECKBOX(MPANEL, ID_CHK_RANDOM, wxT("Random"), wxPoint(XBASE + 220, cbypos += 21), CBSIZ2);
	new MACHECKBOX(MPANEL, ID_CHK_REPTPLAYED, wxT("ReptPlayed"), wxPoint(XBASE + 220, cbypos += 21), CBSIZ2);
	new MACHECKBOX(MPANEL, ID_CHK_LOOPLOADED, wxT("LoopLoaded"), wxPoint(XBASE + 220, cbypos += 21), CBSIZ2, 1); // red
	new MACHECKBOX(MPANEL, ID_CHK_LOOPLIST, wxT("LoopList"), wxPoint(XBASE + 220, cbypos += 21), CBSIZ2);
	new MACHECKBOX(MPANEL, ID_CHK_INVERTRIGHT, wxT("InvertRight"), wxPoint(XBASE + 220, cbypos += 21), CBSIZ2, 1); // red
	new MACHECKBOX(MPANEL, ID_CHK_SINGLEPLAY, wxT("SinglePlay"), wxPoint(XBASE + 220, cbypos += 21), CBSIZ2, 1); // red
	// new MACHECKBOX(MPANEL, ID_CHK_USEM3U, wxT("Use m3u"), wxPoint(XBASE + 220, cbypos += 21), CBSIZ2);
	new MACHECKBOX(MPANEL, ID_CHK_TITLEINFO, wxT("TitleInfo"), wxPoint(XBASE + 220, cbypos += 21), CBSIZ2);
	new MACHECKBOX(MPANEL, ID_CHK_HOSTINENABLE, wxT("Enable"), wxPoint(XBASE + 220, cbypos += (21 + YPOSADJUST_CHK_ENABLE)), CBSIZ2);

	// mono/stereo
	t = new wxStaticText(MPANEL, ID_TEXT_OUTPUT, wxT("Output Stereo"),wxPoint(XBASE + 129, YBASE + 182));
	MASETFONT(t);
	new MACHECKBOX(MPANEL, ID_CHK_MONOLEFT, wxT("L"), wxPoint(XBASE + 129, YBASE + 204), CBSIZ4, 1); // red
	new MACHECKBOX(MPANEL, ID_CHK_MONORIGHT, wxT("R"), wxPoint(XBASE + 149, YBASE + 204), CBSIZ4, 1); // red
	new MACHECKBOX(MPANEL, ID_CHK_MONODIFF, wxT(""), wxPoint(XBASE + 173, YBASE + 204), CBSIZ4, 1); // red
	
	// popuplists	
	#define POPX (XBASE + 125)
	#define POPY (YBASE + 221)
	
	m_ComboOutput = new wxComboBox(MPANEL, ID_POPUP_OUTPUT, wxT(""), wxPoint(POPX, POPY), wxSize(85, 20), 0, NULL, wxCB_READONLY);
 	MASETFONT(m_ComboOutput); // doesn't on all platforms
	m_ComboOutput->Append(fs2ma(mastr2, g_extstrout[kOUTSOUND]));
 	machar label[STRLEN64];
	for(int i = kOUTFIRST; i <= kOUTLAST; i++)
	{
		masprintf(label, STRLEN64, L"%ls file", fs2ma(mastr2, &g_extstrout[i][1]));
		m_ComboOutput->Append(label);
 	}
 
	#define POPY2 (YBASE + 260)
	m_ComboBuffer = new wxComboBox(MPANEL, ID_POPUP_BUFFER, wxT(""),
												wxPoint(POPX + SPSIZ.x, POPY2), wxSize(69, 20), 0, NULL, wxCB_READONLY);
 	MASETFONT(m_ComboBuffer); // doesn't on all platforms
	m_ComboBuffer->Append(wxT("Mem"));
 	m_ComboBuffer->Append(wxT("Vdisk"));
 	m_ComboBuffer->Append(wxT("Direct"));

	t = new wxStaticText(MPANEL, ID_TEXT_BUFFER, wxT("Buffer 2047  MB"),wxPoint(POPX, POPY2 - 14));
	MASETFONT(t);
	s = new wxSpinButton(MPANEL, ID_SPIN_BUFFER, wxPoint(POPX, POPY2 - 1), SPSIZ, wxSP_VERTICAL | wxSP_WRAP);
	s->SetRange(INT_MIN, INT_MAX);
	s->SetValue(0);

	// audio device selection popup lists
	#define POPY3 (YBASE + 300)
	t = new wxStaticText(m_panel, ID_TEXT_AUDIOOUTPUT, AUDIOOUTPUTSTR, wxPoint(XBASE, POPY3 - 12));
	MASETFONT(t);
	m_ComboAudioOutput = new wxComboBox(MPANEL, ID_POPUP_AUDIOOUTPUT, wxT(""), wxPoint(XBASE, POPY3), wxSize(300, 20), 0, NULL, wxCB_READONLY);
 	MASETFONT(m_ComboAudioOutput); // doesn't on all platforms
	
	t = new wxStaticText(m_panel, ID_TEXT_AUDIOHOSTOUT, AUDIOHOSTOUTSTR, wxPoint(XBASE, POPY3 - 12 + 40));
	MASETFONT(t);
	m_ComboAudioHostOut = new wxComboBox(MPANEL, ID_POPUP_AUDIOHOSTOUT, wxT(""), wxPoint(XBASE, POPY3 + 40), wxSize(300, 20), 0, NULL, wxCB_READONLY);
 	MASETFONT(m_ComboAudioHostOut); // doesn't on all platforms

	t = new wxStaticText(m_panel, ID_TEXT_AUDIOINPUT, AUDIOINPUTSTR, wxPoint(XBASE, POPY3 - 12 + 80));
	MASETFONT(t);
	m_ComboAudioInput = new wxComboBox(MPANEL, ID_POPUP_AUDIOINPUT, wxT(""), wxPoint(XBASE, POPY3 + 80), wxSize(300, 20), 0, NULL, wxCB_READONLY);
 	MASETFONT(m_ComboAudioInput); // doesn't on all platforms

	t = new wxStaticText(m_panel, ID_TEXT_AUDIOHOSTIN, AUDIOHOSTINSTR, wxPoint(XBASE, POPY3 - 12 + 120));
	MASETFONT(t);
	m_ComboAudioHostIn = new wxComboBox(MPANEL, ID_POPUP_AUDIOHOSTIN, wxT(""), wxPoint(XBASE, POPY3 + 120), wxSize(300, 20), 0, NULL, wxCB_READONLY);
 	MASETFONT(m_ComboAudioHostIn); // doesn't on all platforms

	// Sizers	
	m_sizerFrame = new wxBoxSizer(wxVERTICAL);
	m_sizerFrame->Add(-1, SLIDERPOSY, 0, 0, 0); // spacer to m_sliderPos
	m_sizerTop = new wxBoxSizer(wxHORIZONTAL);
	m_sizerTop->Add(SLIDERPOSX, -1, 0, 0, 0); // spacer to left edge of m_sliderPos
	m_sizerTop->Add(m_sliderPos, 1, wxEXPAND);
	m_sizerFrame->Add(m_sizerTop, 0, wxEXPAND);
	MPANEL->SetSizer(m_sizerFrame);
#ifdef MADRAGANDDROP
	SetDropTarget(new DnDFile()); // does not work in main window on mac
#endif

} // MyFrameApp


MyFrameApp::~MyFrameApp()
{
	delete m_timerchangeplaylist;
	delete m_timerplayplaylist;
	delete m_timerrunloop;
	delete m_timerplay;
	// delete m_timerstop;	
}


void MyFrameApp::OnGUICheckAltSize(wxCommandEvent &event)
{
	DoAltSize(event.IsChecked() ? 1 : 0, 0);
}


void MyFrameApp::OnGUICheckRandom(wxCommandEvent &event)
{
	DoRandom(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUICheckReptPlayed(wxCommandEvent &event)
{
	DoReptPlayed(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUICheckLoopLoaded(wxCommandEvent &event)
{
	DoLoopLoaded(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUICheckLoopList(wxCommandEvent &event)
{
	DoLoopList(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUICheckEqWindow(wxCommandEvent & WXUNUSED(event))
{
	DoEqWindow();
}


void MyFrameApp::OnGUICheckListWindow(wxCommandEvent &event)
{
	DoListWindow(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUICheckBoxBeg(wxCommandEvent &event)
{
	DoBoxBeg(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUICheckBoxEnd(wxCommandEvent &event)
{
	DoBoxEnd(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUICheckMonoLeft(wxCommandEvent &event)
{
	DoMonoLeft(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUICheckMonoRight(wxCommandEvent &event)
{
	DoMonoRight(event.IsChecked() ? 1 : 0);
}


void MyFrameAdvanced::OnGUICheckUseM3u(wxCommandEvent &event)
{
	DoUseM3u(event.IsChecked() ? 1 : 0);
}

void MyFrameAdvanced::OnGUICheckExitAfter(wxCommandEvent &event)
{
	DoExitAfter(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUICheckSinglePlay(wxCommandEvent &event)
{
	DoSinglePlay(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUICheckEqualizer(wxCommandEvent &event)
{
	DoEqualizer(event.IsChecked() ? 1 : 0); 
}


void MyFrameApp::OnGUICheckEffects(wxCommandEvent &event)
{
	DoEffects(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUICheckLimiter(wxCommandEvent &event)
{
	DoLimiter(event.IsChecked() ? 1 : 0);
}


// void MyFrameApp::OnGUICheckEqExtra(wxCommandEvent &event)
// {
// 	DoEqExtra(event.IsChecked() ? 1 : 0);
// }


void MyFrameApp::OnGUICheckTitleInfo(wxCommandEvent & WXUNUSED(event))
{
	DoTitleInfo(0);
}


void MyFrameApp::OnGUICheckHostInEnable(wxCommandEvent &event)
{
	DoHostInEnable(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUICheckInvertRight(wxCommandEvent &event)
{
	DoInvertRight(event.IsChecked() ? 1 : 0);
}


// void MyFrameApp::OnGUICheckEq31Bands(wxCommandEvent &event)
// {
// 	DoEq31Bands(event.IsChecked() ? 1 : 0);
// }


void MyFrameApp::OnGUICheckReptSong(wxCommandEvent &event)
{
	DoReptSong(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUICheckMonoDiff(wxCommandEvent &event)
{
	DoMonoDiff(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUICheckBalance(wxCommandEvent &event)
{
	DoBalance(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUIButtonQuit(wxCommandEvent & WXUNUSED(event))
{
	DoAppQuit();
}


void MyFrameApp::OnGUIButtonStop(wxCommandEvent & WXUNUSED(event))
{
	DoStop();
}


void MyFrameApp::OnGUIButtonPrevious(wxCommandEvent & WXUNUSED(event))
{
	DoPrevious();
}


void MyFrameApp::OnGUIButtonPlay(wxCommandEvent & WXUNUSED(event))
{
	DoPlay();
}


void MyFrameApp::OnGUIButtonPause(wxCommandEvent & WXUNUSED(event))
{
	DoPause();
}


void MyFrameApp::OnGUIButtonNext(wxCommandEvent & WXUNUSED(event))
{
	DoNext();
}


void MyFrameApp::OnGUISliderVolumeL(wxCommandEvent & WXUNUSED(event))
{
	OnSliderVolumeL(SliderGetValue(WINDOWAPP, ID_SLIDER_VOLUMEL));
}


void MyFrameApp::OnGUISliderVolumeR(wxCommandEvent & WXUNUSED(event))
{
	OnSliderVolumeR(SliderGetValue(WINDOWAPP, ID_SLIDER_VOLUMER));
}


void MyFrameApp::OnGUISliderSeparation(wxCommandEvent & WXUNUSED(event))
{
	OnSliderSeparation(SliderGetValue(WINDOWAPP, ID_SLIDER_VOLUMESEP));
}


void MyFrameApp::OnGUISliderPreamp(wxCommandEvent & WXUNUSED(event))
{
	OnSliderPreamp(SliderGetValue(WINDOWAPP, ID_SLIDER_PREAMP));
}


void MyFrameApp::OnGUISliderPos(wxCommandEvent & WXUNUSED(event))
{
	OnSliderPosition(HSliderGetValue(WINDOWAPP, ID_SLIDER_POSITION));
}


void MyFrameApp::OnGUIPopupOutput(wxCommandEvent & WXUNUSED(event))
{
	m_ComboOutput->Refresh();
	OnPopupOutput(m_ComboOutput->GetSelection());
}


void MyFrameApp::OnGUIPopupBuffer(wxCommandEvent & WXUNUSED(event))
{
	m_ComboBuffer->Refresh();
	OnPopupBuffer(m_ComboBuffer->GetSelection());
}


void MyFrameApp::OnGUIPopupAudioOutput(wxCommandEvent & WXUNUSED(event))
{
	m_ComboAudioOutput->Refresh();
	OnPopupAudioOutput(m_ComboAudioOutput->GetSelection());

} // MyFrameApp::OnGUIPopupAudioOutput


void MyFrameApp::OnGUIPopupAudioHostOut(wxCommandEvent & WXUNUSED(event))
{
	m_ComboAudioHostOut->Refresh();
	OnPopupAudioHostOut(m_ComboAudioHostOut->GetSelection());

} // MyFrameApp::OnGUIPopupAudioHostIn
 
 
void MyFrameApp::OnGUIPopupAudioInput(wxCommandEvent & WXUNUSED(event))
{
	m_ComboAudioInput->Refresh();
	OnPopupAudioInput(m_ComboAudioInput->GetSelection());

} // MyFrameApp::OnGUIPopupAudioInput


void MyFrameApp::OnGUIPopupAudioHostIn(wxCommandEvent & WXUNUSED(event))
{
	m_ComboAudioHostIn->Refresh();
	OnPopupAudioHostIn(m_ComboAudioHostIn->GetSelection());

} // MyFrameApp::OnGUIPopupAudioHostIn


void MyFrameApp::OnGUISpinUpMem(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowBuffer(1);
}


void MyFrameApp::OnGUISpinDownBuffer(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowBuffer(-1);
}


void MyFrameApp::OnGUISpinUpPreamp(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowPreamp(1);
}


void MyFrameApp::OnGUISpinDownPreamp(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowPreamp(-1);
}


void MyFrameApp::OnGUISpinUpSep(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowSeparation(1);
}


void MyFrameApp::OnGUISpinDownSep(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowSeparation(-1);
}


void MyFrameApp::OnGUISpinUpVolR(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowVolumeR(1);
}


void MyFrameApp::OnGUISpinDownVolR(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowVolumeR(-1);
}


void MyFrameApp::OnGUISpinUpVolL(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowVolumeL(1);
}


void MyFrameApp::OnGUISpinDownVolL(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowVolumeL(-1);
}


void MyFrameApp::OnGUISpinUpBeg(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowBegin(1);
}


void MyFrameApp::OnGUISpinDownBeg(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowBegin(-1);
}


void MyFrameApp::OnGUISpinUpEnd(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowEnd(1);
}


void MyFrameApp::OnGUISpinDownEnd(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowEnd(-1);
}


#if defined(MAUSEMABUTTON)
 void MyFrameApp::MakeButtons()
 {
	wxStaticText *t;
	machar label[STRLEN32];
	
	wxImage *upOriginal = new wxImage(wxBitmap(BMUpButton).ConvertToImage());
	wxImage *downOriginal = new wxImage(wxBitmap(BMDownButton).ConvertToImage());
	wxImage *hiliteOriginal = new wxImage(wxBitmap(BMHiliteButton).ConvertToImage());

 #if defined(__MAOSX__) || defined(__MAUNIX__)
	m_upPattern = upOriginal;
	m_downPattern = downOriginal;
	m_hilitePattern = hiliteOriginal;
 #else
	wxColour newColour = wxSystemSettingsNative::GetColour(wxSYS_COLOUR_3DFACE);
	m_upPattern = ChangeImageColour(upOriginal, newColour);
	m_downPattern = ChangeImageColour(downOriginal, newColour);
	m_hilitePattern = ChangeImageColour(hiliteOriginal, newColour);
 #endif

	mastrcpy(label, LABEL_QUIT);
 #if defined(__WXX11__)
	mastrcat(label, L"   "); // x11 needs 3 spaces for shortcuts
 #endif
	t = new wxStaticText(MPANEL, ID_TEXT_BUTTON_QUIT, label, wxPoint(m_ButtonPos + 14, BUTTONTEXTY));
	MASETFONT(t);
	MakeButton((char const **)BMQuit, (char const **)BMQuitDisabled, (char const **)BMQuitMask, ID_BTN_QUIT, true);

	mastrcpy(label, LABEL_PREV);
 #if defined(__WXX11__)
	mastrcat(label, L"   ");
 #endif
	t = new wxStaticText(MPANEL, ID_TEXT_BUTTON_PREVIOUS, label, wxPoint(m_ButtonPos + 6, BUTTONTEXTY));
	MASETFONT(t);
	MakeButton((char const **)BMRewind, (char const **)BMRewindDisabled, (char const **)BMRewindMask, ID_BTN_PREVIOUS, true);

	mastrcpy(label, LABEL_PLAY);
 #if defined(__WXX11__)
	mastrcat(label, L"   ");
 #endif
	t = new wxStaticText(MPANEL, ID_TEXT_BUTTON_PLAY, label, wxPoint(m_ButtonPos + 4, BUTTONTEXTY));
	MASETFONT(t);
	MakeButton((char const **)BMPlay, (char const **)BMPlayDisabled, (char const **)BMPlayMask, ID_BTN_PLAY, true);

	mastrcpy(label, LABEL_PAUSE);
 #if defined(__WXX11__)
	mastrcat(label, L"   ");
 #endif
	t = new wxStaticText(MPANEL, ID_TEXT_BUTTON_PAUSE, label, wxPoint(m_ButtonPos, BUTTONTEXTY));
	MASETFONT(t);
	MakeButton((char const **)BMPause, (char const **)BMPauseDisabled, (char const **)BMPauseMask, ID_BTN_PAUSE, true);
	
	mastrcpy(label, LABEL_STOP);
 #if defined(__WXX11__)
	mastrcat(label, L"   ");
 #endif
	t = new wxStaticText(MPANEL, ID_TEXT_BUTTON_STOP, label, wxPoint(m_ButtonPos + 4, BUTTONTEXTY));
	MASETFONT(t);
	MakeButton((char const **)BMStop, (char const **)BMStopDisabled, (char const **)BMStopMask, ID_BTN_STOP, true);

	mastrcpy(label, LABEL_NEXT);
 #if defined(__WXX11__)
	mastrcat(label, L"   ");
 #endif
	t = new wxStaticText(MPANEL, ID_TEXT_BUTTON_NEXT, label, wxPoint(m_ButtonPos + 4, BUTTONTEXTY));
	MASETFONT(t);
	MakeButton((char const **)BMFFwd, (char const **)BMFFwdDisabled, (char const **)BMFFwdMask, ID_BTN_NEXT, true);

 #if defined(__MAOSX__) || defined(__MAUNIX__)
 #else
	delete m_upPattern;
	delete m_downPattern;
	delete m_hilitePattern;
 #endif

	delete upOriginal;
	delete downOriginal;
	delete hiliteOriginal;

 } // MakeButtons


 #define BUTTONOFFSETX 16
 #define BUTTONOFFSETY 16
 MABUTTON *MyFrameApp::MakeButton(char const **foreground, char const **disabled,
								char const **foremask, int id, bool processdownevents)
 {
	wxImage *color		= new wxImage(wxBitmap(foreground).ConvertToImage());
	wxImage *color_dis	= new wxImage(wxBitmap(disabled).ConvertToImage());
	wxImage *mask 		= new wxImage(wxBitmap(foremask).ConvertToImage());

	wxImage *up2 		= OverlayImage(m_upPattern,		color,		mask, BUTTONOFFSETX,		BUTTONOFFSETY);
	wxImage *hilite2 	= OverlayImage(m_hilitePattern, color,		mask, BUTTONOFFSETX,		BUTTONOFFSETY);
	wxImage *down2		= OverlayImage(m_downPattern,	color,		mask, BUTTONOFFSETX + 1,	BUTTONOFFSETY + 1);
	wxImage *disable2 	= OverlayImage(m_upPattern,		color_dis,	mask, BUTTONOFFSETX,		BUTTONOFFSETY);

	MABUTTON *button	= new MABUTTON(MPANEL, id, wxPoint(m_ButtonPos, m_buttonTop),
										MABUTTONSIZE, up2, hilite2, down2, disable2, processdownevents);

	delete color;
	delete color_dis;
	delete mask;
	delete up2;
	delete down2;
	delete hilite2;
	delete disable2;

	m_ButtonPos += MABUTTONW;
	
	return button;
	
 } // MakeButton
#else // !MAUSEMABUTTON
 void MyFrameApp::MakeButtons()
 {
	wxStaticText *t;
	machar label[STRLEN32];

	mastrcpy(label, LABEL_QUIT);
 #if defined(__WXX11__)
	mastrcat(label, L"   "); // x11 needs 3 spaces for shortcuts
 #endif
	t = new wxStaticText(MPANEL, ID_TEXT_BUTTON_QUIT, wxT("   "), wxPoint(m_ButtonPos + 14, BUTTONTEXTY));
	MASETFONT(t);
	MakeButton(ID_BTN_QUIT, label);

	mastrcpy(label, LABEL_PREV);
 #if defined(__WXX11__)
	mastrcat(label, L"   ");
 #endif
	t = new wxStaticText(MPANEL, ID_TEXT_BUTTON_PREVIOUS, wxT("   "), wxPoint(m_ButtonPos + 6, BUTTONTEXTY));
	MASETFONT(t);
	MakeButton(ID_BTN_PREVIOUS, label);
	
	mastrcpy(label, LABEL_PLAY);
 #if defined(__WXX11__)
	mastrcat(label, L"   ");
 #endif
	t = new wxStaticText(MPANEL, ID_TEXT_BUTTON_PLAY, wxT("   "), wxPoint(m_ButtonPos + 4, BUTTONTEXTY));
	MASETFONT(t);
	MakeButton(ID_BTN_PLAY, label);

	mastrcpy(label, LABEL_PAUSE);
 #if defined(__WXX11__)
	mastrcat(label, L"   ");
 #endif
	t = new wxStaticText(MPANEL, ID_TEXT_BUTTON_PAUSE, wxT("   "), wxPoint(m_ButtonPos, BUTTONTEXTY));
	MASETFONT(t);
	MakeButton(ID_BTN_PAUSE, label);
	
	mastrcpy(label, LABEL_STOP);
 #if defined(__WXX11__)
	mastrcat(label, L"   ");
 #endif
	t = new wxStaticText(MPANEL, ID_TEXT_BUTTON_STOP, wxT("   "), wxPoint(m_ButtonPos + 4, BUTTONTEXTY));
	MASETFONT(t);
	MakeButton(ID_BTN_STOP, label);

	mastrcpy(label, LABEL_NEXT);
 #if defined(__WXX11__)
	mastrcat(label, L"   ");
 #endif
	t = new wxStaticText(MPANEL, ID_TEXT_BUTTON_NEXT, wxT("   "), wxPoint(m_ButtonPos + 4, BUTTONTEXTY));
	MASETFONT(t);
	MakeButton(ID_BTN_NEXT, label);

 } // MakeButtons


 #define BUTTONOFFSETX 16
 #define BUTTONOFFSETY 16
 MABUTTON *MyFrameApp::MakeButton(int id, machar *label)
 {
	MABUTTON *button = new MABUTTON(MPANEL, id, label, wxPoint(m_ButtonPos, m_buttonTop), MABUTTONSIZE);

	m_ButtonPos += MABUTTONW;
	
	return button;
	
 } // MakeButton
#endif // !MAUSEMABUTTON


#if (MAGUIWX == 4) // wx-2.4.2
 #define APPENDSEPARATOR() Append(-1, wxT(""))
 #define APPENDSUBMENU(a, b, c) Append(a, b, c)
#else
 #define APPENDSEPARATOR() AppendSeparator()
 #define APPENDSUBMENU(a, b, c) Append((int)a, (wxString const&)b, (wxMenu*)c, (wxString const&)( wxT("")))
#endif

void MyFrameApp::MakeMenus()
{
	machar mastr[MAXLOGLINELEN];
	
	// submenus
	l_theMenu[MENUTITLEINFO] = new wxMenu;

	((wxMenu *)l_theMenu[MENUTITLEINFO])->Append(ID_SUBMENU_TITLEINFO1, wxT("List Name"), wxT(""), wxITEM_CHECK);
	((wxMenu *)l_theMenu[MENUTITLEINFO])->Append(ID_SUBMENU_TITLEINFO2, wxT("Song Name"), wxT(""), wxITEM_CHECK);
	((wxMenu *)l_theMenu[MENUTITLEINFO])->Append(ID_SUBMENU_TITLEINFO3, wxT("Numbers"),	wxT(""), wxITEM_CHECK);

	l_theMenu[MENUOPENRECENT] = new wxMenu;
	MakeSubMenusRecent();
	
	// file menu
	l_theMenu[MENUFILES] = new wxMenu;
	((wxMenu *)l_theMenu[MENUFILES])->Append(ID_MENU_OPENLIST,  wxT("Open Playlist..."));
	((wxMenu *)l_theMenu[MENUFILES])->Append(ID_MENU_MAKELIST,	wxT("Make Playlist..."));
	((wxMenu *)l_theMenu[MENUFILES])->Append(ID_MENU_SAVELISTAS,wxT("Save Playlist As..."));
	((wxMenu *)l_theMenu[MENUFILES])->APPENDSEPARATOR();
	((wxMenu *)l_theMenu[MENUFILES])->Append(ID_MENU_DELETELIST,wxT("Delete Playlist..."));
	((wxMenu *)l_theMenu[MENUFILES])->APPENDSEPARATOR();
	((wxMenu *)l_theMenu[MENUFILES])->Append(ID_MENU_PLAYFILE,  wxT("Play Song Files..."));
	((wxMenu *)l_theMenu[MENUFILES])->APPENDSEPARATOR();
	((wxMenu *)l_theMenu[MENUFILES])->APPENDSUBMENU(ID_MENU_OPENRECENT, wxT("Open Recent..."), (wxMenu *)l_theMenu[MENUOPENRECENT]);
	((wxMenu *)l_theMenu[MENUFILES])->APPENDSEPARATOR();
	((wxMenu *)l_theMenu[MENUFILES])->Append(ID_MENU_QUIT,		wxT("Quit"));

	// controls menu
	l_theMenu[MENUMACONTROLS] = new wxMenu;
	((wxMenu *)l_theMenu[MENUMACONTROLS])->Append(ID_MENU_STOP,		wxT("Stop"));
	((wxMenu *)l_theMenu[MENUMACONTROLS])->Append(ID_MENU_PAUSE,	wxT("Pause"), wxT(""), wxITEM_CHECK);
	((wxMenu *)l_theMenu[MENUMACONTROLS])->Append(ID_MENU_PREVIOUS,	wxT("Previous"));
	((wxMenu *)l_theMenu[MENUMACONTROLS])->Append(ID_MENU_PLAY,		wxT("Play"));
	((wxMenu *)l_theMenu[MENUMACONTROLS])->Append(ID_MENU_NEXT,		wxT("Next"));
	((wxMenu *)l_theMenu[MENUMACONTROLS])->APPENDSEPARATOR();
	((wxMenu *)l_theMenu[MENUMACONTROLS])->Append(ID_MENU_PLAYCD,	wxT("Play CD"));
	((wxMenu *)l_theMenu[MENUMACONTROLS])->Append(ID_MENU_EJECTCD,	wxT("Eject CD"));
	((wxMenu *)l_theMenu[MENUMACONTROLS])->APPENDSEPARATOR();
	((wxMenu *)l_theMenu[MENUMACONTROLS])->Append(ID_MENU_AUDIOIN,	wxT("Audio In"));

	// settings menu
	l_theMenu[MENUMASETS] = new wxMenu;
	((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_SAVEMASETS,		wxT("Save Settings"));
	((wxMenu *)l_theMenu[MENUMASETS])->APPENDSEPARATOR();
	((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_SETSAVED,			wxT("Saved Settings"));
	((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_SETDEFAULT,		wxT("Default Settings"));
	((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_SETFACTORY,		wxT("Factory Settings"));
	((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_SETCOPY,			wxT("Copy Settings..."));
	((wxMenu *)l_theMenu[MENUMASETS])->APPENDSEPARATOR();
	((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_SAVEMASETSDEFAULT,wxT("Save Settings As Default"));
	((wxMenu *)l_theMenu[MENUMASETS])->APPENDSEPARATOR();
	((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_EQUALIZER,		wxT("Equalizer"), wxT(""), wxITEM_CHECK);
	((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_EQ31BANDS,		wxT("Eq 31 Bands"), wxT(""), wxITEM_CHECK);
	((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_EQEXTRA,			wxT("EqExtra"), wxT(""), wxITEM_CHECK);
	((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_EFFECTS,			wxT("Effects"), wxT(""), wxITEM_CHECK);
	((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_RANDOM,			wxT("Random"), wxT(""), wxITEM_CHECK);
	((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_REPTPLAYED,		wxT("Repeat Played"), wxT(""), wxITEM_CHECK);
	((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_LOOPLOADED,		wxT("Loop Loaded"), wxT(""), wxITEM_CHECK);
	((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_LOOPLIST,			wxT("Loop List"), wxT(""), wxITEM_CHECK);
	((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_MONO,				wxT("Mono"), wxT(""), wxITEM_CHECK);
	((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_INVERTRIGHT,		wxT("Invert Right"), wxT(""), wxITEM_CHECK);
	((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_SINGLEPLAY,		wxT("Single Play"), wxT(""), wxITEM_CHECK);
	((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_INPUTMONITOR,		wxT("Input Monitor"), wxT(""), wxITEM_CHECK);
	
	((wxMenu *)l_theMenu[MENUMASETS])->APPENDSUBMENU(ID_MENU_TITLEINFO, wxT("Title Info..."), (wxMenu *)l_theMenu[MENUTITLEINFO]);

	((wxMenu *)l_theMenu[MENUMASETS])->APPENDSEPARATOR();
	((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_AUTOSTART,		wxT("Auto Start"), wxT(""), wxITEM_CHECK);
	((wxMenu *)l_theMenu[MENUMASETS])->APPENDSEPARATOR();
	((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_ERRLOGGING,		wxT("Error Logging"), wxT(""), wxITEM_CHECK);
	((wxMenu *)l_theMenu[MENUMASETS])->APPENDSEPARATOR();
	((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_SETVDISK,			wxT("Set Virtual Disk..."));
#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
	((wxMenu *)l_theMenu[MENUMASETS])->APPENDSEPARATOR();
	((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_SETAUDIOCD,		wxT("Set Audio CD Drives..."));
#endif
	if(g_thisinstance == 1)
	{
		((wxMenu *)l_theMenu[MENUMASETS])->APPENDSEPARATOR();
		((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_MULTIPLEINST,	wxT("Multiple Instances"), wxT(""), wxITEM_CHECK);
	}
#if defined(MAKEYBOARD)
	((wxMenu *)l_theMenu[MENUMASETS])->APPENDSEPARATOR();
	((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_KEYBOARD,		wxT("Use Keyboard"), wxT(""), wxITEM_CHECK);	
#endif
#if defined(MAMOUSEWHEEL)
	((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_MOUSEWHEEL,	wxT("Use MouseWheel"), wxT(""), wxITEM_CHECK);	
#endif
	((wxMenu *)l_theMenu[MENUMASETS])->APPENDSEPARATOR();
	((wxMenu *)l_theMenu[MENUMASETS])->Append(ID_MENU_ADVANCED,		wxT("Advanced Settings..."));
	// tools menu
	l_theMenu[MENUTOOLS] = new wxMenu;
	((wxMenu *)l_theMenu[MENUTOOLS])->Append(ID_MENU_ZEROLIST,		wxT("Zero List"));
	((wxMenu *)l_theMenu[MENUTOOLS])->APPENDSEPARATOR();
	((wxMenu *)l_theMenu[MENUTOOLS])->Append(ID_MENU_MAKECATALOG,	wxT("Make Catalog"));
#if !defined(NOTEXTVIEWER)
	((wxMenu *)l_theMenu[MENUTOOLS])->Append(ID_MENU_VIEWCATALOG,	wxT("Edit Catalog"));
	((wxMenu *)l_theMenu[MENUTOOLS])->APPENDSEPARATOR();
	((wxMenu *)l_theMenu[MENUTOOLS])->Append(ID_MENU_VIEWPLAYLIST,	wxT("Edit Playlist"));
#endif
	((wxMenu *)l_theMenu[MENUTOOLS])->APPENDSEPARATOR();
#if !defined(NOTEXTVIEWER)
	((wxMenu *)l_theMenu[MENUTOOLS])->Append(ID_MENU_VIEWLOG,		wxT("Edit Log"));
#endif
	((wxMenu *)l_theMenu[MENUTOOLS])->Append(ID_MENU_CLEARLOG,		wxT("Clear Log"));
#if !defined(NOTEXTVIEWER)
 #if defined(MAKEYBOARD)
	((wxMenu *)l_theMenu[MENUTOOLS])->APPENDSEPARATOR();
	((wxMenu *)l_theMenu[MENUTOOLS])->Append(ID_MENU_EDITKEYBINDINGS,wxT("Edit Key Bindings"));
	((wxMenu *)l_theMenu[MENUTOOLS])->Append(ID_MENU_BLANKKEYBINDINGS,wxT("Blank Key Bindings"));
	((wxMenu *)l_theMenu[MENUTOOLS])->Append(ID_MENU_FACTORYKEYBINDINGS,wxT("Factory Key Bindings"));
 #endif
#endif

	((wxMenu *)l_theMenu[MENUTOOLS])->APPENDSEPARATOR();
	((wxMenu *)l_theMenu[MENUTOOLS])->Append(ID_MENU_RESCANAUDIODEVICES,wxT("Rescan Audio Devices"));

	// window menu
	l_theMenu[MENUSHOW] = new wxMenu;
	((wxMenu *)l_theMenu[MENUSHOW])->Append(ID_MENU_SHOWLIST,		wxT("Lists"),		wxT(""), wxITEM_CHECK);
	((wxMenu *)l_theMenu[MENUSHOW])->Append(ID_MENU_SHOWEQ,			wxT("Equalizer"),	wxT(""), wxITEM_CHECK);
	((wxMenu *)l_theMenu[MENUSHOW])->Append(ID_MENU_SHOWEFFECTS,	wxT("Effects"),		wxT(""), wxITEM_CHECK);
	((wxMenu *)l_theMenu[MENUSHOW])->Append(ID_MENU_SHOWFILEOUT,	wxT("File Output"),	wxT(""), wxITEM_CHECK);
	((wxMenu *)l_theMenu[MENUSHOW])->APPENDSEPARATOR();
	((wxMenu *)l_theMenu[MENUSHOW])->Append(ID_MENU_ALTSIZE,		wxT("Alt Sizes"),	 wxT(""), wxITEM_CHECK);
	((wxMenu *)l_theMenu[MENUSHOW])->Append(ID_MENU_SAVEDSIZE ,		wxT("Saved Sizes"),  wxT(""));
	((wxMenu *)l_theMenu[MENUSHOW])->Append(ID_MENU_DEFAULTSIZE ,	wxT("Default Sizes"),wxT(""));
	((wxMenu *)l_theMenu[MENUSHOW])->Append(ID_MENU_FACTORYSIZE,	wxT("Factory Sizes"),wxT(""));

	l_theMenu[MENUHELP] = new wxMenu;
 #if !defined(NOTEXTVIEWER)
	((wxMenu *)l_theMenu[MENUHELP])->Append(ID_MENU_README, wxT("README"));
 #endif
	masprintf(mastr, MAXLOGLINELEN, L"About %ls", MOREAMP_NAME);
	((wxMenu *)l_theMenu[MENUHELP])->Append(ID_MENU_ABOUT, mastr);
	
#if defined(__WXMAC__)
	 // About menu item gets relocated to Apple menu
	 wxApp::s_macAboutMenuItemId = ID_MENU_ABOUT;
#endif

	// menu bar
	wxMenuBar *menuBar = new wxMenuBar();
	g_menubar = menuBar;
	menuBar->Append((wxMenu *)l_theMenu[MENUFILES],
#if defined(__WXMAC__)
															wxT("&File")); // need '&File' for Apple menu
#else
															wxT("File"));
#endif
	menuBar->Append((wxMenu *)l_theMenu[MENUMACONTROLS],	wxT("Controls"));
	menuBar->Append((wxMenu *)l_theMenu[MENUMASETS],		wxT("Settings"));
	menuBar->Append((wxMenu *)l_theMenu[MENUTOOLS],			wxT("Tools"));
	menuBar->Append((wxMenu *)l_theMenu[MENUSHOW],			wxT("Show"));
	menuBar->Append((wxMenu *)l_theMenu[MENUHELP],
#if defined(__WXMAC__)
															wxT("&Help")); // need '&Help' for Apple menu
#else
															wxT("Help"));
#endif
	SetMenuBar(menuBar);

} // MakeMenus


void MyFrameApp::MakeSubMenusRecent()
{
	int i, n;
	fschar fname[MAX_PATH];
	fschar items[MAXOPENRECENT][MAX_PATH];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
#endif

	n = (int)((wxMenu *)l_theMenu[MENUOPENRECENT])->GetMenuItemCount();
	for (i = 0; i < n; i++)
		((wxMenu *)l_theMenu[MENUOPENRECENT])->Destroy(ID_SUBMENU_OPENRECENT1 + i);
		
	if(!GetRecent(items))
		for (i = 0; i < MAXOPENRECENT; i++)
		{
			if(!items[i][0])
				break;
			 FilenameFromPathname(items[i], fname);
			((wxMenu *)l_theMenu[MENUOPENRECENT])->Append(ID_SUBMENU_OPENRECENT1 + i, fs2ma(mastr2, fname));
		}

} // MakeSubMenusRecent

	
void MyFrameApp::OnAbout(wxCommandEvent & WXUNUSED(event))
{
	DoWindow(WINDOWABOUT, 1);
 }


void MyFrameApp::OnGUIMenuQuit(wxCommandEvent & WXUNUSED(event))
{
	DoAppQuit();
}


void MyFrameApp::OnGUIMenuSetVDisk(wxCommandEvent & WXUNUSED(event))
{
	DoSetVDisk();
}


void MyFrameApp::OnGUIMenuMultipleInst(wxCommandEvent &event)
{
	DoMultipleInst(event.IsChecked() ? 1 : 0);
}


#if defined(MAKEYBOARD)
void MyFrameApp::OnGUIMenuKeyboard(wxCommandEvent &event)
{
	DoKeyboard(event.IsChecked() ? 1 : 0);
}
#endif


#if defined(MAMOUSEWHEEL)
void MyFrameApp::OnGUIMenuMouseWheel(wxCommandEvent &event)
{
	DoMouseWheel(event.IsChecked() ? 1 : 0);
}
#endif


void MyFrameApp::OnGUIMenuAdvanced(wxCommandEvent & WXUNUSED(event))
{
	DoWindow(WINDOWADVANCED, 1);
}


#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
void MyFrameApp::OnGUIMenuSetsAudioCD(wxCommandEvent &event)
{
 (void)event;

	DoWindow(WINDOWAUDIOCD, 1);
}
#endif


void MyFrameApp::OnGUIMenuSaveSettings(wxCommandEvent & WXUNUSED(event))
{
	DoSave();
}


void MyFrameApp::OnGUIMenuSaveSettingsDefault(wxCommandEvent & WXUNUSED(event))
{
	DoAsDefaults();
}


void MyFrameApp::OnGUIMenuSetsFactory(wxCommandEvent & WXUNUSED(event))
{
	DoSetsFactory();
}


void MyFrameApp::OnGUIMenuSetsDefault(wxCommandEvent & WXUNUSED(event))
{
	DoSetsDefault();
}


void MyFrameApp::OnGUIMenuSetsSaved(wxCommandEvent & WXUNUSED(event))
{
	DoSetsSaved();
}


void MyFrameApp::OnGUIMenuSetsCopy(wxCommandEvent & WXUNUSED(event))
{
	DoSetsCopy();
}


void MyFrameApp::OnGUIMenuStop(wxCommandEvent & WXUNUSED(event))
{
	DoStop();
}


void MyFrameApp::OnGUIMenuPause(wxCommandEvent & WXUNUSED(event))
{
	DoPause();
}


void MyFrameApp::OnGUIMenuPrevious(wxCommandEvent & WXUNUSED(event))
{
	DoPrevious();
}


void MyFrameApp::OnGUIMenuPlay(wxCommandEvent & WXUNUSED(event))
{
	DoPlay();
}


void MyFrameApp::OnGUIMenuNext(wxCommandEvent & WXUNUSED(event))
{
	DoNext();
}


void MyFrameApp::OnGUIMenuPlayCD(wxCommandEvent & WXUNUSED(event))
{
	DoPlayCD();
}


void MyFrameApp::OnGUIMenuTitleInfo(wxCommandEvent &event)
{
	if(event.GetId() == ID_SUBMENU_TITLEINFO2)
		DoTitleInfo(2);
	else if(event.GetId() == ID_SUBMENU_TITLEINFO3)
		DoTitleInfo(3);
	else
		DoTitleInfo(1);
}


void MyFrameApp::OnGUIMenuRandom(wxCommandEvent &event)
{
	DoRandom(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUIMenuReptPlayed(wxCommandEvent &event)
{
	DoReptPlayed(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUIMenuLoopLoaded(wxCommandEvent &event)
{
	DoLoopLoaded(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUIMenuLoopList(wxCommandEvent &event)
{
	DoLoopList(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUIMenuAutoStart(wxCommandEvent &event)
{
	g_sets.autostart = event.IsChecked() ? 1 : 0;
}


void MyFrameApp::OnGUIMenuErrLogging(wxCommandEvent &event)
{
	DoErrLogging(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUIMenuSinglePlay(wxCommandEvent &event)
{
	DoSinglePlay(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUIMenuInMonitor(wxCommandEvent &event)
{
	DoInMonitor(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUIMenuEqualizer(wxCommandEvent &event)
{
	DoEqualizer(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUIMenuEqExtra(wxCommandEvent &event)
{
	DoEqExtra(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUIMenuEffects(wxCommandEvent &event)
{
	DoEffects(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUIMenuMono(wxCommandEvent &event)
{
	DoMono(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUIMenuInvertRight(wxCommandEvent &event)
{
	DoInvertRight(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUIMenuEq31Bands(wxCommandEvent &event)
{
	DoEq31Bands(event.IsChecked() ? 1 : 0);
}


void MyFrameApp::OnGUIMenuOpenList(wxCommandEvent & WXUNUSED(event))
{
	DoOpenList();
}


void MyFrameApp::OnGUIMenuMakeList(wxCommandEvent & WXUNUSED(event))
{
	DoMakeList();
}


void MyFrameApp::OnGUIMenuSaveListAs(wxCommandEvent & WXUNUSED(event))
{
	DoSaveListAs();
}


void MyFrameApp::OnGUIMenuDeleteList(wxCommandEvent & WXUNUSED(event))
{
	DoDeleteList();
}


void MyFrameApp::OnGUIMenuPlayFile(wxCommandEvent & WXUNUSED(event))
{
	DoPlayFile();
}


void MyFrameApp::OnGUIMenuOpenRecent(wxCommandEvent &event)
{
	DoOpenRecent(event.GetId() - ID_SUBMENU_OPENRECENT1);
}


void MyFrameApp::OnGUIMenuZeroList(wxCommandEvent & WXUNUSED(event))
{
	DoZeroList();
}


void MyFrameApp::OnGUIMenuEjectCD(wxCommandEvent & WXUNUSED(event))
{
	DoEjectCD();
}


void MyFrameApp::OnGUIMenuAudioIn(wxCommandEvent & WXUNUSED(event))
{
	DoAudioIn();
}


void MyFrameApp::OnGUIMenuMakeCatalog(wxCommandEvent & WXUNUSED(event))
{
	DoMakeCatalog();
}


void MyFrameApp::OnGUIMenuShowList(wxCommandEvent & WXUNUSED(event))
{
	ToggleWindow(WINDOWLIST);
}


void MyFrameApp::OnGUIMenuShowEq(wxCommandEvent & WXUNUSED(event))
{
	DoEqWindow();
}


void MyFrameApp::OnGUIMenuShowEffects(wxCommandEvent & WXUNUSED(event))
{
	ToggleWindow(WINDOWEFFECTS);
}


void MyFrameApp::OnGUIMenuShowFileOut(wxCommandEvent & WXUNUSED(event))
{
	ToggleWindow(WINDOWFILEOUT);
}


void MyFrameApp::OnGUICloseRequest(wxCloseEvent & WXUNUSED(event)) // user clicked a App window's close icon X
{
	DoAppQuit();	
}


void MyFrameApp::OnGUIMenuAltSize(wxCommandEvent &event)
{
	DoAltSize(event.IsChecked() ? 1 : 0, 0);
			
} // OnGUIMenuAltSize


void MyFrameApp::OnGUIMenuSavedSize(wxCommandEvent & WXUNUSED(event))
{
	DoSavedSize();
								
} // OnGUIMenuSavedSize


void MyFrameApp::OnGUIMenuDefaultSize(wxCommandEvent & WXUNUSED(event))
{
	DoDefaultSize();
								
} // OnGUIMenuDefaultSize


void MyFrameApp::OnGUIMenuFactorySize(wxCommandEvent & WXUNUSED(event))
{
	DoFactorySize();

} // OnGUIMenuFactorySize


#if !defined(NOTEXTVIEWER)
 void MyFrameApp::OnGUIMenuReadme(wxCommandEvent & WXUNUSED(event))
 {
	DoWindow(WINDOWREADME, 1);

 } // OnGUIMenuReadme


 void MyFrameApp::OnGUIMenuViewCatalog(wxCommandEvent & WXUNUSED(event))
 {
	DoViewCatalog(0);
 }


 void MyFrameApp::OnGUIMenuViewPlaylist(wxCommandEvent & WXUNUSED(event))
 {
	DoWindow(WINDOWPLAYLIST, 1);
 }


 void MyFrameApp::OnGUIMenuViewLog(wxCommandEvent & WXUNUSED(event))
 {
	DoWindow(WINDOWLOG, 1);	
 }

 #if defined(MAKEYBOARD)
  void MyFrameApp::OnGUIMenuEditKeyBindings(wxCommandEvent & WXUNUSED(event))
  {
	DoEditKeyBindings();	
  }


  void MyFrameApp::OnGUIMenuBlankKeyBindings(wxCommandEvent & WXUNUSED(event))
  {
	DoBlankKeyBindings();	
  }


  void MyFrameApp::OnGUIMenuFactoryKeyBindings(wxCommandEvent & WXUNUSED(event))
  {
	DoFactoryKeyBindings();	
  }
 #endif
#endif



void MyFrameApp::OnGUIMenuRescanAudioDevices(wxCommandEvent & WXUNUSED(event))
{
	DoRescanAudioDevices(); 
}

	

void MyFrameApp::OnGUIMenuClearLog(wxCommandEvent & WXUNUSED(event))
{
	maLog(&g_logcmdclear);	
}


void MyFrameApp::OnTimerChangePlaylist(wxTimerEvent &event)
{
	(void)event;

	m_timerchangeplaylist->Stop();
	ChangePlaylist(g_changeplaylist);
	
} // OnTimerChangePlaylist


void MyFrameApp::OnTimerPlayPlaylist(wxTimerEvent &event)
{
	(void)event;

	m_timerplayplaylist->Stop();
	PlayPlaylistB(g_newplaylist);
	
} // OnTimerPlayPlaylist


/*
void MyFrameApp::OnTimerStop(wxTimerEvent & WXUNUSED(event))
{
	m_timerstop->Stop();
	PushButtonSimulateClick(WINDOWAPP, ID_BTN_STOP);
	
} // OnTimerStop
*/


void MyFrameApp::OnTimerRunLoop(wxTimerEvent & WXUNUSED(event))
{
	m_timerrunloop->Stop();
	RunLoop();
	
} // OnTimerRunLoop


void MyFrameApp::OnTimerPlay(wxTimerEvent & WXUNUSED(event))
{
	m_timerplay->Stop();
	PushButtonSimulateClick(WINDOWAPP, ID_BTN_PLAY);
	
} // OnTimerPlay


MyFrameList::MyFrameList(wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& siz, long style)
						: wxFrame((wxWindow *)NULL, id, title, pos, siz, style)
{
	int mx; // e.g. 1024
	int my; // e.g.  768
	wxSize maxsize;
	
	DisplayGetSize(&mx, &my);
	maxsize.x = mx;
	maxsize.y = my;

#if !defined(MANOWXPANEL)
	m_panel = new wxPanel(this, -1, wxDefaultPosition, maxsize, wxCLIP_CHILDREN | wxNO_BORDER);
#endif

#if defined(MAKEYBOARD)
	m_cbdummy =  // for maOnChar focus
		new wxCheckBox(MPANEL, wxID_ANY, wxT(""), wxPoint(XBASE, YBASE + MENUBARH), wxSize(1, 1));
#endif

	// list buttons
	m_ButtonPos = XBASE;
	m_buttonTop = YBASE + MENUBARH; // line up buttons with main window buttons
	MakeButtonsList();
	
	// list checkboxes
 #if defined(__WXGTK__) // adjust ypos
	new MACHECKBOX(MPANEL, ID_CHK_SHOWINDEX, wxT("Show index"), wxPoint(m_ButtonPos + 4, YBASE + MENUBARH + 3 - 2), CBSIZ);
	new MACHECKBOX(MPANEL, ID_CHK_SHOWDIR,   wxT("Show dir"),   wxPoint(m_ButtonPos + 4, YBASE + MENUBARH + 19 - 2), CBSIZ);
	new MACHECKBOX(MPANEL, ID_CHK_SHOWPATH,  wxT("Show path"),  wxPoint(m_ButtonPos + 4, YBASE + MENUBARH + 35 - 2), CBSIZ);
	new MACHECKBOX(MPANEL, ID_CHK_ADDMODE,   wxT("Add mode"),   wxPoint(m_ButtonPos + 84, YBASE + MENUBARH + 3 - 2), CBSIZ);
 #else
	new MACHECKBOX(MPANEL, ID_CHK_SHOWINDEX, wxT("Show index"), wxPoint(m_ButtonPos + 4, YBASE + MENUBARH + 3), CBSIZ);
	new MACHECKBOX(MPANEL, ID_CHK_SHOWDIR,   wxT("Show dir"),   wxPoint(m_ButtonPos + 4, YBASE + MENUBARH + 19), CBSIZ);
	new MACHECKBOX(MPANEL, ID_CHK_SHOWPATH,  wxT("Show path"),  wxPoint(m_ButtonPos + 4, YBASE + MENUBARH + 35), CBSIZ);
	new MACHECKBOX(MPANEL, ID_CHK_ADDMODE,   wxT("Add mode"),   wxPoint(m_ButtonPos + 84, YBASE + MENUBARH + 3), CBSIZ);
 #endif

	// list
#if (defined(__WXMAC__) || defined(__WXGTK__)) && (MAGUIWX != 4) && (LISTXSIZEDEF > 200)
	#define LISTW 200 // else vscrollbar disappears in small window
	#define LISTH 100 // else hscrollbar disappears in small window
#else
	#define LISTW LISTXSIZEDEF
	#define LISTH (LISTYSIZEDEF - MENUBARH - MABUTTONH - 4)
#endif
	#define LISTSIZE wxSize(LISTW, LISTH)
	#define LISTPOS wxPoint(XBASE, YBASE + MENUBARH + MABUTTONH + 4)

#if defined(__WXMSW__)
	#define LISTMAXW (LISTW - SCROLLERWIDTH - 10)
#else
	#define LISTMAXW LISTW
#endif
	
	m_list = new wxListCtrl(MPANEL, ID_LIST_CTRL,
		LISTPOS,
		LISTSIZE,
			wxLC_REPORT | wxLC_ALIGN_LEFT | wxLC_SINGLE_SEL | wxLC_HRULES | wxLC_NO_SORT_HEADER);
	MASETFONT(m_list);
	m_list->InsertColumn(0, wxEmptyString, wxLIST_FORMAT_LEFT, LISTMAXW);

#ifdef MADRAGANDDROP
	SetDropTarget(new DnDFile());
	m_list->SetDropTarget(new DnDFile());
#endif

#if defined(MAKEYBOARD)
	Connect(wxID_ANY, wxEVT_LEFT_UP,
		(wxObjectEventFunction)(wxEventFunction)(wxMouseEventFunction)&MyApp::maOnClick, NULL);
#endif

} // MyFrameList


MyFrameList::~MyFrameList()
{
}


void MyFrameList::OnGUIButtonLoaded(wxCommandEvent & WXUNUSED(event))
{
	OnButtonLoaded();
}


#if defined(MAUSEMABUTTON)
 void MyFrameList::MakeButtonsList()
 {
	wxImage *upOriginal = new wxImage(wxBitmap(BMUpButton).ConvertToImage());
	wxImage *downOriginal = new wxImage(wxBitmap(BMDownButton).ConvertToImage());
	wxImage *hiliteOriginal = new wxImage(wxBitmap(BMHiliteButton).ConvertToImage());

 #if defined(__MAOSX__) || defined(__MAUNIX__)
	m_upPattern = upOriginal;
	m_downPattern = downOriginal;
	m_hilitePattern = hiliteOriginal;
 #else
	wxColour newColour = wxSystemSettingsNative::GetColour(wxSYS_COLOUR_3DFACE);
	m_upPattern = ChangeImageColour(upOriginal, newColour);
	m_downPattern = ChangeImageColour(downOriginal, newColour);
	m_hilitePattern = ChangeImageColour(hiliteOriginal, newColour);
#endif

	MakeButtonList((char const **)BMInfo, (char const **)BMInfoDisabled, (char const **)BMInfoMask, ID_BTN_INFO, true);

	MakeButtonList((char const **)BMSonglist, (char const **)BMSonglistDisabled, (char const **)BMSonglistMask, ID_BTN_PLAYLIST, true);

	MakeButtonList((char const **)BMLoaded, (char const **)BMLoadedDisabled, (char const **)BMLoadedMask, ID_BTN_LOADED, true);

	MakeButtonList((char const **)BMSonginfo, (char const **)BMSonginfoDisabled, (char const **)BMSonginfoMask, ID_BTN_SONGINFO, true);

 #if defined(__MAOSX__) || defined(__MAUNIX__)
 #else
	delete m_upPattern;
	delete m_downPattern;
	delete m_hilitePattern;
 #endif

	delete upOriginal;
	delete downOriginal;
	delete hiliteOriginal;

} // MakeButtonsList


MABUTTON *MyFrameList::MakeButtonList(	char const **foreground, char const **disabled,
										char const **foremask, int id, bool processdownevents)
{
	wxImage *color		= new wxImage(wxBitmap(foreground).ConvertToImage());
	wxImage *color_dis	= new wxImage(wxBitmap(disabled).ConvertToImage());
	wxImage *mask 		= new wxImage(wxBitmap(foremask).ConvertToImage());

	wxImage *up2 		= OverlayImage(m_upPattern,		color,		mask,	BUTTONOFFSETX,		BUTTONOFFSETY);
	wxImage *hilite2 	= OverlayImage(m_hilitePattern,	color,		mask,	BUTTONOFFSETX,		BUTTONOFFSETY);
	wxImage *down2		= OverlayImage(m_downPattern,	color,		mask,	BUTTONOFFSETX + 1,	BUTTONOFFSETY + 1);
	wxImage *disable2 	= OverlayImage(m_upPattern,		color_dis,	mask,	BUTTONOFFSETX,		BUTTONOFFSETY);

	MABUTTON *button	= new MABUTTON(MPANEL, id, wxPoint(m_ButtonPos, m_buttonTop),
										MABUTTONSIZE, up2, hilite2, down2, disable2, processdownevents);

	delete color;
	delete color_dis;
	delete mask;
	delete up2;
	delete down2;
	delete hilite2;
	delete disable2;

	m_ButtonPos += MABUTTONW;

	return button;
	
} // MakeButtonList
#else // !MAUSEMABUTTON
void MyFrameList::MakeButtonsList()
{
	MakeButtonList(ID_BTN_INFO, L"Info");

	MakeButtonList(ID_BTN_PLAYLIST, L"List");

	MakeButtonList(ID_BTN_LOADED, L"Load");

	MakeButtonList(ID_BTN_SONGINFO, L"Song");

} // MakeButtonsList


MABUTTON *MyFrameList::MakeButtonList(int id, machar *label)
{
	MABUTTON *button = new MABUTTON(MPANEL, id, label, wxPoint(m_ButtonPos, m_buttonTop), MABUTTONSIZE);

	m_ButtonPos += MABUTTONW;

	return button;

} // MakeButtonList
#endif // !MAUSEMABUTTON


void MyFrameList::OnGUIButtonList(wxCommandEvent & WXUNUSED(event))
{
	OnButtonList();
}


void MyFrameList::OnGUIButtonInfo(wxCommandEvent & WXUNUSED(event))
{
	OnButtonInfo();
}


void MyFrameList::OnGUIButtonSong(wxCommandEvent & WXUNUSED(event))
{
	OnButtonSong();
}


void MyFrameList::OnGUICheckShowIndex(wxCommandEvent &event)
{
	DoShowIndex(event.IsChecked() ? 1 : 0);
}


void MyFrameList::OnGUICheckShowDir(wxCommandEvent &event)
{
	DoShowDir(event.IsChecked() ? 1 : 0);
}


void MyFrameList::OnGUICheckShowPath(wxCommandEvent &event)
{
	DoShowPath(event.IsChecked() ? 1 : 0);
}


void MyFrameList::OnGUICheckAddMode(wxCommandEvent &event)
{
	DoAddMode(event.IsChecked() ? 1 : 0);
}


void MyFrameList::OnGUIListSelected(wxListEvent & WXUNUSED(event))
{
	OnListWindowSingleClick(0);
}


void MyFrameList::OnGUIListActivated(wxListEvent & WXUNUSED(event))
{
	OnListWindowDoubleClick();
}


void MyFrameList::GetColumn0Width()
{
	int width = m_list->GetColumnWidth(0);
	if(width < (g_sets.mawin[WINDOWLIST].xsize[g_sets.winconf] - SCROLLERWIDTH))
		width = g_sets.mawin[WINDOWLIST].xsize[g_sets.winconf] - SCROLLERWIDTH;
		
	switch(g_sets.listboxmode & (LBOXPLAYERINFO | LBOXPLAYLIST | LBOXSONGSLOADED | LBOXSONGINFO))
	{
		case LBOXPLAYERINFO:
			g_sets.listcolxsize[g_sets.winconf][0] = width;
		break;
		case LBOXPLAYLIST:
			g_sets.listcolxsize[g_sets.winconf][1] = width;
		break;
		case LBOXSONGSLOADED:
			g_sets.listcolxsize[g_sets.winconf][2] = width;
		break;
		case LBOXSONGINFO:
			g_sets.listcolxsize[g_sets.winconf][3] = width;
		break;
		default:
		break;
	}
}

void MyFrameList::OnGUIColumnEndDrag(wxListEvent & WXUNUSED(event))
{
	GetColumn0Width();

} // OnGUIColumnEndDrag


void MyFrameList::SetColumn0Width(int width)
{
	if(!g_w[WINDOWLIST])
		return;

	if(width <= 0)
	{
		switch(g_sets.listboxmode & (LBOXPLAYERINFO | LBOXPLAYLIST | LBOXSONGSLOADED | LBOXSONGINFO))
		{
			case LBOXPLAYERINFO:
				m_list->SetColumnWidth(0, g_sets.listcolxsize[g_sets.winconf][0]);
			break;
			case LBOXPLAYLIST:
				m_list->SetColumnWidth(0, g_sets.listcolxsize[g_sets.winconf][1]);
			break;
			case LBOXSONGSLOADED:
				m_list->SetColumnWidth(0, g_sets.listcolxsize[g_sets.winconf][2]);
			break;
			case LBOXSONGINFO:
				m_list->SetColumnWidth(0, g_sets.listcolxsize[g_sets.winconf][3]);
			break;
			default:
			break;
		}
	}
	else
	{
		m_list->SetColumnWidth(0, width);
		GetColumn0Width();
	}
		
} // SetColumn0Width


MyFrameEq::MyFrameEq( wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& siz, long style)
						: wxFrame((wxWindow *)NULL, id, title, pos, siz, style)
{
	int i;
	wxStaticText *t;
#if !defined(__MAWIN__)
	machar mastr2[EQNAMELEN];
#endif
	
#if !defined(MANOWXPANEL)
	m_panel = new wxPanel(this, -1, wxDefaultPosition, siz, wxCLIP_CHILDREN | wxNO_BORDER);
#endif
	
#if defined(MAKEYBOARD)
	// having a wxListCtrl makes kb focus work...
	new wxListCtrl(MPANEL, -1, wxPoint(0, 0), wxSize(1, 1),
		wxLC_REPORT | wxLC_ALIGN_LEFT | wxLC_SINGLE_SEL | wxLC_HRULES | wxLC_NO_SORT_HEADER);
#endif

	// equalizer buttons
	m_ButtonPos = XBASE;
	m_buttonTop = YBASE + MENUBARH; // line up buttons with main window buttons
	MakeButtonsEq();

	#define PREX (XBASE + 544)
	#define PREY (YBASE + 29 + MENUBARH)

	// equalizer checkbox
	MACHECKBOX *c = new MACHECKBOX(MPANEL,	ID_CHK_EQUALIZERB,	wxT("Equalizer"),	wxPoint(PREX - 90, PREY - 28),	CBSIZ);
	c->SetValue((g_sets.eqflags & MAEQACTIVE) ? true : false);

				c = new MACHECKBOX(MPANEL,	ID_CHK_EQ31BANDSB,	wxT("Eq31Bands"),	wxPoint(PREX - 90, PREY - 10),	CBSIZ);
	c->SetValue((g_sets.eqflags & MAEQ31BANDS) ? true : false);

			c = new MACHECKBOX(MPANEL,		ID_CHK_EQEXTRAB,	wxT("EqExtra"),		wxPoint(PREX - 90, PREY + 8),	CBSIZ);
	c->SetValue((g_sets.eqflags & MAEQEXTRA) ? true : false);

	// zoom checkbox
				c = new MACHECKBOX(MPANEL,	ID_CHK_EQZOOM,		wxT("Zoom"),		wxPoint(PREX - 14, PREY - 28),	CBSIZ3);
	c->SetValue((g_sets.eqflags & MAEQZOOM) ? true : false);

	// link checkbox
				c =	new MACHECKBOX(MPANEL,	ID_CHK_EQLINK,		wxT("Link"),		wxPoint(PREX - 14, PREY - 10),	CBSIZ3);
	c->SetValue((g_sets.eqflags & MAEQLINK) ? true : false);

	// separate left/right checkbox
				c = new MACHECKBOX(MPANEL,	ID_CHK_SEPARATELR,	wxT("Separate L/R"), wxPoint(PREX - 14, PREY + 8), CBSIZ);
	c->SetValue((g_sets.eqflags & MASEPARATELR) ? true : false);

	// offset spin
	t = new wxStaticText(MPANEL, ID_TEXT_EQOFFSET,	wxT("Offset dB"), wxPoint(PREX + 42, PREY - 28));
	MASETFONT(t);
	t = new wxStaticText(MPANEL, ID_TEXT_EQOFFSETVAL,wxT("000"), wxPoint(PREX + 61, PREY - 10));
	MASETFONT(t);
	wxSpinButton *s = new wxSpinButton(MPANEL, ID_SPIN_EQOFFSET, wxPoint(PREX + 42, PREY - 16), SPSIZ, wxSP_VERTICAL | wxSP_WRAP);
	s->SetRange(INT_MIN, INT_MAX);
	s->SetValue(0);
	
	// presets popup
	t = new wxStaticText(MPANEL, ID_TEXT_EQPRESET, wxT("Preset"), wxPoint(PREX + 106, PREY - 28));
	MASETFONT(t);
	
	m_ComboPreset = new wxComboBox(MPANEL, ID_POPUP_EQPRESET, wxT(""),wxPoint(PREX + 100, PREY - 15), wxSize(88, 20), 0, NULL, wxCB_READONLY);
	MASETFONT(m_ComboPreset); // doesn't on all platforms
	
 	for(i = 0; i < EQNUMPRESETS; i++) // PopupButtonAppend
 	{
 		if(fsstrlen(g_sets.eqpresets[i].eqname) == 0 || g_sets.eqpresets[i].eqname[0] == fsT(' '))
 			fssprintf(g_sets.eqpresets[i].eqname, EQNAMELEN, fsT("Preset%d"), i + 1);
 		m_ComboPreset->Append(fs2ma(mastr2, g_sets.eqpresets[i].eqname));
	}
	
	// preset name editor
	t = new wxStaticText(MPANEL, ID_TEXT_EQEDIT, wxT("Preset Name"), wxPoint(PREX + 106, PREY + 8));
	MASETFONT(t);
#if defined(MAKEYBOARD)
	m_dummytexteq = new wxTextCtrl(MPANEL, wxID_ANY, wxT(""), wxPoint(PREX + 108, PREY + 24), wxSize(1, 1), wxTE_PROCESS_ENTER);
#endif
	m_edittexteq = new wxTextCtrl(MPANEL, ID_EDITTEXT_EQ, wxT(""), wxPoint(PREX + 104, PREY + 20), EDSIZ, wxTE_PROCESS_ENTER);
	
	// equalizer sliders
	#define SLX (XBASE)
	#define SLXDEL 24
	#define STXPOS (stxpos += SLXDEL)
	#define SLXPOS (slxpos += SLXDEL)
	#define TEXPOS (texpos += SLXDEL)
	#define SPXPOS (spxpos += SLXDEL)
	#define T0YPOS (SLYPOS + 67)
	#define SLYPOS (YBASE + 80 + MENUBARH)

	int stxpos = SLX - SLXDEL;
	int slxpos = SLX - SLXDEL;
	int texpos = SLX - SLXDEL - 1;
	int spxpos = SLX - SLXDEL + 2;

	#define YSTAGGER 12
	int ystagger;
	
	machar *label;
	int lstyle;
	for(i = 0; i < EQBANDS31; i++)
	{
		lstyle = wxSL_VERTICAL;
		switch(i)
		{
			case  0: label = L"20Hz";	break;
			case  1: label = L"25";		break;
			case  2: label = L"31.5";	lstyle = 0x8000 | wxSL_VERTICAL; break; // use BMSliderThumbA
			case  3: label = L"40";		break;
			case  4: label = L"50";		break;
			case  5: label = L"63";		lstyle = 0x8000 | wxSL_VERTICAL; break;
			case  6: label = L"80";		break;
			case  7: label = L"100";	break;
			case  8: label = L"125";	lstyle = 0x8000 | wxSL_VERTICAL; break;
			case  9: label = L"160";	break;
			case 10: label = L"200";	break;
			case 11: label = L"250";	lstyle = 0x8000 | wxSL_VERTICAL; break;
			case 12: label = L"315";	break;
			case 13: label = L"400";	break;
			case 14: label = L"500";	lstyle = 0x8000 | wxSL_VERTICAL; break;
			case 15: label = L"630";	break;
			case 16: label = L"800";	break;
			case 17: label = L"1kHz";	lstyle = 0x8000 | wxSL_VERTICAL; break;
			case 18: label = L"1.25";	break;
			case 19: label = L"1.6";	break;
			case 20: label = L"2.0";	lstyle = 0x8000 | wxSL_VERTICAL; break;
			case 21: label = L"2.5";	break;
			case 22: label = L"3.15";	break;
			case 23: label = L"4.0";	lstyle = 0x8000 | wxSL_VERTICAL; break;
			case 24: label = L"5.0";	break;
			case 25: label = L"6.3";	break;
			case 26: label = L"8.0";	lstyle = 0x8000 | wxSL_VERTICAL; break;
			case 27: label = L"10";		break;
			case 28: label = L"12.5";	break;
			case 29: label = L"16";		lstyle = 0x8000 | wxSL_VERTICAL; break;
			case 30: label = L"20";		break;
			default: label = L"";		break;
		}
		
		if(i % 2)
			ystagger = YSTAGGER;
		else
			ystagger = 0;
		
		t = new wxStaticText(MPANEL, ID_TEXT_EQBAND0 + i, label, wxPoint(STXPOS, SLYPOS - 6 + ystagger));
		MASETFONT(t);
	
		new MASLIDER(MPANEL, ID_SLIDER_EQ0 + i, (DBSLIDERMAX - DBSLIDERMIN) / 2, DBSLIDERMIN, DBSLIDERMAX,
												wxPoint(SLXPOS, SLYPOS + 21), wxSize(20, 102), lstyle);

		t = new wxStaticText(MPANEL, ID_TEXT_EQ0 + i, wxT("0000"), wxPoint(TEXPOS, SLYPOS + 124 + ystagger), wxDefaultSize, wxALIGN_CENTRE);
		MASETFONT(t);
	
		s = new wxSpinButton(MPANEL, ID_SPIN_EQ0 + i, wxPoint(SPXPOS + 2, SLYPOS + 137 + YSTAGGER), SPSIZ, wxSP_VERTICAL | wxSP_WRAP);
		s->SetRange(INT_MIN, INT_MAX);
		s->SetValue(0);
	}

} // MyFrameEq


MyFrameEq::~MyFrameEq()
{
}


#if defined(MAUSEMABUTTON)
void MyFrameEq::MakeButtonsEq()
{
	wxStaticText *t;
	
	wxImage *upOriginal = new wxImage(wxBitmap(BMUpButton).ConvertToImage());
	wxImage *downOriginal = new wxImage(wxBitmap(BMDownButton).ConvertToImage());
	wxImage *hiliteOriginal = new wxImage(wxBitmap(BMHiliteButton).ConvertToImage());

#if defined(__MAOSX__) || defined(__MAUNIX__)
	m_upPattern = upOriginal;
	m_downPattern = downOriginal;
	m_hilitePattern = hiliteOriginal;
#else
	wxColour newColour = wxSystemSettingsNative::GetColour(wxSYS_COLOUR_3DFACE);
	m_upPattern = ChangeImageColour(upOriginal, newColour);
	m_downPattern = ChangeImageColour(downOriginal, newColour);
	m_hilitePattern = ChangeImageColour(hiliteOriginal, newColour);
#endif

	t = new wxStaticText(MPANEL, ID_TEXT_EQBUTTON_ZOOM,		wxT("- Zoom/Offset +"), wxPoint(m_ButtonPos, BUTTONTEXTY + MENUBARH));
	MASETFONT(t);
	MakeButtonEq(BMMinus, (char const **)BMMinusDisabled,   (char const **)BMMinusMask, ID_BTN_EQMINUS, true);
	MakeButtonEq((char const **)BMPlus, (char const **)BMPlusDisabled,  (char const **)BMPlusMask,  ID_BTN_EQPLUS, true);

	t = new wxStaticText(MPANEL, ID_TEXT_EQBUTTON_MID,		wxT("Mid"),		wxPoint(m_ButtonPos, BUTTONTEXTY + MENUBARH));
	MASETFONT(t);
	MakeButtonEq((char const **)BMMid, (char const **)BMMidMask,		(char const **)BMMidMask,   ID_BTN_EQMID, true);

	t = new wxStaticText(MPANEL, ID_TEXT_EQBUTTON_COPY,		wxT("Copy"),	wxPoint(m_ButtonPos, BUTTONTEXTY + MENUBARH));
	MASETFONT(t);
	MakeButtonEq((char const **)BMCopy, (char const **)BMCopyDisabled,  (char const **)BMCopyMask,  ID_BTN_EQCOPY, true);

	t = new wxStaticText(MPANEL, ID_TEXT_EQBUTTON_PASTE,   wxT("Paste"),	wxPoint(m_ButtonPos, BUTTONTEXTY + MENUBARH));
	MASETFONT(t);
	MakeButtonEq((char const **)BMPaste, (char const **)BMPasteDisabled,(char const **)BMPasteMask, ID_BTN_EQPASTE, true);

	t = new wxStaticText(MPANEL, ID_TEXT_EQBUTTON_RESET,   wxT("Reset"),	wxPoint(m_ButtonPos, BUTTONTEXTY + MENUBARH));
	MASETFONT(t);
	MakeButtonEq((char const **)BMReset, (char const **)BMResetDisabled,(char const **)BMResetMask, ID_BTN_EQRESTORE, true);

	t = new wxStaticText(MPANEL, ID_TEXT_EQBUTTON_SHIFTL,  wxT("ShiftLeft"),wxPoint(m_ButtonPos, BUTTONTEXTY + MENUBARH));
	MASETFONT(t);
	MakeButtonEq((char const **)BMShiftL, (char const **)BMShiftLMask,  (char const **)BMShiftLMask, ID_BTN_EQSHIFTL, true);

	t = new wxStaticText(MPANEL, ID_TEXT_EQBUTTON_SHIFTR,  wxT("ShiftRigh"),wxPoint(m_ButtonPos, BUTTONTEXTY + MENUBARH));
	MASETFONT(t);
	MakeButtonEq((char const **)BMShiftR, (char const **)BMShiftRMask,  (char const **)BMShiftRMask, ID_BTN_EQSHIFTR, true);

	t = new wxStaticText(MPANEL, ID_TEXT_EQBUTTON_INVERT,  wxT(" Invert"),	wxPoint(m_ButtonPos, BUTTONTEXTY + MENUBARH));
	MASETFONT(t);
	MakeButtonEq((char const **)BMInvert, (char const **)BMInvertMask,  (char const **)BMInvertMask, ID_BTN_EQINVERT, true);

#if defined(__MAOSX__) || defined(__MAUNIX__)
#else
	delete m_upPattern;
	delete m_downPattern;
	delete m_hilitePattern;
#endif

	delete upOriginal;
	delete downOriginal;
	delete hiliteOriginal;

} // MakeButtonsEq


MABUTTON *MyFrameEq::MakeButtonEq(	char const **foreground, char const **disabled,
									char const **foremask, int id, bool processdownevents)
{
	wxImage *color		= new wxImage(wxBitmap(foreground).ConvertToImage());
	wxImage *color_dis	= new wxImage(wxBitmap(disabled).ConvertToImage());
	wxImage *mask 		= new wxImage(wxBitmap(foremask).ConvertToImage());

	wxImage *up2 		= OverlayImage(m_upPattern,		color,		mask,	BUTTONOFFSETX,		BUTTONOFFSETY);
	wxImage *hilite2 	= OverlayImage(m_hilitePattern,	color,		mask,	BUTTONOFFSETX,		BUTTONOFFSETY);
	wxImage *down2		= OverlayImage(m_downPattern,	color,		mask,	BUTTONOFFSETX + 1,	BUTTONOFFSETY + 1);
	wxImage *disable2 	= OverlayImage(m_upPattern,		color_dis,	mask,	BUTTONOFFSETX,		BUTTONOFFSETY);

	MABUTTON *button	= new MABUTTON(MPANEL, id, wxPoint(m_ButtonPos, m_buttonTop),
										MABUTTONSIZE, up2, hilite2, down2, disable2, processdownevents);
	delete color;
	delete color_dis;
	delete mask;
	delete up2;
	delete down2;
	delete hilite2;
	delete disable2;

	m_ButtonPos += MABUTTONW;

	return button;
	
} // MakeButtonEq
#else // !MAUSEMABUTTON
void MyFrameEq::MakeButtonsEq()
{
	MakeButtonEq(ID_BTN_EQMINUS, L"Zoom -");
	MakeButtonEq(ID_BTN_EQPLUS, L"Zoom +");
	MakeButtonEq(ID_BTN_EQMID, L"Mid");
	MakeButtonEq(ID_BTN_EQCOPY, L"Copy");
	MakeButtonEq(ID_BTN_EQPASTE, L"Paste");
	MakeButtonEq(ID_BTN_EQRESTORE, L"Reset");
	MakeButtonEq(ID_BTN_EQSHIFTL, L"ShiftLeft");
	MakeButtonEq(ID_BTN_EQSHIFTR, L"ShiftRigh");
	MakeButtonEq(ID_BTN_EQINVERT, L"Invert");

} // MakeButtonsEq


MABUTTON *MyFrameEq::MakeButtonEq(int id, machar *label)
{
	MABUTTON *button = new MABUTTON(MPANEL, id, label, wxPoint(m_ButtonPos, m_buttonTop), MABUTTONSIZE);

	m_ButtonPos += MABUTTONW;

	return button;
	
} // MakeButtonEq
#endif // !MAUSEMABUTTON


void MyFrameEq::OnGUICheckEqZoom(wxCommandEvent &event)
{
	DoEqZoom(event.IsChecked() ? 1 : 0);
}


void MyFrameEq::OnGUICheckEqLink(wxCommandEvent &event)
{
	DoEqLink(event.IsChecked() ? 1 : 0);
}


void MyFrameEq::OnGUICheckEqualizerB(wxCommandEvent &event)
{
	DoEqualizer(event.IsChecked() ? 1 : 0);
}


void MyFrameEq::OnGUICheckEq31BandsB(wxCommandEvent &event)
{
	DoEq31Bands(event.IsChecked() ? 1 : 0);
}


void MyFrameEq::OnGUICheckEqExtraB(wxCommandEvent &event)
{
	DoEqExtra(event.IsChecked() ? 1 : 0);
}


void MyFrameEq::OnGUICheckSeparateLR(wxCommandEvent &event)
{
	DoSeparateLR(event.IsChecked() ? 1 : 0);
}


void MyFrameEq::OnGUIButtonEqMinus(wxCommandEvent & WXUNUSED(event))
{
	OnButtonEqMinus();
}


void MyFrameEq::OnGUIButtonEqPlus(wxCommandEvent & WXUNUSED(event))
{
	OnButtonEqPlus();
}


void MyFrameEq::OnGUIButtonEqMid(wxCommandEvent & WXUNUSED(event))
{
	OnButtonEqMid();
}


void MyFrameEq::OnGUIButtonEqCopy(wxCommandEvent & WXUNUSED(event))
{
	OnButtonEqCopy();
}


void MyFrameEq::OnGUIButtonEqPaste(wxCommandEvent & WXUNUSED(event))
{
	OnButtonEqPaste();
}


void MyFrameEq::OnGUIButtonEqRestore(wxCommandEvent & WXUNUSED(event))
{
	OnButtonEqRestore();
}


void MyFrameEq::OnGUIButtonEqShiftL(wxCommandEvent & WXUNUSED(event))
{
	OnButtonEqShiftL();
}


void MyFrameEq::OnGUIButtonEqShiftR(wxCommandEvent & WXUNUSED(event))
{
	OnButtonEqShiftR();
}


void MyFrameEq::OnGUIButtonEqInvert(wxCommandEvent & WXUNUSED(event))
{
	OnButtonEqInvert();
}


void MyFrameEq::OnGUIPopupEqPreset(wxCommandEvent & WXUNUSED(event))
{
	m_ComboPreset->Refresh();
	OnPopupEqPreset(m_ComboPreset->GetSelection());
}


void MyFrameEq::OnGUIEditTextEq(wxCommandEvent & WXUNUSED(event))
{
	OnEditTextEq();
}


void MyFrameEq::OnGUISliderEq(wxCommandEvent &event)
{
	OnSliderEq(event.GetId() - ID_SLIDER_EQ0, SliderGetValue(WINDOWEQ, event.GetId()), 0);
}


void MyFrameEq::OnGUISpinUpEqOffset(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowEqOffset(1);
}


void MyFrameEq::OnGUISpinDownEqOffset(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowEqOffset(-1);
}


void MyFrameEq::OnGUISpinUpEq(wxSpinEvent &event)
{
	OnLittleArrowEq(event.GetId() - ID_SPIN_EQ0, 1);
}


void MyFrameEq::OnGUISpinDownEq(wxSpinEvent &event)
{
	OnLittleArrowEq(event.GetId() - ID_SPIN_EQ0, -1);
}


MyFrameEq2::MyFrameEq2( wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& siz, long style)
						: wxFrame((wxWindow *)NULL, id, title, pos, siz, style)
{
	int i;
	wxStaticText *t;
	
#if !defined(MANOWXPANEL)
	m_panel = new wxPanel(this, -1, wxDefaultPosition, siz, wxCLIP_CHILDREN | wxNO_BORDER);
#endif
	
#if defined(MAKEYBOARD)
	// having a wxListCtrl makes kb focus work...
	new wxListCtrl(MPANEL, -1, wxPoint(0, 0), wxSize(1, 1),
		wxLC_REPORT | wxLC_ALIGN_LEFT | wxLC_SINGLE_SEL | wxLC_HRULES | wxLC_NO_SORT_HEADER);
#endif

	// equalizer buttons
	m_ButtonPos = XBASE + MABUTTONW + MABUTTONW;
	m_buttonTop = YBASE + MENUBARH; // line up buttons with main window buttons
	MakeButtonsEq();

	#define PREX (XBASE + 544)
	#define PREY (YBASE + 29 + MENUBARH)

	// equalizer2 sliders
	#define SLX (XBASE)
	#define SLXDEL 24
	#define STXPOS (stxpos += SLXDEL)
	#define SLXPOS (slxpos += SLXDEL)
	#define TEXPOS (texpos += SLXDEL)
	#define SPXPOS (spxpos += SLXDEL)
	#define T0YPOS (SLYPOS + 67)
	#define SLYPOS (YBASE + 80 + MENUBARH)

	int stxpos = SLX - SLXDEL;
	int slxpos = SLX - SLXDEL;
	int texpos = SLX - SLXDEL - 1;
	int spxpos = SLX - SLXDEL + 2;

	#define YSTAGGER 12
	int ystagger;
	
	machar *label;
	int lstyle;
	for(i = 0; i < EQBANDS31; i++)
	{
		lstyle = wxSL_VERTICAL;
		switch(i)
		{
			case  0 : label = L"20Hz";   break;
			case  1 : label = L"25";		break;
			case  2 : label = L"31.5";	lstyle = 0x8000 | wxSL_VERTICAL; break; // use BMSliderThumbA
			case  3 : label = L"40";		break;
			case  4 : label = L"50";		break;
			case  5 : label = L"63";		lstyle = 0x8000 | wxSL_VERTICAL; break;
			case  6 : label = L"80";		break;
			case  7 : label = L"100";	break;
			case  8 : label = L"125";	lstyle = 0x8000 | wxSL_VERTICAL; break;
			case  9 : label = L"160";	break;
			case 10 : label = L"200";	break;
			case 11 : label = L"250";	lstyle = 0x8000 | wxSL_VERTICAL; break;
			case 12 : label = L"315";	break;
			case 13 : label = L"400";	break;
			case 14 : label = L"500";	lstyle = 0x8000 | wxSL_VERTICAL; break;
			case 15 : label = L"630";	break;
			case 16 : label = L"800";	break;
			case 17 : label = L"1kHz";   lstyle = 0x8000 | wxSL_VERTICAL; break;
			case 18 : label = L"1.25";	break;
			case 19 : label = L"1.6";	break;
			case 20 : label = L"2.0";	lstyle = 0x8000 | wxSL_VERTICAL; break;
			case 21 : label = L"2.5";	break;
			case 22 : label = L"3.15";	break;
			case 23 : label = L"4.0";	lstyle = 0x8000 | wxSL_VERTICAL; break;
			case 24 : label = L"5.0";	break;
			case 25 : label = L"6.3";	break;
			case 26 : label = L"8.0";	lstyle = 0x8000 | wxSL_VERTICAL; break;
			case 27 : label = L"10";		break;
			case 28 : label = L"12.5";	break;
			case 29 : label = L"16";		lstyle = 0x8000 | wxSL_VERTICAL; break;
			case 30 : label = L"20";		break;
			default : label = L"";		break;
		}
		
		if(i % 2)
			ystagger = YSTAGGER;
		else
			ystagger = 0;
		
		t = new wxStaticText(MPANEL, ID_TEXT_EQ2BAND0 + i, label, wxPoint(STXPOS, SLYPOS - 6 + ystagger));
		MASETFONT(t);
	
		new MASLIDER(MPANEL, ID_SLIDER_EQ2_0 + i, (DBSLIDERMAX - DBSLIDERMIN) / 2, DBSLIDERMIN, DBSLIDERMAX,
																	wxPoint(SLXPOS, SLYPOS + 21), wxSize(20, 102), lstyle);

		t = new wxStaticText(MPANEL, ID_TEXT_EQ2_0 + i, wxT("0000"), wxPoint(TEXPOS, SLYPOS + 124 + ystagger), wxDefaultSize, wxALIGN_CENTRE);
		MASETFONT(t);
	
		wxSpinButton *s = new wxSpinButton(MPANEL, ID_SPIN_EQ2_0 + i, wxPoint(SPXPOS + 2, SLYPOS + 137 + YSTAGGER), SPSIZ, wxSP_VERTICAL | wxSP_WRAP);
		s->SetRange(INT_MIN, INT_MAX);
		s->SetValue(0);
	}

} // MyFrameEq2


MyFrameEq2::~MyFrameEq2()
{
}


#if defined(MAUSEMABUTTON)
void MyFrameEq2::MakeButtonsEq()
{
	wxStaticText *t;
	
	wxImage *upOriginal = new wxImage(wxBitmap(BMUpButton).ConvertToImage());
	wxImage *downOriginal = new wxImage(wxBitmap(BMDownButton).ConvertToImage());
	wxImage *hiliteOriginal = new wxImage(wxBitmap(BMHiliteButton).ConvertToImage());

#if defined(__MAOSX__) || defined(__MAUNIX__)
	m_upPattern = upOriginal;
	m_downPattern = downOriginal;
	m_hilitePattern = hiliteOriginal;
#else
	wxColour newColour = wxSystemSettingsNative::GetColour(wxSYS_COLOUR_3DFACE);
	m_upPattern = ChangeImageColour(upOriginal, newColour);
	m_downPattern = ChangeImageColour(downOriginal, newColour);
	m_hilitePattern = ChangeImageColour(hiliteOriginal, newColour);
#endif

	/*
	t = new wxStaticText(MPANEL, ID_TEXT_EQBUTTON_ZOOM,	wxT("- Zoom/Offset +"),  wxPoint(m_ButtonPos, BUTTONTEXTY + MENUBARH));
	MASETFONT(t);
	MakeButtonEq(BMMinus, (char const **)BMMinusDisabled,   (char const **)BMMinusMask, ID_BTN_EQMINUS, true);
	MakeButtonEq((char const **)BMPlus, (char const **)BMPlusDisabled,  (char const **)BMPlusMask,  ID_BTN_EQPLUS, true);
	*/
	t = new wxStaticText(MPANEL, ID_TEXT_EQBUTTON_MID,		wxT("Mid"),			wxPoint(m_ButtonPos, BUTTONTEXTY + MENUBARH));
	MASETFONT(t);
	MakeButtonEq((char const **)BMMid, (char const **)BMMidMask,		(char const **)BMMidMask,   ID_BTN_EQ2MID, true);

	t = new wxStaticText(MPANEL, ID_TEXT_EQBUTTON_COPY,	wxT("Copy"),			wxPoint(m_ButtonPos, BUTTONTEXTY + MENUBARH));
	MASETFONT(t);
	MakeButtonEq((char const **)BMCopy, (char const **)BMCopyDisabled,  (char const **)BMCopyMask,  ID_BTN_EQ2COPY, true);

	t = new wxStaticText(MPANEL, ID_TEXT_EQBUTTON_PASTE,   wxT("Paste"),		wxPoint(m_ButtonPos, BUTTONTEXTY + MENUBARH));
	MASETFONT(t);
	MakeButtonEq((char const **)BMPaste, (char const **)BMPasteDisabled,(char const **)BMPasteMask, ID_BTN_EQ2PASTE, true);

	t = new wxStaticText(MPANEL, ID_TEXT_EQBUTTON_RESET,   wxT("Reset"),		wxPoint(m_ButtonPos, BUTTONTEXTY + MENUBARH));
	MASETFONT(t);
	MakeButtonEq((char const **)BMReset, (char const **)BMResetDisabled,(char const **)BMResetMask, ID_BTN_EQ2RESTORE, true);
	/*
	t = new wxStaticText(MPANEL, ID_TEXT_EQBUTTON_SHIFTL,  wxT("ShiftLeft"),	wxPoint(m_ButtonPos, BUTTONTEXTY + MENUBARH));
	MASETFONT(t);
	MakeButtonEq((char const **)BMShiftL, (char const **)BMShiftLMask,  (char const **)BMShiftLMask, ID_BTN_EQSHIFTL, true);

	t = new wxStaticText(MPANEL, ID_TEXT_EQBUTTON_SHIFTR,  wxT("ShiftRigh"),	wxPoint(m_ButtonPos, BUTTONTEXTY + MENUBARH));
	MASETFONT(t);
	MakeButtonEq((char const **)BMShiftR, (char const **)BMShiftRMask,  (char const **)BMShiftRMask, ID_BTN_EQSHIFTR, true);

	t = new wxStaticText(MPANEL, ID_TEXT_EQBUTTON_INVERT,  wxT(" Invert"),		wxPoint(m_ButtonPos, BUTTONTEXTY + MENUBARH));
	MASETFONT(t);
	MakeButtonEq((char const **)BMInvert, (char const **)BMInvertMask,  (char const **)BMInvertMask, ID_BTN_EQINVERT, true);
	*/
	
#if defined(__MAOSX__) || defined(__MAUNIX__)
#else
	delete m_upPattern;
	delete m_downPattern;
	delete m_hilitePattern;
#endif

	delete upOriginal;
	delete downOriginal;
	delete hiliteOriginal;

} // MakeButtonsEq


MABUTTON *MyFrameEq2::MakeButtonEq(	char const **foreground, char const **disabled,
									char const **foremask, int id, bool processdownevents)
{
	wxImage *color		= new wxImage(wxBitmap(foreground).ConvertToImage());
	wxImage *color_dis	= new wxImage(wxBitmap(disabled).ConvertToImage());
	wxImage *mask 		= new wxImage(wxBitmap(foremask).ConvertToImage());

	wxImage *up2 		= OverlayImage(m_upPattern,		color,		mask,	BUTTONOFFSETX,		BUTTONOFFSETY);
	wxImage *hilite2 	= OverlayImage(m_hilitePattern,	color,		mask,	BUTTONOFFSETX,		BUTTONOFFSETY);
	wxImage *down2		= OverlayImage(m_downPattern,	color,		mask,	BUTTONOFFSETX + 1,	BUTTONOFFSETY + 1);
	wxImage *disable2 	= OverlayImage(m_upPattern,		color_dis,	mask,	BUTTONOFFSETX,		BUTTONOFFSETY);

	MABUTTON *button	= new MABUTTON(MPANEL, id, wxPoint(m_ButtonPos, m_buttonTop),
										MABUTTONSIZE, up2, hilite2, down2, disable2, processdownevents);
	delete color;
	delete color_dis;
	delete mask;
	delete up2;
	delete down2;
	delete hilite2;
	delete disable2;

	m_ButtonPos += MABUTTONW;

	return button;
	
} // MakeButtonEq
#else // !MAUSEMABUTTON
void MyFrameEq2::MakeButtonsEq()
{
	MakeButtonEq(ID_BTN_EQ2MID, L"Mid");
	MakeButtonEq(ID_BTN_EQ2COPY, L"Copy");
	MakeButtonEq(ID_BTN_EQ2PASTE, L"Paste");
	MakeButtonEq(ID_BTN_EQRESTORE, L"Reset");
	/*
	MakeButtonEq(ID_BTN_EQSHIFTL, L"ShiftLeft");
	MakeButtonEq(ID_BTN_EQSHIFTR, L"ShiftRigh");
	MakeButtonEq(ID_BTN_EQINVERT, L"Invert");
	*/
} // MakeButtonsEq


MABUTTON *MyFrameEq2::MakeButtonEq(int id, machar *label)
{
	MABUTTON *button = new MABUTTON(MPANEL, id, label, wxPoint(m_ButtonPos, m_buttonTop), MABUTTONSIZE);

	m_ButtonPos += MABUTTONW;

	return button;
	
} // MakeButtonEq
#endif // !MAUSEMABUTTON


void MyFrameEq2::OnGUIButtonEqMid(wxCommandEvent & WXUNUSED(event))
{
	OnButtonEq2Mid();
}


void MyFrameEq2::OnGUIButtonEqCopy(wxCommandEvent & WXUNUSED(event))
{
	OnButtonEq2Copy();
}


void MyFrameEq2::OnGUIButtonEqPaste(wxCommandEvent & WXUNUSED(event))
{
	OnButtonEq2Paste();
}


void MyFrameEq2::OnGUIButtonEqRestore(wxCommandEvent & WXUNUSED(event))
{
	OnButtonEq2Restore();
}


void MyFrameEq2::OnGUISliderEq(wxCommandEvent &event)
{
	OnSliderEq2(event.GetId() - ID_SLIDER_EQ0, SliderGetValue(WINDOWEQ2, event.GetId()), 0);
}


void MyFrameEq2::OnGUISpinUpEqOffset(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowEq2Offset(1);
}


void MyFrameEq2::OnGUISpinDownEqOffset(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowEq2Offset(-1);
}


void MyFrameEq2::OnGUISpinUpEq(wxSpinEvent &event)
{
	OnLittleArrowEq2(event.GetId() - ID_SPIN_EQ0, 1);
}


void MyFrameEq2::OnGUISpinDownEq(wxSpinEvent &event)
{
	OnLittleArrowEq2(event.GetId() - ID_SPIN_EQ0, -1);
}


MyFrameEffects::MyFrameEffects( wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& siz, long style)
						: wxFrame((wxWindow *)NULL, id, title, pos, siz, style)
{
	#define EFFX  (XBASE + 32)
	#define EFFY  (YBASE + 22)
	#define EFFDELX (60)
	#define EFFCBY (EFFY + 167)
	wxSpinButton *s;
	wxStaticText *t;
	
#if !defined(MANOWXPANEL)
	m_panel = new wxPanel(this, -1, wxDefaultPosition, siz, wxCLIP_CHILDREN | wxNO_BORDER);
#endif
	
#if defined(MAKEYBOARD)
	// having a wxListCtrl makes kb focus work...
	new wxListCtrl(MPANEL, -1, wxPoint(0, 0), wxSize(1, 1),
		wxLC_REPORT | wxLC_ALIGN_LEFT | wxLC_SINGLE_SEL | wxLC_HRULES | wxLC_NO_SORT_HEADER);
#endif

	// duplicate main effects switch
	m_checkbox = new MACHECKBOX(MPANEL, ID_CHK_EFFECTSB, wxT("Effects"), wxPoint(EFFX + 2, EFFCBY), CBSIZ, 1); // red

	// sbsms
	new MACHECKBOX(MPANEL, ID_CHK_SBSMS, wxT("High quality (see Help/README - sbsms)"), wxPoint(EFFX + 2, EFFCBY + 31), CBSIZ, 1);

	// pitch
	t = new wxStaticText(MPANEL, -1, wxT("Pitch"), wxPoint(EFFX - 2, EFFY - 16));
	MASETFONT(t);
	t = new wxStaticText(MPANEL, -1, wxT("   semitones"), wxPoint(EFFX - 2, EFFY -   1));
	MASETFONT(t);
	new MASLIDER(MPANEL, ID_SLIDER_PITCH, EFFECTSPITCHMAX / 2, EFFECTSPITCHMIN, EFFECTSPITCHMAX,
																		wxPoint(EFFX, EFFY + 21), wxSize(20, 102), wxSL_VERTICAL);
	t = new wxStaticText(MPANEL, ID_TEXT_PITCH0, wxT("-000"), wxPoint(EFFX, EFFY + 124));
	MASETFONT(t);
	s = new wxSpinButton(MPANEL, ID_SPIN_PITCH, wxPoint(EFFX + 2, EFFY + 137), SPSIZ, wxSP_VERTICAL | wxSP_WRAP);
	s->SetRange(INT_MIN, INT_MAX);
	s->SetValue(0);

	t = new wxStaticText(MPANEL,   -1,				wxT("Pitch"), wxPoint(EFFX + EFFDELX - 2, EFFY - 16));
	MASETFONT(t);
	new MASLIDER(MPANEL, ID_SLIDER_PITCH2, EFFECTSPITCH2MAX / 2, EFFECTSPITCH2MIN, EFFECTSPITCH2MAX,
																		wxPoint(EFFX + EFFDELX, EFFY + 21),	wxSize(20, 102), wxSL_VERTICAL);
	t = new wxStaticText(MPANEL, ID_TEXT_PITCH02, wxT("-00.0"), wxPoint(EFFX + EFFDELX, EFFY + 124));
	MASETFONT(t);
	s = new wxSpinButton(MPANEL, ID_SPIN_PITCH2, wxPoint(EFFX + EFFDELX + 2, EFFY + 137), SPSIZ, wxSP_VERTICAL | wxSP_WRAP);
	s->SetRange(INT_MIN, INT_MAX);
	s->SetValue(0);

	new MACHECKBOX(MPANEL, ID_CHK_PITCH, wxT("Pitch"), wxPoint(EFFX + EFFDELX + 2, EFFCBY), CBSIZ);

	// tempo
	t = new wxStaticText(MPANEL, -1, wxT("Tempo"), wxPoint(EFFX + 2 * EFFDELX - 2, EFFY - 16));
	MASETFONT(t);
	t = new wxStaticText(MPANEL, -1, wxT("  x"), wxPoint(EFFX + 2 * EFFDELX - 2, EFFY - 1));
	MASETFONT(t);
	new MASLIDER(MPANEL, ID_SLIDER_TEMPO, EFFECTSMAX / 2, EFFECTSMIN, EFFECTSMAX,
				wxPoint(EFFX + 2 * EFFDELX,		EFFY + 21), wxSize(20, 102), wxSL_VERTICAL);
	t = new wxStaticText(MPANEL, ID_TEXT_TEMPO0, wxT("-000"), wxPoint(EFFX + 2 * EFFDELX, EFFY + 124));
	MASETFONT(t);
	s = new wxSpinButton(MPANEL, ID_SPIN_TEMPO, wxPoint(EFFX + 2 * EFFDELX + 2, EFFY + 137), SPSIZ, wxSP_VERTICAL | wxSP_WRAP);
	s->SetRange(INT_MIN, INT_MAX);
	s->SetValue(0);

	new MACHECKBOX(MPANEL, ID_CHK_TEMPO, wxT("Tempo"), wxPoint(EFFX + 2 * EFFDELX + 2, EFFCBY), CBSIZ);

	// speed
	t = new wxStaticText(MPANEL, -1, wxT("Speed"), wxPoint(EFFX + 3 * EFFDELX - 2, EFFY - 16));
	MASETFONT(t);
	t = new wxStaticText(MPANEL, -1, wxT("   x"), wxPoint(EFFX + 3 * EFFDELX - 2, EFFY - 1));
	MASETFONT(t);
	new MASLIDER(MPANEL, ID_SLIDER_SPEED, EFFECTSMAX / 2, EFFECTSMIN, EFFECTSMAX,
										wxPoint(EFFX + 3 * EFFDELX, EFFY + 21), wxSize(20, 102), wxSL_VERTICAL);
	t = new wxStaticText(MPANEL, ID_TEXT_SPEED0, wxT("-000"), wxPoint(EFFX + 3 * EFFDELX, EFFY + 124));
	MASETFONT(t);
	s = new wxSpinButton(MPANEL, ID_SPIN_SPEED, wxPoint(EFFX + 3 * EFFDELX + 2,	EFFY + 137), SPSIZ, wxSP_VERTICAL | wxSP_WRAP);
	s->SetRange(INT_MIN, INT_MAX);
	s->SetValue(0);

	new MACHECKBOX(MPANEL, ID_CHK_SPEED, wxT("Speed"), wxPoint(EFFX + 3 * EFFDELX + 2,	EFFCBY), CBSIZ);

	// keyboard shortcut labels
	#undef EFFX
	#define EFFX (XBASE + 32 + 22)
	#undef EFFY
	#define EFFY  (YBASE + 22 + 137)

	t = new wxStaticText(MPANEL, ID_TEXT_KEYEFECTS, wxT(""), wxPoint(EFFX - 20, EFFY + 43));
	MASETFONT(t);

	t = new wxStaticText(MPANEL, ID_TEXT_KEYPITCHUP, wxT(""), wxPoint(EFFX - 2, EFFY));
	MASETFONT(t);
	t = new wxStaticText(MPANEL, ID_TEXT_KEYPITCHDOWN, wxT(""), wxPoint(EFFX - 2, EFFY + 11));
	MASETFONT(t);
	t = new wxStaticText(MPANEL, ID_TEXT_KEYPITCHSET, wxT(""), wxPoint(EFFX + EFFDELX - 20,	EFFY + 43));
	MASETFONT(t);
	t = new wxStaticText(MPANEL, ID_TEXT_KEYPITCH2UP, wxT(""), wxPoint(EFFX + EFFDELX - 2, EFFY));
	MASETFONT(t);
	t = new wxStaticText(MPANEL, ID_TEXT_KEYPITCH2DOWN, wxT(""), wxPoint(EFFX + EFFDELX - 2, EFFY + 11));
	MASETFONT(t);
	t = new wxStaticText(MPANEL, ID_TEXT_KEYPITCHRESET, wxT(""), wxPoint(EFFX + EFFDELX - 0, EFFY - 71));
	MASETFONT(t);

	t = new wxStaticText(MPANEL, ID_TEXT_KEYTEMPOSET, wxT(""), wxPoint(EFFX + 2 * EFFDELX - 20,EFFY + 43));
	MASETFONT(t);
	t = new wxStaticText(MPANEL, ID_TEXT_KEYTEMPOUP, wxT(""), wxPoint(EFFX + 2 * EFFDELX - 2, EFFY));
	MASETFONT(t);
	t = new wxStaticText(MPANEL, ID_TEXT_KEYTEMPODOWN, wxT(""), wxPoint(EFFX + 2 * EFFDELX - 2,	EFFY + 11));
	MASETFONT(t);
	t = new wxStaticText(MPANEL, ID_TEXT_KEYTEMPORESET, wxT(""), wxPoint(EFFX + 2 * EFFDELX - 0, EFFY - 51));
	MASETFONT(t);

	t = new wxStaticText(MPANEL, ID_TEXT_KEYSPEEDSET, wxT(""), wxPoint(EFFX + 3 * EFFDELX - 20,EFFY + 43));
	MASETFONT(t);
	t = new wxStaticText(MPANEL, ID_TEXT_KEYSPEEDUP, wxT(""), wxPoint(EFFX + 3 * EFFDELX - 2, EFFY));
	MASETFONT(t);
	t = new wxStaticText(MPANEL, ID_TEXT_KEYSPEEDDOWN, wxT(""), wxPoint(EFFX + 3 * EFFDELX - 2,	EFFY + 11));
	MASETFONT(t);
	t = new wxStaticText(MPANEL, ID_TEXT_KEYSPEEDRESET, wxT(""), wxPoint(EFFX + 3 * EFFDELX - 0, EFFY - 51));
	MASETFONT(t);
	
} // MyFrameEffects


MyFrameEffects::~MyFrameEffects()
{
}


void MyFrameEffects::OnGUISliderPitch(wxCommandEvent &event)
{
	if(event.GetInt() == -1) // double-clicked to reset to saved value
	{
		SliderSetValue(WINDOWEFFECTS, ID_SLIDER_PITCH, g_setsbu.pitchslider);
		OnSliderPitch(g_setsbu.pitchslider);
	}
	else
		OnSliderPitch(SliderGetValue(WINDOWEFFECTS, ID_SLIDER_PITCH));
}


void MyFrameEffects::OnGUISliderPitch2(wxCommandEvent &event)
{
	if(event.GetInt() == -1) // double-clicked to reset to saved value
	{
		SliderSetValue(WINDOWEFFECTS, ID_SLIDER_PITCH2, g_setsbu.pitchslider2);
		OnSliderPitch2(g_setsbu.pitchslider2);
	}
	else
		OnSliderPitch2(SliderGetValue(WINDOWEFFECTS, ID_SLIDER_PITCH2));
}


void MyFrameEffects::OnGUISliderTempo(wxCommandEvent &event)
{
	if(event.GetInt() == -1) // double-clicked to reset to saved value
	{
		SliderSetValue(WINDOWEFFECTS, ID_SLIDER_TEMPO, g_setsbu.temposlider);
		OnSliderTempo(g_setsbu.temposlider);
	}
	else
		OnSliderTempo(SliderGetValue(WINDOWEFFECTS, ID_SLIDER_TEMPO));
}


void MyFrameEffects::OnGUISliderSpeed(wxCommandEvent &event)
{
	if(event.GetInt() == -1) // double-clicked to reset to saved value
	{
		SliderSetValue(WINDOWEFFECTS, ID_SLIDER_SPEED, g_setsbu.speedslider);
		OnSliderSpeed(g_setsbu.speedslider);
	}
	else
		OnSliderSpeed(SliderGetValue(WINDOWEFFECTS, ID_SLIDER_SPEED));
}


void MyFrameEffects::OnGUISpinDownPitch(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowPitch(-1);
}


void MyFrameEffects::OnGUISpinDownPitch2(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowPitch2(-1);
}


void MyFrameEffects::OnGUISpinDownTempo(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowTempo(-1);
}


void MyFrameEffects::OnGUISpinDownSpeed(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowSpeed(-1);
}


void MyFrameEffects::OnGUISpinUpPitch(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowPitch(1);
}


void MyFrameEffects::OnGUISpinUpPitch2(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowPitch2(1);
}


void MyFrameEffects::OnGUISpinUpTempo(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowTempo(1);
}


void MyFrameEffects::OnGUISpinUpSpeed(wxSpinEvent & WXUNUSED(event))
{
	OnLittleArrowSpeed(1);
}


void MyFrameEffects::OnGUICheckPitch(wxCommandEvent &event)
{
	DoPitch(event.IsChecked() ? 1 : 0);
}


void MyFrameEffects::OnGUICheckTempo(wxCommandEvent &event)
{
	DoTempo(event.IsChecked() ? 1 : 0);
}


void MyFrameEffects::OnGUICheckSpeed(wxCommandEvent &event)
{
	DoSpeed(event.IsChecked() ? 1 : 0);
}


void MyFrameEffects::OnGUICheckEffectsB(wxCommandEvent &event)
{
	DoEffects(event.IsChecked() ? 1 : 0);
}


void MyFrameEffects::OnGUICheckSBSMS(wxCommandEvent &event)
{
	DoSBSMS(event.IsChecked() ? 1 : 0);
}


#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
MyFrameAudioCD::MyFrameAudioCD( wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& siz, long style)
								: wxFrame((wxWindow *)NULL, -1, title, pos, siz, style)
{
	m_panel = new wxPanel(this, -1, wxDefaultPosition, siz, wxCLIP_CHILDREN | wxNO_BORDER);

	m_ComboAudioCD = new wxComboBox(m_panel, ID_POPUP_AUDIOCD,  wxT(""), wxPoint(130, 11), wxSize(120, 20), 0, NULL, wxCB_READONLY);
	MASETFONT(m_ComboAudioCD); // doesn't on all platforms
	
	new wxButton(m_panel, ID_BTN_AUDIOCDSET,	wxT("Set the drive"),	wxPoint(5, 10));

	new wxButton(m_panel, wxID_OK,				wxT("Close"),			wxPoint(130, 90));

	new wxButton(m_panel, ID_BTN_AUDIOCDCLEAR,  wxT("Clear the drive"),  wxPoint(5, 50));

	new wxButton(m_panel, ID_BTN_AUDIOCDSAVE,   wxT("Save settings"),	wxPoint(5, 90));

	new MACHECKBOX(m_panel, ID_CHK_CDPARANOIA,  wxT("CD Paranoia"),		wxPoint(138, 55), CBSIZ, 1); // red
	
} // MyFrameAudioCD


MyFrameAudioCD::~MyFrameAudioCD()
{
}


void MyFrameAudioCD::OnGUIPopupAudioCD(wxCommandEvent &event)
{
	m_ComboAudioCD->Refresh();
	OnPopupAudioCD(m_ComboAudioCD->GetSelection());
}


void MyFrameAudioCD::OnGUIButtonAudioCDSet(wxCommandEvent &event)
{
	OnButtonAudioCDSet();
}

void MyFrameAudioCD::OnGUIButtonAudioCDClear(wxCommandEvent &event)
{
	OnButtonAudioCDClear();
}


void MyFrameAudioCD::OnGUIButtonAudioCDSave(wxCommandEvent &event)
{
	OnButtonAudioCDSave();
}


void MyFrameAudioCD::OnGUICheckCdParanoia(wxCommandEvent &event)
{
	DoCdParanoia(event.IsChecked() ? 1 : 0);
}


void MyFrameAudioCD::OnGUIButtonClose(wxCommandEvent &event)
{
	DoWindow(WINDOWAUDIOCD, 0);

} // OnGUIButtonClose
#endif


MyFrameFileOut::MyFrameFileOut( wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& siz, long style)
				: wxFrame((wxWindow *)NULL, id, title, pos, siz, style)
{
	wxStaticText *t;
#if !defined(__MAWIN__)
 	machar mastr2[MAXLOGLINELEN];
#endif
	
	m_panel = new wxPanel(this, -1, wxDefaultPosition, siz, wxCLIP_CHILDREN | wxNO_BORDER);
	
	#undef POPX
	#undef POPY
	#define POPX (XBASE + 20)
	#define POPY (-20)
	#define POPW (160)
	#define POPH (20)
	#define POPXT (POPX)
	#define POPYT (POPY - 12)
	#define YDEL 34
		
	t = new wxStaticText(m_panel, -1, wxT("Put new files in folder"), wxPoint(POPXT, POPYT + 1 * YDEL + 2));
	MASETFONT(t);
	t = new wxStaticText(m_panel, ID_TEXT_FILEOUTDIR, fs2ma(mastr2, g_sets.fileout[g_k[g_sets.output]].outdir), wxPoint(POPXT, POPYT + 1 * YDEL + 18));
	MASETFONT(t);
						
	new wxButton(m_panel, ID_BTN_FILEOUTDIR, wxT("Change folder"), wxPoint(POPXT, POPYT + 2 * YDEL));
	

#if defined(ENCODEMP3) || defined(ENCODEOGG) || defined(ENCODAAC) || defined(ENCODEMP4)
	switch(g_sets.output)
	{		
 #ifdef ENCODEMP3
		case kOUTMP3 :
			t = new wxStaticText(m_panel, -1, wxT("Bitrate"), wxPoint(POPXT, POPYT + 3 * YDEL));
			MASETFONT(t);
			m_ComboBitRate = new wxComboBox(m_panel, ID_POPUP_BITRATE, wxT(""), wxPoint(POPX, POPY  + 3 * YDEL), wxSize(POPW, POPH), 0, NULL, wxCB_READONLY);
			MASETFONT(m_ComboBitRate); // doesn't on all platforms
			m_ComboBitRate->Append(wxT("32 (lower quality, smaller files)"));
			m_ComboBitRate->Append(wxT("40"));
			m_ComboBitRate->Append(wxT("48"));
			m_ComboBitRate->Append(wxT("56"));
			m_ComboBitRate->Append(wxT("64"));
			m_ComboBitRate->Append(wxT("80"));
			m_ComboBitRate->Append(wxT("96"));
			m_ComboBitRate->Append(wxT("112"));
			m_ComboBitRate->Append(wxT("128 (default)"));
			m_ComboBitRate->Append(wxT("160"));
			m_ComboBitRate->Append(wxT("192"));
			m_ComboBitRate->Append(wxT("224"));
			m_ComboBitRate->Append(wxT("256"));
			m_ComboBitRate->Append(wxT("320 (high quality, bigger files)"));

			t = new wxStaticText(m_panel, -1, wxT("Quality"), wxPoint(POPXT, POPYT + 4 * YDEL));
			MASETFONT(t);
			m_ComboQuality = new wxComboBox(m_panel, ID_POPUP_QUALITY, wxT(""), wxPoint(POPX, POPY + 4 * YDEL), wxSize(POPW, POPH), 0, NULL, wxCB_READONLY);
			MASETFONT(m_ComboQuality); // doesn't on all platforms
			m_ComboQuality->Append(wxT("0 (high quality, slow encode)"));
			m_ComboQuality->Append(wxT("1"));
			m_ComboQuality->Append(wxT("2 (default)"));
			m_ComboQuality->Append(wxT("3"));
			m_ComboQuality->Append(wxT("4"));
			m_ComboQuality->Append(wxT("5"));
			m_ComboQuality->Append(wxT("6"));
			m_ComboQuality->Append(wxT("7"));
			m_ComboQuality->Append(wxT("8"));
			m_ComboQuality->Append(wxT("9"));
			m_ComboQuality->Append(wxT("10 (lower quality, fast encode)"));

			t = new wxStaticText(m_panel, -1, wxT("Mode"), wxPoint(POPXT, POPYT + 5 * YDEL));
			MASETFONT(t);
			m_ComboMode = new wxComboBox(m_panel, ID_POPUP_MODE, wxT(""), wxPoint(POPX, POPY + 5 * YDEL), wxSize(POPW, POPH), 0, NULL, wxCB_READONLY);
			MASETFONT(m_ComboMode); // doesn't on all platforms
			m_ComboMode->Append(wxT("Stereo (default)"));
			m_ComboMode->Append(wxT("Joint Stereo"));
			m_ComboMode->Append(wxT("Dual Stereo"));
			m_ComboMode->Append(wxT("Mono"));

			t = new wxStaticText(m_panel, -1, wxT("Sample Rate"), wxPoint(POPXT,  POPYT + 6 * YDEL));
			MASETFONT(t);
			m_ComboSampleRate = new wxComboBox(m_panel, ID_POPUP_SAMPLERATE, wxT(""), wxPoint(POPX, POPY + 6 * YDEL), wxSize(POPW, POPH), 0, NULL, wxCB_READONLY);
			MASETFONT(m_ComboSampleRate); // doesn't on all platforms
			m_ComboSampleRate->Append(wxT("Auto"));
			//m_ComboSampleRate->Append(wxT(" 8000")); // crashes on OSX
			m_ComboSampleRate->Append(wxT("11025"));
			m_ComboSampleRate->Append(wxT("12000"));
			m_ComboSampleRate->Append(wxT("16000"));
			m_ComboSampleRate->Append(wxT("22050"));
			m_ComboSampleRate->Append(wxT("24000"));
			m_ComboSampleRate->Append(wxT("32000"));
			m_ComboSampleRate->Append(wxT("44100 (default)"));
			m_ComboSampleRate->Append(wxT("48000"));
			m_ComboSampleRate->Append(wxT("Same as input"));

		break;
 #endif
 
 #ifdef ENCODEOGG
		case kOUTOGG :
			t = new wxStaticText(m_panel, -1, wxT("Bitrate"),  wxPoint(POPXT, POPYT + 3 * YDEL));
			MASETFONT(t);
			m_ComboBitRate = new wxComboBox(m_panel, ID_POPUP_BITRATE, wxT(""), wxPoint(POPX, POPY + 3 * YDEL), wxSize(POPW, POPH), 0, NULL, wxCB_READONLY);
			MASETFONT(m_ComboBitRate); // doesn't on all platforms
			m_ComboBitRate->Append(wxT("32 (lower quality, smaller files)"));
			m_ComboBitRate->Append(wxT("40"));
			m_ComboBitRate->Append(wxT("48"));
			m_ComboBitRate->Append(wxT("56"));
			m_ComboBitRate->Append(wxT("64"));
			m_ComboBitRate->Append(wxT("80"));
			m_ComboBitRate->Append(wxT("96"));
			m_ComboBitRate->Append(wxT("112"));
			m_ComboBitRate->Append(wxT("128 (default)"));
			m_ComboBitRate->Append(wxT("160"));
			m_ComboBitRate->Append(wxT("192"));
			m_ComboBitRate->Append(wxT("224"));
			m_ComboBitRate->Append(wxT("256"));
			m_ComboBitRate->Append(wxT("320 (high quality, bigger files)"));

			/* not used
			t = new wxStaticText(m_panel, -1, wxT("Quality"), wxPoint(POPXT, POPYT + 4 * YDEL));
			MASETFONT(t);
			m_ComboQuality = new wxComboBox(m_panel, ID_POPUP_QUALITY, wxT(""), wxPoint(POPX, POPY  + 4 * YDEL), wxSize(POPW, POPH), 0, NULL, wxCB_READONLY);
			MASETFONT(m_ComboQuality); // doesn't on all platforms
			m_ComboQuality->Append(wxT("0 (high quality, slow encode)"));
			m_ComboQuality->Append(wxT("1"));
			m_ComboQuality->Append(wxT("2"));
			m_ComboQuality->Append(wxT("3"));
			m_ComboQuality->Append(wxT("4"));
			m_ComboQuality->Append(wxT("5 (default)"));
			m_ComboQuality->Append(wxT("6"));
			m_ComboQuality->Append(wxT("7"));
			m_ComboQuality->Append(wxT("8"));
			m_ComboQuality->Append(wxT("9"));
			m_ComboQuality->Append(wxT("10 (lower quality, fast encode)"));
			*/
			t = new wxStaticText(m_panel, -1, wxT("Mode"), wxPoint(POPXT, POPYT + 5 * YDEL));
			MASETFONT(t);
			m_ComboMode = new wxComboBox(m_panel, ID_POPUP_MODE, wxT(""), wxPoint(POPX,   POPY  + 5 * YDEL), wxSize(POPW, POPH), 0, NULL, wxCB_READONLY);
			MASETFONT(m_ComboMode); // doesn't on all platforms
			m_ComboMode->Append(wxT("Stereo (default)"));
			// m_ComboMode->Append(wxT("Joint Stereo"));
			// m_ComboMode->Append(wxT("Dual Stereo"));
			m_ComboMode->Append(wxT("Mono"));
		break;
 #endif
 
 #ifdef ENCODEAAC
		case kOUTAAC :
			new MACHECKBOX(m_panel, ID_CHK_ADTS, wxT("   ADTS Headers"),  wxPoint(POPXT, POPYT + 3 * YDEL), CBSIZ, 0);
			t = new wxStaticText(m_panel, -1, wxT("Quality"), wxPoint(POPXT, POPYT + 4 * YDEL));
			MASETFONT(t);
			m_ComboQuality = new wxComboBox( m_panel, ID_POPUP_QUALITY, wxT(""), wxPoint(POPX, POPY + 4 * YDEL), wxSize(POPW, POPH), 0, NULL, wxCB_READONLY);
			MASETFONT(m_ComboQuality); // doesn't on all platforms
			m_ComboQuality->Append(wxT("130 (~218 kbps)"));
			m_ComboQuality->Append(wxT("120 (~194 kbps)"));
			m_ComboQuality->Append(wxT("110 (~158 kbps)"));
			m_ComboQuality->Append(wxT("100 (~129 kbps default)"));
			m_ComboQuality->Append(wxT(" 90 (~103 kbps)"));
			m_ComboQuality->Append(wxT(" 80 (~79 kbps)"));
			m_ComboQuality->Append(wxT(" 70 (~62 kbps)"));
		break;
 #endif
 
 #ifdef ENCODEMP4
		case kOUTMP4 :
			new MACHECKBOX(m_panel, ID_CHK_M4A, wxT("   .m4a extension"), wxPoint(POPXT, POPYT + 3 * YDEL), CBSIZ, 0);
			t = new wxStaticText(m_panel, -1, wxT("Quality"), wxPoint(POPXT, POPYT + 4 * YDEL));
			MASETFONT(t);
			m_ComboQuality = new wxComboBox(m_panel, ID_POPUP_QUALITY, wxT(""), wxPoint(POPX, POPY + 4 * YDEL), wxSize(POPW, POPH), 0, NULL, wxCB_READONLY);
			MASETFONT(m_ComboQuality); // doesn't on all platforms
			m_ComboQuality->Append(wxT("130 (~218 kbps)"));
			m_ComboQuality->Append(wxT("120 (~194 kbps)"));
			m_ComboQuality->Append(wxT("110 (~158 kbps)"));
			m_ComboQuality->Append(wxT("100 (~129 kbps default)"));
			m_ComboQuality->Append(wxT(" 90 (~103 kbps)"));
			m_ComboQuality->Append(wxT(" 80 (~79 kbps)"));
			m_ComboQuality->Append(wxT(" 70 (~62 kbps)"));
		break;
 #endif
 
		default :
		break;
	} // switch
#endif
	
} // MyFrameFileOut


MyFrameFileOut::~MyFrameFileOut()
{
}


void MyFrameFileOut::OnGUIPopupBitRate(wxCommandEvent & WXUNUSED(event))
{
	m_ComboBitRate->Refresh();
	OnPopupBitRate(m_ComboBitRate->GetSelection());
}


void MyFrameFileOut::OnGUIPopupQuality(wxCommandEvent & WXUNUSED(event))
{
	m_ComboQuality->Refresh();
	OnPopupQuality(m_ComboQuality->GetSelection());
}


void MyFrameFileOut::OnGUIPopupMode(wxCommandEvent & WXUNUSED(event))
{
	m_ComboMode->Refresh();
	OnPopupMode(m_ComboMode->GetSelection());
}


void MyFrameFileOut::OnGUIPopupSampleRate(wxCommandEvent & WXUNUSED(event))
{
	m_ComboSampleRate->Refresh();
	OnPopupSampleRate(m_ComboSampleRate->GetSelection());
}


void MyFrameFileOut::OnGUIButtonFileOutDir(wxCommandEvent & WXUNUSED(event))
{
	OnButtonFileOutDir();
	
} // OnGUIButtonFileOutDir


#if defined(ENCODEAAC)
 void MyFrameFileOut::OnGUICheckBoxADTS(wxCommandEvent &event)
 {
	DoBoxADTS(event.IsChecked() ? 1 : 0);

 } // OnGUICheckBoxADTS
#endif


#if defined(ENCODEMP4)
 void MyFrameFileOut::OnGUICheckBoxM4A(wxCommandEvent &event)
 {
	DoBoxM4A(event.IsChecked() ? 1 : 0);

 } // OnGUICheckBoxM4A
#endif


MyFrameAbout::MyFrameAbout( wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& siz, long style)
			: wxFrame((wxWindow *)NULL, id, title, pos, siz, style)
{
	m_panel = new wxPanel(this, -1, wxDefaultPosition, siz, wxCLIP_CHILDREN | wxNO_BORDER);
	
#if defined(__WXMSW__) || (defined(__WXMAC__) && (MAGUIWX != 4))
 #define MAABOUTWINY 26 // else buttons are half visible
#else
 #define MAABOUTWINY 1 // else buttons are half visible
#endif
#define MAABOUTWINX 1
#define MAURLHEIGHT 26

	wxSize bsize = wxButton::GetDefaultSize();

#if defined(__WXMSW__)
	if(maIsWin7())
	{
		m_winw = siz.x - (2 * MAABOUTWINX) - XSIZEADJUST7; // else vertical slider is half visible
		m_winy = MAABOUTWINY + YSIZEADJUST7; // else buttons are half visible
	}
	else
#endif
	{
		m_winw = siz.x - (2 * MAABOUTWINX);
		m_winy = MAABOUTWINY;
	}

	m_closebutton = new wxButton(m_panel, wxID_OK, wxT("Close"),
							wxPoint((siz.x - bsize.x) / 2, siz.y - bsize.y - m_winy));
	
	m_urlTextCtrl = new wxTextCtrl(m_panel, -1, MOREAMP_HOME_URL,
							wxPoint(MAABOUTWINX, siz.y - MAURLHEIGHT - 2 - bsize.y - m_winy),
							wxSize(m_winw, MAURLHEIGHT));
	MASETFONT(m_urlTextCtrl);

	m_aboutTextCtrl = new wxTextCtrl(m_panel, -1, g_abouttext, wxPoint(MAABOUTWINX, -1),
						wxSize(m_winw, siz.y - 2 - MAURLHEIGHT - 2 - bsize.y - m_winy),
									wxTE_AUTO_SCROLL | wxTE_READONLY | wxTE_MULTILINE);
	MASETFONT(m_aboutTextCtrl);

} // MyFrameAbout
		

MyFrameAbout::~MyFrameAbout()
{
}


#if !defined(NOTEXTVIEWER)
 MyFrameViewer::MyFrameViewer(int windowid, const wxString& title, const wxPoint& pos, const wxSize& siz, long style)
				: wxFrame((wxWindow *)NULL, windowid, title, pos, siz, style)
 {
	m_windowid = windowid;
	
	m_panel = new wxPanel(this, -1, wxDefaultPosition, siz, wxCLIP_CHILDREN | wxNO_BORDER);
	
 #if defined(__WXMSW__) || (defined(__WXMAC__) && (MAGUIWX != 4))
  #define MAVIEWERWINY 26 // else buttons are half visible
 #else
  #define MAVIEWERWINY 1
 #endif
 #define MAVIEWERWINX 1
 #define MAFINDEIGHT 26

 #if defined(__WXX11__)
  #define MAVIEWERTEOPTIONS wxTE_NOHIDESEL | wxTE_AUTO_SCROLL | wxTE_MULTILINE // | otherwise cannot select text
 #else
  #define MAVIEWERTEOPTIONS wxTE_NOHIDESEL | wxTE_AUTO_SCROLL | wxTE_MULTILINE | wxTE_READONLY
 #endif

#if defined(__WXMSW__)
	if(maIsWin7())
	{
		m_winw = siz.x - (2 * MAVIEWERWINX) - XSIZEADJUST7; // else vertical slider is half visible
		m_winy = MAVIEWERWINY + YSIZEADJUST7; // else buttons are half visible
	}
	else
#endif
	{
		m_winw = siz.x - (2 * MAABOUTWINX); // else buttons are half visible
		m_winy = MAVIEWERWINY;
	}

	wxSize bsize = wxButton::GetDefaultSize();
	
	m_viewerTextCtrl = new MyViewerTextCtrl(m_panel, -1,
 #if defined(__MAOSX__) && defined(__WXMAC__) && (MAGUIWX == 6) && (wxMINOR_VERSION == 6)
			wxT(wxMAC_TEXTCONTROL_USE_MLTE), // works with change to src/mac/carbon/textctrl.cpp
 #else	
			wxT(""),
 #endif
			wxPoint(MAVIEWERWINX, -1),
			wxSize(m_winw,
				siz.y - 2 - MAFINDEIGHT - 2 - bsize.y - m_winy), MAVIEWERTEOPTIONS);

	m_viewerTextCtrl->SetEditable(false); // unset edit mode

	MASETFONT(m_viewerTextCtrl);

	#if (MAGUIWX >= 8) // wx-2.8+ 
		m_viewerTextCtrl->ChangeValue(MALOADINGSTRING); // as of 2.7.1
	#else	
		m_viewerTextCtrl->SetValue(MALOADINGSTRING);
	#endif

	m_findTextCtrl = new MyFindTextCtrl(m_panel, ID_EDITTEXT_VIEWERFIND, wxT(""),
						wxPoint(MAVIEWERWINX, siz.y - MAFINDEIGHT - 2 - bsize.y - m_winy),
						wxSize(m_winw, MAFINDEIGHT));
	// MASETFONT(m_findTextCtrl);
	
	m_findbutton = new wxButton(m_panel, ID_BTN_VIEWERFIND, wxT("Find"),
					wxPoint(MAVIEWERWINX + 2, siz.y - bsize.y - m_winy));

	if(windowid == WINDOWCATALOG)
	{
		m_playbutton = new wxButton(m_panel, ID_BTN_VIEWERPLAY, wxT("Play"),
					wxPoint(MAVIEWERWINX + bsize.x + 2 + bsize.x + bsize.x / 2, siz.y - bsize.y - m_winy));

		m_playStaticText = new wxStaticText(m_panel, ID_TEXT_VIEWERPLAY, wxT(""),
					wxPoint(MAVIEWERWINX + bsize.x + 2 + bsize.x + bsize.x + bsize.x / 2, siz.y - bsize.y - m_winy)
 #if defined(__WXX11__)
				, wxSize(200, -1));
 #else
				);
 #endif
	}

	m_foundStaticText = new wxStaticText(m_panel, ID_TEXT_VIEWERFOUND, wxT(""),
					wxPoint(MAVIEWERWINX + bsize.x + 2, siz.y - bsize.y - m_winy)
 #if defined(__WXX11__)
			, wxSize(200, -1));
 #else
			);
 #endif
	
	// MASETFONT(m_foundStaticText);
	
	m_editbutton = new wxButton(m_panel, ID_BTN_EDITSAVE, wxT("Edit"),
					wxPoint(siz.x - bsize.x - 2 - bsize.x - MAVIEWERWINX, siz.y - bsize.y - m_winy));

	m_closebutton = new wxButton(m_panel, wxID_OK, wxT("Close"),
					wxPoint(siz.x - bsize.x - 2 - MAVIEWERWINX, siz.y - bsize.y - m_winy));
	
 #if defined(__WXMAC__)
	m_timerleftup = new wxTimer(this, ID_TIMER_LEFTUP);
 #endif

} // MyFrameViewer
		

 MyFrameViewer::~MyFrameViewer()
 {
 #if defined(__WXMAC__)
	delete m_timerleftup;
 #endif
 }


 void MyFrameViewer::OnGUIButtonFind(wxCommandEvent & WXUNUSED(event))
 {
	ViewerOnButtonFind(m_windowid);

 } // OnGUIButtonFind


 void MyFrameViewer::OnGUIButtonVPlay(wxCommandEvent & WXUNUSED(event))
 {
	ViewerOnButtonPlay(m_windowid);
	
 } // OnGUIButtonVPlay


 void MyFrameViewer::OnGUIButtonEditSave(wxCommandEvent & WXUNUSED(event))
 {
	ViewerOnButtonEditSave(m_windowid);
	
 } // OnGUIButtonEditSave


 #if defined(__WXMAC__)
  void MyFrameViewer::OnTimerLeftUp(wxTimerEvent &event)
  {
	unsigned int bstate;
	
	// if left is still down
	if((bstate = GetCurrentEventButtonState()) &  (1 << 0)) // primary
		m_timerleftup->Start(80, TRUE); // restart timer
	else
		ViewerOnLeftClick(m_windowid);

  } // OnTimerLeftUp
 #endif


 int MyFrameViewer::get_windowid()
 {
	return m_windowid;
 }


 MyViewerTextCtrl *MyFrameViewer::get_viewerTextCtrl()
 {
	return m_viewerTextCtrl;
 }
#endif


#define PROMPTNUMBUTTONW 60
#ifdef __MAWIN__
 #define PROMPTNUMBUTTONH 26
 #define PROMPTNUMWINY 12 // more button pos adjustments for windows 7
#else
 #define PROMPTNUMBUTTONH 48
#endif
MyFramePromptNum::MyFramePromptNum( wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& siz, long style)
						: wxFrame((wxWindow *)NULL, id, title, pos, siz, style)
{
#if !defined(MANOWXPANEL)
	m_panel = new wxPanel(this, -1, wxDefaultPosition, siz, wxCLIP_CHILDREN | wxNO_BORDER);
#endif
	
	// msg text
	m_textpromptnum = new wxStaticText(MPANEL, ID_TEXT_PROMPTNUM, wxT("Preset"), wxPoint(10, 10));
	MASETFONT(m_textpromptnum);
	
#if defined(__WXMSW__)
	if(maIsWin7())
	{
		m_winx = 16;
		m_winw = PROMPTNUMXSIZEDEF - 20 - XSIZEADJUST7; // else vertical slider is half visible
		m_winy = PROMPTNUMYSIZEDEF - PROMPTNUMBUTTONH - 28 - YSIZEADJUST7; // else buttons are half visible
	}
	else
	{
		m_winx = 0;
		m_winw = PROMPTNUMXSIZEDEF - 20;
		m_winy = PROMPTNUMYSIZEDEF - PROMPTNUMBUTTONH - 28 - PROMPTNUMWINY;
	}
#else
	m_winx = 0;
	m_winw = PROMPTNUMXSIZEDEF - 20;
	m_winy = PROMPTNUMYSIZEDEF - PROMPTNUMBUTTONH - 10;
#endif

	// text editor
	m_edittextpromptnum  = new wxTextCtrl(MPANEL, ID_EDITTEXT_PROMPTNUM, wxT(""), wxPoint(10, 40),
														wxSize(m_winw, 26), wxTE_PROCESS_ENTER);
	
	// buttons
	m_buttonCancel = new wxButton(MPANEL, ID_BTN_PROMPTNUMCANCEL, wxT("Cancel"),
			wxPoint(PROMPTNUMXSIZEDEF - 2 * (PROMPTNUMBUTTONW + 10 + m_winx), m_winy),
			wxSize(PROMPTNUMBUTTONW, PROMPTNUMBUTTONH));

	m_buttonOk = new wxButton(MPANEL, ID_BTN_PROMPTNUMOK, wxT("Ok"),
			wxPoint(PROMPTNUMXSIZEDEF - (PROMPTNUMBUTTONW + 10 + m_winx), m_winy),
			wxSize(PROMPTNUMBUTTONW, PROMPTNUMBUTTONH));

} // MyFramePromptNum


MyFramePromptNum::~MyFramePromptNum()
{
}


void MyFramePromptNum::OnGUIButtonPromptNumOk(wxCommandEvent & WXUNUSED(event))
{
		DoPromptIndexOk();
}


void MyFramePromptNum::OnGUIButtonPromptNumCancel(wxCommandEvent & WXUNUSED(event))
{
	DoWindow(WINDOWPROMPTNUM, 0);
}


void MyFramePromptNum::OnGUIEditTextPromptNum(wxCommandEvent & WXUNUSED(event))
{
	DoPromptIndexOk();
}


MyFrameAdvanced::MyFrameAdvanced( wxWindowID id, const wxString& title,
											const wxPoint& pos, const wxSize& siz, long style)
						: wxFrame((wxWindow *)NULL, id, title, pos, siz, style)
{
	m_panel = new wxPanel(this, -1, wxDefaultPosition, siz, wxCLIP_CHILDREN | wxNO_BORDER);
	
	// closereopen checkbox
	new MACHECKBOX(m_panel, ID_CHK_CLOSEREOPEN,	
#if defined(MACLOSESOUNDDEF) && (MACLOSESOUNDDEF == 0)
	wxT(" Close and re-open sound channel between songs (default no)"),
#else
	wxT(" Close and re-open sound channel between songs (default yes)"),
#endif
		wxPoint(10, 20), CBSIZ, 0);

	// input monitor checkbox
	new MACHECKBOX(m_panel, ID_CHK_INPUTMONITOR, wxT(" Input Monitor (default yes)"), wxPoint(10, 40), CBSIZ, 0);

	// error-log non-playable files
	new MACHECKBOX(m_panel, ID_CHK_LOGNOTPLAYABLE,
							wxT(" Error-log non-playable files when creating a playlist (default no)"),
																				wxPoint(10, 60), CBSIZ, 0); 
	// use m3u files
	new MACHECKBOX(MPANEL, ID_CHK_USEM3U, wxT(" Use m3u files (default no)"), wxPoint(10, 80), CBSIZ, 0);

	// exit after list has played
	new MACHECKBOX(MPANEL, ID_CHK_EXITAFTER, wxT(" Exit after the playlist has played (default no)"), wxPoint(10, 100), CBSIZ, 0);

	// close window button
	m_closebutton = new wxButton(m_panel, wxID_OK, wxT("Close"), wxPoint(siz.x / 2 - 30, siz.y - 60));

} // MyFrameAdvanced


MyFrameAdvanced::~MyFrameAdvanced()
{
}


void MyFrameAdvanced::OnGUICheckCloseReopen(wxCommandEvent &event)
{
	DoCloseReopen(event.IsChecked() ? 1 : 0);
}


void MyFrameAdvanced::OnGUICheckInputMonitor(wxCommandEvent &event)
{
	DoInputMonitor(event.IsChecked() ? 1 : 0);
}


void MyFrameAdvanced::OnGUICheckLogNotPlayable(wxCommandEvent &event)
{
	DoLogNotPlayable(event.IsChecked() ? 1 : 0);
}


BEGIN_EVENT_TABLE(MyFrameApp, wxFrame)
	EVT_CHECKBOX(ID_CHK_ALTSIZE,		MyFrameApp::OnGUICheckAltSize) 
	EVT_CHECKBOX(ID_CHK_RANDOM,			MyFrameApp::OnGUICheckRandom)
	EVT_CHECKBOX(ID_CHK_REPTPLAYED,		MyFrameApp::OnGUICheckReptPlayed)
	EVT_CHECKBOX(ID_CHK_LOOPLOADED,		MyFrameApp::OnGUICheckLoopLoaded)
	EVT_CHECKBOX(ID_CHK_LOOPLIST,		MyFrameApp::OnGUICheckLoopList)
	EVT_CHECKBOX(ID_CHK_EQWINDOW,		MyFrameApp::OnGUICheckEqWindow)
	EVT_CHECKBOX(ID_CHK_LISTWINDOW,		MyFrameApp::OnGUICheckListWindow)
	EVT_CHECKBOX(ID_CHK_BEG,			MyFrameApp::OnGUICheckBoxBeg)
	EVT_CHECKBOX(ID_CHK_END,			MyFrameApp::OnGUICheckBoxEnd)
	EVT_CHECKBOX(ID_CHK_MONOLEFT,		MyFrameApp::OnGUICheckMonoLeft)
	EVT_CHECKBOX(ID_CHK_MONORIGHT,		MyFrameApp::OnGUICheckMonoRight)
	EVT_CHECKBOX(ID_CHK_SINGLEPLAY,		MyFrameApp::OnGUICheckSinglePlay)
	EVT_CHECKBOX(ID_CHK_EQUALIZER,		MyFrameApp::OnGUICheckEqualizer)
	EVT_CHECKBOX(ID_CHK_EFFECTS,		MyFrameApp::OnGUICheckEffects)
	EVT_CHECKBOX(ID_CHK_LIMITER,		MyFrameApp::OnGUICheckLimiter)
	EVT_CHECKBOX(ID_CHK_TITLEINFO,		MyFrameApp::OnGUICheckTitleInfo)
	EVT_CHECKBOX(ID_CHK_HOSTINENABLE,	MyFrameApp::OnGUICheckHostInEnable)
	EVT_CHECKBOX(ID_CHK_INVERTRIGHT,	MyFrameApp::OnGUICheckInvertRight)
	EVT_CHECKBOX(ID_CHK_REPTSONG,		MyFrameApp::OnGUICheckReptSong)
	EVT_CHECKBOX(ID_CHK_MONODIFF,		MyFrameApp::OnGUICheckMonoDiff)
	EVT_CHECKBOX(ID_CHK_BALANCE,		MyFrameApp::OnGUICheckBalance)

	EVT_BUTTON(ID_BTN_PREVIOUS,			MyFrameApp::OnGUIButtonPrevious)
	EVT_BUTTON(ID_BTN_PLAY,				MyFrameApp::OnGUIButtonPlay)
	EVT_BUTTON(ID_BTN_QUIT,				MyFrameApp::OnGUIButtonQuit)
	EVT_BUTTON(ID_BTN_PAUSE,			MyFrameApp::OnGUIButtonPause)
	EVT_BUTTON(ID_BTN_STOP,				MyFrameApp::OnGUIButtonStop)
	EVT_BUTTON(ID_BTN_NEXT,				MyFrameApp::OnGUIButtonNext)

	EVT_SLIDER(ID_SLIDER_VOLUMEL,		MyFrameApp::OnGUISliderVolumeL)
	EVT_SLIDER(ID_SLIDER_VOLUMER,		MyFrameApp::OnGUISliderVolumeR)
	EVT_SLIDER(ID_SLIDER_VOLUMESEP,		MyFrameApp::OnGUISliderSeparation)
	EVT_SLIDER(ID_SLIDER_PREAMP,		MyFrameApp::OnGUISliderPreamp)
	EVT_SLIDER(ID_SLIDER_POSITION,		MyFrameApp::OnGUISliderPos)	
	
	EVT_COMBOBOX(ID_POPUP_OUTPUT,		MyFrameApp::OnGUIPopupOutput)
	EVT_COMBOBOX(ID_POPUP_BUFFER,		MyFrameApp::OnGUIPopupBuffer)
	EVT_COMBOBOX(ID_POPUP_AUDIOOUTPUT,	MyFrameApp::OnGUIPopupAudioOutput)
	EVT_COMBOBOX(ID_POPUP_AUDIOINPUT,	MyFrameApp::OnGUIPopupAudioInput)
	EVT_COMBOBOX(ID_POPUP_AUDIOHOSTIN,	MyFrameApp::OnGUIPopupAudioHostIn)
	EVT_COMBOBOX(ID_POPUP_AUDIOHOSTOUT,	MyFrameApp::OnGUIPopupAudioHostOut)

	EVT_SPIN_UP(ID_SPIN_BUFFER,			MyFrameApp::OnGUISpinUpMem)
	EVT_SPIN_UP(ID_SPIN_PREA,			MyFrameApp::OnGUISpinUpPreamp)
	EVT_SPIN_UP(ID_SPIN_SEP,			MyFrameApp::OnGUISpinUpSep)
	EVT_SPIN_UP(ID_SPIN_VOLR,			MyFrameApp::OnGUISpinUpVolR)
	EVT_SPIN_UP(ID_SPIN_VOLL,			MyFrameApp::OnGUISpinUpVolL)
	EVT_SPIN_UP(ID_SPIN_BEG,			MyFrameApp::OnGUISpinUpBeg)
	EVT_SPIN_UP(ID_SPIN_END,			MyFrameApp::OnGUISpinUpEnd)

	EVT_SPIN_DOWN(ID_SPIN_BUFFER,		MyFrameApp::OnGUISpinDownBuffer)
	EVT_SPIN_DOWN(ID_SPIN_PREA,			MyFrameApp::OnGUISpinDownPreamp)
	EVT_SPIN_DOWN(ID_SPIN_SEP,			MyFrameApp::OnGUISpinDownSep)
	EVT_SPIN_DOWN(ID_SPIN_VOLR,			MyFrameApp::OnGUISpinDownVolR)
	EVT_SPIN_DOWN(ID_SPIN_VOLL,			MyFrameApp::OnGUISpinDownVolL)
	EVT_SPIN_DOWN(ID_SPIN_BEG,			MyFrameApp::OnGUISpinDownBeg)
	EVT_SPIN_DOWN(ID_SPIN_END,			MyFrameApp::OnGUISpinDownEnd)

	EVT_MENU(ID_MENU_ABOUT,				MyFrameApp::OnAbout)
	EVT_MENU(ID_MENU_QUIT,				MyFrameApp::OnGUIMenuQuit)
	EVT_MENU(ID_MENU_OPENLIST,			MyFrameApp::OnGUIMenuOpenList)
	EVT_MENU(ID_MENU_MAKELIST,			MyFrameApp::OnGUIMenuMakeList)
	EVT_MENU(ID_MENU_SAVELISTAS,		MyFrameApp::OnGUIMenuSaveListAs)
	EVT_MENU(ID_MENU_DELETELIST,		MyFrameApp::OnGUIMenuDeleteList)
	EVT_MENU(ID_MENU_PLAYFILE,			MyFrameApp::OnGUIMenuPlayFile)
	EVT_MENU(ID_SUBMENU_OPENRECENT1,	MyFrameApp::OnGUIMenuOpenRecent)
	EVT_MENU(ID_SUBMENU_OPENRECENT2,	MyFrameApp::OnGUIMenuOpenRecent)
	EVT_MENU(ID_SUBMENU_OPENRECENT3,	MyFrameApp::OnGUIMenuOpenRecent)
	EVT_MENU(ID_SUBMENU_OPENRECENT4,	MyFrameApp::OnGUIMenuOpenRecent)
	EVT_MENU(ID_SUBMENU_OPENRECENT5,	MyFrameApp::OnGUIMenuOpenRecent)
	EVT_MENU(ID_SUBMENU_OPENRECENT6,	MyFrameApp::OnGUIMenuOpenRecent)
	EVT_MENU(ID_SUBMENU_OPENRECENT7,	MyFrameApp::OnGUIMenuOpenRecent)
	EVT_MENU(ID_SUBMENU_OPENRECENT8,	MyFrameApp::OnGUIMenuOpenRecent)
	EVT_MENU(ID_SUBMENU_OPENRECENT9,	MyFrameApp::OnGUIMenuOpenRecent)
	EVT_MENU(ID_SUBMENU_OPENRECENT10,	MyFrameApp::OnGUIMenuOpenRecent)

	EVT_MENU(ID_MENU_SETVDISK,			MyFrameApp::OnGUIMenuSetVDisk)

	EVT_MENU(ID_MENU_MULTIPLEINST,		MyFrameApp::OnGUIMenuMultipleInst)

#if defined(MAKEYBOARD)
	EVT_MENU(ID_MENU_KEYBOARD,			MyFrameApp::OnGUIMenuKeyboard)
#endif

#if defined(MAMOUSEWHEEL)
	EVT_MENU(ID_MENU_MOUSEWHEEL,		MyFrameApp::OnGUIMenuMouseWheel)
#endif

	EVT_MENU(ID_MENU_ADVANCED,			MyFrameApp::OnGUIMenuAdvanced)

#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
	EVT_MENU(ID_MENU_SETAUDIOCD,		MyFrameApp::OnGUIMenuSetsAudioCD)
#endif
	
	EVT_MENU(ID_MENU_SETFACTORY,		MyFrameApp::OnGUIMenuSetsFactory)
	EVT_MENU(ID_MENU_SETDEFAULT,		MyFrameApp::OnGUIMenuSetsDefault)
	EVT_MENU(ID_MENU_SETSAVED,			MyFrameApp::OnGUIMenuSetsSaved)
	EVT_MENU(ID_MENU_SETCOPY,			MyFrameApp::OnGUIMenuSetsCopy)
	
	EVT_MENU(ID_MENU_STOP,				MyFrameApp::OnGUIMenuStop)
	EVT_MENU(ID_MENU_PAUSE,				MyFrameApp::OnGUIMenuPause)
	EVT_MENU(ID_MENU_PREVIOUS,			MyFrameApp::OnGUIMenuPrevious)
	EVT_MENU(ID_MENU_PLAY,				MyFrameApp::OnGUIMenuPlay)
	EVT_MENU(ID_MENU_NEXT,				MyFrameApp::OnGUIMenuNext)
	EVT_MENU(ID_MENU_PLAYCD,			MyFrameApp::OnGUIMenuPlayCD)
	EVT_MENU(ID_MENU_EJECTCD,			MyFrameApp::OnGUIMenuEjectCD)
	EVT_MENU(ID_MENU_AUDIOIN,			MyFrameApp::OnGUIMenuAudioIn)

	EVT_MENU(ID_SUBMENU_TITLEINFO1,		MyFrameApp::OnGUIMenuTitleInfo)
	EVT_MENU(ID_SUBMENU_TITLEINFO2,		MyFrameApp::OnGUIMenuTitleInfo)
	EVT_MENU(ID_SUBMENU_TITLEINFO3,		MyFrameApp::OnGUIMenuTitleInfo)

	EVT_MENU(ID_MENU_RANDOM,			MyFrameApp::OnGUIMenuRandom)
	EVT_MENU(ID_MENU_REPTPLAYED,		MyFrameApp::OnGUIMenuReptPlayed)
	EVT_MENU(ID_MENU_LOOPLOADED,		MyFrameApp::OnGUIMenuLoopLoaded)
	EVT_MENU(ID_MENU_LOOPLIST,			MyFrameApp::OnGUIMenuLoopList)
	EVT_MENU(ID_MENU_AUTOSTART,			MyFrameApp::OnGUIMenuAutoStart)
	EVT_MENU(ID_MENU_ERRLOGGING,		MyFrameApp::OnGUIMenuErrLogging)
	EVT_MENU(ID_MENU_SINGLEPLAY,		MyFrameApp::OnGUIMenuSinglePlay)
	EVT_MENU(ID_MENU_INPUTMONITOR,		MyFrameApp::OnGUIMenuInMonitor)
	EVT_MENU(ID_MENU_EQUALIZER,			MyFrameApp::OnGUIMenuEqualizer)
	EVT_MENU(ID_MENU_EFFECTS,			MyFrameApp::OnGUIMenuEffects)
	EVT_MENU(ID_MENU_EQEXTRA,			MyFrameApp::OnGUIMenuEqExtra)
	EVT_MENU(ID_MENU_MONO,				MyFrameApp::OnGUIMenuMono)
	EVT_MENU(ID_MENU_INVERTRIGHT,		MyFrameApp::OnGUIMenuInvertRight)
	EVT_MENU(ID_MENU_EQ31BANDS,			MyFrameApp::OnGUIMenuEq31Bands)
	
	EVT_MENU(ID_MENU_SAVEMASETS,		MyFrameApp::OnGUIMenuSaveSettings)
	EVT_MENU(ID_MENU_SAVEMASETSDEFAULT,	MyFrameApp::OnGUIMenuSaveSettingsDefault)
	EVT_MENU(ID_MENU_ZEROLIST,			MyFrameApp::OnGUIMenuZeroList)
	EVT_MENU(ID_MENU_MAKECATALOG,		MyFrameApp::OnGUIMenuMakeCatalog)
#if !defined(NOTEXTVIEWER)
	EVT_MENU(ID_MENU_VIEWCATALOG,		MyFrameApp::OnGUIMenuViewCatalog)
	EVT_MENU(ID_MENU_VIEWPLAYLIST,		MyFrameApp::OnGUIMenuViewPlaylist)
	EVT_MENU(ID_MENU_VIEWLOG,			MyFrameApp::OnGUIMenuViewLog)
	EVT_MENU(ID_MENU_README,			MyFrameApp::OnGUIMenuReadme)
#endif
#if defined(MAKEYBOARD)
 #if !defined(NOTEXTVIEWER)
	EVT_MENU(ID_MENU_EDITKEYBINDINGS,	MyFrameApp::OnGUIMenuEditKeyBindings)
	EVT_MENU(ID_MENU_BLANKKEYBINDINGS,	MyFrameApp::OnGUIMenuBlankKeyBindings)
	EVT_MENU(ID_MENU_FACTORYKEYBINDINGS,MyFrameApp::OnGUIMenuFactoryKeyBindings)
 #endif
#endif

	EVT_MENU(ID_MENU_RESCANAUDIODEVICES,MyFrameApp::OnGUIMenuRescanAudioDevices)

	EVT_MENU(ID_MENU_CLEARLOG,			MyFrameApp::OnGUIMenuClearLog)
	EVT_MENU(ID_MENU_SHOWLIST,			MyFrameApp::OnGUIMenuShowList)
	EVT_MENU(ID_MENU_SHOWEQ,			MyFrameApp::OnGUIMenuShowEq)
	EVT_MENU(ID_MENU_SHOWEFFECTS,		MyFrameApp::OnGUIMenuShowEffects)
	EVT_MENU(ID_MENU_SHOWFILEOUT,		MyFrameApp::OnGUIMenuShowFileOut)
	EVT_MENU(ID_MENU_ALTSIZE,			MyFrameApp::OnGUIMenuAltSize)
	EVT_MENU(ID_MENU_SAVEDSIZE,			MyFrameApp::OnGUIMenuSavedSize)
	EVT_MENU(ID_MENU_DEFAULTSIZE,		MyFrameApp::OnGUIMenuDefaultSize)
	EVT_MENU(ID_MENU_FACTORYSIZE,		MyFrameApp::OnGUIMenuFactorySize)

	EVT_TIMER(ID_TIMER_CHANGEPLAYLIST,	MyFrameApp::OnTimerChangePlaylist)
	EVT_TIMER(ID_TIMER_PLAYPLAYLIST,	MyFrameApp::OnTimerPlayPlaylist)
	EVT_TIMER(ID_TIMER_RUN,				MyFrameApp::OnTimerRunLoop)
	EVT_TIMER(ID_TIMER_PLAY,			MyFrameApp::OnTimerPlay)
	// EVT_TIMER(ID_TIMER_STOP,			MyFrameApp::OnTimerStop)
	EVT_CLOSE(							MyFrameApp::OnGUICloseRequest) // DoAppQuit
	EVT_SIZE(							MyApp::OnGUISized)
	EVT_MOVE(							MyApp::OnGUIMoved)
END_EVENT_TABLE()


BEGIN_EVENT_TABLE(MyFrameList, wxFrame)
	EVT_BUTTON(ID_BTN_INFO,					MyFrameList::OnGUIButtonInfo)
	EVT_BUTTON(ID_BTN_PLAYLIST,				MyFrameList::OnGUIButtonList)
	EVT_BUTTON(ID_BTN_LOADED,				MyFrameList::OnGUIButtonLoaded)
	EVT_BUTTON(ID_BTN_SONGINFO,				MyFrameList::OnGUIButtonSong)
	EVT_CHECKBOX(ID_CHK_SHOWINDEX,			MyFrameList::OnGUICheckShowIndex)
	EVT_CHECKBOX(ID_CHK_SHOWDIR,			MyFrameList::OnGUICheckShowDir)
	EVT_CHECKBOX(ID_CHK_SHOWPATH,			MyFrameList::OnGUICheckShowPath)
	EVT_CHECKBOX(ID_CHK_ADDMODE,			MyFrameList::OnGUICheckAddMode)
	EVT_LIST_COL_END_DRAG(ID_LIST_CTRL,		MyFrameList::OnGUIColumnEndDrag)
	EVT_LIST_ITEM_SELECTED(ID_LIST_CTRL,	MyFrameList::OnGUIListSelected)
	EVT_LIST_ITEM_ACTIVATED(ID_LIST_CTRL,	MyFrameList::OnGUIListActivated)
	EVT_CLOSE(								MyApp::OnGUICloseRequest)
	EVT_SIZE(								MyApp::OnGUISized)
	EVT_MOVE(								MyApp::OnGUIMoved)
END_EVENT_TABLE()


BEGIN_EVENT_TABLE(MyFrameEq, wxFrame)
	EVT_CHECKBOX(ID_CHK_EQZOOM,		MyFrameEq::OnGUICheckEqZoom)
	EVT_CHECKBOX(ID_CHK_EQLINK,		MyFrameEq::OnGUICheckEqLink)
	EVT_CHECKBOX(ID_CHK_EQUALIZERB, MyFrameEq::OnGUICheckEqualizerB)
	EVT_CHECKBOX(ID_CHK_EQ31BANDSB,	MyFrameEq::OnGUICheckEq31BandsB)
	EVT_CHECKBOX(ID_CHK_EQEXTRAB,   MyFrameEq::OnGUICheckEqExtraB)
	EVT_CHECKBOX(ID_CHK_SEPARATELR, MyFrameEq::OnGUICheckSeparateLR)

	EVT_BUTTON(ID_BTN_EQMINUS,		MyFrameEq::OnGUIButtonEqMinus)
	EVT_BUTTON(ID_BTN_EQPLUS,		MyFrameEq::OnGUIButtonEqPlus)
	EVT_BUTTON(ID_BTN_EQMID,		MyFrameEq::OnGUIButtonEqMid)
	EVT_BUTTON(ID_BTN_EQCOPY,		MyFrameEq::OnGUIButtonEqCopy)
	EVT_BUTTON(ID_BTN_EQPASTE,		MyFrameEq::OnGUIButtonEqPaste)
	EVT_BUTTON(ID_BTN_EQRESTORE,	MyFrameEq::OnGUIButtonEqRestore)
	EVT_BUTTON(ID_BTN_EQSHIFTL,		MyFrameEq::OnGUIButtonEqShiftL)
	EVT_BUTTON(ID_BTN_EQSHIFTR,		MyFrameEq::OnGUIButtonEqShiftR)
	EVT_BUTTON(ID_BTN_EQINVERT,		MyFrameEq::OnGUIButtonEqInvert)

	EVT_COMBOBOX(ID_POPUP_EQPRESET,	MyFrameEq::OnGUIPopupEqPreset)
	EVT_TEXT_ENTER(ID_EDITTEXT_EQ,	MyFrameEq::OnGUIEditTextEq)
	EVT_SLIDER(ID_SLIDER_EQ0,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ1,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ3,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ4,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ5,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ6,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ7,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ8,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ9,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ10,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ11,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ12,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ13,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ14,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ15,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ16,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ17,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ18,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ19,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ20,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ21,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ22,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ23,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ24,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ25,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ26,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ27,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ28,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ29,		MyFrameEq::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ30,		MyFrameEq::OnGUISliderEq)
	
	EVT_SPIN_UP(ID_SPIN_EQOFFSET,	MyFrameEq::OnGUISpinUpEqOffset)
	
	EVT_SPIN_UP(ID_SPIN_EQ0,		MyFrameEq::OnGUISpinUpEq) 
	EVT_SPIN_UP(ID_SPIN_EQ1,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ3,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ4,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ5,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ6,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ7,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ8,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ9,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ10,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ11,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ12,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ13,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ14,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ15,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ16,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ17,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ18,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ19,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ20,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ21,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ22,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ23,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ24,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ25,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ26,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ27,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ28,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ29,		MyFrameEq::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ30,		MyFrameEq::OnGUISpinUpEq)
	
	EVT_SPIN_DOWN(ID_SPIN_EQOFFSET,	MyFrameEq::OnGUISpinDownEqOffset)

	EVT_SPIN_DOWN(ID_SPIN_EQ0,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ1,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ3,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ4,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ5,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ6,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ7,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ8,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ9,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ10,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ11,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ12,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ13,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ14,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ15,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ16,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ17,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ18,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ19,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ20,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ21,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ22,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ23,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ24,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ25,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ26,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ27,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ28,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ29,		MyFrameEq::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ30,		MyFrameEq::OnGUISpinDownEq)
	EVT_CLOSE(						MyApp::OnGUICloseRequest)
	EVT_SIZE(						MyApp::OnGUISized)
	EVT_MOVE(						MyApp::OnGUIMoved)
END_EVENT_TABLE()
	

BEGIN_EVENT_TABLE(MyFrameEq2, wxFrame)
	EVT_BUTTON(ID_BTN_EQ2MID,		MyFrameEq2::OnGUIButtonEqMid)
	EVT_BUTTON(ID_BTN_EQ2COPY,		MyFrameEq2::OnGUIButtonEqCopy)
	EVT_BUTTON(ID_BTN_EQ2PASTE,		MyFrameEq2::OnGUIButtonEqPaste)
	EVT_BUTTON(ID_BTN_EQ2RESTORE,	MyFrameEq2::OnGUIButtonEqRestore)

	EVT_SLIDER(ID_SLIDER_EQ2_0,		MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_1,		MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_2,		MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_3,		MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_4,		MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_5,		MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_6,		MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_7,		MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_8,		MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_9,		MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_10,	MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_11,	MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_12,	MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_13,	MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_14,	MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_15,	MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_16,	MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_17,	MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_18,	MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_19,	MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_20,	MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_21,	MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_22,	MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_23,	MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_24,	MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_25,	MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_26,	MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_27,	MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_28,	MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ2_29,	MyFrameEq2::OnGUISliderEq)
	EVT_SLIDER(ID_SLIDER_EQ30,		MyFrameEq2::OnGUISliderEq)
	
	EVT_SPIN_UP(ID_SPIN_EQ2_0,		MyFrameEq2::OnGUISpinUpEq) 
	EVT_SPIN_UP(ID_SPIN_EQ2_1,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_2,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_3,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_4,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_5,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_6,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_7,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_8,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_9,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_10,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_11,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_12,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_13,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_14,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_15,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_16,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_17,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_18,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_19,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_20,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_21,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_22,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_23,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_24,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_25,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_26,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_27,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_28,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_29,		MyFrameEq2::OnGUISpinUpEq)
	EVT_SPIN_UP(ID_SPIN_EQ2_30,		MyFrameEq2::OnGUISpinUpEq)
	
	EVT_SPIN_DOWN(ID_SPIN_EQ2_0,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_1,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_2,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_3,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_4,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_5,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_6,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_7,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_8,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_9,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_10,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_11,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_12,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_13,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_14,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_15,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_16,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_17,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_18,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_19,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_20,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_21,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_22,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_23,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_24,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_25,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_26,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_27,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_28,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_29,	MyFrameEq2::OnGUISpinDownEq)
	EVT_SPIN_DOWN(ID_SPIN_EQ2_30,	MyFrameEq2::OnGUISpinDownEq)
	EVT_CLOSE(						MyApp::OnGUICloseRequest)
	EVT_SIZE(						MyApp::OnGUISized)
	EVT_MOVE(						MyApp::OnGUIMoved)
END_EVENT_TABLE()
	

BEGIN_EVENT_TABLE(MyFrameEffects, wxFrame)
	EVT_SLIDER(ID_SLIDER_PITCH,		MyFrameEffects::OnGUISliderPitch)
	EVT_SLIDER(ID_SLIDER_PITCH2,	MyFrameEffects::OnGUISliderPitch2)
	EVT_SLIDER(ID_SLIDER_TEMPO,		MyFrameEffects::OnGUISliderTempo)
	EVT_SLIDER(ID_SLIDER_SPEED,		MyFrameEffects::OnGUISliderSpeed)

	EVT_SPIN_DOWN(ID_SPIN_PITCH,	MyFrameEffects::OnGUISpinDownPitch)
	EVT_SPIN_DOWN(ID_SPIN_PITCH2,	MyFrameEffects::OnGUISpinDownPitch2)
	EVT_SPIN_DOWN(ID_SPIN_TEMPO,	MyFrameEffects::OnGUISpinDownTempo)
	EVT_SPIN_DOWN(ID_SPIN_SPEED,	MyFrameEffects::OnGUISpinDownSpeed)
	EVT_SPIN_UP(ID_SPIN_PITCH,		MyFrameEffects::OnGUISpinUpPitch)
	EVT_SPIN_UP(ID_SPIN_PITCH2,		MyFrameEffects::OnGUISpinUpPitch2)
	EVT_SPIN_UP(ID_SPIN_TEMPO,		MyFrameEffects::OnGUISpinUpTempo)
	EVT_SPIN_UP(ID_SPIN_SPEED,		MyFrameEffects::OnGUISpinUpSpeed)

	EVT_CHECKBOX(ID_CHK_PITCH,		MyFrameEffects::OnGUICheckPitch)
	EVT_CHECKBOX(ID_CHK_TEMPO,		MyFrameEffects::OnGUICheckTempo)
	EVT_CHECKBOX(ID_CHK_SPEED,		MyFrameEffects::OnGUICheckSpeed)
	EVT_CHECKBOX(ID_CHK_EFFECTSB,   MyFrameEffects::OnGUICheckEffectsB)
	EVT_CHECKBOX(ID_CHK_SBSMS,		MyFrameEffects::OnGUICheckSBSMS)

	EVT_CLOSE(						MyApp::OnGUICloseRequest)
	EVT_SIZE(						MyApp::OnGUISized)
	EVT_MOVE(						MyApp::OnGUIMoved)
END_EVENT_TABLE()
	

#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
BEGIN_EVENT_TABLE(MyFrameAudioCD, wxFrame)
	EVT_COMBOBOX(ID_POPUP_AUDIOCD,	MyFrameAudioCD::OnGUIPopupAudioCD)
	EVT_BUTTON(ID_BTN_AUDIOCDSET,	MyFrameAudioCD::OnGUIButtonAudioCDSet)
	EVT_BUTTON(ID_BTN_AUDIOCDCLEAR,	MyFrameAudioCD::OnGUIButtonAudioCDClear)
	EVT_BUTTON(ID_BTN_AUDIOCDSAVE,	MyFrameAudioCD::OnGUIButtonAudioCDSave)
	EVT_CHECKBOX(ID_CHK_CDPARANOIA,	MyFrameAudioCD::OnGUICheckCdParanoia)
	EVT_BUTTON(wxID_OK,				MyFrameAudioCD::OnGUIButtonClose)
	EVT_CLOSE(						MyApp::OnGUICloseRequest)
	EVT_SIZE(						MyApp::OnGUISized)
	EVT_MOVE(						MyApp::OnGUIMoved)
END_EVENT_TABLE()
#endif


BEGIN_EVENT_TABLE(MyFrameFileOut, wxFrame)
	EVT_COMBOBOX(ID_POPUP_BITRATE,		MyFrameFileOut::OnGUIPopupBitRate)
	EVT_COMBOBOX(ID_POPUP_QUALITY,		MyFrameFileOut::OnGUIPopupQuality)
	EVT_COMBOBOX(ID_POPUP_MODE,			MyFrameFileOut::OnGUIPopupMode)
	EVT_COMBOBOX(ID_POPUP_SAMPLERATE,   MyFrameFileOut::OnGUIPopupSampleRate)
	EVT_BUTTON(ID_BTN_FILEOUTDIR,		MyFrameFileOut::OnGUIButtonFileOutDir)
#if defined(ENCODEAAC)
	EVT_CHECKBOX(ID_CHK_ADTS,			MyFrameFileOut::OnGUICheckBoxADTS)	
#endif
#if defined(ENCODEMP4)
	EVT_CHECKBOX(ID_CHK_M4A,			MyFrameFileOut::OnGUICheckBoxM4A)
#endif
	EVT_CLOSE(							MyApp::OnGUICloseRequest)
	EVT_SIZE(							MyApp::OnGUISized)
	EVT_MOVE(							MyApp::OnGUIMoved)
END_EVENT_TABLE()
	

BEGIN_EVENT_TABLE(MyFrameAbout, wxFrame)
	EVT_BUTTON(wxID_OK,	MyFrameAbout::OnGUIButtonClose)
	EVT_CLOSE(			MyApp::OnGUICloseRequest)
	EVT_SIZE(			MyApp::OnGUISized)
	EVT_MOVE(			MyApp::OnGUIMoved)
END_EVENT_TABLE()


#if !defined(NOTEXTVIEWER)
 BEGIN_EVENT_TABLE(MyFrameViewer, wxFrame)
	EVT_BUTTON(ID_BTN_VIEWERFIND,   MyFrameViewer::OnGUIButtonFind)
	EVT_BUTTON(ID_BTN_VIEWERPLAY,   MyFrameViewer::OnGUIButtonVPlay)
	EVT_BUTTON(wxID_OK,				MyFrameViewer::OnGUIButtonClose)
	EVT_BUTTON(ID_BTN_EDITSAVE,		MyFrameViewer::OnGUIButtonEditSave)
 #if defined(__WXMAC__)
	EVT_TIMER(ID_TIMER_LEFTUP,		MyFrameViewer::OnTimerLeftUp)
 #endif
	EVT_CLOSE(						MyApp::OnGUICloseRequest)
	EVT_SIZE(						MyApp::OnGUISized)
	EVT_MOVE(						MyApp::OnGUIMoved)
 END_EVENT_TABLE() 
#endif


#if !defined(NOTEXTVIEWER)
 BEGIN_EVENT_TABLE(MyViewerTextCtrl, wxTextCtrl)
	EVT_MOUSE_EVENTS(MyViewerTextCtrl::OnGUIMouse)
 END_EVENT_TABLE()


 BEGIN_EVENT_TABLE(MyFindTextCtrl, wxTextCtrl)
	EVT_CHAR(MyFindTextCtrl::OnGUIChar)
	EVT_MOUSE_EVENTS(MyFindTextCtrl::OnGUIMouse)
 END_EVENT_TABLE()
#endif


BEGIN_EVENT_TABLE(MyFramePromptNum, wxFrame)
	EVT_BUTTON(ID_BTN_PROMPTNUMCANCEL,		MyFramePromptNum::OnGUIButtonPromptNumCancel)
	EVT_BUTTON(ID_BTN_PROMPTNUMOK,			MyFramePromptNum::OnGUIButtonPromptNumOk)
	EVT_TEXT_ENTER(ID_EDITTEXT_PROMPTNUM,	MyFramePromptNum::OnGUIEditTextPromptNum)
	EVT_SIZE(								MyApp::OnGUISized)
	EVT_MOVE(								MyApp::OnGUIMoved)
	EVT_CLOSE(								MyApp::OnGUICloseRequest)
END_EVENT_TABLE()


BEGIN_EVENT_TABLE(MyFrameAdvanced, wxFrame)
	EVT_CHECKBOX(ID_CHK_CLOSEREOPEN,		MyFrameAdvanced::OnGUICheckCloseReopen)
	EVT_CHECKBOX(ID_CHK_INPUTMONITOR,		MyFrameAdvanced::OnGUICheckInputMonitor)
	EVT_CHECKBOX(ID_CHK_LOGNOTPLAYABLE,		MyFrameAdvanced::OnGUICheckLogNotPlayable)
	EVT_CHECKBOX(ID_CHK_USEM3U,				MyFrameAdvanced::OnGUICheckUseM3u)
	EVT_CHECKBOX(ID_CHK_EXITAFTER,			MyFrameAdvanced::OnGUICheckExitAfter)
	EVT_BUTTON(wxID_OK,						MyFrameAdvanced::OnGUIButtonClose)
	EVT_CLOSE(								MyApp::OnGUICloseRequest)
	EVT_SIZE(								MyApp::OnGUISized)
	EVT_MOVE(								MyApp::OnGUIMoved)
END_EVENT_TABLE()

#ifdef __cplusplus
extern "C"
{
#endif

// api
/* not used void SpinButtonEnable(int windowid, int button)
{
#ifdef MATRACEGUIWX
	maTrace("PushButtonEnable");
#endif
	PushButtonEnable(window, button);
}


void SpinButtonDisable(int windowid, int button)
{
#ifdef MATRACEGUIWX
	maTrace("SpinButtonDisable");
#endif
	PushButtonDisable(window, button);
}
*/


void PushButtonPaintUp(int windowid, int button)
{
#ifdef MATRACEGUIWX
	maTrace("PushButtonPaintUp");
#endif
#if defined(MAUSEMABUTTON)
	wxWindow *pb = ((wxWindow *)g_w[windowid])->FindWindowById(button, (wxWindow *)g_w[windowid]);
	
	if(pb)
		((MABUTTON *)pb)->PaintUp();
	else
		maLogAPI(L"PushButtonPaintUp nopb", windowid, button);
#endif
}


void PushButtonPaintDown(int windowid, int button)
{
#ifdef MATRACEGUIWX
	maTrace("PushButtonPaintDown");
#endif
#if defined(MAUSEMABUTTON)
	wxWindow *pb = ((wxWindow *)g_w[windowid])->FindWindowById(button, (wxWindow *)g_w[windowid]);
	
	if(pb)
		((MABUTTON *)pb)->PaintDown();
	else
		maLogAPI(L"PushButtonPaintDown nopb", windowid, button);
#endif
}


/*
void PushButtonPopup(int windowid, int button)
{
#ifdef MATRACEGUIWX
	maTrace("PushButtonPopup");
#endif
	wxWindow *pb = ((wxWindow *)g_w[windowid])->FindWindowById(button, (wxWindow *)g_w[windowid]);
	
	if(pb)
		((MABUTTON *)pb)->PopUp();
	else
		maLogAPI(L"PushButtonPopup nopb", windowid, button);
}
*/

void PushButtonEnable(int windowid, int button)
{
#ifdef MATRACEGUIWX
	maTrace("PushButtonEnable");
#endif
	wxWindow *pb = ((wxWindow *)g_w[windowid])->FindWindowById(button, (wxWindow *)g_w[windowid]);
	
	if(pb)
		pb->Enable(TRUE);
	else
		maLogAPI(L"PushButtonEnable nopb", windowid, button);
}


void PushButtonDisable(int windowid, int button)
{
#ifdef MATRACEGUIWX
	maTrace("PushButtonDisable");
#endif
	wxWindow *pb = ((wxWindow *)g_w[windowid])->FindWindowById(button, (wxWindow *)g_w[windowid]);	
	if(pb)
		pb->Disable();
	else
		maLogAPI(L"PushButtonDisable nopb", windowid, button);
}


void PushButtonSimulateClick(int windowid, int button)
{
#ifdef MATRACEGUIWX
	maTrace("PushButtonSimulateClick");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(button, (wxWindow *)g_w[windowid]);
	
	if(cb)
	{
		wxCommandEvent event(wxEVT_COMMAND_BUTTON_CLICKED, button);
		event.SetEventObject(cb);
		((wxButton *)cb)->Command(event);
	}
	else
		maLogAPI(L"PushButtonSimulateClick no widget", windowid, button);
}


void PushButtonSetDescriptor(int windowid, int button, machar *desc)
{
#ifdef MATRACEGUIWX
	maTrace("PushButtonSetDescriptor");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(button, (wxWindow *)g_w[windowid]);
	
	if(cb)
		((wxButton *)cb)->SetLabel(desc);
	else
		maLogAPI(L"PushButtonSetDescriptor no widget", windowid, button);
}


void PushButtonGetDescriptor(int windowid, int button, machar *desc)
{
#ifdef MATRACEGUIWX
	maTrace("PushButtonGetDescriptor");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(button, (wxWindow *)g_w[windowid]);
	
	if(cb)
		mastrcpy(desc, ((wxButton *)cb)->GetLabel().c_str());
	else
		maLogAPI(L"PushButtonGetDescriptor no widget", windowid, button);
}


void PopupButtonEnable(int windowid, int popup)
{
#ifdef MATRACEGUIWX
	maTrace("PopupButtonEnable");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(popup, (wxWindow *)g_w[windowid]);
	
	if(cb)
		((wxComboBox *)cb)->Enable();
	else
		maLogAPI(L"PopupButtonEnable no widget", windowid, popup);
}


void PopupButtonDisable(int windowid, int popup)
{
#ifdef MATRACEGUIWX
	maTrace("PopupButtonDisable");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(popup, (wxWindow *)g_w[windowid]);
	
	if(cb)
	{
		((wxComboBox *)cb)->Disable();

		// workaround for problem where ComboBox traps mousewheel when disabled
#if defined(__MAWIN__) && defined(MAMOUSEWHEEL)
		if(popup == ID_POPUP_OUTPUT || popup == ID_POPUP_BUFFER)
			if(l_ChkAltSize)
				l_ChkAltSize->SetFocus(); // move focus elsewhere, alt size checkbox is as good as any
#endif
	}
	else
		maLogAPI(L"PopupButtonDisable no widget", windowid, popup);
}


void PopupButtonClear(int windowid, int popup)
{
#ifdef MATRACEGUIWX
	maTrace("PopupButtonClear");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(popup, (wxWindow *)g_w[windowid]);
	
	if(cb)
		((wxComboBox *)cb)->Clear();
	else
		maLogAPI(L"PopupButtonClear no widget", windowid, popup);
}


void PopupButtonAppend(int windowid, int popup, machar *text)
{
#ifdef MATRACEGUIWX
	maTrace("PopupButtonAppend");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(popup, (wxWindow *)g_w[windowid]);
	
	if(cb)
		((wxComboBox *)cb)->Append(text);
	else
		maLogAPI(L"PopupButtonAppend no widget", windowid, popup);
}


void PopupButtonSetValue(int windowid, int popup, int value)
{
#ifdef MATRACEGUIWX
	maTrace("PopupButtonSetValue");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(popup, (wxWindow *)g_w[windowid]);
	
	if(cb)
		((wxComboBox *)cb)->SetSelection(value);
	else
		maLogAPI(L"PopupButtonSetValue no widget", windowid, popup);
}


void PopupButtonSetText(int windowid, int popup, int item, machar *text)
{
#ifdef MATRACEGUIWX
	maTrace("PopupButtonSetText");
#endif
#if !defined(__WXGTK__) // don't have SetString, use SetSelection
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(popup, (wxWindow *)g_w[windowid]);
	
	if(cb)
		((wxComboBox *)cb)->SetString(item, text);
	else
		maLogAPI(L"PopupButtonSetText no widget", windowid, popup);
#endif
}


#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
 void PopupButtonGetSelection(int windowid, int popup, int *item)
 {
 #ifdef MATRACEGUIWX
	maTrace("PopupButtonGetSelection");
 #endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(popup, (wxWindow *)g_w[windowid]);
	
	if(cb)
		*item = ((wxComboBox *)cb)->GetSelection();
	else
	{
		*item = -1;
		maLogAPI(L"PopupButtonGetSelection no widget", windowid, popup);
	}
 }


 void PopupButtonSetSelection(int windowid, int popup, int item)
 {
 #ifdef MATRACEGUIWX
	maTrace("PopupButtonSetSelection");
 #endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(popup, (wxWindow *)g_w[windowid]);
	
	if(cb)
		((wxComboBox *)cb)->SetSelection(item);
	else
		maLogAPI(L"PopupButtonSetSelection no widget", windowid, popup);
 }
#endif


/* void PopupButtonGetText(int windowid, int edittext, int item, machar *outBuffer, int inBufferLength, long *outTextLength)
{
#ifdef MATRACEGUIWX
	maTrace("PopupButtonGetText");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(edittext, (wxWindow *)g_w[windowid]);
	
	if(cb)
	{
		strcpy(outBuffer, ((wxComboBox *)cb)->GetString(item));
		*outTextLength = strlen(outBuffer);
	}
	else
		maLogAPI(L"PopupButtonGetText no widget", windowid, edittext);
}
*/


void LittleArrowEnable(int windowid, int arrow)
{
#ifdef MATRACEGUIWX
	maTrace("LittleArrowEnable");
#endif
	wxSpinButton *cb;
	cb = (wxSpinButton *)((wxWindow *)g_w[windowid])->FindWindowById(arrow, (wxWindow *)g_w[windowid]);
	
	if(cb)
		cb->Enable();
	else
		maLogAPI(L"LittleArrowEnable no widget", windowid, arrow);
}


void LittleArrowDisable(int windowid, int arrow)
{
#ifdef MATRACEGUIWX
	maTrace("LittleArrowDisable");
#endif
	wxSpinButton *cb;
	cb = (wxSpinButton *)((wxWindow *)g_w[windowid])->FindWindowById(arrow, (wxWindow *)g_w[windowid]);
	
	if(cb)
		cb->Disable();
	else
		maLogAPI(L"LittleArrowDisable no widget", windowid, arrow);
}


/*
void StaticTextEnable(int windowid, int statictext)
{
#ifdef MATRACEGUIWX
	maTrace("StaticTextEnable");
#endif
	wxStaticText *cb;
	cb = (wxStaticText *)((wxWindow *)g_w[windowid])->FindWindowById(statictext, (wxWindow *)g_w[windowid]);
	
	if(cb)
		cb->Enable();
	else
		maLogAPI(L"StaticTextEnable no widget", windowid, statictext);
}


void StaticTextDisable(int windowid, int statictext)
{
#ifdef MATRACEGUIWX
	maTrace("StaticTextDisable");
#endif
	wxStaticText *cb;
	cb = (wxStaticText *)((wxWindow *)g_w[windowid])->FindWindowById(statictext, (wxWindow *)g_w[windowid]);
	
	if(cb)
		cb->Disable();
	else
		maLogAPI(L"StaticTextDisable no widget", windowid, statictext);
}
*/

void StaticTextSetText(int windowid, int statictext, machar *text)
{
#ifdef MATRACEGUIWX
	maTrace("StaticTextSetText");
#endif
	wxStaticText *cb;
	cb = (wxStaticText *)((wxWindow *)g_w[windowid])->FindWindowById(statictext, (wxWindow *)g_w[windowid]);
	
	if(cb)
	{
		cb->SetLabel(text);
		cb->Refresh();
	}
	else
		maLogAPI(L"StaticTextSetText no widget", windowid, statictext);
}


void StaticTextSetValue(int windowid, int statictext, int value)
{
#ifdef MATRACEGUIWX
	maTrace("StaticTextSetValue");
#endif
	wxStaticText *cb;
	cb = (wxStaticText *)((wxWindow *)g_w[windowid])->FindWindowById(statictext, (wxWindow *)g_w[windowid]);
	
	if(cb)
	{
		machar lstr[STRLEN64];
		masprintf(lstr, STRLEN64, L"%d", value);
		cb->SetLabel(lstr);
		cb->Refresh();
	}
	else
		maLogAPI(L"StaticTextSetValue no widget", windowid, statictext);
}


void StaticTextRefresh(int windowid, int statictext)
{
#ifdef MATRACEGUIWX
	maTrace("StaticTextRefresh");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(statictext, (wxWindow *)g_w[windowid]);
	
	if(cb)
		((wxStaticText *)cb)->Refresh();
	else
		maLogAPI(L"StaticTextRefresh no widget", windowid, statictext);
}


void EditTextEnable(int windowid, int edittext)
{
#ifdef MATRACEGUIWX
	maTrace("EditTextEnable");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(edittext, (wxWindow *)g_w[windowid]);
	
	if(cb)
		cb->Enable(TRUE);
	else
		maLogAPI(L"EditTextEnable no widget", windowid, edittext);

} // EditTextEnable


void EditTextDisable(int windowid, int edittext)
{
#ifdef MATRACEGUIWX
	maTrace("EditTextDisable");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(edittext, (wxWindow *)g_w[windowid]);
	
	if(cb)
		cb->Enable(FALSE);
	else
		maLogAPI(L"EditTextDisable no widget", windowid, edittext);

} // EditTextDisable


void EditTextSetText(int windowid, int edittext, machar *text)
{
#ifdef MATRACEGUIWX
	maTrace("EditTextSetText");
#endif
	wxTextCtrl *cb;
	
#if !defined(NOTEXTVIEWER)
	if(edittext == ID_EDITTEXT_VIEWERVIEW)
		cb = ((MyFrameViewer *)g_w[windowid])->get_viewerTextCtrl();
	else
#endif
		cb = (wxTextCtrl *)((wxWindow *)g_w[windowid])->FindWindowById(edittext, (wxWindow *)g_w[windowid]);

	if(cb)
	{
		cb->SetValue(text);
		return;
	}
		
	maLogAPI(L"EditTextSetText no widget", windowid, edittext);

} // EditTextSetText


void EditTextGetText(int windowid, int edittext, machar *outBuffer, int inBufferLength, long *outTextLength, long from, long to)
{
#ifdef MATRACEGUIWX
	maTrace("EditTextGetText");
#endif
	long lto;
	wxTextCtrl *cb;
	wxString content;
	
	if(!outTextLength)
	{
		maLogAPI(L"EditTextGetText null outTextLength", windowid, edittext);
		return;
	}

	*outTextLength = 0;
	
	if(windowid < WINDOWFIRST || windowid > WINDOWLAST || !g_w[windowid])
		return;

#if !defined(NOTEXTVIEWER)
	if(edittext == ID_EDITTEXT_VIEWERVIEW)
		cb = ((MyFrameViewer *)g_w[windowid])->get_viewerTextCtrl();
	else
#endif
		cb = (wxTextCtrl *)((wxWindow *)g_w[windowid])->FindWindowById(edittext, (wxWindow *)g_w[windowid]);
	
	if(!cb)
	{
		maLogAPI(L"EditTextGetText no widget", windowid, edittext);
		return;
	}
		
	switch(edittext)
	{
		case ID_EDITTEXT_EQ:
		case ID_EDITTEXT_PROMPTNUM:
#if !defined(NOTEXTVIEWER)
		case ID_EDITTEXT_VIEWERFIND:
		case ID_EDITTEXT_VIEWERVIEW:
#endif
				if(!outBuffer)
				{
					maLogAPI(L"EditTextGetText null outBuffer", windowid, edittext);
					return;
				}
				if((lto = cb->GetLastPosition()) <= 0)
					return; // empty
				if(to >= 0 && to <= lto)
					lto = to;
				if(from >= lto)
				{
					maLogAPI(L"EditTextGetText invalid 'from' param", windowid, edittext);
					return;
				}
				content = cb->GetRange(from, lto);				
				if((*outTextLength = lto - from) >= inBufferLength)
					*outTextLength = inBufferLength - 1;
				mastrncpy(outBuffer, content.wc_str(), *outTextLength);
				outBuffer[*outTextLength] = L'\0';
		break;
		default:
			maLogAPI(L"EditTextGetText no widget", windowid, edittext);		
			return;
		break;
		
	} // switch(edittext)	
	
} // EditTextGetText


#if !defined(NOTEXTVIEWER)
 void EditTextGetSelection(int windowid, int edittext, long *start, long *end)
 {
 #ifdef MATRACEGUIWX
	maTrace("EditTextGetSelection");
 #endif
	wxTextCtrl *cb;
	
	if(edittext == ID_EDITTEXT_VIEWERVIEW)
		cb = ((MyFrameViewer *)g_w[windowid])->get_viewerTextCtrl();
	else
		cb = (wxTextCtrl *)((wxWindow *)g_w[windowid])->FindWindowById(edittext, (wxWindow *)g_w[windowid]);
	
	if(cb)
	{
		cb->GetSelection(start, end);
		return;
	}
		
	*start = *end = 0;
	maLogAPI(L"EditTextGetSelection no widget", windowid, edittext);

 } // EditTextGetSelection


 void EditTextSetSelection(int windowid, int edittext, long start, long end)
 {		
 #ifdef MATRACEGUIWX
	maTrace("EditTextSetSelection");
 #endif
	wxTextCtrl *cb;
	if(edittext == ID_EDITTEXT_VIEWERVIEW)
		cb = (((MyFrameViewer *)g_w[windowid])->get_viewerTextCtrl());
	else
		cb = (wxTextCtrl *)((wxWindow *)g_w[windowid])->FindWindowById(edittext, (wxWindow *)g_w[windowid]);
	
	if(cb)
	{
		cb->SetInsertionPoint(end);
		cb->SetSelection(start, end);
		return;
	}
	
	maLogAPI(L"EditTextSetSelection no widget", windowid, edittext);

 } // EditTextSetSelection


 void EditTextSetEditable(int windowid, int edittext, int editable)
 {
 #ifdef MATRACEGUIWX
	maTrace("EditTextSetEditable");
 #endif
	wxTextCtrl *cb;
	maviewer *v;

	if((v = Viewer(windowid)) == 0)
	{
		maLog(L"EditTextSetEditable Viewer failed");
		return;
	}

	if(edittext == ID_EDITTEXT_VIEWERVIEW)
		cb = (((MyFrameViewer *)g_w[windowid])->get_viewerTextCtrl());
	else
		cb = (wxTextCtrl *)((wxWindow *)g_w[windowid])->FindWindowById(edittext, (wxWindow *)g_w[windowid]);

	if(cb)
	{
		cb->SetEditable((editable == 0) ? false : true);
		return;
	}
		
	maLogAPI(L"EditTextSetEditable no widget", windowid, edittext);

 } // EditTextSetEditable


 int EditTextHasFocus(int windowid, int edittext)
 {
	wxTextCtrl *cb;
	
	if(edittext == ID_EDITTEXT_VIEWERVIEW)
		cb = (((MyFrameViewer *)g_w[windowid])->get_viewerTextCtrl());
	else
		cb = (wxTextCtrl *)((wxWindow *)g_w[windowid])->FindWindowById(edittext, (wxWindow *)g_w[windowid]);
	
	if(cb)
		return (cb->FindFocus() == cb) ? 1 : 0;
		
	maLogAPI(L"EditTextFindFocus no widget", windowid, edittext);

	return 0;
	
 } // EditTextFindFocus


 void EditTextSetFocus(int windowid, int edittext)
 {
	wxTextCtrl *cb;
	
	if(windowid < WINDOWFIRST || windowid > WINDOWLAST || !g_w[windowid])
		return;

	if(edittext == ID_EDITTEXT_VIEWERVIEW)
		cb = (((MyFrameViewer *)g_w[windowid])->get_viewerTextCtrl());
	else
		cb = (wxTextCtrl *)((wxWindow *)g_w[windowid])->FindWindowById(edittext, (wxWindow *)g_w[windowid]);
	
	if(cb)
	{
		cb->SetFocus();
		return;
	}
		
	maLogAPI(L"EditTextSetFocus no widget", windowid, edittext);

 } // EditTextSetFocus


 /* int EditTextLoadFile(int windowid, int edittext, fschar *fpath) // this puts the text in the TextCtrl
 {
 #ifdef MATRACEGUIWX
	maTrace("EditTextLoadFile");
 #endif

	return -1; // not implemented

 } // EditTextLoadFile


 int EditTextSaveFile(int windowid, int edittext, fschar *fpath)
 {
 #ifdef MATRACEGUIWX
	maTrace("EditTexSaveFile");
 #endif

	return -1; // not implemented

 } // EditTexSaveFile
 */
 

 int EditTextGetSize(int windowid, int edittext, size_t *pfsize)
 {
	MyViewerTextCtrl *t; 

	if(edittext == ID_EDITTEXT_VIEWERVIEW)
		if(pfsize)
			if((t = (MyViewerTextCtrl *)(((MyFrameViewer *)g_w[windowid])->get_viewerTextCtrl())) != 0)
			{
				*pfsize = t->GetValue().Len();
#if defined(__MAWIN__)
				*pfsize += t->GetNumberOfLines(); // because we count crlf's as 2, not 1 
#endif
				return 0; // ok
			}

	maLogAPI(L"EditTextGetSize failed", windowid, edittext);

	return 1; // failed
		
 } // EditTextGetSize


 void EditTextShowPosition(int windowid, int edittext, long pos) // text EnsureVisible
 {
	wxTextCtrl *cb;
	MyViewerTextCtrl *t;

	if(windowid < WINDOWFIRST || windowid > WINDOWLAST || !g_w[windowid])
		return;

	if(edittext == ID_EDITTEXT_VIEWERVIEW)
	{
		if((t = (MyViewerTextCtrl *)(((MyFrameViewer *)g_w[windowid])->get_viewerTextCtrl())) != 0)
		{
  #if defined(__WXMAC__) && MAC_OS_X_VERSION_MAX_ALLOWED <= 1040 // MAC_OS_X_VERSION_10_4
			// fails when run on wxMac/10.4, ok when run on 10.6, don't know about 10.5
			if(pos >= 0)
				t->SetInsertionPoint(pos);
			else // scroll to bottom pos < 0
			{
				t->SetInsertionPointEnd(); // end
				t->SetSelection(t->GetLastPosition(), t->GetLastPosition());
				// t->ShowPosition(t->GetLastPosition()); // doesn't help
  #else
			if(pos >= 0)
				t->SetInsertionPoint(pos);
			else
			{
				t->SetInsertionPointEnd(); // end
				t->SetSelection(t->GetLastPosition(), t->GetLastPosition());
  #endif
			}
			return;
		}
	}
	else // never gets here
	{
		cb = (wxTextCtrl *)((wxWindow *)g_w[windowid])->FindWindowById(edittext, (wxWindow *)g_w[windowid]);
		if(cb)
		{
			if(pos >= 0)
				cb->SetInsertionPoint(pos);
			else// scroll to bottom pos < 0
			{
				cb->SetInsertionPointEnd(); // end
				cb->SetSelection(cb->GetLastPosition(), cb->GetLastPosition());
			}
			return;
		}
	}
	maLogAPI(L"EditTextShowPosition no text widget", windowid, edittext);

 } // EditTextShowPosition
#endif


void ListAppend(int windowid, int list, machar *text)
{
#ifdef MATRACEGUIWX
	maTrace("ListAppend");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(list, (wxWindow *)g_w[windowid]);
	
	if(cb)
	{
		long i = ((wxListCtrl *)cb)->GetItemCount();
		((wxListCtrl *)cb)->InsertItem(i, text, 0);
	}
	else
		maLogAPI(L"ListAppend no widget", windowid, list);

} // ListAppend


void ListClear(int windowid, int list)
{
#ifdef MATRACEGUIWX
	maTrace("ListClear");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(list, (wxWindow *)g_w[windowid]);
	
	if(cb)
	{
		int w = ((wxListCtrl *)cb)->GetColumnWidth(0); // we have a one-column list
		if(((wxListCtrl *)cb)->GetSelectedItemCount() > 0)
			((wxListCtrl *)cb)->EnsureVisible(0); // scroll to top of list (mac needs this)
		((wxListCtrl *)cb)->ClearAll();
		((wxListCtrl *)cb)->InsertColumn(0, wxEmptyString, wxLIST_FORMAT_LEFT, w);
	}
	else
		maLogAPI(L"ListClear no widget", windowid, list);
}


int ListGetRowSelected(int windowid, int list)
{
#ifdef MATRACEGUIWX
	maTrace("ListGetRowSelected");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(list, (wxWindow *)g_w[windowid]);
	
	if(cb)
	{
		if(((wxListCtrl *)cb)->GetSelectedItemCount() <= 0)
			return -1; // none selected
#if (MAGUIWX >= 8) // wx-2.8+
		long r = -1;
		if((r = ((wxListCtrl *)cb)->GetNextItem(r, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) 
			return (int)r;
#else
		for(int r = 0; r < ((wxListCtrl *)cb)->GetItemCount(); r++)
			if((((wxListCtrl *)cb)->GetItemState(r, wxLIST_STATE_SELECTED)) & wxLIST_STATE_SELECTED)
				return r;
#endif
		maLogAPI(L"ListGetRowSelected not found", windowid, list);
	}
	else
		maLogAPI(L"ListGetRowSelected no widget", windowid, list);

	return -1;
}


void ListEnsureVisible(int windowid, int list, int row)
{
#ifdef MATRACEGUIWX
	maTrace("ListEnsureVisible");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(list, (wxWindow *)g_w[windowid]);
	
	if(cb)
	{
		if(row + 1 > ((wxListCtrl *)cb)->GetItemCount())
			return;
		((wxListCtrl *)cb)->EnsureVisible(row);
	}
	else
		maLogAPI(L"ListEnsureVisible no widget", windowid, list);
}


void ListSelect(int windowid, int list, int row)
{
#ifdef MATRACEGUIWX
	maTrace("ListSelect");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(list, (wxWindow *)g_w[windowid]);
	
	if(cb)
	{
		if(row + 1 > ((wxListCtrl *)cb)->GetItemCount())
			return;
		g_listselect++;
		((wxListCtrl *)cb)->SetItemState(row, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
	}
	else
		maLogAPI(L"ListSelect no widget", windowid, list);
}


void ListGetText(int windowid, int list, int row, machar *text)
{
#ifdef MATRACEGUIWX
	maTrace("ListGetText");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(list, (wxWindow *)g_w[windowid]);
	
	if(cb)
	{
		if(row >= 0 && row < ((wxListCtrl *)cb)->GetItemCount())
		{
			wxListItem info;
			info.m_itemId = row;
			info.m_col = 0; // we use column 0
			info.m_mask = wxLIST_MASK_TEXT;
			if(((wxListCtrl *)cb)->GetItem(info))
			{
				mastrncpy(text, info.m_text.c_str(), LISTLINELEN);
				text[LISTLINELEN] = L'\0';
				return;
			}
		}
		maLogAPI(L"ListGetText failed to get text", windowid, list);
	}
	else
		maLogAPI(L"ListGetText no widget", windowid, list);
}


/* not used void ListSetText(int windowid, int list, int row, machar *text)
{
#ifdef MATRACEGUIWX
	maTrace("ListSetText");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(list, (wxWindow *)g_w[windowid]);
	
	if(cb)
	{
		if(row >= 0 && row < ((wxListCtrl *)cb)->GetItemCount())
		{
			wxListItem info;
			info.m_itemId = row;
			info.m_col = 0; // we use column 0
			info.m_mask = wxLIST_MASK_TEXT;
			info.m_text = text;
			if(((wxListCtrl *)cb)->SetItem(info))
				return;
		}
		maLogAPI(L"ListSetText failed to get text", windowid, list);
	}
	else
		maLogAPI(L"ListSetText no widget", windowid, list);
}
*/


void ListSetHeader(int windowid, int list, int col, machar *header)
{
#ifdef MATRACEGUIWX
	maTrace("ListSetHeader");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(list, (wxWindow *)g_w[windowid]);
	
	if(cb)
	{
		wxListItem itemCol;
		itemCol.m_mask = wxLIST_MASK_TEXT | wxLIST_MASK_IMAGE;
		itemCol.m_text = header;
		itemCol.m_image = -1;
		if(((wxListCtrl *)cb)->SetColumn(col, itemCol))
			return;

		maLogAPI(L"ListSetHeader failed to set header", windowid, list);
	}
	else
		maLogAPI(L"ListSetHeader no widget", windowid, list);
}


void ListSetColumnWidth()
{
#ifdef MATRACEGUIWX
	maTrace("ListSetColumnWidth");
#endif
	if(g_w[WINDOWLIST])
		((MyFrameList *)g_w[WINDOWLIST])->SetColumn0Width(0);
}


void ListFreeze()
{
#ifdef MATRACEGUIWX
	maTrace("ListFreeze");
#endif
}


void ListThaw()
{
#ifdef MATRACEGUIWX
	maTrace("ListThaw");
#endif
}


void ListResize()
{
#ifdef MATRACEGUIWX
	maTrace("ListResize");
#endif

	if(((MyFrameList *)g_w[WINDOWLIST])->m_list)
	{
		wxSize size = ((MyFrameList *)g_w[WINDOWLIST])->GetClientSize();
		((MyFrameList *)g_w[WINDOWLIST])->m_list->SetSize(0, MENUBARH + MABUTTONH + 4, size.x, size.y - MENUBARH - MABUTTONH - 4);
		((MyFrameList *)g_w[WINDOWLIST])->m_list->SetColumnWidth(0, size.x - SCROLLERWIDTH);
	}

} // ListResize


void MenuSetChecked(int menu, int item, int value)
{
#ifdef MATRACEGUIWX
	maTrace("MenuSetChecked");
#endif
	((wxMenu *)l_theMenu[menu])->Check(item, (value == 0) ? false : true);
}


/* int MenuIsChecked(int menu, int item)
{
#ifdef MATRACEGUIWX
	maTrace("MenuIsChecked");
#endif
	return ((wxMenu *)l_theMenu[menu])->IsChecked(item);
}
*/


void MenuEnable(int menu, int item)
{
#ifdef MATRACEGUIWX
	maTrace("MenuEnable");
#endif
	((wxMenu *)l_theMenu[menu])->Enable(item, true);
}


void MenuDisable(int menu, int item)
{
#ifdef MATRACEGUIWX
	maTrace("MenuDisable");
#endif
	((wxMenu *)l_theMenu[menu])->Enable(item, false);
}


void MenuSetLabel(int item, machar *label)
{
#ifdef MATRACEGUIWX
	maTrace("MenuSetLabel");
#endif
	if(g_menubar)
		((wxMenuBar *)g_menubar)->SetLabel(item, label);
}


void MenuGetLabel(int item, machar *label)
{
#ifdef MATRACEGUIWX
	maTrace("MenuGetLabel");
#endif
	if(g_menubar)
		mastrcpy(label, ((wxMenuBar *)g_menubar)->GetLabel(item).c_str());
	else
		*label = 0;
}


void UpdateSubMenusRecent()
{
#ifdef MATRACEGUIWX
	maTrace("UpdateSubMenusRecent");
#endif
	if(g_w[WINDOWAPP])
		((MyFrameApp *)g_w[WINDOWAPP])->MakeSubMenusRecent();
}


void CheckBoxSetFocus(int windowid, int checkbox)
{
#ifdef MATRACEGUIWX
	maTrace("CheckBoxSetFocus");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(checkbox, (wxWindow *)g_w[windowid]);
	if(cb)
		((MACHECKBOX *)cb)->SetFocus();
	else
		maLogAPI(L"CheckBoxSetFocus no widget", windowid, checkbox);
}


void CheckBoxSetValue(int windowid, int checkbox, int value)
{
#ifdef MATRACEGUIWX
	maTrace("CheckBoxSetValue");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(checkbox, (wxWindow *)g_w[windowid]);
	if(cb)
		((MACHECKBOX *)cb)->SetValue((value == 0) ? false : true);
	else
		maLogAPI(L"CheckBoxSetValue no widget", windowid, checkbox);
}


int CheckBoxGetValue(int windowid, int checkbox)
{
#ifdef MATRACEGUIWX
	maTrace("CheckBoxGetValue");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(checkbox, (wxWindow *)g_w[windowid]);
	
	if(cb)
		return ((MACHECKBOX *)cb)->IsChecked();

	maLogAPI(L"CheckBoxGetValue no widget", windowid, checkbox);

	return 0;
}


void CheckBoxSetDescriptor(int windowid, int checkbox, machar *desc)
{
#ifdef MATRACEGUIWX
	maTrace("CheckBoxSetDescriptor");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(checkbox, (wxWindow *)g_w[windowid]);
	
	if(cb)
		((MACHECKBOX *)cb)->SetLabel(desc);
	else
		maLogAPI(L"CheckBoxSetDescriptor no widget", windowid, checkbox);
}


/* void CheckBoxToggle(int windowid, int checkbox)
{
#ifdef MATRACEGUIWX
	maTrace("CheckBoxToggle");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(checkbox, (wxWindow *)g_w[windowid]);
	
	if(cb)
		((MACHECKBOX *)cb)->SetValue(((MACHECKBOX *)cb)->GetValue() ? FALSE : TRUE);
	else
		maLogAPI(L"CheckBoxToggle no widget", windowid, checkbox);
}
*/


void CheckBoxHide(int windowid, int checkbox)
{
#ifdef MATRACEGUIWX
	maTrace("CheckBoxHide");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(checkbox, (wxWindow *)g_w[windowid]);
	
	if(cb)
	{
		((MACHECKBOX *)cb)->Hide();
#if defined(MAUSEMACHECKBOX)
		((MACHECKBOX *)cb)->LabelHide();
#endif
	}
	else
		maLogAPI(L"CheckBoxHide no widget", windowid, checkbox);
}


void CheckBoxShow(int windowid, int checkbox)
{
#ifdef MATRACEGUIWX
	maTrace("CheckBoxShow");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(checkbox, (wxWindow *)g_w[windowid]);
	
	if(cb)
	{
		((MACHECKBOX *)cb)->Show();
#if defined(MAUSEMACHECKBOX)
		((MACHECKBOX *)cb)->LabelShow();
#endif
	}
	else
		maLogAPI(L"CheckBoxShow no widget", windowid, checkbox);
}


void CheckBoxEnable(int windowid, int checkbox)
{
#ifdef MATRACEGUIWX
	maTrace("CheckBoxEnable");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(checkbox, (wxWindow *)g_w[windowid]);
	
	if(cb)
		((MACHECKBOX *)cb)->Enable(TRUE);
	else
		maLogAPI(L"CheckBoxEnable no widget", windowid, checkbox);
}


void CheckBoxDisable(int windowid, int checkbox)
{
#ifdef MATRACEGUIWX
	maTrace("CheckBoxDisable");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(checkbox, (wxWindow *)g_w[windowid]);
	
	if(cb)
		((MACHECKBOX *)cb)->Enable(FALSE);
	else
		maLogAPI(L"CheckBoxDisable no widget", windowid, checkbox);
}


void maLogAPI(const machar *errtxt, int windowid, int item)
{
	machar mastr[MAXLOGLINELEN];
	// masprintf(mastr, MAXLOGLINELEN, L"%ls window=%d=x%x item=%d", errtxt, window,  (int)g_w[windowid], item); // 32-bit
	// masprintf(mastr, MAXLOGLINELEN, L"%ls window=%d=x%x item=%d", errtxt, window, (long)g_w[windowid], item); // 64-bit
	masprintf(mastr, MAXLOGLINELEN, L"%ls window=%d item=%d", errtxt, windowid, item);
	maLog(mastr);
}


void DisplayGetSize(int *x, int *y)
{
#ifdef MATRACEGUIWX
	maTrace("DisplayGetSize");
#endif
	if(!x || !y)
		maLog(L"DisplayGetSize null x or y parameter");

	wxSize d = wxGetDisplaySize();
	if(x)
		*x = d.x;
	if(y)
		*y = d.y;
}


void WindowClose(int windowid)
{
#ifdef MATRACEGUIWX
	maTrace("WindowClose");
#endif
	if(windowid < WINDOWFIRST || windowid > WINDOWLAST || !g_w[windowid])
		return;

	((wxFrame *)g_w[windowid])->Destroy(); // Close(); 
	
} // WindowClose


void WindowDestroy(int windowid)
{
#ifdef MATRACEGUIWX
	maTrace("WindowDestroy");
#endif
	if(windowid < WINDOWFIRST || windowid > WINDOWLAST || !g_w[windowid])
		return;

	((wxFrame *)g_w[windowid])->Destroy();
}


int WindowIsShown(int windowid)
{
#ifdef MATRACEGUIWX
	maTrace("WindowIsShown");
#endif
	if(windowid < WINDOWFIRST || windowid > WINDOWLAST || !g_w[windowid])
		return 0;
		
	return ((wxFrame *)g_w[windowid])->IsShown();
}


/*
void WindowWidgetSetSize(int windowid, int widget, int xsize, int ysize)
{
#ifdef MATRACEGUIWX
	maTrace("WindowWidgetSetSize");
#endif
}
*/


void WindowSetSize(int windowid, int xsize, int ysize)
{
#ifdef MATRACEGUIWX
	maTrace("WindowSetSize");
#endif
	if(windowid < WINDOWFIRST || windowid > WINDOWLAST || !g_w[windowid])
		return;
	((wxFrame *)g_w[windowid])->SetSize(-1, -1, xsize, ysize, wxSIZE_USE_EXISTING);
}


void WindowMove(int windowid, int xpos, int ypos)
{
#ifdef MATRACEGUIWX
	maTrace("WindowMove");
#endif
	if(windowid < WINDOWFIRST || windowid > WINDOWLAST || !g_w[windowid])
	{
		maLog(L"WindowMove no window");
		return;
	}
	((wxWindow *)g_w[windowid])->Move(xpos, ypos);	
}


void WindowShow(int windowid)
{
#ifdef MATRACEGUIWX
	maTrace("WindowShow");
#endif
	if(windowid < WINDOWFIRST || windowid > WINDOWLAST || !g_w[windowid])
	{
		maLog(L"WindowShow no window");
		return;
	}
	((wxFrame *)g_w[windowid])->Show();
	((wxFrame *)g_w[windowid])->Raise();
}


void WindowSetTitle(int windowid, machar *title)
{
#ifdef MATRACEGUIWX
	machar mastr[MAXLOGLINELEN];
	masprintf(mastr, MAXLOGLINELEN, L"WindowSetTitle %d '%ls'", windowid, title);
	maLog(mastr);
#endif

	if(windowid < WINDOWFIRST || windowid > WINDOWLAST || !g_w[windowid])
	{
		maLog(L"WindowSetTitle no window");
		return;
	}

	((wxFrame *)g_w[windowid])->SetTitle(title);

} // WindowSetTitle


void WindowGetCoordinates(int windowid, int *xpos, int *ypos, int *xsize, int *ysize)
{
#ifdef MATRACEGUIWX
	maTrace("WindowGetCoordinates");
#endif
	if(windowid < WINDOWFIRST || windowid > WINDOWLAST || !g_w[windowid])
	{
		maLog(L"WindowGetCoordinates no window");
		return;
	}
	
	if(!xpos || !ypos || !xsize || !ysize)
	{
		maLog(L"WindowGetCoordinates bad pos or size parameter");
		return;
	}
	
	((wxWindow *)g_w[windowid])->GetSize(xsize, ysize);
	((wxWindow *)g_w[windowid])->GetPosition(xpos, ypos);
	
} // WindowGetCoordinates


void WindowItemHide(int windowid, int item)
{
#ifdef MATRACEGUIWX
	maTrace("WindowItemHide");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(item, (wxWindow *)g_w[windowid]);
	
	if(cb)
		cb->Hide();
	else
		maLogAPI(L"WindowItemHide no widget", windowid, item);
}


void WindowItemShow(int windowid, int item)
{
#ifdef MATRACEGUIWX
	maTrace("WindowItemShow");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(item, (wxWindow *)g_w[windowid]);
	
	if(cb)
		cb->Show();
	else
		maLogAPI(L"WindowItemShow no widget", windowid, item);
}


/*
void SplashWindowMake()
{
#ifdef MATRACEGUIWX
	maTrace("SplashWindowMake");
#endif
	wxPoint pos(g_sets.mawin[WINDOWSPLASH].xpos[g_sets.winconf],  g_sets.mawin[WINDOWSPLASH].ypos[g_sets.winconf]); 
	wxSize  siz(g_sets.mawin[WINDOWSPLASH].xsize[g_sets.winconf], g_sets.mawin[WINDOWSPLASH].ysize[g_sets.winconf]);
	if((g_w[WINDOWSPLASH] = (void *) new MyFrameSplash(WINDOWSPLASH, "MoreAmp starting", pos, siz)) != NULL)
			((MyFrameSplash *)g_w[WINDOWSPLASH])->Show();
	else
		maLog(L"SplashWindowMake failed to create Splash window");
}
*/


void WindowActivate(int windowid)
{
#ifdef MATRACEGUIWX
	maTrace("WindowItemShow");
#endif

	WindowShow(windowid);
	
} // WindowActivate


void ListWindowMake()
{
#ifdef MATRACEGUIWX
	maTrace("ListWindowMake");
#endif
	machar mastr[MAXLOGLINELEN];

	if(!g_w[WINDOWAPP])
	{
		maLog(L"ListWindowMake no App window");
		return;
	}
	
	if(!g_w[WINDOWLIST])
	{		
		wxPoint pos(g_sets.mawin[WINDOWLIST].xpos[g_sets.winconf],  g_sets.mawin[WINDOWLIST].ypos[g_sets.winconf]); 
		wxSize  siz(g_sets.mawin[WINDOWLIST].xsize[g_sets.winconf], g_sets.mawin[WINDOWLIST].ysize[g_sets.winconf]);
		masprintf(mastr, MAXLOGLINELEN, L"%ls List", g_prognametitle);
		MyFrameList *listframe = new MyFrameList(WINDOWLIST, mastr, pos, siz);

		if(listframe)
		{
			g_w[WINDOWLIST] = listframe;

#if !defined(__WXX11__)
 #if defined(__WXMSW__)
            listframe->SetIcon(wxICON(APP_ICON));
 #else
            listframe->SetIcon(wxICON(MoreAmpIcon));
 #endif
#endif
			if(g_w[WINDOWAPP] && WindowIsShown(WINDOWAPP))
				listframe->Show();
		}
		else
			maLog(L"ListWindowMake failed");
	}
	else
		maLog(L"ListWindowMake already made");
		
} // ListWindowMake


void EqualizerWindowMake()
{
#ifdef MATRACEGUIWX
	maTrace("EqualizerWindowMake");
#endif
	machar mastr[MAXLOGLINELEN];
	if(!g_w[WINDOWAPP])
	{
		maLog(L"EqualizerWindowMake no App window");
		return;
	}
	
	if(!g_w[WINDOWEQ])
	{		
		wxPoint pos(g_sets.mawin[WINDOWEQ].xpos[g_sets.winconf],  g_sets.mawin[WINDOWEQ].ypos[g_sets.winconf]); 
		wxSize  siz(g_sets.mawin[WINDOWEQ].xsize[g_sets.winconf], g_sets.mawin[WINDOWEQ].ysize[g_sets.winconf]);
		if(g_sets.eqflags & MASEPARATELR)
			masprintf(mastr, MAXLOGLINELEN, L"%ls Equalizer Left Channel", g_prognametitle);
		else
			masprintf(mastr, MAXLOGLINELEN, L"%ls Equalizer", g_prognametitle);
		MyFrameEq *eqframe = new MyFrameEq( WINDOWEQ, mastr, pos, siz);

		if(eqframe)
		{
			g_w[WINDOWEQ] = eqframe;

#if !defined(__WXX11__)
 #if defined(__WXMSW__)
            eqframe->SetIcon(wxICON(APP_ICON));
 #else
            eqframe->SetIcon(wxICON(MoreAmpIcon));
 #endif
#endif
			if(g_w[WINDOWAPP] && WindowIsShown(WINDOWAPP))
				eqframe->Show();
		}
		else
			maLog(L"EqualizerWindowMake failed");
	}
	else
		maLog(L"EqualizerWindowMake already made");

} // EqualizerWindowMake


void Equalizer2WindowMake()
{
#ifdef MATRACEGUIWX
	maTrace("Equalizer2WindowMake");
#endif
	machar mastr[MAXLOGLINELEN];
	if(!g_w[WINDOWAPP])
	{
		maLog(L"Equalizer2WindowMake no App window");
		return;
	}
	
	if(!g_w[WINDOWEQ2])
	{		
		wxPoint pos(g_sets.mawin[WINDOWEQ2].xpos[g_sets.winconf],  g_sets.mawin[WINDOWEQ2].ypos[g_sets.winconf]); 
		wxSize  siz(g_sets.mawin[WINDOWEQ2].xsize[g_sets.winconf], g_sets.mawin[WINDOWEQ2].ysize[g_sets.winconf]);
		masprintf(mastr, MAXLOGLINELEN, L"%ls Equalizer Right Channel", g_prognametitle);
		MyFrameEq2 *eqframe = new MyFrameEq2( WINDOWEQ2, mastr, pos, siz);

		if(eqframe)
		{
			g_w[WINDOWEQ2] = eqframe;

#if !defined(__WXX11__)
 #if defined(__WXMSW__)
            eqframe->SetIcon(wxICON(APP_ICON));
 #else
            eqframe->SetIcon(wxICON(MoreAmpIcon));
 #endif
#endif
			if(g_w[WINDOWAPP] && WindowIsShown(WINDOWAPP))
				eqframe->Show();
		}
		else
			maLog(L"Equalizer2WindowMake failed");
	}
	else
		maLog(L"Equalizer2WindowMake already made");

} // Equalizer2WindowMake


void EffectsWindowMake()
{
#ifdef MATRACEGUIWX
	maTrace("EffectsWindowMake");
#endif
	machar mastr[MAXLOGLINELEN];
	
	if(!g_w[WINDOWAPP])
	{
		maLog(L"EffectsWindowMake no App window");
		return;
	}
	
	if(!g_w[WINDOWEFFECTS])
	{		
		wxPoint pos(g_sets.mawin[WINDOWEFFECTS].xpos[g_sets.winconf],  g_sets.mawin[WINDOWEFFECTS].ypos[g_sets.winconf]); 
		wxSize  siz(g_sets.mawin[WINDOWEFFECTS].xsize[g_sets.winconf], g_sets.mawin[WINDOWEFFECTS].ysize[g_sets.winconf]);
		masprintf(mastr, MAXLOGLINELEN, L"%ls Effects", g_prognametitle);
		MyFrameEffects *effectsframe = new MyFrameEffects( WINDOWEFFECTS, mastr, pos, siz);

		if(effectsframe)
		{
			g_w[WINDOWEFFECTS] = effectsframe;

#if !defined(__WXX11__)
 #if defined(__WXMSW__)
            effectsframe->SetIcon(wxICON(APP_ICON));
 #else
            effectsframe->SetIcon(wxICON(MoreAmpIcon));
 #endif
#endif
			if(g_w[WINDOWAPP] && WindowIsShown(WINDOWAPP))
				effectsframe->Show();
		}
		else
			maLog(L"EffectsWindowMake failed");
	}
	else
		maLog(L"EffectsWindowMake already made");

} // EffectsWindowMake


#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
void AudioCDWindowMake()
{
#ifdef MATRACEGUIWX
	maTrace("AudioCDWindowMake");
#endif
	machar mastr[MAXLOGLINELEN];
	
	if(!g_w[WINDOWAPP])
	{
		maLog(L"AudioCDWindowMake no App window");
		return;
	}
	
	if(!g_w[WINDOWAUDIOCD])
	{		
		wxPoint pos(g_sets.mawin[WINDOWAUDIOCD].xpos[g_sets.winconf], g_sets.mawin[WINDOWAUDIOCD].ypos[g_sets.winconf]);
		wxSize  siz(g_sets.mawin[WINDOWAUDIOCD].xsize[g_sets.winconf], g_sets.mawin[WINDOWAUDIOCD].ysize[g_sets.winconf]);
		masprintf(mastr, MAXLOGLINELEN, L"%ls AudioCD", g_prognametitle);
		MyFrameAudioCD *audiocdframe = new MyFrameAudioCD( WINDOWAUDIOCD, mastr, pos, siz);

		if(audiocdframe)
		{
			g_w[WINDOWAUDIOCD] = audiocdframe;

#if !defined(__WXX11__)
 #if defined(__WXMSW__)
            audiocdframe->SetIcon(wxICON(APP_ICON));
 #else
            audiocdframe->SetIcon(wxICON(MoreAmpIcon));
 #endif
#endif
			if(g_w[WINDOWAPP] && WindowIsShown(WINDOWAPP))
				audiocdframe->Show();
		}
		else
			maLog(L"AudioCDWindowMake failed");
	}
	else
		maLog(L"AudioCDWindowMake already made");

} // AudioCDWindowMake
#endif


void AdvancedWindowMake()
{
#ifdef MATRACEGUIWX
	maTrace("AdvancedWindowMake");
#endif
	machar mastr[MAXLOGLINELEN];
	
	if(!g_w[WINDOWAPP])
	{
		maLog(L"AdvancedWindowMake no App window");
		return;
	}
	
	if(!g_w[WINDOWADVANCED])
	{		
		wxPoint pos(g_sets.mawin[WINDOWADVANCED].xpos[g_sets.winconf], g_sets.mawin[WINDOWADVANCED].ypos[g_sets.winconf]); 
		wxSize  siz(g_sets.mawin[WINDOWADVANCED].xsize[g_sets.winconf], g_sets.mawin[WINDOWADVANCED].ysize[g_sets.winconf]);
		masprintf(mastr, MAXLOGLINELEN, L"%ls Advanced Settings", g_prognametitle);
		MyFrameAdvanced *advancedframe = new MyFrameAdvanced( WINDOWADVANCED, mastr, pos, siz);

		if(advancedframe)
		{
			g_w[WINDOWADVANCED] = advancedframe;

#if !defined(__WXX11__)
 #if defined(__WXMSW__)
            advancedframe->SetIcon(wxICON(APP_ICON));
 #else
            advancedframe->SetIcon(wxICON(MoreAmpIcon));
 #endif
#endif
			if(g_w[WINDOWAPP] && WindowIsShown(WINDOWAPP))
				advancedframe->Show();
		}
		else
			maLog(L"AdvancedWindowMake failed");
	}
	else
		maLog(L"AdvancedWindowMake already made");

} // AdvancedWindowMake


void FileOutWindowMake()
{
#ifdef MATRACEGUIWX
	maTrace("FileOutWindowMake");
#endif
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
#endif
	machar mastr[MAXLOGLINELEN];
	
	if(!g_w[WINDOWAPP])
	{
		maLog(L"FileOutWindowMake no App window");
		return;
	}
	
	if(!g_w[WINDOWFILEOUT])
	{		
		wxPoint pos(g_sets.mawin[WINDOWFILEOUT].xpos[g_sets.winconf],
					g_sets.mawin[WINDOWFILEOUT].ypos[g_sets.winconf]); 
		wxSize siz(g_sets.mawin[WINDOWFILEOUT].xsize[g_sets.winconf],
					g_sets.mawin[WINDOWFILEOUT].ysize[g_sets.winconf]);
		masprintf(mastr, MAXLOGLINELEN, L"%ls %ls File Output", g_prognametitle,
													fs2ma(mastr2, g_extstrout[g_sets.output]));
		MyFrameFileOut *fileoutframe = new MyFrameFileOut( WINDOWFILEOUT, mastr, pos, siz);

		if(fileoutframe)
		{
			g_w[WINDOWFILEOUT] = fileoutframe;

#if !defined(__WXX11__)
 #if defined(__WXMSW__)
            fileoutframe->SetIcon(wxICON(APP_ICON));
 #else
            fileoutframe->SetIcon(wxICON(MoreAmpIcon));
 #endif
#endif
			if(g_w[WINDOWAPP] && WindowIsShown(WINDOWAPP))
				fileoutframe->Show();
		}
		else
			maLog(L"FileOutWindowMake failed");
	}
	else
		maLog(L"FileOutWindowMake already made");
	
} // FileOutWindowMake


void PromptNumWindowMake()
{
#ifdef MATRACEGUIWX
	maTrace("PromptNumWindowMake");
#endif
	machar mastr[MAXLOGLINELEN];

	if(!g_w[WINDOWAPP])
	{
		maLog(L"PromptNumWindowMake no App window");
		return;
	}
	
	if(!g_w[WINDOWPROMPTNUM])
	{		
		masprintf(mastr, MAXLOGLINELEN, L"%ls Song Index Prompt", g_prognametitle);
		MyFramePromptNum *promptnumframe = new MyFramePromptNum( WINDOWPROMPTNUM, mastr,
			wxPoint(g_sets.mawin[WINDOWPROMPTNUM].xpos[g_sets.winconf], g_sets.mawin[WINDOWPROMPTNUM].ypos[g_sets.winconf]),
			wxSize(g_sets.mawin[WINDOWPROMPTNUM].xsize[g_sets.winconf], g_sets.mawin[WINDOWPROMPTNUM].ysize[g_sets.winconf]));
		if(promptnumframe)
		{
			g_w[WINDOWPROMPTNUM] = promptnumframe;

#if !defined(__WXX11__)
 #if defined(__WXMSW__)
            promptnumframe->SetIcon(wxICON(APP_ICON));
 #else
            promptnumframe->SetIcon(wxICON(MoreAmpIcon));
 #endif
#endif
			// set focus to edit text
			// EditTextSetFocus(WINDOWPROMPTNUM, ID_EDITTEXT_PROMPTNUM);
			promptnumframe->m_edittextpromptnum->SetFocus();

			if(g_w[WINDOWAPP] && WindowIsShown(WINDOWAPP))
				promptnumframe->Show();
		}
		else
			maLog(L"PromptNumWindowMake failed");
	}
	else
		maLog(L"PromptNumWindowMake already made");

} // PromptNumWindowMake


int HSliderGetValue(int windowid, int slider) // for horizontal sliders
{
#ifdef MATRACEGUIWX
	maTrace("HSliderGetValue");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(slider, (wxWindow *)g_w[windowid]);
	
	if(cb)
	{
		int ret = ((MASLIDER *)cb)->GetValue();
		((MASLIDER *)cb)->Refresh();
		return ret;
	}
	maLogAPI(L"HSliderGetValue no widget", windowid, slider);
	return 0;
}


void HSliderSetValue(int windowid, int slider, int value) // for horizontal sliders
{
#ifdef MATRACEGUIWX
	maTrace("HSliderSetValue");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(slider, (wxWindow *)g_w[windowid]);
	
	if(cb)
	{
		((MASLIDER *)cb)->SetValue(value);
		((MASLIDER *)cb)->Refresh();
	}
	else
		maLogAPI(L"HSliderSetValue no widget", windowid, slider);
}


int SliderGetValue(int windowid, int slider) // for vertical sliders
{
#ifdef MATRACEGUIWX
	maTrace("SliderGetValue");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(slider, (wxWindow *)g_w[windowid]);
	
	if(cb)
	{
#if defined(MAUSEMASLIDER)
		int ret = ((MASLIDER *)cb)->GetValue();
#else // upside down, min is always 0
		int ret = ((MASLIDER *)cb)->GetMax() - ((MASLIDER *)cb)->GetValue();
#endif
		((MASLIDER *)cb)->Refresh();
		return ret;
	}
	maLogAPI(L"SliderGetValue no widget", windowid, slider);
	return 0;
}


void SliderSetValue(int windowid, int slider, int value) // for vertical sliders
{
#ifdef MATRACEGUIWX
	maTrace("SliderSetValue");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(slider, (wxWindow *)g_w[windowid]);
	
	if(cb)
	{
#if defined(MAUSEMASLIDER)
		((MASLIDER *)cb)->SetValue(value);
#else // upside down, min is always 0
		((MASLIDER *)cb)->SetValue(((MASLIDER *)cb)->GetMax() - value);
#endif
		((MASLIDER *)cb)->Refresh();
	}
	else
		maLogAPI(L"SliderSetValue no widget", windowid, slider);
}


void SliderEnable(int windowid, int slider)
{
#ifdef MATRACEGUIWX
	maTrace("SliderEnable");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(slider, (wxWindow *)g_w[windowid]);
	
	if(cb)
	{
		((MASLIDER *)cb)->Enable();
		((MASLIDER *)cb)->Refresh();
	}
	else
		maLogAPI(L"SliderEnable no widget", windowid, slider);
}


void SliderDisable(int windowid, int slider)
{
#ifdef MATRACEGUIWX
	maTrace("SliderDisable");
#endif
	wxWindow *cb = ((wxWindow *)g_w[windowid])->FindWindowById(slider, (wxWindow *)g_w[windowid]);
	
	if(cb)
	{
		((MASLIDER *)cb)->Disable();
		((MASLIDER *)cb)->Refresh();
	}
	else
		maLogAPI(L"SliderDisable no widget", windowid, slider);
}


#if (MAGUIWX == 9) // && ! WXWIN_COMPATIBILITY_2_6
 #if !defined(wxOPEN)
  #define wxOPEN                wxFD_OPEN
 #endif
 #if !defined(wxSAVE)
  #define wxSAVE                wxFD_SAVE
 #endif
 #if !defined(wxOVERWRITE_PROMPT)
  #define wxOVERWRITE_PROMPT    wxFD_OVERWRITE_PROMPT
 #endif
 #if !defined(wxHIDE_READONLY)
  #define wxHIDE_READONLY       0x0008
 #endif
 #if !defined(wxFILE_MUST_EXIST)
  #define wxFILE_MUST_EXIST     wxFD_FILE_MUST_EXIST
 #endif
 #if !defined(wxMULTIPLE)
  #define wxMULTIPLE            wxFD_MULTIPLE
 #endif
 #if !defined(wxCHANGE_DIR)
  #define wxCHANGE_DIR          wxFD_CHANGE_DIR
 #endif
#endif
int maFileDialog(fschar *selectedpath, int windowid, machar *prompt, fschar *defdir, machar *deffile, machar *wildcard, int flags)
{
#ifdef MATRACEGUIWX
	maTrace("maFileDialog");
#endif
	int lflags = 0;
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
#endif
	machar mastr[MAXLOGLINELEN];
	
    if(flags & maOPEN)				lflags |= wxOPEN;
	if(flags & maSAVE)				lflags |= wxSAVE;
	if(flags & maOVERWRITE_PROMPT)  lflags |= wxOVERWRITE_PROMPT;
 #if defined(wxHIDE_READONLY) // iff WXWIN_COMPATIBILITY_2_4
	if(flags & maHIDE_READONLY)		lflags |= wxHIDE_READONLY;
 #endif
	if(flags & maFILE_MUST_EXIST)   lflags |= wxFILE_MUST_EXIST;
	if(flags & maCHANGE_DIR)		lflags |= wxCHANGE_DIR;

	wxFileDialog dialog((wxFrame *)g_w[windowid], prompt, fs2ma(mastr2, defdir), deffile, wildcard, lflags);
								
	if(dialog.ShowModal() != wxID_OK) // cancelled
		return 1;

#if defined(__MAWIN__)
	if(mastrlen(fs2ma(mastr2, (fschar *)dialog.GetPath().fn_str())) >= MAX_PATH)
#else
	if(mastrlen(fs2ma(mastr2, (fschar *)dialog.GetPath().fn_str().data())) >= MAX_PATH) // on mac this is UTF-8-MAC
#endif
	{
		masprintf(mastr, MAXLOGLINELEN, L"maFileDialog filepath > max %d : %zu", MAX_PATH - 1, 
#if defined(__MAWIN__)
			mastrlen(fs2ma(mastr2, (fschar *)dialog.GetPath().fn_str())));
#else
			mastrlen(fs2ma(mastr2, (fschar *)dialog.GetPath().fn_str().data()))); // on mac this is UTF-8-MAC
#endif
		maLog(mastr);
		

#if defined(__MAWIN__)
		mastrncpy(mastr, fs2ma(mastr2, (fschar *)dialog.GetPath().fn_str()), MAXLOGLINELEN - 1);
#else
		mastrncpy(mastr, fs2ma(mastr2, (fschar *)dialog.GetPath().fn_str().data()), MAXLOGLINELEN - 1); // on mac this is UTF-8-MAC
#endif
		mastr[MAXLOGLINELEN - 1] = L'\0';
		maLog(mastr);
		return 1;
	}

#if defined(__MAWIN__)
	fsstrcpy(selectedpath, (fschar *)dialog.GetPath().fn_str());
#else
	fsstrcpy(selectedpath, (fschar *)dialog.GetPath().fn_str().data()); // on mac this is UTF-8-MAC
#endif
	
	return 0;
	
} // maFileDialog


void maFileDialogMFree(maArray *parray)
{
#ifdef MATRACEGUIWX
	maTrace("maFileDialogMFree");
#endif

	int i;
	
	if(!parray)
		return;
	if(parray->pitems)
	{
		for(i = 0; i < parray->nitems; i++)
			if(parray->pitems[i])
				MAFREE(parray->pitems[i]);
		MAFREE(parray->pitems);
		parray->pitems = NULL;
	}
	parray->nitems = 0;
}


int maFileDialogM(maArray *parray, int windowid, machar *prompt, fschar *defdir, machar *deffile, machar *wildcard, int flags)
{
#ifdef MATRACEGUIWX
	maTrace("maFileDialogM");
#endif
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
#endif
	int lflags = wxMULTIPLE;
	
	if(flags & maOPEN)				lflags |= wxOPEN;
	if(flags & maSAVE)				lflags |= wxSAVE;
	if(flags & maOVERWRITE_PROMPT)  lflags |= wxOVERWRITE_PROMPT;
#if defined(wxHIDE_READONLY) // iff WXWIN_COMPATIBILITY_2_4
	if(flags & maHIDE_READONLY)		lflags |= wxHIDE_READONLY;
#endif
	if(flags & maFILE_MUST_EXIST)   lflags |= wxFILE_MUST_EXIST;
	if(flags & maCHANGE_DIR)		lflags |= wxCHANGE_DIR;
	
	parray->nitems = 0;
	parray->pitems = NULL;
	
	wxFileDialog dialog((wxFrame *)g_w[windowid], prompt, fs2ma(mastr2, defdir), deffile, wildcard, lflags);
								
	if(dialog.ShowModal() != wxID_OK) // cancelled
		return 1;
	
	wxArrayString *s;
	
	if((s = new wxArrayString()) == NULL)
	{
		maLog(L"maFileDialog new wxArrayString failed");
		return 1;
	}
	
	dialog.GetPaths(*s);
	s->Sort();
		
	size_t count = s->GetCount();
	
	if((parray->pitems = (fschar **)MAMALLOC((count + 1) * sizeof(fschar *))) == NULL)
	{
		delete s;
		maLog(L"maFileDialog MAMALLOC failed");
		return 1;
	}
	
	fschar **p = parray->pitems;
	size_t i;
	for(i = 0; i < count; i++, p++)
	{
		if((*p = (fschar *)MAMALLOC(MAX_PATH * 4)) != 0) // max sizeof(UTF-8) 
  #if defined(__MAWIN__)
			fsstrcpy(*p, (fschar *)(*s)[i].fn_str());
  #else
			fsstrcpy(*p, (fschar *)(*s)[i].fn_str().data()); // on mac this is UTF-8-MAC
  #endif
	}
		
	delete s;
	parray->nitems = (int)count;
	
	return 0;
	
} //maFileDialogM


int maDirSelector(fschar *dirpath, machar *prompt)
{
#ifdef MATRACEGUIWX
	maTrace("maDirSelector");
#endif
	wxString dir = wxDirSelector(prompt);   // wxDirSelector returns an alias' volume, not the alias...
											// osx does not return a cd as a dir (but osx debug version does)
	if(dir.empty()) // cancelled
		return 1;

#if defined(__MAWIN__)
	fsstrcpy(dirpath, (fschar *)dir.fn_str());
#else
	fsstrcpy(dirpath, (fschar *)dir.fn_str().data()); // on mac this is UTF-8-MAC
#endif
	
	return 0;
}


void guiAppYield() 
{
#ifdef MATRACEGUIWX
	maTrace("guiAppYield");
#endif

	g_theApp->Yield(1);

} // guiAppYield


int maMessageBox(machar *s1, machar *s2, int flags)
{
	int lflags = 0;
	machar title[STRLEN256];
	
	if(flags & maYES)				lflags |= wxYES;
	if(flags & maOK)				lflags |= wxOK;
	if(flags & maNO)				lflags |= wxNO;
	if(flags & maYES_NO)			lflags |= wxYES_NO;
	if(flags & maCANCEL)			lflags |= wxCANCEL;
	if(flags & maICON_EXCLAMATION)	lflags |= wxICON_EXCLAMATION;
	if(flags & maICON_HAND)			lflags |= wxICON_HAND;
	if(flags & maICON_QUESTION)		lflags |= wxICON_QUESTION;
	if(flags & maICON_INFORMATION)	lflags |= wxICON_INFORMATION;
	
	masprintf(title, STRLEN256, L"%ls : %ls", g_prognametitle, s2);

	if(wxMessageBox(s1, title, lflags) == wxYES)
		return maYES;
	
	return maNO;

} // maMessageBox


void DoTimerRunLoop(int delay)
{
#ifdef MATRACEGUIWX
	maTrace("DoTimerRunLoop");
#endif
	((MyFrameApp *)g_w[WINDOWAPP])->m_timerrunloop->Start(delay, TRUE);
}


void DoTimerPlay(int delay)
{
#ifdef MATRACEGUIWX
	maTrace("DoTimerPlay");
#endif
	((MyFrameApp *)g_w[WINDOWAPP])->m_timerplay->Start(delay, TRUE);
}


 /* void DoTimerStop(int delay)
 {
 #ifdef MATRACEGUIWX
	maTrace("DoTimerStop");
 #endif
	((MyFrameApp *)g_w[WINDOWAPP])->m_timerstop->Start(delay, TRUE);
 }
 */

void DoTimerChangePlaylist(int delay)
 {
 #ifdef MATRACEGUIWX
	maTrace("DoTimerChangePlaylist");
 #endif
	((MyFrameApp *)g_w[WINDOWAPP])->m_timerchangeplaylist->Start(delay, TRUE);
 }


 void DoTimerPlayPlaylist(int delay)
 {
 #ifdef MATRACEGUIWX
	maTrace("DoTimerPlayPlaylist");
 #endif
	((MyFrameApp *)g_w[WINDOWAPP])->m_timerplayplaylist->Start(delay, TRUE);
 }


#if !defined(NOTEXTVIEWER)
 int ViewerWindowMake(int windowid, machar *wintitle)
 {
	maviewer *v;

	switch(windowid)
	{
#if defined(MAKEYBOARD)
		case WINDOWKEYBINDINGS:
#endif
		case WINDOWCATALOG:
		case WINDOWPLAYLIST:
		case WINDOWREADME:
		case WINDOWLOG:
			g_w[windowid] = (void *) new MyFrameViewer(windowid, wintitle,
							wxPoint(g_sets.mawin[windowid].xpos[g_sets.winconf], g_sets.mawin[windowid].ypos[g_sets.winconf]),
							wxSize(g_sets.mawin[windowid].xsize[g_sets.winconf], g_sets.mawin[windowid].ysize[g_sets.winconf]));
		break;
		default:
			ShowErr(L"ViewerWindowMake bad windowid");
			return 1;
		break;
	}
	
	if(g_w[windowid] == NULL)
	{
		ShowErr(L"ViewerWindowMake failed to create window");
		return 1;
	}

	WindowShow(windowid);

	return ViewerInit(windowid, &v);
	
 } // ViewerWindowMake


#if defined(MATRACEGUIWX) || defined(MAMEMTRACE)
 void maTrace(machar *mastr2)
 {
	maLog(mastr2);
 }
 #endif
#endif


void AboutWindowMake()
{
#ifdef MATRACEGUIWX
	maTrace("AboutWindowMake");
#endif

	if(!g_w[WINDOWAPP])
	{
		maLog(L"AboutWindowMake no App window");
		return;
	}
	
	if(!g_w[WINDOWABOUT])
	{		
		if((g_w[WINDOWABOUT] = (void *) new MyFrameAbout( WINDOWABOUT, wxT("About MoreAmp"), 
							wxPoint(g_sets.mawin[WINDOWABOUT].xpos[g_sets.winconf], g_sets.mawin[WINDOWABOUT].ypos[g_sets.winconf]),
							wxSize(g_sets.mawin[WINDOWABOUT].xsize[g_sets.winconf], g_sets.mawin[WINDOWABOUT].ysize[g_sets.winconf]))) != NULL)
			((MyFrameAbout *)g_w[WINDOWABOUT])->Show();
		else
			maLog(L"AboutWindowMake failed to create About window");
	}
	else
		maLog(L"AboutWindowMake already made");

} // AboutWindowMake


void maGUIfree()
{
}


void maGUIQuit()
{
} // maGUIQuit


#if defined(__WXGTK__)
 extern "C" { void* gdk_window_get_pointer(void*, int*, int*, unsigned int*); }
 // #include <gdk/gdk.h> // for GDK_xxx_MASK
#endif
 int maModKeyDown() // the mouse modifier key or Shift or Alt key, depending on platform
 {
  #if defined(__WXMSW__)
	if(GetKeyState(VK_SHIFT) < 0)
		return 1;
 #endif

  #if defined(__WXGTK__)
	int x, y;
	unsigned int m;
	gdk_window_get_pointer(NULL, &x, &y, &m);
	//if(m & GDK_BUTTON3_MASK)	// GDK_BUTTON1_MASK = 1 << 8, GDK_BUTTON2_MASK = 1 << 9, GDK_BUTTON3_MASK = 1 << 10,
	if(m & (1 << 3) ||			// if(m & GDK_MOD1_MASK) // normally Alt key
	   m & (1 << 0))			// if(m & GDK_SHIFT_MASK)
	// if(m & (1 << 9))			// if(m & GDK_BUTTON2_MASK)
	// if(m & (1 << 10))		// if(m & GDK_BUTTON3_MASK)
		return 1;
  #endif
        
  #if defined(__WXMAC__) && defined(__MAOSX__) // some mice cannot have both right and left button down simultaneously...
	/*
	UInt32 bstate;
	if((bstate = GetCurrentEventButtonState()) & (1 << 1)) // secondary button
		return 1;
	if(bstate & (1 << 0)) // primary)
	*/
		if(GetCurrentEventKeyModifiers() & shiftKey)
			return 1;
	KeyMap Keys; 
	unsigned char *BytePtr = (unsigned char*)Keys; 
	GetKeys(Keys);
	if(BytePtr[7] & 1) // Shift 0x38 == 56 = (7 * 8)
		return 1;
  #endif 

	return 0;
	
 } // maModKeyDown

#ifdef __cplusplus
}
#endif

#endif // MAGUIWX

// eof
