// maPlayer.c

/*
 * MoreAmp
 * Copyright (C) 2004-2014 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include "maConfig.h" // included by all moreamp files

#ifdef __cplusplus
extern "C"
{
#endif

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#include <errno.h>
#include <locale.h>

#ifndef __MAWIN__
 #include <unistd.h>
#endif

#if defined(__MAOSX__)
 #include <sys/stat.h>
 #include <MacMemory.h>
#endif

#if defined(__MAOSX__) || defined(__MACUNIX__)
 #include <AEDataModel.h>
 #include <AppleEvents.h>
 #include <CarbonEvents.h>
 #include <Power.h>
 #include <MacTypes.h>
 #include <pthread.h>
#endif

#ifdef __MAUNIX__
 #include <sys/stat.h>
 #include <pthread.h>
#endif

#ifdef __MAWIN__
 #include <wtypes.h>
 #include <io.h>
 #include <limits.h> /* for INT_MAX */
#endif

#if defined(MAGUINONE) && defined(__MACUNIX__) // osxnogui
#elif defined(MAGUICARBON) // MoreAmpN
#elif !(defined(MAGUINONE) && defined(__MAOSX__)) && (defined(__MAUNIX__) || defined(__MACUNIX__) || defined(__WXGTK__))
 // unix, mac gtk2 with keyboard, mac wxgtk2 with keyboard
 #include <gdk/gdkkeys.h>
#endif

// Application includes

#include "maTypes.h"
#include "maDefs.h"
#include "maGUI.h"
#include "maMemFile.h"
#include "maEqualizer.h"
#include "maPlayer.h"
#include "maAudioPA.h"
#include "maPlayMp3.h"
#include "maPlayOgg.h"
#include "maPlayWma.h"
#include "maPlayAac.h"
#include "maPlayMp4.h"
#include "maPlayFlac.h"
#include "maPlayAif.h"
#include "maEffects.h"
#include "maSongList.h"
#include "maUtils.h"
#include "maTextView.h"

#ifndef MANOCDA
 #include "maCda.h"
 #include "maPlayCda.h"
#endif

#ifdef ENCODEMP3
 #include "maWriteMp3.h"
#endif

#ifdef ENCODEOGG
 #include "maWriteOgg.h"
#endif

#ifdef ENCODEFLAC
 #include "maWriteFlac.h"
#endif

#ifdef ENCODEAAC
 #include "maWriteAac.h"
#endif

#ifdef ENCODEMP4
 #include "maWriteMp4.h"
#endif

#ifdef ENCODEWMA
 #include "maWriteWma.h"
#endif

#ifdef ENCODEWAV
 #include "maWriteWav.h"
#endif

#ifdef ENCODERAW
 #include "maWriteWav.h"
#endif

#ifdef ENCODEAIF
 #include "maWriteAif.h"
#endif

//////////////////////////////////////////////////////////////////////////////////////////////
// defs used in this module only
#define INDEXLEN 6		// length of index field in playlist (incl space) e.g. "00123 MacIntosh HD:mp3:..."
#define POSINCR 100		// increment value in ms for beg and end position little arrows
#define POSDECR POSINCR	// decrement value in ms for beg and end position little arrows
#define MINPOSDELTA 100 // ms minimum g_posend - g_posbeg

// 'seeking' here means using Repeat-Begin-End
#define TITLEPERIOD 200	// ms to wait in RunLoop when not seeking
#define TITLETIME 5		// iterations through RunLoop to refresh display info
#define SEEKPERIOD 50	// ms to wait in RunLoop when seeking
#define SEEKTIME 2		// iterations through RunLoop to refresh display info while seeking

#define FXBUFFERSIZE (MAFIFOBUFSIZE * 4)

// values for l_outputfilter
#define OUTPUT_STEREO	  1
#define OUTPUT_MONO		  2
#define OUTPUT_LEFT		  4
#define OUTPUT_RIGHT	  8
#define OUTPUT_DIFF		 16
#define OUTPUT_REVERSE	512
// values for g_sets.monodiff
#define MONOCHECKBOXL	1
#define MONOCHECKBOXR	2
#define MONOCHECKBOXSUB	4
#define MONOCHECKBOXREV	8

// multiple instances
#define MAINSTANCEMAX 32 
typedef struct // shared among all instances, in memory or on disk
{
	int pid[MAINSTANCEMAX];
	int multipleinst;
} MAINSTANCE;
#define MAINSTANCEFILE L"maInstances.mai"
#if defined(MAUSESHF)
 #define MAKEYFILE l_mainstancefile
#endif
#if defined(MAUSESHM)
 #if defined(__MAWIN__)
  #define MAKEYFILE l_mainstancefile // memory-mapped file
 #else
  #define MAKEYFILE fsT("/tmp") // for ftok
 #endif
#endif

#if defined(__MAUNIX__)
 fschar l_mareadmedir[MAX_PATH];
 #define MAREADMEDIR l_mareadmedir
#else
 #define MAREADMEDIR g_madir
#endif

// l_runloop values
enum
{
    maSTOPPING	= 0,
    maSTARTING	= 1,
    maSCANNING	= 2,
    maLOADING	= 3,
    maPLAYING	= 4,
};
int l_runloop = maSTOPPING;

int l_appwindowmoving = 0;
int l_soundsingleplay = 0;


fschar l_lastsongdir[MAX_PATH];

machar l_keypressed[STRLEN16];

#define MASETTINGSVERSIONISCURRENT 0
#define MASETTINGSVERSIONISPREVIOUS 1
#define MASETTINGSVERSIONISUNKNOWN 2

//////////////////////////////////////////////////////////////////////////////////////////////
// g_... globals also used in other modules
machar g_prognametitle[STRLEN64];
int g_effectschange;
#if defined(__MAWIN__) || defined(__MAUNIX__)
 #define dskFulErr -34
#endif
mauint64 g_posbeg;
mauint64 g_posend = MINPOSDELTA;
mauint64 l_posbegbu = MAUINT64MAX;
mauint64 l_posendbu = MAUINT64MAX;
int g_posbegon;
int g_posendon;
int g_reptsong;
int g_scanstop;
int g_scanfiles;
int g_listbusy;
size_t g_loadedmemsize;
int g_listselect;
int g_songsnum;
void *g_w[NUMWINDOWS];
void *g_menubar;
char *g_memhandle;

fschar g_madir[MAX_PATH];
fschar g_logpath[MAX_PATH];
fschar g_playlistsdir[MAX_PATH];
fschar g_settingsdir[MAX_PATH];
fschar g_recordsdir[MAX_PATH];
fschar g_fileoutdir[MAX_PATH];

int g_loadednum;
fschar g_loadedname[SONGSLOADMAX][MAX_PATH];
fschar g_loadedpath[SONGSLOADMAX][MAX_PATH];
int g_loadedcurrent;

fschar g_settingspath[MAX_PATH];
fschar g_recordspath[MAX_PATH];
#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
 MASETSCDROM g_setscdrom;
#endif

#if defined(MAKEYBOARD)
 fschar g_keybindingspath[MAX_PATH];
#endif

fschar g_settingsdefpath[MAX_PATH];
fschar g_settingsdrop[MAX_PATH];
fschar g_playlistpath[MAX_PATH];
fschar g_catalogpath[MAX_PATH];
fschar g_records[MAX_PATH];
fschar g_playlistdrop[MAX_PATH];
fschar g_playlistcda[MAX_PATH];
#if !defined(NODECODEAUDIOIN)
 fschar g_playlistaudioin[MAX_PATH];
 fschar g_songfileaudioin[MAX_PATH];
#endif
fschar g_playlisttemp[MAX_PATH];
fschar g_readmepath[MAX_PATH];
#if defined(MAKEYBOARD)
#endif
machar *g_playlist0;
MASETS g_sets;
MASETS g_setsbu;

int g_argc;
#if defined(MAGUIWX) || defined(__MAWIN__)
 machar **g_argv; // wchar_t
#else
 char **g_argv;
#endif

char *g_fxoutbuffer = NULL;
char *g_songmemptr[SONGSLOADMAX + 1];
char *g_mafifobuf = NULL;

int g_thisinstance;

#if defined(MAKEYBOARD)
 int g_maOnKeySkip = 0;
#endif

void *g_pmashm = NULL;

FILE *g_tempfilefp;
fschar g_tempfile[MAX_PATH];

machar g_maeol[] = MAEOL; // MAWIN 0x0d0a crlf, else 0x0a lf
fschar g_fseol[] = FSEOL; // MAWIN 0x0d0a crlf, else 0x0a lf

int g_cataloguing;

// input file extensions (order matches playable files enum in Defs.h)
fschar *g_extstr[] =
{
 EXTMP1,
 EXTMP2,
 EXTMP3,
 EXTOGG,
 EXTWMA,
 EXTAIF,
 EXTAIFF,
 EXTWAV,
 EXTRAW,

 /* these could be added to maPlayAif
 EXTAU,		// Sun/NeXT AU format (big endian)
 EXTPAF,	// Ensoniq PARIS file format
 EXTSVX,	// Amiga IFF / SVX8 / SV16 format
 EXTNIST,	// Sphere NIST format
 EXTVOC,	// VOC fil
 EXTIRCAM,	// Berkeley/IRCAM/CARL
 EXTW64,	// Sonic Foundry's 64 bit RIFF/WAV
 EXTMAT4,	// Matlab (tm) V4.2 / GNU Octave 2.0
 EXTMAT5,	// Matlab (tm) V5.0 / GNU Octave 2.1
 EXTPVF,	// Portable Voice Format
 EXTXI,		// Fasttracker 2 Extended Instrument
 EXTHTK,	// HMM Tool Kit format
 */
 EXTAAC,
 EXTM2A,
 EXTM4A,
 EXTMP4,
 EXTFLA,
 EXTFLAC,
 EXTMAA,
 EXTMAF,
 EXTMAS,
 EXTMAR,
 EXTM3U,
 EXTCDA // keep this last
};

// output file extensions (order matches outputs enum in Defs.h)
fschar *g_extstrout[] =
{
 MASTRSOUNDfs,
 
#ifdef ENCODEMP3
 EXTMP3,
#endif

#ifdef ENCODEOGG
 EXTOGG,
#endif

#ifdef ENCODEFLAC
 EXTFLAC,
#endif

#ifdef ENCODEAAC
 EXTAAC,
#endif

#ifdef ENCODEMP4
 EXTMP4,
#endif

#ifdef ENCODEWMA
 EXTWMA,
#endif

#ifdef ENCODEWAV
 EXTWAV,
#endif

#ifdef ENCODEAIF
 EXTAIF,
#endif

#ifdef ENCODERAW
 EXTRAW
#endif
};

MASTATE g_state;

int g_k[kOUTMAX];

int g_droplaunch;
int g_loadedquit;
int g_loadedquitB;
 
 //////////////////////////////////////////////////////////////////////////////////////////////
// functions used only in this module 
int	RunLoad();
int FillSongBuffer(int dest);
int AllocSongRec(MARECORD **ppsongrec, int songsnum);
int ZeroRecords(MARECORD **ppsongrec, int songsnum);
int FileToMem(fschar *from, char *tomem, int maxmem, long *bytesread);
int FileToVDisk(fschar *from, fschar *tomem, int maxmem, long *bytesread);
void PlayerSeek(mauint64 pos);
void SetAppTitle2();
void WaitDecoderStopped();
int RunPlay();
int GetInFiletype(fschar *playpath);
int LaunchPlayThread(fschar *playpath);
int EqDoParamChange();
int GetSongMem();
int GetRecentPlaylistName();
int SetSettingsPaths();
int SetRecordsPaths();
void FileOutWindowDisable();
void FileOutWindowEnable();
void DisableButtonsMenus();
void DisableButtonsMenusByFileType(int filetype);
void ReEnableButtonsMenusByFileType();
void EnableButtonsMenus();
void SetLogMenus();
void SetAppWindowMsg(machar *msg);
void ListWindowSetTitle(machar *msg);
void Init_g_k();
void CheckFileoutSettings();
int CheckSettings(MASETS *psets);
int LoadSettings();
int LoadRecords();
#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
 int LoadFactorySettingsCDRom();
 int SaveSettingsCDRom();
#endif
int LoadFactorySettings(MASETS *psets);
int LoadFactorySettingsAudioIn(MASETS *psets);
int SaveSettingsAsDefault(MASETS *psets);
int CopySettings();
void SetPause(int pause);
void SetSliders(int slider);
void SetSliderCaption(int slider);
void UpdateListWindowButtons();
void SetGUIPopupButtons();

int ListLoaded();
void SetGUIMonoStereo();
void SetGUITitleInfo();
void SetPosSlider(int value);
mauint64 DumpSongInfo(fschar *songfpath, FILE *dumpfp, int nomem, machar **p2, machar **p3, size_t *len3);
int ListPlayerInfo(int newcd);
int ListPlaylist();
void ListWindowSetHeader(machar *header);
int ListWindowGetValue();
void ListWindowAddString(machar *str);
void ListWindowShowRow(int inrow);
void ListWindowClear();
void ListWindowGetRow(machar *str, int row);
void ListWindowShowRowPlaying();
int ListSongInfo();
int GetRecentPlaylist();
void ShowMessageValue(int value);
void OnMsgQuit();
void ApplyChannelSelection(char *soundbuf, unsigned long numFrames);
int MakeOutputFileName();
int maCloseOutputFile();
int maOpenOutputFile();
int maOutputToFile(void* pcmblock, unsigned long frames, unsigned long len);
int maAllocBuffers();
int maFreeBuffers();
int maEraseBuffers();

maFifo *maFifoCreate(maFifo* pfifo, char *storage, unsigned long fifosize);
void maFifoDelete(maFifo* pfifo);	
void maFifoSetStorage(maFifo* pfifo, char *storage);
void maFifoErase(maFifo* pfifo);
unsigned long maFifoGetFilledSpace(maFifo* pfifo);
unsigned long maFifoStore(maFifo* pfifo, char *whereFrom, unsigned long requested);
unsigned long maFifoDraw(maFifo* pfifo, char* whereTo, unsigned long requested);
// void maFifoClear(maFifo* pfifo);
// char *maFifoGetStorage(maFifo* pfifo);

maFifoB *maFifoCreateB(maFifoB* pfifo, char *storage, unsigned long fifosize);
void maFifoDeleteB(maFifoB* pfifo);	
void maFifoSetStorageB(maFifoB* pfifo, char *storage);
void maFifoEraseB(maFifoB* pfifo);
unsigned long maFifoGetFilledSpaceB(maFifoB* pfifo);
unsigned long maFifoStoreB(maFifoB* pfifo, char *whereFrom, unsigned long requested);
// unsigned long maFifoDrawB(maFifo* pfifo, char* whereTo, unsigned long requested);
void maFifoClearB(maFifoB* pfifo);
// char *maFifoGetStorageB(maFifoB* pfifo);

int maOutChain2(char *data, unsigned long len);
int maOutChain3(char *data, unsigned long len);
int maOutChain4(maFifoB* pfifo);
int maOutChain5(void* pcmblock, unsigned long len);

void AppQuit();
int SetVDisk();
#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
 int SetsAudioCD();
 int ClearAudioCD();
#endif
int GetNextSong(int *pm, MARECORD *psongrec);

int InstanceInit();
int InstanceClearThis();
int InstanceSet(int which, int value);
int InstanceWrite(MAINSTANCE *pinstance);
int InstanceSet(int which, int value);
int InstanceRead(MAINSTANCE *pinstance);
int SetMultipleInstances();

int GetVDisk();
void WindowsMoveDelta(int skipwindowid, int xdelta, int ydelta);
void WindowMoveAndSetSize(int windowid, int xpos, int ypos, int xsize, int ysize);
void WindowsMoveAndSetSize();
void WindowSetsFactoryValues(int winconf);
void SetFileoutType(int which);
void LoadFactoryFileout(int which);
#if defined(MAKEYBOARD)
 void UpdateEffectsKbLabels();
 int SaveKeyBindings(int blank);
 int VerifyKeyBindings(int showerr);
 void GetFreeKeyBindings(machar *p);
 void SetFactoryKeyBindings();
 void OnKeyAltSize();
 void OnKeySavedSize();
 void OnKeyDefaultSize();
 void OnKeyFactorySize();
 void OnKeyShowFileOut();
 void OnKeyEffectsWindow();

 void OnKeyCheckRandom(); 
 void OnKeyCheckReptPlayed();
 void OnKeyCheckLoopLoaded();
 void OnKeyCheckLoopList(); 
 void OnKeyCheckSinglePlay();
 void OnKeyCheckInputMonitor(); 
 void OnKeyCheckTitleInfo();
 void OnKeyCheckListWindow(); 
 void OnKeyCheckEqualizer();
 void OnKeyCheckEqWindow(); 
 void OnKeyCheckEq31Bands();
 void OnKeyCheckEqExtra();
 void OnKeyCheckEffects();
 void OnKeyCheckRepeat(); 
 void OnKeyCheckPitch();
 void OnKeyCheckTempo();
 void OnKeyCheckSpeed();
 /*
 void OnKeyCheckAltSize();
 void OnKeyCheckBalance();
 void OnKeyCheckMonoLeft();
 void OnKeyCheckMonoRight();
 void OnKeyCheckMonoDiff();
 void OnKeyCheckLimiter();
 void OnKeyCheckEqZoom();
 void OnKeyCheckEqLink();
 void OnKeyCheckInvertRight();
 void OnKeyCheckQualSpeed();
 void OnKeyCheckQualFilter();
 #if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
  void OnKeyCheckCdParanoia();
 #endif
 void OnKeyCheckBoxBeg();
 void OnKeyCheckBoxEnd();
 void OnKeyCheckBoxM4A();
 void OnKeyCheckShowIndex();
 void OnKeyCheckShowDir();
 void OnKeyCheckShowPath();
 */
 void ShowKeyStatus(machar *status);
 void OnKeyButtonQuit();
 void OnKeyButtonStop();
 void OnKeyButtonPause();
 void OnKeyButtonPlay();
 void OnKeyButtonEqRestore();
 void OnKeyButtonEqInvert();
 void OnKeyPopupEqPreset1();
 void OnKeyPopupEqPreset2();
 void OnKeyPopupEqPreset3();
 void OnKeyPopupEqPreset4();
 void OnKeyResetSliderPitch();
 void OnKeyResetSliderTempo();
 void OnKeyResetSliderSpeed();
 void OnKeyPitchUp();
 void OnKeyPitch2Up();
 void OnKeyTempoUp();
 void OnKeySpeedUp();
 void OnKeyPitchDown();
 void OnKeyPitch2Down();
 void OnKeyTempoDown();
 void OnKeySpeedDown();
 void OnKeyOpenList();
 void OnKeyMakeList();
 void OnKeySaveListAs();
 void OnKeyMenuZero();
 void OnKeyPlayFile();
 void OnKeyPlayCD();
 void OnKeyEjectCD();
 void OnKeyViewAbout();
 void OnKeyViewCatalog();
 void OnKeyViewLog();
 void OnKeyEditBindings();
 void OnKeyViewReadme();
 void OnKeyClearLog();
 void OnKeyButtonInfo();
 void OnKeyButtonList();
 void OnKeyButtonLoaded();
 void OnKeyButtonSong();
 void OnKeyUpArrow();
 void OnKeyDownArrow();
 void OnKeyVolumeDown();
 void OnKeyVolumeUp();
 void OnKeyPlayIndex();
 void OnKeyPlayCatalog();
#endif
int maCheckSettingsVersion(char *settingsversion);
#if !defined(NODECODEAUDIOIN)
 void maPlayAudioIn();
#endif

//////////////////////////////////////////////////////////////////////////////////////////////
// l_... globals used only in this module
mauint64 l_posend;
mauint64 l_poscurrent;
int l_sliderposms;
int l_sliderposmsbu;
int l_quit;
int l_songsnumplayed;
int l_songcurrent = -1; // 0-based index of playlist song currently playing
int l_loadednew;
int l_previous;
int l_next;
int l_songsindex[SONGSLOADMAX];
int l_outputfilter = OUTPUT_STEREO;
int l_posslidervalue;
int l_possliderchanged;
int l_posslidersetvalue;
int l_mnextfile = -1;
int l_playerseeking = 0;
int l_pan = SEPARATIONMAX / 2;
int l_separation = SEPARATIONMAX;
fschar l_recentlist[MAX_PATH];
unsigned long l_endtime;
float l_separationch = 1.;
float l_separationmix = 0.;
float l_panleft = 0.5;
float l_panright = 0.5;
char *l_songmem;
char *l_songmemend;
char *l_songmempos;
char *l_fxoutdata = NULL;
unsigned long l_fxoutlen = 0;
machar l_strclicked[LISTLINELEN + 2];
int l_songmemsize;
fschar l_settingscda[MAX_PATH];
fschar l_settingsaudioin[MAX_PATH];
fschar l_settingspathlast[MAX_PATH];
maFifoB l_mafifoB;
maFifoB *l_pmafifoB;
MARECORD *l_psongrec = NULL;
char *l_monobuf;
unsigned long l_monobytes;
unsigned long l_logdelay;
// fschar l_cwd[MAX_PATH];
fschar l_bindir[MAX_PATH] = {fsT('\0')};
machar *l_allhaveplayedstr = L"All songs in the list have played" MAEOL MAEOL
								L"To play the list again, either :" MAEOL MAEOL
								L"- Zero List from the Tools menu, or" MAEOL MAEOL
								L"- Set the LoopList checkbox.";

machar *l_allhaveplayedcdastr = L"All songs in the list have played" MAEOL MAEOL
								L"To play the list again press Play." MAEOL MAEOL
								L"To avoid this message set the LoopList checkbox.";

EQUPRESET l_eqpresettemp;
int l_noshowrow;

#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
 fschar l_setscdrompath[MAX_PATH];
#endif

#define MALOGLOCK &l_maloglock
#define MAINITLOCK &l_mainitlock
#define MAINSTANCELOCK &l_mainstancelock
MALOCK l_maloglock = {fsT("maloglock"), NULL};
MALOCK l_mainitlock = {fsT("mainitlock"), NULL};
MALOCK l_mainstancelock = {fsT("mainstancelock"), NULL};
#if defined(MAUSESHF) || (defined(MAUSESHM) && defined(__MAWIN__))
 fschar l_mainstancefile[MAX_PATH];
#endif

#if defined(BINARYDISTRIB)
 int l_binbuild = 119;
#else
 int l_binbuild = 0;
#endif

int volminmaxL();
int volminmaxR();
void volumevalstrL(machar *str);
void volumevalstrR(machar *str);
#if !defined(MAGUINONE)
 int l_temptitle;
#endif
fschar l_playlistfs[MAX_PATH];
machar l_playlistma[MAX_PATH];
#if defined(MAKEYBOARD)
 int l_keyindex = 0;
 int l_keycatalog = 0;
#endif

fschar l_loadeddir[MAX_PATH]; // path to vdisk dir to hold loaded songs

FILE *l_fplogpath;	// log file
machar g_logcmdclear; // clear the log
machar l_logcmdexist; // test for existance of a log
int l_logoff;		// no logging flag
int l_logtotal;		// log event count per session
int l_logcount;		// log event count per period
int l_havelog;		// have a log file
int l_logupdate = 0;// reentry flag

int l_catalogcancel;
int l_listisfrozen = 0;

fschar l_outputfile[MAX_PATH];

int l_posbegonbu = 0;
int l_posendonbu = 0;

int l_noreload;
int l_singleplay;
int l_buffersize;

maenvchar *g_macharset = NULL;
maenvchar g_charset[STRLEN32]; // suitable as parameter to iconv_open()
#if defined(MAGUICARBON)
 maenvchar *g_macharsetkb = NULL;
 maenvchar g_charsetkb[STRLEN32]; // suitable as parameter to iconv_open()
#endif

#if defined(MASETLOCALE)
 int masetlocaledata()
 {
 #if defined(__MAWIN__) && defined(MAGUINONE)
	FILE *fp;
	maenvchar codepage[STRLEN64];
 #endif
 	
 #if defined(__MAWIN__)
  #if defined(MAGUINONE)
	maenvchar *locale = NULL;
	if((fp = mapopen(maenvT("chcp"), maenvT("r"))) != NULL)
	{
		while(mafgets(codepage, STRLEN64, fp) != NULL)
			locale = codepage;
		_pclose(fp);
		if(locale && (locale = mastrchr(codepage, ':'))) // "Active code page: 437" -> ". 437"
				*locale = '.';
		if(locale && (locale = masetlocale(LC_CTYPE, locale))) // current code page
			return 0; // ok
		ShowErr(L"masetlocaledata masetlocale chcp failed");
	}
	else
		ShowErr(L"masetlocaledata _popen chcp failed");
  #endif
 #else // not win
		if(masetlocale(LC_CTYPE, maenvT("en_US.UTF-8")))
			return 0; // ok
		ShowErr(L"masetlocaledata masetlocale failed");
 #endif

 	if(!masetlocale(LC_CTYPE, maenvT("")))  // current locale
		ShowErr(L"masetlocaledata current masetlocale failed");

	return 1; // failed

 } // masetlocaledata
#endif
//////////////////////////////////////////////////////////////////////////


int volminmaxL()
{
	if(g_sets.volumesliderleft >= VOLUMEMAX)
		return VOLUMEMAX;
	if(g_sets.volumesliderleft <= VOLUMEMIN)
		return VOLUMEMIN;
	return g_sets.volumesliderleft;

} // volminmaxL


int volminmaxR()
{
	if(g_sets.volumesliderright >= VOLUMEMAX)
		return VOLUMEMAX;
	if(g_sets.volumesliderright <= VOLUMEMIN)
		return VOLUMEMIN;
	return g_sets.volumesliderright;

} // volminmaxR


#define VOLSCALEDB 100.0	// if defined, volume sliders scale is in dB
//#define VOLSCALEAU 79.9	// if defined, volume sliders scale is log audio taper, attenuation at mid-scale = 90%
							// else volume sliders scale is linear

int volumevalL()
{
	if(g_sets.volumesliderleft >= VOLUMEMAX)
		return VOLUMEMAX;
	if(g_sets.volumesliderleft <= VOLUMEMIN)
		return VOLUMEMIN;
#if defined(VOLSCALEDB)
	return (int)(0.5 + (VOLUMEMAX * exp(((g_sets.volumesliderleft - VOLUMEMAX) / (((double)VOLUMEMAX / VOLSCALEDB) * 20.0)))));
#elif defined(VOLSCALEAU)
	return (int)(0.5 + (((double)VOLUMEMAX / VOLSCALEAU) * (pow(VOLSCALEAU + 1.0, (double)g_sets.volumesliderleft / VOLUMEMAX) - 1.0)));	
#else
	return g_sets.volumesliderleft;
#endif

} // volumevalL()


int volumevalR()
{
	if(g_sets.volumesliderright >= VOLUMEMAX)
		return VOLUMEMAX;
	if(g_sets.volumesliderright <= VOLUMEMIN)
		return VOLUMEMIN;
#if defined(VOLSCALEDB)
	return (int)(0.5 + (VOLUMEMAX * exp(((g_sets.volumesliderright - VOLUMEMAX) / (((double)VOLUMEMAX / VOLSCALEDB) * 20.0)))));
#elif defined(VOLSCALEAU)
	return (int)(0.5 + (((double)VOLUMEMAX / VOLSCALEAU) * (pow(VOLSCALEAU + 1.0, (double)g_sets.volumesliderright / VOLUMEMAX) - 1.0)));
#else
	return g_sets.volumesliderright;
#endif

} // volumevalR()


void volumevalstrL(machar *str)
{
#if defined(VOLSCALEAU)
	if(!g_sets.balance) // linear scale for left slider
	{
		masprintf(str, L"%d", volminmaxL()); // 0-1000
		return;
	}
#endif
	
#if defined(VOLSCALEDB) || defined(VOLSCALEAU)
	if(g_sets.volumesliderleft >= VOLUMEMAX)
	{
		mastrcpy(str,  L"max");
		return;
	}
	
	if(g_sets.volumesliderleft <= VOLUMEMIN)
	{
		mastrcpy(str, L"-inf");
		return;
	}
#endif
	
#if defined(VOLSCALEDB)
	masprintf(str, STRLEN64, L"%0.1f", (((double)g_sets.volumesliderleft - VOLUMEMAX) / (VOLUMEMAX / VOLSCALEDB)));
#elif defined(VOLSCALEAU)
	masprintf(str, STRLEN64, L"%0.1f", 20.0 * log((double)volumevalL() / VOLUMEMAX));
#else
	masprintf(str, STRLEN64, L"%d", g_sets.volumesliderleft);
#endif

} // volumevalstrL


void volumevalstrR(machar *str)
{
#if defined(VOLSCALEDB) || defined(VOLSCALEAU)
	if(g_sets.volumesliderright >= VOLUMEMAX)
	{
		mastrcpy(str, L"max");
		return;
	}
	
	if(g_sets.volumesliderright <= VOLUMEMIN)
	{
		mastrcpy(str, L"-inf");
		return;
	}
#endif
	
#if defined(VOLSCALEDB)
	masprintf(str, STRLEN64, L"%0.1f", (((double)g_sets.volumesliderright - VOLUMEMAX) / (VOLUMEMAX / VOLSCALEDB)));
#elif defined(VOLSCALEAU)
	masprintf(str, STRLEN64, L"%0.1f", 20.0 * log((double)volumevalR() / VOLUMEMAX));
#else
	masprintf(str, STRLEN64, L"%d", g_sets.volumesliderright);
#endif

} // volumevalstrR


#if !defined(MANOCDA)
 int maPlaylistMakeCda()
 {
	int i;
	int nitems;
	fschar *pitems[CDAMAXNUMDRIVES];

	nitems = 0;
	for(i = 1; i <= g_cdanumdrives; i++)
	{
		if(g_cdadrive[i].exists && g_cdadrive[i].isready)
			pitems[nitems++] = g_cdadrive[i].name;
	}
			
	if(nitems <= 0)
	{
		ShowErr(L"Insert an Audio CD or open another playlist");
		return 1;
	}
		
	// write a new cda playlist
	if(maPlaylistMake(nitems, pitems, NULL)) // maPlaylistMakeCda
	{
		maLog(L"maPlaylistMakeCda maPlaylistMake failed");
		return 1;
	}
	
	return 0;
	
 } // maPlaylistMakeCda
#endif


int OpenPlaylist(int skipsettings)
{
#if !defined(NOTEXTVIEWER)
	int catalog;
	int playlist;
#endif
	size_t ret0;
	machar *p4;
	machar *p5;
	fschar path[MAX_PATH] = {fsT('\0')};
	fschar songsource[MAX_PATH] = {fsT('\0')};
	machar masongsource[MAXLOGLINELEN];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
 	machar mastr[MAXLOGLINELEN];

#if !defined(MAGUINONE)
	int modkeydown = 0;
	if(g_w[WINDOWAPP] && maModKeyDown())
		modkeydown = 1;
#endif
	
#if !defined(NOTEXTVIEWER)
	if(g_w[WINDOWCATALOG])
	{
		catalog = 1;
		DoWindow(WINDOWCATALOG, 0);
	}
	else
		catalog = 0;
#endif
			
#if !defined(NOTEXTVIEWER)
	if(g_w[WINDOWPLAYLIST])
	{
		playlist = 1;
		DoWindow(WINDOWPLAYLIST, 0);
	}
	else
		playlist = 0;
#endif
			
	if(g_playlist0)
	{
		MAFREE(g_playlist0);
		g_playlist0 = NULL;
		ListWindowClear();
	}
	
	g_songsnum = 0;

	if(g_playlistpath[0] == fsT('\0'))
		return 1;					
	
	if(fsstrlen(g_playlistpath) >= MAX_PATH)
	{
		masprintf(mastr, MAXLOGLINELEN, L"OpenPlaylist g_playlistpath > max %d : %zu", MAX_PATH - 1, fsstrlen(g_playlistpath));
		maLog(mastr);
		fsstrncpy(path, g_playlistpath, MAX_PATH - 1);
		path[MAX_PATH - 1] = fsT('\0');
		maLog((machar *)path);
		g_settingspath[0] = fsT('\0');
		return 1;
	}
	
	if(SetSettingsPaths() || (g_settingspath[0] == fsT('\0')))
	{
		g_settingspath[0] = fsT('\0');
		g_recordspath[0] = fsT('\0');
		masprintf(mastr, MAXLOGLINELEN, L"OpenPlaylist SetSettingsPaths failed on '%ls'", fs2ma(mastr2, g_playlistpath));
		maLog(mastr);
		return 1;
	}
	else if(SetRecordsPaths() || (g_recordspath[0] == fsT('\0')))
	{
		g_settingspath[0] = fsT('\0');
		g_recordspath[0] = fsT('\0');
		masprintf(mastr, MAXLOGLINELEN, L"OpenPlaylist SetRecordsPaths failed on '%ls'", fs2ma(mastr2, g_recordspath));
		maLog(mastr);
		return 1;
	}
	
	// load the playlist file	
	ret0 = mafileread(&g_playlist0, g_playlistpath);
	if(ret0 <= 0 || g_playlist0 == NULL)
	{			
		if(g_playlist0 != NULL)
		{
			MAFREE(g_playlist0);
			g_playlist0 = NULL;
		}
		maLog(L"OpenPlaylist mafileread failed");
		return 1;
	}

	// count the songs
	p4 = g_playlist0;
	while((p5 = memgets(masongsource, MAX_PATH, p4)) != NULL)
	{
		ma2fsfn(songsource, masongsource);
		p4 = p5;
		// skip lines with '#' or files that are not playable or don't exist
		if(songsource[0] != fsT('#'))
		{
			if(SongFileExists(songsource))
				g_songsnum++;		
			else
			{
				masprintf(mastr, MAXLOGLINELEN, L"OpenPlaylist file not playable or not found: '%ls'",
																							fs2ma(mastr2, songsource));
				maLog(mastr);
#if !defined(NODECODEAUDIOIN)
#else
				if(FileNameHasExtension(songsource, EXTMAA))
					maLog(L"OpenPlaylist : audio input not implemented");
#endif
			}
		}
	}
			
	if(g_songsnum == 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"OpenPlaylist no playable songs in playlist '%ls'", fs2ma(mastr2, g_playlistpath));
		maLog(mastr);
		// continue for load settings
	}
			
	// settings :
#if !defined(MAGUINONE)
	if(!modkeydown)
#endif
		if(!skipsettings)
			if(LoadSettings())
			{
				masprintf(mastr, MAXLOGLINELEN, L"OpenPlaylist LoadSettings failed on '%ls'", fs2ma(mastr2, g_playlistpath));
				maLog(mastr);
				return 1;
			}

	// records :
	if(l_psongrec)
	{
		MAFREE(l_psongrec);
		l_psongrec = NULL;
	}
	l_songsnumplayed = 0;

	if(g_songsnum > 0)
	{
		if(PlaylistIsNoRecords())
		{
			if(AllocSongRec(&l_psongrec, g_songsnum))
			{
				masprintf(mastr, MAXLOGLINELEN, L"OpenPlaylist AllocSongRec failed on '%ls'", fs2ma(mastr2, g_playlistpath));
				maLog(mastr);
			}
		}
		else
		{
			if(LoadRecords()) // also allocs and zeroes
			{
				masprintf(mastr, MAXLOGLINELEN, L"OpenPlaylist LoadRecords failed on '%ls'", fs2ma(mastr2, g_playlistpath));
				maLog(mastr);
			}
		}
	}
	
#if !defined(NOTEXTVIEWER)
	if(catalog)
		DoViewCatalog(1);
	if(playlist)
		DoWindow(WINDOWPLAYLIST, 1);
#endif
	
	return 0;
	
} // OpenPlaylist


int ChangePlaylist(fschar *playlist)
{
	int ret;
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
#endif
	machar mastr[MAXLOGLINELEN];

#if !defined(MANOCDA)
	if(!fsstrcasecmp(g_playlistcda, playlist))
		if(CdaInitCDRom()) // ChangePlaylist
		{
			ShowErr(L"Insert an Audio CD or open another playlist");
			return 1;
		}
#endif

	if(g_state.running)
	{
		DoStop();
		fsstrcpy(g_changeplaylist, playlist);
		return 1;
	}

	fsstrcpy(g_playlistpath, playlist);
	g_changeplaylist[0] = fsT('\0');

	if(g_w[WINDOWAPP])
		SetAppWindowTitle(L"");

	// invalidate current vdisk contents
	g_loadednum = 0;

	// count songs and set g_settingspath and g_recordspath	
	if((ret = OpenPlaylist(0)) != 0 && ret != 3)  // 3 means empty playlist 
	{
		masprintf(mastr, MAXLOGLINELEN, L"ChangePlaylist OpenPlaylist failed on '%ls'", fs2ma(mastr2, g_playlistpath));
		maLog(mastr);
		return 1;
	}
	
	UpdateRecent(g_playlistpath, 0);
	
	if(g_w[WINDOWAPP])
	{
		UpdateListWindowList(0);		
		UpdateDisplay();
	}
					
	// start
	if(g_sets.output == kOUTSOUND)
	{
		if(g_sets.autostart)
			if(g_songsnum > 0)
				DoTimerPlay(100); // ChangePlaylist
	}

	return 0;
		
} // ChangePlaylist


int DeletePlaylist(fschar *playlist)
{
	fschar name[MAX_PATH];
	fschar file[MAX_PATH];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
#endif
	machar mastr[MAXLOGLINELEN];
	
	if(!fsstrcmp(g_playlistpath, playlist))
	{
#if defined(__MAOSX__) || defined(__MACUNIX__)
		strcpy(file, playlist);
		StrUTF8MACtoUTF8(file);
#else
		fsstrcpy(file, playlist);
#endif
		masprintf(mastr, MAXLOGLINELEN, L"This is the current playlist, it cannot be deleted while it is in use:%ls%ls%ls",
																g_maeol, g_maeol, fs2ma(mastr2, file));
		ShowErr(mastr);
		return 1;
	}
	
	if(!fsstrcmp(g_playlistdrop, playlist) || !fsstrcmp(g_playlistcda, playlist)
#if !defined(NODECODEAUDIOIN)
		|| !fsstrcmp(g_playlistaudioin, playlist) || !fsstrcmp(g_songfileaudioin, playlist)
#endif
		)
	{
		masprintf(mastr, MAXLOGLINELEN, L"This is an internal file, it cannot be deleted:%ls%ls%ls",
																g_maeol, g_maeol, fs2ma(mastr2, playlist));
		ShowErr(mastr);
		return 1;
	}

	if(FilenameFromPathname(playlist, name))
	{	
		masprintf(mastr, MAXLOGLINELEN, L"DeletePlaylist failed to get filename from '%ls'",
																					fs2ma(mastr2, playlist));
		maLog(mastr);
		return 1;
	}					
	FileNameRemoveExtension(name);

	// delete playlist
	if(maFileExists(playlist))
	{
		if(maunlink(playlist))
		{
			masprintf(mastr, MAXLOGLINELEN, L"DeletePlaylist maunlink errno=%d '%ls'", errno,
																						fs2ma(mastr2, playlist));
			maLog(mastr);
			return 1;
		}		
	}
	
	// delete settings file
	fssprintf(file, MAX_PATH, fsS fsC fsS fsS, g_settingsdir, FILESEP, name, EXTMAS);
	if(maFileExists(file))
	{
		if(maunlink(file))
		{
			masprintf(mastr, MAXLOGLINELEN, L"DeletePlaylist maunlink errno=%d '%ls'", errno, fs2ma(mastr2, file));
			maLog(mastr);
		}		
	}
	
	// delete records file
	fssprintf(file, MAX_PATH, fsS fsC fsS fsS, g_recordsdir, FILESEP, name, EXTMAR);
	if(maFileExists(file))
	{
		if(maunlink(file))
		{
			masprintf(mastr, MAXLOGLINELEN, L"DeletePlaylist maunlink errno=%d '%ls'", errno, fs2ma(mastr2, file));
			maLog(mastr);
		}		
	}
	
	// delete catalog
	fssprintf(file, MAX_PATH, fsS fsC fsS fsS, g_madir, FILESEP, name, EXTTXT);
	if(maFileExists(file))
	{
		if(maunlink(file))
		{
			masprintf(mastr, MAXLOGLINELEN, L"DeletePlaylist maunlink errno=%d '%ls'", errno, fs2ma(mastr2, file));
			maLog(mastr);
		}		
	}
	
	// update the recentplaylist file
	UpdateRecent(playlist, 1);
						
	return 0;
		
} // DeletePlaylist


int PlayPlaylist(fschar *playlist)
{
	fschar path[MAX_PATH];
	machar mastr[MAXLOGLINELEN];
	
	if(fsstrlen(playlist) >= MAX_PATH)
	{
		masprintf(mastr, MAXLOGLINELEN, L"PlayPlaylist file path > max %d : %zu", MAX_PATH - 1, fsstrlen(playlist));
		maLog(mastr);
		fsstrncpy(path, playlist, MAX_PATH - 1);
		path[MAX_PATH - 1] = fsT('\0');
		maLog((machar *)path);
		return 1;
	}
		
	if(!g_w[WINDOWAPP])
		return PlayPlaylistB(playlist);
	
	fsstrcpy(g_newplaylist, playlist);
	DoStop(); // DoTimerStop(1);
	WaitDecoderStopped();
	DoTimerPlayPlaylist(10);
			
	return 1;
	
} // PlayPlaylist
		

int PlayPlaylistB(fschar *playlist)
{
	int err;
	l_mnextfile = -1;
	g_droplaunch = 1;
	l_songsindex[0] = -1;

	err = ChangePlaylist(playlist);

	if(!g_w[WINDOWAPP])
	{
		if(err)
			LoadDefaultSettings(&g_sets);
		AppWindowMakeA();
	}			
	else
		maLog(&l_logcmdexist);
		
	if(err)
		return 1;

	// start
	if(g_sets.output == kOUTSOUND)
	{
		if(fsstrcmp(g_playlistpath, g_playlistdrop) || g_sets.autostart)
			if(g_songsnum > 0)
				DoTimerPlay(100); // PlayPlaylistB
	}
		
	return 0;
	
} // PlayPlaylistB


void UpdateListWindowList(int newcd)
{
#if !defined(MAGUINONE)
	if(!g_w[WINDOWLIST])
		return;
#endif

	l_listisfrozen = 1;
	ListFreeze();
	if(g_sets.listboxmode & LBOXPLAYERINFO)
		ListPlayerInfo(newcd);
	else if(g_sets.listboxmode & LBOXPLAYLIST)
		ListPlaylist();
	else if(g_sets.listboxmode & LBOXSONGSLOADED)
		ListLoaded();
	else if(g_sets.listboxmode & LBOXSONGINFO)
		ListSongInfo();
	else
		maLog(L"UpdateListWindowList g_sets.listboxmode has no value");
	ListThaw();
	l_listisfrozen = 0;

} // UpdateListWindowList


// this is called when timerrunloop fires from the gui event loop,
// or if nogui, from the 'main' loop, 
signed long RunLoop()
{		
	int ret;
	
	 if(g_state.paused)
	{
#if defined(MAGUINONE) && (defined(__MAUNIX__) || defined(__MAWIN__))
		return (g_posendon ? SEEKPERIOD : TITLEPERIOD);
#else
		DoTimerRunLoop(TITLEPERIOD);
		return 0;
#endif
	}

	if(l_runloop == maSTOPPING)
	{
		l_runloop = maSTARTING;	
		DisableButtonsMenus();
		l_quit = 0;
		g_state.running = 1;
		SetAppWindowTitle(L"");
	}

	if(l_runloop == maSTARTING)
	{
		if(!l_noreload)
		{
			if(l_singleplay)
				l_singleplay = 0;
			else
				g_loadednum = 0;
		}
	}
	
	if(l_runloop >= maSTARTING)
	{
		if(g_state.stop & STOP_ALL)
			l_runloop = maSTOPPING;
		else if((ret = RunLoad()) >= 1 && ret <= 4)	// 1 failure 		2 no player
			l_runloop = maSTOPPING;					// 3 no playlist	4 no memory
	}

	if(l_runloop == maSTOPPING) // stop
	{
		if(l_quit)
		{
			AppQuit();
			g_state.running = 0;
		}
		else
		{
			SetAppWindowTitle(L"Stopped");
			g_droplaunch = 0;
			if(g_changeplaylist[0] != fsT('\0'))
				DoTimerChangePlaylist(100);
			g_state.running = 0;
			EnableButtonsMenus();
		}
	}
	else // keep going
	{
#if defined(MAGUINONE) && (defined(__MAUNIX__) || defined(__MAWIN__))
		return (g_posendon ? SEEKPERIOD : TITLEPERIOD);
#else
		DoTimerRunLoop(g_posendon ? SEEKPERIOD : TITLEPERIOD);
		return 0;
#endif
	}
	
	return 0;
	
} // RunLoop


void DoPlay() // Play
{	
	if(g_state.running)
		return;

	l_songsindex[0] = -1;
	g_state.infiletype = kMALAST;

#if !defined(NODECODEAUDIOIN)
	if(PlaylistIsAudioIn())
		g_state.infiletype = kMAMAA;
	else
#endif
	{
#if !defined(MANOCDA)
		if(PlaylistIsCda())
		{
			g_state.infiletype = kMACDA; // prolly don't need this
			if(CdaInitCDRom()) // DoPlay
			{
				ShowErr(L"Insert an Audio CD or open another playlist");
				return;
			}
		}
#endif
	}

	DoTimerRunLoop(100);

} // DoPlay


void DoPause()
{
	if(g_state.paused) // toggle
		SetPause(0);
	else
		SetPause(1);

} // DoPause


void DoStop()
{
	l_catalogcancel = 1;

	if(g_state.scanning)
	{
		g_scanstop = 1;
		SetAppWindowTitle(L"Scanning interrupted");
		g_state.scanning = 0;
		return;
	}
	
	g_state.silence = 1;
	if(g_state.running && !(g_state.stop & STOP_APPQUIT))
		SetAppWindowTitle(L"Stopping...");
	SetPause(0);
	g_state.stop |= STOP_ALL;
	l_noreload = 0;
	g_droplaunch = 0;

} // DoStop



void DoPrevious() // Previous
{	
	PushButtonDisable(WINDOWAPP, ID_BTN_PREVIOUS);
					
	if(g_state.paused)
	{
		g_state.silence = 1;
		SetPause(0);
	}
			
	if(!g_state.running) // from Stopped state
	{
		if(g_sets.buffertype == BUFUSEDIRECT && g_sets.reptplayed && !g_sets.random)
		{
			if((g_droplaunch = l_songcurrent + 1) < 2)	// 0-based row to play + 2
				g_droplaunch = g_songsnum + 1;			// recirculate to last
			DoTimerPlay(100); // DoPrevious
			PushButtonEnable(WINDOWAPP, ID_BTN_PREVIOUS);
			return;
		}

		if(g_songsnum)
		{
			l_noreload = 1;
			EnableButtonsMenus();
			DoTimerPlay(100); // DoPrevious
		}
	}
	
	if(g_sets.buffertype == BUFUSEDIRECT && g_sets.reptplayed && !g_sets.random)
	{
			l_previous = 1;
			l_next = 0;
			l_loadednew = 0;
	}
	else
	{
		if((l_loadednew = g_loadedcurrent) <= 0)
			l_loadednew = 1;
	}
	
	g_loadedquit = g_loadedquitB = 1;
					
	PushButtonEnable(WINDOWAPP, ID_BTN_PREVIOUS);		

} // DoPrevious


void DoNext() // Next
{	
	PushButtonDisable(WINDOWAPP, ID_BTN_NEXT);
	
	if(g_state.paused)
	{
		g_state.silence = 1;
		SetPause(0);
	}
	
	if(!g_state.running) // from Stopped state
	{
		if(g_sets.buffertype == BUFUSEDIRECT && g_sets.reptplayed && !g_sets.random)
		{
			if((g_droplaunch = l_songcurrent + 3) > g_songsnum + 1)	// 0-based row to play + 2
				g_droplaunch = 2;									// recirculate to first
			PushButtonEnable(WINDOWAPP, ID_BTN_NEXT);
			DoTimerPlay(100); // DoNext
			return;
		}

		if(g_songsnum) // some songs are loaded
		{
			if((l_loadednew = g_loadedcurrent + 2) > g_loadednum)
			{
				if(g_loadednum && g_sets.looploaded)
				{
					l_loadednew = 1; // back to first loaded song
					l_noreload = 1;
				}
				else
				{
					l_loadednew = 0;
					l_noreload = 0;
				}
			}
			else
				l_noreload = 1;

			PushButtonEnable(WINDOWAPP, ID_BTN_NEXT);
			DoTimerPlay(100); // DoNext
			return;
		}
	}

	if(g_sets.buffertype == BUFUSEDIRECT && g_sets.reptplayed && !g_sets.random)
	{
			l_previous = 0;
			l_next = 1;
			l_loadednew = 0;
	}
	else
		if((l_loadednew = g_loadedcurrent + 2) > g_loadednum)
		{
			if(g_loadednum && g_sets.looploaded)
			{
				l_loadednew = 1; // back to first loaded song
				l_noreload = 1;
			}
			else
			{
				l_loadednew = 0;
				l_noreload = 0;
			}
		}
		else
			l_noreload = 1;
	
	g_loadedquit = g_loadedquitB = 1;

	PushButtonEnable(WINDOWAPP, ID_BTN_NEXT);

} // DoNext


void DoPlayCD() // Play CD
{
#ifndef MANOCDA
	if(CdaInitCDRom()) // DoPlayCD
	{
		ShowErr(L"Insert an Audio CD");
		return;
	}

	fsstrcpy(g_newplaylist, g_playlistcda);
	DoStop(); // DoTimerStop(1);
	WaitDecoderStopped();
	DoTimerPlayPlaylist(10);
#endif

} // DoPlayCD


void DoAppQuit() // Exit
{
	PushButtonDisable(WINDOWAPP, ID_BTN_QUIT);
	
	g_state.stop |= STOP_APPQUIT;

	if(g_state.running || g_cataloguing)
	{
		l_quit = 1;
		DoStop();
		PushButtonEnable(WINDOWAPP, ID_BTN_QUIT);
	}			
	else
		AppQuit();

} // DoAppQuit


// this is called every iteration through RunLoop
int RunLoad()
{
	int err = 0;

	if(l_runloop == maSTARTING)
		if(g_state.stop & STOP_ALL)
			l_runloop = maSTOPPING;

	if(l_runloop == maSTARTING)
	{
		l_runloop = maSCANNING;

		if(g_loadednum <= 0 || (!g_sets.looploaded && !l_noreload))
		{
			if(!g_playlist0) // get playlist and its settings
				if(OpenPlaylist(0))
					return 3;	
	
			// get mem or vdisk
			if(g_memhandle)
				MAFREE(g_memhandle);
			g_memhandle = NULL; 
			l_songmem = NULL;
			l_songmemend = NULL;
			l_songmempos = NULL;
			g_loadedmemsize = 0;
			l_mnextfile = -1;

			if(g_sets.buffertype == BUFUSEMEM)
			{
				if(GetSongMem())
					return 4;
			}
			else
				if(g_sets.buffertype == BUFUSEVDISK && GetVDisk())
					if(SetVDisk() || GetVDisk()) // prompt for vdisk
						return 1;
		}
		
	} // if(l_runloop == maSTARTING)

	if(l_runloop >= maSTARTING)
	{
		if(l_runloop == maSCANNING)
		{
			if(g_state.stop & STOP_ALL)
				l_runloop = maSTARTING;
		}			

		if(l_runloop == maSCANNING)
		{
			if(g_loadednum <= 0 || (!g_sets.looploaded && !l_noreload))
			{
				if(l_buffersize != g_sets.buffersize)
				{
					l_buffersize = g_sets.buffersize;
					if(g_sets.buffertype == BUFUSEMEM)
						if(GetSongMem())
							return 4;
				}
			
				if((err = FillSongBuffer(g_sets.buffertype)) != 0)
					if(err == 1 || g_sets.buffertype == BUFUSEMEM || g_sets.buffertype == BUFUSEVDISK)
						return 1;
			}
			else if(g_sets.looploaded) // prevent ListWindow update
				l_noreload = 1;

			if(g_sets.mawin[WINDOWLIST].ison && g_sets.listboxmode & LBOXSONGSLOADED && !l_noreload)
				UpdateListWindowList(0);
				
			l_noreload = 0;
		}

		// play the songs
		if(l_runloop)
		{
			if((err = RunPlay()) != 0)
				l_runloop = maSCANNING;
			if(g_state.stop & STOP_ALL)
				l_runloop = maSTARTING;
		}

		if(l_runloop == maSCANNING)
		{
			if(!g_sets.looploaded)
			{
				if(!PlaylistIsNoRecords())
					SaveRecords();
				l_noreload = 0;
			}

			if(g_state.stop & STOP_ALL)
				l_runloop = maSTARTING;
			
			if(!(g_sets.looploaded && !g_droplaunch))
				l_runloop = maSTARTING;
		}		
		if(err)
			l_runloop = maSTARTING;			
	}
	
	return err;
		
} // RunLoad


void DoMakeCatalog()
{		
	int i;
	mauint64 size;
	mauint64 bigsize = 0;
	size_t ret0;
	FILE *fpcatalogfile = NULL;
	int numsongs = 0;
	size_t len;
	size_t len3 = 0;
	fschar *p;
	machar *p0;
	machar *p2 = NULL;
	machar *p3 = NULL;
	machar *p4;
	machar *p5;
	fschar name[MAX_PATH];
	fschar bigfile[MAX_PATH];
	fschar catalogfile[MAX_PATH];
	fschar songsourcefs[MAX_PATH];
	fschar playlistpath[MAX_PATH];
	machar songsource[MAX_PATH] = {L'\0'};
	machar mastr2[MAXLOGLINELEN];
#if !defined(__MAWIN__)
	machar mastr3[MAXLOGLINELEN];
#endif
	machar mastr[MAXLOGLINELEN];
	
	// DisableButtonsMenus();
	g_cataloguing = 1;
	MenuDisable(MENUTOOLS, ID_MENU_MAKECATALOG);
#if !defined(NOTEXTVIEWER)
	MenuDisable(MENUTOOLS, ID_MENU_VIEWCATALOG);
	if(g_w[WINDOWCATALOG])
		PushButtonDisable(WINDOWCATALOG, ID_BTN_EDITSAVE);
#endif
	bigfile[0] = fsT('\0');

	if(g_playlistpath[0] == fsT('\0'))
	{
		maLog(L"DoMakeCatalog no playlist");
		maLog(mastr);
		goto catalogret;
	}		

	fsstrcpy(playlistpath, g_playlistpath);
	if(FilenameFromPathname(playlistpath, name))
	{
		maLog(L"DoMakeCatalog FilenameFromPathname failed");
		fsstrncpy(name, playlistpath, MAX_PATH - 1);
		name[MAX_PATH - 1] = fsT('\0');
		maLog(fs2ma(mastr2, name));
		goto catalogret;
	}

	if((p = fsstrstr(name, g_extstr[kMAMAF])) != 0)
		*p = fsT('\0');

	if((len = fsstrlen(g_madir) + sizeof(FILESEP) + fsstrlen(name) + 4) >= MAX_PATH)
	{
		masprintf(mastr, MAXLOGLINELEN, L"DoMakeCatalog file path > max %d : %zu", MAX_PATH - 1, len);
		maLog(mastr);
		fsstrncpy(name, playlistpath, MAX_PATH - 1);
		name[MAX_PATH - 1] = fsT('\0');
		maLog(fs2ma(mastr2, name));
		goto catalogret;
	}

	fssprintf(catalogfile, MAX_PATH, fsS fsC fsS fsT(".txt"), g_madir, FILESEP, name);

	if((fpcatalogfile = mafopen(catalogfile, FSWBMODE)) == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"DoMakeCatalog mafopen failed on '%ls'", fs2ma(mastr2, catalogfile));
		maLog(mastr);
		goto catalogret;
	}
	
 #if defined(__MAOSX__) || defined(__MAUNIX__)
	machmod(catalogfile, ACCESSPERMS); // 0777
 #endif

	ret0 = mafileread(&p0, g_playlistpath);
	if(ret0 <= 0 || p0 == NULL)
	{			
		if(p0 != NULL)
		{
			MAFREE(p0);
			p0 = NULL;
		}
		maLog(L"DoMakeCatalog mafileread failed");
		goto catalogret;
	}

	// BOM
	fwrite("\xef\xbb\xbf", 3, 1, fpcatalogfile); // BOM_UTF-8

	// header
	masprintf(mastr2, MAXLOGLINELEN, L"%ls catalog", MOREAMP_NAME);
	mafilewrite(0, mastr2, fpcatalogfile);
	
	if(maGetLocalDateTimeString(mastr))
	{
		mastrcpy(mastr2, L" created ");
		mastrcat(mastr2, mastr);
		mastrcat(mastr2, g_maeol);
		mafilewrite(0, mastr2, fpcatalogfile);
	}
	
	mastrcpy(mastr2, L"of playlist : ");
#if defined(__MAOSX__) || defined(__MACUNIX__)
	fsstrcpy(songsourcefs, playlistpath);
	StrUTF8MACtoUTF8(songsourcefs);
	mastrcat(mastr2, fs2ma(mastr3, songsourcefs));
#else
	mastrcat(mastr2, fs2ma(mastr3, playlistpath));
#endif
	mastrcat(mastr2, g_maeol);
	mafilewrite(0, mastr2, fpcatalogfile);
	
	// l_posslidersetvalue = 1;
	// SetPosSlider(-1);
	l_catalogcancel = 0;
	
	masprintf(mastr, MAXLOGLINELEN, L"%ls Catalog... %d", fs2ma(mastr2, name), g_songsnum);
	WindowSetTitle(WINDOWAPP, mastr);

	i = 0;
	p4 = p0;
	while((p5 = memgets(songsource, MAX_PATH, p4)) != NULL)
	{
		p4 = p5;
		if((i % 100) == 0)
		{
			// SetPosSlider(LROUND((-(numsongs * (double)POSMAX) / g_songsnum))); // this is a major slowdown
			masprintf(mastr, MAXLOGLINELEN, L"%ls Catalog... %d/%d", fs2ma(mastr2, name), i, g_songsnum);
			WindowSetTitle(WINDOWAPP, mastr);
			ThreadYield();
			AppYield();
		}
		i++;
		if(l_catalogcancel)
		{
			masprintf(mastr2, MAXLOGLINELEN, L"%lsCATALOG CANCELLED", g_maeol);
			mafilewrite(0, mastr2, fpcatalogfile);
			MAFREE(p0);
			p0 = NULL;
			fclose(fpcatalogfile);
			SetAppWindowMsg(L"Catalog cancelled");
			goto catalogret;
		}
			
		if(songsource[0] == L'\0')
			continue;
			
		// skip lines with '#' or files that are not playable or don't exist
		if(songsource[0] == L'#')
			continue;

		ma2fsfn(songsourcefs, songsource);
		if(!SongFileExists(songsourcefs))
		{
			masprintf(mastr2, MAXLOGLINELEN, L"%ls%ls%ls", g_maeol, g_maeol, MACATALOGERRSTRING);
			mafilewrite(0, mastr2, fpcatalogfile);
			masprintf(mastr2, MAXLOGLINELEN, L"file not playable or not found:%ls'", g_maeol);
			mafilewrite(0, mastr2, fpcatalogfile);
			mafilewrite(0, songsource, fpcatalogfile);

			masprintf(mastr2, MAXLOGLINELEN, L"'%ls", g_maeol); 
			mafilewrite(0, mastr2, fpcatalogfile);
			continue;
		}
			
		numsongs++;		
		
		masprintf(mastr2, MAXLOGLINELEN, L"%ls%05d %ls%ls", g_maeol, numsongs, MACATALOGSTRING, g_maeol);	
		mafilewrite(0, mastr2, fpcatalogfile);
		
		if(l_psongrec)
			masprintf(mastr2, MAXLOGLINELEN, L"%ls +%d%ls", songsource, l_psongrec[numsongs - 1], g_maeol);
		else
			masprintf(mastr2, MAXLOGLINELEN, L"%ls%ls", songsource, g_maeol);
		mafilewrite(0, mastr2, fpcatalogfile);
		
		if((size = DumpSongInfo(songsourcefs, fpcatalogfile, 1, &p2, &p3, &len3)) > bigsize)
		{
			bigsize = size;
			fsstrcpy(bigfile, songsourcefs);
		}
	} // while

	if(p2)
	{
		MAFREE(p2);
		p2 = NULL;
	}
		
	if(p3)
	{
		MAFREE(p3);
		p3 = NULL;
	}
		
	if(p0)
	{
		MAFREE(p0);
		p0 = NULL;
	}
		
	if(bigsize > 0)
	{
		masprintf(mastr2, MAXLOGLINELEN, L"%lsLargest file ", g_maeol);
		mafilewrite(0, mastr2, fpcatalogfile);
		if(bigsize >= 1000000)
		{
			int millions = (int)(bigsize / 1000000);
			int thousands = (int)((bigsize - millions * 1000000) / 1000);
			masprintf(mastr2, MAXLOGLINELEN, L"%d MB (%d,%03d,%03d bytes)", 1 + (int)bigsize / MEGABYTE,
				millions, thousands, ((int)bigsize - millions * 1000000) - thousands * 1000);
			mafilewrite(0, mastr2, fpcatalogfile);
		}
		else if(bigsize >= 1000)
		{
			int thousands = (int)(bigsize / 1000);
			masprintf(mastr2, MAXLOGLINELEN, L"%d kB (%d,%03d bytes)",  1 + (int)bigsize / KILOBYTE,
												thousands, (int)bigsize - thousands * 1000);
			mafilewrite(0, mastr2, fpcatalogfile);
		}
		else
		{
			masprintf(mastr2, MAXLOGLINELEN, L"%d bytes", (int)bigsize);
			mafilewrite(0, mastr2, fpcatalogfile);
		}
		masprintf(mastr2, MAXLOGLINELEN, L" :%ls'%ls'", g_maeol, fs2ma(mastr3, bigfile));
		mafilewrite(0, mastr2, fpcatalogfile);
	}

	fclose(fpcatalogfile);
	
	if(numsongs == 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"DoMakeCatalog no playable songs in playlist '%ls'",
																				fs2ma(mastr2, g_playlistpath));
		maLog(mastr);
	}
	
	SetAppWindowMsg(L"Catalog done");
	
catalogret:	
	SetPosSlider(-1);	

#if !defined(NOTEXTVIEWER)
	if(g_w[WINDOWCATALOG]) // reopen
	{
		DoWindow(WINDOWCATALOG, 0);
		DoViewCatalog(1);		
		// PushButtonEnable(WINDOWCATALOG, ID_BTN_EDITSAVE);
	}
#endif

	g_cataloguing = 0;
#if !defined(NOTEXTVIEWER)
	MenuEnable(MENUTOOLS, ID_MENU_VIEWCATALOG);
#endif
	MenuEnable(MENUTOOLS, ID_MENU_MAKECATALOG);

} // DoMakeCatalog


int GetNextSong(int *pm, MARECORD *psongrec)
{
	int j, k, m, n, p;
	
	if(!psongrec)
		return -1;

	// get song m, 0 <= m < g_songsnum
	if(g_droplaunch >= 2)
			m = g_droplaunch - 2; 	// doubleclicked on playlist item or entered song index to play
	else
	{				
		if(l_mnextfile >= 0) // previously rejected 'large' file
		{
			m = l_mnextfile;
			l_mnextfile = -1;
		}
		else // pick a song from the playlist, index m | 0 <= m < g_songsnum
		{
			if(g_sets.reptplayed)
			{
				if(g_sets.random)
					m = (int)(LROUND((rand() * (double)g_songsnum) / RAND_MAX));
				else
					if((m = *pm + 1) >= g_songsnum)
						m = 0;
			}
			else
			{
				if(l_songsnumplayed >= g_songsnum)			// all have played
				{
					if(!g_sets.looplist)
						return -2;

					ZeroRecords(&psongrec, g_songsnum);		// clear played settings
					ZeroRecords(&l_psongrec, g_songsnum);
					l_songsnumplayed = 0;
					for(j = 0; j < SONGSLOADMAX; j++)		// except those just selected
						if(l_songsindex[j] >= 0)
							psongrec[l_songsindex[j]] = 1;	// so we don't select them again
				}
			
				p = 0;
				for(j = 0; j < SONGSLOADMAX; j++)			// count songs already picked for playing
					if(l_songsindex[j] >= 0)
						p++;

				if((n = g_songsnum - l_songsnumplayed - p) <= 0)
					return -2; // no unplayed or unpicked songs, else n unplayed and unpicked songs to pick from

				if(g_sets.random)
					k = (int)(LROUND((rand() * (double)n) / RAND_MAX)); // kth unplayed and unpicked song, 0 <= k < n
				else
					k = 0;
					
				p = 0;
				for(m = 0; m < g_songsnum; m++) // get playlist index m of the kth unplayed and unpicked song
				{
					if(!psongrec[m]) // song m is unplayed
					{
						for(j = 0; j < SONGSLOADMAX; j++)
							if(l_songsindex[j] == m)
								break;
						if(j >= SONGSLOADMAX) // song m is unpicked
							if(p++ == k)
								break; // m is the kth unplayed and unpicked song
					}
				}
				
			} // !g_sets.reptplayed

		} // l_mnextfile < 0
		
	} // if g_droplaunch < 2

	*pm = m;

	return 0;

} // GetNextSong


int FillSongBuffer(int dest)
{
	machar *p4;
	machar *p5;
	int j, k, m;
	int ret;
	int err;
	int once = 0;
	long memleft;
	long bytescopied = 0;
 	fschar songsource[MAX_PATH];
 	machar masongsource[MAX_PATH];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
	machar mastr3[MAXLOGLINELEN];
#endif
	MARECORD *psongrec = NULL;
	machar mastr[MAXLOGLINELEN];

	l_buffersize = g_sets.buffersize;
	
	if(dest == BUFUSEMEM)
	{
		// clear mem pointers
		memset(g_songmemptr, 0, sizeof(g_songmemptr));
		g_songmemptr[0] = l_songmem;
		memleft = (long)g_loadedmemsize;
	}
	else if(dest == BUFUSEVDISK)
	{
		memleft = l_buffersize * MEGABYTE;	
		// clear l_loadeddir			
		if(maDirExists(l_loadeddir))
		{
			if(maClearDir(l_loadeddir))
			{
				masprintf(mastr, MAXLOGLINELEN, L"FillSongBuffer maClearDir failed on '%ls'",
																			fs2ma(mastr2, l_loadeddir));
				maLog(mastr);
				return 1;
			}
		}
		else if(maCreateDir(l_loadeddir))
		{
			masprintf(mastr, MAXLOGLINELEN, L"maCreateDir failed on '%ls'",
															fs2ma(mastr2, l_loadeddir));
			maLog(mastr);
			return 1;
		}
	}
	else if(dest == BUFUSEDIRECT)
	{
		// do one song file
		memleft = 0;
		g_loadednum = 0;				
		m = l_songsindex[0];
	}
	else
	{
		maLog(L"FillSongBuffer error bad dest");
		return 1;
	}

	if(g_songsnum <= 0)
	{
		maLog(L"FillSongBuffer no playable songs in playlist");
#if defined(NODECODEAUDIOIN)
		if(FileNameHasExtension(g_playlistpath, g_extstr[kMAMAF]))
			maLog(L"FillSongBuffer : audio input not implemented");
#endif
		return 1;
	}
	
	// use a temp local copy of l_psongrec :	
	if(!l_psongrec)
	{
		maLog(L"FillSongBuffer error NULL l_psongrec");
		return 1;
	}
	
	if(AllocSongRec(&psongrec, g_songsnum))
		return 1;
		
	memcpy(psongrec, l_psongrec, g_songsnum * sizeof(MARECORD));
			
	// fill songbuffer with song files :	
	g_loadednum = j = 0; // count of songs loaded
				
	err = ret = 0;
		
	if(g_sets.buffertype == BUFUSEDIRECT && g_sets.reptplayed && !g_sets.random) 
	{
		if(l_previous)
		{
			l_previous = 0;
			if((m = l_songsindex[0] - 2) < -1) // previous
				m = g_songsnum - 2;
		}
		else if(l_next)
		{
			l_next = 0;
			if((m = l_songsindex[0]) >= g_songsnum) // next
				m = -1;
		}
	}
	else
		m = -1; // -1 gets first song
		
	for(k = 0; k < SONGSLOADMAX; k++)
		l_songsindex[k] = -1;
	
	while(ret != dskFulErr && j < g_songsnum && j < SONGSLOADMAX)
	{
		ThreadYield();
		AppYield();
		if(g_state.stop & STOP_ALL)
			goto fillsongerrret;

		/*
		// this is a major slowdown
		if(!g_cataloguing)
			SetPosSlider(LROUND((-((g_loadedmemsize - memleft) * (double)POSMAX) / g_loadedmemsize)));
		*/

		// get song m
		if(g_state.infiletype == kMAMAA)
			m = 0;
		else if((err = GetNextSong(&m, psongrec)) != 0)
		{
			if(err == -2) // all have played
				break;
			masprintf(mastr, MAXLOGLINELEN, L"FillSongBuffer GetNextSong failed for '%ls'", fs2ma(mastr2, g_playlistpath));
			maLog(mastr);
			goto fillsongerrret;
		}
					
		// gets here with song m
					
		// read songname m from playlist :	
		if(!g_playlist0)
		{
			masprintf(mastr, MAXLOGLINELEN, L"FillSongBuffer playlist not loaded '%ls'", fs2ma(mastr2, g_playlistpath));
			maLog(mastr);
			goto fillsongerrret;
		}					

		p4 = g_playlist0;
			
		k = 0;
			
		while(k <= m)
		{
			if((p5 = memgets(masongsource, MAX_PATH, p4)) == NULL) // should never eof
			{
				ma2fsfn(songsource, masongsource);
				masprintf(mastr, MAXLOGLINELEN, L"FillSongBuffer song file'%ls%ls%ls%ls%ls"
								L"'could not be found for index %d in'"
								L"%ls%ls%ls%ls%ls'Re-open a playlist or restart MoreAmp",
								g_maeol, g_maeol, fs2ma(mastr2, songsource), g_maeol, g_maeol,
								m + 1, g_maeol, g_maeol, fs2ma(mastr3, g_playlistpath), g_maeol, g_maeol);
				ShowErr(mastr);
				goto fillsongerrret;
			}
			ma2fsfn(songsource, masongsource);
			p4 = p5;
						
			// skip lines with '#' or files that are not playable or don't exist
			if(songsource[0] == fsT('#'))
				continue;

			if(!SongFileExists(songsource))
			{
				masprintf(mastr, MAXLOGLINELEN, L"FillSongBuffer file not playable or not found: '%ls'",
																						fs2ma(mastr2, songsource));
				maLog(mastr);
				continue;
			}
						
			k++;
		}

		// save song name		
		fsstrcpy(g_loadedpath[j], songsource);
		
		bytescopied = 0;

		if(FileNameHasExtensionPlayable(songsource))
		{
			if(FilenameFromPathname(songsource, g_loadedname[j]))
			{
				masprintf(mastr, MAXLOGLINELEN, L"FillSongBuffer failed to get filename from '%ls'",
																						fs2ma(mastr2, songsource));
				maLog(mastr);
				goto fillsongerrret;
			}

			if(dest == BUFUSEMEM)
			{
				if(!once++)
					SetAppWindowTitle(L"loading memory...");
				ThreadYield();
				AppYield();
				ret = FileToMem(songsource, g_songmemptr[j], memleft, &bytescopied);
			}
			else if(dest == BUFUSEVDISK)
			{
				if(!once++)
					SetAppWindowTitle(L"loading Virtual Disk...");
				ThreadYield();
				AppYield();
				ret = FileToVDisk(songsource, g_loadedname[j], memleft, &bytescopied);
			}
			else // direct
				ret = 0;

			if(ret)
			{
				if(ret == dskFulErr)
				{
					if(bytescopied > 0 && bytescopied <= (long)g_loadedmemsize) // bytescopied holds filesize
						l_mnextfile = m; // 'large' file rejected now will play next time
					else
						l_mnextfile = -1;
					break;
				}
				
				if(dest == BUFUSEMEM)
					masprintf(mastr, MAXLOGLINELEN, L"FillSongBuffer FileToMem failed '%ls'",
																				fs2ma(mastr2, songsource));
				else if(dest == BUFUSEVDISK)
					masprintf(mastr, MAXLOGLINELEN, L"FillSongBuffer FileToVDisk failed '%ls'",
																				fs2ma(mastr2, songsource));
				else
					masprintf(mastr, MAXLOGLINELEN, L"FillSongBuffer FileDirect failed '%ls'",
																				fs2ma(mastr2, songsource));
				maLog(mastr);
				goto fillsongerrret;
			}

			memleft -= bytescopied;
		}
		else // is cda
			fsstrcpy(g_loadedname[j], songsource);
		
		// save song index
		l_songsindex[j] = m;						
		
		// locally mark song as played
		psongrec[m] += 1;
		
		g_songmemptr[j + 1] = g_songmemptr[j] + bytescopied;
		j++;
		
		if(g_droplaunch >= 2) // started by a doubleclick on playlist item
		{
			g_droplaunch = 0;
			break;
		}
		
		if(dest == BUFUSEDIRECT)
			break;
			
	} // while ret != dskFulErr

	if((g_loadednum = j) <= 0)
	{
		if(err == -2)
		{
			if(g_sets.exitafter)
				l_quit = 1;
			else
			{
				if(PlaylistIsCda())
					ShowErr(l_allhaveplayedcdastr);
				else
					ShowErr(l_allhaveplayedstr);
			}
		}
		else if(ret == dskFulErr)
		{
			if(dest == BUFUSEMEM)
			{
				masprintf(mastr, MAXLOGLINELEN, L"There is not enough memory to load%ls%ls'%ls'%ls%ls"
				L"Either :%ls%ls- Increase the Mem Buffer to at least %dMB, or%ls%ls- Set the Buffer mode to Direct.",
				g_maeol, g_maeol, fs2ma(mastr2, g_loadedname[j]), g_maeol, g_maeol,
				g_maeol, g_maeol, (int)(1 + bytescopied / MEGABYTE), g_maeol, g_maeol);
				ShowErr(mastr);
			}
			else if(dest == BUFUSEVDISK)
			{
				masprintf(mastr, MAXLOGLINELEN, L"There is not enough virtual disk space to load%ls%ls'%ls'%ls%ls" 
				L"Either :%ls%ls- Increase the virtual disk size to at least %d MB, or%ls%ls- Set the Buffer mode to Direct.",
				g_maeol, g_maeol, fs2ma(mastr2, g_loadedname[j]), g_maeol, g_maeol, g_maeol, g_maeol, (int)(1 + bytescopied / MEGABYTE), g_maeol, g_maeol);
				ShowErr(mastr);
			}
		}
		goto fillsongerrret;
	}
	
	goto fillsongokret;
	
fillsongerrret:
	if(psongrec)
		MAFREE(psongrec);

	l_noreload = 0;
	
	if(g_sets.titleinfo == 0)
		ResetAppTitle();

	return 1;
	
fillsongokret:

	if(psongrec)
		MAFREE(psongrec);
		
	if(g_sets.titleinfo == 0)
		ResetAppTitle();

	for(k = 0; k < g_loadednum; k++)
		if(l_songsindex[k] < 0 || l_songsindex[k] >= g_songsnum)
		{
			masprintf(mastr, MAXLOGLINELEN, L"FillSongBuffer bad l_songsindex[%d]=%d", k, l_songsindex[k]);
			maLog(mastr);
			return 1;
		}	
	
	return 0;
			
} // FillSongBuffer


int AllocSongRec(MARECORD **ppsongrec, int songsnum)
{
	machar mastr[MAXLOGLINELEN];
	
	if(!ppsongrec)
	{
		maLog(L"AllocSongRec null ppsongrec");
		return 1;
	}
	
	if(*ppsongrec)
	{
		MAFREE(*ppsongrec);
		*ppsongrec = NULL;
	}
							
	if(songsnum <= 0)
	{
		maLog(L"AllocSongRec songsnum <= 0");
		return 1;
	}
		
	if((*ppsongrec = (MARECORD *)MAMALLOC(songsnum * sizeof(MARECORD))) == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"AllocSongRec MAMALLOC failed");
		maLog(mastr);
		return 1;
	}
	
	if(ZeroRecords(ppsongrec, songsnum))
	{
		masprintf(mastr, MAXLOGLINELEN, L"AllocSongRec ZeroRecords failed");
		maLog(mastr);
		return 2;
	}
	
	return 0;
	
} // AllocSongRec


int ZeroRecords(MARECORD **ppsongrec, int songsnum)
{
	machar mastr[MAXLOGLINELEN];
		
	if(*ppsongrec == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"ZeroRecords cannot reset settings, null settings pointer");
		maLog(mastr);
		return 1;
	}
		
	if(songsnum <= 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"ZeroRecords songsnum <= 0");
		maLog(mastr);
		return 1;
	}
		
	memset(*ppsongrec, 0, songsnum * sizeof(MARECORD));
	
	return 0;
	
} // ZeroRecords


int FileToVDisk(fschar *from, fschar *tomem, int maxmem, long *bytesread)
{
	int ret;
	long filesize;
	FILE *fp;
	int n;
	fschar playpath[MAX_PATH];
	fschar playpath2[MAX_PATH + 16];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
#endif
	machar mastr[MAXLOGLINELEN];
	
	if(!from || (from[0] == fsT('\0')))
	{
		maLog(L"FileToVDisk null from filename");
		return 1;
	}
	
	if(!FileNameHasExtensionPlayable(from))
	{
		masprintf(mastr, MAXLOGLINELEN, L"FileToVDisk not a playable file '%ls'", fs2ma(mastr2, from));
		maLog(mastr);
		return 1;
	}

	if(!tomem || (tomem[0] == fsT('\0')))
	{
		maLog(L"FileToVDisk null tomem");
		return 1;
	}
	
	// get file size
	if((fp = mafopen(from, fsT("rb"))) == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"FileToVDisk failed to open '%ls'", fs2ma(mastr2, from));
		maLog(mastr);
		return 1;
	}

	if(fseek(fp, 0, SEEK_END) != 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"FileToVDisk fseek errno=%d", errno);
		maLog(mastr);
	}
	filesize = ftell(fp);
	fclose(fp);
	*bytesread += filesize;
	if(*bytesread > maxmem)
		return dskFulErr;
		
	// if the name already exists in vdisk, rename the song by adding digits 
	fssprintf(playpath, MAX_PATH, fsS fsC fsS, l_loadeddir, FILESEP, tomem);
	n = 0;
	fsstrcpy(playpath2, playpath);
	while(maFileExists(playpath2) && n < 9999) // 9999 arbitrary
	{
		n++;
		// get extension ...
		// cut extension ...
		fssprintf(playpath2, MAX_PATH, fsS fsT("%d"), playpath, n);
		// cat extension ...
		if(fsstrlen(playpath2) >= MAX_PATH)
		{
			masprintf(mastr, MAXLOGLINELEN, L"FileToVDisk file path too long '%ls'", fs2ma(mastr2, playpath2));
			maLog(mastr);
			return 1;
		}
	}
	if(n > 0 && n < 999)
		fsstrcpy(playpath, playpath2);
		
	// copy song to l_loadeddir
	if((ret = maFileCopy(from, playpath, 0)) != 0)
	{
		if(ret == dskFulErr)
			return dskFulErr;
					
		masprintf(mastr, MAXLOGLINELEN, L"FileToVDisk maFileCopy error %d '%ls'", ret, fs2ma(mastr2, from));
		maLog(mastr);
		return 1;
	}

	return(0);
	
} // FileToVDisk


int FileToMem(fschar *from, char *tomem, int maxmem, long *bytesread)
{
	size_t ret;
	long fsize;
	FILE *fp;
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
#endif
	machar mastr[MAXLOGLINELEN];

	*bytesread = 0;

	if(!from || !from[0])
	{
		maLog(L"FileToMem null from filename");
		return 1;
	}

	if(!tomem)
	{
		maLog(L"FileToMem null tomem");
		return 1;
	}
	
	if(!maxmem)
	{
		maLog(L"FileToMem null maxmem");
		return 1;
	}

	// open file
	if((fp = mafopen(from, fsT("rb"))) == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"FileToMem failed on mafopen '%ls'", fs2ma(mastr2, from));
		maLog(mastr);
		return 1;
	}

	// get the file size
	if(fseek(fp, 0, SEEK_END) != 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"FileToMem fseek errno=%d", errno);
		maLog(mastr);
	}

	if((fsize = ftell(fp)) > maxmem)
	{
		fclose(fp);
		*bytesread = fsize;
		return dskFulErr;
	}

	rewind(fp);
	
	// read the file into the passed pointer
	if((ret = fread(tomem, sizeof(char), fsize / sizeof(char), fp)) != fsize / sizeof(char))
	{
		masprintf(mastr, MAXLOGLINELEN, L"FileToMem fread '%ls' failed to read %d oschars, ret=%d",
												from, (int)(fsize / sizeof(char)), ret);
		maLog(mastr);
		fclose(fp);
		return 1;
	}

	fclose(fp);
	
	*bytesread = fsize;

	return(0);

} // FileToMem


void PlayerSeek(mauint64 pos)
{
	mauint64 lpos;
	
	if(l_playerseeking)
		return;

	if((lpos = pos) > g_state.duration)
		lpos = g_state.duration;
		
	l_playerseeking = 1;

	switch(g_state.infiletype)
	{
#ifndef NODECODEMP3
		case kMAMP1:
		case kMAMP2:
		case kMAMP3:
			mp3seek(lpos);
		break;
#endif
		
#ifndef NODECODEOGG
		case kMAOGG:
			oggseek(lpos, g_state.duration);
		break;
#endif
		
#ifndef NODECODEWMA
		case kMAWMA:
			l_playerseeking = 0; // don't seek on wma wmaseek(lpos, g_state.duration);
		break;
#endif
		
#ifndef NODECODEAIF
		case kMAAIF:
		case kMAAIFF:
		case kMARAW:
		/* these could be added to maPlayAif
		case kMAAU:
		case kMAPAF:
		case kMASVX:
		case kMANIST:
		case kMAVOC:
		case kMAIRCAM:
		case kMAW64:
		case kMAMAT4:
		case kMAMAT5:
		case kMAPVF:
		case kMAXI:
		case kMAHTK:
		*/
			aifseek(lpos);
		break;
#endif

#ifndef NODECODEWAV
		case kMAWAV:
			aifseek(lpos); // wavseek
		break;
#endif
		
#ifndef NODECODEAAC
		case kMAAAC:
		case kMAM2A:
			aacseek(lpos);
		break;
#endif

#ifndef NODECODEMP4
		case kMAMP4:
		case kMAM4A:
			mp4seek(lpos);
		break;
#endif

#ifndef NODECODEFLAC
		case kMAFLAC:
		case kMAFLA:
			flacseek(lpos);
		break;
#endif

#ifndef MANOCDA
		case kMACDA:
			cdaseek(lpos);
		break;
#endif

		default:
			l_playerseeking = 0;
		break;
	}

} // PlayerSeek


void SetAppTitle2()
{
	machar tstr[STRLEN32];
	machar title[MAXTITLELEN];
	
	MillisecondsToStrHMS(tstr, g_state.duration - g_state.position);

	if(g_sets.buffertype == BUFUSEDIRECT)		// no songs loaded
	{
		if(g_sets.reptplayed)					// show playlist index (numsongs) timeleft
			masprintf(title, MAXTITLELEN, L"%ls %d (%d) %ls", l_playlistma, l_songsindex[0] + 1, g_songsnum, tstr);
		else									// show playlist index (numplayed/numsongs) timeleft
			masprintf(title, MAXTITLELEN, L"%ls %d (%d/%d) %ls", l_playlistma, l_songsindex[0] + 1, l_songsnumplayed, g_songsnum, tstr);
	}
	else										// songs loaded
	{
		if(g_sets.reptplayed)					// show playlist index (numsongs numloadedplayed/numloaded) timeleft
			masprintf(title, MAXTITLELEN, L"%ls %d (%d %d/%d) %ls", l_playlistma, l_songsindex[g_loadedcurrent] + 1, g_songsnum,
												g_loadedcurrent + 1, g_loadednum, tstr);
		else									// show playlist index (numplayed/numsongs numloadedplayed/numloaded) timeleft
			masprintf(title, MAXTITLELEN, L"%ls %d (%d/%d %d/%d) %ls", l_playlistma, l_songsindex[g_loadedcurrent] + 1, l_songsnumplayed, g_songsnum,
												g_loadedcurrent + 1, g_loadednum, tstr);
	}
	
	WindowSetTitle(WINDOWAPP, title);

} // SetAppTitle2


void ResetAppTitle()
{
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
#endif

	if(g_sets.titleinfo == 1)
		WindowSetTitle(WINDOWAPP, fs2ma(mastr2, g_loadedname[g_loadedcurrent]));
	else if(g_sets.titleinfo == 2)
		SetAppTitle2();
	else
		SetAppWindowTitle(L"");
		
#if !defined(MAGUINONE)
	l_temptitle = 0;
#endif

	maLog(&l_logcmdexist);
	
} // ResetAppTitle


/* int WindowIdFromRef(void *ref)
{
	int windowid;
	
	for(windowid = WINDOWFIRST; windowid <= WINDOWLAST; windowid++)
		if(g_w[windowid] != NULL && g_w[windowid] == ref)
			return windowid;
	return 0;
}
*/


int AnyEffectOn()
{
	if(!(g_sets.effects & EFFECTSON))
		return 0;

	if(g_sets.effects & EFFECTSPITCH && (g_sets.pitchslider != EFFECTSPITCHZERO || g_sets.pitchslider2 != EFFECTSPITCH2ZERO))
		return (g_sets.effects & EFFECTSSBSMS) ? EFFECTSSBSMS : 1;
		
	if(g_sets.effects & EFFECTSTEMPO && g_sets.temposlider != EFFECTSZERO)
		return (g_sets.effects & EFFECTSSBSMS) ? EFFECTSSBSMS : 1;
	
	if(g_sets.effects & EFFECTSSPEED && g_sets.speedslider != EFFECTSZERO)
		return (g_sets.effects & EFFECTSSBSMS) ? EFFECTSSBSMS : 1;
	
	return 0;

} // AnyEffectOn


void WaitDecoderStopped()
{
	while(g_state.pdecoder != NULL)
	{
		AppYield();
		maSleep(10);
	}

} // WaitDecoderStopped


int l_titletime = INT_MAX;
int l_retval;
double l_attenuation;
#if !defined(NODECODEAUDIOIN)
	time_t l_stTime;
#endif
// this is called every iteration through RunLoop from RunLoad
int RunPlay()
{
#ifndef NODECODEMP3
	float mpegattenuationdb;
#endif
	fschar *p;
	fschar playpath[MAX_PATH];
#if !defined(NODECODEAUDIOIN)
	time_t stTime;
	struct tm *tmTime;
#endif	
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
#endif
	machar mastr[MAXLOGLINELEN];

	l_retval = 0;

	if(l_runloop == maSCANNING)
	{
		l_runloop = maLOADING;
		l_attenuation = 123.0;

		FilenameFromPathname(g_playlistpath, l_playlistfs);
		if((p = fsstrstr(l_playlistfs, g_extstr[kMAMAF])) != 0)
			*p = fsT('\0');
		fs2mafn(l_playlistma, l_playlistfs);
		l_separation = g_sets.separationslider;
		l_separationch = (float)(0.5 + ((0.5 * l_separation) / SEPARATIONMAX));
		l_separationmix = (float)(1. - l_separationch);
		l_pan = g_sets.panslider;
		l_panright = (float)(l_pan  / (double)SEPARATIONMAX);
		l_panleft  = (float)((SEPARATIONMAX - l_pan) / (double)SEPARATIONMAX);
		EqDoParamChange();

		g_loadedcurrent = 0;
	}

	if(l_runloop == maLOADING)
	{
		l_runloop = maPLAYING;

		g_state.stop &= !STOP_DECODER;
		g_state.silence = 0;

		if(l_loadednew) // gets set by Next, Previous, listbox doubleclick, index prompt
		{
			g_loadedcurrent = l_loadednew - 1;
			l_loadednew = 0;
			l_previous = 0;
			l_next = 0;
		}
		
		fssprintf(playpath, MAX_PATH, fsS, g_loadedpath[g_loadedcurrent]);		
		
		if(l_psongrec[l_songsindex[g_loadedcurrent]] <= 0)
		{
			l_songsnumplayed++;
			l_psongrec[l_songsindex[g_loadedcurrent]] += 1; // mark song as played					
		}
		
		l_possliderchanged = l_posslidervalue = 0;
		SetPosSlider(-1);

		g_effectschange = AnyEffectOn();

		// reset limiter
		g_state.mpegattenuation = 1.0;

		// output file
		if(g_sets.output != kOUTSOUND)
		{
			// output file name
			if(MakeOutputFileName())
				return 1; // stops
		}

		if(LaunchPlayThread(playpath))
		{
			masprintf(mastr, MAXLOGLINELEN, L"RunPlay LaunchPlayThread failed '%ls'", fs2ma(mastr2, playpath));
			l_runloop = maSCANNING;
			goto runplayret1;
		}

		// now playing g_loadedcurrent
		l_songcurrent = l_songsindex[g_loadedcurrent];
		
		// show new song info		
		l_noshowrow = 0;

		if(!g_state.running || g_sets.titleinfo != 0) // don't repeat songlist name on song changes
			ResetAppTitle();

		l_titletime = INT_MAX;

		if(g_sets.mawin[WINDOWLIST].ison) 
		{
			if(g_sets.listboxmode & (LBOXSONGSLOADED | LBOXPLAYLIST))
			{
				if(!l_noshowrow) // scrolling up or down the list sets this
					ListWindowShowRowPlaying();
			}
			else
				UpdateListWindowList(0);
		}
								
		if(!g_posbegon)
		{
			g_posbeg = 0;
			StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEBEG, ZEROMSSTR);
		}
		
		l_playerseeking = 0;
		g_loadedquit = g_loadedquitB = 0; // gets set by Next, Previous, listbox doubleclick, index prompt

#if !defined(NODECODEAUDIOIN)
		if(g_state.infiletype == kMAMAA)
		{
			l_stTime = time(NULL);
			StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEBEG, NOTIMESTR);
			StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEEND, NOTIMESTR);
		}
#endif

	} // if(l_runloop == maLOADING) on entry, but is now maPLAYING

	// playing the song		 
	if(l_runloop == maPLAYING)
	{
		if(g_state.stop & STOP_ALL) // immediate stop
		{
			l_runloop = maLOADING;
			ReEnableButtonsMenusByFileType();
			goto runplayret2;
		}

		if(!g_state.pdecoder) // song is finished
			l_runloop = maLOADING;
		else
		{
			// seek	to posbeg or end of song		
			if(!g_state.paused)
			{
				if(g_posbegon && g_state.position <= (mauint64)g_posbeg)
					PlayerSeek(g_posbeg);
				else if(g_posendon && g_state.position > (mauint64)g_posend)
				{
					if(g_reptsong)
						PlayerSeek(g_posbeg);
					else
						PlayerSeek(g_state.duration);
				}
				else
					l_playerseeking = 0;
			
				// refresh the display
				if(l_titletime++ >= (g_posendon ? SEEKTIME : TITLETIME))
				{
					l_titletime = 0;

#if !defined(NODECODEAUDIOIN)
					if(g_state.infiletype == kMAMAA)
					{
						stTime = time(NULL) - l_stTime;
						tmTime = gmtime((time_t *)&stTime);
						masprintf(mastr, MAXLOGLINELEN, L"%2.2d:%2.2d:%2.2d", tmTime->tm_hour, tmTime->tm_min, tmTime->tm_sec);
						mastr[8] = 0;	
						StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEPOS, mastr);
					}
					else
#endif
					{
						if(!g_posendon && l_posend != (mauint64)g_posend)
						{
							l_posend = g_posend;
							MillisecondsToStrHMSD(mastr, g_posend);
							StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEEND, mastr);
						}
				
						if(l_possliderchanged) // user moved the pos slider, seek to new pos 
						{
							l_possliderchanged = 0;
							if(!g_state.paused)
								PlayerSeek((int)(LROUND((g_state.duration * (double)l_posslidervalue) / POSMAX)));
						}
						else if(!g_state.paused) // update position slider
						{
							SetPosSlider((int)(LROUND(((g_state.position * (double)POSMAX) / g_state.duration))));
							if(g_state.loadingsbsms)
							{
								if(g_state.loadingsbsms == 1)
								{
									g_state.loadingsbsms = 3; // set title only once
									SetAppWindowTitle(L"loading sbsms...");
								}
								else if(g_state.loadingsbsms == 2)
								{
									g_state.loadingsbsms = 0;
									ResetAppTitle();
								}
							}
						}

						l_playerseeking = 0;
				
						// update display of song duration				
						if(g_sets.titleinfo == 2)
							SetAppTitle2();
#if !defined(MAGUINONE)
						else if(l_temptitle) // clear temp title info (key pressed or volume)
						{
							if(--l_temptitle <= 0)
								ResetAppTitle();
						}
#endif
			
#ifndef NODECODEMP3
						// update clipping				
						if(g_sets.limiter == (PREAMP | CLIPPING) &&
							(g_state.infiletype == kMAMP3 || g_state.infiletype == kMAMP2 || g_state.infiletype == kMAMP1))
						{
							if(l_attenuation != g_state.mpegattenuation)
							{
								mpegattenuationdb = (g_state.mpegattenuation == 1.0) ? 0.0f : (float)(20.0 * log10(g_state.mpegattenuation));
								masprintf(mastr, MAXLOGLINELEN, L"%.1f", mpegattenuationdb);
								CheckBoxSetDescriptor(WINDOWAPP, ID_CHK_LIMITER, mastr);
								l_attenuation = g_state.mpegattenuation;
								// sudden attenuation more than about 20dB is likely due to a spike from bad encoding, cancel it
								if((mpegattenuationdb + SliderDBFromPos(g_sets.preampslider)) < -20.0)
									g_state.mpegattenuation = MP3ATTENUATIONFLAG; // flag for maPlayMp3() to reset attenuation
							}
						}
#endif
					}

					// an io stream has been reopened from the decoder thread 
					if(g_state.iostreamopened)
					{
						g_state.iostreamopened = 0;
						if(g_w[WINDOWLIST] && g_sets.listboxmode & LBOXPLAYERINFO)
							UpdateListWindowList(0);
					}

				} // l_titletime

				if(g_loadedquit || g_state.stop)
					l_runloop = maLOADING;
			}
			else // paused
				if(l_titletime++ >= 1)
					l_titletime = 0;
	
		} // !paused

	} // if(l_runloop == maPLAYING)

	if(l_runloop == maLOADING) // after looping through l_runloop == maPLAYING
	{
		// close		
		if(g_loadedquit)
			g_state.stop |= STOP_DECODER;
		else
			g_state.stop |= STOP_REQUEST;

		// wait for decoder thread to finish
		WaitDecoderStopped();

		ReEnableButtonsMenusByFileType();
		
		if(g_state.stop & STOP_ALL)
			l_runloop = maSCANNING;
		else if(!g_loadedquitB && g_sets.singleplay)
		{
			DoStop();
			l_singleplay = 1;
			l_runloop = maSCANNING;
		}
		else
			if(!l_loadednew && ++g_loadedcurrent >= g_loadednum)
				l_runloop = maSCANNING;
	} // if(l_runloop == maLOADING)
					
	if(l_runloop == maSCANNING)
	{
		goto runplayret3;	
	
	// error returns			
runplayret1:
		maLog(mastr);
		return 0;

runplayret2:
		l_retval = 1;
	
		// all returns	
runplayret3:
		// close
		g_state.stop |= STOP_REQUEST;

		// wait for decoder thread to finish
		WaitDecoderStopped();

	} // if(l_runloop == maSCANNING)
	
	return l_retval;
		
} // RunPlay


int GetInFiletype(fschar *playpath)
{
	int i;
	
	for(i = kMAFIRSTPLAYBLE; i < kMALAST; i++)
		if(FileNameHasExtension(playpath, g_extstr[i]))
			return i;
			
#ifndef MANOCDA
	if(IsAudioCd(playpath))
		return kMACDA;
#endif
			
	return kMALAST; // not playable

} // GetInFiletype


#if !defined(NODECODEAUDIOIN)
 char l_framein[PAMAXFRAMESIN * PABYTESPERFRAME];
 void maPlayAudioIn()
 {
	unsigned long bytes;

	if(g_ppafifoin == NULL)
	{
		maLog(L"maPlayAudioIn null g_ppafifoin");
		return;
	}

	// format of this decoder output
 #ifdef WORDS_BIGENDIAN
	g_state.inwordsbigendian = 1;
 #else
	g_state.inwordsbigendian = 0;
 #endif

	if(maIOStart() < 0)
	{
		maLog(L"maPlayAudioIn maIOStart failed");
		return;
	}
	
	memset(l_framein, 0, PAMAXFRAMESIN * PABYTESPERFRAME);

	for (; ;)
	{
		if(g_state.stop & (STOP_ALL | STOP_DECODER))
			break;

		if((bytes = maFifoGetFilledSpace(g_ppafifoin)) > 0)
		{
			maFifoDraw(g_ppafifoin, l_framein, bytes);

			if(maOutChain1((char *)l_framein, bytes) != 0) // error, can't continue
			{
				g_state.stop |= STOP_ALL;
				break;
			}
		}

		MAFIFOSEMWAIT(g_ppafifoin->semaphore);
	}

	maOutChainFlush();

 } // maPlayAudioIn
#endif


int LaunchPlayThread(fschar *playpath)
{
	int err;
	int infiletype;
	machar mastr[MAXLOGLINELEN];

	if((infiletype = GetInFiletype(playpath)) >= kMALAST)
		return -1; // not playable

	fsstrcpy(g_state.inpath, playpath);
	g_state.infiletype = infiletype;
	g_state.nomem = (g_sets.buffertype == BUFUSEMEM) ? 0 : 1;

	switch(infiletype)
	{
#ifndef NODECODEMP3
		case kMAMP3:
		case kMAMP2:
		case kMAMP1:
			g_state.pdecoder = (threadparametertype)maPlayMp3;
			g_state.pcmmax = 12;
		break;
#endif
		
#ifndef NODECODEOGG
		case kMAOGG:
			g_state.pdecoder = (threadparametertype)maPlayOgg;
			g_state.pcmmax = 12;
		break;
#endif

#ifndef NODECODEWMA
		case kMAWMA:
			g_state.pdecoder = (threadparametertype)maPlayWma;
			g_state.pcmmax = 12;
		break;
#endif

#ifndef NODECODEAAC
		case kMAAAC:
		case kMAM2A:
			g_state.pdecoder = (threadparametertype)maPlayAac;
			g_state.pcmmax = 12;
		break;
#endif

#ifndef NODECODEMP4
		case kMAM4A:
		case kMAMP4:
			g_state.pdecoder = (threadparametertype)maPlayMp4;
			g_state.pcmmax = 12;
		break;
#endif

#ifndef NODECODEFLAC
		case kMAFLAC:
		case kMAFLA:
			g_state.pdecoder = (threadparametertype)maPlayFlac;
			g_state.pcmmax = 8;
		break;
#endif

#if !defined(NODECODEAUDIOIN)
		case kMAMAA:
			// DoSinglePlay(1);
			g_state.pdecoder = (threadparametertype)maPlayAudioIn;
			g_state.pcmmax = 0;
		break;
#endif

#ifndef NODECODEAIF
		case kMAAIF:
		case kMAAIFF:
		case kMARAW:		
		/* these could be added to maPlayAif
		case kMAAU:
		case kMAPAF:
		case kMASVX:
		case kMANIST:
		case kMAVOC:
		case kMAIRCAM:
		case kMAW64:
		case kMAMAT4:
		case kMAMAT5:
		case kMAPVF:
		case kMAXI:
		case kMAHTK:
		*/
			g_state.pdecoder = (threadparametertype)maPlayAif;
			g_state.pcmmax = 1;
		break;
#endif

#ifndef NODECODEWAV
		case kMAWAV:
			g_state.pdecoder = (threadparametertype)maPlayAif;
			g_state.pcmmax = 1;
		break;
#endif

#ifndef MANOCDA
		case kMACDA:
			g_state.nomem = 1; // always for cda
			g_state.pdecoder = (threadparametertype)maPlayCda;
			g_state.pcmmax = 0;
		break;
#endif

		default:
			g_state.pdecoder = NULL;
			g_state.pcmmax = 0;
		return 1;

	} // case

#if defined(MAPLAYSBSMS)
	if(AnyEffectOn() == EFFECTSSBSMS && g_state.infiletype != kMAMAA && g_state.infiletype != kMACDA)
	{
		if((err = maThreadCreate((void *(*)(void *))PlayThread, (threadparametertype)maPlaySbsms)) != 0)
		{
			masprintf(mastr, MAXLOGLINELEN, L"LaunchPlayThread maThreadCreate maPlaySbsms error %d", err);
			maLog(mastr);
			return err;
		}
		DisableButtonsMenusByFileType(kMAWMA); // any filetype that doesn't support seek
	}
	else
#endif
	{
		if((err = maThreadCreate((void *(*)(void *))PlayThread, g_state.pdecoder)) != 0)
		{
			masprintf(mastr, MAXLOGLINELEN, L"LaunchPlayThread maThreadCreate error %d", err);
			maLog(mastr);
			return err;
		}
		DisableButtonsMenusByFileType(g_state.infiletype);
	}

	return 0;

} // LaunchPlayThread


void PlayThread(threadparametertype parameter)
{
	parameter();

	g_state.pdecoder = NULL;

} // PlayThread


int MadDoParamChange()
{	
#ifndef NODECODEMP3
	// mad equalizer as preamp
	mp3eqset(g_sets.limiter, SliderDBFromPos(g_sets.preampslider));
#endif
		
	return 0;
					
} // MadDoParamChange


int EqDoParamChange()
{	
	int i;
	machar mastr[MAXLOGLINELEN];
		
	if(g_sets.eqpreset < 0 || g_sets.eqpreset >= EQNUMPRESETS)
	{
		masprintf(mastr, MAXLOGLINELEN, L"EqDoParamChange eqpreset=%d", g_sets.eqpreset + 1);
		maLog(mastr);
		return 1;
	}

	if(g_sets.eqflags & MAEQACTIVE)
	{
		for(i = 0; i < EQBANDS31; i++)
		{
			g_EqualizerSettings.Levels[i] = g_sets.eqpresets[g_sets.eqpreset].eqslider[i];
			g_EqualizerSettings.Levels2[i] = g_sets.eq2presets[g_sets.eqpreset].eqslider[i];
		}
		maEqualizerSetLevels(g_sets.eqflags & MAEQ31BANDS, g_sets.eqpresets[g_sets.eqpreset].eqoffset);
		g_EqualizerSettings.Enabled = 1;
	}
	else
		g_EqualizerSettings.Enabled = 0;

	return 0;
					
} // EqDoParamChange


#if defined(__MAWIN__)
 int GetSongMem() // win
 {
	size_t request;
	MEMORYSTATUS memstatus; // link with /LARGEADDRESSAWARE
	machar mastr[MAXLOGLINELEN];

	// free current song memory
	if(g_memhandle)
	{
		MAFREE(g_memhandle);
		g_memhandle = NULL; 
		l_songmem = NULL;
		l_songmemend = NULL;
		l_songmempos = NULL;
		g_loadedmemsize = 0;
	}

	l_buffersize = g_sets.buffersize;

	// find largest contiguous block
	GlobalMemoryStatus(&memstatus);

	if((request = (size_t)l_buffersize * MEGABYTE) > (size_t)memstatus.dwAvailPhys)
	{
		masprintf(mastr, MAXLOGLINELEN, L"GetSongMem requested %d MB (%u bytes), only %u bytes are available",
			l_buffersize, (size_t)l_buffersize * MEGABYTE, (size_t)memstatus.dwAvailPhys);
		maLog(mastr);
	}
		
	l_songmem = g_memhandle = (char *)MAMALLOC(request);
	
	if(g_memhandle == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"GetSongMem MAMALLOC failed for %u MB (%u bytes)", request / MEGABYTE, request);
		maLog(mastr);
		return 1;	
	}
	
	g_loadedmemsize = request;
	l_songmempos = l_songmem;
	l_songmemend = l_songmem + request;
	
	return 0;
	
 } // GetSongMem
#else
 int GetSongMem() // unix and mac
 {
	size_t request;
	machar mastr[MAXLOGLINELEN];

	// free current song memory
	if(g_memhandle)
	{
		MAFREE(g_memhandle);
		g_memhandle = NULL; 
		l_songmem = NULL;
		l_songmemend = NULL;
		l_songmempos = NULL;
		g_loadedmemsize = 0;
	}
	
	l_buffersize = g_sets.buffersize;
	
	request = l_buffersize * MEGABYTE;
	
	l_songmem = g_memhandle = (char *)MAMALLOC(request);
	
	if(g_memhandle == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"GetSongMem MAMALLOC failed for %d MB (%lu bytes)", l_buffersize, (unsigned long)request);
		maLog(mastr);
		return 1;	
	}
	
	g_loadedmemsize = request;
	l_songmempos = l_songmem;
	l_songmemend = l_songmem + request;
	
	return 0;
	
 } // GetSongMem
#endif


int SetSettingsPaths()
{
	fschar name[MAX_PATH];
	size_t len;
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];
	
	if(g_playlistpath[0] == fsT('\0'))
		return 1;
	
	if(fsstrlen(g_playlistpath) >= MAX_PATH)
	{
		masprintf(mastr, MAXLOGLINELEN, L"SetSettingsPaths g_playlistpath > max %d : %zu", MAX_PATH - 1, fsstrlen(g_playlistpath));
		maLog(mastr);
		fsstrncpy(name, g_playlistpath, MAX_PATH - 1);
		name[MAX_PATH - 1] = fsT('\0');
		maLog(fs2ma(mastr2, name));
		g_settingspath[0] = fsT('\0');
		return 1;
	}

	if(FilenameFromPathname(g_playlistpath, name))
	{	
		masprintf(mastr, MAXLOGLINELEN, L"SetSettingsPaths failed to get filename from '%ls'", fs2ma(mastr2, g_playlistpath));
		maLog(mastr);
		g_settingspath[0] = fsT('\0');
		return 1;
	}
					
	FileNameRemoveExtension(name);

	if((len = fsstrlen(g_settingsdir) + sizeof(FILESEP) + fsstrlen(name) + fsstrlen(EXTMAS)) >= MAX_PATH)
	{
		masprintf(mastr, MAXLOGLINELEN, L"SetSettingsPaths g_settingspath > max %d : %zu", MAX_PATH - 1, len);
		maLog(mastr);
		if(len < MAXLOGLINELEN - 1)
		{
			masprintf(mastr, MAXLOGLINELEN, L"%ls%lc%ls%ls", g_settingsdir, FILESEP, name, EXTMAS);
			maLog(mastr);
		}
		
		g_settingspath[0] = fsT('\0');
		return 1;
	}
	
	fssprintf(g_settingspath, MAX_PATH, fsS fsC fsS fsS, g_settingsdir, FILESEP, name, EXTMAS);

	return 0;
		
} // SetSettingsPaths


int SetRecordsPaths()
{
	fschar name[MAX_PATH];
	size_t len;
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];
	
	if(g_playlistpath[0] == fsT('\0'))
		return 1;
	
	if(fsstrlen(g_playlistpath) >= MAX_PATH)
	{
		masprintf(mastr, MAXLOGLINELEN, L"SetRecordsPaths g_playlistpath > max %d : %zu", MAX_PATH - 1, fsstrlen(g_playlistpath));
		maLog(mastr);
		fsstrncpy(name, g_playlistpath, MAX_PATH - 1);
		name[MAX_PATH - 1] = fsT('\0');
		maLog(fs2ma(mastr2, name));
		g_recordspath[0] = fsT('\0');
		return 1;
	}

	if(FilenameFromPathname(g_playlistpath, name))
	{	
		masprintf(mastr, MAXLOGLINELEN, L"SetRecordsPaths failed to get filename from '%ls'",
																		fs2ma(mastr2, g_playlistpath));
		maLog(mastr);
		g_recordspath[0] = fsT('\0');
		return 1;
	}
					
	FileNameRemoveExtension(name);

	if((len = fsstrlen(g_recordsdir) + sizeof(FILESEP) + fsstrlen(name) + fsstrlen(EXTMAR)) >= MAX_PATH)
	{
		masprintf(mastr, MAXLOGLINELEN, L"SetRecordsPaths g_recordspath > max %d : %zu", MAX_PATH - 1, len);
		maLog(mastr);
		if(len < MAXLOGLINELEN - 1)
		{
			masprintf(mastr, MAXLOGLINELEN, L"%ls%lc%ls%ls", g_recordsdir, FILESEP, name, EXTMAS);
			maLog(mastr);
		}
		
		g_recordspath[0] = fsT('\0');
		return 1;
	}
	
	fssprintf(g_recordspath, MAX_PATH, fsS fsC fsS fsS, g_recordsdir, FILESEP, name, EXTMAR);

	return 0;
		
} // SetRecordsPaths


int GetRecentPlaylistName()
{
	size_t ret0;
	machar *p0;
	machar *p4;
	machar playlist[MAX_PATH];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
#endif
	machar mastr[MAXLOGLINELEN];
	
	p0 = NULL;
	ret0 = mafileread(&p0, l_recentlist);
	if(ret0 <= 0 || p0 == NULL)
	{			
		if(p0 != NULL)
			MAFREE(p0);
		maLog(L"GetRecentPlaylistName mafileread failed");
		return 1;
	}
	
	playlist[0] = L'\0';
	g_playlistpath[0] = fsT('\0');
	
	p4 = p0;
	memgets(playlist, MAX_PATH, p4);
	MAFREE(p0);
	
	if(playlist[0])
		ma2fsfn(g_playlistpath, playlist);
	if(fsstrlen(g_playlistpath) < 1)
	{
		g_playlistpath[0] = fsT('\0');
		masprintf(mastr, MAXLOGLINELEN, L"GetRecentPlaylistName memgets '%ls'", fs2ma(mastr2, l_recentlist));
		maLog(mastr);	
		return 2;
	}

	return 0;
	
} // GetRecentPlaylistName
	

int UpdateRecent(fschar *item, int del)
{
	int i;
	fschar litem[MAX_PATH];
	fschar items[MAXOPENRECENT][MAX_PATH];
	FILE *fp;
	machar mastr2[MAXLOGLINELEN];
#if !defined(__MAWIN__)
	machar mastr3[MAXLOGLINELEN];
#endif
	machar mastr[MAXLOGLINELEN];
	
	// read the file into the array
	if(GetRecent(items))
		return 1;
		
	// update the array
	if(del) // delete item
	{
		if(items[0][0] == fsT('\0')) // empty list
			return 0;
		fsstrcpy(litem, item);
		for(i = 0; i < MAXOPENRECENT; i++)
			if(!fsstrcmp(litem, items[i]))
				break; // item to delete is at index i
		if(i < MAXOPENRECENT)
		{
			for(; i < MAXOPENRECENT - 1; i++) // shift items up one
			{
				if(items[i + 1][0] == fsT('\0')) // end of list
				{
					items[i][0] = fsT('\0');
					break;
				}
				fsstrcpy(items[i], items[i + 1]);
			}
		}
	}
	else // add item
	{
		fsstrcpy(litem, item);
		for(i = 0; i < MAXOPENRECENT; i++)
			if(!fsstrcmp(litem, items[i]))
				break; // already have the current playlist at index i
		if(i == 0) // the current playlist is already at the top of the array
			return 0;
		if(i == MAXOPENRECENT)
			i--;
		for(; i > 0; i--) // shift items down one
			fsstrcpy(items[i], items[i - 1]);
		fsstrcpy(items[0], litem);
	}
	
	// rewrite the array to the file
	if((fp = mafopen(l_recentlist, FSWBMODE)) == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"UpdateRecent mafopen '%ls'", fs2ma(mastr2, l_recentlist));
		maLog(mastr);
		return 1;
	}
	
	// BOM
	fwrite("\xef\xbb\xbf", 3, 1, fp); // BOM_UTF-8

	for(i = 0; i < MAXOPENRECENT; i++)
	{
		if(!items[i][0])
			break;
		masprintf(mastr2, MAXLOGLINELEN, L"%ls", fs2ma(mastr3, items[i]));
		mastrcat(mastr2, g_maeol);
		if(mafilewrite(0, mastr2, fp) == 0)
		{
			masprintf(mastr, MAXLOGLINELEN, L"UpdateRecent mafilewrite failed '%ls' failed", fs2ma(mastr2, items[i]));
			maLog(mastr);
		}
	}
	
	fclose(fp);
		
#if defined(__MAOSX__) || defined(__MAUNIX__)
	machmod(l_recentlist, ACCESSPERMS); // 0777
#endif

	UpdateSubMenusRecent();

	return 0;
	
} // UpdateRecent


int GetRecent(fschar items[][MAX_PATH])
{
	size_t ret0;
	machar *p0;
	machar *p4;
	machar *p5;
	int i, j;
	fschar fpath[MAX_PATH];
	machar playlist[MAX_PATH];
	
	// clear the array	
	for(i = 0; i < MAXOPENRECENT; i++)
		items[i][0] = fsT('\0');

	// read the file into the array
	if(maFileExists(l_recentlist))
	{
		p0 = NULL;
		ret0 = mafileread(&p0, l_recentlist);
		if(ret0 <= 0 || p0 == NULL)
		{			
			if(p0 != NULL)
				MAFREE(p0);
			maLog(L"GetRecent mafileread failed");
			return 1;
		}
	
		p4 = p0;
		for(i = 0, j = 0; i < MAXOPENRECENT; i++)
		{
			p5 = memgets(playlist, MAX_PATH, p4);
			if(p5 == NULL) // should never eof
				break;
			p4 = p5;
			if(playlist[0])
			{
				ma2fsfn(fpath, playlist);
				if(maFileExists(fpath))
					fsstrcpy(items[j++], fpath);
			}
		}
		MAFREE(p0);
	}

	return 0;

} // GetRecent


void FileOutWindowDisable()
{
	if(!g_w[WINDOWFILEOUT])
		return;
		
	PushButtonDisable(WINDOWFILEOUT, ID_BTN_FILEOUTDIR);

#if defined(ENCODEMP3) || defined(ENCODEOGG) || defined(ENCODEAAC) || defined(ENCODEMP4) 

	switch(g_sets.output)
	{
		
 #ifdef ENCODEMP3
		case kOUTMP3:
			PopupButtonDisable(WINDOWFILEOUT, ID_POPUP_QUALITY);
			PopupButtonDisable(WINDOWFILEOUT, ID_POPUP_BITRATE);
			PopupButtonDisable(WINDOWFILEOUT, ID_POPUP_MODE);
			PopupButtonDisable(WINDOWFILEOUT, ID_POPUP_SAMPLERATE);
		break;
 #endif
 
 #ifdef ENCODEOGG
		case kOUTOGG:
			// PopupButtonDisable(WINDOWFILEOUT, ID_POPUP_QUALITY); // not used
			PopupButtonDisable(WINDOWFILEOUT, ID_POPUP_BITRATE);
			PopupButtonDisable(WINDOWFILEOUT, ID_POPUP_MODE);
		break;
 #endif
 
 #ifdef ENCODEAAC
		case kOUTAAC:
			PopupButtonDisable(WINDOWFILEOUT, ID_POPUP_QUALITY);
			CheckBoxDisable(WINDOWFILEOUT, ID_CHK_ADTS);
		break;
 #endif
 
 #ifdef ENCODEMP4
		case kOUTMP4:
			PopupButtonDisable(WINDOWFILEOUT, ID_POPUP_QUALITY);
			CheckBoxDisable(WINDOWFILEOUT, ID_CHK_M4A);
		break;
 #endif
 
		default:
		break;
	} // switch
		
#endif

} // FileOutWindowDisable


void FileOutWindowEnable()
{
	if(!g_w[WINDOWFILEOUT])
		return;
		
	PushButtonEnable(WINDOWFILEOUT, ID_BTN_FILEOUTDIR);

#if defined(ENCODEMP3) || defined(ENCODEOGG) || defined(ENCODEAAC) || defined(ENCODEMP4) 

	switch(g_sets.output)
	{
 #ifdef ENCODEMP3
		case kOUTMP3:
			PopupButtonEnable(WINDOWFILEOUT, ID_POPUP_QUALITY);
			PopupButtonEnable(WINDOWFILEOUT, ID_POPUP_BITRATE);
			PopupButtonEnable(WINDOWFILEOUT, ID_POPUP_MODE);
			PopupButtonEnable(WINDOWFILEOUT, ID_POPUP_SAMPLERATE);
		break;
 #endif
 
 #ifdef ENCODEOGG
		case kOUTOGG:
			// PopupButtonEnable(WINDOWFILEOUT, ID_POPUP_QUALITY); // not used
			PopupButtonEnable(WINDOWFILEOUT, ID_POPUP_BITRATE);
			PopupButtonEnable(WINDOWFILEOUT, ID_POPUP_MODE);
		break;
 #endif
 
 #ifdef ENCODEAAC
		case kOUTAAC:
			PopupButtonEnable(WINDOWFILEOUT, ID_POPUP_QUALITY);
			CheckBoxEnable(WINDOWFILEOUT, ID_CHK_ADTS);
		break;
 #endif
 
 #ifdef ENCODEMP4
		case kOUTMP4:
			PopupButtonEnable(WINDOWFILEOUT, ID_POPUP_QUALITY);
			CheckBoxEnable(WINDOWFILEOUT, ID_CHK_M4A);
		break;
 #endif
 
		default:
		break;
	} // switch
		
#endif

} // FileOutWindowEnable


void DisableButtonsMenus()
{
	if(g_state.running)
		return;
		
	g_state.stop &= !STOP_ALL;
	l_logoff = g_sets.logflags & kMALOGOFF;
	l_logcount = 0;
	
	PushButtonDisable(WINDOWAPP, ID_BTN_PLAY);			// Play
	MenuDisable(MENUMACONTROLS, ID_MENU_PLAY);			// Play
#if defined(MANOCDA)
	MenuDisable(MENUMACONTROLS, ID_MENU_PLAYCD);		// Play CD
	MenuDisable(MENUMACONTROLS, ID_MENU_EJECTCD);		// Eject CD
#endif
#if defined(NODECODEAUDIOIN)
	MenuDisable(MENUMACONTROLS, ID_MENU_AUDIOIN);		// Audio In mode
#endif
	MenuDisable(MENUMASETS, ID_MENU_SETVDISK);			// Set Vdisk
#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
	MenuDisable(MENUMASETS, ID_MENU_SETAUDIOCD);		// Set Audio CD	
#endif
	MenuDisable(MENUMASETS, ID_MENU_SETFACTORY);		// Factory settings
	MenuDisable(MENUMASETS, ID_MENU_SETDEFAULT);		// Default settings
	MenuDisable(MENUMASETS, ID_MENU_SETSAVED);			// Saved settings
	MenuDisable(MENUMASETS, ID_MENU_SETCOPY);			// Copy settings from another playlist
	MenuDisable(MENUFILES, ID_MENU_MAKELIST);			// Make List
	MenuDisable(MENUFILES, ID_MENU_SAVELISTAS);			// Save List As
	PopupButtonDisable(WINDOWAPP, ID_POPUP_OUTPUT);		// Output
	PopupButtonDisable(WINDOWAPP, ID_POPUP_BUFFER);		// Mem/VDisk/Direct
	MenuDisable(MENUTOOLS, ID_MENU_RESCANAUDIODEVICES);
	PopupButtonDisable(WINDOWAPP, ID_POPUP_AUDIOOUTPUT);
	PopupButtonDisable(WINDOWAPP, ID_POPUP_AUDIOINPUT);
	PopupButtonDisable(WINDOWAPP, ID_POPUP_AUDIOHOSTIN);
	PopupButtonDisable(WINDOWAPP, ID_POPUP_AUDIOHOSTOUT);
	CheckBoxDisable(WINDOWAPP, ID_CHK_HOSTINENABLE);
	if(g_w[WINDOWADVANCED])
		CheckBoxDisable(WINDOWADVANCED, ID_CHK_CLOSEREOPEN);
	FileOutWindowDisable();
	
} // DisableButtonsMenus


void DisableButtonsMenusByFileType(int filetype)
{	
#if !defined(NODECODEAUDIOIN) 
	if(filetype == kMAMAA)
	{
		PushButtonDisable(WINDOWAPP, ID_BTN_PREVIOUS);
		PushButtonDisable(WINDOWAPP, ID_BTN_NEXT);
		MenuDisable(MENUMACONTROLS, ID_MENU_PREVIOUS);
		MenuDisable(MENUMACONTROLS, ID_MENU_NEXT);
		CheckBoxDisable(WINDOWAPP, ID_CHK_SINGLEPLAY);
		CheckBoxDisable(WINDOWAPP, ID_CHK_REPTSONG);
		CheckBoxDisable(WINDOWAPP, ID_CHK_BEG);
		CheckBoxDisable(WINDOWAPP, ID_CHK_END);
	}
	else
#endif
	if(filetype == kMAWMA || filetype == kMAAAC)
	{
		l_posbegbu = g_posbeg;
		l_posendbu = g_posend;
		g_posbegon = 0;
		g_posendon = 0;
		CheckBoxDisable(WINDOWAPP, ID_CHK_BEG);
		CheckBoxDisable(WINDOWAPP, ID_CHK_END);
		CheckBoxSetValue(WINDOWAPP, ID_CHK_BEG, 0);
		CheckBoxSetValue(WINDOWAPP, ID_CHK_END, 0);
	}

} // DisableButtonsMenusByFileType


void ReEnableButtonsMenusByFileType()
{
	machar lstr[32];

	if(g_state.stop & STOP_APPQUIT)
		return;

#if !defined(NODECODEAUDIOIN) 
	PushButtonEnable(WINDOWAPP, ID_BTN_PREVIOUS);
	PushButtonEnable(WINDOWAPP, ID_BTN_NEXT);
	MenuEnable(MENUMACONTROLS, ID_MENU_PREVIOUS);
	MenuEnable(MENUMACONTROLS, ID_MENU_NEXT);
	CheckBoxEnable(WINDOWAPP, ID_CHK_SINGLEPLAY);
	CheckBoxEnable(WINDOWAPP, ID_CHK_REPTSONG);
#endif
	CheckBoxEnable(WINDOWAPP, ID_CHK_BEG);
	CheckBoxEnable(WINDOWAPP, ID_CHK_END);
	g_posbegon = l_posbegonbu;
	g_posendon = l_posendonbu;
	CheckBoxSetValue(WINDOWAPP, ID_CHK_BEG, g_posbegon);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_END, g_posendon);
	if(l_posbegbu != MAUINT64MAX)
	{
		g_posbeg = l_posbegbu;
		MillisecondsToStrHMSD(lstr, g_posbeg);
		StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEBEG, lstr);
		l_posbegbu = MAUINT64MAX;
	}
	if(l_posendbu != MAUINT64MAX)
	{
		g_posend = l_posendbu;
		MillisecondsToStrHMSD(lstr, g_posend);
		StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEEND, lstr);
		l_posendbu = MAUINT64MAX;
	}

} // ReEnableButtonsMenusByFileType


void EnableButtonsMenus()
{	
	if(g_state.running || g_state.stop & STOP_APPQUIT)
		return;
		
	PushButtonEnable(WINDOWAPP, ID_BTN_PLAY);			// Play
	MenuEnable(MENUMACONTROLS, ID_MENU_PLAY);			// Play
#if !defined(MANOCDA)
	MenuEnable(MENUMACONTROLS, ID_MENU_PLAYCD);			// Play CD
	MenuEnable(MENUMACONTROLS, ID_MENU_EJECTCD);		// Eject CD
#endif
	MenuEnable(MENUMASETS, ID_MENU_SETVDISK);			// Set Vdisk
#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
	MenuEnable(MENUMASETS, ID_MENU_SETAUDIOCD);			// Set Audio CD	
#endif
	MenuEnable(MENUMASETS, ID_MENU_SETFACTORY);			// Factory settings
	MenuEnable(MENUMASETS, ID_MENU_SETDEFAULT);			// Default settings
	MenuEnable(MENUMASETS, ID_MENU_SETSAVED);			// Saved settings
	MenuEnable(MENUMASETS, ID_MENU_SETCOPY);			// Copy settings from another playlist
	MenuEnable(MENUFILES, ID_MENU_MAKELIST);			// Make List
	MenuEnable(MENUFILES, ID_MENU_SAVELISTAS);			// Save List As
	PopupButtonEnable(WINDOWAPP, ID_POPUP_OUTPUT);		// Output
	PopupButtonEnable(WINDOWAPP, ID_POPUP_BUFFER);		// Mem/VDisk/Direct

	PushButtonEnable(WINDOWAPP, ID_BTN_PREVIOUS);
	PushButtonEnable(WINDOWAPP, ID_BTN_NEXT);
	MenuEnable(MENUMACONTROLS, ID_MENU_PREVIOUS);
	MenuEnable(MENUMACONTROLS, ID_MENU_NEXT);
	CheckBoxEnable(WINDOWAPP, ID_CHK_SINGLEPLAY);

	MenuEnable(MENUTOOLS, ID_MENU_RESCANAUDIODEVICES);

	PopupButtonEnable(WINDOWAPP, ID_POPUP_AUDIOOUTPUT);
	PopupButtonEnable(WINDOWAPP, ID_POPUP_AUDIOINPUT);
	PopupButtonEnable(WINDOWAPP, ID_POPUP_AUDIOHOSTIN);
	PopupButtonEnable(WINDOWAPP, ID_POPUP_AUDIOHOSTOUT);

#if !defined(NODECODEAUDIOIN)
	CheckBoxEnable(WINDOWAPP, ID_CHK_HOSTINENABLE);
#endif
	if(g_w[WINDOWADVANCED])
		CheckBoxEnable(WINDOWADVANCED, ID_CHK_CLOSEREOPEN);

	ReEnableButtonsMenusByFileType();

	FileOutWindowEnable();

} // EnableButtonsMenus


void SetLogMenus()
{	
	if(g_w[WINDOWAPP])
	{
		if(l_havelog)
		{
#if !defined(NOTEXTVIEWER)
			MenuEnable(MENUTOOLS, ID_MENU_VIEWLOG);
#endif
			MenuEnable(MENUTOOLS, ID_MENU_CLEARLOG);
		}
		else
		{
#if !defined(NOTEXTVIEWER)
			MenuDisable(MENUTOOLS, ID_MENU_VIEWLOG);
#endif
			MenuDisable(MENUTOOLS, ID_MENU_CLEARLOG);
		}
	}
	
} // SetLogMenus


void DoSave()
{
	if(g_playlistpath[0] == fsT('\0'))
	{
		DoAsDefaults();
		return;
	}
	
	DisableButtonsMenus();

	memcpy(&g_setsbu, &g_sets, sizeof(MASETS));
	if(!SaveSettings() && !SaveRecords())
		if(!g_state.running)
			SetAppWindowTitle(L"Settings saved");

	EnableButtonsMenus();
	
} // DoSave


void DoEjectCD()
{
#if !defined(MANOCDA)
	int i;

	if(g_state.running && !fsstrcmp(g_playlistpath, g_playlistcda))
	{
		ShowErr(L"Eject CD : Stop first");
		return;
	}

	for(i = 1; i <= CDAMAXNUMDRIVES; i++)
		if(g_cdadrive[i].exists && g_cdadrive[i].isready)
			CdaEject(i);
	
#endif
	
} // DoEjectCD


void DoAudioIn()
{
#if !defined(NODECODEAUDIOIN)
	fsstrcpy(g_newplaylist, g_playlistaudioin);
	DoStop(); // DoTimerStop(1);
	WaitDecoderStopped();
	DoTimerPlayPlaylist(10);
#endif

} // DoAudioIn


void OnButtonList()
{
	if(g_listbusy)
		return;
#if !defined(MAGUINONE)
	if(!g_w[WINDOWLIST])
		return;
#endif
	g_listbusy = 1;
		
	PushButtonDisable(WINDOWLIST, ID_BTN_PLAYLIST);
					
	g_sets.listboxmode &= ~(LBOXPLAYERINFO | LBOXPLAYLIST | LBOXSONGSLOADED | LBOXSONGINFO);
	g_sets.listboxmode |= LBOXPLAYLIST;
	UpdateListWindowList(0);
		
	PushButtonEnable(WINDOWLIST, ID_BTN_PLAYLIST);
	
	g_listbusy = 0;
	
} // OnButtonList


void OnButtonSong()
{
	if(g_listbusy)
		return;
#if !defined(MAGUINONE)
	if(!g_w[WINDOWLIST])
		return;
#endif
	g_listbusy = 1;
		
	PushButtonDisable(WINDOWLIST, ID_BTN_SONGINFO);
					
	DisableButtonsMenus();

	g_sets.listboxmode &= ~(LBOXPLAYERINFO | LBOXPLAYLIST | LBOXSONGSLOADED | LBOXSONGINFO);
	g_sets.listboxmode |= LBOXSONGINFO;
	UpdateListWindowList(0);

	EnableButtonsMenus();

	PushButtonEnable(WINDOWLIST, ID_BTN_SONGINFO);		
	
	g_listbusy = 0;
		
} // OnButtonSong


void OnButtonInfo()
{
	if(g_listbusy)
		return;
#if !defined(MAGUINONE)
	if(!g_w[WINDOWLIST])
		return;
#endif
	g_listbusy = 1;
		
	PushButtonDisable(WINDOWLIST, ID_BTN_INFO);
					
	DisableButtonsMenus();

#ifndef MANOCDA
	if(!g_state.running)
		CdaInitCDRom(); // OnButtonInfo
#endif

	g_sets.listboxmode &= ~(LBOXPLAYERINFO | LBOXPLAYLIST | LBOXSONGSLOADED | LBOXSONGINFO);
	g_sets.listboxmode |= LBOXPLAYERINFO;
	UpdateListWindowList(0);
			
	EnableButtonsMenus();

	PushButtonEnable(WINDOWLIST, ID_BTN_INFO);		
	
	g_listbusy = 0;
		
} // OnButtonInfo


void OnButtonLoaded()
{
	if(g_listbusy)
		return;
#if !defined(MAGUINONE)
	if(!g_w[WINDOWLIST])
		return;
#endif
	g_listbusy = 1;
		
	PushButtonDisable(WINDOWLIST, ID_BTN_LOADED);
					
	DisableButtonsMenus();

	g_sets.listboxmode &= ~(LBOXPLAYERINFO | LBOXPLAYLIST | LBOXSONGSLOADED | LBOXSONGINFO);
	g_sets.listboxmode |= LBOXSONGSLOADED;
	UpdateListWindowList(0);
		
	EnableButtonsMenus();

	PushButtonEnable(WINDOWLIST, ID_BTN_LOADED);
	
	g_listbusy = 0;
		
} // OnButtonLoaded


void DoZeroList()
{
	DisableButtonsMenus();

	if(!ZeroRecords(&l_psongrec, g_songsnum))
	{	
		l_songsnumplayed = 0;	
		SetAppWindowTitle(L"Zeroed");
	}
	
	if(g_sets.mawin[WINDOWLIST].ison && g_sets.listboxmode & LBOXPLAYERINFO)
		UpdateListWindowList(0); // for updated play count
	
	SaveRecords();
	
	EnableButtonsMenus();

} // DoZeroList


void DoBoxBeg(int checked)
{
	machar lstr[32];

	if(checked)
	{
		g_posbegon = l_posbegonbu = 1;
		
		if(g_state.paused)
			g_posbeg = (mauint64)(LROUND(((g_state.duration * (double)l_posslidervalue) / POSMAX)));
		else
			g_posbeg = g_state.position;
		
		if(g_posbeg >= g_posend)
			g_posbeg = g_posend - MINPOSDELTA;
		
		MillisecondsToStrHMSD(lstr, g_posbeg);
	}
	else
	{
		g_posbegon = l_posbegonbu = 0;
		g_posbeg = 0;
		mastrcpy(lstr, ZEROMSSTR);
	}
	
	l_playerseeking = 0;
	
	StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEBEG, lstr);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_BEG, checked);
	
} // DoBoxBeg


void DoBoxEnd(int checked)
{
	machar lstr[32];

	if(checked)
	{
		g_posendon = l_posendonbu = 1;
		
		if(g_state.paused)
			g_posend = l_poscurrent;
		else
			g_posend = g_state.position;
		
		if(g_posend <= g_posbeg)
			g_posend = g_posbeg + MINPOSDELTA;		
		if(g_posend < MINPOSDELTA)
			g_posend = MINPOSDELTA;
		
		MillisecondsToStrHMSD(lstr, g_posend);
	}
	else
	{
		g_posendon = l_posendonbu = 0;
		g_posend = g_state.duration;
		MillisecondsToStrHMSD(lstr, g_posend);
	}
	
	l_playerseeking = 0;
	
	l_posend = g_posend;
	StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEEND, lstr);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_END, checked);

} // DoBoxEnd


#if defined(ENCODEMP4)
 void DoBoxM4A(int checked)
 {
	if(checked)
		g_sets.fileout[g_k[g_sets.output]].mode |= kACCM4A;
	else
		g_sets.fileout[g_k[g_sets.output]].mode &= ~kACCM4A;

	CheckBoxSetValue(WINDOWFILEOUT, ID_CHK_M4A, checked);

 } // DoBoxM4A
#endif


#if defined(ENCODEAAC)
 void DoBoxADTS(int checked)
 {
	if(checked)
		g_sets.fileout[g_k[g_sets.output]].mode |= kACCADTS;
	else
		g_sets.fileout[g_k[g_sets.output]].mode &= ~kACCADTS;

	CheckBoxSetValue(WINDOWFILEOUT, ID_CHK_ADTS, checked);
		
 } // DoBoxADTS
#endif


void DoCloseReopen(int checked)
{
	if(checked)
		g_sets.fileout[kOUTSOUND].mode = 1;
	else
		g_sets.fileout[kOUTSOUND].mode = 0;

} // DoCloseReopen


void DoLogNotPlayable(int checked)
{
	if(checked)
	{
		maLog(L"Error Logging of non-playable files turned on");
		g_sets.logflags |= kMALOGNOTPLAYABLE;
	}
	else
	{
		maLog(L"Error Logging of non-playable files turned off");
		g_sets.logflags &= ~kMALOGNOTPLAYABLE;
	}
	
} // DoLogNotPlayable


void DoUseM3u(int checked)
{
	if(checked)
		g_sets.usem3u = 1;
	else
		g_sets.usem3u = 0;

} // DoUseM3u


void DoExitAfter(int checked)
{
	if(checked)
		g_sets.exitafter = 1;
	else
		g_sets.exitafter = 0;

} // DoUseM3u


void DoOpenList()
{
	fschar file[MAX_PATH];
	machar mastr[MAXLOGLINELEN];
	
	DisableButtonsMenus();
	
	masprintf(mastr, MAXLOGLINELEN, L"%ls (*.maf)|*.maf|Any file (*)|*", g_prognametitle);
	if(!maFileDialog(file, WINDOWAPP, L"Open a playlist", g_playlistsdir, L"", mastr, maOPEN | maFILE_MUST_EXIST))
		ChangePlaylist(file);
	
	EnableButtonsMenus();
	
} // DoOpenList


void DoDeleteList()
{
	fschar file[MAX_PATH];
	machar mastr[MAXLOGLINELEN];
	
	DisableButtonsMenus();
	
	masprintf(mastr, MAXLOGLINELEN, L"%ls (*.maf)|*.maf|(*.maa)|*.maa|Any file (*)|*", g_prognametitle);
	
	for (; ;)
	{
		if(!maFileDialog(file, WINDOWAPP, L"Delete a playlist", g_playlistsdir, L"", mastr, maOPEN | maFILE_MUST_EXIST))
			DeletePlaylist(file);
		else
			break;
	}
	
	EnableButtonsMenus();
	
} // DoDeleteList


void DoPlayFile()
{
	int err = 1;
	int i, j;
	fschar newplaylist[MAX_PATH];
	maArray ar;
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
#endif
	machar mastr[MAXLOGLINELEN];
		
	DisableButtonsMenus();
	
	if(maFileDialogM(&ar, WINDOWAPP, L"Open files to play", l_lastsongdir, L"",
		L"Any file|*|mp3|*.mp3|ogg|*.ogg|wma|*.wma|wav|*.wav|aif|*.aif|aiff|*.aiff|cda|*.cda|aac|*.aac|mp4|*.mp4|m2a|*.m2a|m4a|*.m4a",
				maOPEN | maFILE_MUST_EXIST | maMULTIPLE))
		goto onmenuplayfileret;
				
	// save dir for next time	
	j = 0;
	fsstrcpy(l_lastsongdir, ar.pitems[0]);
	for(i = 0; i < (int)fsstrlen(l_lastsongdir); i++)
		if(l_lastsongdir[i] == FILESEP)
			j = i;
	l_lastsongdir[j] = fsT('\0');
				
	newplaylist[0] = fsT('\0');
	err = maPlaylistMake(ar.nitems, ar.pitems, newplaylist); // DoPlayFile
	maFileDialogMFree(&ar);
	if(err)
	{
		masprintf(mastr, MAXLOGLINELEN, L"DoPlayFile maPlaylistMake failed on '%ls'",
																		fs2ma(mastr2, newplaylist));
		maLog(mastr);
		goto onmenuplayfileret;
	}
				
	err = ChangePlaylist(newplaylist);

onmenuplayfileret:
	EnableButtonsMenus();
	
	if(!err)
		DoTimerPlay(100); // DoPlayFile
	
} // DoPlayFile


void DoOpenRecent(int index)
{
	fschar items[MAXOPENRECENT][MAX_PATH];

	if(!GetRecent(items))
		if(items[index][0])
		{
#if !defined(MANOCDA)
			if(!fsstrcasecmp(g_playlistcda, items[index]))
				if(CdaInitCDRom()) // DoOpenRecent
				{
					ShowErr(L"Insert an Audio CD or open another playlist");
					return;
				}
#endif
			ChangePlaylist(items[index]);
		}
		
} // DoOpenRecent


void DoSetVDisk()
{
	DisableButtonsMenus();
	
	if(!SetVDisk())
		GetVDisk();
		
	UpdateListWindowList(0);
			
	EnableButtonsMenus();
	
} // DoSetVDisk


void DoMultipleInst(int checked)
{
	if(g_thisinstance == 1) // always true here
	{
		g_sets.multipleinst = checked;
		MenuSetChecked(MENUMASETS, ID_MENU_MULTIPLEINST, (g_sets.multipleinst) ? 1 : 0);
		SetMultipleInstances();
	}
	
} // DoMultipleInst


#if defined(MAKEYBOARD)
void DoKeyboard(int checked)
{
	g_sets.keyboard = checked;
	MenuSetChecked(MENUMASETS, ID_MENU_KEYBOARD, (g_sets.keyboard) ? 1 : 0);
	if(!checked)
		DoTitleInfo(g_sets.titleinfo + 1);
	
	UpdateMenuKbLabels();
	UpdateEffectsKbLabels();
	UpdateButtonLabels();

	SetLogMenus(); // in case updating their label left them wrongly enabled

} // DoKeyboard
#endif


#if defined(MAMOUSEWHEEL)
void DoMouseWheel(int checked)
{
	g_sets.mousewheel = checked;
	MenuSetChecked(MENUMASETS, ID_MENU_MOUSEWHEEL, (g_sets.mousewheel) ? 1 : 0);
			
} // DoMouseWheel
#endif


#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
void OnButtonAudioCDSet()
{
	DisableButtonsMenus();
	
	SetsAudioCD();
		
	EnableButtonsMenus();
	
} // OnButtonAudioCDSet


void OnButtonAudioCDClear()
{
	ClearAudioCD();
		
} // OnButtonAudioCDClear


void OnButtonAudioCDSave()
{
	int item;	

	PopupButtonGetSelection(WINDOWAUDIOCD, ID_POPUP_AUDIOCD, &item);
	if(item > -1)
		g_setscdrom.item = item;

	SaveSettingsCDRom();
		
} // OnButtonAudioCDSave
#endif


void DoAsDefaults()
{
	if(maMessageBox(L"Really save current settings as Default settings ?",
					L"NEW DEFAULT SETTINGS", maYES_NO | maICON_QUESTION) != maYES)
		return;
	
	DisableButtonsMenus();
	
	if(!SaveSettingsAsDefault(&g_sets))
		if(!g_state.running)
			SetAppWindowTitle(L"Settings saved");
		
	EnableButtonsMenus();

} // DoAsDefaults


void DoSetsFactory()
{
	DisableButtonsMenus();
	
	/* if(maModKeyDown()) // opens all windows
	{
		int windowid;

		for(windowid = WINDOWFIRST; windowid <= WINDOWLAST; windowid++)
		{
			if(windowid == WINDOWSPLASH)
				continue;
 #if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
 #else
			if(windowid == WINDOWAUDIOCD)
				continue;
 #endif
			DoWindow(windowid, 1);
		}
		
		EnableButtonsMenus();
		return;
	}
	*/

	LoadFactorySettings(&g_sets);

	maRescanAudioDevices(g_audiohostin, g_audiohostout);

	UpdateDisplay();
	MadDoParamChange();
	EqDoParamChange();
		
	EnableButtonsMenus();
	
} // DoSetsFactory


void DoSetsDefault()
{
	DisableButtonsMenus();
	
	LoadDefaultSettings(&g_sets);
	UpdateDisplay();
	MadDoParamChange();
	EqDoParamChange();
		
	EnableButtonsMenus();

} // DoSetsDefault


void DoSetsSaved()
{
	DisableButtonsMenus();
	
	l_settingspathlast[0] = fsT('\0');
	LoadSettings();
	UpdateDisplay();
	MadDoParamChange();
	EqDoParamChange();
		
	EnableButtonsMenus();

} // DoSetsSaved


void DoSetsCopy()
{
	DisableButtonsMenus();
	
	if(!CopySettings())
	{
		UpdateDisplay();
		MadDoParamChange();
		EqDoParamChange();
	}	
	EnableButtonsMenus();
	
} // DoSetsCopy


void DoMakeList()
{
	fschar *pitems[1];
	fschar dirpath[MAX_PATH];
	fschar newplaylist[MAX_PATH];
	
	DisableButtonsMenus();

	dirpath[0] = fsT('\0');

	// prompt for items for a new playlist (dirs, files, links, cd, ...)
	if(maDirSelector(dirpath,
#ifdef __MAWIN__
		 L"Select a FOLDER containing song files and/or shortcuts, or an Audio CD"
#endif

#if defined(__MAUNIX__)
		 L"Select a DIRECTORY containing song files and/or links, or an Audio CD"
#endif

#ifdef __MAOSX__
		 L"Select a FOLDER containing song files and/or aliases" // wxDirSelector can't handle Audio CD
#endif
		))
		goto OnButtonMakeListRet; // cancelled

	// make a new playlist
	fsstrcpy(newplaylist, dirpath); // prompt
	pitems[0] = dirpath;
	if(maPlaylistMake(1, pitems, newplaylist)) // DoMakeList
		goto OnButtonMakeListRet;
	
	// new playlist becomes current playlist
	ChangePlaylist(newplaylist);

OnButtonMakeListRet:

	SetAppWindowTitle(L"Stopped");

	EnableButtonsMenus();
	
} // DoMakeList


void DoSaveListAs()
{
	DisableButtonsMenus();

	SavePlaylistAs();
	
	EnableButtonsMenus();
	
} // DoSaveListAs


double g_eqzoomval = -1.0;
EQUPRESET l_eqpresetstmp[EQNUMPRESETS];
EQUPRESET l_eq2presetstmp[EQNUMPRESETS];
void OnButtonEqRaise();
void OnButtonEqLower();

void OnButtonEqRaise()
{
	int band;
	
	if(g_w[WINDOWEQ])
		PushButtonDisable(WINDOWEQ, ID_BTN_EQPLUS);
					
	// first time in tmpsave eq settings
	if(g_eqzoomval < 0.0)
	{
		g_eqzoomval = 1.0;
		memcpy( l_eqpresetstmp,  g_sets.eqpresets, EQNUMPRESETS * sizeof(EQUPRESET));
		memcpy(l_eq2presetstmp, g_sets.eq2presets, EQNUMPRESETS * sizeof(EQUPRESET));
	}
	
	// do nothing if all >= max
	for(band = 0; band < EQBANDS31; band++)
		if(g_sets.eqpresets[g_sets.eqpreset].eqslider[band] < DBSLIDERMAX ||
			g_sets.eq2presets[g_sets.eqpreset].eqslider[band] < DBSLIDERMAX)
			break;
	
	if(band >= EQBANDS31) // all >= max
	{
		for(band = 0; band < EQBANDS31; band++)
		{
			g_sets.eqpresets[g_sets.eqpreset].eqslider[band] =
			g_sets.eq2presets[g_sets.eqpreset].eqslider[band] = DBSLIDERMAX;
		}
		if(g_w[WINDOWEQ])
			PushButtonEnable(WINDOWEQ, ID_BTN_EQPLUS);		
		return;
	}
				
	for(band = 0; band < EQBANDS31; band++)
	{		
		g_sets.eqpresets[g_sets.eqpreset].eqslider[band] += SLIDERINCR;
		g_sets.eq2presets[g_sets.eqpreset].eqslider[band] += SLIDERINCR;
		if(g_sets.eqpresets[g_sets.eqpreset].eqslider[band] > DBSLIDERMAX)
			g_sets.eqpresets[g_sets.eqpreset].eqslider[band] = DBSLIDERMAX;
		if(g_sets.eqpresets[g_sets.eqpreset].eqslider[band] < DBSLIDERMIN)
			g_sets.eqpresets[g_sets.eqpreset].eqslider[band] = DBSLIDERMIN;
		if(g_sets.eq2presets[g_sets.eqpreset].eqslider[band] > DBSLIDERMAX)
			g_sets.eq2presets[g_sets.eqpreset].eqslider[band] = DBSLIDERMAX;
		if(g_sets.eq2presets[g_sets.eqpreset].eqslider[band] < DBSLIDERMIN)
			g_sets.eq2presets[g_sets.eqpreset].eqslider[band] = DBSLIDERMIN;
	}			
				
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ2])
		SetSliders(EQ2SLIDERS);
	
	if(g_w[WINDOWEQ])
	{
		SetSliders(EQSLIDERS);
		PushButtonEnable(WINDOWEQ, ID_BTN_EQPLUS);		
	}
	
} // OnButtonEqRaise


void OnButtonEqLower()
{
	int band;
	
	if(g_w[WINDOWEQ])
		PushButtonDisable(WINDOWEQ, ID_BTN_EQMINUS);
	
	// first time in tmpsave eq settings	
	if(g_eqzoomval < 0.0)
	{
		g_eqzoomval = 1.0;
		memcpy(l_eqpresetstmp,   g_sets.eqpresets, EQNUMPRESETS * sizeof(EQUPRESET));
		memcpy(l_eq2presetstmp, g_sets.eq2presets, EQNUMPRESETS * sizeof(EQUPRESET));
	}
	
	// do nothing if all <= min			
	for(band = 0; band < EQBANDS31; band++)
		if(g_sets.eqpresets[g_sets.eqpreset].eqslider[band] > DBSLIDERMIN ||
		   g_sets.eq2presets[g_sets.eqpreset].eqslider[band] > DBSLIDERMIN)
			break;
	
	if(band >= EQBANDS31) // all <= min
	{
		for(band = 0; band < EQBANDS31; band++)
		{
			g_sets.eqpresets[g_sets.eqpreset].eqslider[band] =
			g_sets.eq2presets[g_sets.eqpreset].eqslider[band] = DBSLIDERMIN;
		}
		if(g_w[WINDOWEQ])
			PushButtonEnable(WINDOWEQ, ID_BTN_EQMINUS);		
		return;
	}
			
	for(band = 0; band < EQBANDS31; band++)
	{		
		g_sets.eqpresets[g_sets.eqpreset].eqslider[band] -= SLIDERDECR;
		g_sets.eq2presets[g_sets.eqpreset].eqslider[band] -= SLIDERDECR;
		if(g_sets.eqpresets[g_sets.eqpreset].eqslider[band] > DBSLIDERMAX)
			g_sets.eqpresets[g_sets.eqpreset].eqslider[band] = DBSLIDERMAX;
		if(g_sets.eqpresets[g_sets.eqpreset].eqslider[band] < DBSLIDERMIN)
			g_sets.eqpresets[g_sets.eqpreset].eqslider[band] = DBSLIDERMIN;
		if(g_sets.eq2presets[g_sets.eqpreset].eqslider[band] > DBSLIDERMAX)
			g_sets.eq2presets[g_sets.eqpreset].eqslider[band] = DBSLIDERMAX;
		if(g_sets.eq2presets[g_sets.eqpreset].eqslider[band] < DBSLIDERMIN)
			g_sets.eq2presets[g_sets.eqpreset].eqslider[band] = DBSLIDERMIN;
	}
			
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ2])
		SetSliders(EQ2SLIDERS);
	
	if(g_w[WINDOWEQ])
	{
		SetSliders(EQSLIDERS);
		PushButtonEnable(WINDOWEQ, ID_BTN_EQMINUS);		
	}
	
} // OnButtonEqLower


void OnButtonEqPlus()
{
	int band;
	int val[EQBANDS31];
	int val2[EQBANDS31];
	int mid = (DBSLIDERMAX - DBSLIDERMIN) / 2;
	int imax = 0;
	int nmax = 0;
	
	if(!(g_sets.eqflags & MAEQZOOM))
	{
		OnButtonEqRaise();
		return;
	}
					
	if(g_w[WINDOWEQ])
		PushButtonDisable(WINDOWEQ, ID_BTN_EQPLUS);

	// first time in tmpsave eq settings	
	if(g_eqzoomval < 0.0)
	{
		g_eqzoomval = 1.0;
		memcpy(l_eqpresetstmp,   g_sets.eqpresets, EQNUMPRESETS * sizeof(EQUPRESET));
		memcpy(l_eq2presetstmp, g_sets.eq2presets, EQNUMPRESETS * sizeof(EQUPRESET));
	}
	
	// 5% increase	
	g_eqzoomval = g_eqzoomval + 0.05;
	
	for(band = 0; band < EQBANDS31; band++)
	{
		imax = 0;
		if((val[band] = (int)(LROUND(
			mid + g_eqzoomval * (l_eqpresetstmp[g_sets.eqpreset].eqslider[band] - mid)))) > DBSLIDERMAX || 
																			val[band] < DBSLIDERMIN)
			imax++;
		if((val2[band] = (int)(LROUND(
			mid + g_eqzoomval * (l_eq2presetstmp[g_sets.eqpreset].eqslider[band] - mid)))) > DBSLIDERMAX || 
																	val2[band] < DBSLIDERMIN || imax)
			nmax++;
	}
	
	if(nmax >= EQBANDS31) // all sliders on limits
	{
		if(g_w[WINDOWEQ])
			PushButtonEnable(WINDOWEQ, ID_BTN_EQPLUS);
		return;
	}
			
	for(band = 0; band < EQBANDS31; band++)
	{	
	
		if(val[band] > DBSLIDERMAX)
			val[band] = DBSLIDERMAX;
		else if(val[band] < DBSLIDERMIN)
			val[band] = DBSLIDERMIN;
		g_sets.eqpresets[g_sets.eqpreset].eqslider[band] = val[band];

		if(val2[band] > DBSLIDERMAX)
			val2[band] = DBSLIDERMAX;
		else if(val2[band] < DBSLIDERMIN)
			val2[band] = DBSLIDERMIN;
		g_sets.eq2presets[g_sets.eqpreset].eqslider[band] = val2[band];
	}
				
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ2])
		SetSliders(EQ2SLIDERS);
	
	if(g_w[WINDOWEQ])
	{
		SetSliders(EQSLIDERS);
		PushButtonEnable(WINDOWEQ, ID_BTN_EQPLUS);		
	}
	
} // OnButtonEqPlus


void OnButtonEqMinus()
{
	int band;
	int mid = (DBSLIDERMAX - DBSLIDERMIN) / 2;
	
	if(!(g_sets.eqflags & MAEQZOOM))
	{
		OnButtonEqLower();
		return;
	}
					
	if(g_w[WINDOWEQ])
		PushButtonDisable(WINDOWEQ, ID_BTN_EQMINUS);
	
	// first time in tmpsave eq settings	
	if(g_eqzoomval < 0.0)
	{
		g_eqzoomval = 1.0;
		memcpy(l_eqpresetstmp,  g_sets.eqpresets,  EQNUMPRESETS * sizeof(EQUPRESET));
		memcpy(l_eq2presetstmp, g_sets.eq2presets, EQNUMPRESETS * sizeof(EQUPRESET));
	}
	
	// 5% decrease	
	if((g_eqzoomval = g_eqzoomval - 0.05) < 0.0)
		g_eqzoomval = 0.0;
	
	for(band = 0; band < EQBANDS31; band++)
	{
		 g_sets.eqpresets[g_sets.eqpreset].eqslider[band] = (int)(LROUND(
			mid + g_eqzoomval * ( l_eqpresetstmp[g_sets.eqpreset].eqslider[band] - mid)));
		g_sets.eq2presets[g_sets.eqpreset].eqslider[band] = (int)(LROUND(
			mid + g_eqzoomval * (l_eq2presetstmp[g_sets.eqpreset].eqslider[band] - mid)));
	}
	
	for(band = 0; band < EQBANDS31; band++)
	{		
		if(g_sets.eqpresets[g_sets.eqpreset].eqslider[band] > DBSLIDERMAX)
			g_sets.eqpresets[g_sets.eqpreset].eqslider[band] = DBSLIDERMAX;
		if(g_sets.eqpresets[g_sets.eqpreset].eqslider[band] < DBSLIDERMIN)
			g_sets.eqpresets[g_sets.eqpreset].eqslider[band] = DBSLIDERMIN;

		if(g_sets.eq2presets[g_sets.eqpreset].eqslider[band] > DBSLIDERMAX)
			g_sets.eq2presets[g_sets.eqpreset].eqslider[band] = DBSLIDERMAX;
		if(g_sets.eq2presets[g_sets.eqpreset].eqslider[band] < DBSLIDERMIN)
			g_sets.eq2presets[g_sets.eqpreset].eqslider[band] = DBSLIDERMIN;
	}

	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ2])
		SetSliders(EQ2SLIDERS);
	
	if(g_w[WINDOWEQ])
	{
		SetSliders(EQSLIDERS);
		PushButtonEnable(WINDOWEQ, ID_BTN_EQMINUS);		
	}
	
} // OnButtonEqMinus


void OnButtonEqCopy()
{			
	if(g_w[WINDOWEQ])
		PushButtonDisable(WINDOWEQ, ID_BTN_EQCOPY);
					
	memcpy(&l_eqpresettemp, &g_sets.eqpresets[g_sets.eqpreset], sizeof(EQUPRESET));
	
	if(g_w[WINDOWEQ])
		PushButtonEnable(WINDOWEQ, ID_BTN_EQCOPY);		
	
} // OnButtonEqCopy


void OnButtonEqPaste()
{			
	if(g_w[WINDOWEQ])
		PushButtonDisable(WINDOWEQ, ID_BTN_EQPASTE);
					
	g_eqzoomval = -1.0;

	memcpy(&g_sets.eqpresets[g_sets.eqpreset], &l_eqpresettemp, sizeof(EQUPRESET) - sizeof(((EQUPRESET *) 0)->eqname));
	
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ])
	{
		StaticTextSetValue(WINDOWEQ, ID_TEXT_EQOFFSETVAL, g_sets.eqpresets[g_sets.eqpreset].eqoffset);
		SetSliders(EQSLIDERS);
		PushButtonEnable(WINDOWEQ, ID_BTN_EQPASTE);		
	}
	
} // OnButtonEqPaste


void OnButtonEqRestore()
{
	if(g_w[WINDOWEQ])
		PushButtonDisable(WINDOWEQ, ID_BTN_EQRESTORE);
	
	g_eqzoomval = -1.0;
	
	memcpy(&g_sets.eqpresets[g_sets.eqpreset], &g_setsbu.eqpresets[g_sets.eqpreset], sizeof(EQUPRESET) - sizeof(((EQUPRESET *) 0)->eqname));
						
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ])
	{
		StaticTextSetValue(WINDOWEQ, ID_TEXT_EQOFFSETVAL, g_sets.eqpresets[g_sets.eqpreset].eqoffset);
		SetSliders(EQSLIDERS);
		PushButtonEnable(WINDOWEQ, ID_BTN_EQRESTORE);		
	}
	
} // OnButtonEqRestore


void OnButtonEqMid()
{
	int band;
	
	if(g_w[WINDOWEQ])
		PushButtonDisable(WINDOWEQ, ID_BTN_EQMID);
					
	// first time in tmpsave eq settings	
	if(g_eqzoomval < 0.0)
		memcpy(l_eqpresetstmp, g_sets.eqpresets, EQNUMPRESETS * sizeof(EQUPRESET));
	
	g_eqzoomval = 0.0;
	
	for(band = 0; band < EQBANDS31; band++)
		g_sets.eqpresets[g_sets.eqpreset].eqslider[band] = DBSLIDERMAX / 2; // mid == 31
	
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ])
	{
		SetSliders(EQSLIDERS);
		PushButtonEnable(WINDOWEQ, ID_BTN_EQMID);		
	}
	
} // OnButtonEqMid


void OnButtonEqShiftL()
{
	int band;
	
	if(g_w[WINDOWEQ])
		PushButtonDisable(WINDOWEQ, ID_BTN_EQSHIFTL);
					
	// first time in tmpsave eq settings	
	if(g_eqzoomval < 0.0)
		memcpy(l_eqpresetstmp, g_sets.eqpresets, EQNUMPRESETS * sizeof(EQUPRESET));
	
	for(band = 0; band < EQBANDS31 - 1; band++)
	{
		g_sets.eqpresets[g_sets.eqpreset].eqslider[band] =
								g_sets.eqpresets[g_sets.eqpreset].eqslider[band + 1];
		l_eqpresetstmp[g_sets.eqpreset].eqslider[band] =
								l_eqpresetstmp[g_sets.eqpreset].eqslider[band + 1];
		g_sets.eq2presets[g_sets.eqpreset].eqslider[band] =
								g_sets.eq2presets[g_sets.eqpreset].eqslider[band + 1];
		l_eq2presetstmp[g_sets.eqpreset].eqslider[band] =
								l_eq2presetstmp[g_sets.eqpreset].eqslider[band + 1];
	}
	
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ2])
		SetSliders(EQ2SLIDERS);
	
	if(g_w[WINDOWEQ])
	{
		SetSliders(EQSLIDERS);
		PushButtonEnable(WINDOWEQ, ID_BTN_EQSHIFTL);		
	}
	
} // OnButtonEqShiftL


void OnButtonEqShiftR()
{
	int band;
	
	if(g_w[WINDOWEQ])
		PushButtonDisable(WINDOWEQ, ID_BTN_EQSHIFTR);
					
	// first time in tmpsave eq settings	
	if(g_eqzoomval < 0.0)
		memcpy(l_eqpresetstmp, g_sets.eqpresets, EQNUMPRESETS * sizeof(EQUPRESET));
	
	for(band = EQBANDS31 - 1; band > 0; band--)
	{
		g_sets.eqpresets[g_sets.eqpreset].eqslider[band] =
								g_sets.eqpresets[g_sets.eqpreset].eqslider[band - 1];
		l_eqpresetstmp[g_sets.eqpreset].eqslider[band] =
								l_eqpresetstmp[g_sets.eqpreset].eqslider[band - 1];
		g_sets.eq2presets[g_sets.eqpreset].eqslider[band] =
								g_sets.eq2presets[g_sets.eqpreset].eqslider[band - 1];
		l_eq2presetstmp[g_sets.eqpreset].eqslider[band] =
								l_eq2presetstmp[g_sets.eqpreset].eqslider[band - 1];
	}
									
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ2])
		SetSliders(EQ2SLIDERS);
	
	if(g_w[WINDOWEQ])
	{
		SetSliders(EQSLIDERS);
		PushButtonEnable(WINDOWEQ, ID_BTN_EQSHIFTR);		
	}
	
} // OnButtonEqShiftR


void OnButtonEqInvert()
{
	int band;
	
	if(g_w[WINDOWEQ])
		PushButtonDisable(WINDOWEQ, ID_BTN_EQINVERT);
					
	// first time in tmpsave eq settings	
	if(g_eqzoomval < 0.0)
		memcpy(l_eqpresetstmp, g_sets.eqpresets, EQNUMPRESETS * sizeof(EQUPRESET));
	
	for(band = 0; band < EQBANDS31; band++)
	{
		g_sets.eqpresets[g_sets.eqpreset].eqslider[band] =
						DBSLIDERMAX - g_sets.eqpresets[g_sets.eqpreset].eqslider[band];
		l_eqpresetstmp[g_sets.eqpreset].eqslider[band] =
						DBSLIDERMAX - l_eqpresetstmp[g_sets.eqpreset].eqslider[band];
		g_sets.eq2presets[g_sets.eqpreset].eqslider[band] =
						DBSLIDERMAX - g_sets.eq2presets[g_sets.eqpreset].eqslider[band];
		l_eq2presetstmp[g_sets.eqpreset].eqslider[band] =
						DBSLIDERMAX - l_eq2presetstmp[g_sets.eqpreset].eqslider[band];
	}
									
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ2])
		SetSliders(EQ2SLIDERS);
	
	if(g_w[WINDOWEQ])
	{
		SetSliders(EQSLIDERS);
		PushButtonEnable(WINDOWEQ, ID_BTN_EQINVERT);		
	}
	
} // OnButtonEqInvert


void OnButtonEq2Copy()
{			
	if(g_w[WINDOWEQ2])
		PushButtonDisable(WINDOWEQ2, ID_BTN_EQ2COPY);
					
	memcpy(&l_eqpresettemp, &g_sets.eq2presets[g_sets.eqpreset], sizeof(EQUPRESET));
	
	if(g_w[WINDOWEQ2])
		PushButtonEnable(WINDOWEQ2, ID_BTN_EQ2COPY);		
	
} // OnButtonEq2Copy


void OnButtonEq2Paste()
{			
	if(g_w[WINDOWEQ2])
		PushButtonDisable(WINDOWEQ2, ID_BTN_EQ2PASTE);
					
	g_eqzoomval = -1.0;

	memcpy(&g_sets.eq2presets[g_sets.eqpreset], &l_eqpresettemp, sizeof(EQUPRESET) - sizeof(((EQUPRESET *) 0)->eqname));
	
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ2])
	{
		SetSliders(EQ2SLIDERS);
		PushButtonEnable(WINDOWEQ2, ID_BTN_EQ2PASTE);		
	}
	
} // OnButtonEq2Paste


void OnButtonEq2Restore()
{
	if(g_w[WINDOWEQ2])
		PushButtonDisable(WINDOWEQ2, ID_BTN_EQ2RESTORE);
	
	g_eqzoomval = -1.0;
	
	memcpy(&g_sets.eq2presets[g_sets.eqpreset], &g_setsbu.eq2presets[g_sets.eqpreset], sizeof(EQUPRESET) - sizeof(((EQUPRESET *) 0)->eqname));
						
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ2])
	{
		SetSliders(EQ2SLIDERS);
		PushButtonEnable(WINDOWEQ2, ID_BTN_EQ2RESTORE);		
	}
	
} // OnButtonEq2Restore


void OnButtonEq2Mid()
{
	int band;
	
	if(g_w[WINDOWEQ2])
		PushButtonDisable(WINDOWEQ2, ID_BTN_EQ2MID);
					
	// first time in tmpsave eq settings	
	if(g_eqzoomval < 0.0)
		memcpy(l_eqpresetstmp, g_sets.eq2presets, EQNUMPRESETS * sizeof(EQUPRESET));
	
	g_eqzoomval = 0.0;
	
	for(band = 0; band < EQBANDS31; band++)
		g_sets.eq2presets[g_sets.eqpreset].eqslider[band] = DBSLIDERMAX / 2; // mid == 31
	
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();

	if(g_w[WINDOWEQ2])
	{
		SetSliders(EQ2SLIDERS);
		PushButtonEnable(WINDOWEQ2, ID_BTN_EQ2MID);		
	}
	
} // OnButtonEq2Mid


void OnButtonFileOutDir()
{
	fschar dirpath[MAX_PATH];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];

	if(g_sets.fileout[g_k[g_sets.output]].outdir && g_sets.fileout[g_k[g_sets.output]].outdir[0])
		fsstrcpy(dirpath, g_sets.fileout[g_k[g_sets.output]].outdir); // default dir for dirSelector
	else
		dirpath[0] = fsT('\0');
	
	if(!maDirSelector(dirpath,
#if defined(__MAUNIX__)
		L"Select a DIRECTORY in which to put the new files"
#else
		L"Select a FOLDER in which to put the new files"
#endif
	)) // else cancelled
		fsstrcpy(g_sets.fileout[g_k[g_sets.output]].outdir, dirpath);

	StaticTextSetText(WINDOWFILEOUT, ID_TEXT_FILEOUTDIR, fs2ma(mastr2, g_sets.fileout[g_k[g_sets.output]].outdir));
	
	if(!maDirExists(g_sets.fileout[g_k[g_sets.output]].outdir))
	{
		masprintf(mastr, MAXLOGLINELEN, L"File output folder does not exist:%ls%ls'%ls'",
								g_maeol, g_maeol, fs2ma(mastr2, g_sets.fileout[g_k[g_sets.output]].outdir));
		ShowErr(mastr);
	}

} // OnButtonFileOutDir


void DoEqZoom(int checked) 
{
	if(checked)
		g_sets.eqflags |= MAEQZOOM;
	else
		g_sets.eqflags &= ~MAEQZOOM;

	CheckBoxSetValue(WINDOWEQ, ID_CHK_EQZOOM, checked);

} // DoEqZoom


void DoEqLink(int checked) 
{
	if(checked)
		g_sets.eqflags |= MAEQLINK;
	else
		g_sets.eqflags &= ~MAEQLINK;
		
	if(g_w[WINDOWEQ])
		CheckBoxSetValue(WINDOWEQ, ID_CHK_EQLINK, checked);

} // DoEqLink


void DoTitleInfo(int menu)
{
	fschar *p;
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif

	if(menu)
		g_sets.titleinfo = menu - 1;
	else if(++g_sets.titleinfo > 2)
		g_sets.titleinfo = 0;
		
	SetGUITitleInfo();
	
	if(g_sets.titleinfo == 1)
	{
		if(g_state.running)
			WindowSetTitle(WINDOWAPP, fs2ma(mastr2, g_loadedname[g_loadedcurrent]));
		else
			WindowSetTitle(WINDOWAPP, L"(Song name)");
	}
	else if(g_sets.titleinfo == 2)
	{
		if(g_state.running)
		{
			FilenameFromPathname(g_playlistpath, l_playlistfs);
			if((p = fsstrstr(l_playlistfs, g_extstr[kMAMAF])) != 0)
				*p = fsT('\0');				
			fs2mafn(l_playlistma, l_playlistfs);
			SetAppTitle2();
		}
		else
			WindowSetTitle(WINDOWAPP, L"(Numbers)");
	}
	else
		SetAppWindowTitle(L"");
	
} // DoTitleInfo


void DoHostInEnable(int checked)
{
	if(checked)
	{
		g_sets.hostinenable = 1;
	}
	else
	{
		g_sets.hostinenable = 0;
	}
		
} // DoHostInEnable


void DoInputMonitor(int checked) 
{
	if(checked)
	{
		g_sets.inputmonitor = 1;
		MenuSetChecked(MENUMASETS, ID_MENU_INPUTMONITOR, 1);
	}
	else
	{
		g_sets.inputmonitor = 0;
		MenuSetChecked(MENUMASETS, ID_MENU_INPUTMONITOR, 0);
	}
} // DoInputMonitor


void OnPopupMem()
{
	if(g_sets.buffertype == BUFUSEMEM)
		return;

	l_strclicked[0] = L'\0';
	g_loadednum = 0;
	g_sets.buffertype = BUFUSEMEM;	
	UpdateListWindowButtons();
	if(!(g_sets.listboxmode & LBOXPLAYLIST)) // don't refresh the Playlist
		UpdateListWindowList(0);
		
} // OnPopupMem


void OnPopupVDisk()
{
	if(g_sets.buffertype == BUFUSEVDISK)
		return;
		
	g_loadednum = 0;
	l_strclicked[0] = L'\0';
	g_sets.buffertype = BUFUSEVDISK;
	UpdateListWindowButtons();
	if(!(g_sets.listboxmode & LBOXPLAYLIST)) // don't refresh the Playlist
		UpdateListWindowList(0);
		
} // OnPopupVDisk


void OnPopupDirect()
{
	if(g_sets.buffertype == BUFUSEDIRECT)
		return;

	g_loadednum = 0;
	l_strclicked[0] = L'\0';
	g_sets.buffertype = BUFUSEDIRECT;
	UpdateListWindowButtons();
	if(!(g_sets.listboxmode & LBOXPLAYLIST)) // don't refresh the Playlist
		UpdateListWindowList(0);
		
} // OnPopupDirect


void DoReptSong(int checked) 
{
	g_reptsong = checked;
	CheckBoxSetValue(WINDOWAPP, ID_CHK_REPTSONG, checked);

} // DoReptSong


void DoBalance(int checked)
{ 
	if(!checked)
	{
		g_sets.volumesliderleft = volminmaxL();
		g_sets.volumesliderright = volminmaxR();
	}
	
	CheckBoxSetValue(WINDOWAPP, ID_CHK_BALANCE, checked);

	g_sets.balance = checked;
	SetSliders(VOLUMESLIDERS);	
	
} // DoBalance


void DoMonoLeft(int checked)
{ 
	if(checked)
		g_sets.monodiff |= MONOCHECKBOXL;
	else
		g_sets.monodiff &= ~MONOCHECKBOXL;

	CheckBoxSetValue(WINDOWAPP, ID_CHK_MONOLEFT, checked);

	UpdateMonoDiff();
	SetGUIMonoStereo();
	
} // DoMonoLeft


void DoMonoRight(int checked)
{ 
	if(checked)
		g_sets.monodiff |= MONOCHECKBOXR;
	else
		g_sets.monodiff &= ~MONOCHECKBOXR;

	CheckBoxSetValue(WINDOWAPP, ID_CHK_MONORIGHT, checked);

	UpdateMonoDiff();
	SetGUIMonoStereo();
	
} // DoMonoRight


void DoMonoDiff(int checked) // Mono Diff or Stereo Reverse
{ 
	if(checked)
	{
		if((g_sets.monodiff & MONOCHECKBOXL) && (g_sets.monodiff & MONOCHECKBOXR))  // mono LR
			g_sets.monodiff |= MONOCHECKBOXSUB;										// mono diff
		else if((!(g_sets.monodiff & MONOCHECKBOXL)) && (!(g_sets.monodiff & MONOCHECKBOXR))) // stereo
			g_sets.monodiff |= MONOCHECKBOXREV;										// stereo reverse
	}
	else
	{
		if((g_sets.monodiff & MONOCHECKBOXL) && (g_sets.monodiff & MONOCHECKBOXR))
			g_sets.monodiff &= ~MONOCHECKBOXSUB;
		else if((!(g_sets.monodiff & MONOCHECKBOXL)) && (!(g_sets.monodiff & MONOCHECKBOXR)))
			g_sets.monodiff &= ~MONOCHECKBOXREV;
	}

	CheckBoxSetValue(WINDOWAPP, ID_CHK_MONODIFF, checked);

	SetGUIMonoStereo();
	
} // DoMonoDiff


void UpdateMonoDiff() // for Mono Diff or Stereo Reverse
{ 
	if((  g_sets.monodiff & MONOCHECKBOXL &&
		   g_sets.monodiff & MONOCHECKBOXR &&		// mono LR
		   g_sets.monodiff & MONOCHECKBOXSUB) ||	// mono diff
		(!(g_sets.monodiff & MONOCHECKBOXL) &&
		 !(g_sets.monodiff & MONOCHECKBOXR) &&		// stereo	
		   g_sets.monodiff & MONOCHECKBOXREV))		// stereo reverse
		CheckBoxSetValue(WINDOWAPP, ID_CHK_MONODIFF, 1);
	else
		CheckBoxSetValue(WINDOWAPP, ID_CHK_MONODIFF, 0);

} // UpdateMonoDiff


void DoLimiter(int checked)
{
	if(checked)
	{
		g_sets.limiter = PREAMP | CLIPPING;
		CheckBoxSetDescriptor(WINDOWAPP, ID_CHK_LIMITER, L"0.0");
	}
	else
	{
		g_sets.limiter = PREAMP;
		CheckBoxSetDescriptor(WINDOWAPP, ID_CHK_LIMITER, L"Limit");
	}
		
	CheckBoxSetValue(WINDOWAPP, ID_CHK_LIMITER, checked);
	
	g_state.mpegattenuation = 1.0;
	
	MadDoParamChange();
	
} // DoLimiter


void DoSeparateLR(int checked)
{
	machar mastr[MAXLOGLINELEN];
	
	if(checked)
	{
		CheckBoxSetValue(WINDOWEQ, ID_CHK_SEPARATELR, 1);
		g_sets.eqflags |= MASEPARATELR;
		DoWindow(WINDOWEQ2, 1);
	}
	else
	{
		CheckBoxSetValue(WINDOWEQ, ID_CHK_SEPARATELR, 0);
		g_sets.eqflags &= ~MASEPARATELR;
		// DoWindow(WINDOWEQ2, 0);
	}

	EqDoParamChange();

	if(g_w[WINDOWEQ])
	{
		if(g_sets.eqflags & MASEPARATELR)
			masprintf(mastr, MAXLOGLINELEN, L"%ls Equalizer Left Channel", g_prognametitle);
		else
			masprintf(mastr, MAXLOGLINELEN, L"%ls Equalizer", g_prognametitle);
		WindowSetTitle(WINDOWEQ, mastr);
	}
	
	if(g_w[WINDOWEQ2])
	{
		if(g_sets.eqflags & MASEPARATELR)
			masprintf(mastr, MAXLOGLINELEN, L"%ls Equalizer Right Channel", g_prognametitle);
		else
			masprintf(mastr, MAXLOGLINELEN, L"%ls Equalizer Right Channel (Off)", g_prognametitle);
		WindowSetTitle(WINDOWEQ2, mastr);
	}
	
} // DoSeparateLR


void SetAppWindowTitle(machar *msg)
{
	fschar *p;
	fschar name[STRLEN256];
	machar mastr2[STRLEN512];
#if !defined(__MAWIN__)
	machar mastr3[MAXLOGLINELEN];	
#endif
	
	if(g_playlistpath[0] != fsT('\0'))
	{
		FilenameFromPathname(g_playlistpath, name);
		if((p = fsstrstr(name, g_extstr[kMAMAF])) != 0)
			*p = fsT('\0');
		else
			name[0] = fsT('\0');
	}
	else
		name[0] = fsT('\0');

	mastrcpy(mastr2, g_prognametitle);
	mastrcat(mastr2, L" ");
	mastrcat(mastr2, fs2ma(mastr3, name));
	mastrcat(mastr2, L" ");
	mastrcat(mastr2, msg);
	WindowSetTitle(WINDOWAPP, mastr2);

} // SetAppWindowTitle


void SetAppWindowMsg(machar *msg) // SetAppWindowTitle without argv0
{
	fschar *p;
	fschar name[STRLEN256];
	machar mastr2[STRLEN512];
#if !defined(__MAWIN__)
	machar mastr3[MAXLOGLINELEN];	
#endif
	
	if(g_playlistpath[0] != fsT('\0'))
	{
		FilenameFromPathname(g_playlistpath, name);
		if((p = fsstrstr(name, g_extstr[kMAMAF])) != 0)
			*p = fsT('\0');
	}
	else
		name[0] = fsT('\0');
		
	mastrcpy(mastr2, fs2ma(mastr3, name));
	mastrcat(mastr2, L" ");
	mastrcat(mastr2, msg);
	WindowSetTitle(WINDOWAPP, mastr2);

} // SetAppWindowMsg


void ListWindowSetTitle(machar *msg)
{
	machar mastr2[STRLEN512];
	
	mastrcpy(mastr2, g_prognametitle);
	mastrcat(mastr2, L" ");
	mastrcat(mastr2, msg);
	WindowSetTitle(WINDOWLIST, mastr2);

} // ListWindowSetTitle


void Init_g_k()
{
	int i;

	for(i = 0; i < kOUTMAX; i++)
		g_k[i] = 0;
	
	// init indices for g_sets.fileout[g_k[]]

	i = 1; // kOUTFIRST to kOUTLAST

 #ifdef ENCODEMP3
	g_k[i++] = 1;
 #endif
 
#ifdef ENCODEOGG
	g_k[i++] = 2;
#endif
 
 #ifdef ENCODEFLAC
	g_k[i++] = 3;
 #endif
 
 #ifdef ENCODEAAC
	g_k[i++] = 4;
 #endif
 
 #ifdef ENCODEMP4
	g_k[i++] = 5;
 #endif
 
 #ifdef ENCODEWMA
	g_k[i++] = 6;
 #endif

 #ifdef ENCODEWAV
	g_k[i++] = 7;
 #endif
 
 #ifdef ENCODEAIF
	g_k[i++] = 8;
 #endif
 
 #ifdef ENCODERAW
	g_k[i++] = 9;
 #endif
 
} // void Init_g_k()


void CheckFileoutSettings()
{
 #if (defined(ENCODEMP3) || defined(ENCODEOGG) || defined(ENCODEFLAC) || defined(ENCODEAAC) \
  || defined(ENCODEMP4) || defined(ENCODEWMA) || defined(ENCODEWAV)  || defined(ENCODEAIF))
	int i = 1; // kOUTFIRST
 #endif

 #ifdef ENCODEMP3
	if(g_sets.fileout[g_k[i]].type != kMAMP3)
		LoadFactoryFileout(i);
	i++;
 #endif
 
#ifdef ENCODEOGG
	if(g_sets.fileout[g_k[i]].type != kMAOGG)
		LoadFactoryFileout(i);
	i++;
 #endif
 
 #ifdef ENCODEFLAC
	if(g_sets.fileout[g_k[i]].type != kMAFLAC)
		LoadFactoryFileout(i);
	i++;
 #endif
 
 #ifdef ENCODEAAC
	if(g_sets.fileout[g_k[i]].type != kMAAAC)
		LoadFactoryFileout(i);
	i++;
 #endif
 
 #ifdef ENCODEMP4
	if(g_sets.fileout[g_k[i]].type != kMAMP4)
		LoadFactoryFileout(i);
	i++;
 #endif
 
 #ifdef ENCODEWMA
	if(g_sets.fileout[g_k[i]].type != kMAWMA)
		LoadFactoryFileout(i);
	i++;
 #endif

 #ifdef ENCODEWAV
	if(g_sets.fileout[g_k[i]].type != kMAWAV)
		LoadFactoryFileout(i);
	i++;
 #endif
 
 #ifdef ENCODEAIF
	if(g_sets.fileout[g_k[i]].type != kMAAIF)
		LoadFactoryFileout(i);
	i++;
 #endif
 
 #ifdef ENCODERAW
	if(g_sets.fileout[g_k[i]].type != kMARAW)
		LoadFactoryFileout(i);
	i++;
 #endif
} // CheckFileoutSettings


int CheckSettings(MASETS *psets)
{
	int i, j;
	int mx; // e.g. 1024
	int my; // e.g.  768

	DisplayGetSize(&mx, &my);
	
	if(psets->winconf < 0 || psets->winconf >= NUMWINCONF)
		psets->winconf = 0;

	if(psets->mawin[WINDOWABOUT].xpos[psets->winconf] < XPOSMIN) psets->mawin[WINDOWABOUT].xpos[psets->winconf] = ABOUTXPOSDEF;
	if(psets->mawin[WINDOWABOUT].ypos[psets->winconf] < YPOSMIN) psets->mawin[WINDOWABOUT].ypos[psets->winconf] = ABOUTYPOSDEF;

	if(psets->mawin[WINDOWKEYBINDINGS].xpos[psets->winconf] < XPOSMIN) psets->mawin[WINDOWKEYBINDINGS].xpos[psets->winconf] = KEYBINDINGSXPOSDEF;
	if(psets->mawin[WINDOWKEYBINDINGS].ypos[psets->winconf] < YPOSMIN) psets->mawin[WINDOWKEYBINDINGS].ypos[psets->winconf] = KEYBINDINGSYPOSDEF;

	if(psets->mawin[WINDOWADVANCED].xpos[psets->winconf] < XPOSMIN) psets->mawin[WINDOWADVANCED].xpos[psets->winconf] = ADVANCEDXPOSDEF;
	if(psets->mawin[WINDOWADVANCED].ypos[psets->winconf] < YPOSMIN) psets->mawin[WINDOWADVANCED].ypos[psets->winconf] = ADVANCEDYPOSDEF;

	if(psets->mawin[WINDOWLOG].xpos[psets->winconf] < XPOSMIN) psets->mawin[WINDOWLOG].xpos[psets->winconf] = LOGXPOSDEF;
	if(psets->mawin[WINDOWLOG].ypos[psets->winconf] < YPOSMIN) psets->mawin[WINDOWLOG].ypos[psets->winconf] = LOGYPOSDEF;
	if(psets->mawin[WINDOWLOG].xpos[psets->winconf] > mx - abs(XPOSMIN)) psets->mawin[WINDOWLOG].xpos[psets->winconf] = LOGXPOSDEF;
	if(psets->mawin[WINDOWLOG].ypos[psets->winconf] > mx - abs(YPOSMIN)) psets->mawin[WINDOWLOG].ypos[psets->winconf] = LOGYPOSDEF;
	if(psets->mawin[WINDOWLOG].xsize[psets->winconf] >= abs(mx - psets->mawin[WINDOWLOG].xpos[psets->winconf])) psets->mawin[WINDOWLOG].xsize[psets->winconf] = LOGXSIZEDEF;
	if(psets->mawin[WINDOWLOG].ysize[psets->winconf] >= abs(my - psets->mawin[WINDOWLOG].ypos[psets->winconf])) psets->mawin[WINDOWLOG].ysize[psets->winconf] = LOGYSIZEDEF;
	
	if(psets->mawin[WINDOWREADME].xpos[psets->winconf] < XPOSMIN) psets->mawin[WINDOWREADME].xpos[psets->winconf] = READMEXPOSDEF;
	if(psets->mawin[WINDOWREADME].ypos[psets->winconf] < YPOSMIN) psets->mawin[WINDOWREADME].ypos[psets->winconf] = READMEYPOSDEF;
	if(psets->mawin[WINDOWREADME].xpos[psets->winconf] > mx - abs(XPOSMIN)) psets->mawin[WINDOWREADME].xpos[psets->winconf] = READMEXPOSDEF;
	if(psets->mawin[WINDOWREADME].ypos[psets->winconf] > mx - abs(YPOSMIN)) psets->mawin[WINDOWREADME].ypos[psets->winconf] = READMEYPOSDEF;
	if(psets->mawin[WINDOWREADME].xsize[psets->winconf] >= abs(mx - psets->mawin[WINDOWREADME].xpos[psets->winconf])) psets->mawin[WINDOWREADME].xsize[psets->winconf] = READMEXSIZEDEF;
	if(psets->mawin[WINDOWREADME].ysize[psets->winconf] >= abs(my - psets->mawin[WINDOWREADME].ypos[psets->winconf])) psets->mawin[WINDOWREADME].ysize[psets->winconf] = READMEYSIZEDEF;
	
	if(psets->mawin[WINDOWCATALOG].xpos[psets->winconf] < XPOSMIN) psets->mawin[WINDOWCATALOG].xpos[psets->winconf] = CATALOGXPOSDEF;
	if(psets->mawin[WINDOWCATALOG].ypos[psets->winconf] < YPOSMIN) psets->mawin[WINDOWCATALOG].ypos[psets->winconf] = CATALOGYPOSDEF;
	if(psets->mawin[WINDOWCATALOG].xpos[psets->winconf] > mx - abs(XPOSMIN) ) psets->mawin[WINDOWCATALOG].xpos[psets->winconf] = CATALOGXPOSDEF;
	if(psets->mawin[WINDOWCATALOG].ypos[psets->winconf] > mx - abs(YPOSMIN) ) psets->mawin[WINDOWCATALOG].ypos[psets->winconf] = CATALOGYPOSDEF;
	if(psets->mawin[WINDOWCATALOG].xsize[psets->winconf] >= abs(mx - psets->mawin[WINDOWCATALOG].xpos[psets->winconf])) psets->mawin[WINDOWCATALOG].xsize[psets->winconf] = CATALOGXSIZEDEF;
	if(psets->mawin[WINDOWCATALOG].ysize[psets->winconf] >= abs(my - psets->mawin[WINDOWCATALOG].ypos[psets->winconf])) psets->mawin[WINDOWCATALOG].ysize[psets->winconf] = CATALOGYSIZEDEF;
	if(psets->mawin[WINDOWCATALOG].xsize[psets->winconf] < XSIZEMIN) psets->mawin[WINDOWCATALOG].xsize[psets->winconf] = CATALOGXSIZEDEF;
	if(psets->mawin[WINDOWCATALOG].ysize[psets->winconf] < YSIZEMIN) psets->mawin[WINDOWCATALOG].ysize[psets->winconf] = CATALOGYSIZEDEF;
	
	if(psets->mawin[WINDOWPLAYLIST].xpos[psets->winconf] < XPOSMIN) psets->mawin[WINDOWPLAYLIST].xpos[psets->winconf] = PLAYLISTXPOSDEF;
	if(psets->mawin[WINDOWPLAYLIST].ypos[psets->winconf] < YPOSMIN) psets->mawin[WINDOWPLAYLIST].ypos[psets->winconf] = PLAYLISTYPOSDEF;
	if(psets->mawin[WINDOWPLAYLIST].xpos[psets->winconf] > mx - abs(XPOSMIN) ) psets->mawin[WINDOWPLAYLIST].xpos[psets->winconf] = PLAYLISTXPOSDEF;
	if(psets->mawin[WINDOWPLAYLIST].ypos[psets->winconf] > mx - abs(YPOSMIN) ) psets->mawin[WINDOWPLAYLIST].ypos[psets->winconf] = PLAYLISTYPOSDEF;
	if(psets->mawin[WINDOWPLAYLIST].xsize[psets->winconf] >= abs(mx - psets->mawin[WINDOWPLAYLIST].xpos[psets->winconf])) psets->mawin[WINDOWPLAYLIST].xsize[psets->winconf] = PLAYLISTXSIZEDEF;
	if(psets->mawin[WINDOWPLAYLIST].ysize[psets->winconf] >= abs(my - psets->mawin[WINDOWPLAYLIST].ypos[psets->winconf])) psets->mawin[WINDOWPLAYLIST].ysize[psets->winconf] = PLAYLISTYSIZEDEF;
	if(psets->mawin[WINDOWPLAYLIST].xsize[psets->winconf] < XSIZEMIN) psets->mawin[WINDOWPLAYLIST].xsize[psets->winconf] = PLAYLISTXSIZEDEF;
	if(psets->mawin[WINDOWPLAYLIST].ysize[psets->winconf] < YSIZEMIN) psets->mawin[WINDOWPLAYLIST].ysize[psets->winconf] = PLAYLISTYSIZEDEF;
	
	for(i = 0; i < NUMWINCONF; i++)
	{
		if(psets->mawin[WINDOWAPP].xpos[i] < XPOSMIN) psets->mawin[WINDOWAPP].xpos[i] = i == 0 ? XPOSDEF : XPOSDEF2;
		if(psets->mawin[WINDOWAPP].ypos[i] < YPOSMIN) psets->mawin[WINDOWAPP].ypos[i] = i == 0 ? YPOSDEF : YPOSDEF2;
		if(psets->mawin[WINDOWEQ].xpos[i] < XPOSMIN) psets->mawin[WINDOWEQ].xpos[i] = i == 0 ? EQXPOSDEF : EQXPOSDEF2;
		if(psets->mawin[WINDOWEQ].ypos[i] < YPOSMIN) psets->mawin[WINDOWEQ].ypos[i] = i == 0 ? EQYPOSDEF : EQYPOSDEF2;
		if(psets->mawin[WINDOWLIST].xpos[i] < XPOSMIN) psets->mawin[WINDOWLIST].xpos[i] = i == 0 ? LISTXPOSDEF : LISTXPOSDEF2;
		if(psets->mawin[WINDOWLIST].ypos[i] < YPOSMIN) psets->mawin[WINDOWLIST].ypos[i] = i == 0 ? LISTYPOSDEF : LISTYPOSDEF2;
	
		if(psets->mawin[WINDOWAPP].xsize[i] < XSIZEMIN)	psets->mawin[WINDOWAPP].xsize[i] = i == 0 ? XSIZEDEF : XSIZEDEF2;
		if(psets->mawin[WINDOWAPP].ysize[i] < YSIZEMIN)	psets->mawin[WINDOWAPP].ysize[i] = i == 0 ? YSIZEDEF : YSIZEDEF2;
		if(psets->mawin[WINDOWEQ].xsize[i] < 0)	psets->mawin[WINDOWEQ].xsize[i] = i == 0 ? EQXSIZEDEF : EQXSIZEDEF2;
		if(psets->mawin[WINDOWEQ].ysize[i] < 0)	psets->mawin[WINDOWEQ].ysize[i] = i == 0 ? EQYSIZEDEF : EQYSIZEDEF2;
		if(psets->mawin[WINDOWEQ2].xsize[i] < 0) psets->mawin[WINDOWEQ2].xsize[i] = i == 0 ? EQ2XSIZEDEF : EQ2XSIZEDEF2;
		if(psets->mawin[WINDOWEQ2].ysize[i] < 0) psets->mawin[WINDOWEQ2].ysize[i] = i == 0 ? EQ2YSIZEDEF : EQ2YSIZEDEF2;
		if(psets->mawin[WINDOWLIST].xsize[i] < 0) psets->mawin[WINDOWLIST].xsize[i] = i == 0 ? LISTXSIZEDEF : LISTXSIZEDEF2;
		if(psets->mawin[WINDOWLIST].ysize[i] < 0) psets->mawin[WINDOWLIST].ysize[i] = i == 0 ? LISTYSIZEDEF : LISTYSIZEDEF2;

		if(psets->mawin[WINDOWAPP].xpos[i] > mx) psets->mawin[WINDOWAPP].xpos[i]  = i == 0 ? XPOSDEF : XPOSDEF2;
		if(psets->mawin[WINDOWAPP].ypos[i] > my) psets->mawin[WINDOWAPP].ypos[i]  = i == 0 ? YPOSDEF : YPOSDEF2;
		if(psets->mawin[WINDOWAPP].xsize[i] >= abs(mx - psets->mawin[WINDOWAPP].xpos[i])) psets->mawin[WINDOWAPP].xsize[i] = i == 0 ? XSIZEDEF : XSIZEDEF2;
		if(psets->mawin[WINDOWAPP].ysize[i] >= abs(my - psets->mawin[WINDOWAPP].ypos[i])) psets->mawin[WINDOWAPP].ysize[i] = i == 0 ? YSIZEDEF : YSIZEDEF2;
		if(psets->mawin[WINDOWEQ].xsize[i] >= abs(mx - psets->mawin[WINDOWEQ].xpos[i])) psets->mawin[WINDOWEQ].xsize[i]  = i == 0 ? EQXSIZEDEF : EQXSIZEDEF2;
		if(psets->mawin[WINDOWEQ].ysize[i] >= abs(my - psets->mawin[WINDOWEQ].ypos[i])) psets->mawin[WINDOWEQ].ysize[i]  = i == 0 ? EQYSIZEDEF : EQYSIZEDEF2;
		if(psets->mawin[WINDOWEQ2].xsize[i] >= abs(mx - psets->mawin[WINDOWEQ2].xpos[i])) psets->mawin[WINDOWEQ2].xsize[i] = i == 0 ? EQ2XSIZEDEF : EQ2XSIZEDEF2;
		if(psets->mawin[WINDOWEQ2].ysize[i]	>= abs(my - psets->mawin[WINDOWEQ2].ypos[i])) psets->mawin[WINDOWEQ2].ysize[i] = i == 0 ? EQ2YSIZEDEF : EQ2YSIZEDEF2;
		if(psets->mawin[WINDOWLIST].xsize[i] >= abs(mx - psets->mawin[WINDOWLIST].xpos[i])) psets->mawin[WINDOWLIST].xsize[i] = i == 0 ? LISTXSIZEDEF : LISTXSIZEDEF2;
		if(psets->mawin[WINDOWLIST].ysize[i] >= abs(my - psets->mawin[WINDOWLIST].ypos[i])) psets->mawin[WINDOWLIST].ysize[i] = i == 0 ? LISTYSIZEDEF : LISTYSIZEDEF2;

		for(j = 0; j < 4; j++)
			if(psets->listcolxsize[i][j] < psets->mawin[WINDOWLIST].xsize[i] - SCROLLERWIDTH)
				psets->listcolxsize[i][j] = psets->mawin[WINDOWLIST].xsize[i] - SCROLLERWIDTH;
	} // for i

	// check parameter sanity
	if(psets->balance)
	{
	 if(psets->volumesliderleft < (VOLUMEMIN*2) || psets->volumesliderleft > (VOLUMEMAX*2))		psets->volumesliderleft	 = (VOLUMEMAX - VOLUMEMIN) / 2;
	 if(psets->volumesliderright < (VOLUMEMIN*2) || psets->volumesliderright > (VOLUMEMAX*2))	psets->volumesliderright = (VOLUMEMAX - VOLUMEMIN) / 2;
	}
	else
	{
	 if(psets->volumesliderleft < VOLUMEMIN		|| psets->volumesliderleft	> VOLUMEMAX)		psets->volumesliderleft	 = (VOLUMEMAX - VOLUMEMIN) / 2;
	 if(psets->volumesliderright< VOLUMEMIN		|| psets->volumesliderright	> VOLUMEMAX)		psets->volumesliderright = (VOLUMEMAX - VOLUMEMIN) / 2;
	}
	if(psets->separationslider  < SEPARATIONMIN	|| psets->separationslider  >  SEPARATIONMAX)	psets->separationslider	= (SEPARATIONMAX - SEPARATIONMIN) / 2;
	if(psets->panslider			< SEPARATIONMIN	|| psets->panslider			>  SEPARATIONMAX)	psets->panslider		= (SEPARATIONMAX - SEPARATIONMIN) / 2;
	if(psets->preampslider		< PREAMPMIN		|| psets->preampslider		>  PREAMPMAX)		psets->preampslider		= (PREAMPMAX - PREAMPMIN) / 2;
	if(psets->buffersize		< 0				|| psets->buffersize		>  SONGMEMMAX)		psets->buffersize		= SONGMEMDEF;
	if(psets->buffertype		< 0				|| psets->buffertype		>= BUFUSEMAX)		psets->buffertype		= BUFUSEMEM;
	if(psets->output			< 0				|| psets->output			>  kOUTLAST)		psets->output			= kOUTSOUND;
	if(psets->eqpreset			< 0				|| psets->eqpreset			>= EQNUMPRESETS)	psets->eqpreset			= 0;

	for(i = 0; i < EQNUMPRESETS; i++)
	{
		if(psets->eqpresets[i].eqoffset > DBRANGE || psets->eqpresets[i].eqoffset < -DBRANGE)
			psets->eqpresets[i].eqoffset = 0;
		if(psets->eq2presets[i].eqoffset > DBRANGE || psets->eq2presets[i].eqoffset < -DBRANGE)
			psets->eq2presets[i].eqoffset = 0;
			
		for(j = 0; j < EQBANDS31; j++)
		{
			if(psets->eqpresets[i].eqslider[j] < DBSLIDERMIN || psets->eqpresets[i].eqslider[j] > DBSLIDERMAX)
				psets->eqpresets[i].eqslider[j] = (DBSLIDERMAX - DBSLIDERMIN) / 2;
			if(psets->eq2presets[i].eqslider[j] < DBSLIDERMIN || psets->eq2presets[i].eqslider[j] > DBSLIDERMAX)
				psets->eq2presets[i].eqslider[j] = (DBSLIDERMAX - DBSLIDERMIN) / 2;
		}

 		if(fsstrlen(psets->eqpresets[i].eqname) >= EQNAMELEN)
			(psets->eqpresets[i].eqname)[EQNAMELEN - 1] = fsT('\0');
	}
	
	// listbox
	if((psets->listboxmode & (LBOXPLAYERINFO | LBOXPLAYLIST | LBOXSONGSLOADED | LBOXSONGINFO)) == 0)
		psets->listboxmode |= LBOXPLAYERINFO;
	
#if defined(MAKEYBOARD) && defined(MAGUINONE)
	psets->keyboard = 1;
#endif

	CheckFileoutSettings();
	
	// various strings
	psets->audiohostinname[MAX_PATH - 1] = 0;
	psets->audiohostoutname[MAX_PATH - 1] = 0;
	psets->audiodevinname[MAX_PATH - 1] = 0;
	psets->audiodevoutname[MAX_PATH - 1] = 0;
	psets->futurepath[MAX_PATH - 1] = 0;

	return 0;

} // CheckSettings()


// for updating settings from a previous settings version
#define MOREAMP_SETTINGS_VERSION_STRING "0.1.29"				// first moreamp version to use this settings format
#define MOREAMP_SETTINGS_VERSION_STRINGL L"0.1.29"				// wchar_t of same
#define MOREAMP_SETTINGS_VERSION_LEN 6							// actual length of version string (not VERSIONSTRINGSIZE)
#define PREVIOUS_MOREAMP_SETTINGS_VERSION_STRING "0.1.27"		// previous settings version that needs updating to MOREAMP_SETTINGS_VERSION_STRING
#define PREVIOUS_MOREAMP_SETTINGS_VERSION_LEN 6					// actual length of previous version string (not VERSIONSTRINGSIZE)
#define PREVIOUSMASETS MASETS_127								// previous MASETS struct
#define FixSettingsPrevious FixSettings_129						// function that gets called to fix the new settings
#define EQNAMELEN22 32
#define MAX_PATH22 260
#define VERSIONSTRINGSIZE22 15
void FixSettings_129(FILE *fp, MASETS *psets);

typedef struct
{
	int autostart;					// start player when app is launched
	int random;						// get songs from playlist randomly
	int reptplayed;					// allow to repeat a song when in random mode 
	int looploaded;					// loop on songs that have been loaded in mem or vdisk
	int looplist;					// loop on playlist else stop when all have played 
	int highpriority;				// decoder task riority
	int effects; 					// effects active
	int limiter; 					// mad limiter
	int balance;					// free or joined left and right volume controls
	int monodiff;					// L-R in mono; reverse L,R in stereo
	int invertright;				// invert right channel
	int eqflags;					// equalizer flags
	int eqpreset;					// eq preset index
	int volumesliderleft; 			// volume L value
	int volumesliderright; 			// volume R value
	int separationslider; 			// separation value (stereo) 
	int panslider;					// pan value (mono)
	int preampslider;   			// preamp value (mad)
	EQUPRESET eqpresets[EQNUMPRESETS];  // equalizer values
	int pitchslider;				// pitch effect value
	int pitchslider2;				// pitch effect value (fine)
	int temposlider;				// tempo effect value
	int speedslider;				// speed effect value
	fschar vdiskname[MAX_PATH]; 	// virtual disk name
	int buffertype;					// type of buffering for songs (mem, vdisk, direct)
	int output;						// output to use from Output popup
	int buffersize;					// max mem or vdisk used, in Mbytes
	int titleinfo;					// put info in window's title bar
	int winconf;					// player window configuration
	int singleplay;					// stop afer every song
	FILEOUT fileout[kOUTMAX];		// file output parameters	
	int logflags;					// stop or limit logging
	int logmax;						// max log entries per session
	int logevents;					// max log entries per fast log period
	int logperiod;					// fast log period
	int multipleinst;				// multiple instances
	int keyboard;					// use keyboard
	int listcolxsize[NUMWINCONF][4];// h scrollable if wider than list window 
	int mousewheel;					// use mousewheel
	int usem3u;						// use m3u files
	int addmode;					// add new files to existing madrop entries
	EQUPRESET eq2presets[EQNUMPRESETS]; // equalizer 2 values

	// new values in this settings version 0 1 27 :
	MAWINDOW mawin[NUMWINDOWS];		// all windows
	int listboxmode;				// show list window | which to show
	int exitafter;					// exit after list has played (if no looplist)
	int future1;					// for future use before having to update settings version
	int future2;
	int future3;
	int future4;
	int future5;
	int future6;
	int future7;

	char settingsversion[VERSIONSTRINGSIZE]; // first version to use this settings format, keep this here (last)
} MASETS_127;


// #define FIXSETTTINGSTEST  // this is for testing our code, not for release
void FixSettings_129(FILE *fp, MASETS *psets)
{
#if defined(FIXSETTTINGSTEST)
	int n;
	char marker;
	char *p;
#endif

	PREVIOUSMASETS prev_sets;
	machar mastr[MAXLOGLINELEN];

	// get previous settings
	if(fseek(fp, 0L - sizeof(PREVIOUSMASETS), SEEK_END) != 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"FixSettings_129 fseek errno=%d", errno);
		maLog(mastr);
	}

	if(fread(&prev_sets, 1, sizeof(PREVIOUSMASETS), fp) != sizeof(PREVIOUSMASETS))
	{
		masprintf(mastr, MAXLOGLINELEN, L"FixSettings_129 fread errno=%d", errno);
		maLog(mastr);
	}
	
#if defined(FIXSETTTINGSTEST)
	memset((void*)psets, 0, sizeof(MASETS)); // set markers
	// set all new new settings bytes to some unique marker not in previous settings
	// so we can later see what wasn't updated.
	j = 0;
	n = 255;
	marker = 0;
	while(n-- > 0)
	{
		p = (char *)&prev_sets;
		marker++;
		for(i = 0; i < sizeof(PREVIOUSMASETS); i++)
			if(*p++ == marker)
				break;
		if(i >= sizeof(PREVIOUSMASETS))
			break;
	}
	if(n <= 0)
		maLog(L"FixSettings_129 no marker found");
	memset((void*)psets, marker, sizeof(MASETS)); // set markers
#endif

	// recover all previous settings
	memcpy(psets, &prev_sets, sizeof(MASETS));

	// init new setings
	psets->inputmonitor = 1;
	psets->hostinenable = 0;
	psets->audiohostinname[0] = 0;
	psets->audiohostoutname[0] = 0;
	psets->audiodevinname[0] = 0;
	psets->audiodevoutname[0] = 0;
	psets->futurepath[0] = 0;
	strcpy(psets->settingsversion, MOREAMP_SETTINGS_VERSION_STRING);
	fsstrcpy(g_sets.fileout[kOUTRAW].outdir, g_fileoutdir);
#if defined(FIXSETTTINGSTEST)
	// check that everything was updated. All markers should have been overwritten
	if(n > 0)
	{
		j = 0;
		p = (char *)psets;
		for(i = 0; i < sizeof(MASETS); i++)
		{
			if(*p++ == marker)
			{
				masprintf(mastr, MAXLOGLINELEN, L"FixSettings_129 missed something %lu-%lu",
					(unsigned long)j, (unsigned long)i);
				maLog(mastr);
				break; // only show the first missed setting
			}
			else
				j = i;
		}
		/*
		// show some MASETS member offsets for locating the missed setting shown above
		masprintf(mastr, MAXLOGLINELEN, L"FixSettings_129 autostart %lu", (void *)&psets->autostart - (void *)&psets->autostart); maLog(mastr);
		masprintf(mastr, MAXLOGLINELEN, L"FixSettings_129 preampslider %lu", (void *)&psets->preampslider - (void *)&psets->autostart); maLog(mastr);
		masprintf(mastr, MAXLOGLINELEN, L"FixSettings_129 pitchslider %lu", (void *)&psets->pitchslider - (void *)&psets->autostart); maLog(mastr);
		masprintf(mastr, MAXLOGLINELEN, L"FixSettings_129 buffertype %lu", (void *)&psets->buffertype - (void *)&psets->autostart); maLog(mastr);
		masprintf(mastr, MAXLOGLINELEN, L"FixSettings_129 buffersize %lu", (void *)&psets->buffersize - (void *)&psets->autostart); maLog(mastr);
		masprintf(mastr, MAXLOGLINELEN, L"FixSettings_129 singleplay %lu", (void *)&psets->singleplay - (void *)&psets->autostart); maLog(mastr);
		masprintf(mastr, MAXLOGLINELEN, L"FixSettings_129 fileout %lu", (void *)&psets->fileout - (void *)&psets->autostart); maLog(mastr);
		masprintf(mastr, MAXLOGLINELEN, L"FixSettings_129 logflags %lu", (void *)&psets->logflags - (void *)&psets->autostart); maLog(mastr);
		masprintf(mastr, MAXLOGLINELEN, L"FixSettings_129 listcolxsize[0][0] %lu", (void *)&psets->listcolxsize[0][0]  - (void *)&psets->autostart); maLog(mastr);
		masprintf(mastr, MAXLOGLINELEN, L"FixSettings_129 mousewheel %lu", (void *)&psets->mousewheel - (void *)&psets->autostart); maLog(mastr);
		masprintf(mastr, MAXLOGLINELEN, L"FixSettings_129 eq2presets[0] %lu", (void *)&psets->eq2presets[0].eqslider[0] - (void *)&psets->autostart); maLog(mastr);
		masprintf(mastr, MAXLOGLINELEN, L"FixSettings_129 listboxmode %lu",  (void *)&psets->listboxmode - (void *)&psets->autostart); maLog(mastr);
		masprintf(mastr, MAXLOGLINELEN, L"FixSettings_129 future1 %lu", (void *)&psets->future1 - (void *)&psets->autostart); maLog(mastr);
		masprintf(mastr, MAXLOGLINELEN, L"FixSettings_129 hostinenable %lu", (void *)&psets->hostinenable - (void *)&psets->autostart); maLog(mastr);
		masprintf(mastr, MAXLOGLINELEN, L"FixSettings_129 settingsversion %lu", (void *)psets->settingsversion - (void *)&psets->autostart); maLog(mastr);
		masprintf(mastr, MAXLOGLINELEN, L"FixSettings_129 sizeof(MASETS) %lu", sizeof(MASETS)); maLog(mastr);
		*/
	}
#endif

	// FixDefaultSettings_0xx();

} // FixSettings_129


int maCheckSettingsVersion(char *settingsversion)
{
	// currrent settings version
	if(!strncmp(settingsversion, MOREAMP_SETTINGS_VERSION_STRING, (size_t)MOREAMP_SETTINGS_VERSION_LEN))
		return MASETTINGSVERSIONISCURRENT; // current settings don't change

#ifdef PREVIOUS_MOREAMP_SETTINGS_VERSION_STRING
	// previous settings version
	if(!strncmp(settingsversion, PREVIOUS_MOREAMP_SETTINGS_VERSION_STRING, (size_t)PREVIOUS_MOREAMP_SETTINGS_VERSION_LEN))
		return MASETTINGSVERSIONISPREVIOUS; // current settings will be copied from previous version
#endif

	// any other unknown (older or newer) settings version
	return MASETTINGSVERSIONISUNKNOWN; // current settings will be set to factory defaults
}


int LoadSettings() // load settings from a settings file, or create one if none exists or if error
{
	int ret = 0;
	int ret2 = 0;
	int skip = 0;
	int version;
	char settingsversion[VERSIONSTRINGSIZE];
	FILE *fp;
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
#endif
	machar mastr[MAXLOGLINELEN];

	if(g_songsnum <= 0 && !PlaylistIsCda())
		g_songsnum = 0;

	if(g_settingspath[0] == fsT('\0'))
	{
		maLog(L"LoadSettings cannot load settings, no file name");
		return 2;	
	}
			
	if(PlaylistIsCda())
	{
		if(!fsstrcasecmp(g_settingspath, l_settingscda))
			skip = 1;
	}
	else
	{
		if(!fsstrcasecmp(g_settingspath, g_settingsdrop))
			skip = 1;
	}

	ret = g_songsnum;
	if(skip && !fsstrcmp(l_settingspathlast, g_settingspath)) // don't read settings
		ret2 = 1;
	else if((fp = mafopen(g_settingspath, fsT("rb"))) != NULL) // reread settings
	{
		// check version
		if(fseek(fp, 0L - sizeof(g_sets.settingsversion), SEEK_END) != 0)
		{
			masprintf(mastr, MAXLOGLINELEN, L"LoadSettings fseek errno=%d", errno);
			maLog(mastr);
		}

		if(fread(settingsversion, 1, sizeof(g_sets.settingsversion), fp) != sizeof(g_sets.settingsversion))
		{
			masprintf(mastr, MAXLOGLINELEN, L"LoadSettings fread errno=%d", errno);
			maLog(mastr);
		}

		if((version = maCheckSettingsVersion(settingsversion)) == MASETTINGSVERSIONISCURRENT)
		{
			// check size
			if(!skip)
			{
				if(fseek(fp, 0, SEEK_END) != 0)
				{
					masprintf(mastr, MAXLOGLINELEN, L"LoadSettings fseek errno=%d", errno);
					maLog(mastr);
				}
				if(ftell(fp) != sizeof(MASETS))
					ret = -1; // bad size
			}
			if(ret != -1) // size ok
			{
				// get settings
				rewind(fp);
				ret2 = (int)fread(&g_sets, sizeof(MASETS), 1, fp);
				memcpy(&g_setsbu, &g_sets, sizeof(MASETS));
				fsstrcpy(l_settingspathlast, g_settingspath);
			}
		}
#ifdef PREVIOUS_MOREAMP_SETTINGS_VERSION_STRING
		else if(version == MASETTINGSVERSIONISPREVIOUS)
		{
			// check size
			if(!skip)
			{
				if(fseek(fp, 0, SEEK_END) != 0)
				{
					masprintf(mastr, MAXLOGLINELEN, L"LoadSettings fseek errno=%d", errno);
					maLog(mastr);
				}
				if(ftell(fp) != (long)(/* sizeof(MARECORD) * g_songsnum + */ sizeof(PREVIOUSMASETS)))
					ret = -1; // bad size, could be either bad song records or bad settings
			}
			if(ret != -1) // size ok
			{
				ret = g_songsnum;
				
				// get settings
				FixSettingsPrevious(fp, &g_sets);
				ret2 = 1;
				memcpy(&g_setsbu, &g_sets, sizeof(MASETS));
				fsstrcpy(l_settingspathlast, g_settingspath);
				
				fclose(fp);
				fp = NULL;
				SaveSettings();
				masprintf(mastr, MAXLOGLINELEN, L"LoadSettings updated '%ls' to settings version %ls",
					fs2ma(mastr2, g_settingspath), MOREAMP_SETTINGS_VERSION_STRINGL);
				maLog(mastr);
			}
		}
#endif
		else // MASETTINGSVERSIONISUNKNOWN
		{
			masprintf(mastr, MAXLOGLINELEN, L"LoadSettings '%ls' unable to determine if previous settings version is %ls",
				fs2ma(mastr2, g_settingspath), MOREAMP_SETTINGS_VERSION_STRINGL);
			maLog(mastr);
		}
		if(fp)
			fclose(fp);		
	}
	else
		ret = ret2 = -2; // cannot open settings file

	// error or wrong size, make a new settings file :	
	if(ret != g_songsnum || ret2 != 1)
	{			
		// if dropping songs try to keep previous settings
		if(ret2 != 1)
		{
#if !defined(NODECODEAUDIOIN)
			if(!fsstrcasecmp(g_settingspath, l_settingsaudioin))
			{
				if(LoadFactorySettingsAudioIn(&g_sets)) // get audioin factory settings
					return 4;
			}
			else
#endif
			{
				if(LoadDefaultSettings(&g_sets)) // get settings from defaults
					return 4;
			}
		}
					
		memcpy(&g_setsbu, &g_sets, sizeof(MASETS));

		if(SaveSettings())
			return 6;

		fsstrcpy(l_settingspathlast, g_settingspath);
		
	} // if error or wrong size
	
	CheckSettings(&g_sets);

	if(g_thisinstance == 1)
		if(ret2 == 1) 
			SetMultipleInstances();
		// else was done in LoadDefaultSettings or LoadFactorySettings

	return 0;

} // LoadSettings


int SaveSettings()
{
	size_t ret;
	FILE *fp;
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
#endif
	machar mastr[MAXLOGLINELEN];
	
	if(g_settingspath[0] == fsT('\0'))
	{
		masprintf(mastr, MAXLOGLINELEN, L"SaveSettings failed, null settings path");
		maLog(mastr);
		return 6;
	}

	if((fp = mafopen(g_settingspath, fsT("wb"))) == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"SaveSettings mafopen failed on '%ls'", fs2ma(mastr2, g_settingspath));
		maLog(mastr);
		return 3;
	}
	
#if defined(__MAOSX__) || defined(__MAUNIX__)
	machmod(g_settingspath, ACCESSPERMS); // 0777
#endif

	strcpy(g_sets.settingsversion, MOREAMP_SETTINGS_VERSION_STRING);
	strcpy(g_setsbu.settingsversion, MOREAMP_SETTINGS_VERSION_STRING);

	ret = fwrite((void *)&g_setsbu, sizeof(MASETS), 1, fp);
		
	fclose(fp);
	
	if(ret != 1)
	{
		masprintf(mastr, MAXLOGLINELEN, L"SaveSettings fwrite blocks=%d erno=%d '%ls'",
											ret, errno, fs2ma(mastr2, g_settingspath));
		maLog(mastr);	
		return 5;
	}
	
	return 0;
	
} // SaveSettings


#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
int LoadSettingsCDRom()
{
	int ret = 0;
	FILE *fp;
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];
		
	if(l_setscdrompath[0] == fsT('\0'))
	{
		maLog(L"LoadSettingsCDRom cannot load settings, no file name");
		return 1;	
	}
	if((fp = mafopen(l_setscdrompath, fsT("rb"))) == NULL)
	{
		if(errno != ENOENT) // 2
		{
			masprintf(mastr, MAXLOGLINELEN, L"LoadSettingsCDRom mafopen errno=%d '%ls'",
													errno, fs2ma(mastr2, l_setscdrompath));
			maLog(mastr);
		}
		maLog(L"LoadSettingsCDRom setting factory cdrom settings");
		LoadFactorySettingsCDRom();
		SaveSettingsCDRom();
		return 1;
	}

	if((ret = fread((void *)&g_setscdrom, sizeof(MASETSCDROM), 1, fp)) != 1)
	{			
		fclose(fp);
		masprintf(mastr, MAXLOGLINELEN, L"LoadSettingsCDRom fread errno=%d '%ls'",
													errno, fs2ma(mastr2, l_setscdrompath));
		maLog(mastr);
		maLog(L"LoadSettingsCDRom setting factory cdrom settings");
		LoadFactorySettingsCDRom();
		SaveSettingsCDRom();
		return 1;
	}
	
	fclose(fp);

	return 0;
	
} // LoadSettingsCDRom


int SaveSettingsCDRom()
{
	int ret;
	FILE *fp;
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];
	
	if((fp = mafopen(l_setscdrompath, fsT("wb"))) == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"SaveSettingsCDRom mafopen failed on '%ls'",
																fs2ma(mastr2, l_setscdrompath));
		maLog(mastr);
		return 6;
	}
	machmod(l_setscdrompath, ACCESSPERMS); // 0777
	ret = fwrite((void *)&g_setscdrom, sizeof(MASETSCDROM), 1, fp);
	fclose(fp);
	if(ret != 1)
	{
		masprintf(mastr, MAXLOGLINELEN, L"SaveSettingsCDRom blocks=%d fwrite '%ls'",
														ret, fs2ma(mastr2, l_setscdrompath));
		maLog(mastr);	
		return 7;
	}

	return 0;
	
} // SaveSettingsCDRom
#endif


int CopySettings()  // prompt for a settings file and load
{
	MASETS sets;
	FILE *fp;
	int version;
	long s;
	int multipleinst = g_sets.multipleinst;
	char settingsversion[VERSIONSTRINGSIZE];
	fschar settingspath[MAX_PATH];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
#endif
	machar mastr[MAXLOGLINELEN];

	masprintf(mastr, MAXLOGLINELEN, L"%ls (*.mas)|*.mas|Any file (*)|*", g_prognametitle);
	if(maFileDialog(settingspath, WINDOWAPP, L"Open a settings file (.mas)",
								g_settingsdir, L"", mastr, maOPEN | maFILE_MUST_EXIST))
		return 1;

	if((fp = mafopen(settingspath, fsT("rb"))) == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CopySettings cannot open file '%ls'",
																fs2ma(mastr2, settingspath));
		ShowErr(mastr);
		return 1;
	}

	// check extension
	if(!FileNameHasExtension(settingspath, EXTMAS))
	{
		masprintf(mastr, MAXLOGLINELEN, L"CopySettings\nFile\n'%ls'\nis not a settings file (.mas)",
															fs2ma(mastr2, settingspath));
		ShowErr(mastr);
		fclose(fp);
		return 1;
	}
	
	// check version
	fseek(fp, 0L - sizeof(sets.settingsversion), SEEK_END);
	if(fread(settingsversion, 1, sizeof(settingsversion), fp) != sizeof(settingsversion))
	{
		masprintf(mastr, MAXLOGLINELEN, L"CopySettings fread errno %d on '%ls'",
														errno, fs2ma(mastr2, settingspath));
		ShowErr(mastr);
		fclose(fp);
		return 1;
	}
	
	if((version = maCheckSettingsVersion(settingsversion)) != MASETTINGSVERSIONISCURRENT)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CopySettings\nFile\n'%ls'\nis an older or damaged settings file\nor not a settings file.",
															fs2ma(mastr2, settingspath));
		ShowErr(mastr);
		fclose(fp);
		return 1;
	}

	// get settings
	s = sizeof(MASETS);
	if(fseek(fp, -s, SEEK_END) != 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"CopySettings fseek errno=%d", errno);
		maLog(mastr);
	}
	if(fread(&sets, 1, sizeof(MASETS), fp) != sizeof(MASETS))
	{
		masprintf(mastr, MAXLOGLINELEN, L"CopySettings fread errno %d on '%ls'", errno, fs2ma(mastr2, settingspath));
		ShowErr(mastr);
		fclose(fp);
		return 1;
	}
	
	memcpy(&g_sets, &sets, sizeof(MASETS));

	fclose(fp);		

	CheckSettings(&g_sets);

	if(multipleinst != g_sets.multipleinst && g_thisinstance == 1)
		SetMultipleInstances();
	
	return 0;

} // CopySettings


int LoadRecords()	// allocate in mem a copy of the records file
{					// load records from the records file, or recreate it if it doesn't exist or if error
	int i;
	int size = 0;
	FILE *fp;
	machar mastr[MAXLOGLINELEN];
		
	if(g_songsnum <= 0)
	{
		maLog(L"LoadRecords g_songsnum <= 0");
		return 1;
	}

	if(l_psongrec)
	{
		MAFREE(l_psongrec);
		l_psongrec = NULL;
	}

	l_songsnumplayed = 0;
	
	if(g_songsnum <= 0)
	{
		maLog(L"LoadRecords g_songsnum <= 0");
		return 1;
	}

	if(g_recordspath[0] == fsT('\0'))
	{
		maLog(L"LoadRecords cannot load records, no file name");
		return 0;	
	}
			
	if(AllocSongRec(&l_psongrec, g_songsnum))
		return 1;

	if(PlaylistIsNoRecords()) // we don't save records for these
		size = g_songsnum;
	else if((fp = mafopen(g_recordspath, fsT("rb"))) != NULL) 
	{
		if(fseek(fp, 0, SEEK_END) != 0)
		{
			masprintf(mastr, MAXLOGLINELEN, L"LoadRecords fseek errno=%d", errno);
			maLog(mastr);
		}
		if(ftell(fp) == (long)(sizeof(MARECORD) * g_songsnum))
		{
			rewind(fp);	
			size = (int)fread(l_psongrec, sizeof(MARECORD), g_songsnum, fp);
		}
		fclose(fp);		
	}

	if(size != g_songsnum) // error or wrong size or playlist has changed, make a new records file
	{			
		if(ZeroRecords(&l_psongrec, g_songsnum))
			return 1;
				
		if(SaveRecords())
			return 1;
			
		return 0;
	}

	// cda or file loaded ok, count songs that have played
	for(i = 0; i < g_songsnum; i++)
	{
		if(l_psongrec[i])
			l_songsnumplayed++;
	}

	return 0;

} // LoadRecords


int SaveRecords()
{
	size_t ret;
	FILE *fp;
	machar mastr[MAXLOGLINELEN];
	
	if(g_recordspath[0] == fsT('\0'))
	{
		masprintf(mastr, MAXLOGLINELEN, L"SaveRecords failed, null records path");
		maLog(mastr);
		return 6;
	}

	if((fp = mafopen(g_recordspath, fsT("wb"))) == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"SaveRecords mafopen failed on '%ls'", g_recordspath);
		maLog(mastr);
		return 3;
	}
	
#if defined(__MAOSX__) || defined(__MAUNIX__)
	machmod(g_recordspath, ACCESSPERMS); // 0777
#endif

	// song record		
	if(l_psongrec && g_songsnum > 0)
	{
		ret = fwrite(l_psongrec, sizeof(MARECORD), g_songsnum, fp);
	
		if(ret != (size_t)g_songsnum)
		{
			masprintf(mastr, MAXLOGLINELEN, L"SaveRecords fwrite %d records instead of %d in '%ls'", ret, g_songsnum, g_recordspath);
			maLog(mastr);
			fclose(fp);
			return 4;
		}
	}

	fclose(fp);
	
	return 0;
	
} // SaveRecords


void DoPromptIndexOk() // ok response from the dialog text field (Enter)
{
	int i;
	long outlen;
	machar mastr2[MAPROMPTMAX + 2];

	if(!g_w[WINDOWPROMPTNUM])
		return;
	
	EditTextGetText(WINDOWPROMPTNUM, ID_EDITTEXT_PROMPTNUM, mastr2, MAPROMPTMAX, &outlen, 0, -1);
	
	if(mastrlen(mastr2) > 0)
	{
		if(!string2int(mastr2, &i))
		{
			if(i >= 1 && i <= g_songsnum) // 1-based index of song to play
			{
				if(g_state.paused)
				{
					g_state.silence = 1;
					SetPause(0);
				}
				g_droplaunch = i + 1; // 0-based row to play + 2

				if(g_state.running) // exit song loop
				{
					g_loadedcurrent = g_loadednum = 0;
					g_loadedquit = g_loadedquitB = 1;
				}
				else
					DoTimerPlay(100); // DoPromptIndexOk
			}
		}
	}
	
	DoWindow(WINDOWPROMPTNUM, 0);
	
} // DoPromptIndexOk


#if defined(MAKEYBOARD)
 // #define MAKEYBOARDTRACE
 void ShowKeyStatus(machar *status)
 {
	machar mastr[MAXLOGLINELEN];

	masprintf(mastr, MAXLOGLINELEN, L"key %ls : %ls", l_keypressed, status);
	WindowSetTitle(WINDOWAPP, mastr);
 #if !defined(MAGUINONE)
	l_temptitle = 1;
 #endif
 }


 void OnKeyButtonQuit()
 {
	ShowKeyStatus(L"Quit");
	DoAppQuit();
 }


 void OnKeyButtonStop()
 {
	ShowKeyStatus(L"Stop");
	DoStop();
 }


 void OnKeyButtonPause()
 {
	DoPause();
	if(g_state.paused)
		ShowKeyStatus(L"Pause on");
	else
		ShowKeyStatus(L"Pause off");
 }


 void OnKeyButtonPrevious()
 {
	ShowKeyStatus(L"Previous");
	DoPrevious();
 }


 void OnKeyButtonPlay()
 {
	ShowKeyStatus(L"Play");
	DoPlay();
 }


 void OnKeyButtonNext()
 {
	ShowKeyStatus(L"Next");
	DoNext();
 }


 void OnKeyButtonEqRestore()
 {
	ShowKeyStatus(L"Eq reset");
	OnButtonEqRestore();
 }


 void OnKeyButtonEqInvert()
 {
	ShowKeyStatus(L"Eq invert");
	OnButtonEqInvert();
 }


 void OnKeyPopupEqPreset1()
 {
	ShowKeyStatus(L"Eq Preset 1");
	OnPopupEqPreset(0);
 }


 void OnKeyPopupEqPreset2()
 {
	ShowKeyStatus(L"Eq Preset 2");
	OnPopupEqPreset(1);
 }


 void OnKeyPopupEqPreset3()
 {
	ShowKeyStatus(L"Eq Preset 3");
	OnPopupEqPreset(2);
 }


 void OnKeyPopupEqPreset4()
 {
	ShowKeyStatus(L"Eq Preset 4");
	OnPopupEqPreset(3);
 }


 void OnKeyResetSliderPitch()
 {
	ShowKeyStatus(L"Pitch effect reset");
	if(g_w[WINDOWEFFECTS])
		SliderSetValue(WINDOWEFFECTS, ID_SLIDER_PITCH, g_setsbu.pitchslider);
	OnSliderPitch(g_setsbu.pitchslider);
	if(g_w[WINDOWEFFECTS])
		SliderSetValue(WINDOWEFFECTS, ID_SLIDER_PITCH2, g_setsbu.pitchslider2);
	OnSliderPitch2(g_setsbu.pitchslider2);
 }


 void OnKeyResetSliderTempo()
 {
	ShowKeyStatus(L"Tempo effect reset");
	if(g_w[WINDOWEFFECTS])
		SliderSetValue(WINDOWEFFECTS, ID_SLIDER_TEMPO, g_setsbu.temposlider);
	OnSliderTempo(g_setsbu.temposlider);
 }


 void OnKeyResetSliderSpeed()
 {
	ShowKeyStatus(L"Speed effect reset");
	if(g_w[WINDOWEFFECTS])
		SliderSetValue(WINDOWEFFECTS, ID_SLIDER_SPEED, g_setsbu.speedslider);
	OnSliderSpeed(g_setsbu.speedslider);
 }


 void OnKeyPitchUp()
 {
	ShowKeyStatus(L"Pitch semitone up");
	OnLittleArrowPitch(1);
 }


 void OnKeyPitch2Up()
 {
	ShowKeyStatus(L"Pitch up");
	OnLittleArrowPitch2(1);
 }


 void OnKeyTempoUp()
 {
	ShowKeyStatus(L"Tempo up");
	OnLittleArrowTempo(1);
 }


 void OnKeySpeedUp()
 {
	ShowKeyStatus(L"Speed up");
	OnLittleArrowSpeed(1);
 }


 void OnKeyPitchDown()
 {
	ShowKeyStatus(L"Pitch semitone down");
	OnLittleArrowPitch(-1);
 }


 void OnKeyPitch2Down()
 {
	ShowKeyStatus(L"Pitch down");
	OnLittleArrowPitch2(-1);
 }


 void OnKeyTempoDown()
 {
	ShowKeyStatus(L"Tempo down");
	OnLittleArrowTempo(-1);
 }


 void OnKeySpeedDown()
 {
	ShowKeyStatus(L"Speed down");
	OnLittleArrowSpeed(-1);
 }


 void OnKeyOpenList()
 {
 #if !defined(MAGUINONE)
	if(g_state.running)
		ShowKeyStatus(L"OpenList... Stop first");
	else
	{
		ShowKeyStatus(L"OpenList");
		DoOpenList();
	}
 #endif
 }


 void OnKeyMakeList()
 {
 #if !defined(MAGUINONE)
	if(g_state.running)
		ShowKeyStatus(L"MakeList... Stop first");
	else
	{
		ShowKeyStatus(L"MakeList");
		DoMakeList();
	}
 #endif
 }


 void OnKeySaveListAs()
 {
 #if !defined(MAGUINONE)
	if(g_state.running)
		ShowKeyStatus(L"SaveListAs... Stop first");
	else
	{
		ShowKeyStatus(L"SaveListAs");
		DoSaveListAs();
	}
 #endif
 }


 void OnKeyMenuZero()
 {
	ShowKeyStatus(L"ZeroList");
	DoZeroList();
 }


 void OnKeyPlayFile()
 {
 #if !defined(MAGUINONE)
	if(g_state.running)
		ShowKeyStatus(L"PlayFile... Stop first");
	else
	{
		ShowKeyStatus(L"PlayFile");
		DoPlayFile();
	}
 #endif
 }


 void OnKeyPlayCD()
 {
#if !defined(MANOCDA)
	ShowKeyStatus(L"PlayCD");
	DoPlayCD();
#endif
 }


 void OnKeyEjectCD()
 {
#if !defined(MANOCDA)
	ShowKeyStatus(L"EjectCD");
	DoEjectCD();
#endif
 }


 void OnKeyViewAbout()
 {
	if(g_w[WINDOWABOUT])
	{
		ShowKeyStatus(L"About window off");
		DoWindow(WINDOWABOUT, 0);
	}
	else
	{
		ShowKeyStatus(L"About window on");
		DoWindow(WINDOWABOUT, 1);
	}
 }


 void OnKeyViewCatalog()
 {
 #if (!defined(NOTEXTVIEWER) && !defined(MAGUINONE)) || (defined(NOTEXTVIEWER) && defined(MAGUINONE))
	if(g_w[WINDOWCATALOG])
	{
		ShowKeyStatus(L"Catalog window off");
		DoWindow(WINDOWCATALOG, 0);
	}
	else
	{
		ShowKeyStatus(L"Catalog window on");
		DoViewCatalog(0);
	}
 #endif
 }


 void OnKeyViewLog()
 {
  #if (!defined(NOTEXTVIEWER) && !defined(MAGUINONE)) || (defined(NOTEXTVIEWER) && defined(MAGUINONE))
	if(g_w[WINDOWLOG])
	{
		ShowKeyStatus(L"Log window off");
		DoWindow(WINDOWLOG, 0);
	}
	else
	{
		ShowKeyStatus(L"Log window on");
		DoWindow(WINDOWLOG, 1);
	}
 #endif
 }


 void OnKeyEditBindings()
 {
 #if (!defined(NOTEXTVIEWER) && !defined(MAGUINONE)) || (defined(NOTEXTVIEWER) && defined(MAGUINONE))
	if(g_w[WINDOWKEYBINDINGS])
	{
		ShowKeyStatus(L"Key Bindings window off");
		DoWindow(WINDOWKEYBINDINGS, 0);
	}
	else
	{
		ShowKeyStatus(L"Key Bindings window on");
		DoEditKeyBindings();
	}
 #endif
 }


 void OnKeyViewReadme()
 {
 #if (!defined(NOTEXTVIEWER) && !defined(MAGUINONE)) || (defined(NOTEXTVIEWER) && defined(MAGUINONE))
	if(g_w[WINDOWREADME])
	{
		ShowKeyStatus(L"README window off");
		DoWindow(WINDOWREADME, 0);
	}
	else
	{
		ShowKeyStatus(L"README window on");
		DoWindow(WINDOWREADME, 1);
	}
 #endif
 }


 void OnKeyClearLog()
 {
	ShowKeyStatus(L"Clear log");
	maLog(&g_logcmdclear);
 }


 void OnKeyButtonInfo()
 {
	ShowKeyStatus(L"List playerinfo");
	OnButtonInfo();
 }


 void OnKeyButtonList()
 {
	ShowKeyStatus(L"List playlist");
	OnButtonList();
 }


 void OnKeyButtonLoaded()
 {
	ShowKeyStatus(L"List songs loaded");
	OnButtonLoaded();
 }


 void OnKeyButtonSong()
 {
	ShowKeyStatus(L"List song info");
	OnButtonSong();
 }


 void OnKeyUpArrow() // from kb shortcut 'i', not from kb arrow key
 {
	fschar fskey[STRLEN16] = {fsT('\0')};
	fskey[4] = MAWXK_UP;
	ShowKeyStatus(L"List scroll up");
	l_noshowrow = 1;
	maOnKey(fskey);
 }


 void OnKeyDownArrow() // from kb shortcut 'k', not from kb arrow key
 {
	fschar fskey[STRLEN16] = {fsT('\0')};
	fskey[4] = MAWXK_DOWN;
	ShowKeyStatus(L"List scroll down");
	l_noshowrow = 1;
	maOnKey(fskey);
 }


 void OnKeyShowFileOut()
 {
	if(g_sets.output >= kOUTFIRST && g_sets.output <= kOUTLAST)
		ToggleWindow(WINDOWFILEOUT);
	if(g_sets.mawin[WINDOWFILEOUT].ison)
		ShowKeyStatus(L"FileOut window on");
	else
		ShowKeyStatus(L"FileOut window off");
 }


 void OnKeyEffectsWindow()
 {
	ToggleWindow(WINDOWEFFECTS);
	if(g_sets.mawin[WINDOWEFFECTS].ison) 
		ShowKeyStatus(L"Effects window on");
	else
		ShowKeyStatus(L"Effects window off");
 }


 void OnKeyAltSize()
 {
	ShowKeyStatus(L"Alt window sizes");
	DoAltSize(g_sets.winconf ? 0 : 1, 1);
 }


 void OnKeyFactorySize()
 {
	ShowKeyStatus(L"Factory window sizes");
	DoFactorySize();
 }


 void OnKeySavedSize()
 {
	ShowKeyStatus(L"Saved window sizes");
	DoSavedSize();
 }


 void OnKeyDefaultSize()
 {
	ShowKeyStatus(L"Default window sizes");
	DoDefaultSize();
 }


 void OnKeyCheckRandom()
 {
	if(g_sets.random)
		ShowKeyStatus(L"Random off");
	else
		ShowKeyStatus(L"Random on");
	DoRandom(g_sets.random ? 0 : 1);
 }

 
 void OnKeyCheckReptPlayed()
 {
	if(g_sets.reptplayed)
		ShowKeyStatus(L"RepeatPlayed off");
	else
		ShowKeyStatus(L"RepeatPlayed on");
	DoReptPlayed(g_sets.reptplayed ? 0 : 1);
 }


 void OnKeyCheckLoopLoaded()
 {
	if(g_sets.looploaded)
		ShowKeyStatus(L"LoopLoaded off");
	else
		ShowKeyStatus(L"LoopLoaded on");
	DoLoopLoaded(g_sets.looploaded ? 0 : 1);
 }


 void OnKeyCheckLoopList()
 {
	if(g_sets.looplist)
		ShowKeyStatus(L"LoopList off");
	else
		ShowKeyStatus(L"LoopList on");
	DoLoopList(g_sets.looplist ? 0 : 1);
 }

 
 void OnKeyCheckSinglePlay()
 {
	if(g_sets.singleplay)
		ShowKeyStatus(L"SinglePlay off");
	else
		ShowKeyStatus(L"SinglePlay on");
	DoSinglePlay(g_sets.singleplay ? 0 : 1);
 }


 void OnKeyCheckEffects()
 {
	if(g_sets.effects & EFFECTSON)
		ShowKeyStatus(L"Effects off");
	else
		ShowKeyStatus(L"Effects on");
	DoEffects(g_sets.effects & EFFECTSON ? 0 : 1);
 }


 void OnKeyCheckRepeat()
 {
	if(g_reptsong)
		ShowKeyStatus(L"Repeat off");
	else
		ShowKeyStatus(L"Repeat on");
	DoReptSong(g_reptsong ? 0 : 1);
 }

 
 void OnKeyCheckPitch()
 {
	if(g_sets.effects & EFFECTSPITCH)
		ShowKeyStatus(L"Pitch effect off");
	else
		ShowKeyStatus(L"Pitch effect on");
	DoPitch(g_sets.effects & EFFECTSPITCH ? 0 : 1);
 }


 void OnKeyCheckTempo()
 {
	if(g_sets.effects & EFFECTSTEMPO)
		ShowKeyStatus(L"Tempo effect off");
	else
		ShowKeyStatus(L"Tempo effect on");
	DoTempo(g_sets.effects & EFFECTSTEMPO ? 0 : 1);
 }


 void OnKeyCheckSpeed()
 {
	if(g_sets.effects & EFFECTSSPEED)
		ShowKeyStatus(L"Speed effect off");
	else
		ShowKeyStatus(L"Speed effect on");
	DoSpeed(g_sets.effects & EFFECTSSPEED ? 0 : 1);
 }


 void OnKeyCheckEq31Bands()
 {
	if(g_sets.eqflags & MAEQ31BANDS)
		ShowKeyStatus(L"Eq31Bands off");
	else
		ShowKeyStatus(L"Eq31Bands on");
	DoEq31Bands(g_sets.eqflags & MAEQ31BANDS ? 0 : 1);
 }


 void OnKeyCheckEqExtra()
 {
	if(g_sets.eqflags & MAEQEXTRA)
		ShowKeyStatus(L"EqExtra off");
	else
		ShowKeyStatus(L"EqExtra on");
	
	DoEqExtra(g_sets.eqflags & MAEQEXTRA ? 0 : 1);
 }


 void OnKeyCheckEqWindow()
 {
 #if !defined(MAGUINONE)
	if(g_sets.mawin[WINDOWEQ].ison)
		ShowKeyStatus(L"Equalizer window off");
	else
		ShowKeyStatus(L"Equalizer window on");
	DoEqWindow();
 #endif
 }

 
 void OnKeyCheckEqualizer()
 {
	if(g_sets.eqflags & MAEQACTIVE)
		ShowKeyStatus(L"Equalizer off");
	else
		ShowKeyStatus(L"Equalizer on");
	DoEqualizer(g_sets.eqflags & MAEQACTIVE ? 0 : 1);
 }


 void OnKeyCheckInputMonitor()
 {
	if(g_sets.inputmonitor)
		ShowKeyStatus(L"InputMonitor off");
	else
		ShowKeyStatus(L"InputMonitor on");
	if(g_w[WINDOWADVANCED])
		CheckBoxSetValue(WINDOWADVANCED, ID_CHK_INPUTMONITOR, g_sets.inputmonitor ? 0 : 1);
	DoInputMonitor(g_sets.inputmonitor ? 0 : 1);
 }

 
 void OnKeyCheckListWindow()
 {
	if(g_sets.mawin[WINDOWLIST].ison)
		ShowKeyStatus(L"List window off");
	else
		ShowKeyStatus(L"List window on");
	DoListWindow(g_sets.mawin[WINDOWLIST].ison ? 0 : 1);
 }

 
 void OnKeyCheckTitleInfo()
 {
	DoTitleInfo(0);
 }


 /*
 void OnKeyCheckAltSize()
 {
	DoAltSize(g_sets.winconf ? 0 : 1, 1);
 }


 void OnKeyCheckBalance()
 {
	if(g_sets.balance)
		ShowKeyStatus(L"Balance off");
	else
		ShowKeyStatus(L"Balance on");
	DoBalance(g_sets.balance ? 0 : 1);
 }


 void OnKeyCheckMonoLeft()
 {
	if(g_sets.monodiff & MONOCHECKBOXL)
		ShowKeyStatus(L"MonoLeft off");
	else
		ShowKeyStatus(L"MonoLeft on");
	DoMonoLeft(g_sets.monodiff & MONOCHECKBOXL ? 0 : 1);
 }


 void OnKeyCheckMonoRight()
 {
	if(g_sets.monodiff & MONOCHECKBOXR)
		ShowKeyStatus(L"MonoRight off");
	else
		ShowKeyStatus(L"MonoRight on");
	DoMonoRight(g_sets.monodiff & MONOCHECKBOXR ? 0 : 1);
 }


 void OnKeyCheckMonoDiff()
 {
	if(g_sets.monodiff & (MONOCHECKBOXSUB | MONOCHECKBOXREV))
		ShowKeyStatus(L"Mono L+R");
	else
		ShowKeyStatus(L"Mono L-R");
	DoMonoDiff(g_sets.monodiff & (MONOCHECKBOXSUB | MONOCHECKBOXREV) ? 0 : 1);
 }


 void OnKeyCheckLimiter()
 {
	if(g_sets.limiter)
		ShowKeyStatus(L"Limiter off");
	else
		ShowKeyStatus(L"Limiter on");
	DoLimiter(g_sets.limiter ? 0 : 1);
 }


 void OnKeyCheckEqZoom()
 {
	DoEqZoom(g_sets.eqflags & MAEQZOOM ? 0 : 1);
 }


 void OnKeyCheckEqLink()
 {
	DoEqLink(g_sets.eqflags & MAEQLINK ? 0 : 1);
 }


 void OnKeyCheckInvertRight()
 {
	if(g_sets.invertright)
		ShowKeyStatus(L"InvertRight off");
	else
		ShowKeyStatus(L"InvertRight on");
	DoInvertRight(g_sets.invertright ? 0 : 1);
 }


 #if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
 void OnKeyCheckCdParanoia()
 {
	if(g_setscdrom.cdparanoia)
		ShowKeyStatus(L"CD Paranoia off");
	else
		ShowKeyStatus(L"CD Paranoia on");
	DoCdParanoia(g_setscdrom.cdparanoia ? 0 : 1);
 }
 #endif


 void OnKeyCheckBoxBeg()
 {
 #if !defined(MAGUINONE)
	if(g_posbegon)
		ShowKeyStatus(L"Begin position off");
	else
		ShowKeyStatus(L"Begin position on");
 #endif
	DoBoxBeg(g_posbegon ? 0 : 1);
 }


 void OnKeyCheckBoxEnd()
 {
 #if !defined(MAGUINONE)
	if(g_posendon)
		ShowKeyStatus(L"End position off");
	else
		ShowKeyStatus(L"End position on");
 #endif
	DoBoxEnd(g_posendon ? 0 : 1);
 }


 #if defined(ENCODEMP4)
 void OnKeyCheckBoxM4A()
  {
	DoBoxM4A(g_sets.fileout[g_k[g_sets.output]].mode & kACCM4A ? 0 : 1);
  }
 #endif


 void OnKeyCheckShowIndex()
 {
	if(g_w[WINDOWLIST])
		CheckBoxSetValue(WINDOWLIST, ID_CHK_SHOWINDEX, g_sets.listboxmode & LBOXSHOWINDEX ? 0 : 1);
	DoShowIndex(g_sets.listboxmode & LBOXSHOWINDEX ? 0 : 1);
 }


 void OnKeyCheckShowDir()
 {
	if(g_w[WINDOWLIST])
		CheckBoxSetValue(WINDOWLIST, ID_CHK_SHOWDIR, g_sets.listboxmode & LBOXSHOWDIR ? 0 : 1);
	DoShowDir(g_sets.listboxmode & LBOXSHOWDIR ? 0 : 1);
 }


 void OnKeyCheckShowPath()
 {
	if(g_w[WINDOWLIST])
		CheckBoxSetValue(WINDOWLIST, ID_CHK_SHOWPATH, g_sets.listboxmode & LBOXSHOWPATH ? 0 : 1);
	DoShowPath(g_sets.listboxmode & LBOXSHOWPATH ? 0 : 1);
 }
 */


 void OnKeyVolumeDown()
 {
	ChangeVolume(-MAKEYVOLUMEDELTA);
 }


 void OnKeyVolumeUp()
 {
	ChangeVolume(MAKEYVOLUMEDELTA);
 }


 void OnKeyPlayIndex()
 {
	machar mastr[MAXLOGLINELEN];

	if(l_keyindex)
		return; // don't reenter
	l_keyindex = 1;
		
	g_maOnKeySkip = 1;
 
	ShowKeyStatus(L"Play index");
	
	masprintf(mastr, MAXLOGLINELEN, L"Enter the index of a song to play (1 to %d)", g_songsnum);

	DoWindow(WINDOWPROMPTNUM, 1);
	
	if(!g_w[WINDOWPROMPTNUM])
	{
		g_maOnKeySkip = 0;
		maLog(L"OnKeyPlayIndex failed");
		l_keyindex = 0;
		return;
	}
	
	StaticTextSetText(WINDOWPROMPTNUM, ID_TEXT_PROMPTNUM, mastr);

 #if defined(MAGUINONE)
	DoPromptIndexOk();
 #endif

 } // OnKeyPlayIndex


 void OnKeyPlayCatalog()
 {
 #if !defined(NOTEXTVIEWER)
	if(!g_w[WINDOWCATALOG])
		return;

	if(l_keycatalog)
		return; // don't reenter
	l_keycatalog = 1;
	
	ViewerOnButtonPlay(WINDOWCATALOG);
	
	l_keycatalog = 0;

 #endif
 } // OnKeyPlayCatalog


 MAKEYBINDINGS g_keybindings[MANUMKEYS]; // initialized by SetFactoryKeyBindings()

 machar l_keybindingstext[] =
 MAEOL L"This file is created by MoreAmp and will be" MAEOL
 L"restored with factory settings if deleted." MAEOL
 MAEOL L"Edit between the double-quotes the single keyboard" MAEOL
 L"character that you wish to use as a shortcut" MAEOL
 L"to the function next to it." MAEOL
 MAEOL L"Do not use the double-quote character as a shortcut." MAEOL
 MAEOL L"To disable a shortcut to a function put nothing" MAEOL
 L"(no space) between the double-quotes." MAEOL
 MAEOL L"You may edit any of the text here but do not change"
 MAEOL L"the order of the functions.";

 int SaveKeyBindings(int blank)
 {
	int i;
	size_t ret;
	FILE *fp;
	machar mastr2[MAXLOGLINELEN];
#if !defined(__MAWIN__)
	machar mastr3[MAXLOGLINELEN];
#endif
	machar mastr[MAXLOGLINELEN];
	
	if((fp = mafopen(g_keybindingspath, FSWBMODE)) == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"SaveSettingsCDRom mafopen failed on '%ls'",
																fs2ma(mastr2, g_keybindingspath));
		maLog(mastr);
		return 1;
	}

 #if defined(__MAOSX__) || defined(__MAUNIX__)
	machmod(g_keybindingspath, ACCESSPERMS); // 0777
 #endif
	
	// BOM
	fwrite("\xef\xbb\xbf", 3, 1, fp); // BOM_UTF-8

	if(blank)
		for(i = 0; i < MANUMKEYS; i++)
			g_keybindings[i].keyx.makey[0] = L'\0';

	for(i = 0; i < MANUMKEYS; i++)
	{
		if(i == 0)
		{
			masprintf(mastr2, MAXLOGLINELEN, L"CONTROLS  ");
			mafilewrite(0, mastr2, fp);
		}
		
		if(i == 8)
		{
			masprintf(mastr2, MAXLOGLINELEN, L"%ls%lsSETTINGS  ", g_maeol, g_maeol);
			mafilewrite(0, mastr2, fp);
		}
		
		if(i == 16)
		{
			masprintf(mastr2, MAXLOGLINELEN, L"%ls%lsLIST  ", g_maeol, g_maeol);
			mafilewrite(0, mastr2, fp);
		}

		if(i == 23)
		{
			masprintf(mastr2, MAXLOGLINELEN, L"PlaySelection(Enter)  ");
			mafilewrite(0, mastr2, fp);
		}

		if(i == 24)
		{
			masprintf(mastr2, MAXLOGLINELEN, L"%ls%lsEQUALIZER  ", g_maeol, g_maeol);
			mafilewrite(0, mastr2, fp);
		}
		
		if(i == 34)
		{
			masprintf(mastr2, MAXLOGLINELEN, L"%ls%lsEFFECTS  ", g_maeol, g_maeol);
			mafilewrite(0, mastr2, fp);
		}
		
		if(i == 36)
		{
			masprintf(mastr2, MAXLOGLINELEN, L"%ls%lsPITCH  ", g_maeol, g_maeol);
			mafilewrite(0, mastr2, fp);
		}
		
		if(i == 42)
		{
			masprintf(mastr2, MAXLOGLINELEN, L"%ls%lsTEMPO  ", g_maeol, g_maeol);
			mafilewrite(0, mastr2, fp);
		}
		
		if(i == 46)
		{
			masprintf(mastr2, MAXLOGLINELEN, L"%ls%lsSPEED  ", g_maeol, g_maeol);
			mafilewrite(0, mastr2, fp);
		}
		
		if(i == 50)
		{
			masprintf(mastr2, MAXLOGLINELEN, L"%ls%lsFILE OUT  ", g_maeol, g_maeol);
			mafilewrite(0, mastr2, fp);
		}
		
		if(i == 51)
		{
			masprintf(mastr2, MAXLOGLINELEN, L"%ls%lsVIEW  ", g_maeol, g_maeol);
			mafilewrite(0, mastr2, fp);
		}
		
		if(i == 57)
		{
			masprintf(mastr2, MAXLOGLINELEN, L"%ls%lsWINDOW SIZES  ", g_maeol, g_maeol);
			mafilewrite(0, mastr2, fp);
		}

		if(i == 61)
		{
			masprintf(mastr2, MAXLOGLINELEN, L"%ls%lsFILE  ", g_maeol, g_maeol);
			mafilewrite(0, mastr2, fp);
		}

		if(i == 66)
		{
			masprintf(mastr2, MAXLOGLINELEN, L"%ls%lsVOLUME  ", g_maeol, g_maeol);
			mafilewrite(0, mastr2, fp);
		}

		if(g_keybindings[i].keyx.makey[0] == L'\0')
		{
			if(g_keybindings[i].labeld)
				masprintf(mastr2, MAXLOGLINELEN, L"%ls%lc%lc  ",
					g_keybindings[i].labeld, MAKEYDELIM, MAKEYDELIM);
			else
				masprintf(mastr2, MAXLOGLINELEN, L"%ls%lc%lc  ",
					g_keybindings[i].label, MAKEYDELIM, MAKEYDELIM);
		}
		else
		{
			if(g_keybindings[i].labeld)
				masprintf(mastr2, MAXLOGLINELEN, L"%ls%lc%ls%lc  ",
					g_keybindings[i].labeld, MAKEYDELIM, g_keybindings[i].keyx.makey, MAKEYDELIM);
			else
				masprintf(mastr2, MAXLOGLINELEN, L"%ls%lc%ls%lc  ",
					g_keybindings[i].label, MAKEYDELIM, g_keybindings[i].keyx.makey, MAKEYDELIM);
		}
		
		ret = mafilewrite(0, mastr2, fp);
		if(ret == 0)
		{
			fclose(fp);
			masprintf(mastr, MAXLOGLINELEN, L"SaveKeyBindings mafilewrite failed '%ls'",
																fs2ma(mastr2, g_keybindingspath));
			maLog(mastr);	
			return 1;
		}
	}
	
	masprintf(mastr2, MAXLOGLINELEN, L"%ls%ls%ls%ls", g_maeol, g_maeol, g_maeol,
															fs2ma(mastr3, g_keybindingspath));
	ret = mafilewrite(0, mastr2, fp);
	if(ret == 0)
	{
		fclose(fp);
		masprintf(mastr, MAXLOGLINELEN, L"SaveKeyBindings mafilewrite failed '%ls'",
																fs2ma(mastr2, g_keybindingspath));
		maLog(mastr);	
		return 1;
	}

	ret = mafilewrite(0, l_keybindingstext, fp);
	if(ret == 0)
	{
		fclose(fp);
		masprintf(mastr, MAXLOGLINELEN, L"SaveKeyBindings mafilewrite failed '%ls'",
																fs2ma(mastr2, g_keybindingspath));
		maLog(mastr);	
		return 1;
	}

	fclose(fp);

	if(blank)
	{
		masprintf(mastr, MAXLOGLINELEN, L"SaveKeyBindings blank file '%ls'",
																	fs2ma(mastr2, g_keybindingspath));
		// maLog(mastr);
		// masprintf(mastr, MAXLOGLINELEN, L"To restore factory keybindings delete file '%ls'",
		//															fs2ma(mastr2, g_keybindingspath));
	}
	else
		masprintf(mastr, MAXLOGLINELEN, L"SaveKeyBindings factory settings '%ls'",
																	fs2ma(mastr2, g_keybindingspath));
	maLog(mastr);	

	return 0;
	
 } // SaveKeyBindings


 int LoadKeyBindings(int showerr)
 {
	int i;
	int j;
	int k;
	size_t ret0;
	machar *p = NULL;
	machar *q;
	machar *r;
	machar *s;
	machar mastr[MAXLOGLINELEN];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
#endif

	if(g_keybindingspath[0] == fsT('\0'))
	{
		maLog(L"LoadKeyBindings cannot load settings, no file name");
		return 1;	
	}
			
	if(!maFileExists(g_keybindingspath))
	{
		SetFactoryKeyBindings();
		SaveKeyBindings(0);
		if(!maFileExists(g_keybindingspath))
		{
			masprintf(mastr, MAXLOGLINELEN, L"LoadKeyBindings No key bindings file '%ls'",
																		fs2ma(mastr2, g_keybindingspath));
			maLog(mastr);
			return 1;
		}
	}

	ret0 = mafileread(&p, g_keybindingspath);

	if(ret0 <= 0 || p == NULL)
	{			
		if(p != NULL)
			MAFREE(p);
		maLog(L"LoadKeyBindings mafileread failed");
		return 2;
	}

	s = p + ret0;
	for(q = p; q < s; q++)
		if(*q == L'\0')
			*q = L' '; // in case someone stuck a 0 in there
	q = p;
	i = 0;
	p[ret0] = L'\0';
	
	while((r = mastrchr(q, MAKEYDELIM)) != 0) // first delimiter of a pair
	{
		if(++r >= s)
			break;
		if((q = mastrchr(r, MAKEYDELIM)) == 0) // second delimiter
			break;
		if(r == q) // "" empty
			g_keybindings[i].keyx.makey[0] = L'\0';
		else
		{
			j = 0;
			k = 4; // sizeof(makey) less null terminator
			while(r < q && k--)
				g_keybindings[i].keyx.makey[j++] = *r++; // first machar after first delimiter
			g_keybindings[i].keyx.makey[j] = L'\0';
		}
		if(++i >= MANUMKEYS)
			break;
		q++;
	}
	
	if(p != NULL)
		MAFREE(p);
	
	if(showerr)
	{
		if(i < 1)
		{
			masprintf(mastr, MAXLOGLINELEN, L"LoadKeyBindings found no entries (no double-quote pairs)");
			ShowErr(mastr);
			for(i = 0; i < MANUMKEYS; i++)
				g_keybindings[i].keyx.makey[0] = L'\0';
		}
		else if(i < MANUMKEYS)
		{
			masprintf(mastr, MAXLOGLINELEN,
				L"LoadKeyBindings only found %d entries (double-quote pairs) instead of %d", i, MANUMKEYS);
			ShowErr(mastr);
			for(; i < MANUMKEYS; i++)
				g_keybindings[i].keyx.makey[0] = L'\0';
		}
		else
			VerifyKeyBindings(1);
	}
	else
	{
		if(i < 1)
		{
			SetFactoryKeyBindings();
			SaveKeyBindings(0);
			masprintf(mastr, MAXLOGLINELEN,
				L"LoadKeyBindings found no entries in '%ls', set factory bindings",
															fs2ma(mastr2, g_keybindingspath));
			maLog(mastr);
			return 3;
		}
		if(i < MANUMKEYS)
		{
			masprintf(mastr, MAXLOGLINELEN,
				L"LoadKeyBindings only found %d entries instead of %d in '%ls'",
												i, MANUMKEYS, fs2ma(mastr2, g_keybindingspath));
			maLog(mastr);
			// return 3;
		}
		VerifyKeyBindings(0);
	}
	
	UpdateMenuKbLabels();
	UpdateEffectsKbLabels();
	UpdateButtonLabels();
	
	SetLogMenus();

	return 0;
	
 } // LoadKeyBindings


 int VerifyKeyBindings(int showerr)
 {
	int i, j;
	machar unused[STRLEN256];
	machar mastr[MAXLOGLINELEN];
	machar cstr2[MAXLOGLINELEN];

	for(i = 0; i < MANUMKEYS; i++)
	{
		if(g_keybindings[i].keyx.makey == L'\0')
			continue;
		for(j = i + 1; j < MANUMKEYS; j++)
			if(!mastrcmp(g_keybindings[i].keyx.makey, g_keybindings[j].keyx.makey))
			{
				masprintf(mastr, MAXLOGLINELEN,
					L"VerifyKeyBindings duplicate key '%ls' for %ls and %ls",
							g_keybindings[i].keyx.makey, g_keybindings[i].label, g_keybindings[j].label);
				if(showerr)
				{
					GetFreeKeyBindings(unused);
					if(unused[0] != L'\0')
					{
						masprintf(cstr2, MAXLOGLINELEN, L"%ls%lsavailable keys :%ls", g_maeol, g_maeol, g_maeol);
						mastrcat(cstr2, unused);
						mastrcat(mastr, cstr2);
					}
					ShowErr(mastr);
				}
				else
					maLog(mastr);
				return 1;
			}
	}
	
	return 0;
	
 } // VerifyKeyBindings


 void GetFreeKeyBindings(machar *p)
 {
	int i, j;
	fschar fsstr[STRLEN16];
	machar mastr2[STRLEN16];
	
	if(p == NULL)
		return;
		
	fsstr[1] = fsT('\0');
	p[0] = L'\0';
	for(j = 33; j < 127; j++) // ascii '!' to tilde ~
	{
		fsstr[0] = (fschar)j;
		if(fsstr[0] == MAKEYDELIM) // if(c[0] == MAKEYDELIM)
			continue;
		fs2mafn(mastr2, fsstr);
		for(i = 0; i < MANUMKEYS; i++)
			if(!mastrcmp(g_keybindings[i].keyx.makey, mastr2))
				break;
		if(i >= MANUMKEYS)
			mastrcat(p, mastr2);
	}
	
 } // GetFreeKeyBindings


 void SetFactoryKeyBindings()
 {
	MAKEYBINDINGS keybindings[MANUMKEYS] =
	{
/* default, xmms, requested, preferred */
				//"CONTROLS  " 0-7 yxcvbnm lower left row of (my) keyboard
/* a, q, Q,   */ {{{L'Q', L'\0'}}, L"Quit", NULL, OnKeyButtonQuit, ID_MENU_QUIT, ID_TEXT_BUTTON_QUIT},
/* b, z, y,   */ {{{L'y', L'\0'}}, L"Prev", NULL, OnKeyButtonPrevious, ID_MENU_PREVIOUS, ID_TEXT_BUTTON_PREVIOUS},
/* c, x,  ,   */ {{{L'x', L'\0'}}, L"Play", NULL, OnKeyButtonPlay, ID_MENU_PLAY, ID_TEXT_BUTTON_PLAY},
/* d, c,  ,   */ {{{L'c', L'\0'}}, L"Pause", NULL, OnKeyButtonPause, ID_MENU_PAUSE, ID_TEXT_BUTTON_PAUSE},
/* e, v,  ,   */ {{{L'v', L'\0'}}, L"Stop", NULL, OnKeyButtonStop, ID_MENU_STOP, ID_TEXT_BUTTON_STOP},
/* f, b,  ,   */ {{{L'b', L'\0'}}, L"Next", NULL, OnKeyButtonNext, ID_MENU_NEXT, ID_TEXT_BUTTON_NEXT},
/* g,  ,  ,   */ {{{L'N', L'\0'}}, L"PlayCd", NULL, OnKeyPlayCD, ID_MENU_PLAYCD, 0},
/* h,  ,  ,   */ {{{L'M', L'\0'}}, L"EjectCd", NULL, OnKeyEjectCD, ID_MENU_EJECTCD, 0},

				//"\nSETTINGS" 8
/* i, r,  ,   */ {{{L'r', L'\0'}}, L"Repeat", NULL, OnKeyCheckRepeat, 0, 0},
/* j, s,  ,   */ {{{L's', L'\0'}}, L"Random", NULL, OnKeyCheckRandom, ID_MENU_RANDOM, 0},
/* k,  ,  ,   */ {{{L'j', L'\0'}}, L"ReptPlayed", NULL, OnKeyCheckReptPlayed, ID_MENU_REPTPLAYED, 0},
/* l,  ,  ,   */ {{{L'n', L'\0'}}, L"LooopLoaded", NULL, OnKeyCheckLoopLoaded, ID_MENU_LOOPLOADED, 0},
/* m,  ,  ,   */ {{{L'o', L'\0'}}, L"LoopList", NULL, OnKeyCheckLoopList, ID_MENU_LOOPLIST, 0},
/* n,  , 1,   */ {{{L'1', L'\0'}}, L"SinglePlay", NULL, OnKeyCheckSinglePlay, ID_MENU_SINGLEPLAY, 0},
/* o,  ,  ,   */ {{{L'g', L'\0'}}, L"InputMonitor", NULL, OnKeyCheckInputMonitor, ID_MENU_INPUTMONITOR, 0},
/* p,  ,  ,   */ {{{L'a', L'\0'}}, L"TitleInfo", NULL, OnKeyCheckTitleInfo, ID_MENU_TITLEINFO, 0},

				//"\nLIST" 16
/* q,  ,  ,   */ {{{L'q', L'\0'}}, L"ListWindow", L"Window", OnKeyCheckListWindow, ID_MENU_SHOWLIST, 0},
/* r,  ,  ,   */ {{{L'w', L'\0'}}, L"PlayerInfo", L"Info", OnKeyButtonInfo, 0, 0},
/* s,  ,  ,   */ {{{L't', L'\0'}}, L"Playlist", L"List", OnKeyButtonList, 0, 0},
/* t,  ,  ,   */ {{{L'u', L'\0'}}, L"SongsLoaded", L"Load", OnKeyButtonLoaded, 0, 0},
/* u,  ,  ,   */ {{{L'p', L'\0'}}, L"SongInfo", L"Song", OnKeyButtonSong, 0, 0},
/* v,  ,  , i */ {{{L'i', L'\0'}}, L"ScrollUp", NULL, OnKeyUpArrow, 0, 0},
/* w,  ,  , k */ {{{L'k', L'\0'}}, L"ScrollDown", NULL, OnKeyDownArrow, 0, 0},
				//"PlaySelection(Enter)" 23
/* x,  ,  ,   */ {{{L'P', L'\0'}}, L"ZeroList", NULL, OnKeyMenuZero, ID_MENU_ZEROLIST, 0},

				//"\nEQUALIZER" 24
/* y,  ,  , E */ {{{L'E', L'\0'}}, L"Equalizer", L"", OnKeyCheckEqualizer, ID_MENU_EQUALIZER, 0},
/* z,  ,  , e */ {{{L'e', L'\0'}}, L"EqWindow", L"Window", OnKeyCheckEqWindow, ID_MENU_SHOWEQ, 0},
/* A,  ,  ,   */ {{{L'A', L'\0'}}, L"Eq31Bands", L"31Bands", OnKeyCheckEq31Bands, ID_MENU_EQ31BANDS, 0},
/* B,  ,  ,   */ {{{L'B', L'\0'}}, L"EqExtra", L"Extra", OnKeyCheckEqExtra, ID_MENU_EQEXTRA, 0},
/* C,  ,  ,   */ {{{L'C', L'\0'}}, L"EqReset", L"Reset", OnKeyButtonEqRestore, 0, 0},
/* D,  ,  ,   */ {{{L'D', L'\0'}}, L"EqInvert", L"Invert", OnKeyButtonEqInvert, 0, 0},
/* E,  ,  ,   */ {{{L'G', L'\0'}}, L"EqPreset1", L"Preset1", OnKeyPopupEqPreset1, 0, 0},
/* F,  ,  ,   */ {{{L'H', L'\0'}}, L"EqPreset2", L"Preset2", OnKeyPopupEqPreset2, 0, 0},
/* G,  ,  ,   */ {{{L'J', L'\0'}}, L"EqPreset3", L"Preset3", OnKeyPopupEqPreset3, 0, 0},
/* H,  ,  ,   */ {{{L'K', L'\0'}}, L"EqPreset4", L"Preset4", OnKeyPopupEqPreset4, 0, 0},

				//"\nEFFECTS" 35
/* I,  ,  , F */ {{{L'F', L'\0'}}, L"Effects", L"", OnKeyCheckEffects, ID_MENU_EFFECTS, ID_TEXT_KEYEFECTS},
/* J,  ,  , f */ {{{L'f', L'\0'}}, L"EffectsWindow", L"Window", OnKeyEffectsWindow, ID_MENU_SHOWEFFECTS, 0},

				//"\nPITCH" 36 34567 upper left row of keaboard
/* K,  ,  ,   */ {{{L'R', L'\0'}}, L"Pitch", L"", OnKeyCheckPitch, 0, ID_TEXT_KEYPITCHSET},
/* L,  ,  , 3 */ {{{L'3', L'\0'}}, L"PitchDown", L"SemitoneDown", OnKeyPitchDown, 0, ID_TEXT_KEYPITCHDOWN},
/* M,  ,  , 4 */ {{{L'4', L'\0'}}, L"PitchDown2", L"Down", OnKeyPitch2Down, 0, ID_TEXT_KEYPITCH2DOWN},
/* N,  ,  , 5 */ {{{L'5', L'\0'}}, L"PitchUp2", L"Up", OnKeyPitch2Up, 0, ID_TEXT_KEYPITCH2UP},
/* O,  ,  , 6 */ {{{L'6', L'\0'}}, L"PitchUp", L"SemitoneUp", OnKeyPitchUp, 0, ID_TEXT_KEYPITCHUP},
/* P,  ,  , 7 */ {{{L'7', L'\0'}}, L"PitchReset", L"Reset", OnKeyResetSliderPitch, 0, ID_TEXT_KEYPITCHRESET},

				//"\nTEMPO" 42 890 upper right row of keaboard
/* Q,  , T,   */ {{{L'T', L'\0'}}, L"Tempo", L"", OnKeyCheckTempo, 0, ID_TEXT_KEYTEMPOSET},
/* R,  , 8,   */ {{{L'8', L'\0'}}, L"TempoDown", L"Down", OnKeyTempoDown, 0, ID_TEXT_KEYTEMPODOWN},
/* S,  , 9,   */ {{{L'9', L'\0'}}, L"TempoUp", L"Up", OnKeyTempoUp, 0, ID_TEXT_KEYTEMPOUP},
/* T,  , 0,   */ {{{L'0', L'\0'}}, L"TempoReset", L"Reset", OnKeyResetSliderTempo, 0, ID_TEXT_KEYTEMPORESET},

				//"\nSPEED" 46 UIO middle upper right row of keyboard
/* U,  ,  ,   */ {{{L'Z', L'\0'}}, L"Speed", L"", OnKeyCheckSpeed, 0, ID_TEXT_KEYSPEEDSET},
/* W,  ,  , I */ {{{L'U', L'\0'}}, L"SpeedDown", L"Down", OnKeySpeedDown, 0, ID_TEXT_KEYSPEEDDOWN},
/* V,  ,  , O */ {{{L'I', L'\0'}}, L"SpeedUp", L"Up", OnKeySpeedUp, 0, ID_TEXT_KEYSPEEDUP},
/* X,  ,  , P */ {{{L'O', L'\0'}}, L"SpeedReset", L"Reset", OnKeyResetSliderSpeed, 0, ID_TEXT_KEYSPEEDRESET},

				//"\nFILE OUT" 50
/* Y,  ,  ,   */ {{{L'Y', L'\0'}}, L"FileOutWindow", L"Window", OnKeyShowFileOut, ID_MENU_SHOWFILEOUT, 0},

				//"\nVIEW" 51   
/* Z,  ,  ,   */ {{{L'h', L'\0'}}, L"EditKeyBindings", L"KeyBindings", OnKeyEditBindings, ID_MENU_EDITKEYBINDINGS, 0},
/* 0,  ,  ,   */ {{{L'2', L'\0'}}, L"ViewCatalog", L"Catalog", OnKeyViewCatalog, ID_MENU_VIEWCATALOG, 0},
/* 1,  ,  ,   */ {{{L'd', L'\0'}}, MAREADME, L"Readme", OnKeyViewReadme, ID_MENU_README, 0},
/* 2,  ,  ,   */ {{{L'm', L'\0'}}, L"About", NULL, OnKeyViewAbout, ID_MENU_ABOUT, 0},
/* 3,  ,  ,   */ {{{L'z', L'\0'}}, L"ViewLog", L"Log", OnKeyViewLog, ID_MENU_VIEWLOG, 0},
/* 4,  ,  ,   */ {{{L'S', L'\0'}}, L"ClearLog", NULL, OnKeyClearLog, ID_MENU_CLEARLOG, 0},

				//"\nWINDOW SIZES" 57
/* 5,  ,  ,   */ {{{L'X', L'\0'}}, L"WindowAltSizes", L"Alt", OnKeyAltSize, ID_MENU_ALTSIZE, 0},
/* 6,  ,  ,   */ {{{L'V', L'\0'}}, L"WindowSavedSizes", L"Saved", OnKeySavedSize, ID_MENU_SAVEDSIZE, 0},
/* 7,  ,  ,   */ {{{L'W', L'\0'}}, L"WindowDefaultSizes", L"Default", OnKeyDefaultSize, ID_MENU_DEFAULTSIZE, 0},
/* (,  ,  ,   */ {{{L'(', L'\0'}}, L"WindowFactorySizes", L"Factory", OnKeyFactorySize, ID_MENU_FACTORYSIZE, 0},

				//"\nFILE" 61
/* 8,  ,  ,   */ {{{L'*', L'\0'}}, L"OpenList", NULL, OnKeyOpenList, ID_MENU_OPENLIST, 0},
/* 9, L,  ,   */ {{{L'L', L'\0'}}, L"MakeList", NULL, OnKeyMakeList, ID_MENU_MAKELIST, 0},
/* +,  ,  ,   */ {{{L'+', L'\0'}}, L"SaveListAs", NULL, OnKeySaveListAs, ID_MENU_SAVELISTAS, 0},
/* -, l,  ,   */ {{{L'l', L'\0'}}, L"PlayFile", NULL, OnKeyPlayFile, ID_MENU_PLAYFILE, 0},
/* ?,  ,  ,   */ {{{L'?', L'\0'}}, L"PlayIndex", NULL, OnKeyPlayIndex, 0, 0},

				//"\nVOLUME" 66
/* ,,dn,  ,   */ {{{L',', L'\0'}}, L"VolumeDown", NULL, OnKeyVolumeDown, 0, 0},
/* .,up,  ,   */ {{{L'.', L'\0'}}, L"VolumeUp", NULL, OnKeyVolumeUp, 0, 0},
/* !,  ,  ,   */ {{{L'-', L'\0'}}, L"PlayCatalog", NULL, OnKeyPlayCatalog, 0, 0},
	};

	memcpy(g_keybindings, keybindings, sizeof(g_keybindings));
	
 } // SetFactoryKeyBindings


 #if !defined(MAGUIGTK2)
  void UpdateMenuKbLabels()
  {
	int i;
	int menuid;
	machar *p;
	machar nulabel[STRLEN128];
	machar key[STRLEN16];
	
	if(!g_w[WINDOWAPP])
		return;

  #if defined(MAGUIWX)
	if(g_menubar)
	{
  #endif
		for(menuid = ID_MENU_FIRST; menuid <= ID_MENU_LAST; menuid++)
		{
			if(menuid >= ID_SUBMENU_OPENRECENT1 && menuid <= ID_SUBMENU_OPENRECENT1 + MAXOPENRECENT - 1)
				continue; // these may not exist
			if(menuid == ID_MENU_MULTIPLEINST && g_thisinstance != 1)
				continue; // else we get an exception on multiple instances
			MenuGetLabel(menuid, nulabel);
			if((p = mastrchr(nulabel, '\'')) != 0)
				*(p - 1) = 0;
			if(g_sets.keyboard)
			{
				for(i = 0; i < MANUMKEYS; i++)
				{
					if(g_keybindings[i].keyx.makey[0] != L'\0' && menuid == g_keybindings[i].menuid)
					{
						masprintf(key, STRLEN16, L" '%ls'", g_keybindings[i].keyx.makey);
						mastrcat(nulabel, key);
						break;
					}
				}
			}
  #if defined(__WXMAC__)
			if(menuid == ID_MENU_ABOUT
   #if !defined(NOTEXTVIEWER)
			 || menuid == ID_MENU_README
   #endif
				)
				; // do something because these two labels don't get updated...
  #endif
			MenuSetLabel(menuid, nulabel);
		}
  #if defined(MAGUIWX)
	}
  #endif
	
  } // UpdateMenuKbLabels
 #endif


 void UpdateEffectsKbLabels()
 {
	int i;
	int textid;
	machar nulabel[STRLEN16];
	
	if(!g_w[WINDOWEFFECTS])
		return;
	
	for(textid = ID_TEXT_KEYEFFECTS_FIRST; textid <= ID_TEXT_KEYEFFECTS_LAST; textid++)
	{
		for(i = 0; i < MANUMKEYS; i++)
			if(textid == g_keybindings[i].textid)
			{
				if(g_sets.keyboard && g_keybindings[i].keyx.makey[0] != L'\0')
					masprintf(nulabel, STRLEN16, L"'%ls'", g_keybindings[i].keyx.makey);
				else
					nulabel[0] = L'\0';
				StaticTextSetText(WINDOWEFFECTS, textid, nulabel);
				break;
			}
	}
	
 } // UpdateEffectsKbLabels


 void UpdateButtonLabels()
 {
	int i;
	int textid;
 #if defined(MAUSEMABUTTON)
	machar *p;
 #endif
	machar nulabel[STRLEN128];
	machar key[STRLEN16];
	
	if(!g_w[WINDOWAPP])
		return;

	for(textid = ID_TEXT_BUTTON_FIRST; textid <= ID_TEXT_BUTTON_LAST; textid++)
	{
 #if defined(MAUSEMABUTTON)
		switch(textid)
		{
			case ID_TEXT_BUTTON_PREVIOUS:
				mastrcpy(nulabel, LABEL_PREV);
			break;
			case ID_TEXT_BUTTON_QUIT:
				mastrcpy(nulabel, LABEL_QUIT);
			break;
			case ID_TEXT_BUTTON_PLAY:
				mastrcpy(nulabel, LABEL_PLAY);
			break;
			case ID_TEXT_BUTTON_PAUSE:
				mastrcpy(nulabel, LABEL_PAUSE);
			break;
			case ID_TEXT_BUTTON_STOP:
				mastrcpy(nulabel, LABEL_STOP);
			break;
			case ID_TEXT_BUTTON_NEXT:
				mastrcpy(nulabel, LABEL_NEXT);
			break;
		}
		if((p = mastrchr(nulabel, L'\'')) != 0)
			*p = L'\0';
  #if defined(__MAUNIX__) // s.b. __WXX11__
		else
		{
			p = nulabel + mastrlen(nulabel) - 1;
				while(*p == L' ') // initial labels need 3 spaces at end, remove them
					*(p--) = L'\0';
		}
  #endif
 #else
  #if defined(MAGUICARBON)
		nulabel[0] = L'\0';
  #else
		mastrcpy(nulabel, L"   "); // three spaces
  #endif
 #endif
		for(i = 0; i < MANUMKEYS; i++)
		{
			if(g_sets.keyboard && g_keybindings[i].keyx.makey[0] != L'\0' && textid == g_keybindings[i].textid)
			{
				masprintf(key, STRLEN16, L"'%ls'", g_keybindings[i].keyx.makey);
				mastrcat(nulabel, key);
				break;
			}
		}
		StaticTextSetText(WINDOWAPP, textid, nulabel);
	}

 } // UpdateButtonLabels


 int maOnKey(fschar *fskey)
 {
	int i;
 #if !defined(__MAWIN__)
	fschar fsstr[STRLEN16] = {fsT('\0')};
 #endif
	machar makey[STRLEN16];
 #if !defined(MAGUINONE)
	int numrows;

	if(g_maOnKeySkip)
		return 1;

	if(fskey != NULL && fskey[4] != L'\0')
	{
		if(fskey[4] == MAWXK_ESCAPE)
		{
			WindowActivate(WINDOWAPP);
			return 0;
		}

		if(fskey[4] == MAWXK_UP || fskey[4] == MAWXK_DOWN || fskey[4] == MAWXK_RETURN || fskey[4] == MAWXK_RETURN_NUMPAD)
		{
			if(!g_w[WINDOWLIST])
				return 1;

			if(fskey[4] == MAWXK_RETURN || fskey[4] == MAWXK_RETURN_NUMPAD)
			{
				OnListWindowDoubleClick();
				return 0;
			}
			switch(g_sets.listboxmode & (LBOXPLAYLIST | LBOXSONGSLOADED))
			{
				case LBOXSONGSLOADED:
					numrows = g_loadednum;
				break;
				case LBOXPLAYLIST:
					numrows = g_songsnum;
				break;
				default:
					return 1;
				break;
			}
			if((i = ListGetRowSelected(WINDOWLIST, ID_LIST_CTRL)) < 0)
			{
				ListSelect(WINDOWLIST, ID_LIST_CTRL, 0);
				ListEnsureVisible(WINDOWLIST, ID_LIST_CTRL, 0);
				return 0;
			}
			if(fskey[4] == MAWXK_UP)
			{
				if(--i < 0)
					return 1;
			}
			else // MAWXK_DOWN
			{
				if(++i > numrows)
					return 1;
			}
			ListSelect(WINDOWLIST, ID_LIST_CTRL, i);
			ListEnsureVisible(WINDOWLIST, ID_LIST_CTRL, i);
			return 0;
		}
		return 1;
	}
 #endif
	
	if(fskey == NULL || !g_sets.keyboard)
		return 1;
	
	makey[0]= makey[1] = makey[2] = makey[3] = L'\0';
	
 #if defined(__MAWIN__)
	fs2mafn(makey, fskey);
 #else
	strcpy((char *)fsstr, (char *)fskey);
#if defined(MAGUINONE) && defined(__MACUNIX__) // osxnogui
	uint32_t uchar = ma_keyval_to_unicode(*(unsigned int *)&fsstr[0]);
	ma_unichar_to_utf8(uchar, fsstr);
#elif defined(MAGUICARBON) // MoreAmpN
	 if(g_charsetkb[0] != maenvT('\0'))
		StrCHARSETKBtoUTF8(fsstr); // now UTF-8 precomposed e.g. ê is now 0xc3aa
#elif !(defined(MAGUINONE) && defined(__MAOSX__)) && (defined(__MAUNIX__) || defined(__MACUNIX__) || defined(__WXGTK__))
	// unix, mac gtk2 with keyboard, mac wxgtk2 with keyboard
	gunichar uchar = gdk_keyval_to_unicode(* &fsstr[0]); // unix
	g_unichar_to_utf8(uchar, fsstr);
#endif
	fs2ma(makey, fsstr);
 #endif

  	for(i = 0; i < MANUMKEYS; i++)
 	{
 		if(!mastrcmp(makey, g_keybindings[i].keyx.makey))
 			break;
	}
	
	if(i >= MANUMKEYS)
	{
  #if defined(MAKEYBOARDTRACE)
		machar mastr[MAXLOGLINELEN];
		masprintf(mastr, MAXLOGLINELEN, L"maOnKey key=%ls", makey);
		maLog(mastr);
  #endif
		return 0;
	}		
	
  #if defined(MAKEYBOARDTRACE)
	{
		machar mastr[MAXLOGLINELEN];
		masprintf(mastr, MAXLOGLINELEN, L"maOnKey key=%ls label=%ls",
												makey, g_keybindings[i].label);
		maLog(mastr);
	}
  #endif

	if(g_keybindings[i].function)
	{
		mastrcpy(l_keypressed, makey);
		(*g_keybindings[i].function)();
	}
		
	return 0;
	
 } // maOnKey
#else // !MAKEYBOARD
 int maOnKey(fschar *fskey)
 {
 #if !defined(MAGUINONE)
	int i;
	int numrows;

	if(fskey != NULL && fskey[4] != L'\0')
	{
		if(fskey[4] == MAWXK_ESCAPE)
		{
			WindowActivate(WINDOWAPP);
			return 0;
		}

		if(fskey[4] == MAWXK_UP || fskey[4] == MAWXK_DOWN || fskey[4] == MAWXK_RETURN || fskey[4] == MAWXK_RETURN_NUMPAD)
		{
			if(!g_w[WINDOWLIST])
				return 1;
			
			if(fskey[4] == MAWXK_RETURN || fskey[4] == MAWXK_RETURN_NUMPAD)
			{
				OnListWindowDoubleClick();
				return 0;
			}
			switch(g_sets.listboxmode & (LBOXPLAYLIST | LBOXSONGSLOADED))
			{
				case LBOXSONGSLOADED:
					numrows = g_loadednum;
				break;
				case LBOXPLAYLIST:
					numrows = g_songsnum;
				break;
				default:
					return 1;
				break;
			}
			if((i = ListGetRowSelected(WINDOWLIST, ID_LIST_CTRL)) < 0)
			{
				ListSelect(WINDOWLIST, ID_LIST_CTRL, 0);
				ListEnsureVisible(WINDOWLIST, ID_LIST_CTRL, 0);
				return 0;
			}
			if(fskey[4] == MAWXK_UP)
			{
				if(--i < 0)
					return 1;
			}
			else // MAWXK_DOWN
			{
				if(++i > numrows)
					return 1;
			}
			ListSelect(WINDOWLIST, ID_LIST_CTRL, i);
			ListEnsureVisible(WINDOWLIST, ID_LIST_CTRL, i);
			return 0;
		}
		return 1;
	}
 #endif
	
	return 1;
	
 } // maOnKey
#endif // !MAKEYBOARD


void WindowSetsFactoryValues(int winconf)
{
	switch(winconf)
	{
		case 0:
			g_sets.mawin[WINDOWSPLASH].xpos[0] = XPOSDEF;
			g_sets.mawin[WINDOWSPLASH].ypos[0] = YPOSDEF;
			g_sets.mawin[WINDOWSPLASH].xsize[0] = XSIZEDEF;
			g_sets.mawin[WINDOWSPLASH].ysize[0] = YSIZEDEF;

			g_sets.mawin[WINDOWAPP].xpos[0] = XPOSDEF;
			g_sets.mawin[WINDOWAPP].ypos[0] = YPOSDEF;
			g_sets.mawin[WINDOWAPP].xsize[0] = XSIZEDEF;
			g_sets.mawin[WINDOWAPP].ysize[0] = YSIZEDEF;

			g_sets.mawin[WINDOWLIST].xpos[0] = LISTXPOSDEF;
			g_sets.mawin[WINDOWLIST].ypos[0] = LISTYPOSDEF;
			g_sets.mawin[WINDOWLIST].xsize[0] = LISTXSIZEDEF;
			g_sets.mawin[WINDOWLIST].ysize[0] = LISTYSIZEDEF;

			g_sets.mawin[WINDOWEQ].xpos[0] = EQXPOSDEF;
			g_sets.mawin[WINDOWEQ].ypos[0] = EQYPOSDEF;
			g_sets.mawin[WINDOWEQ].xsize[0] = EQXSIZEDEF;
			g_sets.mawin[WINDOWEQ].ysize[0] = EQYSIZEDEF;

			g_sets.mawin[WINDOWEQ2].xpos[0] = EQ2XPOSDEF;
			g_sets.mawin[WINDOWEQ2].ypos[0] = EQ2YPOSDEF;
			g_sets.mawin[WINDOWEQ2].xsize[0] = EQ2XSIZEDEF;
			g_sets.mawin[WINDOWEQ2].ysize[0] = EQ2YSIZEDEF;
		
			g_sets.mawin[WINDOWEFFECTS].xpos[0] = EFFECTSXPOSDEF;
			g_sets.mawin[WINDOWEFFECTS].ypos[0] = EFFECTSYPOSDEF;
			g_sets.mawin[WINDOWEFFECTS].xsize[0] = EFFECTSXSIZEDEF;
			g_sets.mawin[WINDOWEFFECTS].ysize[0] = EFFECTSYSIZEDEF;

			g_sets.mawin[WINDOWABOUT].xpos[0] = ABOUTXPOSDEF;
			g_sets.mawin[WINDOWABOUT].ypos[0] = ABOUTYPOSDEF;
			g_sets.mawin[WINDOWABOUT].xsize[0] = ABOUTXSIZEDEF;
			g_sets.mawin[WINDOWABOUT].ysize[0] = ABOUTYSIZEDEF;
		
			g_sets.mawin[WINDOWREADME].xpos[0] = READMEXPOSDEF;
			g_sets.mawin[WINDOWREADME].ypos[0] = READMEYPOSDEF;
			g_sets.mawin[WINDOWREADME].xsize[0] = READMEXSIZEDEF;
			g_sets.mawin[WINDOWREADME].ysize[0] = READMEYSIZEDEF;

			g_sets.mawin[WINDOWCATALOG].xpos[0] = CATALOGXPOSDEF;
			g_sets.mawin[WINDOWCATALOG].ypos[0] = CATALOGYPOSDEF;
			g_sets.mawin[WINDOWCATALOG].xsize[0] = CATALOGXSIZEDEF;
			g_sets.mawin[WINDOWCATALOG].ysize[0] = CATALOGYSIZEDEF;
		
			g_sets.mawin[WINDOWPLAYLIST].xpos[0] = PLAYLISTXPOSDEF;
			g_sets.mawin[WINDOWPLAYLIST].ypos[0] = PLAYLISTYPOSDEF;
			g_sets.mawin[WINDOWPLAYLIST].xsize[0] = PLAYLISTXSIZEDEF;
			g_sets.mawin[WINDOWPLAYLIST].ysize[0] = PLAYLISTYSIZEDEF;
		
			g_sets.mawin[WINDOWLOG].xpos[0] = LOGXPOSDEF;
			g_sets.mawin[WINDOWLOG].ypos[0] = LOGYPOSDEF;
			g_sets.mawin[WINDOWLOG].xsize[0] = LOGXSIZEDEF;
			g_sets.mawin[WINDOWLOG].ysize[0] = LOGYSIZEDEF;

			g_sets.mawin[WINDOWKEYBINDINGS].xpos[0] = KEYBINDINGSXPOSDEF;
			g_sets.mawin[WINDOWKEYBINDINGS].ypos[0] = KEYBINDINGSYPOSDEF;
			g_sets.mawin[WINDOWKEYBINDINGS].xsize[0] = KEYBINDINGSXSIZEDEF;
			g_sets.mawin[WINDOWKEYBINDINGS].ysize[0] = KEYBINDINGSYSIZEDEF;
		
			g_sets.mawin[WINDOWADVANCED].xpos[0] = ADVANCEDXPOSDEF;
			g_sets.mawin[WINDOWADVANCED].ypos[0] = ADVANCEDYPOSDEF;
			g_sets.mawin[WINDOWADVANCED].xsize[0] = ADVANCEDXSIZEDEF;
			g_sets.mawin[WINDOWADVANCED].ysize[0] = ADVANCEDYSIZEDEF;

			g_sets.mawin[WINDOWAUDIOCD].xpos[0] = AUDIOCDXPOSDEF;
			g_sets.mawin[WINDOWAUDIOCD].ypos[0] = AUDIOCDYPOSDEF;
			g_sets.mawin[WINDOWAUDIOCD].xsize[0] = AUDIOCDXSIZEDEF;
			g_sets.mawin[WINDOWAUDIOCD].ysize[0] = AUDIOCDYSIZEDEF;

			g_sets.mawin[WINDOWPROMPTNUM].xpos[0] = XPOSDEF;
			g_sets.mawin[WINDOWPROMPTNUM].ypos[0] = YPOSDEF;
			g_sets.mawin[WINDOWPROMPTNUM].xsize[0] = PROMPTNUMXSIZEDEF;
			g_sets.mawin[WINDOWPROMPTNUM].ysize[0] = PROMPTNUMYSIZEDEF;
		
			g_sets.mawin[WINDOWFILEOUT].xpos[0] = FILEOUTXPOSDEF;
			g_sets.mawin[WINDOWFILEOUT].ypos[0] = FILEOUTYPOSDEF;
			g_sets.mawin[WINDOWFILEOUT].xsize[0] = FILEOUTXSIZEDEF;
			g_sets.mawin[WINDOWFILEOUT].ysize[0] = FILEOUTYSIZEDEF;

			g_sets.listcolxsize[0][0] = g_sets.listcolxsize[0][1] =
			g_sets.listcolxsize[0][2] = g_sets.listcolxsize[0][3] = LISTCOLXSIZEDEF;
		break;

		case 1:
			g_sets.mawin[WINDOWSPLASH].xpos[1] = XPOSDEF;
			g_sets.mawin[WINDOWSPLASH].ypos[1] = YPOSDEF;
			g_sets.mawin[WINDOWSPLASH].xsize[1] = XSIZEDEF;
			g_sets.mawin[WINDOWSPLASH].ysize[1] = YSIZEDEF;

			g_sets.mawin[WINDOWAPP].xpos[1] = XPOSDEF2;
			g_sets.mawin[WINDOWAPP].ypos[1] = YPOSDEF2;
			g_sets.mawin[WINDOWAPP].xsize[1] = XSIZEDEF2;
			g_sets.mawin[WINDOWAPP].ysize[1] = YSIZEDEF2;

			g_sets.mawin[WINDOWLIST].xpos[1] = LISTXPOSDEF2;
			g_sets.mawin[WINDOWLIST].ypos[1] = LISTYPOSDEF2;
			g_sets.mawin[WINDOWLIST].xsize[1] = LISTXSIZEDEF2;
			g_sets.mawin[WINDOWLIST].ysize[1] = LISTYSIZEDEF2;

			g_sets.mawin[WINDOWEQ].xpos[1] = EQXPOSDEF2;
			g_sets.mawin[WINDOWEQ].ypos[1] = EQYPOSDEF2;
			g_sets.mawin[WINDOWEQ].xsize[1] = EQXSIZEDEF2;
			g_sets.mawin[WINDOWEQ].ysize[1] = EQYSIZEDEF2;

			g_sets.mawin[WINDOWEQ2].xpos[1] = EQ2XPOSDEF2;
			g_sets.mawin[WINDOWEQ2].ypos[1] = EQ2YPOSDEF2;
			g_sets.mawin[WINDOWEQ2].xsize[1] = EQ2XSIZEDEF2;
			g_sets.mawin[WINDOWEQ2].ysize[1] = EQ2YSIZEDEF2;
		
			g_sets.mawin[WINDOWEFFECTS].xpos[1] = EFFECTSXPOSDEF2;
			g_sets.mawin[WINDOWEFFECTS].ypos[1] = EFFECTSYPOSDEF2;
			g_sets.mawin[WINDOWEFFECTS].xsize[1] = EFFECTSXSIZEDEF2;
			g_sets.mawin[WINDOWEFFECTS].ysize[1] = EFFECTSYSIZEDEF2;

			g_sets.mawin[WINDOWABOUT].xpos[1] = ABOUTXPOSDEF;
			g_sets.mawin[WINDOWABOUT].ypos[1] = ABOUTYPOSDEF;
			g_sets.mawin[WINDOWABOUT].xsize[1] = ABOUTXSIZEDEF;
			g_sets.mawin[WINDOWABOUT].ysize[1] = ABOUTYSIZEDEF;
		
			g_sets.mawin[WINDOWREADME].xpos[1] = READMEXPOSDEF;
			g_sets.mawin[WINDOWREADME].ypos[1] = READMEYPOSDEF;
			g_sets.mawin[WINDOWREADME].xsize[1] = READMEXSIZEDEF;
			g_sets.mawin[WINDOWREADME].ysize[1] = READMEYSIZEDEF;

			g_sets.mawin[WINDOWCATALOG].xpos[1] = CATALOGXPOSDEF;
			g_sets.mawin[WINDOWCATALOG].ypos[1] = CATALOGYPOSDEF;
			g_sets.mawin[WINDOWCATALOG].xsize[1] = CATALOGXSIZEDEF;
			g_sets.mawin[WINDOWCATALOG].ysize[1] = CATALOGYSIZEDEF;
		
			g_sets.mawin[WINDOWPLAYLIST].xpos[1] = PLAYLISTXPOSDEF;
			g_sets.mawin[WINDOWPLAYLIST].ypos[1] = PLAYLISTYPOSDEF;
			g_sets.mawin[WINDOWPLAYLIST].xsize[1] = PLAYLISTXSIZEDEF;
			g_sets.mawin[WINDOWPLAYLIST].ysize[1] = PLAYLISTYSIZEDEF;
		
			g_sets.mawin[WINDOWLOG].xpos[1] = LOGXPOSDEF;
			g_sets.mawin[WINDOWLOG].ypos[1] = LOGYPOSDEF;
			g_sets.mawin[WINDOWLOG].xsize[1] = LOGXSIZEDEF;
			g_sets.mawin[WINDOWLOG].ysize[1] = LOGYSIZEDEF;

			g_sets.mawin[WINDOWKEYBINDINGS].xpos[1] = KEYBINDINGSXPOSDEF;
			g_sets.mawin[WINDOWKEYBINDINGS].ypos[1] = KEYBINDINGSYPOSDEF;
			g_sets.mawin[WINDOWKEYBINDINGS].xsize[1] = KEYBINDINGSXSIZEDEF;
			g_sets.mawin[WINDOWKEYBINDINGS].ysize[1] = KEYBINDINGSYSIZEDEF;
		
			g_sets.mawin[WINDOWADVANCED].xpos[1] = ADVANCEDXPOSDEF;
			g_sets.mawin[WINDOWADVANCED].ypos[1] = ADVANCEDYPOSDEF;
			g_sets.mawin[WINDOWADVANCED].xsize[1] = ADVANCEDXSIZEDEF;
			g_sets.mawin[WINDOWADVANCED].ysize[1] = ADVANCEDYSIZEDEF;

			g_sets.mawin[WINDOWAUDIOCD].xpos[1] = AUDIOCDXPOSDEF;
			g_sets.mawin[WINDOWAUDIOCD].ypos[1] = AUDIOCDYPOSDEF;
			g_sets.mawin[WINDOWAUDIOCD].xsize[1] = AUDIOCDXSIZEDEF;
			g_sets.mawin[WINDOWAUDIOCD].ysize[1] = AUDIOCDYSIZEDEF;

			g_sets.mawin[WINDOWPROMPTNUM].xpos[1] = XPOSDEF;
			g_sets.mawin[WINDOWPROMPTNUM].ypos[1] = YPOSDEF;
			g_sets.mawin[WINDOWPROMPTNUM].xsize[1] = PROMPTNUMXSIZEDEF;
			g_sets.mawin[WINDOWPROMPTNUM].ysize[1] = PROMPTNUMYSIZEDEF;
		
			g_sets.mawin[WINDOWFILEOUT].xpos[1] = FILEOUTXPOSDEF2;
			g_sets.mawin[WINDOWFILEOUT].ypos[1] = FILEOUTYPOSDEF2;
			g_sets.mawin[WINDOWFILEOUT].xsize[1] = FILEOUTXSIZEDEF2;
			g_sets.mawin[WINDOWFILEOUT].ysize[1] = FILEOUTYSIZEDEF2;

			g_sets.listcolxsize[1][0] = g_sets.listcolxsize[1][1] =
			g_sets.listcolxsize[1][2] = g_sets.listcolxsize[1][3] = LISTCOLXSIZEDEF2;
		break;
		default:
		break;
	}

#if defined(__WXMSW__)
		if(maIsWin7() && (winconf == 0 || winconf == 1))
		{
			g_sets.mawin[WINDOWAPP].ysize[winconf] += YSIZEADJUST7b;
			g_sets.mawin[WINDOWLIST].ysize[winconf] += YSIZEADJUST7b;
			g_sets.mawin[WINDOWEQ].ypos[winconf] += YSIZEADJUST7b;
			g_sets.mawin[WINDOWEQ].ysize[winconf] += YSIZEADJUST7b;
			g_sets.mawin[WINDOWEQ2].ypos[winconf] += YSIZEADJUST7b;
			g_sets.mawin[WINDOWEQ2].ysize[winconf] += YSIZEADJUST7b;
			g_sets.mawin[WINDOWEFFECTS].ypos[winconf] += YSIZEADJUST7b;
			g_sets.mawin[WINDOWEFFECTS].ysize[winconf] += YSIZEADJUST7b;
			g_sets.mawin[WINDOWFILEOUT].ypos[winconf] += YSIZEADJUST7b;
			g_sets.mawin[WINDOWFILEOUT].ysize[winconf] += YSIZEADJUST7b;
			g_sets.mawin[WINDOWADVANCED].ysize[winconf] += YSIZEADJUST7b;
		}
#endif

} // WindowSetsFactoryValues


void SetFileoutType(int which)
{
	int i, first, last;
	
	if(which < 0)
	{
		first = kOUTFIRST;
		last = kOUTLAST;
		for(i = 1; i < kOUTMAX; i++) // 0 is kOUTSOUND
			g_sets.fileout[g_k[i]].type = -1;
	}
	else
		first = last = which;
			
	for(i = first; i <= last; i++)
	{
 #ifdef ENCODEMP3
		if(i == kOUTMP3)
		{
			g_sets.fileout[g_k[i]].type = kMAMP3;
			continue;
		}
 #endif
 #ifdef ENCODEOGG
		if(i == kOUTOGG)
		{
			g_sets.fileout[g_k[i]].type = kMAOGG;
			continue;
		}

 #endif
 #ifdef ENCODEFLAC
		if(i == kOUTFLAC)
		{
			g_sets.fileout[g_k[i]].type = kMAFLAC;
			continue;
		}
 #endif
 #ifdef ENCODEAAC
		if(i == kOUTAAC)
		{
			g_sets.fileout[g_k[i]].type = kMAAAC;
			continue;
		}
 #endif
 #ifdef ENCODEMP4
		if(i == kOUTMP4)
		{
			g_sets.fileout[g_k[i]].type = kMAMP4;
			continue;
		}
 #endif
 #ifdef ENCODEWMA
		if(i == kOUTWMA)
		{
			g_sets.fileout[g_k[i]].type = kMAWMA;
			continue;
		}
 #endif
 #ifdef ENCODEWAV
		if(i == kOUTWAV)
		{
			g_sets.fileout[g_k[i]].type = kMAWAV;
			continue;
		}
 #endif
 #ifdef ENCODEAIF
		if(i == kOUTAIF)
		{
			g_sets.fileout[g_k[i]].type = kMAAIF;
			continue;
		}
 #endif
 #ifdef ENCODERAW
		if(i == kOUTRAW)
		{
			g_sets.fileout[g_k[i]].type = kMARAW;
			continue;
		}
 #endif
	}

} // SetFileoutType


void LoadFactoryFileout(int which)
{
	int i, first, last;
	
	if(which < 0)
	{
		first = kOUTFIRST;
		last = kOUTLAST;
	}
	else
		first = last = which;
	
	SetFileoutType(which);

	for(i = first; i <= last; i++)
	{
		g_sets.fileout[g_k[i]].bitrate = FILEOUTBITRATEDEF;
		g_sets.fileout[g_k[i]].quality = FILEOUTQUALITYDEF;
		g_sets.fileout[g_k[i]].mode = FILEOUTMODEDEF;
		g_sets.fileout[g_k[i]].samplerate = FILEOUTSAMPLERATEDEF;
		fsstrcpy(g_sets.fileout[g_k[i]].outdir, g_fileoutdir);
		switch(g_sets.fileout[g_k[i]].type)
		{
			case kMAMP3:
				g_sets.fileout[g_k[i]].quality = FILEOUTQUALITYMP3DEF;
			break;
			case kMAOGG:
				g_sets.fileout[g_k[i]].quality = FILEOUTQUALITYOGGDEF; // not used
			break;
			case kMAAAC:
				g_sets.fileout[g_k[i]].quality = FILEOUTQUALITYAACDEF;
				g_sets.fileout[g_k[i]].mode |= kACCADTS;
			break;
			case kMAMP4:
				g_sets.fileout[g_k[i]].quality = FILEOUTQUALITYMP4DEF;
			break;
			default:
			break;
		}		
	}

	if(which < 0 || which == kOUTSOUND)
		g_sets.fileout[kOUTSOUND].mode = MACLOSESOUNDDEF;

} // LoadFactoryFileout

	
int LoadFactorySettings(MASETS *psets)
{
	int i, j;
	int mx, my;
	int windowid;
	
	for(i = 0; i < NUMWINCONF; i++)
		WindowSetsFactoryValues(i);

	// if any window is offscreen (pretty much arbitrarily...) bring all windows on screen
	DisplayGetSize(&mx, &my);

	if(mx > XPOSMIN && my > YPOSMIN) // if screen is not too small...
	{	
		for(i = 0; i < NUMWINCONF; i++)
		{
			if(psets->mawin[WINDOWAPP].xpos[i] > mx - abs(XPOSMIN) || psets->mawin[WINDOWAPP].ypos[i] > mx - abs(YPOSMIN) ||
				psets->mawin[WINDOWEQ].xpos[i] > mx - abs(XPOSMIN) || psets->mawin[WINDOWEQ].ypos[i] > mx - abs(YPOSMIN) ||
				psets->mawin[WINDOWEQ2].xpos[i]	> mx - abs(XPOSMIN) || psets->mawin[WINDOWEQ2].ypos[i] > mx - abs(YPOSMIN) ||
				psets->mawin[WINDOWLIST].xpos[i] > mx - abs(XPOSMIN) || psets->mawin[WINDOWLIST].ypos[i] > mx - abs(YPOSMIN))
			{
				psets->mawin[WINDOWAPP].xpos[i] = XPOSMIN;
				psets->mawin[WINDOWAPP].ypos[i] = YPOSMIN;
				psets->mawin[WINDOWEQ].xpos[i] = psets->mawin[WINDOWAPP].xpos[i] + XPOSMIN;
				psets->mawin[WINDOWEQ].ypos[i] = psets->mawin[WINDOWAPP].ypos[i] + YPOSMIN;
				psets->mawin[WINDOWEQ2].xpos[i] = psets->mawin[WINDOWAPP].xpos[i] + XPOSMIN + EQ2POSDELTA;
				psets->mawin[WINDOWEQ2].ypos[i] = psets->mawin[WINDOWAPP].ypos[i] + YPOSMIN + EQ2POSDELTA;
				psets->mawin[WINDOWLIST].xpos[i] = psets->mawin[WINDOWEQ].xpos[i] + XPOSMIN;
				psets->mawin[WINDOWLIST].ypos[i] = psets->mawin[WINDOWEQ].ypos[i] + YPOSMIN;
			}

			if(psets->mawin[WINDOWAPP].xsize[i] >= abs(mx - psets->mawin[WINDOWAPP].xpos[i])) psets->mawin[WINDOWAPP].xsize[i] = mx / 4;
			if(psets->mawin[WINDOWAPP].ysize[i] >= abs(my - psets->mawin[WINDOWAPP].ypos[i])) psets->mawin[WINDOWAPP].ysize[i] = my / 4;
			if(psets->mawin[WINDOWEQ].xsize[i] >= abs(mx - psets->mawin[WINDOWEQ].xpos[i])) psets->mawin[WINDOWEQ].xsize[i] = mx / 2;
			if(psets->mawin[WINDOWEQ].ysize[i] >= abs(my - psets->mawin[WINDOWEQ].ypos[i])) psets->mawin[WINDOWEQ].ysize[i] = psets->mawin[WINDOWAPP].ysize[i];
			if(psets->mawin[WINDOWEQ2].xsize[i] >= abs(mx - psets->mawin[WINDOWEQ2].xpos[i])) psets->mawin[WINDOWEQ2].xsize[i] = mx / 2;
			if(psets->mawin[WINDOWEQ2].ysize[i] >= abs(my - psets->mawin[WINDOWEQ2].ypos[i])) psets->mawin[WINDOWEQ2].ysize[i] = psets->mawin[WINDOWAPP].ysize[i];
			if(psets->mawin[WINDOWLIST].xsize[i] >= abs(mx - psets->mawin[WINDOWLIST].xpos[i]))	psets->mawin[WINDOWLIST].xsize[i] = mx / 4;
			if(psets->mawin[WINDOWLIST].ysize[i] >= abs(my - psets->mawin[WINDOWLIST].ypos[i]))	psets->mawin[WINDOWLIST].ysize[i] = psets->mawin[WINDOWAPP].ysize[i];

			if(psets->mawin[WINDOWEQ].xpos[i] > mx - abs(XPOSMIN) || psets->mawin[WINDOWEQ].ypos[i] > mx - abs(YPOSMIN) ||
				psets->mawin[WINDOWLIST].xpos[i] > mx - abs(XPOSMIN) || psets->mawin[WINDOWLIST].ypos[i] > mx - abs(YPOSMIN))
			{
				psets->mawin[WINDOWAPP].xpos[i] = 0;
				psets->mawin[WINDOWAPP].ypos[i] = 0;
				psets->mawin[WINDOWEQ].xpos[i] = psets->mawin[WINDOWAPP].xpos[i] + XPOSMIN;
				psets->mawin[WINDOWEQ].ypos[i] = psets->mawin[WINDOWAPP].ypos[i] + YPOSMIN;
				psets->mawin[WINDOWEQ2].xpos[i] = psets->mawin[WINDOWAPP].xpos[i] + XPOSMIN + EQ2POSDELTA;
				psets->mawin[WINDOWEQ2].ypos[i] = psets->mawin[WINDOWAPP].ypos[i] + YPOSMIN + EQ2POSDELTA;
				psets->mawin[WINDOWLIST].xpos[i] = psets->mawin[WINDOWEQ].xpos[i] + XPOSMIN;
				psets->mawin[WINDOWLIST].ypos[i] = psets->mawin[WINDOWEQ].ypos[i] + YPOSMIN;
			}
		} // for i
	}
	
	psets->volumesliderleft		= VOLUMEMAX;
	psets->volumesliderright	= VOLUMEMAX;
	psets->separationslider		= SEPARATIONMAX;
	psets->panslider			= SEPARATIONMIN + (SEPARATIONMAX - SEPARATIONMIN) / 2;
	psets->preampslider			= PREAMPMIN + 5 * (PREAMPMAX - PREAMPMIN) / 8; // +4 db
	psets->pitchslider			= EFFECTSPITCHZERO;
	psets->pitchslider2			= EFFECTSPITCH2ZERO;
	psets->temposlider			= EFFECTSZERO;
	psets->speedslider			= EFFECTSZERO;
	psets->autostart = 0;
	psets->singleplay = 0;
	psets->random = 0;
	psets->reptplayed = 0;
	psets->looploaded = 0;
	psets->looplist = 1;
	psets->inputmonitor = 1;
	psets->effects = EFFECTSPITCH | EFFECTSTEMPO | EFFECTSSPEED;
	psets->limiter = PREAMP | CLIPPING; 
	psets->balance = 0;
	psets->monodiff = 0;
	psets->invertright = 0;
	psets->eqflags = MAEQACTIVE | MAEQ31BANDS | MAEQEXTRA | MAEQZOOM | MAEQLINK;
	
	psets->listboxmode = LBOXSONGSLOADED | LBOXSHOWINDEX | LBOXSHOWPATH;

	psets->mawin[0].ison = 0;
	for(windowid = WINDOWFIRST; windowid <= WINDOWLAST; windowid++)
		psets->mawin[windowid].ison = 0;
	psets->mawin[WINDOWAPP].ison = 1;
	psets->mawin[WINDOWLIST].ison = 1;
	psets->mawin[WINDOWEQ].ison = 1;

	psets->eqpreset = 0;
	psets->vdiskname[0] = fsT('\0');
	psets->buffertype = BUFUSEMEM;
	psets->output = kOUTSOUND;
	psets->buffersize = SONGMEMDEF;
	psets->titleinfo = 0;
	psets->winconf = 0;
	psets->logflags = kMALOGMAX | kMALOGRATE;
	psets->logmax = MALOGMAXDEF;
	psets->logevents = MALOGEVENTSDEF;
	psets->logperiod = MALOGPERIODDEF;
	psets->multipleinst = 1;
	
	// equalizer offsets and names
	for(i = 0; i < EQNUMPRESETS; i++)
 	{
 		psets->eqpresets[i].eqoffset = 0;
 		psets->eq2presets[i].eqoffset = 0;
		fssprintf(psets->eqpresets[i].eqname, EQNAMELEN, fsT("Preset%d"), i + 1);
		for(j = 0; j < EQBANDS31; j++)
		{
			psets->eqpresets[i].eqslider[j] = DBSLIDERMIN + (DBSLIDERMAX - DBSLIDERMIN) / 2;
			psets->eq2presets[i].eqslider[j] = DBSLIDERMIN + (DBSLIDERMAX - DBSLIDERMIN) / 2;
		}
	}
	
	// equalizer settings (for DBSLIDERMIN == 0 and DBSLIDERMAX == 63 and range -DBRANGE...DBRANGE dB)
	fsstrcpy(psets->eqpresets[0].eqname, fsT("Rock"));
	psets->eqpresets[0].eqoffset = 4; // 4db
	psets->eqpresets[0].eqslider[0] = // SliderPosFromDB(8);
	psets->eqpresets[0].eqslider[1] = // SliderPosFromDB(8);
	psets->eqpresets[0].eqslider[2] = SliderPosFromDB(8);
	psets->eqpresets[0].eqslider[3] = // SliderPosFromDB(7);
	psets->eqpresets[0].eqslider[4] = SliderPosFromDB(7);
	psets->eqpresets[0].eqslider[5] = SliderPosFromDB(6);
	psets->eqpresets[0].eqslider[6] = SliderPosFromDB(4.5);
	psets->eqpresets[0].eqslider[7] = SliderPosFromDB(2.5);
	psets->eqpresets[0].eqslider[8] = SliderPosFromDB(0.5);
	psets->eqpresets[0].eqslider[9] = SliderPosFromDB(-1.5);
	psets->eqpresets[0].eqslider[10] = SliderPosFromDB(-3.5);
	psets->eqpresets[0].eqslider[11] = SliderPosFromDB(-4.5);
	psets->eqpresets[0].eqslider[12] = SliderPosFromDB(-6.5);
	psets->eqpresets[0].eqslider[13] = SliderPosFromDB(-8.5);
	psets->eqpresets[0].eqslider[14] = SliderPosFromDB(-10);
	psets->eqpresets[0].eqslider[15] = SliderPosFromDB(-9.5);
	psets->eqpresets[0].eqslider[16] = SliderPosFromDB(-8);
	psets->eqpresets[0].eqslider[17] = SliderPosFromDB(-7);
	psets->eqpresets[0].eqslider[18] = SliderPosFromDB(-4);
	psets->eqpresets[0].eqslider[19] = SliderPosFromDB(-2.5);
	psets->eqpresets[0].eqslider[20] = SliderPosFromDB(1);
	psets->eqpresets[0].eqslider[21] = SliderPosFromDB(2.5);
	psets->eqpresets[0].eqslider[22] = SliderPosFromDB(4);
	psets->eqpresets[0].eqslider[23] = // SliderPosFromDB(6);
	psets->eqpresets[0].eqslider[24] = // SliderPosFromDB(6);
	psets->eqpresets[0].eqslider[25] = SliderPosFromDB(6);
	psets->eqpresets[0].eqslider[26] = // SliderPosFromDB(5.5);
	psets->eqpresets[0].eqslider[27] = // SliderPosFromDB(5.5);
	psets->eqpresets[0].eqslider[28] = // SliderPosFromDB(5.5);
	psets->eqpresets[0].eqslider[29] = // SliderPosFromDB(5.5);
	psets->eqpresets[0].eqslider[30] = SliderPosFromDB(5.5);
	
	fsstrcpy(psets->eqpresets[1].eqname, fsT("Up"));
	fsstrcpy(psets->eqpresets[2].eqname, fsT("Down"));

	for(j = 0; j < EQBANDS31; j++)
	{
		psets->eqpresets[1].eqslider[j] =
				DBSLIDERMIN + ((j * (DBSLIDERMAX - DBSLIDERMIN)) / (EQBANDS31 - 1)); // up
	
		psets->eqpresets[2].eqslider[EQBANDS31 - 1 - j] =
				DBSLIDERMIN + ((j * (DBSLIDERMAX - DBSLIDERMIN)) / (EQBANDS31 - 1)); // down
	}
	memcpy(psets->eq2presets, psets->eqpresets, sizeof(psets->eq2presets));
	
	// different settings for cda	
	if(PlaylistIsCda())
	{
		psets->titleinfo = 1;
		// psets->random = 0; already set above
		psets->reptplayed = 1;
		psets->limiter = 0; 
		psets->buffertype = BUFUSEDIRECT; // cda does not use memory but builds a 'loaded' list
		psets->listboxmode = LBOXPLAYLIST | LBOXSHOWINDEX | LBOXSHOWPATH;
	}
	
	LoadFactoryFileout(-1);

	if(g_thisinstance == 1)
		SetMultipleInstances();

 #if defined(MAKEYBOARD) || defined(MAGUINONE)
	psets->keyboard = 1;
 #else
	psets->keyboard = 0;
 #endif

 #if defined(MAMOUSEWHEEL)
	psets->mousewheel = 1;
 #else
	psets->mousewheel = 0;
 #endif

	psets->usem3u = USEM3UDEF;
	psets->exitafter = EXITAFTERDEF;
	psets->addmode = ADDMODEDEF;

	g_audiohostout = 0;
	g_audiodeviceout = 0;
	g_audiohostin = 0;
	g_audiodevicein = 0;
 
	psets->audiohostinname[0] = 0;
	psets->audiohostoutname[0] = 0;
	psets->audiodevinname[0] = 0;
	psets->audiodevoutname[0] = 0;
	psets->hostinenable = CHECKENABLEDEF;

	strcpy(psets->settingsversion, MOREAMP_SETTINGS_VERSION_STRING);

	return 0;
	
} // LoadFactorySettings


#if !defined(NODECODEAUDIOIN)
 int LoadFactorySettingsAudioIn(MASETS *psets)
 {
	int err;
	
	if((err = LoadFactorySettings(psets)) != 0)
	{
		maLog(L"LoadFactorySettingsAudioIn LoadFactorySettings failed");
		return err;
	}
	
	psets->listboxmode = LBOXPLAYERINFO | LBOXSHOWINDEX | LBOXSHOWPATH;			// List window off
	psets->eqflags = MAEQZOOM | MAEQLINK;										// Eq off
	psets->limiter = 0;															// Preamp off
	psets->preampslider = PREAMPMIN + (PREAMPMAX - PREAMPMIN) / 2;				// midpoint = 0 dB
	psets->volumesliderleft = psets->volumesliderright =
						(int)((VOLUMEMIN + 0.7 * (VOLUMEMAX - VOLUMEMIN)));		// Volume -30 dB
	psets->buffertype = BUFUSEDIRECT;											// Buffer Direct
 
	return 0;
	
 } // LoadFactorySettingsAudioIn
#endif


#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
int LoadFactorySettingsCDRom()
{
	int i;
	
	g_setscdrom.cdparanoia = 0;	

	for(i = 0; i <= CDAMAXNUMDRIVES; i++)
		fssprintf(g_setscdrom.cdaname[i], MAX_PATH, fsT("(drive%02d)"), i);

	strcpy(g_setscdrom.cdaname[1], MACDANAMEDEF);

	g_setscdrom.item = 0;
	
	return 0;
	
} // LoadFactorySettingsCDRom
#endif


int LoadDefaultSettings(MASETS *psets)
{
	size_t ret;
	int version;
	FILE *fp;
	char settingsversion[VERSIONSTRINGSIZE];
	machar mastr[MAXLOGLINELEN];

	if((fp = mafopen(g_settingsdefpath, fsT("rb"))) == NULL)
		return LoadFactorySettings(psets);

	// check version
	if(fseek(fp, 0L - sizeof(psets->settingsversion), SEEK_END) != 0)
	{
		fclose(fp);
		masprintf(mastr, MAXLOGLINELEN, L"LoadDefaultSettings fseek errno=%d", errno);
		maLog(mastr);
		maLog(L"LoadDefaultSettings failed, saving factory settings as defaults");
		if(LoadFactorySettings(psets))
			return 1;
		SaveSettingsAsDefault(psets);
		return 0;
	}
	
	if(fread(settingsversion, 1, sizeof(psets->settingsversion), fp) != sizeof(psets->settingsversion))
	{
		fclose(fp);
		masprintf(mastr, MAXLOGLINELEN, L"LoadDefaultSettings fread errno=%d", errno);
		maLog(mastr);
		maLog(L"LoadDefaultSettings failed, saving factory settings as defaults");
		if(LoadFactorySettings(psets))
			return 1;
		SaveSettingsAsDefault(psets);
		return 0;
	}
	
	if((version = maCheckSettingsVersion(settingsversion)) == MASETTINGSVERSIONISCURRENT)
	{
		rewind(fp);
		ret = fread(psets, sizeof(MASETS), 1, fp);
		if(ret != 1)
		{
			masprintf(mastr, MAXLOGLINELEN, L"LoadDefaultSettings fseek errno=%d", errno);
			maLog(mastr);
			fclose(fp);
			maLog(L"LoadDefaultSettings failed, saving factory settings as defaults");
			if(LoadFactorySettings(psets))
				return 1;
			SaveSettingsAsDefault(psets);
			return 0;
		}
	}
#ifdef PREVIOUS_MOREAMP_SETTINGS_VERSION_STRING
	else if(version == MASETTINGSVERSIONISPREVIOUS)
	{
		FixSettingsPrevious(fp, psets);
		fclose(fp);
		fp = NULL;
		SaveSettingsAsDefault(psets);
		masprintf(mastr, MAXLOGLINELEN,
			L"LoadDefaultSettings updated default settings to settings version %ls", MOREAMP_SETTINGS_VERSION_STRINGL);
		maLog(mastr);
	}
#endif
	else // MASETTINGSVERSIONISUNKNOWN
	{
		maLog(L"LoadDefaultSettings unknown settings version, saving factory settings as defaults");
		fclose(fp);
		if(LoadFactorySettings(psets))
			return 1;
		SaveSettingsAsDefault(psets);
		return 0;
	}
		
	if(fp)
		fclose(fp);		
		
	CheckSettings(psets);
	
	if(g_thisinstance == 1)
		SetMultipleInstances();
			
	return 0;
	
} // LoadDefaultSettings


int SaveSettingsAsDefault(MASETS *psets)
{
	size_t ret;
	FILE *fp;
	machar mastr[MAXLOGLINELEN];

	if((fp = mafopen(g_settingsdefpath, fsT("wb"))) == NULL)
	{		
		masprintf(mastr, MAXLOGLINELEN, L"SaveSettingsAsDefault error mafopen '%ls'", g_settingsdefpath);
		maLog(mastr);
		return 1;
	}

#if defined(__MAOSX__) || defined(__MAUNIX__)
	machmod(g_settingsdefpath, ACCESSPERMS); // 0777
#endif

	// save current settings into default settings file	
	ret = fwrite(psets, sizeof(MASETS), 1, fp);
	
	fclose(fp);		
		
	if(ret != 1)
	{		
		masprintf(mastr, MAXLOGLINELEN, L"%d SaveSettingsAsDefault fwrite failed '%ls'", ret, g_settingsdefpath);
		maLog(mastr);
		return 2;
	}
				
	return 0;
	
} // SaveSettingsAsDefault


void SetPause(int pause)
{
	if(pause && !g_state.paused && g_state.running)
	{
		g_state.paused = 1;
		maStopPAStreams();
		PushButtonPaintDown(WINDOWAPP, ID_BTN_PAUSE);
		l_sliderposmsbu = l_sliderposms;
		MenuSetChecked(MENUMACONTROLS, ID_MENU_PAUSE, 1);
	}
	else if(g_state.paused)
	{
		g_state.paused = 0;
		maStartPAStreams();
		if(!(g_state.stop & STOP_APPQUIT))
		{
			PushButtonPaintUp(WINDOWAPP, ID_BTN_PAUSE);
			if(l_sliderposmsbu != l_sliderposms && g_state.running) // user moved slider while paused
			{
				l_sliderposmsbu = l_sliderposms;
				PlayerSeek(l_sliderposms);
				l_playerseeking = 0;
			}
			MenuSetChecked(MENUMACONTROLS, ID_MENU_PAUSE, 0);
		}
	}
	else
		if(!(g_state.stop & STOP_APPQUIT))
			MenuSetChecked(MENUMACONTROLS, ID_MENU_PAUSE, 0);
	
} // SetPause


void SetSliders(int slider)
{
	int i, first, last;
	
	if((slider == EQSLIDERS || (slider >= EQSLIDERFIRST && slider <= EQSLIDERLAST)) && !g_w[WINDOWEQ])
		return;
	
	if((slider == EQ2SLIDERS || (slider >= EQ2SLIDERFIRST && slider <= EQ2SLIDERLAST)) && !g_w[WINDOWEQ2])
		return;
	
	// setup loop first to last consecutive sliders to set	
	if(slider == ALLSLIDERS)
	{
		first = EFFECTSSLIDERS;
		
		if(g_w[WINDOWEQ2])
			last = EQ2SLIDERLAST;
		else if(g_w[WINDOWEQ])
			last = EQSLIDERLAST;
		else
			last = PREAMPSLIDER;
	}
	else if(slider == EQSLIDERS)
	{
		first = EQSLIDERFIRST;
		last = EQSLIDERLAST;
	}
	else if(slider == EQ2SLIDERS)
	{
		first = EQ2SLIDERFIRST;
		last = EQ2SLIDERLAST;
	}
	else if(slider == EFFECTSSLIDERS) // all effects sliders
	{
		first = EFFECTSSLIDERSPEED;
		last = EFFECTSSLIDERPITCH;
	}
	else
		first = last = slider;
	
	// set sliders	
	for(i = first; i <= last; i++)
	{
		switch(i)
		{
			case VOLUMESLIDERLEFT:
				SliderSetValue(WINDOWAPP, ID_SLIDER_VOLUMEL, volminmaxL());
				SetSliderCaption(VOLUMESLIDERLEFT);
				continue;
			break;
			
			case VOLUMESLIDERRIGHT:
				SliderSetValue(WINDOWAPP, ID_SLIDER_VOLUMER, volminmaxR());
				SetSliderCaption(VOLUMESLIDERRIGHT);
				continue;
			break;
			
			case SEPARATIONSLIDER:
				if(l_outputfilter == OUTPUT_DIFF || l_outputfilter == OUTPUT_MONO) // pan
					SliderSetValue(WINDOWAPP, ID_SLIDER_VOLUMESEP, g_sets.panslider);
				else
					SliderSetValue(WINDOWAPP, ID_SLIDER_VOLUMESEP, g_sets.separationslider);
				SetSliderCaption(SEPARATIONSLIDER);
				continue;
			break;
			
			case PREAMPSLIDER:
				SliderSetValue(WINDOWAPP, ID_SLIDER_PREAMP, g_sets.preampslider);
				SetSliderCaption(PREAMPSLIDER);
				continue;
			break;
			
			case VOLUMESLIDERS: // both left and right volume
				SliderSetValue(WINDOWAPP, ID_SLIDER_VOLUMEL, volminmaxL());
				SliderSetValue(WINDOWAPP, ID_SLIDER_VOLUMER, volminmaxR());
				SetSliderCaption(VOLUMESLIDERS);
				continue;
			break;
			
			case EFFECTSSLIDERPITCH:
				if(g_w[WINDOWEFFECTS])
				{
					SliderSetValue(WINDOWEFFECTS, ID_SLIDER_PITCH, g_sets.pitchslider);
					SetSliderCaption(EFFECTSSLIDERPITCH);
				}
				continue;
			break;

			case EFFECTSSLIDERPITCH2:
				if(g_w[WINDOWEFFECTS])
				{
					SliderSetValue(WINDOWEFFECTS, ID_SLIDER_PITCH2, g_sets.pitchslider2);
					SetSliderCaption(EFFECTSSLIDERPITCH2);
				}
				continue;
			break;

			case EFFECTSSLIDERTEMPO:
				if(g_w[WINDOWEFFECTS])
				{
					SliderSetValue(WINDOWEFFECTS, ID_SLIDER_TEMPO, g_sets.temposlider);
					SetSliderCaption(EFFECTSSLIDERTEMPO);
				}
				continue;
			break;

			case EFFECTSSLIDERSPEED:
				if(g_w[WINDOWEFFECTS])
				{
					SliderSetValue(WINDOWEFFECTS, ID_SLIDER_SPEED, g_sets.speedslider);
					SetSliderCaption(EFFECTSSLIDERSPEED);
				}
				continue;
			break;

			case EFFECTSSLIDERS: // all effect sliders
				if(g_w[WINDOWEFFECTS])
				{
					SliderSetValue(WINDOWEFFECTS, ID_SLIDER_PITCH, g_sets.pitchslider);
					SetSliderCaption(EFFECTSSLIDERPITCH);
					SliderSetValue(WINDOWEFFECTS, ID_SLIDER_PITCH2, g_sets.pitchslider2);
					SetSliderCaption(EFFECTSSLIDERPITCH2);
					SliderSetValue(WINDOWEFFECTS, ID_SLIDER_TEMPO, g_sets.temposlider);
					SetSliderCaption(EFFECTSSLIDERTEMPO);
					SliderSetValue(WINDOWEFFECTS, ID_SLIDER_SPEED, g_sets.speedslider);
					SetSliderCaption(EFFECTSSLIDERSPEED);
				}
				continue;
			break;

			default:
				if(i >= EQSLIDERFIRST && i <= EQSLIDERLAST)
				{
					SliderSetValue(WINDOWEQ, ID_SLIDER_EQ0 + i,  g_sets.eqpresets[g_sets.eqpreset].eqslider[i]);	
					SetSliderCaption(i);
				}
				else if(i >= EQ2SLIDERFIRST && i <= EQ2SLIDERLAST)
				{
					SliderSetValue(WINDOWEQ2, ID_SLIDER_EQ0 + i, g_sets.eq2presets[g_sets.eqpreset].eqslider[i - EQ2SLIDERFIRST]);	
					SetSliderCaption(i);
				}
				continue;
			break;
		} // switch
			
	} // for

} // SetSliders


void SetSliderCaption(int slider)
{
	machar mastr2[STRLEN64];
	
	if(!g_w[WINDOWEQ] && slider >= EQSLIDERFIRST && slider <= EQSLIDERLAST)
		return;
	if(!g_w[WINDOWEQ2] && slider >= EQ2SLIDERFIRST && slider <= EQ2SLIDERLAST)
		return;
		
	switch(slider)
	{
		case VOLUMESLIDERLEFT:
			volumevalstrL(mastr2);
			StaticTextSetText(WINDOWAPP, ID_TEXT_100VOLL, mastr2);
		break;
			
		case VOLUMESLIDERRIGHT:
			volumevalstrR(mastr2);
			StaticTextSetText(WINDOWAPP, ID_TEXT_100VOLR, mastr2);
		break;
			
		case VOLUMESLIDERS: // both Left and Right
			volumevalstrL(mastr2);
			StaticTextSetText(WINDOWAPP, ID_TEXT_100VOLL, mastr2);
			volumevalstrR(mastr2);
			StaticTextSetText(WINDOWAPP, ID_TEXT_100VOLR, mastr2);
		break;
			
		case SEPARATIONSLIDER:
			if(l_outputfilter == OUTPUT_DIFF || l_outputfilter == OUTPUT_MONO) // pan
				masprintf(mastr2, STRLEN64, L"%d", (g_sets.panslider * SEPARATIONCALE) / SEPARATIONMAX);
			else
				masprintf(mastr2, STRLEN64, L"%d", (g_sets.separationslider * SEPARATIONCALE) / SEPARATIONMAX);
			StaticTextSetText(WINDOWAPP, ID_TEXT_100SEP, mastr2);
		break;
			
		case PREAMPSLIDER:
			masprintf(mastr2, STRLEN64, L"%0.1f", SliderDBFromPos(g_sets.preampslider));
			StaticTextSetText(WINDOWAPP, ID_TEXT_0, mastr2);
		break;

		case EFFECTSSLIDERPITCH:
			if(g_w[WINDOWEFFECTS])
			{
				masprintf(mastr2, STRLEN64, L"%d", g_sets.pitchslider - EFFECTSPITCHMAX / 2);
				StaticTextSetText(WINDOWEFFECTS, ID_TEXT_PITCH0, mastr2);
			}
		break;

		case EFFECTSSLIDERPITCH2:
			if(g_w[WINDOWEFFECTS])
			{
				masprintf(mastr2, STRLEN64, L"%.2f", ((g_sets.pitchslider2 * 2.0f) / EFFECTSMAX - 1.0f));
				StaticTextSetText(WINDOWEFFECTS, ID_TEXT_PITCH02, mastr2);
			}
		break;

		case EFFECTSSLIDERTEMPO:
			if(g_w[WINDOWEFFECTS])
			{
				masprintf(mastr2, STRLEN64, L"%.2f", (g_sets.temposlider * 2.0f) / EFFECTSMAX + 0.48f); // 0.48 to 2.48
				StaticTextSetText(WINDOWEFFECTS, ID_TEXT_TEMPO0, mastr2);
			}
		break;

		case EFFECTSSLIDERSPEED:
			if(g_w[WINDOWEFFECTS])
			{
				masprintf(mastr2, STRLEN64, L"%.2f", (g_sets.speedslider * 2.0f) / EFFECTSMAX + 0.48f); // 0.48 to 2.48
				StaticTextSetText(WINDOWEFFECTS, ID_TEXT_SPEED0, mastr2);
			}
		break;

		default:
			if(slider >= EQSLIDERFIRST && slider <= EQSLIDERLAST)
			{
				masprintf(mastr2, STRLEN64, L"%0.1f",
							SliderDBFromPos(g_sets.eqpresets[g_sets.eqpreset].eqslider[slider]));
				StaticTextSetText(WINDOWEQ, ID_TEXT_EQ0 + slider, mastr2);	
			}
			else if(slider >= EQ2SLIDERFIRST && slider <= EQ2SLIDERLAST)
			{
				if(g_w[WINDOWEQ2])
				{		
					masprintf(mastr2, STRLEN64, L"%0.1f",
							SliderDBFromPos(g_sets.eq2presets[g_sets.eqpreset].eqslider[slider - EQ2SLIDERFIRST]));
					StaticTextSetText(WINDOWEQ2, ID_TEXT_EQ0 + slider, mastr2);	
				}
			}
		break;
		
	} // switch

} // SetSliderCaption


void UpdateMenus()
{
	MenuSetChecked(MENUSHOW,	ID_MENU_ALTSIZE,	(g_sets.winconf) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_RANDOM,		(g_sets.random) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_REPTPLAYED,	(g_sets.reptplayed) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_LOOPLOADED,	(g_sets.looploaded) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_LOOPLIST,	(g_sets.looplist) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_AUTOSTART,	(g_sets.autostart) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_ERRLOGGING,	(g_sets.logflags & kMALOGOFF) ? 0 : 1);
	MenuSetChecked(MENUMASETS,	ID_MENU_SINGLEPLAY,	(g_sets.singleplay) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_INPUTMONITOR,	(g_sets.inputmonitor) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_EQUALIZER,	(g_sets.eqflags & MAEQACTIVE) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_EFFECTS,	(g_sets.effects & EFFECTSON) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_EQEXTRA,	(g_sets.eqflags & MAEQEXTRA) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_INVERTRIGHT,(g_sets.invertright) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_MONO,		(g_sets.monodiff & (MONOCHECKBOXL + MONOCHECKBOXR)) ? 1 : 0);
	MenuSetChecked(MENUMASETS,	ID_MENU_EQ31BANDS,	(g_sets.eqflags & MAEQ31BANDS) ? 1 : 0);

	if(g_thisinstance == 1)
		MenuSetChecked(MENUMASETS, ID_MENU_MULTIPLEINST, (g_sets.multipleinst) ? 1 : 0);
#if defined(MAGUICARBON) || defined(MAGUIGTK2)
 	else
		MenuDisable(MENUMASETS, ID_MENU_MULTIPLEINST);
#endif

#if defined(MAKEYBOARD)
	MenuSetChecked(MENUMASETS, ID_MENU_KEYBOARD,	(g_sets.keyboard) ? 1 : 0);
#endif
#if defined(MAMOUSEWHEEL)
	MenuSetChecked(MENUMASETS, ID_MENU_MOUSEWHEEL,	(g_sets.mousewheel) ? 1 : 0);
#endif
	MenuSetChecked(MENUSHOW,	ID_MENU_SHOWEQ,			(g_sets.mawin[WINDOWEQ].ison) ? 1 : 0);
	MenuSetChecked(MENUSHOW,	ID_MENU_SHOWEFFECTS,	(g_sets.mawin[WINDOWEFFECTS].ison) ? 1 : 0);
	MenuSetChecked(MENUSHOW,	ID_MENU_SHOWLIST,		(g_sets.mawin[WINDOWLIST].ison) ? 1 : 0);
	MenuSetChecked(MENUSHOW,	ID_MENU_SHOWFILEOUT,	(g_sets.mawin[WINDOWFILEOUT].ison) ? 1 : 0);
}


void UpdateDisplay()
{
	machar mastr2[STRLEN64];
	
	WindowMoveAndSetSize(WINDOWAPP, g_sets.mawin[WINDOWAPP].xpos[g_sets.winconf], g_sets.mawin[WINDOWAPP].ypos[g_sets.winconf],
								g_sets.mawin[WINDOWAPP].xsize[g_sets.winconf], g_sets.mawin[WINDOWAPP].ysize[g_sets.winconf]);
	UpdateMenus();
	
	CheckBoxSetValue(WINDOWAPP, ID_CHK_ALTSIZE,		(g_sets.winconf) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_BALANCE,		(g_sets.balance) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_RANDOM,		(g_sets.random) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_REPTPLAYED,	(g_sets.reptplayed) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_LOOPLOADED,	(g_sets.looploaded) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_LOOPLIST,	(g_sets.looplist) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_EQWINDOW,	(g_sets.mawin[WINDOWEQ].ison) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_EFFECTS,		(g_sets.effects & EFFECTSON) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_LISTWINDOW,	(g_sets.mawin[WINDOWLIST].ison) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_SINGLEPLAY,	(g_sets.singleplay) ? 1 : 0);
	// CheckBoxSetValue(WINDOWAPP, ID_CHK_USEM3U,	(g_sets.usem3u) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_EQUALIZER,	(g_sets.eqflags & MAEQACTIVE) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP,	ID_CHK_MONOLEFT,	(g_sets.monodiff & MONOCHECKBOXL) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_MONORIGHT,	(g_sets.monodiff & MONOCHECKBOXR) ? 1 : 0);
	UpdateMonoDiff(); //		ID_CHK_MONODIFF
	CheckBoxSetValue(WINDOWAPP,	ID_CHK_LIMITER,		(g_sets.limiter & CLIPPING) ? 1 : 0);
	// CheckBoxSetValue(WINDOWAPP,	ID_CHK_EQEXTRA,	(g_sets.eqflags & MAEQEXTRA) ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP,	ID_CHK_INVERTRIGHT,	(g_sets.invertright) ? 1 : 0);
	// CheckBoxSetValue(WINDOWAPP, ID_CHK_EQ31BANDS,(g_sets.eqflags & MAEQ31BANDS) ? 1 : 0);
#if !defined(NODECODEAUDIOIN)
	CheckBoxSetValue(WINDOWAPP,	ID_CHK_HOSTINENABLE,(g_sets.hostinenable) ? 1 : 0);
#else
	CheckBoxSetValue(WINDOWAPP,	ID_CHK_HOSTINENABLE, 0);
	CheckBoxDisable(WINDOWAPP, ID_CHK_HOSTINENABLE);
#endif

	masprintf(mastr2, STRLEN64, L"Song Mem %d", g_sets.buffersize);
	StaticTextSetText(WINDOWAPP, ID_TEXT_BUFFER, mastr2);

	SetGUIPopupButtons();

	SetGUIPopupButtonsAudioOut();
	SetGUIPopupButtonsAudioIn();

	SetGUIMonoStereo();
	SetGUITitleInfo();
	
	if(g_sets.limiter & CLIPPING)
		CheckBoxSetDescriptor(WINDOWAPP, ID_CHK_LIMITER, L"0.0");
	else
		CheckBoxSetDescriptor(WINDOWAPP, ID_CHK_LIMITER, L"Limit");
	
	// create/deswtroy windows
	// DoWindow(WINDOWSPLASH, g_sets.mawin[WINDOWSPLASH].ison);
	DoWindow(WINDOWLIST, g_sets.mawin[WINDOWLIST].ison);	
	DoWindow(WINDOWEQ, g_sets.mawin[WINDOWEQ].ison);	
	DoWindow(WINDOWEQ2, (g_sets.mawin[WINDOWEQ].ison && (g_sets.eqflags & MASEPARATELR)) ? 1 : 0);
	DoWindow(WINDOWEFFECTS, g_sets.mawin[WINDOWEFFECTS].ison);
	DoWindow(WINDOWABOUT, g_sets.mawin[WINDOWABOUT].ison);
	DoWindow(WINDOWADVANCED, g_sets.mawin[WINDOWADVANCED].ison);
#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
	DoWindow(WINDOWAUDIOCD, g_sets.mawin[WINDOWAUDIOCD].ison);
#endif
	DoWindow(WINDOWPROMPTNUM, g_sets.mawin[WINDOWPROMPTNUM].ison);
	DoWindow(WINDOWFILEOUT, g_sets.mawin[WINDOWFILEOUT].ison);

#if !defined(NOTEXTVIEWER)
	DoWindow(WINDOWREADME, g_sets.mawin[WINDOWREADME].ison);
	DoWindow(WINDOWCATALOG, g_sets.mawin[WINDOWCATALOG].ison);
	DoWindow(WINDOWPLAYLIST, g_sets.mawin[WINDOWPLAYLIST].ison);
	DoWindow(WINDOWKEYBINDINGS, g_sets.mawin[WINDOWKEYBINDINGS].ison);
	DoWindow(WINDOWLOG, g_sets.mawin[WINDOWLOG].ison);
#endif

	SetSliders(ALLSLIDERS);

#if defined(MAKEYBOARD)
	UpdateMenuKbLabels();
	UpdateEffectsKbLabels();
	UpdateButtonLabels();
#endif

	SetLogMenus();

	if(g_sets.output >= kOUTFIRST && g_sets.output <= kOUTLAST)
		MenuEnable(MENUSHOW, ID_MENU_SHOWFILEOUT);
	else
		MenuDisable(MENUSHOW, ID_MENU_SHOWFILEOUT);

#if defined(MANOCDA)
	MenuDisable(MENUMACONTROLS, ID_MENU_PLAYCD);
	MenuDisable(MENUMACONTROLS, ID_MENU_EJECTCD);
#endif
#if !defined(NODECODEAUDIOIN)
#else
	MenuDisable(MENUMACONTROLS, ID_MENU_AUDIOIN);
#endif

} // UpdateDisplay


void UpdateListWindowButtons()
{
	if(!g_w[WINDOWLIST])
		return;

	switch(g_sets.buffertype)
	{
		case BUFUSEMEM:
		case BUFUSEVDISK:
			PushButtonSetDescriptor(WINDOWLIST, ID_BTN_LOADED, L"Load");
			PushButtonEnable(WINDOWLIST, ID_BTN_LOADED);
		break;

		case BUFUSEDIRECT:
			PushButtonSetDescriptor(WINDOWLIST, ID_BTN_LOADED, L"(Direct)");
			PushButtonDisable(WINDOWLIST, ID_BTN_LOADED);
		break;

		default:
		break;
	}

} // UpdateListWindowButtons


void SetGUIPopupButtons()
{
	if(g_sets.output >= kOUTSOUND && g_sets.output <= kOUTLAST)
		PopupButtonSetValue(WINDOWAPP, ID_POPUP_OUTPUT, g_sets.output); 
	else
		PopupButtonSetValue(WINDOWAPP, ID_POPUP_OUTPUT, kOUTSOUND); 

	switch(g_sets.buffertype)
	{
		case BUFUSEMEM:	
			PopupButtonSetValue(WINDOWAPP, ID_POPUP_BUFFER, 0);
		break;
		case BUFUSEVDISK:
			PopupButtonSetValue(WINDOWAPP, ID_POPUP_BUFFER, 1);
		break;
		case BUFUSEDIRECT:
		default:
			PopupButtonSetValue(WINDOWAPP, ID_POPUP_BUFFER, 2);
		break;
	}

	if(g_w[WINDOWEQ])
		PopupButtonSetValue(WINDOWEQ, ID_POPUP_EQPRESET, g_sets.eqpreset);
	
} // SetGUIPopupButtons


void SetGUIPopupButtonsAudioIn()	// fill the audio popups and select the item matching the one in prefs
{									// we expect to have updated g_paudio* and g_sets before coming here
	PaHostApiIndex hostin;
	PaDeviceIndex device;
	int inindex; // 0-based index for popup
	machar mastr2[MAXLOGLINELEN];
	
    PopupButtonClear(WINDOWAPP, ID_POPUP_AUDIOHOSTIN);
    PopupButtonClear(WINDOWAPP, ID_POPUP_AUDIOINPUT);
    
	// basic checks on host info
	if(g_numaudiohosts <= 0)
	{
		maLog(L"SetGUIPopupButtonsAudioIn failed, no hosts");
		return;
	}

 	if(!g_paudiohost)
	{
		maLog(L"SetGUIPopupButtonsAudioIn failed, null pointer");
		return;
	}

	for(hostin = 0; hostin < g_numaudiohosts; hostin++)
		if(!g_paudiohost[hostin] || !g_paudiohost[hostin]->info)
		{
			maLog(L"SetGUIPopupButtonsAudioIn failed, null host pointer 2");
			return;
		}
		
	// fill the hosts popups
	for(hostin = 0; hostin < g_numaudiohosts; hostin++)
	{
		if(!g_paudiohost[hostin] || !g_paudiohost[hostin]->info)
			continue; // 
		PopupButtonAppend(WINDOWAPP, ID_POPUP_AUDIOHOSTIN,
					os2ma(mastr2, (char *)g_paudiohost[hostin]->info->name));
	}

	for(hostin = 0; hostin < g_numaudiohosts; hostin++)
	{
		if(!strcmp(g_paudiohost[hostin]->info->name, g_sets.audiohostinname))
			break;		
	}
	if(hostin >= g_numaudiohosts) // no match, use default
 	{
		hostin = 0; // use first host listed as default
		strcpy(g_sets.audiohostinname, g_paudiohost[hostin]->info->name);
	}
	g_audiohostin = hostin;
	PopupButtonSetValue(WINDOWAPP, ID_POPUP_AUDIOHOSTIN, hostin);

	// basic checks on device info for the host currently selected
	if(!g_paudiohost[hostin]->paudiodevice)
	{
		masprintf(mastr2, MAXLOGLINELEN,
			L"SetGUIPopupButtonsAudioIn null device pointer for %ls",
							os2ma(mastr2, (char *)g_paudiohost[hostin]->info->name));
		maLog(mastr2);
		return;
	}
	for(device = 0; device < g_paudiohost[hostin]->info->deviceCount; device++)
		if(!g_paudiohost[hostin]->paudiodevice[device])
		{
			masprintf(mastr2, MAXLOGLINELEN,
					L"SetGUIPopupButtonsAudioIn device pointer for %ls device %d",
							os2ma(mastr2, (char *)g_paudiohost[hostin]->info->name), device);
			maLog(mastr2);
			return;
		}

	// fill the device popups with devices from the host currently selected
	for(device = 0, inindex = -1; device < g_paudiohost[hostin]->info->deviceCount; device++)
	{
		if(g_paudiohost[hostin]->paudiodevice[device]->info->maxInputChannels > 0)
		{
			inindex++;
			g_paudiohost[hostin]->paudiodevice[device]->index = inindex;
            PopupButtonAppend(WINDOWAPP, ID_POPUP_AUDIOINPUT,
                os2ma(mastr2, (char *)g_paudiohost[hostin]->paudiodevice[device]->info->name));
		}
	}

	if((inindex = maSetAudioInputDevice()) >= 0)
        PopupButtonSetValue(WINDOWAPP, ID_POPUP_AUDIOINPUT, inindex);
	else // no input channels at all for this host
		g_audiodevicein = -1; // cry

} // SetGUIPopupButtonsAudioIn


void SetGUIPopupButtonsAudioOut()	// fill the audio popups and select the item matching the one in prefs
{									// we expect to have updated g_paudio* and g_sets before coming here
	PaHostApiIndex hostout;
	PaDeviceIndex device;
	int outindex; // 0-based index for popup
	machar mastr2[MAXLOGLINELEN];

	PopupButtonClear(WINDOWAPP, ID_POPUP_AUDIOHOSTOUT);
	PopupButtonClear(WINDOWAPP, ID_POPUP_AUDIOOUTPUT);
	
	// basic checks on host info
	if(g_numaudiohosts <= 0)
	{
		maLog(L"SetGUIPopupButtonsAudioOut failed, no hosts");
		return;
	}

	if(!g_paudiohost)
	{
		maLog(L"SetGUIPopupButtonsAudioOut failed, null pointer");
		return;
	}
	for(hostout = 0; hostout < g_numaudiohosts; hostout++)
		if(!g_paudiohost[hostout] || !g_paudiohost[hostout]->info)
		{
			maLog(L"SetGUIPopupButtonsAudioOut failed, null host pointer 2");
			return;
		}
		
	// fill the hosts popups
	for(hostout = 0; hostout < g_numaudiohosts; hostout++)
	{
		if(!g_paudiohost[hostout] || !g_paudiohost[hostout]->info)
			continue; // 
		PopupButtonAppend(WINDOWAPP, ID_POPUP_AUDIOHOSTOUT,
					os2ma(mastr2, (char *)g_paudiohost[hostout]->info->name));
	}
	for(hostout = 0; hostout < g_numaudiohosts; hostout++)
	{
		if(!strcmp(g_paudiohost[hostout]->info->name, g_sets.audiohostoutname))
			break;		
	}
	if(hostout >= g_numaudiohosts) // no match, use default
 	{
		hostout = 0; // use first host listed as default
		strcpy(g_sets.audiohostoutname, g_paudiohost[hostout]->info->name);
	}
	g_audiohostout = hostout;
	PopupButtonSetValue(WINDOWAPP, ID_POPUP_AUDIOHOSTOUT, hostout);

	// basic checks on device info for the host currently selected
	if(!g_paudiohost[hostout]->paudiodevice)
	{
		masprintf(mastr2, MAXLOGLINELEN,
			L"SetGUIPopupButtonsAudioOut null device pointer for %ls",
							os2ma(mastr2, (char *)g_paudiohost[hostout]->info->name));
		maLog(mastr2);
		return;
	}
	for(device = 0; device < g_paudiohost[hostout]->info->deviceCount; device++)
		if(!g_paudiohost[hostout]->paudiodevice[device])
		{
			masprintf(mastr2, MAXLOGLINELEN,
					L"SetGUIPopupButtonsAudioOut device pointer for %ls device %d",
							os2ma(mastr2, (char *)g_paudiohost[hostout]->info->name), device);
			maLog(mastr2);
			return;
		}

	// fill the device popups with devices from the host currently selected
	for(device = 0, outindex = -1, outindex = -1; device < g_paudiohost[hostout]->info->deviceCount; device++)
	{
		if(g_paudiohost[hostout]->paudiodevice[device]->info->maxOutputChannels > 0)
		{
			outindex++;
			g_paudiohost[hostout]->paudiodevice[device]->index = outindex;
			PopupButtonAppend(WINDOWAPP, ID_POPUP_AUDIOOUTPUT,
					os2ma(mastr2, (char *)g_paudiohost[hostout]->paudiodevice[device]->info->name));
		}
	}

	if((outindex = maSetAudioOutputDevice()) >= 0)
		PopupButtonSetValue(WINDOWAPP, ID_POPUP_AUDIOOUTPUT, outindex);
	else // no output channels at all for this host
		g_audiodeviceout = -1; // weep

} // SetGUIPopupButtonsAudioOut


int ListLoaded()
{
	int j;
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	
#if !defined(MAGUINONE)
	if(!g_w[WINDOWLIST])
		return 1;
#endif
		
	CheckBoxHide(WINDOWLIST, ID_CHK_SHOWINDEX);
	CheckBoxHide(WINDOWLIST, ID_CHK_SHOWDIR);
	CheckBoxHide(WINDOWLIST, ID_CHK_SHOWPATH);
	CheckBoxHide(WINDOWLIST, ID_CHK_ADDMODE);

	ListWindowSetTitle(MASONGSLOADEDTITLE);
	ListWindowClear();
	ListWindowSetHeader(MASONGSLOADED);
	
	if(g_sets.buffertype == BUFUSEDIRECT)
	{
		ListWindowAddString(MADIRECTMODE);	
		return 1;
	}

	if(g_loadednum <= 0)
	{
		ListWindowAddString(MANOSONGSLOADED);	
		ListWindowShowRow(-1); // scroll to top, no hilite
		return 1;
	}
			
	for(j = 0; j < g_loadednum; j++)
		ListWindowAddString(fs2ma(mastr2, g_loadedname[j]));
		
	ListWindowShowRow(g_loadedcurrent);
	
	ListSetColumnWidth();
	
	return 0;
	
} // ListLoaded


void SetGUIMonoStereo()
{
	if(CheckBoxGetValue(WINDOWAPP, ID_CHK_MONOLEFT))
	{	
		StaticTextSetText(WINDOWAPP, ID_TEXT_OUTPUT, L"Output Mono");
		if(CheckBoxGetValue(WINDOWAPP, ID_CHK_MONORIGHT)) // L and R both checked
		{
			CheckBoxShow(WINDOWAPP, ID_CHK_MONODIFF);			
			StaticTextSetText(WINDOWAPP, ID_TEXT_SEP, L"Pan");
			StaticTextSetText(WINDOWAPP, ID_TEXT_M, L"L");
			StaticTextSetText(WINDOWAPP, ID_TEXT_S, L"R");
			
			if(CheckBoxGetValue(WINDOWAPP, ID_CHK_MONODIFF))
			{
				l_outputfilter = OUTPUT_DIFF;
				CheckBoxSetDescriptor(WINDOWAPP, ID_CHK_MONODIFF, L"L-R");
			}
			else
			{
				l_outputfilter = OUTPUT_MONO;
				CheckBoxSetDescriptor(WINDOWAPP, ID_CHK_MONODIFF, L"L+R");
			}

			SliderEnable(WINDOWAPP, ID_SLIDER_VOLUMESEP);
			SetSliders(SEPARATIONSLIDER);
			MenuSetChecked(MENUMASETS, ID_MENU_MONO, 1);
		}
		else // L only
		{
			CheckBoxHide(WINDOWAPP, ID_CHK_MONODIFF);
			CheckBoxSetDescriptor(WINDOWAPP, ID_CHK_MONODIFF, L"");
			l_outputfilter = OUTPUT_LEFT;
			StaticTextSetText(WINDOWAPP, ID_TEXT_SEP, L"");
			StaticTextSetText(WINDOWAPP, ID_TEXT_M, L"");
			StaticTextSetText(WINDOWAPP, ID_TEXT_S, L"");
			SliderDisable(WINDOWAPP, ID_SLIDER_VOLUMESEP);
			MenuSetChecked(MENUMASETS, ID_MENU_MONO, 1);
		}
	}
	else if(CheckBoxGetValue(WINDOWAPP, ID_CHK_MONORIGHT)) // R only
	{	
		CheckBoxHide(WINDOWAPP, ID_CHK_MONODIFF);
		CheckBoxSetDescriptor(WINDOWAPP, ID_CHK_MONODIFF, L"");
		l_outputfilter = OUTPUT_RIGHT;
		StaticTextSetText(WINDOWAPP, ID_TEXT_OUTPUT, L"Output Mono");
		StaticTextSetText(WINDOWAPP, ID_TEXT_SEP, L"");
		StaticTextSetText(WINDOWAPP, ID_TEXT_M, L"");
		StaticTextSetText(WINDOWAPP, ID_TEXT_S, L"");
		SliderDisable(WINDOWAPP, ID_SLIDER_VOLUMESEP);
		MenuSetChecked(MENUMASETS, ID_MENU_MONO, 1);
	}
	else
	{
		CheckBoxSetDescriptor(WINDOWAPP, ID_CHK_MONODIFF, L"");
		CheckBoxShow(WINDOWAPP, ID_CHK_MONODIFF);
		CheckBoxSetDescriptor(WINDOWAPP, ID_CHK_MONODIFF, L"rev");
		StaticTextSetText(WINDOWAPP, ID_TEXT_OUTPUT, L"Output Stereo");
		StaticTextSetText(WINDOWAPP, ID_TEXT_SEP, L"Sep");
		StaticTextSetText(WINDOWAPP, ID_TEXT_M, L"m");
		StaticTextSetText(WINDOWAPP, ID_TEXT_S, L"s");
		
		l_outputfilter = OUTPUT_STEREO;
		
		if(CheckBoxGetValue(WINDOWAPP, ID_CHK_MONODIFF))
			l_outputfilter |= OUTPUT_REVERSE;

		SliderEnable(WINDOWAPP, ID_SLIDER_VOLUMESEP);
		SetSliders(SEPARATIONSLIDER);
		MenuSetChecked(MENUMASETS, ID_MENU_MONO, 0);
	}
	
} // SetGUIMonoStereo


void SetGUITitleInfo()
{
	if(g_sets.titleinfo == 1)
	{
		CheckBoxSetValue(WINDOWAPP, ID_CHK_TITLEINFO, 1);
		MenuSetChecked(MENUMASETS, ID_SUBMENU_TITLEINFO1, 0);
		MenuSetChecked(MENUMASETS, ID_SUBMENU_TITLEINFO2, 1);
		MenuSetChecked(MENUMASETS, ID_SUBMENU_TITLEINFO3, 0);
	}
	else if(g_sets.titleinfo == 2)
	{
		CheckBoxSetValue(WINDOWAPP, ID_CHK_TITLEINFO, 1);
		MenuSetChecked(MENUMASETS, ID_SUBMENU_TITLEINFO1, 0);
		MenuSetChecked(MENUMASETS, ID_SUBMENU_TITLEINFO2, 0);
		MenuSetChecked(MENUMASETS, ID_SUBMENU_TITLEINFO3, 1);
	}
	else
	{
		CheckBoxSetValue(WINDOWAPP, ID_CHK_TITLEINFO, 0);
		MenuSetChecked(MENUMASETS, ID_SUBMENU_TITLEINFO1, 1);
		MenuSetChecked(MENUMASETS, ID_SUBMENU_TITLEINFO2, 0);
		MenuSetChecked(MENUMASETS, ID_SUBMENU_TITLEINFO3, 0);
	}
	
} // SetGUITitleInfo


void SetPosSlider(int value)
{
	int lvalue = value;
	
	if(value >= 0)
	{	
		lvalue = value;
		l_posslidersetvalue = 1;
	}
	else
	{	
		lvalue = -value;
		l_posslidersetvalue = -1;
	}
	
	HSliderSetValue(WINDOWAPP, ID_SLIDER_POSITION, lvalue);
	
	OnSliderPosition(lvalue);
	
} // SetPosSlider


// this gets called after moving the position slider manually and from SetPosSlider())
void OnSliderPosition(int value)
{	
	machar mastr2[32];
	
	switch(l_posslidersetvalue)
	{
		case 1: //  SetValue was called and value is a fraction of g_state.length
			l_poscurrent = (int)(LROUND(((g_state.duration * (double)value) / POSMAX)));
			MillisecondsToStrHMSD(mastr2, l_poscurrent);
			StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEPOS, mastr2);
		break;
		
		case 0: // the slider was moved manually
			if(g_state.running) 
			{	
				l_poscurrent = l_sliderposms = (int)(LROUND((g_state.duration * (double)value) / POSMAX));
				MillisecondsToStrHMSD(mastr2, l_sliderposms);
				StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEPOS, mastr2);
				l_posslidervalue = value;
				l_possliderchanged = 1;
			}
			else
				StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEPOS, NOTIMESTR);
		break;
		
		default:
			StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEPOS, ZEROMSSTR); // NOTIMESTR);
		break;
		
	} // switch
	
	l_posslidersetvalue = 0;

} // OnSliderPosition


void OnSliderVolumeR(int value) // Right
{	
	g_sets.volumesliderright = value;
	
	if(!g_sets.balance)
	{
		g_sets.volumesliderleft = g_sets.volumesliderright = volminmaxR();
		SliderSetValue(WINDOWAPP, ID_SLIDER_VOLUMEL, g_sets.volumesliderleft);		
		SetSliderCaption(VOLUMESLIDERLEFT);
	}

	maOutputSetVolume((kVOLMAX * volumevalL())  / VOLUMEMAX, (kVOLMAX * volumevalR()) / VOLUMEMAX);
		
	SetSliderCaption(VOLUMESLIDERRIGHT);
		
} // OnSliderVolumeR


void OnSliderVolumeL(int value) // Left
{	
	g_sets.volumesliderleft = value;
	
	if(!g_sets.balance)
	{
		g_sets.volumesliderright = g_sets.volumesliderleft = volminmaxL();
		SliderSetValue(WINDOWAPP, ID_SLIDER_VOLUMER, g_sets.volumesliderright);
		SetSliderCaption(VOLUMESLIDERRIGHT);		
	}
	
	maOutputSetVolume((kVOLMAX * volumevalL())  / VOLUMEMAX, (kVOLMAX * volumevalR()) / VOLUMEMAX);
	
	SetSliderCaption(VOLUMESLIDERLEFT);
		
} // OnSliderVolumeL


void OnSliderSeparation(int value) // Separation
{	
	if(l_outputfilter == OUTPUT_DIFF || l_outputfilter == OUTPUT_MONO) // pan
	{
		g_sets.panslider = value;
		l_pan = value;
		l_panright = (float)(l_pan  / (double)SEPARATIONMAX);
		l_panleft  = (float)((SEPARATIONMAX - l_pan) / (double)SEPARATIONMAX);
	}
	else // separation
	{
		g_sets.separationslider = value;
		l_separation = value;
		l_separationch  = (float)(0.5 + ((0.5 * l_separation) / SEPARATIONMAX));
		l_separationmix = (float)(1. - l_separationch);
	}
	
	SetSliderCaption(SEPARATIONSLIDER);
	
} // OnSliderSeparation


void OnSliderPreamp(int value)
{	
	g_sets.preampslider = value;	
	SetSliderCaption(PREAMPSLIDER);	
	MadDoParamChange();
	
} // OnSliderPreamp


void OnSliderEq(int slider, int value, int setpos)
{	
	int i;
	int first, last;
	int delta;
	
	g_eqzoomval = -1.0;
	if(slider >= EQSLIDERFIRST && slider <= EQSLIDERLAST)
		g_sets.eqpresets[g_sets.eqpreset].eqslider[slider] = value;
	else if(slider >= EQ2SLIDERFIRST && slider <= EQ2SLIDERLAST)
		g_sets.eq2presets[g_sets.eqpreset].eqslider[slider - EQ2SLIDERFIRST] = value;
	SetSliderCaption(slider);
	if(setpos)
		SetSliders(slider);

	if(g_sets.eqflags & MAEQLINK)
	{
		switch(slider)
		{
			case 2:
				first = 0;
				last = 4;
				
				g_sets.eqpresets[g_sets.eqpreset].eqslider[0] =
				g_sets.eqpresets[g_sets.eqpreset].eqslider[1] = value;
					
				delta = (value - g_sets.eqpresets[g_sets.eqpreset].eqslider[5]) / 3;
				
				g_sets.eqpresets[g_sets.eqpreset].eqslider[3] = 
					g_sets.eqpresets[g_sets.eqpreset].eqslider[5] + delta + delta;
							
				g_sets.eqpresets[g_sets.eqpreset].eqslider[4] = 
					g_sets.eqpresets[g_sets.eqpreset].eqslider[5] + delta;		
			break;
			
			case 5: case 8: case 11: case 14: case 17: case 20: case 23: case 26:
				first = slider - 2;
				last = slider + 2;
				
				delta = (g_sets.eqpresets[g_sets.eqpreset].eqslider[slider] - 
						g_sets.eqpresets[g_sets.eqpreset].eqslider[slider - 3]) / 3;

				g_sets.eqpresets[g_sets.eqpreset].eqslider[slider - 2] = 
					g_sets.eqpresets[g_sets.eqpreset].eqslider[slider - 3] + delta;		
				
				g_sets.eqpresets[g_sets.eqpreset].eqslider[slider - 1] = 
					g_sets.eqpresets[g_sets.eqpreset].eqslider[slider - 3] + delta + delta;
							
				delta = (g_sets.eqpresets[g_sets.eqpreset].eqslider[slider] - 
							g_sets.eqpresets[g_sets.eqpreset].eqslider[slider + 3]) / 3;
				
				g_sets.eqpresets[g_sets.eqpreset].eqslider[slider + 1] = 
					g_sets.eqpresets[g_sets.eqpreset].eqslider[slider + 3] + delta + delta;
							
				g_sets.eqpresets[g_sets.eqpreset].eqslider[slider + 2] = 
					g_sets.eqpresets[g_sets.eqpreset].eqslider[slider + 3] + delta;		
			break;
			
			case 29:
				first = 27;
				last = 30;
				
				delta = (g_sets.eqpresets[g_sets.eqpreset].eqslider[29] - 
							g_sets.eqpresets[g_sets.eqpreset].eqslider[26]) / 3;
				
				g_sets.eqpresets[g_sets.eqpreset].eqslider[27] = 
					g_sets.eqpresets[g_sets.eqpreset].eqslider[26] + delta;
							
				g_sets.eqpresets[g_sets.eqpreset].eqslider[28] = 
					g_sets.eqpresets[g_sets.eqpreset].eqslider[26] + delta + delta;
					
				g_sets.eqpresets[g_sets.eqpreset].eqslider[30] = value;		
			break;
			
			default: // not one of 10 bands, skip the 'for i' below
				first = 1;
				last = 0;
			break;
		}
		
		for(i = first; i <= last; i++)
		{
			if(i == slider)
				continue;
			g_eqzoomval = -1.0;
			SetSliders(i);
		}
	}
		
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();
	
} // OnSliderEq


void OnSliderEq2(int slider, int value, int setpos)
{	
	int i;
	int lslider;
	int first, last;
	int delta;
	
	lslider = slider - EQ2SLIDERFIRST;
	g_eqzoomval = -1.0;
	g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider] = value;
	SetSliderCaption(slider);
	if(setpos)
		SetSliders(slider);

	if(g_sets.eqflags & MAEQLINK)
	{
		switch(lslider)
		{
			case 2:
				first = 0 + EQ2SLIDERFIRST;
				last = 4 + EQ2SLIDERFIRST;
				
				g_sets.eq2presets[g_sets.eqpreset].eqslider[0] =
				g_sets.eq2presets[g_sets.eqpreset].eqslider[1] = value;
					
				delta = (value - g_sets.eq2presets[g_sets.eqpreset].eqslider[5]) / 3;
				
				g_sets.eq2presets[g_sets.eqpreset].eqslider[3] = 
					g_sets.eq2presets[g_sets.eqpreset].eqslider[5] + delta + delta;
							
				g_sets.eq2presets[g_sets.eqpreset].eqslider[4] = 
					g_sets.eq2presets[g_sets.eqpreset].eqslider[5] + delta;		
			break;
			
			case 5: case 8: case 11: case 14: case 17: case 20: case 23: case 26:
				first = lslider - 2 + EQ2SLIDERFIRST;
				last = lslider + 2 + EQ2SLIDERFIRST;
				
				delta = (g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider] - 
						g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider - 3]) / 3;

				g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider - 2] = 
					g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider - 3] + delta;		
				
				g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider - 1] = 
					g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider - 3] + delta + delta;
							
				delta = (g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider] - 
							g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider + 3]) / 3;
				
				g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider + 1] = 
					g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider + 3] + delta + delta;
							
				g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider + 2] = 
					g_sets.eq2presets[g_sets.eqpreset].eqslider[lslider + 3] + delta;		
			break;
			
			case 29:
				first = 27 + EQ2SLIDERFIRST;
				last = 30 + EQ2SLIDERFIRST;
				
				delta = (g_sets.eq2presets[g_sets.eqpreset].eqslider[29] - 
							g_sets.eq2presets[g_sets.eqpreset].eqslider[26]) / 3;
				
				g_sets.eq2presets[g_sets.eqpreset].eqslider[27] = 
					g_sets.eq2presets[g_sets.eqpreset].eqslider[26] + delta;
							
				g_sets.eq2presets[g_sets.eqpreset].eqslider[28] = 
					g_sets.eq2presets[g_sets.eqpreset].eqslider[26] + delta + delta;
					
				g_sets.eq2presets[g_sets.eqpreset].eqslider[30] = value;		
			break;
			
			default: // not one of 10 bands, skip the 'for i' below
				first = 1;
				last = 0;
			break;
		}
		
		for(i = first; i <= last; i++)
		{
			if(i == slider)
				continue;
			g_eqzoomval = -1.0;
			SetSliders(i);
		}
	}
		
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();
	
} // OnSliderEq2


void OnEditTextEq()
{
	int i;
	long outlen;
	fschar fsstr[EQNAMELEN];
	machar mastr2[MAXLOGLINELEN];

	EditTextGetText(WINDOWEQ, ID_EDITTEXT_EQ, mastr2, EQNAMELEN, &outlen, 0, -1);
	ma2fsfn(fsstr, mastr2);
	if(fsstrlen(fsstr) == 0)
		fssprintf(fsstr, EQNAMELEN, fsT("Preset%d"), g_sets.eqpreset + 1);
	if(fsstrcmp(fsstr, g_sets.eqpresets[g_sets.eqpreset].eqname))
	{
		fsstrcpy(g_sets.eqpresets[g_sets.eqpreset].eqname, fsstr);
#if defined(MAGUICARBON) || defined(MAGUIGTK2)
		PopupButtonSetText(WINDOWEQ, ID_POPUP_EQPRESET, g_sets.eqpreset, fs2ma(mastr2, g_sets.eqpresets[g_sets.eqpreset].eqname));
		if(g_sets.eqpreset == 0) // force a redraw
			i = 1;
		else
			i = 0;
		PopupButtonSetValue(WINDOWEQ, ID_POPUP_EQPRESET, i);
#else
		PopupButtonClear(WINDOWEQ, ID_POPUP_EQPRESET); // PopupButtonSetText fails...
		for(i = 0; i < EQNUMPRESETS; i++)
			PopupButtonAppend(WINDOWEQ, ID_POPUP_EQPRESET, fs2ma(mastr2, g_sets.eqpresets[i].eqname));
#endif
		PopupButtonSetValue(WINDOWEQ, ID_POPUP_EQPRESET, g_sets.eqpreset);
	}

} // OnEditTextEq


void OnSliderPitch(int value)
{	
	g_sets.pitchslider = value;	
	SetSliderCaption(EFFECTSSLIDERPITCH);	
	if(g_sets.effects & EFFECTSON && g_sets.effects & EFFECTSPITCH)
		g_effectschange = 1;
		
} // OnSliderPitch


void OnSliderPitch2(int value)
{	
	g_sets.pitchslider2 = value;	
	SetSliderCaption(EFFECTSSLIDERPITCH2);	
	if(g_sets.effects & EFFECTSON && g_sets.effects & EFFECTSPITCH)
		g_effectschange = 1;
		
} // OnSliderPitch2


void OnSliderTempo(int value)
{	
	g_sets.temposlider = value;	
	SetSliderCaption(EFFECTSSLIDERTEMPO);	
	if(g_sets.effects & EFFECTSON && g_sets.effects & EFFECTSTEMPO)
		g_effectschange = 1;
		
} // OnSliderTempo


void OnSliderSpeed(int value)
{	
	g_sets.speedslider = value;	
	SetSliderCaption(EFFECTSSLIDERSPEED);	
	if(g_sets.effects & EFFECTSON && g_sets.effects & EFFECTSSPEED)
		g_effectschange = 1;
		
} // OnSliderSpeed


#define SPIN_BUFFER_INCREMENT 16
void OnLittleArrowBuffer(int value)
{
	int incr;
	machar mastr2[STRLEN64];

	if(g_sets.buffertype == BUFUSEVDISK)
		incr = 1;
	else
		incr = SPIN_BUFFER_INCREMENT;
		
	if(value != -1 && value != 1)
		return;

	if(value > 0) // up
		g_sets.buffersize += incr;
	else
		g_sets.buffersize -= incr;
	
	if(g_sets.buffersize > SONGMEMMAX)
		g_sets.buffersize = SONGMEMMAX;

	if(g_sets.buffersize < SONGMEMMIN)
		g_sets.buffersize = SONGMEMMIN;
		
	masprintf(mastr2, STRLEN64, L"Song Mem %d", g_sets.buffersize);
	StaticTextSetText(WINDOWAPP, ID_TEXT_BUFFER, mastr2);
	
} // OnLittleArrowBuffer


void OnLittleArrowVolumeL(int value)
{
	int val;
	
	if(value != -1 && value != 1)
		return;

	val = g_sets.volumesliderleft;
		
	if(value > 0)	// up
		val += 1;
	else			// down
		val -= 1;
			
	OnSliderVolumeL(val);

	SetSliders(VOLUMESLIDERLEFT);
			
} // OnLittleArrowVolumeL


void OnLittleArrowVolumeR(int value)
{
	int val;
	
	if(value != -1 && value != 1)
		return;

	val = g_sets.volumesliderright;
		
	if(value > 0)	// up
			val += 1;
	else			// down
			val -= 1;
			
	OnSliderVolumeR(val);
	
	SetSliders(VOLUMESLIDERRIGHT);
	
} // OnLittleArrowVolumeR


void ChangeVolume(int delta)
{
	int val;
	machar mastr[MAXLOGLINELEN];
	
	if(!delta)
		return;

	if(g_sets.balance)
	{
		if(delta > 0 && g_sets.volumesliderleft >= VOLUMEMAX && g_sets.volumesliderright >= VOLUMEMAX)
			return;
		if(delta < 0 && g_sets.volumesliderleft <= VOLUMEMIN && g_sets.volumesliderright <= VOLUMEMIN)
			return;
	}

	val = g_sets.volumesliderright + delta;
	
	OnSliderVolumeR(val);
	SetSliders(VOLUMESLIDERRIGHT);
	
	if(g_sets.balance)
	{
		val = g_sets.volumesliderleft + delta;
		OnSliderVolumeL(val);
		SetSliders(VOLUMESLIDERLEFT);
	}
	
	masprintf(mastr, MAXLOGLINELEN, L"Volume %d%%", (int)((100. * volminmaxL()) / VOLUMEMAX));
	WindowSetTitle(WINDOWAPP, mastr);
#if !defined(MAGUINONE)
	l_temptitle = 1;
#endif
	
} // ChangeVolume


void OnLittleArrowSeparation(int value)
{
	int val;

	if(value != -1 && value != 1)
		return;

	if(l_outputfilter == OUTPUT_DIFF || l_outputfilter == OUTPUT_MONO) // pan
		val = g_sets.panslider;
	else
		val = g_sets.separationslider;
	
	if(value > 0)	// up
		val += 1;
	else			// down
		val -= 1;
			
	if(val > SEPARATIONMAX)
		val = SEPARATIONMAX;
	else if(val < SEPARATIONMIN)
		val = SEPARATIONMIN;

	if(l_outputfilter == OUTPUT_DIFF || l_outputfilter == OUTPUT_MONO) // pan
	{
		g_sets.panslider = val;
		l_pan = val;
		l_panright = (float)(l_pan  / (double)SEPARATIONMAX);
		l_panleft  = (float)((SEPARATIONMAX - l_pan) / (double)SEPARATIONMAX);
	}
	else
	{
		g_sets.separationslider = val;
		l_separation = val;
		l_separationch  = (float)(0.5 + ((0.5 * l_separation) / SEPARATIONMAX));
		l_separationmix = (float)(1. - l_separationch);
	}

	SetSliders(SEPARATIONSLIDER);
			
} // OnLittleArrowSeparation


void OnLittleArrowPreamp(int value)
{
	if(value != -1 && value != 1)
		return;

	if(value > 0)	// up
		g_sets.preampslider += 1;
	else			// down
		g_sets.preampslider -= 1;
			
	if(g_sets.preampslider > PREAMPMAX)
		g_sets.preampslider = PREAMPMAX;
	else if(g_sets.preampslider < PREAMPMIN)
		g_sets.preampslider = PREAMPMIN;

	SetSliders(PREAMPSLIDER);
			
	MadDoParamChange();

} // OnLittleArrowPreamp


void OnLittleArrowBegin(int value)
{
	machar mastr2[32];
	maint64 ms;
	
	if((value != -1 && value != 1) || !g_posbegon)
		return;
		
	ms = g_posbeg;	

	if(value > 0)	// up
	{
		if((mauint64)(ms += POSINCR) >= g_posend)
			ms = g_posend - MINPOSDELTA;
	}
	else			// down
		ms -= POSDECR;

	if(ms < 0)
		ms = 0;

	MillisecondsToStrHMSD(mastr2, ms);
	StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEBEG, mastr2);	

	g_posbeg = (mauint64)ms;
					
} // OnLittleArrowBegin


void OnLittleArrowEnd(int value)
{
	machar mastr2[32];
	maint64 ms;	

	if((value != -1 && value != 1) || !g_posendon)
		return;

	ms = g_posend;	

	if(value > 0)	// up
	{
		if((mauint64)(ms += POSINCR) > g_state.duration)
			ms = g_state.duration;
	}
	else			// down
		ms -= POSDECR;
	
	if(ms < (maint64)(g_posbeg + MINPOSDELTA))
		ms = (maint64)(g_posbeg + MINPOSDELTA);

	MillisecondsToStrHMSD(mastr2, ms);
	StaticTextSetText(WINDOWAPP, ID_TEXT_TIMEEND, mastr2);

	g_posend = ms;
	
} // OnLittleArrowEnd


void OnLittleArrowEqOffset(int value)
{
	if(value != -1 && value != 1)
		return;

	if(value > 0)	// up
	{
		if(++g_sets.eqpresets[g_sets.eqpreset].eqoffset > DBRANGE)
			g_sets.eqpresets[g_sets.eqpreset].eqoffset = DBRANGE;
	}
	else			// down
	{
		if(--g_sets.eqpresets[g_sets.eqpreset].eqoffset < -DBRANGE)
			g_sets.eqpresets[g_sets.eqpreset].eqoffset = -DBRANGE;
	}

	if(g_w[WINDOWEQ])
		StaticTextSetValue(WINDOWEQ, ID_TEXT_EQOFFSETVAL, g_sets.eqpresets[g_sets.eqpreset].eqoffset);

	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();
	
} // OnLittleArrowEqOffset


void OnLittleArrowEq(int band, int value)
{
	int val;
	
	if(value == 1)
	{
		if((val = g_sets.eqpresets[g_sets.eqpreset].eqslider[band]) >= DBSLIDERMAX)
			return;
		
		OnSliderEq(band, val + 1, 1);
	}
	else if(value == -1)
	{
		if((val = g_sets.eqpresets[g_sets.eqpreset].eqslider[band]) <= DBSLIDERMIN)
			return;
	
		OnSliderEq(band, val - 1, 1);
	}
	
} // OnLittleArrowEq


void OnLittleArrowEq2Offset(int value)
{
	if(value != -1 && value != 1)
		return;

	if(value > 0)	// up
	{
		if(++g_sets.eq2presets[g_sets.eqpreset].eqoffset > DBRANGE)
			g_sets.eq2presets[g_sets.eqpreset].eqoffset = DBRANGE;
	}
	else			// down
	{
		if(--g_sets.eq2presets[g_sets.eqpreset].eqoffset < -DBRANGE)
			g_sets.eq2presets[g_sets.eqpreset].eqoffset = -DBRANGE;
	}

	if(g_w[WINDOWEQ2])
		StaticTextSetValue(WINDOWEQ2, ID_TEXT_EQOFFSETVAL, g_sets.eq2presets[g_sets.eqpreset].eqoffset);

	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();
	
} // OnLittleArrowEq2Offset


void OnLittleArrowEq2(int band, int value)
{
	int val;
	
	if(value == 1)
	{
		if((val = g_sets.eq2presets[g_sets.eqpreset].eqslider[band - EQ2SLIDERFIRST]) >= DBSLIDERMAX)
			return;
		
		OnSliderEq(band, val + 1, 1);
	}
	else if(value == -1)
	{
		if((val = g_sets.eq2presets[g_sets.eqpreset].eqslider[band - EQ2SLIDERFIRST]) <= DBSLIDERMIN)
			return;
	
		OnSliderEq(band, val - 1, 1);
	}
	
} // OnLittleArrowEq2


void OnLittleArrowPitch(int value)
{
	int val;
	
	if(value != -1 && value != 1)
		return;

	val = g_sets.pitchslider;
		
	if(value > 0)	// up
		val += 1;
	else			// down
		val -= 1;
			
	if(val > EFFECTSPITCHMAX)
		val = EFFECTSPITCHMAX;
	else if(val < EFFECTSPITCHMIN)
		val = EFFECTSPITCHMIN;

	g_sets.pitchslider = val;
	
	SetSliders(EFFECTSSLIDERPITCH);
	
	if(g_sets.effects & EFFECTSON && g_sets.effects & EFFECTSPITCH)
		g_effectschange = 1;
	
} // OnLittleArrowPitch


void OnLittleArrowPitch2(int value)
{
	int val;
	
	if(value != -1 && value != 1)
		return;

	val = g_sets.pitchslider2;
		
	if(value > 0)	// up
		val += 1;
	else			// down
		val -= 1;
			
	if(val > EFFECTSMAX)
		val = EFFECTSMAX;
	else if(val < EFFECTSMIN)
		val = EFFECTSMIN;

	g_sets.pitchslider2 = val;
	
	SetSliders(EFFECTSSLIDERPITCH2);
	
	if(g_sets.effects & EFFECTSON && g_sets.effects & EFFECTSPITCH)
		g_effectschange = 1;
	
} // OnLittleArrowPitch2


void OnLittleArrowTempo(int value)
{
	int val;
	
	if(value != -1 && value != 1)
		return;

	val = g_sets.temposlider;
		
	if(value > 0)	// up
		val += 1;
	else			// down
		val -= 1;
			
	if(val > EFFECTSMAX)
		val = EFFECTSMAX;
	else if(val < EFFECTSMIN)
		val = EFFECTSMIN;

	g_sets.temposlider = val;
	
	SetSliders(EFFECTSSLIDERTEMPO);
	
	if(g_sets.effects & EFFECTSON && g_sets.effects & EFFECTSTEMPO)
		g_effectschange = 1;
	
} // OnLittleArrowTempo


void OnLittleArrowSpeed(int value)
{
	int val;
	
	if(value != -1 && value != 1)
		return;

	val = g_sets.speedslider;
		
	if(value > 0)	// up
		val += 1;
	else			// down
		val -= 1;
			
	if(val > EFFECTSMAX)
		val = EFFECTSMAX;
	else if(val < EFFECTSMIN)
		val = EFFECTSMIN;

	g_sets.speedslider = val;
	
	SetSliders(EFFECTSSLIDERSPEED);
	
	if(g_sets.effects & EFFECTSON && g_sets.effects & EFFECTSSPEED)
		g_effectschange = 1;
	
} // OnLittleArrowSpeed


void DoEqWindow()
{
	ToggleWindow(WINDOWEQ);
	DoWindow(WINDOWEQ2, (g_sets.mawin[WINDOWEQ].ison && (g_sets.eqflags & MASEPARATELR)) ? 1 : 0);
}


void DoRandom(int checked)
{
	g_sets.random = checked;
	MenuSetChecked(MENUMASETS, ID_MENU_RANDOM, checked);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_RANDOM, checked);
}


void DoReptPlayed(int checked)
{
	g_sets.reptplayed = checked;
	CheckBoxSetValue(WINDOWAPP, ID_CHK_REPTPLAYED, checked);
}


void DoLoopLoaded(int checked)
{
	g_sets.looploaded = checked;
	CheckBoxSetValue(WINDOWAPP, ID_CHK_LOOPLOADED, checked);
}


void DoLoopList(int checked)
{
	g_sets.looplist = checked;
	CheckBoxSetValue(WINDOWAPP, ID_CHK_LOOPLIST, checked);
}


void DoErrLogging(int checked)
{
	if(checked)
	{
		g_sets.logflags &= ~kMALOGOFF;
		l_logoff = 0;
		maLog(L"Error Logging turned on");
	}
	else
	{
		maLog(L"Error Logging turned off");
		g_sets.logflags |= kMALOGOFF;
		l_logoff = 1;
	}

} // DoErrLogging


void DoSinglePlay(int checked)
{
	g_sets.singleplay = checked;
	CheckBoxSetValue(WINDOWAPP, ID_CHK_SINGLEPLAY, checked);
	MenuSetChecked(MENUMASETS, ID_MENU_SINGLEPLAY, checked);
	if(g_sets.output == kOUTSOUND)
		l_soundsingleplay = checked;
}


void DoInMonitor(int checked)
{
	g_sets.inputmonitor = checked;
	if(g_w[WINDOWADVANCED])
		CheckBoxSetValue(WINDOWADVANCED, ID_CHK_INPUTMONITOR, checked);
}


void DoEqualizer(int checked)
{
	if(checked)
		g_sets.eqflags |= MAEQACTIVE;
	else
		g_sets.eqflags &= ~MAEQACTIVE;

	MenuSetChecked(MENUMASETS, ID_MENU_EQUALIZER, checked);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_EQUALIZER, checked);
	if(g_w[WINDOWEQ])
		CheckBoxSetValue(WINDOWEQ, ID_CHK_EQUALIZERB, checked);
		
	EqDoParamChange();

} // DoEqualizer


void DoEqExtra(int checked)
{
	if(checked)
		g_sets.eqflags |= MAEQEXTRA;
	else
		g_sets.eqflags &= ~MAEQEXTRA;

	// CheckBoxSetValue(WINDOWAPP, ID_CHK_EQEXTRA, checked);
	MenuSetChecked(MENUMASETS, ID_MENU_EQEXTRA, checked);
	if(g_w[WINDOWEQ])
		CheckBoxSetValue(WINDOWEQ, ID_CHK_EQEXTRAB, checked);
	eqcfg.extra_filtering = checked;

	EqDoParamChange();

} // DoEqExtra


void DoEffects(int checked)
{
	if(checked)
	{
		g_sets.effects |= EFFECTSON;
		DoWindow(WINDOWEFFECTS, 1);
		if(g_sets.effects & (EFFECTSTEMPO | EFFECTSPITCH | EFFECTSSPEED))
			g_effectschange = 1;
	}
	else
	{
		g_sets.effects &= ~EFFECTSON;
		if(g_w[WINDOWEFFECTS])
			CheckBoxSetValue(WINDOWEFFECTS, ID_CHK_EFFECTSB, 0);
			
		CheckBoxSetValue(WINDOWAPP, ID_CHK_EFFECTS,  0);
		MenuSetChecked(MENUMASETS, ID_MENU_EFFECTS, 0);
	}
	
} // DoEffects


void DoMono(int checked)
{
	if(checked)
	{
		g_sets.monodiff |= MONOCHECKBOXL;
		g_sets.monodiff |= MONOCHECKBOXR;
	}
	else
	{
		g_sets.monodiff &= ~MONOCHECKBOXL;
		g_sets.monodiff &= ~MONOCHECKBOXR;
	}
	
	CheckBoxSetValue(WINDOWAPP, ID_CHK_MONOLEFT, checked);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_MONORIGHT, checked);
	UpdateMonoDiff();
	SetGUIMonoStereo();

} // DoMono


void DoInvertRight(int checked)
{
	g_sets.invertright = checked;
	CheckBoxSetValue(WINDOWAPP, ID_CHK_INVERTRIGHT, checked);
}


void DoEq31Bands(int checked)
{
	if(checked)
	{
		g_sets.eqflags |= MAEQ31BANDS;
		maEqualizerReset(EQBANDS31, 1, (g_sets.eqflags & MAEQEXTRA) ? 1 : 0);	
	}
	else
	{
		g_sets.eqflags &= ~MAEQ31BANDS;
		maEqualizerReset(EQBANDS10, 1, (g_sets.eqflags & MAEQEXTRA) ? 1 : 0);	
	}
		
	// CheckBoxSetValue(WINDOWAPP, ID_CHK_EQ31BANDS, checked);
	MenuSetChecked(MENUMASETS, ID_MENU_EQ31BANDS, checked);
	if(g_w[WINDOWEQ])
		CheckBoxSetValue(WINDOWEQ, ID_CHK_EQ31BANDSB, checked);
	
	EqDoParamChange();

} // DoEq31Bands


void DoViewCatalog(int silent)
{
	fschar *p;
	fschar name[MAX_PATH];
	fschar catalogpath[MAX_PATH];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];
	
	g_catalogpath[0] = fsT('\0');
	
	if(g_playlistpath[0] == fsT('\0'))
	{
		maLog(L"DoViewCatalog no playlist");
		return;
	}		

	if(FilenameFromPathname(g_playlistpath, name))
	{
		maLog(L"DoViewCatalog FilenameFromPathname failed");
		fsstrncpy(name, g_playlistpath, MAX_PATH - 1);
		name[MAX_PATH - 1] = fsT('\0');
		maLog(fs2ma(mastr2, name));
		return;
	}
	if((p = fsstrstr(name, g_extstr[kMAMAF])) != 0)
		*p = fsT('\0');

	fssprintf(catalogpath, MAX_PATH, fsS fsC fsS fsT(".txt"), g_madir, FILESEP, name);
	if(!maFileExists(catalogpath))
	{
		if(!silent)
		{
			masprintf(mastr, MAXLOGLINELEN,
				L"No catalog file%ls%ls'%ls'%ls%lsMake one with the Make Catalog menu item.",
							g_maeol, g_maeol, fs2ma(mastr2, catalogpath), g_maeol, g_maeol);
			ShowErr(mastr);
		}
		return;
	}
	
	fsstrcpy(g_catalogpath, catalogpath);
	DoWindow(WINDOWCATALOG, 1);
	
} // DoViewCatalog


#if defined(MAKEYBOARD)
 void DoEditKeyBindings()
 {
	machar mastr[MAXLOGLINELEN];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
#endif

	if(!maFileExists(g_keybindingspath)) // in case the key bindings file was deleted while this app was open
	{
		SetFactoryKeyBindings();
		SaveKeyBindings(0);
		if(!maFileExists(g_keybindingspath))
		{
			masprintf(mastr, MAXLOGLINELEN, L"DoEditKeyBindings No key bindings file '%ls'",
																		fs2ma(mastr2, g_keybindingspath));
			maLog(mastr);
			return;
		}
	}

	DoWindow(WINDOWKEYBINDINGS, 1);

 } // DoEditKeyBindings


 void DoBlankKeyBindings()
 {
	machar mastr[MAXLOGLINELEN];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
#endif

	if(maFileExists(g_keybindingspath))
	{
		masprintf(mastr, MAXLOGLINELEN, L"The current key bindings will be overwritten"
			L"%ls%ls'%ls'", g_maeol, g_maeol, fs2ma(mastr2, g_keybindingspath));
		if(maMessageBox(mastr, L"CONTINUE ?", maYES_NO | maICON_QUESTION) != maYES)
			return;
	}

	if(g_w[WINDOWKEYBINDINGS])
		DoWindow(WINDOWKEYBINDINGS, 0);
	SaveKeyBindings(1);
	LoadKeyBindings(0);
	
	DoWindow(WINDOWKEYBINDINGS, 1);

  } // DoBlankKeyBindings


 void DoFactoryKeyBindings()
 {
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
#endif
	machar mastr[MAXLOGLINELEN];

	if(maFileExists(g_keybindingspath))
	{
		masprintf(mastr, MAXLOGLINELEN, L"The current key bindings will be overwritten"
			L"%ls%ls'%ls'", g_maeol, g_maeol, fs2ma(mastr2, g_keybindingspath));
		if(maMessageBox(mastr, L"CONTINUE ?", maYES_NO | maICON_QUESTION) != maYES)
			return;
	}

	if(g_keybindingspath[0] == fsT('\0'))
	{
		maLog(L"DoFactoryKeyBindings no keybindings file name");
		return;	
	}
			
	if(maFileExists(g_keybindingspath))
		maunlink(g_keybindingspath);

	LoadKeyBindings(0);

	if(g_w[WINDOWKEYBINDINGS])
		DoWindow(WINDOWKEYBINDINGS, 0);

	DoWindow(WINDOWKEYBINDINGS, 1);

 } // DoFactoryKeyBindings
#endif


 void DoRescanAudioDevices()
{
	maRescanAudioDevices(g_audiohostin, g_audiohostout);
	SetGUIPopupButtonsAudioOut();
	SetGUIPopupButtonsAudioIn();

} // DoRescanAudioDevices

  

void DoAltSize(int checked, int kbcommand)
{
	g_sets.winconf = checked;

	CheckBoxSetValue(WINDOWAPP, ID_CHK_ALTSIZE, checked);
	MenuSetChecked(MENUSHOW, ID_MENU_ALTSIZE, g_sets.winconf ? 1 : 0);

	if(!kbcommand && maModKeyDown())
	{
		// WindowsCopyPositions();
		int windowid;
		int winconf = g_sets.winconf ? 0 : 1;
	
		for(windowid = WINDOWFIRST; windowid <= WINDOWLAST; windowid++)
		{
			g_sets.mawin[windowid].xpos[g_sets.winconf] = g_sets.mawin[windowid].xpos[winconf];
			g_sets.mawin[windowid].ypos[g_sets.winconf] = g_sets.mawin[windowid].ypos[winconf];
		}

	}

	WindowsMoveAndSetSize();

} // DoAltSize


void DoSavedSize()
{
	int winconf;
	int windowid;
	FILE *fp;
	long s;
	MASETS settings;
	fschar *settingspath;
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
#endif
	machar mastr[MAXLOGLINELEN];

	if(g_settingspath[0] != fsT('\0'))
		settingspath = g_settingspath;
	else
	{
		settingspath = g_settingsdefpath;
		if(!maFileExists(settingspath))
		{
			DoFactorySize(0);
			return;
		}
	}
	
	if((fp = mafopen(settingspath, fsT("rb"))) == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"DoSavedSize mafopen failed on '%ls'", fs2ma(mastr2, settingspath));
		maLog(mastr);
		return;
	}

	s = sizeof(MASETS);
	fseek(fp, -s, SEEK_END);		
	if(fread(&settings, 1, sizeof(MASETS), fp) != sizeof(MASETS))
	{
		fclose(fp);		
		masprintf(mastr, MAXLOGLINELEN, L"DoSavedSize fread failed on '%ls'", fs2ma(mastr2, settingspath));
		maLog(mastr);
		return;
	}
	
	fclose(fp);		

	g_sets.winconf = settings.winconf;
	
	for(winconf = 0; winconf < NUMWINCONF; winconf++)
		for(windowid = WINDOWFIRST; windowid <= WINDOWLAST; windowid++)
		{
			g_sets.mawin[windowid].xpos[winconf] = settings.mawin[windowid].xpos[winconf];
			g_sets.mawin[windowid].ypos[winconf] = settings.mawin[windowid].ypos[winconf];
			g_sets.mawin[windowid].xsize[winconf] = settings.mawin[windowid].xsize[winconf];
			g_sets.mawin[windowid].ysize[winconf] = settings.mawin[windowid].ysize[winconf];
		}

	MenuSetChecked(MENUSHOW, ID_MENU_ALTSIZE, g_sets.winconf ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_ALTSIZE, g_sets.winconf ? 1 : 0);
		
	WindowsMoveAndSetSize();
							
} // DoSavedSize


void DoDefaultSize()
{
	int winconf;
	int windowid;
	MASETS settings;

	LoadDefaultSettings(&settings);
	
	g_sets.winconf = settings.winconf;
	
	for(winconf = 0; winconf < NUMWINCONF; winconf++)
		for(windowid = WINDOWFIRST; windowid <= WINDOWLAST; windowid++)
		{
			g_sets.mawin[windowid].xpos[winconf] = settings.mawin[windowid].xpos[winconf];
			g_sets.mawin[windowid].ypos[winconf] = settings.mawin[windowid].ypos[winconf];
			g_sets.mawin[windowid].xsize[winconf] = settings.mawin[windowid].xsize[winconf];
			g_sets.mawin[windowid].ysize[winconf] = settings.mawin[windowid].ysize[winconf];
		}

	MenuSetChecked(MENUSHOW, ID_MENU_ALTSIZE, g_sets.winconf ? 1 : 0);
	CheckBoxSetValue(WINDOWAPP, ID_CHK_ALTSIZE, g_sets.winconf ? 1 : 0);
		
	WindowsMoveAndSetSize();
							
} // DoDefaultSize


void DoFactorySize()
{
	int winconf;

	for(winconf = 0; winconf < NUMWINCONF; winconf++)
		WindowSetsFactoryValues(winconf);

	WindowsMoveAndSetSize();
							
} // DoFactorySize


void DoListWindow(int checked) 
{
	DoWindow(WINDOWLIST, checked);
	
} // DoListWindow


void DoPitch(int checked)
{
	if(checked)
		g_sets.effects |= EFFECTSPITCH;
	else
		g_sets.effects &= ~EFFECTSPITCH;
		
	if(g_sets.effects & EFFECTSON)
		g_effectschange = 1;

	if(g_w[WINDOWEFFECTS])
		CheckBoxSetValue(WINDOWEFFECTS, ID_CHK_PITCH, checked);

} // DoPitch


void DoTempo(int checked)
{
	if(checked)
		g_sets.effects |= EFFECTSTEMPO;
	else
		g_sets.effects &= ~EFFECTSTEMPO;
		
	if(g_sets.effects & EFFECTSON)
		g_effectschange = 1;

	if(g_w[WINDOWEFFECTS])
		CheckBoxSetValue(WINDOWEFFECTS, ID_CHK_TEMPO, checked);

} // DoTempo


void DoSpeed(int checked)
{
	if(checked)
		g_sets.effects |= EFFECTSSPEED;
	else
		g_sets.effects &= ~EFFECTSSPEED;
		
	if(g_w[WINDOWEFFECTS])
		CheckBoxSetValue(WINDOWEFFECTS, ID_CHK_SPEED, checked);

	if(g_sets.effects & EFFECTSON)
		g_effectschange = 1;
}


void DoSBSMS(int checked)
{
	if(checked)
		g_sets.effects |= EFFECTSSBSMS;
	else
		g_sets.effects &= ~EFFECTSSBSMS;
	
	if(g_w[WINDOWEFFECTS])
		CheckBoxSetValue(WINDOWEFFECTS, ID_CHK_SBSMS, checked);
	
} // DoSBSMS


#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
void DoCdParanoia(int checked)
{
	g_setscdrom.cdparanoia = checked;		
	if(g_w[WINDOWAUDIOCD])
		CheckBoxSetValue(WINDOWAUDIOCD, ID_CHK_CDPARANOIA, checked);
}
#endif


void UpdateWindowFileout()
{
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif

	switch(g_sets.output)
	{
		case kOUTSOUND:
		return;
 #ifdef ENCODEMP3
		case kOUTMP3:
			PopupButtonSetValue(WINDOWFILEOUT, ID_POPUP_QUALITY, g_sets.fileout[g_k[g_sets.output]].quality);
			PopupButtonSetValue(WINDOWFILEOUT, ID_POPUP_BITRATE, g_sets.fileout[g_k[g_sets.output]].bitrate);
			PopupButtonSetValue(WINDOWFILEOUT, ID_POPUP_MODE, g_sets.fileout[g_k[g_sets.output]].mode);
			PopupButtonSetValue(WINDOWFILEOUT, ID_POPUP_SAMPLERATE, g_sets.fileout[g_k[g_sets.output]].samplerate);
		break;
 #endif
 #ifdef ENCODEOGG
		case kOUTOGG:
			//PopupButtonSetValue(WINDOWFILEOUT, ID_POPUP_QUALITY, g_sets.fileout[g_k[g_sets.output]].quality); // not used
			PopupButtonSetValue(WINDOWFILEOUT, ID_POPUP_BITRATE, g_sets.fileout[g_k[g_sets.output]].bitrate);
			PopupButtonSetValue(WINDOWFILEOUT, ID_POPUP_MODE, g_sets.fileout[g_k[g_sets.output]].mode);
		break;
 #endif
 #ifdef ENCODEAAC
		case kOUTAAC:
			PopupButtonSetValue(WINDOWFILEOUT, ID_POPUP_QUALITY, g_sets.fileout[g_k[g_sets.output]].quality);
			CheckBoxSetValue(WINDOWFILEOUT, ID_CHK_ADTS, g_sets.fileout[g_k[g_sets.output]].mode & kACCADTS ? 1 : 0);
		break;
 #endif
 #ifdef ENCODEMP4
		case kOUTMP4:
			PopupButtonSetValue(WINDOWFILEOUT, ID_POPUP_QUALITY, g_sets.fileout[g_k[g_sets.output]].quality);
			CheckBoxSetValue(WINDOWFILEOUT, ID_CHK_M4A, g_sets.fileout[g_k[g_sets.output]].mode & kACCM4A ? 1 : 0);
		break;
 #endif
		default:
		break;
	}
	
	StaticTextSetText(WINDOWFILEOUT, ID_TEXT_FILEOUTDIR, fs2ma(mastr2, g_sets.fileout[g_k[g_sets.output]].outdir));
	
} // UpdateWindowFileout


void DoWindow(int windowid, int show)
{
	int i;
	int wason = 1;
#if defined(MAGUICARBON)
		int xpos;
		int ypos;
#endif	
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
#endif
	machar mastr[MAXLOGLINELEN];
	
	if(windowid < WINDOWFIRST || windowid > WINDOWLAST)
	{
		masprintf(mastr, MAXLOGLINELEN, L"DoWindow WindowMake invalid windowid=%d", windowid);
		maLog(mastr);
		return;
	}
	
	if(show)
	{
		if(!g_w[windowid]) // make it
		{
			wason = 0;
#if defined(MAGUICARBON)
			// on MacOS X the window manager seems to put the window where it likes,
			// so we save the current position settings, then make the window, and then
			// reposition the window to the saved positions
			xpos = g_sets.mawin[windowid].xpos[g_sets.winconf];
			ypos = g_sets.mawin[windowid].ypos[g_sets.winconf];
#endif	
			switch(windowid)
			{
				/*
				case WINDOWSPLASH:
					SplashWindowMake();
				break;
				*/
				case WINDOWAPP:
					AppWindowMake();
#if defined(USE_KEYBOARD) && defined(MAGUIWX)
					CheckBoxSetFocus(WINDOWAPP, ID_CHK_EQWINDOW); // arbitrary initial sink for char events
#endif
				break;
				case WINDOWEQ:
					EqualizerWindowMake();
				break;
				case WINDOWEQ2:
					Equalizer2WindowMake();
				break;
				case WINDOWLIST:
					ListWindowMake();
				break;
				case WINDOWEFFECTS:
					EffectsWindowMake();
				break;
				case WINDOWADVANCED:
					AdvancedWindowMake();
				break;
				case WINDOWAUDIOCD:
#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
					AudioCDWindowMake();
#endif
				break;
				case WINDOWFILEOUT:
					FileOutWindowMake();
				break;
				case WINDOWPROMPTNUM:
					PromptNumWindowMake();
				break;
#if !defined(NOTEXTVIEWER) || (defined(NOTEXTVIEWER) && defined(MAGUINONE))
				case WINDOWREADME:
					ViewerWindowMake(WINDOWREADME, L"MoreAmp README");
				break;
				case WINDOWCATALOG:
					ViewerWindowMake(WINDOWCATALOG, L"MoreAmp CATALOG");
 #if !defined(NOTEXTVIEWER)
					l_playindex = 0;
 #endif
 				break;
				case WINDOWPLAYLIST:
					ViewerWindowMake(WINDOWPLAYLIST, L"MoreAmp PLAYLIST");
				break;
				case WINDOWLOG:
					ViewerWindowMake(WINDOWLOG, L"MoreAmp ERROR LOG");
				break;
 #if defined(MAKEYBOARD)
				case WINDOWKEYBINDINGS:
					ViewerWindowMake(WINDOWKEYBINDINGS, L"MoreAmp KEYBINDINGS");
				break;
 #endif	
#endif	
				case WINDOWABOUT:
					AboutWindowMake();
				break;
				default:
				break;
			}
#if defined(MAGUICARBON)
			if(g_w[windowid])
			{
				g_sets.mawin[windowid].xpos[g_sets.winconf] = xpos;
				g_sets.mawin[windowid].ypos[g_sets.winconf] = ypos;
				WindowMoveAndSetSize(windowid,
					g_sets.mawin[windowid].xpos[g_sets.winconf], g_sets.mawin[windowid].ypos[g_sets.winconf],
					g_sets.mawin[windowid].xsize[g_sets.winconf], g_sets.mawin[windowid].ysize[g_sets.winconf]);
			}
#endif
		}
		
		if(g_w[windowid] || wason) // update it
		{
			g_sets.mawin[windowid].ison = 1;
			switch(windowid)
			{
				case WINDOWEQ:
					WindowMoveAndSetSize(WINDOWEQ, g_sets.mawin[WINDOWEQ].xpos[g_sets.winconf], g_sets.mawin[WINDOWEQ].ypos[g_sets.winconf],
										g_sets.mawin[WINDOWEQ].xsize[g_sets.winconf], g_sets.mawin[WINDOWEQ].ysize[g_sets.winconf]);
					MenuSetChecked(MENUSHOW, ID_MENU_SHOWEQ, 1);
					CheckBoxSetValue(WINDOWAPP, ID_CHK_EQWINDOW, 1);
					SetSliders(EQSLIDERS);
					EditTextSetText(WINDOWEQ, ID_EDITTEXT_EQ, fs2ma(mastr2, g_sets.eqpresets[g_sets.eqpreset].eqname));
					StaticTextSetValue(WINDOWEQ, ID_TEXT_EQOFFSETVAL, g_sets.eqpresets[g_sets.eqpreset].eqoffset);
					CheckBoxSetValue(WINDOWEQ, ID_CHK_EQZOOM, (g_sets.eqflags & MAEQZOOM) ? 1 : 0);
					CheckBoxSetValue(WINDOWEQ, ID_CHK_EQLINK, (g_sets.eqflags & MAEQLINK) ? 1 : 0);
					CheckBoxSetValue(WINDOWEQ, ID_CHK_SEPARATELR, (g_sets.eqflags & MASEPARATELR) ? 1 : 0);
					CheckBoxSetValue(WINDOWEQ, ID_CHK_EQUALIZERB, g_sets.eqflags & MAEQACTIVE ? 1 : 0);
					CheckBoxSetValue(WINDOWEQ, ID_CHK_EQ31BANDSB, g_sets.eqflags & MAEQ31BANDS ? 1 : 0);
					CheckBoxSetValue(WINDOWEQ, ID_CHK_EQEXTRAB, g_sets.eqflags & MAEQEXTRA ? 1 : 0);
					for(i = 0; i < EQNUMPRESETS; i++)
					{
						if(fsstrlen(g_sets.eqpresets[i].eqname) == 0 || g_sets.eqpresets[i].eqname[0] == fsT(' '))
							fssprintf(g_sets.eqpresets[i].eqname, EQNAMELEN, fsT("Preset%d"), i + 1);
						PopupButtonSetText(WINDOWEQ, ID_POPUP_EQPRESET, i, fs2ma(mastr2, g_sets.eqpresets[i].eqname));
					}
					PopupButtonSetValue(WINDOWEQ, ID_POPUP_EQPRESET, g_sets.eqpreset);
#if defined(USE_KEYBOARD) && defined(MAGUIWX)
					CheckBoxSetFocus(WINDOWEQ, ID_CHK_EQZOOM); // arbitrary initial sink for char events
#endif
				break;
				case WINDOWEQ2:
					WindowMoveAndSetSize(WINDOWEQ2, g_sets.mawin[WINDOWEQ2].xpos[g_sets.winconf], g_sets.mawin[WINDOWEQ2].ypos[g_sets.winconf],
										g_sets.mawin[WINDOWEQ2].xsize[g_sets.winconf], g_sets.mawin[WINDOWEQ2].ysize[g_sets.winconf]);
					SetSliders(EQ2SLIDERS);
#if defined(USE_KEYBOARD) && defined(MAGUIWX)
					// CheckBoxSetFocus(WINDOWEQ2, ID_CHK_EQ2???); // arbitrary initial sink for char events
#endif
				break;
				case WINDOWLIST:
					MenuSetChecked(MENUSHOW, ID_MENU_SHOWLIST, 1);
					CheckBoxSetValue(WINDOWAPP, ID_CHK_LISTWINDOW, 1);
					CheckBoxSetValue(WINDOWLIST, ID_CHK_SHOWINDEX, (g_sets.listboxmode & LBOXSHOWINDEX) ? 1 : 0);
					CheckBoxSetValue(WINDOWLIST, ID_CHK_SHOWDIR, (g_sets.listboxmode & LBOXSHOWDIR) ? 1 : 0);
					CheckBoxSetValue(WINDOWLIST, ID_CHK_SHOWPATH, (g_sets.listboxmode & LBOXSHOWPATH) ? 1 : 0);
					CheckBoxSetValue(WINDOWLIST, ID_CHK_ADDMODE, g_sets.addmode ? 1 : 0);
					UpdateListWindowButtons();
					UpdateListWindowList(0);
#if defined(USE_KEYBOARD) && defined(MAGUIWX)
					CheckBoxSetFocus(WINDOWLIST, ID_CHK_SHOWINDEX); // arbitrary initial sink for char events
#endif
				break;
				case WINDOWEFFECTS:
					MenuSetChecked(MENUSHOW, ID_MENU_SHOWEFFECTS, 1);
					MenuSetChecked(MENUMASETS, ID_MENU_EFFECTS, (g_sets.effects & EFFECTSON) ? 1 : 0);
					CheckBoxSetValue(WINDOWAPP, ID_CHK_EFFECTS, (g_sets.effects & EFFECTSON) ? 1 : 0);
					CheckBoxSetValue(WINDOWEFFECTS, ID_CHK_PITCH, (g_sets.effects & EFFECTSPITCH) ? 1 : 0);
					CheckBoxSetValue(WINDOWEFFECTS, ID_CHK_TEMPO, (g_sets.effects & EFFECTSTEMPO) ? 1 : 0);
					CheckBoxSetValue(WINDOWEFFECTS, ID_CHK_SPEED, (g_sets.effects & EFFECTSSPEED) ? 1 : 0);
					CheckBoxSetValue(WINDOWEFFECTS, ID_CHK_EFFECTSB, (g_sets.effects & EFFECTSON) ? 1 : 0); // "Effects"
					CheckBoxSetValue(WINDOWEFFECTS, ID_CHK_SBSMS, (g_sets.effects & EFFECTSSBSMS) ? 1 : 0); // "High quality"
					SetSliders(EFFECTSSLIDERS);
#if defined(MAKEYBOARD)
					UpdateEffectsKbLabels();
#endif
#if defined(USE_KEYBOARD) && defined(MAGUIWX)
					CheckBoxSetFocus(WINDOWEFFECTS, ID_CHK_EFFECTSB); // arbitrary initial sink for char events
#endif
				break;
				case WINDOWADVANCED:
					CheckBoxSetValue(WINDOWADVANCED, ID_CHK_CLOSEREOPEN, g_sets.fileout[kOUTSOUND].mode ? 1 : 0);
					CheckBoxSetValue(WINDOWADVANCED, ID_CHK_INPUTMONITOR, g_sets.inputmonitor ? 1 : 0);			
					CheckBoxSetValue(WINDOWADVANCED, ID_CHK_LOGNOTPLAYABLE, (g_sets.logflags & kMALOGNOTPLAYABLE) ? 1 : 0);			
					CheckBoxSetValue(WINDOWADVANCED, ID_CHK_USEM3U, (g_sets.usem3u) ? 1 : 0);
					CheckBoxSetValue(WINDOWADVANCED, ID_CHK_EXITAFTER, (g_sets.exitafter) ? 1 : 0);
					break;
				case WINDOWAUDIOCD:
#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)

					CheckBoxSetValue(WINDOWAUDIOCD, ID_CHK_CDPARANOIA, (g_setscdrom.cdparanoia) ? 1 : 0);			
					PopupButtonSetValue(WINDOWAUDIOCD, ID_POPUP_AUDIOCD, 1);
			
					for(i = 1; i <= CDAMAXNUMDRIVES; i++)
						PopupButtonAppend(WINDOWAUDIOCD, ID_POPUP_AUDIOCD, fs2ma(mastr2, g_setscdrom.cdaname[i]));
					PopupButtonSetSelection(WINDOWAUDIOCD, ID_POPUP_AUDIOCD, g_setscdrom.item);
#endif
				break;
				case WINDOWFILEOUT:
					MenuSetChecked(MENUSHOW, ID_MENU_SHOWFILEOUT, 1);
					UpdateWindowFileout();
				break;
				/* nothing to do here
				case WINDOWPROMPTNUM:
				break;	
#if !defined(NOTEXTVIEWER)
				case WINDOWREADME:
				break;
				case WINDOWCATALOG:
				break;
				case WINDOWPLAYLIST:
				break;
				case WINDOWLOG:
				break;
 #if defined(MAKEYBOARD)
				case WINDOWKEYBINDINGS:
				break;
 #endif	
#endif	
#if !defined(MAGUINONE)
				case WINDOWABOUT:
				break;
#endif	
				*/
				default:
				break;
			}
			
			if(WindowIsShown(WINDOWAPP))
				WindowShow(windowid); // brings to front if already shown
		}
#if !defined(MAGUINONE) // g_w[windowid] is always NULL
		else
		{
			masprintf(mastr, MAXLOGLINELEN, L"DoWindow WindowMake failed windowid=%d wason=%d", windowid, wason);
			maLog(mastr);
		}
#endif
	}
	else // destroy it
	{
		if(g_w[windowid])
		{
			switch(windowid)
			{
				case WINDOWEQ:
					if(!(g_state.stop & STOP_APPQUIT))
					{
						MenuSetChecked(MENUSHOW, ID_MENU_SHOWEQ, 0);
						CheckBoxSetValue(WINDOWAPP, ID_CHK_EQWINDOW, 0);
					}
				break;
				case WINDOWLIST:
#if defined(MAGUICARBON) || defined(MAGUIGTK2)
					g_linenum = 0;
#endif
					if(!(g_state.stop & STOP_APPQUIT))
					{
						MenuSetChecked(MENUSHOW, ID_MENU_SHOWLIST, 0);
						CheckBoxSetValue(WINDOWAPP, ID_CHK_LISTWINDOW, 0);
					}
				break;
				case WINDOWEFFECTS:
					if(!(g_state.stop & STOP_APPQUIT))
						MenuSetChecked(MENUSHOW, ID_MENU_SHOWEFFECTS, 0);
				break;
#if !defined(NOTEXTVIEWER) && !defined(MAGUINONE)
 #if defined(MAKEYBOARD)
				case WINDOWCATALOG:
					l_keycatalog = 0;	
 #endif
				case WINDOWREADME:
				case WINDOWPLAYLIST:
				case WINDOWLOG:
 #if defined(MAKEYBOARD)
				case WINDOWKEYBINDINGS:
 #endif	
					ViewerOnWindowClosing(windowid);
				break;
#endif	
				case WINDOWFILEOUT: 
					if(!(g_state.stop & STOP_APPQUIT))
						MenuSetChecked(MENUSHOW, ID_MENU_SHOWFILEOUT, 0);
				break;
#if defined(MAKEYBOARD)
				case WINDOWPROMPTNUM:
 					l_keyindex = 0;
                    g_maOnKeySkip = 0;
				break;	
#endif
				default:
				break;
			}
			
			// close it
			WindowClose(windowid);
			g_w[windowid] = NULL;
		}
		g_sets.mawin[windowid].ison = 0;
	}
	
} // DoWindow


void ToggleWindow(int windowid)
{
	DoWindow(windowid, g_sets.mawin[windowid].ison ? 0 : 1);
}


mauint64 DumpSongInfo(fschar *songfpath, FILE *dumpfp, int nomem, machar **p2, machar **p3, size_t *len3)
{
	MASTATE state;
#if defined(__MAWIN__) && !defined(MANOCDA)
	int cdaactive = 0;
	int drive;
	int track;
	machar tstr[STRLEN32];
#endif
	machar lstr[STRLEN512];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];

	memset((machar *)&state, 0, sizeof(MASTATE));
	state.nomem = nomem;
	fsstrcpy(state.inpath, songfpath);
	state.dumpfp = dumpfp;
	
	if(FileNameHasExtensionPlayable(songfpath))
	{
		if(nomem)
		{
			if((state.fp = mafopen(songfpath, fsT("rb"))) == NULL)
			{
				masprintf(mastr, MAXLOGLINELEN, L"DumpSongInfo %d mafopen '%ls'", errno, fs2ma(mastr2, songfpath));
				maLog(mastr);
				return 0;
			}
		}
		else
		{
			if((state.fp = memfopen(songfpath)) == NULL)
			{
				masprintf(mastr, MAXLOGLINELEN, L"DumpSongInfo %d memfopen", errno);
				maLog(mastr);
				return 0;
			}
		}
		
		state.size = maGetFileSize(state.fp, nomem);

		if(state.size >= 1000000)
		{
			int millions = (int)(state.size / 1000000);
			int thousands = (int)((state.size - millions * 1000000) / 1000);
			masprintf(lstr, STRLEN512, L"Size : %d,%03d,%03d bytes", millions, thousands, ((int)state.size - millions * 1000000) - thousands * 1000);
		}
		else
		if(state.size >= 1000)
		{
			int thousands = (int)(state.size / 1000);
			masprintf(lstr, STRLEN512, L"Size : %d,%03d bytes", thousands, (int)state.size - thousands * 1000);
		}
		else
			masprintf(lstr, STRLEN512, L"Size : %d bytes", (int)state.size);
		DumpString(state.dumpfp, lstr, p2, p3, len3);

#ifndef NODECODEMP3
		if(FileNameHasExtension(songfpath, g_extstr[kMAMP3]) ||
			FileNameHasExtension(songfpath, g_extstr[kMAMP2]) ||
			FileNameHasExtension(songfpath, g_extstr[kMAMP1]))
		{
			if(DumpInfoMp3(&state, p2, p3, len3))
				DumpString(state.dumpfp, L"DumpInfoMp3 failed", p2, p3, len3);
		}
		else
#endif

#ifndef NODECODEOGG
		if(FileNameHasExtension(songfpath, g_extstr[kMAOGG]))
		{
			if(DumpInfoOgg(&state, p2, p3, len3))
				DumpString(state.dumpfp, L"DumpInfoOgg failed", p2, p3, len3);
		}
		else 
#endif

#ifndef NODECODEFLAC
		if(FileNameHasExtension(songfpath, g_extstr[kMAFLAC]) ||
			FileNameHasExtension(songfpath, g_extstr[kMAFLA]))
		{
			if(DumpInfoFlac(&state, p2, p3, len3))
				DumpString(state.dumpfp, L"DumpInfoFlac failed", p2, p3, len3);
		}
		else 
#endif
			
#ifndef NODECODEWMA
		if(FileNameHasExtension(songfpath, g_extstr[kMAWMA]))
		{
			if(DumpInfoWma(&state, p2, p3, len3))
				DumpString(state.dumpfp, L"DumpInfoWma failed", p2, p3, len3);
		}
		else
#endif

#ifndef NODECODEAAC
		if(FileNameHasExtension(songfpath, g_extstr[kMAAAC]) ||
			FileNameHasExtension(songfpath, g_extstr[kMAM2A]) ||
			FileNameHasExtension(songfpath, g_extstr[kMAMP2]))
		{
			if(DumpInfoAac(&state, p2, p3, len3))
				DumpString(state.dumpfp, L"DumpInfoAac failed", p2, p3, len3);
		}
		else
#endif

#ifndef NODECODEMP4
		if(FileNameHasExtension(songfpath, g_extstr[kMAMP4]) ||
			FileNameHasExtension(songfpath, g_extstr[kMAM4A]))
		{
			if(DumpInfoMp4(&state, p2, p3, len3))
				DumpString(state.dumpfp, L"DumpInfoMp4 failed", p2, p3, len3);
		}
		else
#endif

#ifndef NODECODEAIF
		if(FileNameHasExtension(songfpath, g_extstr[kMAAIF]) ||
			FileNameHasExtension(songfpath, g_extstr[kMAAIFF]))
		{
			if(DumpInfoAif(&state, p2, p3, len3))
				DumpString(state.dumpfp, L"DumpInfoAif failed", p2, p3, len3);
		}
		else
#endif

#ifndef NODECODEWAV
		if(FileNameHasExtension(songfpath, g_extstr[kMAWAV]))
		{
			if(DumpInfoWav(&state, p2, p3, len3))
				DumpString(state.dumpfp, L"DumpInfoWav failed", p2, p3, len3);
		}
		else
#endif
			; // valuable semicolon here

		if(nomem)
			fclose(state.fp);
		else
			memfclose(state.fp);
	}
	
#if defined(__MAWIN__) && !defined(MANOCDA)
	else if((drive = IsAudioCd(songfpath)) != 0)
	{
		if(g_cdaactive == 0)
		{
			if(CdaOpenFile(songfpath))
			{
				masprintf(mastr, MAXLOGLINELEN, L"DumpSongInfo CdaOpenFile failed on '%ls'", fs2ma(mastr2, g_state.inpath));
				maLog(mastr);
				CdaCloseFile(g_cdaactive);
				return 0;
			}
			cdaactive = 1;
		}

		if(drive == g_cdaactive)
		{
			if(FileNameHasExtension(songfpath, g_extstr[kMACDA]))
				track = fsatoi(&songfpath[fsstrlen(songfpath) - 6 ]);
			else
				track = fsatoi(&songfpath[fsstrlen(songfpath) - 2 ]);
			masprintf(lstr, STRLEN512, L"Duration : %ls", SecondsToStrHMS(tstr, GetTrackLength(track)));
			DumpString(state.dumpfp, lstr, p2, p3, len3);
		}

		if(cdaactive)
			CdaCloseFile(g_cdaactive);
	}
#endif
	
	return state.size;

} // DumpSongInfo


int ListPlayerInfo(int newcd)
{
#ifndef MANOCDA
	int i;
#endif
	fschar name[STRLEN256];
	machar mastr2[MAXLOGLINELEN];	
	machar mastr[MAXLOGLINELEN];
	
#if !defined(MAGUINONE)
	if(!g_w[WINDOWLIST])
		return 1;
#endif
		
	CheckBoxHide(WINDOWLIST, ID_CHK_SHOWINDEX);
	CheckBoxHide(WINDOWLIST, ID_CHK_SHOWDIR);
	CheckBoxHide(WINDOWLIST, ID_CHK_SHOWPATH);
	CheckBoxHide(WINDOWLIST, ID_CHK_ADDMODE);
	
	name[0] = fsT('\0');
	
	ListWindowSetTitle(L"Player Info");
	ListWindowClear();
	ListWindowSetHeader(L"Info");

	ListWindowAddString(L"");

	if(g_playlistpath[0] == fsT('\0'))
		masprintf(mastr, MAXLOGLINELEN, L"No playlist");
	else if(FilenameFromPathname(g_playlistpath, name))
		masprintf(mastr, MAXLOGLINELEN, L"Missing playlist '%ls'", fs2ma(mastr2, name));
	else if(!l_psongrec || g_songsnum <= 0)
		masprintf(mastr, MAXLOGLINELEN, L"Bad or empty '%ls' : %d songs", fs2ma(mastr2, name), g_songsnum);
	else
		masprintf(mastr, MAXLOGLINELEN, L"'%ls'", fs2ma(mastr2, g_playlistpath));
	ListWindowAddString(mastr);
	ListWindowAddString(L"");

	// masprintf(mastr, MAXLOGLINELEN, L"Settings version %ls", g_sets.settingsversion);
	// ListWindowAddString(mastr);
	// ListWindowAddString(L"");

#if !defined(NODECODEAUDIOIN)
	if(g_state.infiletype == kMAMAA || PlaylistIsAudioIn())
	{
		if(g_sets.hostinenable)
			masprintf(mastr, MAXLOGLINELEN, L"Audio input host API: '%ls'", os2ma(mastr2, g_sets.audiohostinname));
		else
			masprintf(mastr, MAXLOGLINELEN, L"Audio input host API disabled, using same as output"); 
		ListWindowAddString(mastr);
		masprintf(mastr, MAXLOGLINELEN, L"Audio input device: %d '%ls'",
			g_paudiohost[g_audiohostin]->paudiodevice[g_audiodevicein]->devindex, os2ma(mastr2, g_sets.audiodevinname));
		ListWindowAddString(mastr);
		masprintf(mastr, MAXLOGLINELEN, L"Channels=%d  Samplerate=%d", g_state.inchannels, g_state.insamplerate);
		ListWindowAddString(mastr);
		ListWindowAddString(L"");
		masprintf(mastr, MAXLOGLINELEN, L"Audio output host API: '%ls'", os2ma(mastr2, g_sets.audiohostoutname));
		ListWindowAddString(mastr);
		masprintf(mastr, MAXLOGLINELEN, L"Audio output device : %d '%ls'",
			g_paudiohost[g_audiohostout]->paudiodevice[g_audiodeviceout]->devindex, os2ma(mastr2, g_sets.audiodevoutname));
		ListWindowAddString(mastr);
		masprintf(mastr, MAXLOGLINELEN, L"Channels=%d  Samplerate=%d", g_state.outchannels, g_state.outsamplerate);
	}
	else
#endif
	{
 #if !defined(MANOCDA)
 		if(newcd)
			masprintf(mastr, MAXLOGLINELEN, L"%d songs, 0 plays", g_cdadrive[g_cdaactive].tracklast);
		else
#endif
			masprintf(mastr, MAXLOGLINELEN, L"%d songs, %d plays", g_songsnum, l_songsnumplayed);
		ListWindowAddString(mastr);
		ListWindowAddString(L"");
		
		if(g_sets.buffertype == BUFUSEMEM)
			masprintf(mastr, MAXLOGLINELEN, L"Current buffer = %.2f MB", (float)(g_loadedmemsize / (double)MEGABYTE));
		else if(g_sets.buffertype == BUFUSEVDISK)
			masprintf(mastr, MAXLOGLINELEN, L"Virtual Disk : '%ls'", fs2ma(mastr2, g_sets.vdiskname));
		else
			masprintf(mastr, MAXLOGLINELEN, L"Direct from disk (no buffer)");
		if(g_state.running)
		{
			ListWindowAddString(L"");
			masprintf(mastr, MAXLOGLINELEN, L"Audio output host API: '%ls'", os2ma(mastr2, g_sets.audiohostoutname));
			ListWindowAddString(mastr);
			masprintf(mastr, MAXLOGLINELEN, L"Audio output device : %d '%ls'",
				g_paudiohost[g_audiohostout]->paudiodevice[g_audiodeviceout]->devindex, os2ma(mastr2, g_sets.audiodevoutname));
			ListWindowAddString(mastr);
			masprintf(mastr, MAXLOGLINELEN, L"Channels=%d  Samplerate=%d", g_state.outchannels, g_state.outsamplerate);
		}
	}
	ListWindowAddString(mastr);
	ListWindowAddString(L"");
	
#ifndef MANOCDA
	for(i = 1; i <= CDAMAXNUMDRIVES; i++)
	{
		if(g_cdadrive[i].exists)
		{
#ifdef __MAWIN__
			masprintf(mastr, MAXLOGLINELEN, 
				L"CDRom '%lc:' : ready=%d active=%d tracks=%d",
				g_cdadrive[i].fsvolrefnum, g_cdadrive[i].isready, g_cdadrive[i].isactive, g_cdadrive[i].tracklast);
#endif

#if defined(__MAOSX__) || defined(MACDAOSX)
			masprintf(mastr, MAXLOGLINELEN,
				L"CDRom '%ls:' : ready=%d tracks=%d",
				fs2ma(mastr2, g_cdadrive[i].name), g_cdadrive[i].isready, g_cdadrive[i].tracklast);
#endif

#ifdef __MAUNIX__
			if(g_cdadrive[i].isready)
				masprintf(mastr, MAXLOGLINELEN, L"CDRom '%ls' : ready=%d tracks=%d",
					fs2ma(mastr2, g_cdadrive[i].name), g_cdadrive[i].isready, g_cdadrive[i].tracklast);
			else
				masprintf(mastr, MAXLOGLINELEN, L"CDRom '%ls' : ready=%d",
					fs2ma(mastr2, g_cdadrive[i].name), g_cdadrive[i].isready);
#endif

			ListWindowAddString(mastr);
		}	
	}
#endif
	
	ListWindowAddString(L"");
	
	if(!maModKeyDown())
		return 0;
		
	masprintf(mastr, MAXLOGLINELEN, L"masetlocale(LC_CTYPE)=" maenvS, masetlocale(LC_CTYPE, NULL));	
	ListWindowAddString(mastr);
	masprintf(mastr, MAXLOGLINELEN, L"magetcharset()=" maenvS, magetcharset());	
	ListWindowAddString(mastr);
	masprintf(mastr, MAXLOGLINELEN, L"MACHARSET=" maenvS, magetenv(maenvT("MACHARSET")));	
	ListWindowAddString(mastr);
#if defined(MAGUICARBON)
	masprintf(mastr, MAXLOGLINELEN, L"MACHARSETKB=" maenvS, magetenv(maenvT("MACHARSETKB")));	
	ListWindowAddString(mastr);
#endif
	masprintf(mastr, MAXLOGLINELEN, L"g_charset=" maenvS, g_charset);	
	ListWindowAddString(mastr);
#if defined(MAGUICARBON)
	masprintf(mastr, MAXLOGLINELEN, L"g_charsetkb=" maenvS, g_charsetkb);	
	ListWindowAddString(mastr);	
#endif
	ListWindowShowRow(-1); // scroll to top, no hilite
	ListSetColumnWidth();
		
	return 0;
	
} // ListPlayerInfo


int ListPlaylist()
{
	int i, l;
	size_t ret0;
	machar *p0;
	machar *p4;
	machar *p5;
	fschar path[MAX_PATH];
	fschar name[MAX_PATH];
	machar header[32];
	fschar songpath[MAX_PATH];
	machar masongpath[MAX_PATH];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];
	
#if !defined(MAGUINONE)
	if(!g_w[WINDOWLIST])
		return 1;
#endif
		
	CheckBoxShow(WINDOWLIST, ID_CHK_SHOWINDEX);
	CheckBoxShow(WINDOWLIST, ID_CHK_SHOWDIR);
	CheckBoxShow(WINDOWLIST, ID_CHK_SHOWPATH);
	if(!fsstrcmp(g_playlistpath, g_playlistdrop))
		CheckBoxShow(WINDOWLIST, ID_CHK_ADDMODE);
	else
		CheckBoxHide(WINDOWLIST, ID_CHK_ADDMODE);
	
	header[0] = 0;
	if(g_sets.listboxmode & LBOXSHOWINDEX)
		mastrcpy(header, L"Index  ");
	if(g_sets.listboxmode & LBOXSHOWDIR)
		mastrcat(header, L"Dir / File name");
	else if(g_sets.listboxmode & LBOXSHOWPATH)
		mastrcat(header, L"File path");
	else
		mastrcat(header, L"File name");
	ListWindowSetTitle(L"Playlist");
	ListWindowClear();
	ListWindowSetHeader(header);

	songpath[0] = fsT('\0');
	if(g_playlistpath[0] == fsT('\0'))
	{
		ListWindowAddString(L"(No playlist)");
		return 1;
	}
	else if(FilenameFromPathname(g_playlistpath, name))
	{
		masprintf(mastr, MAXLOGLINELEN, L"(Missing playlist '%ls'", os2fsma(mastr2, g_playlistpath));
		ListWindowAddString(mastr);
		return 1;
	}
	else if(!PlaylistIsCda() && (!l_psongrec || g_songsnum <= 0))
	{
		masprintf(mastr, MAXLOGLINELEN, L"Bad or empty playlist '%ls' : %d songs", os2fsma(mastr2, g_playlistpath), g_songsnum);
		ListWindowAddString(mastr);
		return 1;
	}

	ret0 = mafileread(&p0, g_playlistpath);
	if(ret0 <= 0 || p0 == NULL)
	{			
		if(p0 != NULL)
			MAFREE(p0);
		maLog(L"ListPlaylist mafileread failed");
		return 2;
	}

	i = 0;
	
	p4 = p0;
	for (; ;)
	{
		songpath[0] = fsT('\0');
		
		if((p5 = memgets(masongpath, MAX_PATH, p4)) == NULL)
			break;
		ma2fsfn(songpath, masongpath);
		p4 = p5;
			
		// skip lines with '#' or files that don't exist
		if(songpath[0] == fsT('#') || !SongFileExists(songpath))
			continue;

		path[0] = fsT('\0');
		
 #if !defined(MAGUINONE)
 		if(g_sets.listboxmode & LBOXSHOWINDEX)
 #endif
 			fssprintf(path, MAX_PATH, fsT("%05d "), 1 + i);

		if(g_sets.listboxmode & LBOXSHOWDIR)
		{
			l = (int)fsstrlen(songpath) - 1;						// .../dir/name.mp3
			while(l >= 0 && songpath[l--] != FILESEP){};	// .../dir
			while(l >= 0 && songpath[l--] != FILESEP){};	// ...
			if(l < 0)										// no leading filesep e.g. "Audio CD/Track01.cda"
				l = -2;
			fsstrcat(path, &songpath[l + 2]);				// dir/name.mp3
		}
		else if(g_sets.listboxmode & LBOXSHOWPATH)
			fsstrcat(path, songpath);
		else
		{
			FilenameFromPathname(songpath, name);
			fsstrcat(path, name);
		}

#if defined(__MAOSX__) || defined(__MACUNIX__)
		StrUTF8toUTF8MAC(path);
#endif				
		ListWindowAddString(fs2ma(mastr2, path));
				
		i++;
						
	} // while
				
	MAFREE(p0);
	
	if(g_state.running)
		ListWindowShowRowPlaying();
	else
		ListWindowShowRow(-1); // scroll to top, no hilite
		
	ListSetColumnWidth();

	return 0;
	
} // ListPlaylist


void ListWindowShowRowPlaying()
{
	l_noshowrow = 0;
	if(l_listisfrozen)
		ListThaw();
	if(g_sets.mawin[WINDOWLIST].ison && g_sets.listboxmode & LBOXSONGSLOADED)
		ListWindowShowRow(g_loadedcurrent);
	else if(g_sets.mawin[WINDOWLIST].ison && g_sets.listboxmode & LBOXPLAYLIST)
		ListWindowShowRow(l_songsindex[g_loadedcurrent]);
	if(l_listisfrozen)
		ListFreeze();
	
} // ListWindowShowRowPlaying


int ListWindowGetValue()
{
	if(!g_w[WINDOWLIST])
		return -1;
		
	return ListGetRowSelected(WINDOWLIST, ID_LIST_CTRL);
		
} // ListWindowGetValue


void ListWindowAddString(machar *str)
{
#if !defined(MAGUINONE)
	if(!g_w[WINDOWLIST])
		return;
#endif
		
	ListAppend(WINDOWLIST, ID_LIST_CTRL, str);
	
} // ListWindowAddString


void ListWindowShowRow(int inrow) // 0-based
{
	int row;
	
	if(!g_w[WINDOWLIST])
		return;
	
	if((row = inrow) < 0) // no hilite if row < 0
	{
		row = -inrow;
#if defined(MAGUICARBON)
		if(inrow == -1)
			row = 0;
#endif
	}
	
	// scroll to make row visible
	ListEnsureVisible(WINDOWLIST, ID_LIST_CTRL, row);
	
	// hilite highlight row
	if(inrow >= 0)
		ListSelect(WINDOWLIST, ID_LIST_CTRL, row);
	
} // ListWindowShowRow


void ListWindowClear()
{
	if(!g_w[WINDOWLIST])
		return;
		
	ListClear(WINDOWLIST, ID_LIST_CTRL);
	
} // ListWindowClear


void ListWindowSetHeader(machar *header)
{
#if !defined(MAGUINONE)
	if(!g_w[WINDOWLIST])
		return;
#endif

	ListSetHeader(WINDOWLIST, ID_LIST_CTRL, 0, header);
	
} // ListWindowSetHeader


void ListWindowGetRow(machar *str, int row)
{
	ListGetText(WINDOWLIST, ID_LIST_CTRL, row, str);

} // ListWindowGetRow


int ListSongInfo()
{
	int nosongselected = 0;
	int clicked = 0;
	int nomem;
	size_t len3 = 0;
	fschar fpath[MAX_PATH];
	fschar name[MAX_PATH];
	machar *p2 = NULL;
	machar *p3 = NULL;
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
	fschar fsstr[MAX_PATH];
#endif

#if !defined(MAGUINONE)
	if(!g_w[WINDOWLIST])
		return 1;
#endif
		
	CheckBoxHide(WINDOWLIST, ID_CHK_SHOWINDEX);
	CheckBoxHide(WINDOWLIST, ID_CHK_SHOWDIR);
	CheckBoxHide(WINDOWLIST, ID_CHK_SHOWPATH);
	CheckBoxHide(WINDOWLIST, ID_CHK_ADDMODE);

	if(l_strclicked[0] != L'\0') // show song info for song clicked in playlist or loaded list
	{
		clicked = 1;
		fsstrcpy(fpath, ma2fs(fsstr, l_strclicked));
		l_strclicked[0] = L'\0';
	}
	else if(g_loadednum <= 0)
		nosongselected = 1;
	else
	{
		if(g_loadedpath[g_loadedcurrent][0] == fsT('\0'))
			nosongselected = 1;
		else
			fsstrcpy(fpath, g_loadedpath[g_loadedcurrent]);
	}

	g_sets.listboxmode &= ~(LBOXPLAYERINFO | LBOXPLAYLIST | LBOXSONGSLOADED | LBOXSONGINFO);
	g_sets.listboxmode |= LBOXSONGINFO;

	ListWindowSetTitle(MASONGINFOTITLE);
	ListWindowClear();
	ListWindowSetHeader(MASONGINFO);
	if(nosongselected)
	{
		ListWindowAddString(MANOSONGSELECTED);
		return 0;
	}
	
	FilenameFromPathname(fpath, name);
 	ListWindowAddString(fs2ma(mastr2, name));

	if(g_loadednum <= 0 && !clicked)
	{
		ListWindowShowRow(-1); // scroll to top, no hilite
		return 0;
	}
		
#ifndef MANOCDA
	if(IsAudioCd(fpath))
	{
		nomem = 1;	
	 	ListWindowAddString(L"(on CD)");
	}
	else
#endif
	if(SongIsInMemory(fpath))
	{
		nomem = 0;	
	 	ListWindowAddString(L"(loaded)");
	}
	else
	{
		nomem = 1;	
	 	ListWindowAddString(L"(on disk)");
	}

	DumpSongInfo(fpath, NULL, nomem, &p2, &p3, &len3);

	if(p2)
	{
		MAFREE(p2);
		p2 = NULL;
	}

	if(p3)
	{
		MAFREE(p3);
		p3 = NULL;
	}
		
	ListWindowShowRow(-1); // scroll to top, no hilite
	
	ListSetColumnWidth();

	return 0;

} // ListSongInfo


int GetRecentPlaylist() // called from OnInit when app is starting
{
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];

	g_playlistpath[0] = fsT('\0');

	// get last playlist used
	if(maFileExists(l_recentlist))
		GetRecentPlaylistName();
	if(g_playlistpath[0] != fsT('\0') && !maFileExists(g_playlistpath))
	{
		if(g_playlistpath[0] != fsT('\0'))
			masprintf(mastr, MAXLOGLINELEN, L"Playlist not found :"
				L"%ls%ls'%ls'%ls%ls"
				L"Open a playlist with the 'Open Playlist' menu"
				L"%lsor%ls"
				L"Make one with the 'Make Playlist' menu"
				L"%lsor%ls"
				L"Drop files on the MoreAmp icon or window.",
				g_maeol, g_maeol, fs2ma(mastr2, g_playlistpath), g_maeol,
				g_maeol, g_maeol, g_maeol, g_maeol, g_maeol); 
		else
			masprintf(mastr, MAXLOGLINELEN, L"No playlist.%ls%ls"
				L"Make one with the 'Make Playlist' menu%ls"
				L"or%ls"
				L"Drop files on the MoreAmp icon or window.",
				g_maeol, g_maeol, g_maeol, g_maeol);

		g_playlistpath[0] = fsT('\0');
		ShowErr(mastr);
		return 1;
	}
	
	if(OpenPlaylist(0))
		return 1;

	return 0;
	
} // GetRecentPlaylist


int MakeOutputFileName()
{
	fschar *p;
	fschar songname[ MAX_PATH];
#if defined(ENCODEMP4) || defined(ENCODEFLAC)
 #if defined(__MAWIN__)
	int len;
 #endif
  	int needansi;
    struct stat sb;
	char tempdir[MAX_PATH];
	char tempfile[MAX_PATH];
#endif
	machar mastr2[MAXLOGLINELEN];	
	machar mastr[MAXLOGLINELEN];
	
	// output file
	if(!maDirExists(g_sets.fileout[g_k[g_sets.output]].outdir))
	{
		masprintf(mastr, MAXLOGLINELEN, L"File output folder does not exist:%ls%ls'%ls'",
							g_maeol, g_maeol, fs2ma(mastr2, g_sets.fileout[g_k[g_sets.output]].outdir));
		ShowErr(mastr);
		return 1;
	}
	
	fsstrcpy(songname, g_loadedname[g_loadedcurrent]);
	
#if defined(__MAWIN__)
	while((p = fsstrchr(songname, VOLSEP)) != 0)
		*p = fsT('_'); // mainly for cda files, e.g. "D:\Track01.cda"
#endif

	while((p = fsstrchr(songname, FILESEP)) != 0)
		*p = fsT('_');
	
	fssprintf(l_outputfile, MAX_PATH, fsS fsC fsS fsS, g_sets.fileout[g_k[g_sets.output]].outdir,
													FILESEP, songname, g_extstrout[g_sets.output]);

	if(fsstrlen(l_outputfile) > fsstrlen(g_sets.fileout[g_k[g_sets.output]].outdir) + 1 + NAMELENMAX)
		fsstrcpy(&l_outputfile[fsstrlen(g_sets.fileout[g_k[g_sets.output]].outdir) + NAMELENMAX - 4],
																			g_extstrout[g_sets.output]);

	// set extension for .m4a
#ifdef ENCODEMP4
	if(g_sets.output == kOUTMP4 && g_sets.fileout[kOUTMP4].mode & kACCM4A)
		fsstrcpy(&l_outputfile[fsstrlen(l_outputfile) - 4], EXTM4A); // - fsstrlen(EXTMP4)
#endif
	
	if(maFileExists(l_outputfile))
	{
		masprintf(mastr, MAXLOGLINELEN, L"Output file already exists and will be overwritten%ls%ls'%ls'",
															g_maeol, g_maeol, fs2ma(mastr2, l_outputfile));
		if(maMessageBox(mastr, L"CONTINUE ?", maYES_NO | maICON_QUESTION) != maYES)
			return 1;
		
		if(maunlink(l_outputfile))
		{
			masprintf(mastr, MAXLOGLINELEN, L"MakeOutputFileName failed to delete '%ls'",
																		fs2ma(mastr2, l_outputfile));
			maLog(mastr);
			return 1;
		}		
	}

#if defined(ENCODEMP4) || defined(ENCODEFLAC)
	needansi = 0;
 #if defined(ENCODEMP4)
	if(g_sets.output == kOUTMP4)
		needansi = 1;
 #endif
 #if defined(ENCODEFLAC)
	if(g_sets.output == kOUTFLAC)
		needansi = 1;
 #endif
	if(needansi)
	{
		// make an ansi tempfile name. This is for libs that need ansi filepaths, like flac and faac (mp4).
		// don't use g_madir because that can be utf8. Use a system temp folder.
 #if defined(__MAWIN__)
		if((len = GetTempPathA(MAX_PATH, tempdir)) <= 0)
		{
			masprintf(mastr, MAXLOGLINELEN, L"MakeOutputFileName GetTempPathA failed err=%d", GetLastError());
			maLog(mastr);
			return 1;
		}
		tempdir[len - 1] = '\0'; // remove trailing '\'
 #else
		strcpy(tempdir, "/tmp");
 #endif
		if(!(stat(tempdir, &sb) == 0 && (sb.st_mode & S_IFDIR)))
		{
			masprintf(mastr, MAXLOGLINELEN, L"MakeOutputFileName temp dir '%ls' does not exist",
																					os2ma(mastr2, tempdir));
			maLog(mastr);
			return 1;
		}
		sprintf(tempfile, "%s%cmatemp%d", tempdir, FILESEP, g_thisinstance);
		if((stat(tempfile, &sb) == 0 && (sb.st_mode & S_IFREG)))
		{
			if(osunlink(tempfile)) 
			{
				masprintf(mastr, MAXLOGLINELEN, L"MakeOutputFileName unlink failed on '%ls'",
																					os2ma(mastr2, tempfile));
				maLog(mastr);
			}
		}
		strcpy((char *)g_tempfile, tempfile);
	}
	else
#endif
		fsstrcpy(g_tempfile, l_outputfile);

	return 0;
	
} // MakeOutputFileName


int maCloseOutputFile()
{
#if defined(ENCODEMP4) || defined(ENCODEFLAC)
	FILE *fpfrom;
	FILE *fpto;
	int err;
	size_t bytesread;
	machar buf[STRLEN512];
#endif
#if defined(ENCODEMP4) || defined(ENCODEFLAC)
	int needansi;
	machar mastr2[MAXLOGLINELEN];
#endif
	machar mastr[MAXLOGLINELEN];
	
	if(!g_tempfilefp)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maCloseOutputFile file not open '%ls'", g_tempfile);
		maLog(mastr);
		return 1;
	}
	
	if(g_tempfilefp == DUMMYOUTFILEP) // dummy close when outputfile is handled by lib
		g_tempfilefp = NULL;
	else if(fclose(g_tempfilefp))
	{
		masprintf(mastr, MAXLOGLINELEN, L"maCloseOutputFile fclose errno=%d on '%ls'", errno, g_tempfile);
		maLog(mastr);
		g_tempfilefp = NULL;
		return 1;
	}

	g_tempfilefp = NULL;

#if defined(ENCODEMP4) || defined(ENCODEFLAC)
	needansi = 0;
 #if defined(ENCODEMP4)
	if(g_sets.output == kOUTMP4)
		needansi = 1;
 #endif
 #if defined(ENCODEFLAC)
	if(g_sets.output == kOUTFLAC)
		needansi = 1;
 #endif
	if(needansi)
	{
		// copy tempfile to output file
		err = 0;	
		if((fpfrom = mafopenansi((char *)g_tempfile, "rb")) == NULL)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maCloseOutputFile mafopenansi errno=%d '%ls'", errno,
																			os2ma(mastr2, (char *)g_tempfile));
			maLog(mastr);
			return 1; // not dskFulErr
		}
		if((fpto = mafopen(l_outputfile, fsT("wb"))) == NULL)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maCloseOutputFile fopen errno=%d '%ls'", errno,
																				fs2ma(mastr2, l_outputfile));
			maLog(mastr);
			if(fclose(fpfrom) != 0)
			{
				masprintf(mastr, MAXLOGLINELEN, L"maCloseOutputFile fclose errno=%d '%ls'", errno,
																			os2ma(mastr2, (char *)g_tempfile));
				maLog(mastr);
			}
			return 1;
		}
		errno = 0;
		while((bytesread = fread(buf, 1, sizeof(buf), fpfrom)) > 0)
		{
			if(fwrite(buf, 1, bytesread, fpto) != bytesread)
			{
				masprintf(mastr, MAXLOGLINELEN, L"maCloseOutputFile fwrite errno=%d '%ls'", errno,
																			os2ma(mastr2, (char *)g_tempfile));
				maLog(mastr);
				err = 1;
				break;
			}
		}
		if(bytesread <= 0)
		{
			if(!feof(fpfrom))
			{
				masprintf(mastr, MAXLOGLINELEN, L"maCloseOutputFile fread errno=%d '%ls'", errno,
																			os2ma(mastr2, (char *)g_tempfile));
				maLog(mastr);
				err = 2;
			}
		}	
		if(fclose(fpfrom) != 0)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maCloseOutputFile fclose errno=%d '%ls'", errno,
																			os2ma(mastr2, (char *)g_tempfile));
			maLog(mastr);
			err = 3;
		}	
		if(fclose(fpto) != 0)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maCloseOutputFile fclose errno=%d '%ls'", errno,
																				fs2ma(mastr2, l_outputfile));
			maLog(mastr);
			err = 4;
		}
 #if defined(__MAOSX__) || defined(__MAUNIX__)
		if(machmod(l_outputfile, ACCESSPERMS) == -1) // 0777
		{
			masprintf(mastr, MAXLOGLINELEN, L"maCloseOutputFile machmod errno=%d '%ls'", errno,
																				fs2ma(mastr2, l_outputfile));
			maLog(mastr);
			err = 5;
		}
 #endif
		// delete tempfile
		if(osunlink((char *)g_tempfile))
		{
			masprintf(mastr, MAXLOGLINELEN, L"maCloseOutputFile unlink errno=%d '%ls'", errno,
																			os2ma(mastr2, (char *)g_tempfile));
			maLog(mastr);
			err = 6;
		}

		return err;
	} // needansi	
#endif
	
	return 0;
	
} // maCloseOutputFile


int maOpenOutputFile()
{
	machar mastr[MAXLOGLINELEN];
	
#ifdef ENCODEMP4
	if(g_sets.output == kOUTMP4) // this is opened by the lib
		g_tempfilefp = NULL; // this is used by the lib
	else
#endif

#ifdef ENCODEFLAC
	if(g_sets.output == kOUTFLAC) // this is opened by the lib
		g_tempfilefp = DUMMYOUTFILEP; // this is not used by the lib, dummy value for close, etc
	else
#endif
	{
		if((g_tempfilefp = mafopen(g_tempfile, fsT("w+b"))) == NULL)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maOpenOutputFile mafopen '%ls'", g_tempfile);
			maLog(mastr);
			return 1;
		}

#if defined(__MAOSX__) || defined(__MAUNIX__)
		machmod(g_tempfile, ACCESSPERMS); // 0777
#endif
	}
	
	// format specific 'open'
	switch(g_sets.output)
	{
#ifdef ENCODEMP3
		case kOUTMP3:
			if(!OpenWriteMp3())
				return 0;
		break;
#endif
		
#ifdef ENCODEOGG
		case kOUTOGG:
			if(!OpenWriteOgg())
				return 0;
		break;
#endif
		
#ifdef ENCODEAIF
		case kOUTAIF:
			if(!OpenWriteAif())
				return 0;
		break;
#endif
		
#ifdef ENCODEWAV
		case kOUTWAV:
			if(!OpenWriteWav(0))
				return 0;
		break;
#endif

#ifdef ENCODERAW
		case kOUTRAW:
			if(!OpenWriteWav(1))
				return 0;
		break;
#endif
		
#ifdef ENCODEAAC
		case kOUTAAC:
			if(!OpenWriteAac())
				return 0;
		break;
#endif
		
#ifdef ENCODEMP4
		case kOUTMP4:
			if(!OpenWriteMp4())
				return 0;
		break;
#endif
		
#ifdef ENCODEWMA
		case kOUTWMA:
			if(!OpenWriteWma())
				return 0;
		break;
#endif
		
#ifdef ENCODEFLAC
		case kOUTFLAC:
			if(!OpenWriteFlac())
				return 0;
		break;
#endif
		
		default:
		break;
	}
	
	// error:
	if(g_tempfile != NULL)
		maCloseOutputFile();
		
	return 1;
	
} // maOpenOutputFile


int maOutputToFile(void* pcmblock, unsigned long frames, unsigned long len)
{
	int err = 0;
	
	if(!g_tempfilefp)
	{
		maLog(L"maOutputToFile error null g_tempfilefp");
		return 1;
	}
	
	switch(g_sets.output)
	{
#ifdef ENCODEMP3
		case kOUTMP3:
			err = WriteMp3(pcmblock, frames);
		break;
#endif
		
#ifdef ENCODEOGG
		case kOUTOGG:
			err = WriteOgg(pcmblock, len);
		break;
#endif
		
#ifdef ENCODEAIF
		case kOUTAIF:
			err = WriteAif(pcmblock, frames, len);
		break;
#endif
		
#ifdef ENCODEWAV
		case kOUTWAV:
			err = WriteWav(pcmblock, len);
		break;
#endif

#ifdef ENCODERAW
		case kOUTRAW:
			err = WriteWav(pcmblock, len);
		break;
#endif
		
#ifdef ENCODEAAC
		case kOUTAAC:
			err = WriteAac(pcmblock, len);
		break;
#endif
		
#ifdef ENCODEMP4
		case kOUTMP4:
			err = WriteMp4(pcmblock, len);
		break;
#endif
		
#ifdef ENCODEWMA
		case kOUTWMA:
			err = WriteWma(pcmblock, frames, len);
		break;
#endif
		
#ifdef ENCODEFLAC
		case kOUTFLAC:
			err = WriteFlac(pcmblock, len);
		break;
#endif
		
		default:
			err = 1;
		break;
	}
	
	return err;
	
} // maOutputToFile


// always stereo in, Hiword is right channel, Loword is left channel
// a frame is MAOUTBYTES wide (two shorts)
void ApplyChannelSelection(char *soundbuf, unsigned long numFrames)
{
#define LEFT *(short *)p
#define RIGHT *((short *)p + 1)
	unsigned long n;
	int *p;
	short s;

	n = numFrames;	
	p = (int *)soundbuf;
	
	switch(l_outputfilter & 0xff)
	{	
		case OUTPUT_MONO:
			if(g_sets.invertright) // invert right channel
			{
				while(n-- > 0)
				{
					LEFT  = (short)(LEFT * l_panleft);
					RIGHT = (short)(RIGHT * l_panright);
					LEFT = (short)(LEFT + RIGHT);
					RIGHT = -LEFT; p++;
				}
			}
			else
			{
				while(n-- > 0)
				{
					LEFT  = (short)(LEFT * l_panleft);
					RIGHT = (short)(RIGHT * l_panright);
					LEFT = (short)(LEFT + RIGHT);
					RIGHT = LEFT; p++;
				}
			}
		break;

		case OUTPUT_LEFT:
			if(g_sets.invertright)
			{
				while(n-- > 0)
				{
					RIGHT = -LEFT; p++;
				}
			}
			else
			{
				while(n-- > 0)
				{
					RIGHT = LEFT; p++;
				}
			}
		break;
		
		case OUTPUT_RIGHT:
			if(g_sets.invertright)
			{
				while(n-- > 0)
				{
					LEFT = RIGHT;
					RIGHT = -RIGHT; p++;
				}
			}
			else
			{
				while(n-- > 0)
				{
					LEFT = RIGHT; p++;
				}
			}
		break;
		
		case OUTPUT_DIFF:
			if(g_sets.invertright)
			{
				while(n-- > 0)
				{
					LEFT  = (short)(LEFT * l_panleft);
					RIGHT = (short)(RIGHT * l_panright);
					LEFT = (short)((LEFT - RIGHT));
					RIGHT = -LEFT; p++;
				}
			}
			else
			{
				while(n-- > 0)
				{
					LEFT  = (short)(LEFT * l_panleft);
					RIGHT = (short)(RIGHT * l_panright);
					LEFT = (short)((LEFT - RIGHT));
					RIGHT = LEFT; p++;
				}
			}
		break;
		
		case OUTPUT_STEREO:
		default:
			if(g_sets.invertright)
				while(n-- > 0)
				{
					RIGHT = -RIGHT; p++;
				}
		break;
	}

	if(l_outputfilter & OUTPUT_STEREO)
	{
		if(l_separation < SEPARATIONMAX)
		{
			n = numFrames;
			p = (int *)soundbuf;
			
			while(n-- > 0)
			{
				s     = (short)(((LEFT)  * l_separationch) + ((RIGHT) * l_separationmix));
				RIGHT = (short)(((RIGHT) * l_separationch) + ((LEFT)  * l_separationmix));
				LEFT = s; p++;
			}
		}
	
		if(l_outputfilter & OUTPUT_REVERSE)
		{
			n = numFrames;
			p = (int *)soundbuf;
			
			while(n-- > 0)
			{
				s = LEFT;
				LEFT = RIGHT;
				RIGHT = s; p++;
			}
		}
	}

} // ApplyChannelSelection


void OnListWindowSingleClick(int row) // single (or first of two) click in listbox or ListSelect()
{
	int lrow;
	int songsnum;
	size_t ret0;
	machar *p0;
	machar *p4;
	machar *p5;
	fschar songsource[MAX_PATH];
	machar masongsource[MAX_PATH];
	machar lstr[LISTLINELEN + 2];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	
	if(g_listselect > 0)
	{
		if(--g_listselect < 0)
			g_listselect = 0;
		return;
	}
		
	switch(g_sets.listboxmode & (LBOXPLAYERINFO | LBOXPLAYLIST | LBOXSONGSLOADED | LBOXSONGINFO))
	{
		case LBOXPLAYLIST:
			
			if(row > 0)
				lrow = row - 1;
			else if((lrow = ListWindowGetValue()) < 0)
				break;
			
			if(g_sets.listboxmode & LBOXSHOWPATH)
			{
				if(g_sets.listboxmode & LBOXSHOWINDEX)
				{
					ListWindowGetRow(lstr, lrow);
					mastrcpy(l_strclicked, &lstr[INDEXLEN]);
				}
				else
					ListWindowGetRow(l_strclicked, lrow);
			}
			else // get path from playlist
			{
				l_strclicked[0] = L'\0';
				// load the playlist file	
				ret0 = mafileread(&p0, g_playlistpath);
				if(ret0 <= 0 || p0 == NULL)
				{			
					if(p0 != NULL)
						MAFREE(p0);
					maLog(L"OnListWindowSingleClick mafileread failed");
					return;
				}

				// get row'th entry
				songsnum = 0;

				p4 = p0;
				while((p5 = memgets(masongsource, MAX_PATH, p4)) != NULL)
				{
					ma2fsfn(songsource, masongsource);
					p4 = p5;
					// skip lines with '#' or files that are not playable or don't exist
					if(songsource[0] != fsT('#'))
					{
						if(SongFileExists(songsource))
							if(songsnum++ == lrow)
							{
#if defined(__MAOSX__) || defined(__MACUNIX__)
		StrUTF8toUTF8MAC(songsource);
#endif				
								 mastrcpy(l_strclicked, fs2ma(mastr2, songsource));
								 break;
							}
					}
				}
				MAFREE(p0);
			}
		break;

		case LBOXSONGSLOADED:
			if(g_loadednum <= 0)
				return;
			if(row > 0)
				lrow = row - 1;
			else if((lrow = ListWindowGetValue()) < 0)
				return;
			mastrcpy(l_strclicked, fs2ma(mastr2, g_loadedpath[lrow]));
		break;
		
		case LBOXPLAYERINFO:
		case LBOXSONGINFO:
		default:
		break;

	} // case

} // OnListWindowSingleClick


void OnListWindowDoubleClick() // doubleclick in listbox
{
	int row;

	l_strclicked[0] = L'\0';
	
	if((row = ListWindowGetValue()) < 0)
		return;
	
	switch(g_sets.listboxmode & (LBOXPLAYERINFO | LBOXPLAYLIST | LBOXSONGSLOADED | LBOXSONGINFO))
	{	
		case LBOXPLAYLIST:			
			g_droplaunch = row + 2; // 0-based row to play + 2
			if(g_state.paused)
			{
				g_state.silence = 1;
				SetPause(0);
			}
			if(g_state.running) // exit song loop
			{
				g_loadedcurrent = g_loadednum = 0;
				g_loadedquit = g_loadedquitB = 1;
			}
			else
				DoTimerPlay(100); // OnListWindowDoubleClick
		break;

		case LBOXSONGSLOADED:	
			if(g_loadednum <= 0)
				break;	
			if(g_state.paused)
			{
				g_state.silence = 1;
				SetPause(0);
			}
			l_loadednew = row + 1;
			if(g_state.running) // exit song loop
				g_loadedquit = g_loadedquitB = 1;
			else
			{
				l_noreload = 1;
				DoTimerPlay(100); // OnListWindowDoubleClick
			}			
		break;

		case LBOXPLAYERINFO:
		case LBOXSONGINFO:
		default:
		break;
		
	} // case

} // OnListWindowDoubleClick


void DoShowIndex(int checked)
{
	if(checked)
		g_sets.listboxmode |= LBOXSHOWINDEX;
	else
		g_sets.listboxmode &= ~LBOXSHOWINDEX;
	OnButtonList();
}


void DoShowDir(int checked)
{
	if(checked)
	{
		g_sets.listboxmode |= LBOXSHOWDIR;
		g_sets.listboxmode &= ~LBOXSHOWPATH;
		CheckBoxSetValue(WINDOWLIST, ID_CHK_SHOWPATH, 0);
	}
	else
		g_sets.listboxmode &= ~LBOXSHOWDIR;
	OnButtonList();
}


void DoShowPath(int checked)
{
	if(checked)
	{
		g_sets.listboxmode |= LBOXSHOWPATH;
		g_sets.listboxmode &= ~LBOXSHOWDIR;
		CheckBoxSetValue(WINDOWLIST, ID_CHK_SHOWDIR, 0);
	}
	else
		g_sets.listboxmode &= ~LBOXSHOWPATH;
	OnButtonList();
}


void DoAddMode(int checked)
{
	if(checked)
		g_sets.addmode = 1;
	else
		g_sets.addmode = 0;
}


void OnPopupOutput(int output)
{
	if(output >= kOUTFIRST && output <= kOUTLAST)
	{
		if(g_sets.mawin[WINDOWFILEOUT].ison && g_sets.output != output) // close previous
			DoWindow(WINDOWFILEOUT, 0);
		g_sets.output = output;
		DoSinglePlay(1);
		DoWindow(WINDOWFILEOUT, 1);
		MenuEnable(MENUSHOW, ID_MENU_SHOWFILEOUT);
	}
	else
	{
		MenuDisable(MENUSHOW, ID_MENU_SHOWFILEOUT);
		g_sets.output = kOUTSOUND;
		DoSinglePlay(l_soundsingleplay ? 1 : 0);
		DoWindow(WINDOWFILEOUT, 0); 
	}
	
	return;

} // OnPopupOutput


void OnPopupBuffer(int buffer)
{
	switch(buffer)
	{
		case 0:
			OnPopupMem();
		break;
		case 1:
			OnPopupVDisk();
		break;
		case 2:
		default:
			OnPopupDirect();
		break;
	}
	
	return;

} // OnPopupBuffer


void OnPopupAudioOutput(int value)
{
	PaDeviceIndex device;

	if((device = maGetAudioOutputIndex(value)) < 0)
	{
		maLog(L"OnPopupAudioOutput failed, bad device index");
		return;
	}

	strcpy(g_sets.audiodevoutname, g_paudiohost[g_audiohostout]->paudiodevice[device]->info->name);
	
	maSetAudioOutputDevice();

} // OnPopupAudioOutput


void OnPopupAudioHostOut(int value)
{	
	if(g_audiohostout == value)
		return; // no change

	maRescanAudioDevices(-1, value);
	SetGUIPopupButtonsAudioOut();

} // OnPopupAudioHostOut
 

void OnPopupAudioInput(int value)
{
	PaDeviceIndex device;
	
	if((device = maGetAudioInputIndex(value)) < 0)
	{
		maLog(L"OnPopupAudioInput failed, bad device index");
		return;
	}

	strcpy(g_sets.audiodevinname, g_paudiohost[g_audiohostin]->paudiodevice[device]->info->name);

	maSetAudioInputDevice();

} // OnPopupAudioInput


void OnPopupAudioHostIn(int value)
{	
	if(g_audiohostin == value)
		return; // no change

	maRescanAudioDevices(value, -1);
	SetGUIPopupButtonsAudioIn();

} // OnPopupAudioHostIn


void OnPopupEqPreset(int preset)
{
	int i;
	long outlen;
	fschar fsstr[EQNAMELEN];
	machar mastr[MAXLOGLINELEN];
	
	if(preset == g_sets.eqpreset)
		return;
	
	if(preset >= EQNUMPRESETS)
	{
		masprintf(mastr, MAXLOGLINELEN, L"OnPopupEqPreset preset %d >= max %d", preset, EQNUMPRESETS);
		maLog(mastr);
		return;	
	}
	
	if(g_w[WINDOWEQ])
	{
		EditTextGetText(WINDOWEQ, ID_EDITTEXT_EQ, mastr, EQNAMELEN, &outlen, 0, -1);
		ma2fsfn(fsstr, mastr);
		if(fsstrlen(fsstr) == 0)
			fssprintf(fsstr, EQNAMELEN, fsT("Preset%d"), g_sets.eqpreset + 1);
		if(fsstrcmp(fsstr, g_sets.eqpresets[g_sets.eqpreset].eqname))
		{
#if defined(MAGUICARBON) || defined(MAGUIGTK2)
			PopupButtonSetText(WINDOWEQ, ID_POPUP_EQPRESET, g_sets.eqpreset, fs2ma(mastr, g_sets.eqpresets[g_sets.eqpreset].eqname));
			if(g_sets.eqpreset == 0) // force a redraw
				i = 1;
			else
				i = 0;
			PopupButtonSetValue(WINDOWEQ, ID_POPUP_EQPRESET, i);
#else
			fsstrcpy(g_sets.eqpresets[g_sets.eqpreset].eqname, ma2fs(fsstr, mastr)); // PopupButtonSetText fails...
			PopupButtonClear(WINDOWEQ, ID_POPUP_EQPRESET);
			for(i = 0; i < EQNUMPRESETS; i++)
				PopupButtonAppend(WINDOWEQ, ID_POPUP_EQPRESET, fs2ma(mastr, g_sets.eqpresets[i].eqname));
#endif
		}
		PopupButtonSetValue(WINDOWEQ, ID_POPUP_EQPRESET, preset);
	}
		
	g_eqzoomval = -1.0;
	g_sets.eqpreset = preset;
	
	if(g_w[WINDOWEQ])
	{
		EditTextSetText(WINDOWEQ, ID_EDITTEXT_EQ, fs2ma(mastr, g_sets.eqpresets[g_sets.eqpreset].eqname));
		StaticTextSetValue(WINDOWEQ, ID_TEXT_EQOFFSETVAL, g_sets.eqpresets[g_sets.eqpreset].eqoffset);
	}
	
	if(g_sets.eqflags & MAEQACTIVE)
		EqDoParamChange();
	
	if(g_w[WINDOWEQ])
		SetSliders(EQSLIDERS);
	
	if(g_w[WINDOWEQ2])
		SetSliders(EQ2SLIDERS);
	
	return;

} // OnPopupEqPreset


void OnPopupBitRate(int bitrate)
{
	g_sets.fileout[g_k[g_sets.output]].bitrate = bitrate;
}


void OnPopupQuality(int quality)
{
	g_sets.fileout[g_k[g_sets.output]].quality = quality;
}


void OnPopupMode(int mode)
{
	g_sets.fileout[g_k[g_sets.output]].mode = mode;
}


void OnPopupSampleRate(int samplerate)
{
	g_sets.fileout[g_k[g_sets.output]].samplerate = samplerate;
}


#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
void OnPopupAudioCD(int preset)
{
	machar mastr[MAXLOGLINELEN];
	
	if(preset >= CDAMAXNUMDRIVES)
	{
		masprintf(mastr, MAXLOGLINELEN, L"OnPopupAudioCD preset %d >= max %d", preset, CDAMAXNUMDRIVES);
		maLog(mastr);
		return;	
	}
	
	// zilch...
	
	return;

} // OnPopupAudioCD
#endif


int maAllocBuffers()
{
#ifndef NODECODEMP3
	// mp3 decoder input buffer
	if(g_mp3inpbuf == NULL)
	{
		if((g_mp3inpbuf = (unsigned char *)MAMALLOC(MP3INPBUFSIZE)) == NULL)
		{
			maLog(L"maAllocBuffers MAMALLOC g_mp3inpbuf failed");
			return 1;
		}	
	}

	// mp3 decoder output buffer
	if(g_mp3decbuf == NULL)
	{
		if((g_mp3decbuf = (unsigned char *)MAMALLOC(MP3DECBUFSIZE)) == NULL)
		{
			maLog(L"maAllocBuffers MAMALLOC g_mp3decbuf failed");
			return 1;
		}
	}
#endif

	// allocate memory for mafifobuf
	if(!g_mafifobuf)
		if((g_mafifobuf = MAMALLOC(MAFIFOBUFSIZE)) == NULL)
		{
			maFreeBuffers();
			maLog(L"maAllocBuffers MAMALLOC g_mafifobuf failed");
			return 1;
		}			
	memset(g_mafifobuf, 0, MAFIFOBUFSIZE);

	// create mafifobuf
	if(!l_pmafifoB)
	{
		if((l_pmafifoB = maFifoCreateB(&l_mafifoB, g_mafifobuf, (unsigned long)MAFIFOBUFSIZE)) == NULL)
		{
			maFreeBuffers();
			maLog(L"maFifoCreateB failed");
			return 1;
		}
	}

	if(!g_ppafifoout)
	{
		if((g_ppafifoout = maFifoCreate(&g_pafifoout, g_pafifooutbuf, (unsigned long)PAFIFOOUTBUFSIZE)) == NULL)
		{
			maFreeBuffers();
			maLog(L"maAllocBuffers maFifoCreate g_pafifoout failed");
			return 1;
		}
	}

#if !defined(NODECODEAUDIOIN)
	if(!g_ppafifoin)
	{
		if((g_ppafifoin = maFifoCreate(&g_pafifoin, g_pafifoinbuf, (unsigned long)PAFIFOINBUFSIZE)) == NULL)
		{
			maFreeBuffers();
			maLog(L"maAllocBuffers maFifoCreate g_pafifoin failed");
			return 1;
		}
	}
#endif


	return 0;
	
} // maAllocBuffers


int maFreeBuffers()
{
	if(g_mafifobuf)
	{
			MAFREE(g_mafifobuf);
			g_mafifobuf = 0;
	}

	if(l_pmafifoB)
	{
		maFifoDeleteB(l_pmafifoB);
		l_pmafifoB = NULL;
	}
	
#if !defined(NODECODEAUDIOIN)
	if(g_ppafifoin)
	{
		maFifoDelete(g_ppafifoin);
		g_ppafifoin = NULL;
	}
#endif

	if(g_ppafifoout)
	{
		maFifoDelete(g_ppafifoout);
		g_ppafifoout = NULL;
	}

	if(l_monobuf)
	{
		MAFREE(l_monobuf);
		l_monobuf = NULL;
	}

#ifndef NODECODEMP3
	if(g_mp3inpbuf)
	{
		MAFREE(g_mp3inpbuf);
		g_mp3inpbuf = NULL;
	}

	if(g_mp3decbuf)
	{
		MAFREE(g_mp3decbuf);
		g_mp3decbuf = NULL;
	}
#endif

	return 0;
	
} // maFreeBuffers


int maEraseBuffers()
{
	if(l_pmafifoB)
		maFifoEraseB(l_pmafifoB);
	
	if(g_ppafifoout)
		maFifoErase(g_ppafifoout);

	if(l_monobuf && l_monobytes)
		memset(l_monobuf, 0, l_monobytes);

#if !defined(NODECODEAUDIOIN)
	if(g_ppafifoin) 
		maFifoErase(g_ppafifoin);
#endif

#ifndef NODECODEMP3
	if(g_mp3inpbuf)
		memset(g_mp3inpbuf, 0, MP3INPBUFSIZE);

	if(g_mp3decbuf)
		memset(g_mp3decbuf, 0, MP3DECBUFSIZE);
#endif

	return 0;
	
} // maEraseBuffers


int maIOStart() // called from PlayThreadXX only
{
	// reset some audio output parameters
	maFifoSetStorageB(l_pmafifoB, g_mafifobuf);

	maEraseBuffers();

	maEqualizerReset((g_sets.eqflags & MAEQ31BANDS) ? EQBANDS31 : EQBANDS10, 1,
									g_sets.eqflags & MAEQEXTRA);
	EqDoParamChange();

	maOutputSetVolume((kVOLMAX * volumevalL())  / VOLUMEMAX, (kVOLMAX * volumevalR()) / VOLUMEMAX);

	// open audio output
	if(g_sets.output == kOUTSOUND)
	{
		if(g_sets.hostinenable)
		{
#if !defined(NODECODEAUDIOIN)
			if(g_state.infiletype == kMAMAA)
				if(maOpenPAStreamIn())
					return -1;
#endif
			if(maOpenPAStreamOut(g_audiohostout, g_audiodeviceout))
				return -1;
		}
 		else
		{
#if !defined(NODECODEAUDIOIN)
			if(g_state.infiletype == kMAMAA)
			{
				if(maOpenPAStreamInOut(g_audiohostout, g_audiodevicein, g_audiodeviceout))	
					return -1;
			}
			else
#endif
				if(maOpenPAStreamOut(g_audiohostout, g_audiodeviceout))
					return -1;
		}
	}
	else // output to file
	{
		if(g_sets.hostinenable)
		{
#if !defined(NODECODEAUDIOIN)
			if(g_state.infiletype == kMAMAA)
			{
 				if(maOpenPAStreamIn())
					return -1;
				if(g_sets.inputmonitor)
					if(maOpenPAStreamOut(g_audiohostout, g_audiodeviceout))
						return -1;
			}
#endif
		}
#if !defined(NODECODEAUDIOIN)
		else // not hostinenable
 		{
 			if(g_state.infiletype == kMAMAA)
			{
				if(g_sets.inputmonitor)
				{
					if(maOpenPAStreamInOut(g_audiohostout, g_audiodevicein, g_audiodeviceout))
						return -1;
 				}
				else
					if(maOpenPAStreamIn())
						return -1;
			}
 		}
#endif
		if(g_tempfilefp)
		{
			maLog(L"maIOStart warning output file was already open");
			g_tempfilefp = NULL;
		}

		if(maOpenOutputFile())
		{
			maLog(L"maIOStart maOpenOutputFile failed");
			return -1;
		}
	}

	maStartPAStreams();

	return 1; // ok
		
} // maIOStart


void maIOEnd() // called from maOutChainFlush
{
	if(g_sets.output == kOUTSOUND || g_state.infiletype == kMAMAA)
	{
		if(!(g_state.stop & STOP_ALL))
			maWaitForFifosEmpty(30000); // 30 secs with big fifo

#if !defined(NODECODEAUDIOIN)
		maClosePAStreamIn();
		maClosePAStreamInOut();
#endif
		maClosePAStreamOut(1, 0); // wait, don't force	
	}

	if(g_sets.output != kOUTSOUND && g_tempfilefp)
	{
		switch(g_sets.output)
		{
#ifdef ENCODEMP3
			case kOUTMP3:
				CloseWriteMp3();
			break;
#endif

#ifdef ENCODEOGG
			case kOUTOGG:
				CloseWriteOgg();
			break;
#endif

#ifdef ENCODEFLAC
			case kOUTFLAC:
				CloseWriteFlac();
			break;
#endif
		
#ifdef ENCODEAAC
			case kOUTAAC:
				CloseWriteAac();
			break;
#endif

#ifdef ENCODEMP4
			case kOUTMP4:
				CloseWriteMp4();
			break;
#endif

#ifdef ENCODEAIF
			case kOUTAIF:
				CloseWriteAif();
			break;
#endif
		
#ifdef ENCODEWAV
			case kOUTWAV:
				CloseWriteWav();
			break;
#endif

#ifdef ENCODERAW
			case kOUTRAW:
				CloseWriteWav();
			break;
#endif

#ifdef ENCODEWMA
			case kOUTWMA:
				CloseWriteWma();
			break;
#endif
			default:
			break;
		}

		maCloseOutputFile();
	}

} // maIOEnd
	

void maWaitForFifosEmpty(int max)
{
	// wait delay ms max or until fifos are empty
	if(max > 0)
	{		
		unsigned long t = maGetLocalTimeMillis() + max;
		while((g_ppafifoout && maFifoGetFilledSpace(g_ppafifoout) > 0) ||
				(l_pmafifoB && maFifoGetFilledSpaceB(l_pmafifoB) > 0))
		{
			maSleep(1);
			if(maGetLocalTimeMillis() > t)
				break;
		}
	}

} // maWaitForFifosEmpty


// maFifo functions:
maFifo *maFifoCreate(maFifo* pfifo, char *storage, unsigned long fifosize)
{
	if(!pfifo || (pfifo->storage = storage) == NULL)
		return NULL;
        
	pfifo->size = fifosize;
	memset(pfifo->storage, 0, pfifo->size);
	pfifo->contains = pfifo->start = 0;

	if(maSemaphoreNew(&pfifo->semaphore))
	{
		pfifo->semaphore = NULL;
		maLog(L"maFifoCreate maSemaphoreNew semaphore failed");
		return NULL;
	}

#if defined(__MAWIN__)
	InitializeCriticalSection(&pfifo->lock);
#else // mac, unix
    if(maMutexNew(&pfifo->lock))
	{
		maLog(L"maFifoCreate maMutexNew failed");
		return NULL;
	}
   
#endif
	return pfifo;

} // maFifoCreate


void maFifoDelete(maFifo* pfifo)
{
	if(!pfifo)
		return;

	pfifo->size = 0;
	pfifo->contains = 0;
	pfifo->start = 0;
	pfifo->storage = NULL;

	if(pfifo->semaphore)
	{
		if(maSemaphoreDelete(pfifo->semaphore))
			maLog(L"maFifoDelete maSemaphoreDelete failed");
		pfifo->semaphore = NULL;
	}

#if defined(__MAWIN__)	
	DeleteCriticalSection(&pfifo->lock);
#else // mac, unix
    if(maMutexDelete(&pfifo->lock))
        maLog(L"maFifoDelete maLockDelete failed");
#endif

} // maFifoDelete


unsigned long maFifoGetFreeSpace(maFifo* pfifo)
{
	unsigned long nbytes;
	
	if(!pfifo)
        return 0;

	MAFIFOLOCKWAIT(pfifo->lock);

	nbytes = pfifo->size - pfifo->contains;

	MAFIFOLOCKRELEASE(pfifo->lock);

	return nbytes;

} // maFifoGetFreeSpace


unsigned long maFifoGetFilledSpace(maFifo* pfifo)
{
	unsigned long nbytes;
	
	if(!pfifo)
        return 0;

	MAFIFOLOCKWAIT(pfifo->lock);

	nbytes = pfifo->contains;

	MAFIFOLOCKRELEASE(pfifo->lock);

	return nbytes;

} // maFifoGetFilledSpace


unsigned long maFifoStore(maFifo* pfifo, char *whereFrom, unsigned long requested)
{
	unsigned long fifoend;
	unsigned long amount;
	unsigned long amount1;
	unsigned long amount2;
	unsigned long amount3;

	if(!pfifo)
        return 0;
        
	MAFIFOLOCKWAIT(pfifo->lock);

	if(!pfifo->storage)
	{
		MAFIFOLOCKRELEASE(pfifo->lock);
		return 0;
	}

	if((amount = requested) > (amount3 = pfifo->size - pfifo->contains)) // = maFifoGetFreeSpace(pfifo))
		amount = amount3; // = maFifoGetFreeSpace(pfifo);
	
	fifoend = (pfifo->start + pfifo->contains) % pfifo->size;
	amount1 = pfifo->size - fifoend;

	if(amount <= amount1)
		memmove(pfifo->storage + fifoend, whereFrom, amount);
	else
	{
		amount2 = amount - amount1;
		memmove(pfifo->storage + fifoend, whereFrom, amount1);
		memmove(pfifo->storage, whereFrom + amount1, amount2);
	}

	pfifo->contains += amount;
	
	MAFIFOLOCKRELEASE(pfifo->lock);
	
	return amount;
	
} // maFifoStore


unsigned long maFifoDraw(maFifo* pfifo, char* whereTo, unsigned long requested)
{
	unsigned long amount;
	unsigned long amount1;
	unsigned long amount2;

	if(!pfifo)
        return 0;
        
	MAFIFOLOCKWAIT(pfifo->lock);

	if(!pfifo->storage)
	{
		MAFIFOLOCKRELEASE(pfifo->lock);
		return 0;
	}

	if((amount = requested) > pfifo->contains)
		amount = pfifo->contains;
	
	amount1 = pfifo->size - pfifo->start;

	if(amount <= amount1)
		memmove(whereTo, pfifo->storage + pfifo->start, amount);
	else
	{
		amount2 = amount - amount1;

		memmove(whereTo, pfifo->storage + pfifo->start, amount1);

		memmove(whereTo+amount1, pfifo->storage, amount2);
	}
	
	pfifo->contains -= amount;
	pfifo->start += amount;
	
	if(pfifo->start >= pfifo->size)
		pfifo->start -= pfifo->size;
	
	MAFIFOLOCKRELEASE(pfifo->lock);

	return amount;
	
} // maFifoDraw


void maFifoErase(maFifo* pfifo)
{
	if(!pfifo)
        return;
        
	MAFIFOLOCKWAIT(pfifo->lock);
	
	pfifo->contains = pfifo->start = 0;

	if(!pfifo->storage)
	{
		MAFIFOLOCKRELEASE(pfifo->lock);
		return;		
	}

	memset(pfifo->storage, 0, pfifo->size);

	MAFIFOLOCKRELEASE(pfifo->lock);

} // maFifoErase


void maFifoSetStorage(maFifo* pfifo, char *storage)
{
	if(!pfifo)
        return;
        
	MAFIFOLOCKWAIT(pfifo->lock);

	pfifo->storage = storage;

	if(!pfifo->storage)
	{
		MAFIFOLOCKRELEASE(pfifo->lock);
		return;
	}

	pfifo->contains = pfifo->start = 0; // maFifoClear(pfifo);

	MAFIFOLOCKRELEASE(pfifo->lock);	

} // maFifoSetStorage


/*
void maFifoClear(maFifo* pfifo)
{
    if(!pfifo)
        return;
        
	MAFIFOLOCKWAIT(pfifo->lock);
		
	pfifo->contains = pfifo->start = 0;

	MAFIFOLOCKRELEASE(pfifo->lock);

} // maFifoClear


char *maFifoGetStorage(maFifo* pfifo)
{
	char *p;

    if(!pfifo)
        return NULL;
        
    MAFIFOLOCKWAIT(pfifo->lock);

	p = pfifo->storage;

	MAFIFOLOCKRELEASE(pfifo->lock);

	return p;

} // maFifoGetStorage
*/


// maFifo B functions, these are called from the same thread so no critical sections or locks
maFifoB *maFifoCreateB(maFifoB* pfifo,char *storage, unsigned long fifosize)
{
	if(!pfifo || (pfifo->storage = storage) == NULL)
		return NULL;
	
	pfifo->size = fifosize;
	pfifo->contains = pfifo->start = 0; // maFifoClear(pfifo);

	if(maSemaphoreNew(&pfifo->semaphore))
	{
		pfifo->semaphore = NULL;
		maLog(L"maFifoCreateB maSemaphoreNew semaphore failed");
		return NULL;
	}

	return pfifo;

} // maFifoCreateB


void maFifoDeleteB(maFifoB* pfifo)
{
	if(!pfifo)
		return;

	pfifo->size = 0;
	pfifo->contains = 0;
	pfifo->start = 0;
	pfifo->storage = NULL;

	if(pfifo->semaphore)
	{
		if(maSemaphoreDelete(pfifo->semaphore))
			maLog(L"maFifoDeleteB maSemaphoreDelete semaphore failed");
		pfifo->semaphore = NULL;
	}

} // maFifoDeleteB


unsigned long maFifoGetFreeSpaceB(maFifoB* pfifo)
{
	unsigned long nbytes;

    if(!pfifo)
        return 0;

	nbytes = pfifo->size - pfifo->contains;

	return nbytes;

} // maFifoGetFreeSpaceB


unsigned long maFifoGetFilledSpaceB(maFifoB* pfifo)
{
	unsigned long nbytes;

    if(!pfifo)
        return 0;
        
	nbytes = pfifo->contains;

	return nbytes;

} // maFifoGetFilledSpaceB


unsigned long maFifoStoreB(maFifoB* pfifo, char *whereFrom, unsigned long requested)
{
	unsigned long fifoend;
	unsigned long amount;
	unsigned long amount1;
	unsigned long amount2;
	unsigned long amount3;

	if(!pfifo || !pfifo->storage)
		return 0;

	if((amount = requested) > (amount3 = pfifo->size - pfifo->contains)) // = maFifoGetFreeSpace(pfifo))
		amount = amount3; // = maFifoGetFreeSpace(pfifo);
	
	fifoend = (pfifo->start + pfifo->contains) % pfifo->size;
	amount1 = pfifo->size - fifoend;

	if(amount <= amount1)
		memmove(pfifo->storage + fifoend, whereFrom, amount);
	else
	{
		amount2 = amount - amount1;
		memmove(pfifo->storage + fifoend, whereFrom, amount1);
		memmove(pfifo->storage, whereFrom + amount1, amount2);
	}

	pfifo->contains += amount;
	
	return amount;
	
} // maFifoStoreB


void maFifoEraseB(maFifoB* pfifo)
{	
	if(!pfifo)
        return;
    
    pfifo->contains = pfifo->start = 0;

	if(!pfifo->storage)
		return;		

	memset(pfifo->storage, 0, pfifo->size);

} // maFifoEraseB


void maFifoSetStorageB(maFifoB* pfifo, char *storage)
{
	if(!pfifo)
        return;

	pfifo->storage = storage;

	if(!pfifo->storage)
		return;

	pfifo->contains = pfifo->start = 0; // maFifoClear(pfifo);

} // maFifoSetStorageB


/*
unsigned long maFifoDrawB(maFifo* pfifo, char* whereTo, unsigned long requested)
{
	unsigned long amount;
	unsigned long amount1;
	unsigned long amount2;

	if(!pfifo->storage)
		return 0;

	if((amount = requested) > pfifo->contains)
		amount = pfifo->contains;
	
	amount1 = pfifo->size - pfifo->start;

	if(amount <= amount1)
		memmove(whereTo, pfifo->storage + pfifo->start, amount);
	else
	{
		amount2 = amount - amount1;

		memmove(whereTo, pfifo->storage + pfifo->start, amount1);

		memmove(whereTo+amount1, pfifo->storage, amount2);
	}
	
	pfifo->contains -= amount;
	pfifo->start += amount;
	
	if(pfifo->start >= pfifo->size)
		pfifo->start -= pfifo->size;
	
	return amount;
	
} // maFifoDrawB
*/

void maFifoClearB(maFifoB* pfifo)
{	
	if(!pfifo)
        return;

	pfifo->contains = pfifo->start = 0;

} // maFifoClearB


/*
char *maFifoGetStorageB(maFifoB* pfifo)
{
	char *p;

	if(!pfifo)
        return NULL;
        
	p = pfifo->storage;

	return p;

} // maFifoGetStorageB
*/
// end // maFifo functions


// this receives size bytes of pcm data from PlayThreadXX and formats it to 2-channel interleaved machine-endian
// then passes it to the next out chain function.
int maOutChain1(char *buf, unsigned long size)
{
	unsigned long i;
	unsigned long bytes = (g_state.inchannels == 1) ? (MAFIFOBUFSIZE / 2) : MAFIFOBUFSIZE;
	unsigned long bytesleft = size;
	unsigned long monobytes;
	short *p;
	short *q;
	char *pbuf = buf;
	
	if(buf == NULL)
	{
		if(size != 0)
		{
			maLog(L"maOutChain1 NULL buf");
			return 1; // error
		}
		else // NULL and 0 means flush
		{
			// nothing to flush here, pass it on
			maOutChain2(NULL, 0);
			return 0;
		}
	}

#ifdef WORDS_BIGENDIAN
	if(!g_state.inwordsbigendian)
#else
	if(g_state.inwordsbigendian)
#endif
		for(i = 0, p = (short *)buf; i < size >> 1; i++, p++)
			*p = ((((*p)>>8)&0xFF)+(((*p)&0xFF)<<8));
	
	while(!(g_state.stop & STOP_ALL) && bytesleft > 0)
	{	
		if(bytes > bytesleft)
			bytes = bytesleft;

		// pass data on to output processing chain
		if(g_state.inchannels != 1)
		{
			if(!g_state.ininterleaved)
				maLog(L"maOutChain1 !g_state.ininterleaved"); // never expect this
			// else	
				maOutChain2(pbuf, bytes);
		}
		else // one channel -> two channels : abcd -> aabbccdd
		{			
			monobytes = bytes << 1;

			// if bytes increased, realloc a mono buffer
			if(l_monobuf && monobytes > l_monobytes)
			{
				MAFREE(l_monobuf);
				l_monobuf = NULL;
			}

			if(l_monobuf == NULL)
			{
				if((l_monobuf = MAMALLOC(monobytes)) == NULL)
				{
					maLog(L"maOutChain1 MAMALLOC failed");
					maOutChain2(pbuf, bytes); // sound funny
				}
				l_monobytes = monobytes;
			}
	
			p = (short *)l_monobuf;
			q = (short *)pbuf;
			for(i = 0; i < (bytes >> 1); i++)
			{
				*p++ = *q;
				*p++ = *q++;
			}
			
			maOutChain2(l_monobuf, monobytes);
		}
			
		pbuf += bytes;
		bytesleft -= bytes;
			
	} // while bytesleft
	
	return 0;
	
} // maOutChain1


// this applies in-place the effects (pitch, tempo, speed) to len bytes of pcm data
// then passes it to the next out chain function
// or just dumps pcm data to a buffer for maPlaySbsms
int maOutChain2(char *data, unsigned long len)
{
	if(data == NULL)
	{
		if(len != 0)
		{
			maLog(L"maOutChain2 NULL data");
			return 1; // error
		}
		else // NULL and 0 means flush
		{
#if defined(MAPLAYSBSMS)
			if(g_state.pdecoder != (threadparametertype)maPlaySbsms)
#endif
				if(AnyEffectOn())
				{
					l_fxoutlen = FXBUFFERSIZE;
					ApplyEffects(NULL, 0, l_fxoutdata, &l_fxoutlen); // flush the effects
					if(l_fxoutlen > 0)
						maOutChain3(l_fxoutdata, l_fxoutlen);
				}
			maOutChain3(NULL, 0); // flush the output
			return 0;
		}
	}

#if defined(MAPLAYSBSMS)
	// if called from maPlaySbsms with pcmnbytes > 0, create or resize a sbsms pcm buffer,
	// dump the decoded pcm data into it and return
	if(g_state.pdecoder == (threadparametertype)maPlaySbsms)
	{
		while(g_state.paused)
			maSleep(100);
		if(g_state.pcmnbytes > 0)
		{
			if(g_state.pcm == NULL)
			{
				// initital alloctation
				if((g_state.pcm = (char *)MAMALLOC(g_state.pcmmax)) == NULL)
				{
					machar mastr[MAXLOGLINELEN];
 #if !defined(__MAWIN__)
					machar mastr2[MAXLOGLINELEN];	
 #endif
					masprintf(mastr, MAXLOGLINELEN, L"maOutChain2 MAMALLOC failed for %u bytes for '%ls'",
														g_state.pcmmax, fs2ma(mastr2, g_state.inpath));
					maLog(mastr);
					return 1;
				}
				g_state.pcmpos = g_state.pcm;
				g_state.pcmnbytes = 0;
			}
			if((g_state.pcmnbytes += len) > g_state.pcmmax)
			{
				// resize the buffer
				char *p;
				g_state.pcmmax += len;
				if((p = (char *)MAREALLOC((void *)g_state.pcm, g_state.pcmmax)) == NULL)
				{
					machar mastr[MAXLOGLINELEN];
 #if !defined(__MAWIN__)
					machar mastr2[MAXLOGLINELEN];	
 #endif
					masprintf(mastr, MAXLOGLINELEN, L"maOutChain2 MAREALLOC failed for %u bytes for '%ls'",
													g_state.pcmmax, fs2ma(mastr2, g_state.inpath));
					maLog(mastr);
					return 1; // g_state.pcm still valid
				}
				memcpy(p + g_state.pcmnbytes - len, data, (size_t)len);
				g_state.pcm = p;
				g_state.pcmpos = p + g_state.pcmnbytes;
				return 0;
			}
			memcpy(g_state.pcmpos, data, (size_t)len);
			g_state.pcmpos += len;
			return 0;
		} // g_state.pcmnbytes > 0
		else
			maOutChain3(data, len);
	} // thread maPlaySbsms
	else
#endif // MAPLAYSBSMS
	if(AnyEffectOn())
	{
		if(g_fxoutbuffer == NULL)
		{
			if((g_fxoutbuffer = MAMALLOC(FXBUFFERSIZE)) == NULL)
			{
				maLog(L"WriteFiFo MAMALLOC g_fxoutbuffer failed");
				maOutChain3(data, len); // pass the pcm data on unchanged
				return 0;
			}
			l_fxoutdata = g_fxoutbuffer;
		}
	
		l_fxoutlen = FXBUFFERSIZE;
		ApplyEffects(data, len, l_fxoutdata, &l_fxoutlen);
			
		if(l_fxoutlen > 0)
			maOutChain3(l_fxoutdata, l_fxoutlen);
	}
	else
		maOutChain3(data, len); // pass the pcm data on unchanged
							
	return 0;

} // maOutChain2


// this receives len bytes of pcm data and dumps it into g_mafifobuf
// then it returns, or if g_mafifobuf is full it calls the next out chain function
int maOutChain3(char *data, unsigned long len)
{
	unsigned long roomleft;
	unsigned long bytesleft;
	unsigned long bytesstored;
	char *p;
	
	maFifoB* pfifo = l_pmafifoB;

	if(len > 0)
	{
		if((roomleft = maFifoGetFreeSpaceB(pfifo)) == 0)
			return 0; // overrrun

		p = data;
		if(len < roomleft)
		{
			maFifoStoreB(pfifo, p, len);
			return 0;
		}

		// len >= roomleft
		p = data;
		bytesleft = len;
		bytesstored = roomleft;
		while(bytesleft > 0)
		{
			maFifoStoreB(pfifo, p, bytesstored);
			if((roomleft -= bytesstored) <= 0)
				maOutChain4(pfifo); // this returns after the fifo is emptied
			bytesleft -= bytesstored;
			p += bytesstored;
			roomleft = maFifoGetFreeSpaceB(pfifo);
			bytesstored = (bytesleft < roomleft) ? bytesleft : roomleft;
		}
	
		return 0;

	} // len > 0

	// len == 0 means flush fifo
	maOutChain4(pfifo);

	return 0;

} // maOutChain3


// this applies the channel selection (mono, stereo, left, right, etc)
// and the equalization on g_mafifobuf buffer
int maOutChain4(maFifoB* pfifo)
{
	int err = 0;
	unsigned long available;
	unsigned long nframes;

	if((available = maFifoGetFilledSpaceB(pfifo)) == 0) // this s.b. MAFIFOBUFSIZE except at the end of a song
		return 0; // no pcm data in fifo

	nframes = available / MAOUTBYTES; // 16-bit stereo, so MAFIFOBUFSIZE / 4
		
	ApplyChannelSelection(g_mafifobuf, nframes);
		
	if(g_sets.eqflags & MAEQACTIVE)
		maEqualizeBlock(g_mafifobuf, available); // equalize the fifo buffer

	if(g_sets.output == kOUTSOUND)
		err = maOutChain5(g_mafifobuf,available);
	else
	{
#if !defined(NODECODEAUDIOIN)
		if(g_sets.inputmonitor && g_state.infiletype == kMAMAA)
               maOutChain5(g_mafifobuf, available);
#endif
		err = maOutputToFile(g_mafifobuf, nframes, available); 
	}
		
	if(err)

		return 1;
		
	maFifoSetStorageB(pfifo, g_mafifobuf); // resets the fifo counters
	
	return 0;

} // maOutChain4


// this gets called when PlayThreadXX has finished generating pcm data 
void maOutChainFlush()
{
	maOutChain1(NULL, 0);
	g_state.position = 0;
	maIOEnd();

} // maOutChainFlush


// this is where the decoder thread blocks
int maOutChain5(void* pcmblock, unsigned long len)
{
	while(maFifoGetFreeSpace(g_ppafifoout) < len)
	{
#if !defined(NODECODEAUDIOIN)
		if(g_state.infiletype == kMAMAA)
			return 0; // don't block the audioin stream
#endif
		// this blocks PlayThreadXX until maCallbackOut has made room in pafifoout
		MAFIFOSEMWAIT(g_ppafifoout->semaphore);
		if(g_state.stop & STOP_ALL)
			return 1;
	}

	maFifoStore(g_ppafifoout, (char *)pcmblock, len);
			
	return 0;

} // maOutChain5


void DumpString(FILE *fp, machar *p, machar **p2, machar **p3, size_t *len3)
{
 #if defined(__MAWIN__)
	size_t len;
	machar *lp;
	machar *lpb;
	machar *errstr;
	machar marker[4];
	size_t sizep2 = 4096;
 #else
	machar mastr2[MAXLOGLINELEN + 8];
 #endif

	if(!p || *p == L'\0')
		return;

	if(!fp)
	{
		ListWindowAddString(p);
		return;
	}
	
#if !defined(__MAWIN__)
	mastrncpy(mastr2, p, MAXLOGLINELEN);
	mastr2[MAXLOGLINELEN] = L'\0';
	mastrcat(mastr2, g_maeol);
	mafilewrite(0, mastr2, fp);
	return;
#else
	if((len = mastrlen(p)) <= 0)
		return;
		
	// setup buffer p3 twice size of p, and p2 twice size of p3	
	if(*p3 == NULL)
		*len3 = 0;
	if(len > *len3) // allocate bigger p3 and p2 buffers
	{
		if(*p3)
			MAFREE(*p3);
		if(*p2)
			MAFREE(*p2);
		*len3 = len + len;
		if((*p3 = MAMALLOC((*len3 + 2) * sizeof(machar))) == NULL)
		{
			errstr = L"DumpString MAMALLOC p3 failed";
			mafprintf(fp, L"%ls%ls", errstr, g_maeol);
			maLog(errstr);
			// mafprintf(fp, L"%ls%ls", p, g_maeol);
			return;
		}
		sizep2 = (*len3 + *len3 + 2) * sizeof(machar);
		if((*p2 = MAMALLOC(sizep2)) == NULL)
		{
			MAFREE(*p3);
			*p3 = NULL;
			errstr = L"DumpString MAMALLOC p2 failed";
			mafprintf(fp, L"%ls%ls", errstr, g_maeol);
			maLog(errstr);
			// mafprintf(fp, L"%ls%ls", p, g_maeol);
			return;
		}
		**p3 = **p2 = 0;
	}

	// fix any eol garbage from id3 tags etc.
	// change any lfcr to crlf and any single cr or lf to crlf

	// make a unique marker string e.g. 0xffff to remporarily replace any crlf and lfcr
	marker[0] = (unsigned char)0xff;
	marker[1] = (unsigned char)0xff;
	marker[2] = L'\0';
	while(mastrstr(p, marker)) // && marker[0] && marker[1])
	{
		if(marker[0])
			marker[0]--;
		else if(marker[1])
			marker[1]--;
		else
		{
			errstr = L"DumpString cannot make a unique marker, keeping raw info";
			mafprintf(fp, L"%ls%ls", errstr, g_maeol);
			maLog(errstr);
			mafprintf(fp, L"%ls%ls", p, g_maeol);
			return;
		}
	}

	// replace g_maeol (crlf) with marker
	lp = p;
	while((lp = mastrstr(lp, g_maeol)) != 0)
	{
		*lp++ = marker[0];
		*lp++ = marker[1];
	}

	// replace lfcr with marker
	lp = p;
	while((lp = mastrstr(lp, L"\n\r")) != 0)
	{
		*lp++ = marker[0];
		*lp++ = marker[1];
	}

	// change any \r to marker and save to p3
	**p3 = 0;
	lp = lpb = p;
	while((lp = mastrchr(lp, L'\r')) != 0)
	{
		*lp = 0;
		mastrcat(*p3, lpb);
		mastrcat(*p3, marker);
		lpb = ++lp;
	}
	mastrcpy(*p3, p);
	
	// change any \n to marker and save to p2
	**p2 = 0;
	lp = lpb = *p3;
	while((lp = mastrchr(lp, L'\n')) != 0)
	{
		*lp = 0;
		mastrcat(*p2, lpb);
		mastrcat(*p2, marker);
		lpb = ++lp;
	}
	masprintf(*p2, sizep2, L"%ls%ls", *p3, marker);

	// replace all markers with crlf
	lp = *p2;
	while((lp = mastrstr(lp, marker)) != 0)
	{
		*lp++ = g_maeol[0];
		*lp++ = g_maeol[1];
	}

	// save to file
	mafilewrite(0, *p2, fp);
#endif

} // DumpString


#if defined(__MAOSX__)
 // events handled outside the gui
 static pascal OSErr AEQuit(const AppleEvent *theAppleEvent, AppleEvent *theReply, long Refcon);
 pascal OSErr AEQuit(const AppleEvent *theAppleEvent, AppleEvent *theReply, long Refcon)
 {
	(void)theAppleEvent; (void)theReply; (void)Refcon;

	DoAppQuit();

	return noErr;
 }


 static pascal OSErr AEOpenDoc(const AppleEvent *theAppleEvent, AppleEvent *theReply, long Refcon);
 // make a playlist from the FSSpecs of files, dirs or links passed in a mac AEOpenDoc event
 pascal OSErr AEOpenDoc(const AppleEvent *theAppleEvent, AppleEvent *theReply, long Refcon)
 {
	(void)Refcon; (void)theReply;
	DoArgsOpenDocOSX((void *)theAppleEvent);
	
	return noErr;
 }
#endif


int l_oninit = 0;
int OnInit_()
{
	int err;
	fschar dir[MAX_PATH + 2];
	machar instancestr[STRLEN16];
#if !defined(NODECODEAUDIOIN)
	FILE *fp;
 	machar mastr2[MAXLOGLINELEN];
 #if !defined(__MAWIN__)
 	machar mastr3[MAXLOGLINELEN];
 #endif
#endif
#if !defined(__MAWIN__)
	fschar fsstr[MAX_PATH];
#endif
	machar mastr[MAXLOGLINELEN];
	
	if(l_oninit) // sometimes wx calls OnInit again
		return 1;
	
	l_oninit = 1;

#if defined(MAGUINONE)
 printf("Initializing...\n");
#endif

	g_macharset = magetenv(maenvT("MACHARSET"));
	if(g_macharset && g_macharset[0] != maenvT('\0'))
		maenvstrcpy(g_charset, g_macharset);
	else
		maenvstrcpy(g_charset, magetcharset());
#if defined(MAGUICARBON)
	g_macharsetkb = magetenv(maenvT("MACHARSETKB"));
	if(g_macharsetkb && g_macharsetkb[0] != maenvT('\0'))
		maenvstrcpy(g_charsetkb, g_macharsetkb);
	else
		maenvstrcpy(g_charsetkb, g_charset);
#endif
	
#if defined(MASETLOCALE)
	masetlocaledata(); // sets our locale to the current Code Page (win) or en_US.UTF-8
#endif

#if defined(MAGUICARBON) && !defined(NOTEXTVIEWER)
	OSErr status;
	TXNInitOptions options;
	TXNMacOSPreferredFontDescription defaults;  
	defaults.fontID = 0; // kTXNDefaultFontName;	// ATSUFindFontFromName ("Times Roman", strlen("Times Roman"),
													// kFontFullName, kFontNoPlatform, kFontNoScript, kFontNoLanguage, &theFontID); 
	defaults.pointSize = kTXNDefaultFontSize;
	defaults.fontStyle = kTXNDefaultFontStyle;
	defaults.encoding = CFStringGetSystemEncoding();
	options = 0; 
	status = TXNInitTextension (&defaults, 1, options);
	if(status != noErr)
        ShowErr(L"OnInit_ TXNInitTextension failed");
#endif

	// init fileout indices
	Init_g_k();
	
	// song index before startup in direct no-random repeat mode
	l_songsindex[0] = -1;
		
	// seed random generator for random mode
	srand(maGetLocalTimeMillis());

#if defined(__MAUNIX__)
	// unix install data dir
	if(!strcmp("${prefix}/share", INSTALL_DATADIR)) // did not have configure --datadir...
		sprintf(l_mareadmedir, "%s%cshare%cmoreamp", INSTALL_PREFIX, FILESEP, FILESEP);
	else // had configure --datadir=/INSTALL_DATADIR...
		sprintf(l_mareadmedir, "%s%cmoreamp", INSTALL_DATADIR, FILESEP);
#endif

	// get moreampdir
	if(!magetmoreampdir(dir))
	{
		ShowErr(L"OnInit_ magetmoreampdir failed, cannot continue");
		goto oninitretnok;
	}
	fsstrcpy(g_madir, dir);
	
	// create MoreAmp dir if missing
	if(!maDirExists(g_madir))
	{
		if(maCreateDir(g_madir))
		{
			masprintf(mastr, MAXLOGLINELEN, L"OnInit_ failed to create MoreAmp dir%ls%ls'%ls'%ls%lscannot continue",
				g_maeol, g_maeol, g_madir, g_maeol, g_maeol);
			ShowErr(mastr);
			goto oninitretnok;
		}
#if defined(__MAOSX__) || defined(__MAUNIX__)
		machmod(g_madir, ACCESSPERMS); // 0777
#endif
	}	

	// set log path
	fssprintf(g_logpath, MAX_PATH, fsS fsC fsS fsS, g_madir, FILESEP, MALOG, EXTTXT);

	/* not used // get cwd
	if(!magetcwd(l_cwd, MAX_PATH))
	{
		ShowErr(L"OnInit_ magetcwd failed, cannot continue");
		goto oninitretnok;
	}
	*/
	
	// set readme path
	fssprintf(g_readmepath, MAX_PATH, fsS fsC fsS, MAREADMEDIR, FILESEP, MAREADMEFILE); // installed, default
	if(!maFileExists(g_readmepath))
	{
		fssprintf(g_readmepath, MAX_PATH, fsS fsC fsS, g_madir, FILESEP, MAREADMEFILE); // moreamp dir
		if(!maFileExists(g_readmepath))
		{
			// fssprintf(g_readmepath, MAX_PATH, fsS fsC fsS, l_cwd, FILESEP, MAREADMEFILE); // cwd
			// if(!maFileExists(g_readmepath))
			// {
				if(!l_bindir[0] && !magetbindir(l_bindir))
					maLog(L"OnInit_ no l_bindir");
				else
					fssprintf(g_readmepath, MAX_PATH, fsS fsC fsS, l_bindir, FILESEP, MAREADMEFILE); // with ma binary
				if(!maFileExists(g_readmepath))
				{
					fssprintf(g_readmepath, MAX_PATH, fsT("README file not found '") fsS fsC fsS fsT("'"), MAREADMEDIR, FILESEP, MAREADMEFILE); // give up
					maLog(fs2ma(mastr2, g_readmepath));
					fssprintf(g_readmepath, MAX_PATH, fsS fsC fsS, MAREADMEDIR, FILESEP, MAREADMEFILE); // give up					
				}
			// }
		}
	}

	// create 'MASETTINGSDIR' dir if missing
	fssprintf(g_settingsdir, MAX_PATH, fsS fsC fsS, g_madir, FILESEP, MASETTINGSDIR); // initial
	if(!maDirExists(g_settingsdir))
	{
		if(maCreateDir(g_settingsdir))
		{
			masprintf(mastr, MAXLOGLINELEN, L"OnInit_ failed to create Settings dir%ls%ls'%ls'%ls%lscannot continue",
				g_maeol, g_maeol, fs2ma(mastr2, g_settingsdir), g_maeol, g_maeol);
			ShowErr(mastr);
			goto oninitretnok;
		}		
	}		

	// create 'MARECORDSDIR' dir if missing
	fssprintf(g_recordsdir, MAX_PATH, fsS fsC fsS, g_madir, FILESEP, MARECORDSDIR); // initial
	if(!maDirExists(g_recordsdir))
	{
		if(maCreateDir(g_recordsdir))
		{
			masprintf(mastr, MAXLOGLINELEN, L"OnInit_ failed to create Records dir%ls%ls'%ls'%ls%lscannot continue",
				g_maeol, g_maeol, fs2ma(mastr2, g_recordsdir), g_maeol, g_maeol);
			ShowErr(mastr);
			goto oninitretnok;
		}		
	}		

	instancestr[0] = L'\0'; // instance number to cat to strings

#if defined(MAUSESHF) || (defined(MAUSESHM) && defined(__MAWIN__))
	fssprintf(l_mainstancefile, MAX_PATH, fsS fsC fsS, g_settingsdir, FILESEP, MAINSTANCEFILE);
#endif
	if(maNamedSemaphoreLock(MAINITLOCK))
	{
		ShowErr(L"OnInit_ maNamedSemaphoreLock failed");
		return 0;
	}
	
	if((g_thisinstance = InstanceInit()) < 0)
	{
		maLog(L"Multiple instances not allowed, cannot continue");
		goto oninitretnok;
	}
	if(g_thisinstance == 0)
	{
		ShowErr(L"OnInit_ InstanceInit failed, cannot continue");
		goto oninitretnok;
	}
	if(g_thisinstance > 1)
		masprintf(instancestr, STRLEN16, L"%d", g_thisinstance);

	if(instancestr[0] != L'\0')
		masprintf(g_prognametitle, STRLEN32, L"%ls%ls", MOREAMP_NAME, instancestr);
	else
		masprintf(g_prognametitle, STRLEN32, L"%ls", MOREAMP_NAME);

	// set some globals
	
#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
	fssprintf(l_setscdrompath, MAX_PATH, fsS fsC fsS fsC fsS fsS, g_madir, FILESEP, MASETTINGSDIR, FILESEP, MASETSCDROMFILE, EXTMAS);
#endif

#if defined(MAKEYBOARD)
	fssprintf(g_keybindingspath, MAX_PATH, fsS fsC fsS fsC fsS fsS, g_madir, FILESEP, MASETTINGSDIR, FILESEP, MAKEYBINDINGSFILE, EXTTXT);
#endif

	// "~/moreamp/masettings/" "~/moreamp/masettings2/" ...
	if(instancestr[0] != L'\0')
		fssprintf(g_settingsdir, MAX_PATH, fsS fsC fsS fsS, g_madir, FILESEP, MASETTINGSDIR, ma2fs(fsstr, instancestr));
	else
		fssprintf(g_settingsdir, MAX_PATH, fsS fsC fsS, g_madir, FILESEP, MASETTINGSDIR);
	fssprintf(g_playlistsdir, MAX_PATH, fsS fsC fsS, g_madir, FILESEP, MAPLAYLISTSDIR);
	fssprintf(g_fileoutdir, MAX_PATH, fsS fsC fsS, g_madir, FILESEP, MAFILEOUTDIR);

	// "~/moreamp/masettings/madrop.maf" "~/moreamp/masettings2/madrop.maf" ...
	fssprintf(g_playlistdrop, MAX_PATH, fsS fsC fsS fsS, g_settingsdir, FILESEP, PLAYLISTDROP, EXTMAF);
	// "~/moreamp/masettings/macda.maf" "~/moreamp/masettings2/macda.maf" ...
	fssprintf(g_playlistcda, MAX_PATH, fsS fsC fsS fsS, g_settingsdir, FILESEP, PLAYLISTCDA, EXTMAF);
#if !defined(NODECODEAUDIOIN)
	// "~/moreamp/masettings/maaudioin.maf"
	fssprintf(g_playlistaudioin, MAX_PATH, fsS fsC fsS fsS, g_settingsdir, FILESEP, PLAYLISTAUDIOIN, EXTMAF);
	// "~/moreamp/masettings/maaudioin.maa"
	fssprintf(g_songfileaudioin, MAX_PATH, fsS fsC fsS fsS, g_settingsdir, FILESEP, PLAYLISTAUDIOIN, EXTMAA);
#endif
	// "~/moreamp/masettings/maplaylisttmp.maf" "~/moreamp/masettings2/maplaylisttmp.maf" ...
	fssprintf(g_playlisttemp, MAX_PATH, fsS fsC fsS fsS, g_settingsdir, FILESEP, PLAYLISTTMP, EXTMAF);
		
	// "~/moreamp/masettings/madrop.mas" "~/moreamp/masettings2/madrop.mas" ...
	fssprintf(g_settingsdrop, MAX_PATH, fsS fsC fsS fsS, g_settingsdir, FILESEP, PLAYLISTDROP, EXTMAS);

	// "~/moreamp/masettings/madefault.mas" "~/moreamp/masettings2/madefault.mas" ...
	fssprintf(g_settingsdefpath, MAX_PATH, fsS fsC fsS fsS, g_settingsdir, FILESEP, MASETSDEFAULT, EXTMAS);
		
	// "~/moreamp/masettings/macda.mas" "~/moreamp/masettings2/macda.mas" ...
	fssprintf(l_settingscda, MAX_PATH, fsS fsC fsS fsS, g_settingsdir, FILESEP, PLAYLISTCDA, EXTMAS);

	// "~/moreamp/masettings/maaudioin.mas" "~/moreamp/masettings2/maaudioin.mas" ...
	fssprintf(l_settingsaudioin, MAX_PATH, fsS fsC fsS fsS, g_settingsdir, FILESEP, PLAYLISTAUDIOIN, EXTMAS);

	// "~/moreamp/masettings/marecentlistname.txt" "~/moreamp/masettings2/marecentlistname.txt" ...
	fssprintf(l_recentlist, MAX_PATH, fsS fsC fsS, g_settingsdir, FILESEP, RECENTPLAYLIST);

	l_settingspathlast[0] = fsT('\0');
	
	// create 'MAPLAYLISTSDIR' dir if missing
	if(!maDirExists(g_playlistsdir))
	{
		if(maCreateDir(g_playlistsdir))
		{
			masprintf(mastr, MAXLOGLINELEN, L"OnInit_ failed to create Playlists dir%ls%ls'%ls'%ls%lscannot continue",
				g_maeol, g_maeol, fs2ma(mastr2, g_playlistsdir), g_maeol, g_maeol);
			ShowErr(mastr);
			goto oninitretnok;
		}		
	}		

	// create 'MASETTINGSDIR' dir if missing
	if(!maDirExists(g_settingsdir))
	{
		if(maCreateDir(g_settingsdir))
		{
			masprintf(mastr, MAXLOGLINELEN, L"OnInit_ failed to create Settings dir%ls%ls'%ls'%ls%lscannot continue",
				g_maeol, g_maeol, fs2ma(mastr2, g_settingsdir), g_maeol, g_maeol);
			ShowErr(mastr);
			goto oninitretnok;
		}		
	}		
	
	// create 'MAFILEOUTDIR' dir if missing
	if(!maDirExists(g_fileoutdir))
	{
		if(maCreateDir(g_fileoutdir))
		{
			masprintf(mastr, MAXLOGLINELEN, L"OnInit_ failed to create FileOut dir%ls%ls'%ls'%ls%lscannot continue",
				g_maeol, g_maeol, fs2ma(mastr2, g_fileoutdir), g_maeol, g_maeol);
			ShowErr(mastr);
			goto oninitretnok;
		}		
	}

#if !defined(NODECODEAUDIOIN)
	// create g_playlistaudioin and g_songfileaudioin if missing
	if(!maFileExists(g_songfileaudioin))
	{
		if((fp = mafopen(g_songfileaudioin, fsT("wb"))) == NULL)
		{
			masprintf(mastr, MAXLOGLINELEN, L"OnInit_ mafopen failed on '%ls'", g_songfileaudioin);
			maLog(mastr);
		}
		else
		{
				fclose(fp);
 #if defined(__MAOSX__) || defined(__MAUNIX__)
			machmod(g_songfileaudioin, ACCESSPERMS); // 0777
 #endif
		}
	}
	if(!maFileExists(g_playlistaudioin))
	{
		if((fp = mafopen(g_playlistaudioin, fsT("wb"))) == NULL)
		{
			masprintf(mastr, MAXLOGLINELEN, L"OnInit_ mafopen failed on '%ls'", g_playlistaudioin);
			maLog(mastr);
		}
		else
		{
			// BOM
			fwrite("\xef\xbb\xbf", 3, 1, fp); // BOM_UTF-8

			masprintf(mastr2, MAXLOGLINELEN, L"%ls%ls", fs2ma(mastr3, g_songfileaudioin), g_maeol);
			mafilewrite(0, mastr2, fp);

			fclose(fp);
 #if defined(__MAOSX__) || defined(__MAUNIX__)
			machmod(g_playlistaudioin, ACCESSPERMS); // 0777
 #endif
		}
	}
#endif

	// load keyboard bindings
#if defined(MAKEYBOARD)
	SetFactoryKeyBindings();
	if(LoadKeyBindings(0) == 2) // just created g_keybindingspath
		LoadKeyBindings(0);
#endif

	// get CDRom drives
#if !defined(MANOCDA)
 #if defined(__MAUNIX__) && !defined(MACDAOSX)
	LoadSettingsCDRom();
 #endif
	if(!CdaInitCDRom()) // OnInit_
		maPlaylistMakeCda();
#endif

	// allocate buffers
	if(maAllocBuffers())
	{
		ShowErr(L"OnInit_ maAllocBuffers failed, cannot continue");
		goto oninitretnok;
	}
	
	maScanAudioDevices();

	/*
	if(maSemaphoreNew(&g_state.playersem))
	{
		g_state.playersem = NULL;
		maLog(L"OnInit_ maSemaphoreNew playersem failed");
	}
	*/

	if(DoPassedArgs())
		goto oninitretok; // had passed args, settings got loaded and window created

	if((err = GetRecentPlaylist()) != 0)
		LoadDefaultSettings(&g_sets);
		
	if(!g_w[WINDOWAPP])
		AppWindowMakeA();
	
	// autostart
	if(!err && !maModKeyDown() && g_sets.autostart && g_sets.output == kOUTSOUND && g_songsnum > 0)
	{
		if(g_sets.exitafter)
			ZeroRecords(&l_psongrec, g_songsnum);
		DoTimerPlay(100); // OnInit_
	}
	else
		SetAppWindowTitle(L"Stopped");

oninitretok:
	if(maNamedSemaphoreUnlock(MAINITLOCK))
	{
		ShowErr(L"OnInit_ maNamedSemaphoreUnlock failed");
		return 0;
	}

#if defined(MAGUINONE)
 printf("Ready\n");
#endif

	return 1; // ok
	
oninitretnok:
	if(maNamedSemaphoreUnlock(MAINITLOCK))
		ShowErr(L"OnInit_ maNamedSemaphoreUnlock failed");
	
	return 0; // failed
	
} // OnInit_


int ClearInitLock()
{
	if(maNamedSemaphoreUnlock(MAINITLOCK))
	{
		maLog(L"ClearInitLock maNamedSemaphoreUnlock failed");
		return 1;
	}
	
	return 0;
}


// set or reset multiple instances flag in the shared instance table
int SetMultipleInstances()
{
	MAINSTANCE instance;
	int multiple;
	
	// read the current instance table
	if(InstanceRead(&instance))
	{
		ShowErr(L"SetMultipleInstances InstanceRead failed");
		return 1;
	}
	
	if(g_sets.multipleinst)
	{
		if(instance.multipleinst) // already set to multiple instances
			return 0;
		multiple = 1;
	}
	else
	{
		if(!instance.multipleinst) // already set to no multiple instances
			return 0;
		multiple = 0;
	}
	
	if(InstanceSet(MAINSTANCEMAX, multiple))
	{
		ShowErr(L"SetMultipleInstances InstanceSet failed");
		return 1;
	}
	
	return 0;
	
} // SetMultipleInstances


#if defined(__MAOSX__) || defined(MACDAOSX)
 #if !defined(MANOCDA)
  #if (1) // TARGET_API_MAC_CARBON
  pascal OSStatus OnDiskEvent(EventHandlerCallRef handler, EventRef event, void *data);
  pascal OSStatus OnDiskEvent(EventHandlerCallRef handler, EventRef event, void *data)
  {
	(void)handler; (void)data;

	int clas = GetEventClass(event);
	int kind = GetEventKind(event);
	EventParamType outActualType = 0;
	UInt32 outActualSize = 0;
	short volnum = 0;
	OSErr err;

	if(clas == kEventClassVolume && (kind == kEventVolumeMounted || kind == kEventVolumeUnmounted))
	{
		if((err = GetEventParameter(	event,					// EventRef inEvent,
										kEventParamDirectObject,// EventParamName inName,
										typeFSVolumeRefNum,		// EventParamType inDesiredType,
 										&outActualType,			// can be NULL
 										2,						// UInt32 inBufferSize,
 										&outActualSize, 		// UInt32 *outActualSize, // can be NULL
                                        &volnum)) != noErr)
			return eventNotHandledErr;
 								
		if(!CdaInitCDRom()) // OnDiskEvent
           maPlaylistMakeCda();
        if(g_sets.mawin[WINDOWLIST].ison)
        {
            if(g_sets.listboxmode & LBOXPLAYERINFO)
                UpdateListWindowList(1);
            else if(PlaylistIsCda())
                 UpdateListWindowList(0);
       }
	}
	
	return eventNotHandledErr; //noErr;
	
  } // OnDiskEvent
  #endif // TARGET_API_MAC_CARBON
 #endif
#endif


#if defined(__MAOSX__) || defined(MACDAOSX)
void InitAppleEvents(void *app)
{
 #if !TARGET_API_MAC_CARBON
  (void)app;
 #endif

 #if !defined(MACDAOSX)
	// on mac need this to quit program when mac is shut off
	AEInstallEventHandler(kCoreEventClass, kAEQuitApplication, NewAEEventHandlerUPP(AEQuit), 0, 0);
	
	// on mac need this for dropped files
	AEInstallEventHandler(kCoreEventClass, kAEOpenDocuments, NewAEEventHandlerUPP(AEOpenDoc), 0, 0);
 #endif
 	
 #if !defined(MANOCDA)
  #if !TARGET_API_MAC_CARBON
	// AEInstallEventHandler(kAEDiskEvent, diskEvt, NewAEEventHandlerUPP(OnDiskEvent), 0, 0);
  #else
	static const EventTypeSpec eventList[] = {{kEventClassVolume, kEventVolumeMounted},
											  {kEventClassVolume, kEventVolumeUnmounted},};
	// OSStatus err = 
	InstallApplicationEventHandler(NewEventHandlerUPP(OnDiskEvent),
							sizeof(eventList)/sizeof(eventList[0]), eventList, app, NULL);
	// if(err) printf("InstallApplicationEventHandler err=%d\n", (int)err);
  #endif // !TARGET_API_MAC_CARBON
 #endif
}
#endif
	

void AppQuit() // app exit
{
	int windowid;

	/*
	if(g_state.playersem)
	{
		if(maSemaphoreDelete(g_state.playersem))
			maLog(L"AppQuit maSemaphoreDelete playersem failed");		
		g_state.playersem = NULL;
	}
	*/

	// wait for decoder thread to finish
	WaitDecoderStopped();
	
	maPaTerminate(); // AppQuit

	CleanupEffects();

	if(g_memhandle)
	{
		MAFREE(g_memhandle);
		g_memhandle = NULL; 
		l_songmem = NULL; 
	}

#if !defined(MANOCDA)&& defined(__MAWIN__)
	CdaDeInitCDRom();
#endif

	// close all windows
	for(windowid = WINDOWFIRST; windowid <= WINDOWLAST; windowid++)
	{
		if(windowid == WINDOWAPP) // skip WINDOWAPP
			continue;
		DoWindow(windowid, 0);
	}

	WindowDestroy(WINDOWAPP);
	g_w[WINDOWAPP] = NULL;

	if(l_fplogpath)
	{
		fflush(l_fplogpath);
		fclose(l_fplogpath);
	}
	
	maFreeBuffers();
	
	InstanceClearThis();

#if defined(MAUSESHM)
	if(g_pmashm)
	{
		MAFREE(g_pmashm);
		g_pmashm = NULL;
	}
#endif
	
	if(l_psongrec)
	{
		MAFREE(l_psongrec);
		l_psongrec = NULL;
	}
	
	if(g_playlist0)
	{
		MAFREE(g_playlist0);
		g_playlist0 = NULL;
	}	

	maGUIfree();

#if defined(MAMEMTRACE)
{
machar mastr[MAXLOGLINELEN];
masprintf(mastr, MAXLOGLINELEN, L"g_nmamallocs=%d g_nmacallocs=%d g_ndupallocs=%d (%d) g_nmafrees=%d g_ndupfrees=%d (%d)",
	g_nmamallocs, g_nmacallocs, g_ndupallocs, g_nmamallocs + g_nmacallocs + g_ndupallocs,
	g_nmafrees, g_ndupfrees, g_nmafrees + g_ndupfrees);
maLog(mastr);
maTrace(mastr);
}
#endif

	maGUIQuit();

} // AppQuit


int SetVDisk()
{
	fschar dirpath[MAX_PATH];
	
	if(g_sets.vdiskname && g_sets.vdiskname[0])
		fsstrcpy(dirpath, g_sets.vdiskname); // default dir for dirSelector
	else
		dirpath[0] = fsT('\0');
	
	if(maDirSelector(dirpath, L"Select a FOLDER on a virtual disk"))	
		return 1; // cancelled

	fsstrcpy(g_sets.vdiskname, dirpath);
	
	return 0;

} // SetVDisk


#if !defined(MANOCDA) && defined(__MAUNIX__) && !defined(MACDAOSX)
 int SetsAudioCD()
 {
	int i;
	int item;
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	PopupButtonGetSelection(WINDOWAUDIOCD, ID_POPUP_AUDIOCD, &item);
	if(item == -1)
		return 1;

	item++;
	
	if(maFileDialog(g_setscdrom.cdaname[item], WINDOWAPP, L"Select or enter the name of an audio cd drive",
					 "/dev", fs2ma(mastr2, g_setscdrom.cdaname[item]), L"(*)|*|Any file (*)|*", maOPEN)) 
		return 1; // cancelled

	
#if defined(MAGUICARBON) || defined(MAGUIGTK2)
	PopupButtonSetText(WINDOWAUDIOCD, ID_POPUP_AUDIOCD, item, fs2ma(mastr2, g_setscdrom.cdaname[item + 1]));
	if(g_sets.eqpreset == 0) // force a redraw
		i = 1;
	else
		i = 0;
	PopupButtonSetValue(WINDOWEQ, ID_POPUP_AUDIOCD, i);
#else
	// PopupButtonSetText fails...
	PopupButtonClear(WINDOWAUDIOCD, ID_POPUP_AUDIOCD);
	for(i = 1; i <= CDAMAXNUMDRIVES; i++)
		PopupButtonAppend(WINDOWAUDIOCD, ID_POPUP_AUDIOCD, fs2ma(mastr2, g_setscdrom.cdaname[i]));
#endif
	PopupButtonSetValue(WINDOWAUDIOCD, ID_POPUP_AUDIOCD, item - 1);

	return 0;

 } // SetsAudioCD


 int ClearAudioCD()
 {
	int i;
	int item;
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	
	PopupButtonGetSelection(WINDOWAUDIOCD, ID_POPUP_AUDIOCD, &item);
	if(item == -1)
		return 1;

	fssprintf(g_setscdrom.cdaname[item + 1], MAX_PATH, fsT("(drive%02d)"), item + 1);
	
 #if defined(MAGUICARBON) || defined(MAGUIGTK2)
	PopupButtonSetText(WINDOWAUDIOCD, ID_POPUP_AUDIOCD, item, fs2ma(mastr2, g_setscdrom.cdaname[item + 1]));
	if(g_sets.eqpreset == 0) // force a redraw
		i = 1;
	else
		i = 0;
	PopupButtonSetValue(WINDOWEQ, ID_POPUP_AUDIOCD, i);
 #else
	// PopupButtonSetText fails...
	PopupButtonClear(WINDOWAUDIOCD, ID_POPUP_AUDIOCD);
	for(i = 1; i <= CDAMAXNUMDRIVES; i++)
		PopupButtonAppend(WINDOWAUDIOCD, ID_POPUP_AUDIOCD, fs2ma(mastr2, g_setscdrom.cdaname[i]));
 #endif
	PopupButtonSetSelection(WINDOWAUDIOCD, ID_POPUP_AUDIOCD, item);

	return 0;

 } // ClearAudioCD
#endif


int InstanceWrite(MAINSTANCE *pinstance)
{
	if(!g_pmashm)
	{
		ShowErr(L"InstanceWrite null g_pmashm");
		return 1;
	}

	if(mashmwrite((char *)pinstance, 1, sizeof(MAINSTANCE), g_pmashm) != sizeof(MAINSTANCE))
	{
		ShowErr(L"InstanceWrite mashmwrite failed");
		return 1;
	}
	
	return 0;
	
} // InstanceWrite


int InstanceRead(MAINSTANCE *pinstance)
{
	// attach to our shared instance table, create one if needed
	if(!g_pmashm)
	{
		if((g_pmashm = mashmopen(sizeof(MAINSTANCE), MAKEYFILE)) == NULL)
		{
			ShowErr(L"InstanceRead mashmopen failed");
			return 1;
		}
	}

	if(mashmread((char *)pinstance, 1, sizeof(MAINSTANCE), g_pmashm) != sizeof(MAINSTANCE))
	{
		ShowErr(L"mashmread mashmread failed");
		return 1;
	}
	
	return 0;
		
} // InstanceRead


// get a number n for masettings1, masettings2, ... masettingsn
int InstanceInit()
{
	int i;
	int ninstances;
	int pid;
	MAINSTANCE instance;
#if defined(__MAWIN__)
	int l;
	MAINSTANCE linstance;
#endif
	machar mastr[MAXLOGLINELEN];
	
	// wait here until no other instance has this lock
	if(maNamedSemaphoreLock(MAINSTANCELOCK))
	{
		ShowErr(L"InstanceInit maNamedSemaphoreLock failed");
		return 0;
	}

	// read the shared instance table
	if(InstanceRead(&instance))
	{
		ShowErr(L"InstanceInit InstanceRead failed");
		maNamedSemaphoreUnlock(MAINSTANCELOCK);
		return 0;
	}

	// do some integrity checks on our pid table, e.g. in case a previous
	// instance failed to clear its pid table entry when it quit (killed or crashed)

	// clear pid table entries for instances not found to be running
#if defined(__MAWIN__)
	// on win (but not win9x) we know how to get an array of MoreAmp pids
	memset((void *)&linstance, 0, sizeof(MAINSTANCE));
	if(maNumProcesses(linstance.pid))
	{
		for(i = 0; i < MAINSTANCEMAX; i++)
			if(instance.pid[i])
			{
				for(l = 0; l < MAINSTANCEMAX; l++)
					if(instance.pid[i] == linstance.pid[l])
						break;
				if(l >= MAINSTANCEMAX)
					instance.pid[i] = 0; // no longer running, clear it
			}
	}
	else // some non-NT win
#endif
	{
		// otherwise, we only know how to check for a given MoreAmp pid running
		for(i = 0; i < MAINSTANCEMAX; i++)
			if(instance.pid[i] && maNumProcesses(&instance.pid[i]) != 1)
				instance.pid[i] = 0; // no longer running, clear it
	}

	// count instances running according to our pid table
	ninstances = 0;
	for(i = 0; i < MAINSTANCEMAX; i++)
		if(instance.pid[i] != 0)
			ninstances++;
			
	// quit if multiple instances are not allowed
	if(ninstances && !instance.multipleinst && instance.pid[0] != 0)
	{
		maNamedSemaphoreUnlock(MAINSTANCELOCK);
		return -1;
	}

	// put our pid in the first free slot in the pid table
	pid = magetpid();
	for(i = 0; i < MAINSTANCEMAX; i++)
		if(instance.pid[i] == pid)
			instance.pid[i] = 0;
	for(i = 0; i < MAINSTANCEMAX; i++)
		if(instance.pid[i] == 0)
			break;
	if(i >= MAINSTANCEMAX)
	{
		masprintf(mastr, MAXLOGLINELEN, L"InstanceInit cannot run more than %d instances", MAINSTANCEMAX);
		ShowErr(mastr);
		maNamedSemaphoreUnlock(MAINSTANCELOCK);
		return 0;
	}
	instance.pid[i] = pid;
	if(InstanceWrite(&instance))
	{
		ShowErr(L"InstanceInit InstanceWrite failed");
		maNamedSemaphoreUnlock(MAINSTANCELOCK);
		return 0;
	}
	
	maNamedSemaphoreUnlock(MAINSTANCELOCK);
		
	// we are 1-based instance i + 1
	return i + 1;
	
} // InstanceInit


// clear the pid field of this instance (called on app exit)
int InstanceClearThis()
{
	int i;
	int pid;
	MAINSTANCE instance;
	machar mastr[MAXLOGLINELEN];
	
	if(maNamedSemaphoreLock(MAINSTANCELOCK))
	{
		maLog(L"InstanceClearThis maNamedSemaphoreLock failed");
		return 1;
	}

	if(InstanceRead(&instance))
	{
		ShowErr(L"InstanceClearThis InstanceRead failed");
		maNamedSemaphoreUnlock(MAINSTANCELOCK);
		return 1;
	}
	
	pid = magetpid();
	for(i = 0; i < MAINSTANCEMAX; i++)
		if(instance.pid[i] == pid)
			break;
	
	if(i >= MAINSTANCEMAX)
	{
		masprintf(mastr, MAXLOGLINELEN, L"InstanceClearThis pid %d not found in instance file", pid);
		maLog(mastr);
		maNamedSemaphoreUnlock(MAINSTANCELOCK);
		return 1;
	}
		
	instance.pid[i] = 0;
		
	if(InstanceWrite(&instance))
	{
		ShowErr(L"InstanceClearThis InstanceWrite failed");
		maNamedSemaphoreUnlock(MAINSTANCELOCK);
		return 1;
	}

	maNamedSemaphoreUnlock(MAINSTANCELOCK);

	return 0;
	
} // InstanceClearThis


int InstanceClearAll()
{
	int i;
	MAINSTANCE instance;
	MALOCK *lock;

	lock = MALOGLOCK;
	if(maNamedSemaphoreOpen(&(lock->psem), lock->name))
		ShowErr(L"InstanceClearAll maNamedSemaphoreOpen MALOGLOCK failed");
	maNamedSemaphoreUnlock(lock);
	
	lock = MAINITLOCK;
	if(maNamedSemaphoreOpen(&(lock->psem), lock->name))
		ShowErr(L"InstanceClearAll maNamedSemaphoreOpen MAINITLOCK failed");
	maNamedSemaphoreUnlock(lock);
	
	lock = MAINSTANCELOCK;
	if(maNamedSemaphoreOpen(&(lock->psem), lock->name))
		ShowErr(L"InstanceClearAll maNamedSemaphoreOpen MAINSTANCELOCK failed");
	maNamedSemaphoreUnlock(lock);

	if(InstanceRead(&instance))
		ShowErr(L"InstanceClearAll InstanceRead failed");

	for(i = 0; i < MAINSTANCEMAX; i++)
		instance.pid[i] = 0;
		
	if(InstanceWrite(&instance))
		ShowErr(L"InstanceClearAll InstanceWrite failed");

	return 0;
	
} // InstanceClearAll


int InstanceListAll()
{
	int i, j;
	MAINSTANCE instance;

	if(InstanceRead(&instance))
	{
		printf("InstanceListAll InstanceRead failed\n");
		maNamedSemaphoreUnlock(MAINSTANCELOCK);
		return 1;
	}
	
	j = 0;
	for(i = 0; i < MAINSTANCEMAX; i++)
	{
		if(instance.pid[i])
		{
			j++;
			printf("InstanceListAll %2d pid=%d\n", i + 1, instance.pid[i]);
		}
	}
	
	printf("InstanceListAll this pid=%d multipleinst=%d locks=%d\n", magetpid(), instance.multipleinst, j);

	return 0;
	
} // InstanceListAll


// set some field of an instance table
int InstanceSet(int which, int value)
{
	MAINSTANCE instance;
	machar mastr[MAXLOGLINELEN];
	
	if(maNamedSemaphoreLock(MAINSTANCELOCK))
	{
		maLog(L"InstanceSet maNamedSemaphoreLock failed");
		return 1;
	}

	if(InstanceRead(&instance))
	{
		ShowErr(L"InstanceSet InstanceRead failed");
		maNamedSemaphoreUnlock(MAINSTANCELOCK);
		return 1;
	}
	
	if(which < 0 || which > MAINSTANCEMAX)
	{
		masprintf(mastr, MAXLOGLINELEN, L"InstanceSet index=%d out of range", which);
		maLog(mastr);
		maNamedSemaphoreUnlock(MAINSTANCELOCK);
		return 1;
	}
		
	if(which == MAINSTANCEMAX)
		instance.multipleinst = value;
	else
		instance.pid[which] = value;
		
	if(InstanceWrite(&instance))
	{
		ShowErr(L"InstanceSet InstanceWrite failed");
		maNamedSemaphoreUnlock(MAINSTANCELOCK);
		return 1;
	}

	maNamedSemaphoreUnlock(MAINSTANCELOCK);

	return 0;
	
} // InstanceSet


void maLog(const machar *str)
{
	int logoff;
#if !defined(NOTEXTVIEWER) && !defined(MAGUINONE)
	maviewer *v;
#endif
	time_t stTime;
	struct tm *tmTime;
	machar dateline[STRLEN64];
	machar line[MAXLOGLINELEN];
	int needbom;
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];
#endif
	machar mastr[MAXLOGLINELEN + MAXLOGLINELEN];
		
	if((l_logoff || g_sets.logflags & kMALOGOFF) && str != &g_logcmdclear && str != &l_logcmdexist)
		return;

	// some functions called from here might call maLog
	logoff = l_logoff;
	l_logoff = 1;
	
	if(maNamedSemaphoreLock(MALOGLOCK))
	{
		ShowErr(L"maLog maNamedSemaphoreLock failed");
		l_logoff = logoff;
		return;
	}

	if(str == &l_logcmdexist)
	{
		if(maFileExists(g_logpath))
			l_havelog = 1;
		else
			l_havelog = 0;
		SetLogMenus();
		goto maLogret;
	}
	
	if(!l_havelog)
	{
		l_havelog = 1;
		needbom = 1;
		SetLogMenus();
	}
	else
		needbom = 0;
	
	if(!l_fplogpath)
	{
		if((l_fplogpath = mafopen(g_logpath, FSABMODE)) == NULL)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maLog failed to open logfile '%ls'", fs2ma(mastr2, g_logpath));
			ShowErr(mastr);
			goto maLogret;
		}

		// BOM
		if(needbom)
			fwrite("\xef\xbb\xbf", 3, 1, l_fplogpath); // BOM_UTF-8		
	}

	if(str == &g_logcmdclear)
	{
		fclose(l_fplogpath);
		l_fplogpath = NULL;
		if(maunlink(g_logpath))
		{
			masprintf(mastr, MAXLOGLINELEN, L"maLog failed to delete '%ls'", fs2ma(mastr2, g_logpath));
			ShowErr(mastr);
		}
		l_logcount = l_logtotal = 0;
		l_havelog = 0;
		SetLogMenus();
		goto maLogret;
	}
	
	if(logoff || g_sets.logflags & kMALOGOFF)
		goto maLogret;
		
	stTime = time(NULL);
	tmTime = localtime((time_t *)&stTime);

	masprintf(dateline, STRLEN64, L"%2.2d:%2.2d:%2.2d", tmTime->tm_hour, tmTime->tm_min, tmTime->tm_sec);
	dateline[8] = 0;	

	masprintf(line, MAXLOGLINELEN, L"%d %ls %ls%ls", g_thisinstance, dateline, str, g_maeol);

	mafilewrite(0, line, l_fplogpath);	
	fflush(l_fplogpath);

	if(g_sets.logflags & kMALOGMAX)
	{
		if(++l_logtotal >= g_sets.logmax)
		{			
			g_state.stop |= STOP_ALL; // maLog too many errors
			logoff = 1;
			masprintf(line, MAXLOGLINELEN, L"%ls Too many total errors : %d. Logging is now turned off.%ls",
																	dateline, l_logtotal, g_maeol);
			mafilewrite(0, line, l_fplogpath);
			fflush(l_fplogpath);
			fclose(l_fplogpath);
			l_fplogpath = NULL;			
			masprintf(line, MAXLOGLINELEN,
				L"Too many total errors : %d.%ls%lsLogging is now turned off.%ls%ls"
				L"You should quit MoreAmp and see file%ls%ls'%ls'.",
				l_logtotal, g_maeol, g_maeol, g_maeol, g_maeol, g_maeol, g_maeol, fs2ma(mastr2, g_logpath));
			ShowErr(line);
			goto maLogret;
		}
	}
	
	if(g_sets.logflags & kMALOGRATE)
	{
		if(!l_logcount)
			l_logdelay = maGetLocalTimeMillis() + g_sets.logperiod * 1000;
	
		if(++l_logcount >= g_sets.logevents)
		{
			if(maGetLocalTimeMillis() < l_logdelay)
			{
				g_state.stop |= STOP_ALL; // maLog too many errors too fast
				masprintf(line, MAXLOGLINELEN, L"%ls Too many errors : %d in less than %d seconds%ls", dateline, l_logcount, g_sets.logperiod, g_maeol);
				mafilewrite(0, line, l_fplogpath);
				fflush(l_fplogpath);				
				masprintf(line, MAXLOGLINELEN, L"Too many errors : %d in less than %d seconds.%ls%lsSee file '%ls'.",
						l_logcount, g_sets.logperiod, g_maeol, g_maeol, fs2ma(mastr2, g_logpath));
				ShowErr(line);
			}
			l_logcount = 0;
		}
	}
	
maLogret:
	if(l_fplogpath)
	{
		fclose(l_fplogpath); // for the ClearLog function
		l_fplogpath = NULL;
	}
	if(maNamedSemaphoreUnlock(MALOGLOCK))
		ShowErr(L"maLog maNamedSemaphoreUnlock failed");
	
#if defined(MAGUINONE)
	maprintf(L"%ls%ls", str, g_maeol);
#else
 #if !defined(NOTEXTVIEWER)
	if(g_w[WINDOWLOG] && !l_logupdate) // update the log window
	{
		l_logupdate = 1;
		if((v = Viewer(WINDOWLOG)) != 0)
		{
			ViewerDisable(WINDOWLOG, v);
			ViewerFree(v);
			ViewerLoad(WINDOWLOG, v);
			ViewerEnable(WINDOWLOG, v);
		}
		// scroll to last line:
		EditTextShowPosition(WINDOWLOG, ID_EDITTEXT_VIEWERVIEW, -1); // -1 == end
		l_logupdate = 0;
	}
 #endif
#endif

	l_logoff = logoff;
	return; // for maLogret label
	
} // maLog


void DoAppWindowDrop() // called when dropping files
{
	fsstrcpy(g_settingspath, g_settingsdrop);
	fsstrcpy(g_playlistpath, g_playlistdrop);
	LoadSettings();
	DoWindow(WINDOWAPP, 1);
}


void AppWindowMakeA()
{
	AppWindowMake();
#if defined(USE_KEYBOARD) && defined(MAGUIWX)
	CheckBoxSetFocus(WINDOWAPP, ID_CHK_EQWINDOW); // arbitrary initial sink for char events
#endif
	UpdateDisplay();
	SetAppWindowTitle(L"");
	maLog(&l_logcmdexist);

} // AppWindowMakeA


// gets vdisk volume name from settings and checks existence, and builds songdir path
int GetVDisk()
{
	if(!g_sets.vdiskname || g_sets.vdiskname[0] == fsT('\0') || !maDirExists(g_sets.vdiskname))
		return 1; // error

	// maybe do some more checks
	//...
	
	fssprintf(l_loadeddir, MAX_PATH, fsS fsC fsS fsT("%d"), g_sets.vdiskname, FILESEP, MAVDISKTMP, g_thisinstance);
		
	return 0;
	
} // GetVDisk


// this is called when a window has been moved
void DoWindowMoved(int windowid)
{
	int xnew;
	int ynew;
	int dummy;
 	machar mastr[MAXLOGLINELEN];
	
	if(l_appwindowmoving)
		return;

	if(windowid == WINDOWAPP)
	{
		if(!maModKeyDown()) // move other windows
		{
			l_appwindowmoving = 1; 
			WindowGetCoordinates(WINDOWAPP, &xnew, &ynew, &dummy, &dummy);
			WindowsMoveDelta(WINDOWAPP, xnew - g_sets.mawin[WINDOWAPP].xpos[g_sets.winconf],
										ynew - g_sets.mawin[WINDOWAPP].ypos[g_sets.winconf]);
		}

		WindowGetCoordinates(WINDOWAPP,
			&g_sets.mawin[WINDOWAPP].xpos[g_sets.winconf], &g_sets.mawin[WINDOWAPP].ypos[g_sets.winconf],
			&dummy, &dummy);
		l_appwindowmoving = 0;
		return;
	}

	if(windowid < WINDOWFIRST || windowid > WINDOWLAST)
	{
		masprintf(mastr, MAXLOGLINELEN, L"DoWindowMoved invalid windowid=%d", windowid);
		maLog(mastr);
		return;
	}
	
	WindowGetCoordinates(windowid,
		&g_sets.mawin[windowid].xpos[g_sets.winconf], &g_sets.mawin[windowid].ypos[g_sets.winconf],
		&dummy, &dummy);
	
	return; 
	
} // DoWindowMoved


// this is called when a window has been resized
void DoWindowResized(int windowid)
{
#if defined(MAGUIGTK2)
	int wnew;
	int hnew;
#endif
	int dummy;
 	machar mastr[MAXLOGLINELEN];

	if(windowid < WINDOWFIRST || windowid > WINDOWLAST)
	{
		masprintf(mastr, MAXLOGLINELEN, L"DoWindowResized invalid windowid=%d", windowid);
		maLog(mastr);
		return;
	}

#if defined(MAGUIGTK2)
	// resize play position slider
	if(windowid == WINDOWAPP)
	{
		WindowGetCoordinates(windowid, &dummy, &dummy, &wnew, &hnew);
		if(g_sets.mawin[windowid].xsize[g_sets.winconf] != hnew)
			WindowWidgetSetSize(WINDOWAPP, ID_SLIDER_POSITION, wnew, hnew);
		g_sets.mawin[windowid].xsize[g_sets.winconf] = wnew;
		g_sets.mawin[windowid].ysize[g_sets.winconf] = hnew;
		return;
	}
#endif
	
	WindowGetCoordinates(windowid, &dummy, &dummy,
		&g_sets.mawin[windowid].xsize[g_sets.winconf], &g_sets.mawin[windowid].ysize[g_sets.winconf]);

#if defined(MAGUIWX) || defined(MAGUICARBON)
	if(windowid == WINDOWLIST)
		ListResize();
#endif
	
} // DoWindowResized


// g_sets xpos, ypos for skipwindowid have already been updated 
void WindowsMoveDelta(int skipwindowid, int xdelta, int ydelta)
{
	int windowid;

	for(windowid = WINDOWFIRST; windowid <= WINDOWLAST; windowid++)
	{
		if(skipwindowid != windowid)
		{
			g_sets.mawin[windowid].xpos[g_sets.winconf] += xdelta;
			g_sets.mawin[windowid].ypos[g_sets.winconf] += ydelta;
			if(g_w[windowid])
				WindowMove(windowid,
				 g_sets.mawin[windowid].xpos[g_sets.winconf], g_sets.mawin[windowid].ypos[g_sets.winconf]);
		}
	}
	
} // WindowsMoveDelta


void WindowMoveAndSetSize(int windowid, int xpos, int ypos, int xsize, int ysize)
{
	WindowMove(windowid, xpos, ypos);
	WindowSetSize(windowid, xsize, ysize);
}


void WindowsMoveAndSetSize()
{
	int windowid;
	
	for(windowid = WINDOWFIRST; windowid <= WINDOWLAST; windowid++)
	{
		if(g_w[windowid])
		{
			WindowMoveAndSetSize(windowid,
				g_sets.mawin[windowid].xpos[g_sets.winconf], g_sets.mawin[windowid].ypos[g_sets.winconf],
				g_sets.mawin[windowid].xsize[g_sets.winconf], g_sets.mawin[windowid].ysize[g_sets.winconf]);
		}
	}
	
} // WindowsMoveAndSetSize


machar g_abouttext[] =
#if defined(MAGUICARBON)
	L"\n"
#endif

	L" MoreAmp "
	MOREAMP_VERSION_STRING

	L" (unicode)  (C) 2004-2014 moreforge" MAEOL MAEOL

	L" Play aac, aif, fla, flac, m2a, m4a, mp1, mp2, mp3, mp4 sound, ogg, wav, wma, raw files," MAEOL
	L" Audio CD, and live audio input." MAEOL MAEOL

	L" Create aac, aif, flac, mp3, mp4, ogg, wav, raw files." MAEOL MAEOL

	L" On Mac OS X, Win32, Linux/Unix/GTK2." MAEOL MAEOL

	L" This program is free software which anyone can redistribute and/or modify under the terms of the" MAEOL
	L" GNU General Public License as published by the Free Software Foundation; either version 2 of the" MAEOL
	L" License, or (optionally) any later version." MAEOL MAEOL
	
	L" This program is distributed WITHOUT ANY WARRANTY, without even the implied warranty of" MAEOL
	L" MERCHANTABILITYor FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License" MAEOL
	L" for more details." MAEOL MAEOL
	
	L" This program should come with a copy of the GNU General Public License (the file named 'COPYING'" MAEOL
	L" if not, write to :" MAEOL
	L" The Free Software Foundation, Inc." MAEOL
	L" 59 Temple Place, Suite 330" MAEOL
	L" Boston, MA  02111-1307  USA" MAEOL MAEOL
	
	L" This program should also come with a file named 'COPYINGOggVorbis', a copy of the file named" MAEOL
	L" 'COPYING' in the public releases of libogg and libvorbis, Copyright (C) 2002, Xiph.org Foundation." MAEOL MAEOL

	L" This program uses all or parts of the following GNU GPL licensed or otherwise free software :" MAEOL

#if defined(MAGUIWX)
	L" audacity (extracts) gui buttons and sliders" MAEOL
#endif

#if !defined(MANOCDA)
 #if defined(__MAWIN__)
	L" cdex/cdrip (extracts) cda reader" MAEOL
 #endif
 #if defined(__MAUNIX__)
	L" cdparanoia (as released) cda reader" MAEOL
 #endif
#endif

	L" eq-xmms (extracts) equalizer" MAEOL

#if !defined(NODECODEFLAC) || defined(ENCODEFLAC)
	L" flac (as released) flac encoder/decoder" MAEOL
#endif

#if !defined(NODECODEWMA)
	L" ffmpeg (extracts) wma decoder" MAEOL
#endif

#if defined(ENCODEMP3)
	L" lame (as released) mp3 encoder" MAEOL
#endif

#if defined(ENCODEAAC) || defined(ENCODEMP4) 
	L" libfaac (as released) aac mp4 encoder" MAEOL
#endif

#if !defined(NODECODEAAC) || !defined(NODECODEMP4) 
	L" libfaad (as released) aac, m2a, m4a, mp4 decoder" MAEOL
#endif

#if !defined(NODECODEMP3)
	L" libid3tag (modified) id3 tag reader" MAEOL
#endif

#if defined(USE_LIBJACK)
 	L" libjack (as released) Jack Audio Connection Kit" MAEOL
#endif

#if !defined(NODECODEMP3)
 	L" libmad (as released) MPEG decoder" MAEOL
#endif

#if !defined(NODECODEOGG) || defined(ENCODEOGG) 
	L" libogg (as released) ogg encoder/decoder" MAEOL
#endif

#if !defined(NODECODEAIF) || !defined(NODECODEWAV) || defined(ENCODEAIF) || defined(ENCODEWAV)
	L" libsndfile (extracts) aif wav raw reader/writer" MAEOL
#endif

#if !defined(NODECODEOGG) || defined(ENCODEOGG) 
	L" libvorbis (as released) ogg encoder/decoder" MAEOL
#endif

	L" libz/zlib (as released) gz decompression" MAEOL

#if !defined(NODECODEAUDIOIN)
  	L" portaudio (as released) audio input/output" MAEOL
#else
 	L" portaudio (as released) audio output" MAEOL
#endif

#if defined(USE_LIBSOUNDTOUCH)
	L" libSoundTouch (as released) pitch/tempo/speed effects" MAEOL
#else
	L" libSoundTouch (extracts) pitch/tempo/speed effects" MAEOL
#endif

#if defined(USE_LIBSBSMS)
	L" libsbsms (as released) hq pitch/tempo effects" MAEOL
#endif

	L"                                                                " // 64 spaces for gui info
	; // valuable semi-colon here

#ifdef __cplusplus
}
#endif

// eof
