// maUtils.cpp

/*
 * MoreAmp
 * Copyright (C) 2004-2014 moreforge
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include "maConfig.h" // included by all moreamp files

#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include <stdlib.h>
#include <fcntl.h>
#include <time.h>
#include <signal.h>
#include <locale.h>
#if !defined(__MAWIN__)
 #include <xlocale.h>
#endif

#ifdef __MAOSX__
 #include <unistd.h>
 #include <sys/types.h>
 #include <inttypes.h>
 #include <wchar.h>
 #include <wctype.h>
 #include <arpa/inet.h> // for htonl
 #include <sys/stat.h>
 #include <semaphore.h>
 #include <MacTypes.h>
 #include <DriverServices.h>
 #define min(a,b) (a < b ? a : b)
 #include <dirent.h>
 #include <sys/time.h> // <time.h>
#endif

#if defined(__MAOSX__) || defined(__MAUNIX__) // HAVE_ICONV
 #if !defined(HAVE_ICONV)
  #define HAVE_ICONV 1 // we always have it...
 #endif
 #if defined(HAVE_ICONV)
  #include <assert.h>
  #include <errno.h>
  #include <langinfo.h>
  #include <iconv.h>
  #include <stdlib.h>
  #include <string.h>

  // ICONV_CONST is defined in maConfigMacOSX.h or maConfigUnix.h by configure
  // depending on the prototype for iconv in the iconv.h that configure finds
  // (const char or plain char). There can be different iconv.h's all over the system,
  // in /usr/include, /usr/local/include, the various mac SDK's (10.4 10.5 10.6).
  // It is redefined below for xcode and wx builds because the one in maConfigMacOSX.h or
  // maConfigUnix.h (from a cli build) may not match be one in the mac SDK (xcode), or the
  // one in setup.h depending what iconv.h wx found when it was built.
  
  #if defined(__MAOSX__) || defined(__MACUNIX__)
   #include <AvailabilityMacros.h>
  #endif

  #if defined(MAC_OS_X_VERSION_MAX_ALLOWED) && (MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4)
   #if defined(__XCODE__)       // xcode uses iconv in SDK
    #if defined(ICONV_CONST)    // also defined in mac wx setup.h
     #undef ICONV_CONST
    #endif
    #if defined(MAGUIWX)        // xcode wx
     #define ICONV_CONST const
    #endif
    #if defined(MAGUIGTK2)      // xcode gtk2
     #define ICONV_CONST const
    #endif
    #if defined(MAGUINONE)      // xcode nogui
     #define ICONV_CONST const
    #endif
    #if defined(MAGUICARBON)    // xcode carbon
     #define ICONV_CONST const
    #endif
   #else
    #if defined(MAGUIWX)        // cli wx
     #if defined(ICONV_CONST)   // defined in mac wx setup.h
      #undef ICONV_CONST
     #endif
     #define ICONV_CONST const
    #endif
    /*
    #if defined(MAGUIGTK2)      // cli gtk2
     #define ICONV_CONST const
    #endif
    #if defined(MAGUINONE)      // cli nogui
      #define ICONV_CONST const
    #endif
    #if defined(MAGUICARBON)    // cli carbon
     #define ICONV_CONST const
    #endif
    */
   #endif
  #else
   #if defined(MAC_OS_X_VERSION_MAX_ALLOWED) && (MAC_OS_X_VERSION_MAX_ALLOWED > MAC_OS_X_VERSION_10_4) // 10.6
    #if defined(__XCODE__)      // xcode uses iconv in SDK
     #if defined(ICONV_CONST)   // also defined in mac wx setup.h
      #undef ICONV_CONST
     #endif
     #if defined(MAGUIWX)       // xcode wx
      #define ICONV_CONST
     #endif
     #if defined(MAGUIGTK2)     // xcode gtk2
      #define ICONV_CONST const
     #endif
     #if defined(MAGUINONE)     // xcode nogui
      #define ICONV_CONST
     #endif
     #if defined(MAGUICARBON)   // xcode carbon
      #define ICONV_CONST
     #endif
    #else
     #if defined(MAGUIWX)       // cli wx
      #if defined(ICONV_CONST)   // defined in mac wx setup.h
       #undef ICONV_CONST
      #endif
      #define ICONV_CONST const
     #endif
     /*
     #if defined(MAGUIGTK2)     // cli gtk2
      #define ICONV_CONST const
     #endif
     #if defined(MAGUINONE)     // cli nogui
      // #define ICONV_CONST const // different in /usr and /usr/local
     #endif
     #if defined(MAGUICARBON)   // cli carbon
      // #define ICONV_CONST const // different in /usr and /usr/local
     #endif
     */
    #endif
   /*
   #else                        // not mac (we don't have MAC_OS_X_VERSION_MAX_ALLOWED)
    #if !defined(ICONV_CONST)
     #define ICONV_CONST
    #endif
    */
   #endif
  #endif // 10.6
 #endif
 
 #if !defined(ICONV_CONST)
  #define ICONV_CONST
 #endif
#endif

#if defined(__MAUNIX__) && !defined(__MACUNIX__)
 #if !defined(HAVE_LANGINFO_CODESET)
  #define HAVE_LANGINFO_CODESET 1 // locale_charset is in libgettextlib, and in old libiconv
 #endif
#endif

#if defined(__MAOSX__) || defined(__MACUNIX__)
 #include <MacApplication.h> 
#endif

#ifdef __MAUNIX__
 #include <unistd.h>
 #include <sys/types.h>
 #include <inttypes.h>
 #include <wchar.h>
 #include <wctype.h>
 #include <arpa/inet.h> // for htonl
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <semaphore.h>
 #include <pthread.h>
 #include <sched.h>
 #define min(a,b) (a < b ? a : b)
 #include <dirent.h>
#endif

// posix with sys V
#if defined(__MAOSX__) || defined(__MACUNIX__)
 #include <sys/sem.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <time.h>
 #include <pthread.h>
#endif

#ifdef __MAWIN__
 #include <io.h>
 #include <sys/types.h>
 #include <wchar.h>
 #include <sys/stat.h>
 #include <process.h>
 #include <SHLOBJ.H>
 #include <SHLWAPI.H>
 #include <DIRECT.H>

 #include <WINUSER.H>
 #include <windows.h>
 #include <WINPERF.H>

 #include <sys/timeb.h>
#endif

#include "maTypes.h"
#include "maMemFile.h"

 // shared mem
#if defined(MAUSESHM)
 #if defined(__MAWIN__)
  typedef struct
  {
	char *addr;
	size_t size;
	int alone;
	HANDLE file;
	HANDLE map;
	LPVOID view;
  } MASHM;
 #endif
 #if defined(__MAUNIX__) || defined(__MAOSX__)
  #include <sys/ipc.h>
  #include <sys/shm.h> 
  typedef struct
  {
	char *addr;
	int size;
	int alone;
	key_t key;
	int shmid;
  } MASHM;
 #endif
#endif

#include "maDefs.h"
#include "maGUI.h"
#include "maPlayer.h"
#include "maCda.h"
#include "maUtils.h"

#ifdef __cplusplus
 extern "C"
 {
#endif

// posix with sys V
#if defined(__MAOSX__) || defined(__MACUNIX__)
 int masem_init(int *semid, int pshared, unsigned int value);
 int masem_wait(int *semid);
 int masem_post(int *semid); 
 int masem_destroy(int *semid);
 int masem_setvalue(int *semid, int value);
 int mainitsem(key_t key, int nsems);
#endif

int magetpid()
{
#ifdef __MAWIN__
	return GetCurrentProcessId();
#endif

#if defined(__MAOSX__) || defined(__MAUNIX__)
	return getpid();
#endif

} // magetpid


// return the number and a pid array of 'moreamp' processes running
// or if pid[0] whether or not 'moreamp' process pid[0] is running
int maNumProcessesWin(int *pid);
int maNumProcesses(int *pid)
{
 #if defined(__MAUNIX__) && !defined(__MACUNIX__)
	int ret;
	char cmd[STRLEN128];
	machar mastr[MAXLOGLINELEN];
	// we do this in the shell and we don't return an aray of pids
	if(pid && !*pid)
		return 0;
	if(pid && *pid)
		sprintf(cmd, "exit `ps xc -opid,command | fgrep %d | fgrep %s | fgrep -v fgrep | wc -l`", 
																		*pid, MOREAMP_NAMEos);
	else
		sprintf(cmd, "exit `ps xc -opid,command | fgrep %s | fgrep -v fgrep | wc -l`",
																		MOREAMP_NAMEos);
	if(((ret = system(cmd)) & 0xff) != 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maNumProcesses system ret=0x%x errno=%d", ret, errno);
		ShowErr(mastr);
		return 0;
	}
	return ret >> 8;
#endif
 
#if defined(__MAOSX__) || defined(__MACUNIX__)
	int ret;
	char cmd[STRLEN128];
	machar mastr[MAXLOGLINELEN];
	// we do this in the shell and we don't return an array of pids
	if(pid && !*pid)
		return 0;
	if(pid && *pid)
		ossprintf(cmd, STRLEN128, "exit `ps xc -opid,command | fgrep %d | fgrep -i %s | fgrep -v fgrep | wc -l`", 
														*pid, MOREAMP_NAMEos);
	else
		ossprintf(cmd, STRLEN128, "exit `ps xc -opid,command | fgrep -i %s | fgrep -v fgrep | wc -l`",
															MOREAMP_NAMEos);
	if(((ret = system((char *)cmd)) & 0xff) != 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maNumProcesses system ret=0x%x errno=%d", ret, errno);
		ShowErr(mastr);
		return 0;
	}
	return ret >> 8;
#endif
 
#if defined(__MAWIN__)
	// not implemented on win9x
	if(maIsWin9x())
		return (pid && *pid) ? 1 : 0;
	return maNumProcessesWin(pid);
#endif 
} // maNumProcesses


void ShowErr(const machar *msg)
{
	maMessageBox((machar *)msg, (machar *)L"", maOK | maICON_EXCLAMATION);
		
} // ShowErr


// dbsliders (eq and preamp) range from -DBRANGE to +DBRANGE
double SliderDBFromPos(int value)
{
	if(value == ((DBSLIDERMAX - DBSLIDERMIN) / 2))
		return 0.0; // instead of the nearest non-zero db
		
	if(value >= DBSLIDERMAX)
		return DBRANGE;
		
	if(value <= DBSLIDERMIN)
		return -DBRANGE;

	return ((2. * DBRANGE * value) / (DBSLIDERMAX - DBSLIDERMIN)) - DBRANGE;

} // SliderDBFromPos


int SliderPosFromDB(double db)
{
	if(db <= -DBRANGE)
		return DBSLIDERMIN;
		
	if(db >= DBRANGE)
		return DBSLIDERMAX;

	return (int)(LROUND(((db + DBRANGE) * (DBSLIDERMAX - DBSLIDERMIN)) / (2. * DBRANGE)));
	 
} // SliderPosFromDB


int SongIsInMemory(fschar *filepath)
{
	int i;
	
	if(!g_memhandle || g_loadedmemsize <= 0)
		return 0;
		
	if(g_sets.buffertype != BUFUSEMEM)
		return 0;
		
	for(i = 0; i < g_loadednum; i++)
	{
		if(!fsstrcasecmp(filepath, g_loadedpath[i]))
			return 1;
	}
		
	return 0;
	
} // SongIsInMemory


int PlaylistIsNoRecords() // playlist that don't use record files
{
	if(PlaylistIsCda() || PlaylistIsDrop()
#if !defined(NODECODEAUDIOIN)
						|| PlaylistIsAudioIn()
#endif
		)
		return 1;
		
	return 0;
	
} // PlaylistIsNoRecords


int PlaylistIsCda()
{
	if(!fsstrcasecmp(g_playlistcda, g_playlistpath))
		return 1;
		
	return 0;
	
} // PlaylistIsCda


int PlaylistIsDrop()
{
	if(!fsstrcasecmp(g_playlistdrop, g_playlistpath))
		return 1;
		
	return 0;
	
} // PlaylistIsDrop


#if !defined(NODECODEAUDIOIN)
 int PlaylistIsAudioIn()
 {
	if(!fsstrcasecmp(g_playlistaudioin, g_playlistpath))
		return 1;
		
	return 0;
	
 } // PlaylistIsAudioIn
#endif


int SongFileExists(fschar *songfile)
{
		if(FileNameHasExtensionPlayable(songfile))
			return maFileExists(songfile);
#ifndef MANOCDA
		// maybe cda
		return IsAudioCd(songfile);
#else
		return 0;
#endif
	
} // SongFileExists 


int PathnameFromFilename(fschar *filename, fschar *pathname) // e.g. .../dir/name.ext -> .../dir
{
	int l;
	fschar *p;

	if(!filename || (l = (int)fsstrlen(filename)) == 0)
		return 1;

	fsstrcpy(pathname, filename);
	p = pathname + l;
	while(l-- > 0)
	{
		p--;
		if(*p == FILESEP)
		{
			*p = fsT('\0');
			return 0;
		}
	}
	
	pathname[0] = fsT('\0');
	return 1;

} // PathnameFromFilename


int FilenameFromPathname(fschar *pathname, fschar *filename) // e.g. .../dir/name.ext -> name.ext
{
	size_t len;
	fschar *p;
	fschar name[MAX_PATH];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];
	
	if(!pathname || (len = fsstrlen(pathname)) == 0)
		return 1;
		
	if(len >= MAX_PATH)
	{
		masprintf(mastr, MAXLOGLINELEN, L"FilenameFromPathname file path > max %d : %zu", MAX_PATH - 1, len);
		maLog(mastr);
		fsstrncpy(name, pathname, MAX_PATH - 1);
		name[MAX_PATH - 1] = fsT('\0');
		maLog(fs2ma(mastr2, name));
		return 1;
	}

	p = pathname + len;
	while(len-- > 0)
	{
		p--;
		if(*p == FILESEP)
		{
			p++;
			fsstrcpy(filename, p);
			return 0;
		}
	}
	
	// no file separator, already a filename
	fsstrcpy(filename, pathname);
	
	return 0;

} // FilenameFromPathname


int FileNameRemoveExtension(fschar *filename)
{
	int l;
	fschar *p;
	
	if(!filename || (l = (int)fsstrlen(filename)) == 0)
		return 0;

	p = filename + l;
	while(l-- > 0)
	{
		p--;
		if(*p == fsT('.'))
		{
			*p = fsT('\0');
			return 1; // ext was removed
		}
	}
	
	return 1;

} // FileNameRemoveExtension


int FileNameHasExtension(fschar *fname, const fschar *ext)
{
	int i, l, lx;	
	machar lext[16];
	machar lfname[MAX_PATH];

	if(!fname || !ext || (fname[0] == fsT('\0')) || (ext[0] == fsT('\0')) || 
							fsstrlen((fschar *)ext) >= fsstrlen((fschar *)fname))
		return 0;
	lx = (int)mastrlen(fs2mafn(lext, (fschar *)ext));
	l = (int)mastrlen(fs2mafn(lfname, (fschar *)fname));
	for(i = 0, l -= lx; i < lx; i++)
	{
		if(matolower(lfname[l++]) != matolower(lext[i]))
			return 0;
	}

	return 1;

} // FileNameHasExtension


int FileNameHasExtensionPlayable(fschar *fname)
{
	int i;

	for(i = kMAFIRSTPLAYBLE; i <= kMALASTPLAYBLE; i++) // not kMACDA
	{
#ifdef NODECODEMP3
		if(i == kMAMP1) continue;
		if(i == kMAMP2) continue;
		if(i == kMAMP3) continue;
#endif

#ifdef NODECODEOGG
		if(i == kMAOGG) continue;
#endif

#ifdef NODECODEFLAC
		if(i == kMAFLAC) continue;
		if(i == kMAFLA) continue;
#endif

#ifdef NODECODEAIF
		if(i == kMAAIF) continue;
		if(i == kMAAIFF) continue;
#endif

#ifdef NODECODEWAV
		if(i == kMAWAV) continue;
#endif

#ifdef NODECODEAAC
		if(i == kMAAAC) continue;
		if(i == kMAM2A) continue;
#endif

#ifdef NODECODEMP4
		if(i == kMAMP4) continue;
		if(i == kMAM4A) continue;
#endif

#ifdef NODECODEWMA
		if(i == kMAWMA) continue;
#endif

#if !defined(NODECODEAUDIOIN)
#else
		if(i == kMAMAA) continue;
#endif

		if(FileNameHasExtension(fname, g_extstr[i]))
			return 1;
	}
	return 0;
		
} // FileNameHasExtensionPlayable


int FileIsPlayable(fschar *filepath)
{
	const machar *err = NULL;
	fschar *lfilepath = filepath;
#if defined(__MAOSX__) || defined(__MACUNIX__)
	fschar name[MAX_PATH];
#endif
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];
	
	// check extension playable
	if(!FileNameHasExtensionPlayable(lfilepath) && !FileNameHasExtension(lfilepath, g_extstr[kMACDA]))
	{
#if defined(__MAOSX__) || defined(__MACUNIX__)
		// skip the file if it is named '.DS_Store' or 'Icon^M' or 'Icon' (mac invisible)
		if(!FilenameFromPathname(lfilepath, name))
			if(!fsstrcmp(name, fsT(".DS_Store")) || !fsstrcmp(name, fsT("Icon\015")) || !fsstrcmp(name,  fsT("Icon")))
				goto noplay;
#endif

#ifndef MANOCDA
		if(!IsAudioCd(lfilepath))
		{
			if(!(g_sets.logflags & kMALOGOFF) && (g_sets.logflags & kMALOGNOTPLAYABLE))
				err = L"not a playable file or on an Audio CD";
			goto noplay;
		}
#else
		if(!(g_sets.logflags & kMALOGOFF) && (g_sets.logflags & kMALOGNOTPLAYABLE))
			err = L"not a playable file";
		goto noplay;
#endif
	}
			
	return 1; // playable
	
noplay:

	if(err)
	{
		masprintf(mastr, MAXLOGLINELEN, L"FileIsPlayable %ls '%ls'", err,
																fs2ma(mastr2, filepath));
		maLog(mastr);
	}
	
	return 0;

} // FileIsPlayable
	

int maCreateDir(fschar *dirname)
{
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif

	if(!dirname || (dirname[0] == fsT('\0')))
	{
		maLog(L"maCreateDir null dirname");
		return 1;
	}
 
	if(mamkdir(dirname))
	{
		machar mastr[MAXLOGLINELEN];
		masprintf(mastr, MAXLOGLINELEN, L"maCreateDir failed '%ls'", fs2ma(mastr2, dirname));
		maLog(mastr);
		return 1;
	}

#if defined(__MAOSX__) || defined(__MAUNIX__)
	machmod(dirname, ACCESSPERMS); // 0777
#endif

	return(0);

} // maCreateDir


int maFileExists(fschar *filepath)
{
    struct fsstatstruct sb;

	if(!filepath || (filepath[0] == fsT('\0')))
	{
		maLog(L"maFileExists null filepath");
		return 0;
	}
	
	return fsstat(filepath, &sb) == 0 && (sb.st_mode & S_IFREG);
		
} // maFileExists


int maDirExists(fschar *dirpath)
{
    struct fsstatstruct sb;

	if(!dirpath || (dirpath[0] == fsT('\0')))
	{
		maLog(L"maDirExists null dirname");
		return 0;
	}
	
    return fsstat(dirpath, &sb) == 0 && (sb.st_mode & S_IFDIR);
	
} // maDirExists


int maRenameDir(fschar *fromdirpath, fschar *todirpath)
{
    int ret;
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
	machar mastr3[MAXLOGLINELEN];
#endif
	machar mastr[MAXLOGLINELEN];

	if(!fromdirpath || (fromdirpath[0] == fsT('\0')))
	{
		maLog(L"maRenameDir null fromdirpath");
		return 1;
	}
	
	if(!todirpath || (todirpath[0] == fsT('\0')))
	{
		maLog(L"maRenameDir null todirpath");
		return 1;
	}
	
#if defined(__MAWIN__)
    ret = _wrename(fs2ma(mastr2, fromdirpath), fs2ma(mastr3, todirpath));
#else
    ret = rename((const char *)fromdirpath, (const char *)todirpath);
#endif
	
	if(ret == 0)
		return 0; // ok
	
	masprintf(mastr, MAXLOGLINELEN, L"maRenameDir '%ls' to '%ls' failed, errno=%d",
					fs2ma(mastr2, fromdirpath), fs2ma(mastr3, todirpath), errno);
	maLog(mastr);
	
	return 1; // failed

} // maRenameDir


// maClearDir is only called from FillVirtualDisk, deletes files in dir, not in subdirs
#if defined(__MAOSX__) || defined(__MAUNIX__)
int maClearDir(fschar *dirpath)
{
	DIR *dir;
	struct dirent *dt;
	fschar file[MAX_PATH];
	machar mastr2[MAXLOGLINELEN];	
	machar mastr[MAXLOGLINELEN];
	
	if(!dirpath || (dirpath[0] == fsT('\0')))
	{
		maLog(L"maClearDir null dirpath");
		return 1;
	}
	
	// be safe
	if(!fsstrstr(dirpath, (fschar *)MAVDISKTMP))
	{
		masprintf(mastr, MAXLOGLINELEN, L"maClearDir '%ls' not in '%ls'", MAVDISKTMP,
																				fs2ma(mastr2, dirpath));
		maLog(mastr);
		return 1;
	}

	fschar *ldirname = dirpath;
	
	if((dir = opendir((const char *)ldirname)) == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maClearDir opendir failed on '%ls'", fs2ma(mastr2,dirpath));
		maLog(mastr);
		return 0;
	}
	
	while((dt = readdir(dir)))
	{
 		if(fsstrcmp((fschar *)dt->d_name, fsT(".")) && fsstrcmp((fschar *)dt->d_name, fsT("..")))
		{
			fssprintf(file, MAX_PATH, fsS fsC fsS, ldirname, FILESEP, (fschar *)dt->d_name);
			if(unlink((char *)file))
			{
				masprintf(mastr, MAXLOGLINELEN, L"maClearDir maunlink errno=%d '%ls'", errno,
																				fs2ma(mastr2, file));
				maLog(mastr);
			}		
		}
	}
	
	if(closedir(dir) == -1)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maClearDir closedir errno=%d on '%ls'", errno,
																				fs2ma(mastr2,dirpath));
		maLog(mastr);
	}

	return 0;

} // maClearDir (__MAOSX__) || (__MAUNIX__)
#endif


#if defined(__MAWIN__)
 int maClearDir(fschar *dirpath)
 {
	HANDLE hFind;
	WIN32_FIND_DATA FindData;
	fschar dirwild[MAX_PATH];
	fschar file[MAX_PATH];
	machar mastr[MAXLOGLINELEN];

	if(!dirpath || (dirpath[0] == fsT('\0')))
	{
		maLog(L"maClearDir null dirpath");
		return 1;
	}
	
	// be safe
	if(!fsstrstr(dirpath, MAVDISKTMP))
	{
		masprintf(mastr, MAXLOGLINELEN, L"maClearDir '%ls' not in '%ls'", MAVDISKTMP,
																				fs2ma(mastr2,dirpath));
		maLog(mastr);
		return 1;
	}

	fssprintf(dirwild, MAX_PATH, fsS fsC fsT("*"), fs2ma(mastr2, dirpath), FILESEP);
	hFind = FindFirstFile(dirwild, &FindData);
	if(hFind == INVALID_HANDLE_VALUE)
		return 1;
	for (; ;)
	{
		if(fsstrcmp(FindData.cFileName, fsT(".")) && fsstrcmp(FindData.cFileName, fsT("..")))
		{
			fssprintf(file, MAX_PATH, fsS fsC fsS, dirpath, FILESEP, FindData.cFileName);
			if(_wunlink(file))
			{
				masprintf(mastr, MAXLOGLINELEN, L"maClearDir unlink errno=%d '%ls'", errno,
																					fs2ma(mastr2, file));
				maLog(mastr);
			}		
		}
		if(!FindNextFile(hFind, &FindData))
			break;
	}
	FindClose(hFind);

	return 0;

 } // maClearDir __MAWIN__
#endif


int maFileCopy(fschar *frompath, fschar *topath, int concat)
{  
	int err = 0;
	FILE *fpfrom, *fpto;
	size_t bytesread;
	fschar *mode;
	machar buf[STRLEN512];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];
	
	if(!frompath || (frompath[0] == fsT('\0')))
	{
		maLog(L"maFileCopy null frompath");
		return 1;
	}
	
	if(!topath || (topath[0] == fsT('\0')))
	{
		maLog(L"maFileCopy null topath");
		return 1;
	}
	
	if((fpfrom = mafopen(frompath, (fschar *)fsT("rb"))) == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maFileCopy fopen errno=%d '%ls'", errno, fs2ma(mastr2, frompath));
		maLog(mastr);
		return 1; // not dskFulErr
	}

	if(concat)
		mode = (fschar *)fsT("ab"); // "ab" or "a" ?
	else
		mode = (fschar *)fsT("wb");
	if((fpto = mafopen(topath, mode)) == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maFileCopy fopen errno=%d '%ls'", errno, fs2ma(mastr2, topath));
		maLog(mastr);
		if(fclose(fpfrom) != 0)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maFileCopy fclose errno=%d '%ls'", errno, fs2ma(mastr2, frompath));
			maLog(mastr);
		}
		return 1;
	}

	if(concat)
	{
		if(fseek(fpto, 0, SEEK_END) != 0)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maFileConcat fseek errno=%d '%ls'", errno, fs2ma(mastr2, topath));
			maLog(mastr);
			if(fclose(fpfrom) != 0)
			{
				masprintf(mastr, MAXLOGLINELEN, L"maFileCopy fclose errno=%d '%ls'", errno, fs2ma(mastr2, frompath));
				maLog(mastr);
			}
			if(fclose(fpto) != 0)
			{
				masprintf(mastr, MAXLOGLINELEN, L"maFileCopy fclose errno=%d '%ls'", errno, fs2ma(mastr2, topath));
				maLog(mastr);
			}
			return 1;
		}
	}
	
	errno = 0;
	while((bytesread = fread(buf, 1, sizeof(buf), fpfrom)) > 0)
	{
		if(fwrite(buf, 1, bytesread, fpto) != bytesread)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maFileCopy fwrite errno=%d '%ls'", errno, fs2ma(mastr2, topath));
			maLog(mastr);
			err = 1;
			break;
		}
	}

	if(bytesread <= 0)
	{
		if(!feof(fpfrom))
		{
			masprintf(mastr, MAXLOGLINELEN, L"maFileCopy fread errno=%d '%ls'", errno, fs2ma(mastr2, frompath));
			maLog(mastr);
			err = 2;
		}
	}
	
	if(fclose(fpfrom) != 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maFileCopy fclose errno=%d '%ls'", errno, fs2ma(mastr2, frompath));
		maLog(mastr);
		err = 3;
	}
	
	if(fclose(fpto) != 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maFileCopy fclose errno=%d '%ls'", errno, fs2ma(mastr2, topath));
		maLog(mastr);
		err = 4;
	}

#if defined(__MAOSX__) || defined(__MAUNIX__)
	if(machmod(topath, ACCESSPERMS) == -1) // 0777
	{
		masprintf(mastr, MAXLOGLINELEN, L"maFileCopy machmod errno=%d '%ls'", errno, fs2ma(mastr2, topath));
		maLog(mastr);
		err = 5;
	}
#endif
	
	return err;
	
} // maFileCopy


int maFileConcat(fschar *frompath, fschar *topath)
{  
	int err = 0;
	FILE *fpfrom, *fpto;
	size_t bytesread;
	fschar buf[STRLEN512];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];
	
	if(!frompath || (frompath[0] == fsT('\0')))
	{
		maLog(L"maFileConcat null frompath");
		return 1;
	}
	
	if(!topath || (topath[0] == fsT('\0')))
	{
		maLog(L"maFileConcat null topath");
		return 1;
	}
	
	if((fpfrom = mafopen(frompath, (fschar *)fsT("rb"))) == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maFileConcat fopen errno=%d '%ls'", errno, fs2ma(mastr2, frompath));
		maLog(mastr);
		return 1;
	}

	if((fpto = mafopen(topath, (fschar *)fsT("wb"))) == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maFileConcat fopen errno=%d '%ls'", errno, fs2ma(mastr2, topath));
		maLog(mastr);
		if(fclose(fpfrom) != 0)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maFileConcat fclose errno=%d '%ls'", errno, fs2ma(mastr2, frompath));
			maLog(mastr);
		}
		return 1;
	}

	if(fseek(fpto, 0, SEEK_END) != 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maFileConcat fseek errno=%d '%ls'", errno, fs2ma(mastr2, topath));
		maLog(mastr);
		return 1;
	}
	
	errno = 0;
	while((bytesread = fread(buf, 1, sizeof(buf), fpfrom)) > 0)
	{
		if(fwrite(buf, 1, bytesread, fpto) != bytesread)
		{
			masprintf(mastr, MAXLOGLINELEN, L"maFileConcat fwrite errno=%d '%ls'", errno, fs2ma(mastr2, topath));
			maLog(mastr);
			err = 1;
			break;
		}
	}

	if(bytesread <= 0)
	{
		if(!feof(fpfrom))
		{
			masprintf(mastr, MAXLOGLINELEN, L"maFileConcat fread errno=%d '%ls'", errno, fs2ma(mastr2, frompath));
			maLog(mastr);
			err = 2;
		}
	}
	
	if(fclose(fpfrom) != 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maFileConcat fclose errno=%d '%ls'", errno, fs2ma(mastr2, frompath));
		maLog(mastr);
		err = 3;
	}
	
	if(fclose(fpto) != 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maFileConcat fclose errno=%d '%ls'", errno, fs2ma(mastr2, topath));
		maLog(mastr);
		err = 4;
	}

#if defined(__MAOSX__) || defined(__MAUNIX__)
	if(machmod(topath, ACCESSPERMS) == -1) // 0777
	{
		masprintf(mastr, MAXLOGLINELEN, L"maFileConcat machmod errno=%d '%ls'", errno, fs2ma(mastr2, topath));
		maLog(mastr);
		err = 5;
	}
#endif
	
	return err;
	
} // maFileConcat


// returns 1-based drive index if fname is a drive or a track
// and the drive is ready, else 0
#if defined(__MAUNIX__)
 int CdIsInited = 0;
#endif
int IsAudioCd(fschar *fname)
{
#ifdef MANOCDA
	return 0;
#else
	int i;
 #if !defined(__MAUNIX__) || defined(MACDAOSX)
	fschar *p;
	fschar name[MAX_PATH];
	fschar cdaname[MAX_PATH];
 #endif
	
	if(!fname || !fname[0])
	{
		maLog(L"IsAudioCd null fname");
		return 0;	
	}

 #if defined(__MAOSX__) || defined(MACDAOSX) // on osx ignore first '/'
	if(fname[0] == VOLSEP)
		fsstrcpy(name, &fname[1]);
	else
		fsstrcpy(name, fname);
	if((p = fsstrchr(name, FILESEP)))
		*p = fsT('\0');
	for(i = 1; i <= g_cdanumdrives; i++)
	{
		fsstrcpy(cdaname, g_cdadrive[i].name);
		if((p = fsstrchr(cdaname, FILESEP)))
			*p = fsT('\0');
		if(!fsstrcmp(name, cdaname))
			if(g_cdadrive[i].isready)
				return i;
	}
	return 0;
 #elif defined(__MAUNIX__)
  #if !defined(MACDAOSX)
	 if(!CdIsInited)
	{
		CdIsInited = 1;
		if(g_cdanumdrives < 1)
		{
			LoadSettingsCDRom();
			CdaInitCDRom();
		}
	}
	for(i = 1; i <= g_cdanumdrives; i++)
	{
		if(fsstrstr(fname, g_cdadrive[i].name) == fname)
			if(g_cdadrive[i].isready)
				return i;
	}
	return 0;
  #endif
 #else // __MAWIN__
	fsstrcpy(name, fname);
	if((p = fsstrchr(name, FILESEP)) != 0)
		*p = fsT('\0');
	for(i = 1; i <= g_cdanumdrives; i++)
	{
		fsstrcpy(cdaname, g_cdadrive[i].name);
		if((p = fsstrchr(cdaname, FILESEP)) != 0)
			*p = fsT('\0');
		if(!fsstrcmp(name, cdaname))
			if(g_cdadrive[i].isready)
				return i;
	}
	return 0;
 #endif
#endif
} // IsAudioCd


machar *SecondsToStrHMS(machar *strhms, int xs)
{
	int h, m, s = xs;

	if(strhms)
	{
		if(xs >= 0)
		{
			h = s / SECSPERHOUR;
			s -= h * SECSPERHOUR;
			m = s / SECSPERMIN;
			s -= m * SECSPERMIN;

			if(h > 0)
				masprintf(strhms, STRLEN32, L"%d:%02d:%02d", h, m, s);
			else
				masprintf(strhms, STRLEN32, L"%d:%02d", m, s);
		}
		else
			masprintf(strhms, STRLEN32, L"?");
	}

	return strhms;
	
} // SecondsToStrHMS


machar *MillisecondsToStrHMS(machar *strhms, mauint64 xms)
{
	mauint64 h, m, s, ms = xms + 500; // round to second
	
	if(strhms)
	{
		h = ms / MSPERHOUR;
		if(h)
			ms -= h * MSPERHOUR;
		m = ms / MSPERMIN;
		if(m)
			ms -= m * MSPERMIN;
		s = ms / MSPERSEC;
		masprintf(strhms, STRLEN32, L"%02d:%02d:%02d", (int)h, (int)m, (int)s);
	}

	return strhms;
	
} // MillisecondsToStrHMS


machar *MillisecondsToStrHMSD(machar *strhms, mauint64 xms)
{
	mauint64 h, m, s, d, ms = xms + 50; // round to tenth of second
	
	if(strhms)
	{
		h = ms / MSPERHOUR;
		if(h)
			ms -= h * MSPERHOUR;
		m = ms / MSPERMIN;
		if(m)
			ms -= m * MSPERMIN;		
		s = ms / MSPERSEC;
		if(s)
			ms -= s * MSPERSEC;
		d = ms / MSPERDS; 
		masprintf(strhms, STRLEN32, L"%02d:%02d:%02d.%d", (int)h, (int)m, (int)s, (int)d);
	}

	return strhms;
	
} // MillisecondsToStrHMSD


int StrHMSToMilliSeconds(machar *strhms)
{
	machar lstr[32];

	if(mastrlen(strhms) != mastrlen(ZEROMSSTR)) // "00:00:00.0"
		return -1;
	
	if(!isdigit(strhms[ 0])) return -1; // hours
	if(!isdigit(strhms[ 1])) return -1;
	if(strhms[2] != ':') return -1;
	
	if(!isdigit(strhms[ 3])) return -1; // minutes
	if(!isdigit(strhms[ 4])) return -1;
	if(strhms[5] != ':') return -1;
	
	if(!isdigit(strhms[ 6])) return -1; // seconds
	if(!isdigit(strhms[ 7])) return -1;
	if(strhms[8] != L'.') return -1;
	
	if(!isdigit(strhms[ 9])) return -1; // milliseconds
	
	mastrcpy(lstr, strhms);
	lstr[2] = lstr[5] = lstr[8] = 0;
	
	return maatoi(lstr)     * MSPERHOUR + maatoi(&lstr[3]) * MSPERMIN +
		   maatoi(&lstr[6]) * MSPERSEC  + maatoi(&lstr[9]) * MSPERDS;

} // StrHMSToMilliSeconds


FILE *mafopen(fschar *filename, fschar *mode)
{
	FILE *ret;
	fschar name[MAX_PATH];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];
	
	if(fsstrlen(filename) >= MAX_PATH)
	{
		masprintf(mastr, MAXLOGLINELEN, L"mafopen file path > max %d : %zu", MAX_PATH - 1, fsstrlen(filename));
		maLog(mastr);
		fsstrncpy(name, filename, MAX_PATH - 1);
		name[MAX_PATH - 1] = fsT('\0');
		maLog(fs2ma(mastr2, name));
		return NULL;
	}
	
 	ret = fsfopen(filename, mode);
	if(ret == NULL && errno != ENOENT) // not No such file
	{
		masprintf(mastr, MAXLOGLINELEN, L"mafopen fopen failed errno=%d", errno);
		maLog(mastr);
		fsstrncpy(name, filename, MAX_PATH - 1);
		name[MAX_PATH - 1] = fsT('\0');
		maLog(fs2ma(mastr2, name));
	}

	return ret;
	
} // mafopen


#if defined(ENCODEMP4) || defined(ENCODEFLAC)
 FILE *mafopenansi(char *filename, char *mode)
 {
	FILE *ret;
	char name[MAX_PATH];
	machar mastr2[MAXLOGLINELEN];
	machar mastr[MAXLOGLINELEN];
	
	if(strlen(filename) >= MAX_PATH)
	{
		masprintf(mastr, MAXLOGLINELEN, L"mafopenansi file path > max %d : %zu", MAX_PATH - 1, strlen(filename));
		maLog(mastr);
		strncpy(name, filename, MAX_PATH - 1);
		name[MAX_PATH - 1] = '\0';
		maLog(os2ma(mastr2, name));
		return NULL;
	}
	
 #if defined(__MAOSX__) || defined(__MACUNIX__)
	char filenamebsd[MAX_PATH];
	strcpy(filenamebsd, filename);
	StrCHARSETtoUTF8MAC(filenamebsd);
	ret = fopen(filenamebsd, mode);
 #else
 	ret = fopen(filename, mode);
 #endif

	if(ret == NULL && errno != ENOENT) // not No such file
	{
		masprintf(mastr, MAXLOGLINELEN, L"mafopenansi fopen failed errno=%d", errno);
		maLog(mastr);
		strncpy(name, filename, MAX_PATH - 1);
		name[MAX_PATH - 1] = '\0';
		maLog(os2ma(mastr2, name));
	}

	return ret;
	
 } // mafopenansi
#endif	


#if defined(__MAOSX__) || defined(__MAUNIX__)
 int machmod(fschar *filename, int mode)
 {
	return chmod(filename, mode);
 }
#endif


int mamkdir(fschar *filename)
{
#if defined(__MAOSX__)  
	return mkdir((const char *)filename, 0);
#else
 #if defined(__MAUNIX__)
	return mkdir(filename, 0);
 #else // __MAWIN__
	return !CreateDirectory(filename, 0);
 #endif
#endif
}


int maunlink(fschar *filename)
{
	if(fsunlink(filename))
		return errno;

	return 0;
}


/*
fschar *mafgets(fschar *s, int n, FILE *fp)
{
	int i;
	maint c, d;
	fschar *p;
	machar mastr[MAXLOGLINELEN];
	
	if(n < 1)
	{
		masprintf(mastr, MAXLOGLINELEN, L"mafgets invalid size parameter=%d", n);
		maLog(mastr);
		return NULL;
	}

	i = 0;
	p = s; // &s[i]
	errno = 0;
	
	// get one line
	for (; ;)
	{
		if(i >= n)
		{
			masprintf(mastr, MAXLOGLINELEN, L"mafgets line too long max=%d", n - 1);
			maLog(mastr);
			return NULL;
		}
		
		c = mafgetc(fp);
		if(errno)
		{
			masprintf(mastr, MAXLOGLINELEN, L"mafgets 1 mafgetc errno=%d", errno);
			maLog(mastr);
			return NULL;
		}
		
		if(c == fseof) // feof(fp) // if it was an error instead of real EOF then errno showed it
		{
			if(i == 0) /// empty file
				return NULL;
			break;
		}
		
		// test for eol
		if(c == fsT('\r') || c == fsT('\n'))
		{
			for (; ;)
			{
				d = mafgetc(fp);
				if(errno)
				{
					masprintf(mastr, MAXLOGLINELEN, L"mafgets 2 mafgetc errno=%d", errno);
					maLog(mastr);
					return NULL;
				}
				
				if(d == fseof) // feof(fp)
					break;
					
				if(d != fsT('\r') && d != fsT('\n')) // first machar of next line, push it  back
				{
					maungetc(d, fp);
					break;
				}
			}
			break;
		}

		*p = c; // s[i] = c
		p++;
		i++;
	}
			
	if(i == 0) /// file only has eols
		return NULL;

	*p = fsT('\0'); // string terminator

	if(s[0] == '\xef' && s[1] == '\xbb' && s[2] == '\xbf') // BOM_UTF-8
		memmove(s, &s[3], fsstrlen(s) - 2); // include terminating \0

	return s;
	
} // mafgets
*/


machar *memgets(machar *s, int n, machar *p0)
{
	machar c;
	machar *p;
	size_t len;
	machar mastr[MAXLOGLINELEN];
	
	if(n < 1)
	{
		masprintf(mastr, MAXLOGLINELEN, L"memgets invalid size parameter=%d", n);
		maLog(mastr);
		return NULL;
	}

	// get one line
	p = p0;
	while((c = *p++) != L'\r' && c != L'\n' && c != L'\0');
	p--;
	
	len = p - p0;
	
	if(len == 0 && c == L'\0')
	{
		s[0] = L'\0';
		return NULL;
	}
	
	if(len >= (size_t)(n - 1))
	{
		masprintf(mastr, MAXLOGLINELEN, L"memgets line too long max=%d", n - 1);
		maLog(mastr);
		len = n - 1;
	}
	
	memcpy(s, p0, len * sizeof(machar));
	s[len] = L'\0';

	if(c != L'\0') // was eol
		while((c = *p++) == L'\r' || c == L'\n');
	
	p--;
	return p; // next line (or eof)
	
} // memgets


int mastrcasecmp(machar *s1, machar *s2)
{
	size_t i, l;

	if(!s1 || !s2)
		return 1;

	if((l = mastrlen(s1)) != mastrlen(s2))
		return 1;

	for(i = 0; i < l; i++)
		if(matolower(s1[i]) != matolower(s2[i]))
			return 1;

	return 0;

} // mastrcasecmp


int mastrncasecmp(machar *s1, machar *s2, size_t n)
{
	int i;
	size_t l1, l2;

	if(!s1 || !s2)
		return 1;

	if((l1 = mastrlen(s1)) != (l2 = mastrlen(s2)))
	{
		if(n > min(l1, l2))
			return 1;
		l1 = n;
	}
	else
		l1 = min(n, l2);

	for(i = 0; (size_t)i < l1; i++)
	{
		if(matolower(s1[i]) != matolower(s2[i]))
			return 1;
	}

	return 0;

} // mastrncasecmp


machar *mastrcasestr(machar *s1, machar *s2)
{
	maint i1;
	maint i2;
	machar *p1, *p2;
	machar *pos = s1;
	
mastrcasestrin:
	while(*pos != L'\0')
	{
		// if(matolower(*pos++) == matolower(*s2))
		i1 = *pos++;
		i2 = *s2;
		if(matolower(i1) == matolower(i2))
		{
			p1 = pos;
			p2 = s2 + 1;
			
			while(*p2)
			{
				i1 = *p1++;
				i2 = *p2++;
				if(matolower(i1) != matolower(i2))
					goto mastrcasestrin;
			}
			return --pos;
		}
	}
 
	return NULL;
 
} // mastrcasestr


#if defined(__MAOSX__)
 int maGetRootVolumeName(fschar *name)
 {
	int i;
	FSRef fsRef;
	FSSpec spec;

	if(FSPathMakeRef((const unsigned char *)"/", &fsRef, NULL) != noErr)
		return 1;

	if(FSGetCatalogInfo(&fsRef, kFSCatInfoNone, NULL, NULL, &spec, NULL) != noErr)
		return 1;

	// deprecated p2cstrcpy(name, spec.name);
	i = spec.name[0];
	name[i] = 0;
	while(i > 0)
	{
		name[i - 1] = spec.name[i];
		i--;
	}
	
	return 0;

 } // maGetRootVolumeName
#endif


#ifdef __MAWIN__
 int maGetLongPathName(fschar *infname, fschar *outfname, int len)
 {
	return ::GetLongPathName(infname, outfname, len);
 }


 int maWinResolveShortcut(fschar *pathname)
 {
	LPCWSTR pszShortcutFile;
	HRESULT hres;
	IShellLink *psl;
	machar szGotPath[MAX_PATH];
	WIN32_FIND_DATA wfd;
	int result = 1; // error
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif

	pszShortcutFile = pathname;

	// Get a pointer to the IShellLink interface.
	hres = CoCreateInstance(CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER,
											IID_IShellLink, (void **)&psl);
	if(SUCCEEDED (hres))
	{
		IPersistFile *ppf;

		// Get a pointer to the IPersistFile interface.
		hres = psl->QueryInterface (IID_IPersistFile, (void **)&ppf);

		if(SUCCEEDED(hres))
		{
			WORD wsz[MAX_PATH]; // buffer for Unicode string

			// Ensure that the string consists of Unicode characters.
			memcpy(wsz, pszShortcutFile, MAX_PATH * sizeof(WORD));

			// Load the shortcut.
			hres = ppf->Load((LPCOLESTR)&wsz[0], STGM_READ);

			if(SUCCEEDED(hres))
			{
				// Resolve the shortcut.
				hres = psl->Resolve(NULL, SLR_ANY_MATCH);
				if(SUCCEEDED(hres))
				{
					mastrcpy(szGotPath, fs2ma(mastr2, (fschar *)pszShortcutFile));
					// Get the path to the shortcut target.
					hres = psl->GetPath(szGotPath, MAX_PATH, (WIN32_FIND_DATA *)&wfd, SLGP_SHORTPATH);
					if(!SUCCEEDED(hres))
						;
					else
					{
						if(::GetLongPathName(szGotPath, pathname, MAX_PATH)) // returns length
							result = 0; // ok
					}
				}
			}
			// Release the pointer to IPersistFile.
			ppf->Release();
		}
		// Release the pointer to IShellLink.
		psl->Release();
	}
	
	return result;

 } // maWinResolveShortcut
#endif


 // named semaphores, for multiple instances
int maNamedSemaphoreOpen(void **ppsem, fschar *name)
{
#if defined(__MAWIN__)
	int err;
#endif	
	machar mastr[MAXLOGLINELEN];

	if(!ppsem)
	{
		maLog(L"maNamedSemaphoreOpen ppsem null");
		return 1;
	}
	
	if(!name || !name[0])
	{
		maLog(L"maSemaphoreOpen name null");
		return 1;
	}
	
	// signaled
#if defined(__MAOSX__) || defined(__MAUNIX__)
	if((*ppsem = sem_open((char *)name, O_CREAT, 0666, 1)) == (sem_t *)SEM_FAILED)
	{
		*ppsem = NULL;
		masprintf(mastr, MAXLOGLINELEN, L"maNamedSemaphoreOpen sem_open '%ls' errno=%d", name, errno);
		maLog(mastr);
		return 1;
	}
#endif

#if defined(__MAWIN__)
	*ppsem = CreateMutex(NULL, FALSE, name);
	err = GetLastError();
	if(err && err != ERROR_ACCESS_DENIED && err != ERROR_ALREADY_EXISTS)
 	{
		*ppsem = NULL;
		masprintf(mastr, MAXLOGLINELEN, L"maNamedSemaphoreOpen CreateMutex '%ls' err=%d", fs2ma(mastr2, name), err);
		maLog(mastr);
		return 1;
	}
#endif

	return 0;
	
} // maNamedSemaphoreOpen


/* #if defined(__MAOSX__) || defined(__MACUNIX__)
  int maNamedSemaphoreUnlink(void *psem, char *name)
  {
	if(!name || !name[0])
	{
		maLog(L"maNamedSemaphoreUnlink name null");
		return 1;
	}
	
	/w* EINVAL 22 Invalid argument
	if(sem_unlink(name) == -1)
	{
		machar mastr[MAXLOGLINELEN];
		masprintf(mastr, MAXLOGLINELEN, L"maNamedSemaphoreUnlink sem_unlink '%ls' errno=%d", name, errno);
		maLog(mastr);
		return 1;
	} *w/

	return 0;
	
  } // maNamedSemaphoreUnlink
#endif
 */


int maNamedSemaphorePost(void *psem)
{
	if(!psem)
	{
		maLog(L"maNamedSemaphorePost psem null");
		return 1;
	}

#if defined(__MAWIN__)
	ReleaseMutex(psem);
#endif

#if defined(__MAOSX__) || defined(__MAUNIX__)	
	sem_post((sem_t *)psem);
#endif

	return 0;
	
} // maNamedSemaphorePost


int maNamedSemaphoreWait(void *psem)
{
	if(!psem)
	{
		maLog(L"maNamedSemaphoreWait psem null");
		return 1;
	}

#if defined(__MAWIN__)
	switch (WaitForSingleObject(psem, INFINITE))
	{
		case WAIT_OBJECT_0:
			return 0;
		break;

		case WAIT_ABANDONED:
			// maLog(L"maNamedSemaphoreWait WaitForSingleObject abandoned");
			return 0;
		break;

		case WAIT_TIMEOUT:
			maLog(L"maNamedSemaphoreWait WaitForSingleObject timeout");
		break;

		case WAIT_FAILED:
		default:
			maLog(L"maNamedSemaphoreWait WaitForSingleObject failed");
		break;
	}
		return 1;
 #endif

 #if defined(__MAOSX__) || defined(__MAUNIX__)	
	sem_wait((sem_t *)psem);
	return 0;
 #endif

} // maNamedSemaphoreWait


#if defined(MAUSESHM) || defined(MAUSESHF)
 int maNamedSemaphoreLock(MALOCK *lock)
 {
	machar mastr[MAXLOGLINELEN];

	if(!lock)
	{
		ShowErr(L"maNamedSemaphoreLock null lock");
		return 1;
	}

	if(!lock->psem)
	{
		if(!lock->name[0])
		{
			ShowErr(L"maNamedSemaphoreLock null lock name");
			return 1;
		}

		if(maNamedSemaphoreOpen(&(lock->psem), lock->name))
		{   
			masprintf(mastr, MAXLOGLINELEN, L"maNamedSemaphoreLock '%ls' maNamedSemaphoreOpen failed", lock->name);
			ShowErr(mastr);
			return 1;
		}
	}

	if(maNamedSemaphoreWait(lock->psem))
	{
		masprintf(mastr, MAXLOGLINELEN, L"maNamedSemaphoreLock '%ls' maSemaphoreWait failed", lock->name);
		ShowErr(mastr);
		return 1;
	}
	
	return 0;

 } // maNamedSemaphoreLock


 int maNamedSemaphoreUnlock(MALOCK *lock)
 {
	machar mastr[MAXLOGLINELEN];
	
	if(!lock || !lock->psem)
	{
		ShowErr(L"maNamedSemaphoreUnlock null lock");
		return 1;
	}

	if(maNamedSemaphorePost(lock->psem))
	{
		masprintf(mastr, MAXLOGLINELEN, L"maNamedSemaphoreUnlock '%ls' maSemaphorePost failed", lock->name);
		ShowErr(mastr);
		return 1;
	}
	
	return 0;
	
 } // maNamedSemaphoreUnlock
#endif


/* int maNamedSemaphoreClose(MALOCK *lock) // not used, the system does this on process exit
{
	machar mastr[MAXLOGLINELEN];

	if(!lock || !lock->psem)
	{
		ShowErr(L"maNamedSemaphoreClose null lock");
		return 1;
	}

#if defined(__MAWIN__)
	if(CloseHandle(lock->psem) = 0)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maNamedSemaphoreClose '%ls' CloseHandle failed", lock->name);
		ShowErr(mastr);
		return 1;
	}
#endif

#if defined(__MAOSX__) || defined(__MAUNIX__)	
	if(sem_close(lock->psem) == -1)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maNamedSemaphoreClose '%ls' sem_close failed errno=%d", lock->name, errno);
		ShowErr(mastr);
		return 1;
	}
#endif

	return 0;

} // maNamedSemaphoreClose
*/


int maSemaphoreNew(void **ppsem)
{
	if(!ppsem)
	{
		maLog(L"maSemaphoreNew ppsem null");
		return 1;
	}
	
	if(*ppsem)
	{
		maLog(L"maSemaphoreNew *ppsem not null");
		return 1;
	}
	
#if defined(__MAWIN__)
	if((*ppsem = CreateEvent(0, FALSE, TRUE, 0)) == NULL)
	{
		maLog(L"maSemaphoreNew CreateEvent failed");
		return 1;
	}
#endif

#if defined(__MAUNIX__) && !defined(__MACUNIX__)
	if((*ppsem = MAMALLOC(sizeof(sem_t))) == NULL)
	{
		maLog(L"maSemaphoreNew MAMALLOC failed");
		return 1;
	}
	sem_init((sem_t *)*ppsem, 0, 0);
#endif

#if defined(__MAOSX__) || defined(__MACUNIX__)
	if((*ppsem = MAMALLOC(sizeof(sem_t))) == NULL)
	{
		maLog(L"maSemaphoreNew MAMALLOC failed");
		return 1;
	}
	masem_init((sem_t *)*ppsem, 0, 0);
#endif

	return 0;
	
} // maSemaphoreNew


int maSemaphoreDelete(void *psem)
{
	if(!psem)
	{
		maLog(L"maSemaphoreDelete psem null");
		return 1;
	}

#if defined(__MAWIN__)
	CloseHandle(psem);
#endif

#if defined(__MAUNIX__) && !defined(__MACUNIX__)
	sem_destroy((sem_t *)psem);
    MAFREE(psem);
#endif

#if defined(__MAOSX__) || defined(__MACUNIX__)
	masem_destroy((sem_t *)psem);
    MAFREE(psem);
#endif
	return 0;
	
} // maSemaphoreDelete


int maSemaphorePost(void *psem)
{
	if(!psem)
	{
		maLog(L"maSemaphorePost psem null");
		return 1;
	}

#if defined(__MAWIN__)
	SetEvent(psem);
#endif


#if defined(__MAUNIX__) && !defined(__MACUNIX__)
	sem_post((sem_t *)psem);
#endif

#if defined(__MAOSX__) || defined(__MACUNIX__)
	masem_post((sem_t *)psem);
#endif

	return 0;
	
} // maSemaphorePost


int g_errMPWaitOnSemaphore = 0;
int maSemaphoreWait(void *psem)
{
	if(!psem)
	{
		maLog(L"maSemaphoreWait psem null");
		return 1;
	}

#if defined(__MAWIN__)
	switch (WaitForSingleObject(psem, INFINITE))
	{
		case WAIT_OBJECT_0:
			return 0;
		break;

		case WAIT_ABANDONED:
			if(!g_errMPWaitOnSemaphore++) // once only
				maLog(L"maSemaphoreWait WaitForSingleObject abandoned");
		break;

		case WAIT_TIMEOUT:
			if(!g_errMPWaitOnSemaphore++)
				maLog(L"maSemaphoreWait WaitForSingleObject timeout");
		break;

		case WAIT_FAILED:
		default:
			if(!g_errMPWaitOnSemaphore++)
				maLog(L"maSemaphoreWait WaitForSingleObject failed");
		break;
	}
		return 1;
#endif

#if defined(__MAUNIX__) && !defined(__MACUNIX__)
	sem_wait((sem_t *)psem);
	return 0;
#endif

#if defined(__MAOSX__) || defined(__MACUNIX__)
	masem_wait((sem_t *)psem);
	return 0;
#endif

} // maSemaphoreWait


// mutex for critical section
#if defined(__MAUNIX__) || defined(__MAOSX__)
 int maMutexNew(pthread_mutex_t *plock)
 {
	if(!plock)
	{
		maLog(L"maMutexNew plock null");
		return 1;
	}

	if(pthread_mutex_init(plock, NULL))
	{
		maLog(L"maMutexNew pthread_mutex_init failed");
		return 1;
	}

    return 0;
    
 } // maMutexNew


 int maMutexLock(pthread_mutex_t *plock)
 {
    if(pthread_mutex_lock(plock))
    {
		maLog(L"maMutexLock pthread_mutex_lock failed");
		return 1;
	}
  
    return 0;
    
 } // maMutexLock
 

 int maMutexUnlock(pthread_mutex_t *plock)
 {
    if(pthread_mutex_unlock(plock))
    {
		maLog(L"maMutexUnlock pthread_mutex_unlock failed");
		return 1;
	}
   
    return 0;
    
 } // maMutexUnlock


 int maMutexDelete(pthread_mutex_t *plock)
 {
    if(pthread_mutex_destroy(plock))
    {
		maLog(L"maMutexDelete pthread_mutex_destroy failed");
		return 1;
	}
   
    return 0;
    
 } // maMutexDelete
#endif


void ThreadYield() // the calling thread wants to give up its time slice
{
#ifdef __MAWIN__
	Sleep(0);
#endif

#if defined(__MAOSX__) || defined(__MAUNIX__)
	usleep(0);
#endif

} // ThreadYield


void AppYield() 
{
	guiAppYield();
	
} // AppYield


int maSleep(unsigned int millis)
{
#if defined(__MAOSX__)
	MPEventID event;
	machar mastr[MAXLOGLINELEN];

	OSStatus status = MPCreateEvent(&event);	
	if(status)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maSleep %d MPCreateEvent error %d", millis, (int)status);
		maLog(mastr);
		return 1;
	}
	status = MPWaitForEvent(event, NULL, millis);
	if(status && status != kMPTimeoutErr) // -29296)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maSleep %d MPWaitForEvent error %d", millis, (int)status);
		maLog(mastr);
	}
	MPDeleteEvent(event); 
 	if(status)
		return 1;
#endif

#ifdef __MAWIN__
	Sleep(millis);
#endif

#if defined(__MAUNIX__)
	if(usleep(millis * 1000) != 0
 #if defined(MAGUINONE)
		&& errno != EINTR // 4, when we ctl-C out
 #endif
		) //
	{
		machar mastr[MAXLOGLINELEN];
		masprintf(mastr, MAXLOGLINELEN, L"maSleep %d usleep errno=%d", millis * 1000, errno);
		maLog(mastr);
		return 1;
	}
#endif

	return 0;
	
} // maSleep


// posix with sys V
#if defined(__MAOSX__) || defined(__MACUNIX__)
 key_t l_masemkey = 0;
 int masem_init(int *sem, int pshared, unsigned int value)
 {
	int retry = 65;

	if(value > SEM_VALUE_MAX)
	{
		maLog(L"masem_init bad sem value");
		return  -1;
	}

	if(l_masemkey == 0)
		l_masemkey = maGetLocalTimeMillis(); // need something unique system-wide here, ftok not enough

	while((*sem = mainitsem(l_masemkey++, 1)) == -2) // errno == EEXIST
		if(retry-- <= 0)
		{
			maLog(L"masem_init mainitsem unable to create a unique sem");
			return  -1;
		}
	
	return masem_setvalue(sem, value);

 } // masem_init


 int masem_wait(int *semid)
 {
	struct sembuf sb;
	sb.sem_num = 0;
	sb.sem_op = -1;
	sb.sem_flg = 0;

	if (semop(*semid, &sb, 1) == -1)
	{
		maLog(L"masem_wait semop failed");
		return -1;
	}

	return 0;

 } // masem_wait


 int masem_post(int *semid)
 {
	struct sembuf sb;
	sb.sem_num = 0;
	sb.sem_op = 1;
	sb.sem_flg = 0;

	if (semop(*semid, &sb, 1) == -1)
	{
		maLog(L"masem_post semop failed");
		return -1;
	}

	return 0;

 } // masem_post


 int masem_destroy(int *semid)
 {
    int ret;

	if((ret = semctl(*semid, 0, IPC_RMID)) == -1)
		maLog(L"masem_destroy semctl failed");

	return ret;

 } // masem_destroy


 int mainitsem(key_t key, int nsems)
 {
	int semid;

	if ((semid = semget(key, nsems, IPC_CREAT | IPC_EXCL | 0666)) == -1)
	{
		if (errno == EEXIST)
		{
			/*
			if ((semid = semget(key, nsems, 0)) < 0)
				maLog(L"mainitsem semget existing sem failed");
			*/
			return -2; // go back to retry with new key
		}
		else // more errors
	        maLog(L"mainitsem semget failed");
	}

	return semid;

 } // mainitsem


 int masem_setvalue(int *semid, int value)
 {
    int ret;
        
	if((ret = semctl(*semid, 0, SETVAL, value)) == -1)
        maLog(L"masem_setvalue semctl failed");
    
	return ret;

 } // masem_setvalue
#endif // defined(__MAOSX__) || defined(__MACUNIX__) posix with sys V


int maThreadCreate(threadfunctiontype threadfunction, threadparametertype threadparameter)
{
#if defined(__MAWIN__)
    machar mastr[MAXLOGLINELEN];
 	DWORD thread_id;
	HANDLE threadplayhandle;

	if((threadplayhandle = CreateThread(0, 0, (LPTHREAD_START_ROUTINE)threadfunction,
														threadparameter, 0, &thread_id)) == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maThreadCreate CreateThread error %08X", GetLastError());
		maLog(mastr);
		g_state.pdecoder = NULL;
		return 1;
	}

	return 0;
#endif

#if defined(__MAUNIX__) || defined(__MAOSX__)
	pthread_t pthread[1];
    machar mastr[MAXLOGLINELEN];

	pthread_attr_t attr;
	pthread_attr_init(&attr);
	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);

    if(pthread_create(&pthread[0], &attr, threadfunction, (void *)threadparameter))
	{
		masprintf(mastr, MAXLOGLINELEN, L"maThreadCreate pthread_create failed errno=%d", errno);
		maLog(mastr);
		g_state.pdecoder = NULL;
		return 1;
    }

	return 0;
#endif

} // maThreadCreate


#if defined(MAUSESHF)
 // instance table, shared file instead of shm
 void *mashmopen(size_t shmsize, char *keyfile)
 {
	char *pinstance;
	machar mastr[MAXLOGLINELEN];
	
	if(!keyfile)
	{
		ShowErr(L"mashmopen null keyfile");	
		return NULL;
	}

	if(maFileExists(keyfile))
		return (void *)keyfile; // ok
	
	if((pinstance = (char *)MAMALLOC(shmsize)) == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"mashmopen MAMALLOC failed on '%ls'", keyfile);
		ShowErr(mastr);	
		return NULL;
	}
	
	memset(pinstance, 0, shmsize);
	
	if(mashmwrite(pinstance, 1, shmsize, keyfile) != (int)shmsize)
	{
		MAFREE(pinstance);
		masprintf(mastr, MAXLOGLINELEN, L"mashmopen mashmwrite failed on '%ls'", keyfile);
		ShowErr(mastr);	
		return NULL;
	}

	MAFREE(pinstance);
	
	return (void *)keyfile; // ok
	
 } // mashmopen


 int mashmclose(void *pmashm)
 {
	return 0;
	
 } // mashmclose


 int mashmwrite(char *ptr, size_t size, size_t nmemb, void *pmashm)
 {
	int ret;
	int data_len;
	FILE *fp;
	machar mastr[MAXLOGLINELEN];

	if(!ptr || !pmashm || ((fschar *)pmashm)[0] == fsT('\0'))
	{   
		ShowErr(L"mashmwrite null pointer to shared file");
		return 0;
	}
	
	if((data_len = size * nmemb) <= 0)
	{   
		ShowErr(L"mashmwrite data length is 0");
		return 0;
	}
		
	if((fp = mafopen((fschar *)pmashm, fsT("wb"))) == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"mashmwrite mafopen failed on '%ls'", fs2ma(mastr2, pmashm));
		ShowErr(mastr);
		return NULL;
	}
	
	ret = fwrite(ptr, size, nmemb, fp);
		
	fclose(fp);
	
	if(ret != (int)nmemb)
	{
		masprintf(mastr, MAXLOGLINELEN, L"mashmwrite fwrite failed on '%ls'", fs2ma(mastr2, pmashm));
		ShowErr(mastr);	
		return NULL;
	}
	
	return nmemb;
	
 } // mashmwrite


 int mashmread(char *ptr, size_t size, size_t nmemb, void *pmashm)
 {
	int ret;
	int data_len;
	FILE *fp;
	machar mastr[MAXLOGLINELEN];

	if(!ptr || !pmashm || ((fschar *)pmashm)[0] == fsT('\0'))
	{   
		ShowErr(L"mashmread null pointer to shared file");
		return 0;
	}
	
	if((data_len = size * nmemb) <= 0)
	{   
		ShowErr(L"mashmread data length is 0");
		return 0;
	}
		
	if((fp = mafopen((fschar *)pmashm, fsT("rb"))) == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"mashmread mafopen failed on '%ls'", fs2ma(mastr2, pmashm));
		ShowErr(mastr);
		return NULL;
	}
	
	ret = fread(ptr, size, nmemb, fp);
	
	fclose(fp);
	
	if(ret != (int)nmemb)
	{
		masprintf(mastr, MAXLOGLINELEN, L"mashmread fread failed on '%ls'", fs2ma(mastr2, pmashm));
		ShowErr(mastr);	
		return NULL;
	}
	
	return nmemb;
	
 } // mashmread


 int mashmdelete(void *pmashm)
 {
	machar mastr[MAXLOGLINELEN];

	if(!pmashm || ((fschar *)pmashm)[0] == fsT('\0'))
	{   
		ShowErr(L"mashmdelete null pointer to shared file");
		return 0;
	}

	if(maunlink((fschar *)pmashm))
	{
		masprintf(mastr, MAXLOGLINELEN, L"mashmdelete failed to delete '%ls'", fs2ma(mastr2, pmashm));
		ShowErr(mastr);
		return 1;
	}		
	
	return 0;
 }
#endif // MAUSESHF


#if defined(MAUSESHM)
 // shared mem
 void *mashmopen(size_t shmsize, fschar *keyfile)
 {
	MASHM *pmashm;	
	machar mastr[MAXLOGLINELEN];

	if((pmashm = (MASHM *)MAMALLOC(sizeof(MASHM))) == NULL)
	{
		ShowErr(L"mashmopen MAMALLOC failed");
		return NULL;
	}
	memset(pmashm, 0, sizeof(MASHM));
	pmashm->size = shmsize;

 #if defined(__MAWIN__)
	// on win we share mamory mapped file 'keyfile'
	if((pmashm->file = CreateFile(keyfile, GENERIC_WRITE | GENERIC_READ, // LPCTSTR keyfile
									FILE_SHARE_WRITE | FILE_SHARE_READ,
						NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL))
											== INVALID_HANDLE_VALUE)
	{
		MAFREE(pmashm);
		masprintf(mastr, MAXLOGLINELEN, L"mashmopen CreateFile error %d on '%ls'",
											GetLastError(), keyfile);
		ShowErr(mastr);
		return NULL;
	}
	if((pmashm->map = CreateFileMapping(pmashm->file, NULL,
					PAGE_READWRITE, 0, (int)shmsize, fsT("mMaAkKeEyYfFiIlLeE"))) == NULL)
	{
		CloseHandle(pmashm->file);
		MAFREE(pmashm);
		masprintf(mastr, MAXLOGLINELEN, L"mashmopen CreateFileMapping error %d on '%ls'",
											GetLastError(), keyfile);
		ShowErr(mastr);
		return NULL;
	}
	pmashm->alone = (GetLastError() == ERROR_ALREADY_EXISTS) ? 0 : 1; // maNumProcesses == 1
	pmashm->view = MapViewOfFile(pmashm->map, FILE_MAP_WRITE, 0, 0, 0);
	if(pmashm->view == NULL)
	{
		CloseHandle(pmashm->map);
		CloseHandle(pmashm->file);
		MAFREE(pmashm);
		masprintf(mastr, MAXLOGLINELEN, L"mashmopen MapViewOfFile error %d on '%ls'",
											GetLastError(), keyfile);
		ShowErr(mastr);
		return NULL;
	}
	pmashm->addr = (char *)pmashm->view;
	if(pmashm->alone)
		memset(pmashm->addr, 0, pmashm->size); // we are the only instance
 #endif
 #if defined(__MAOSX__) || defined(__MAUNIX__)
	// on unix we share a memory segment
	struct shmid_ds shm;
	if((pmashm->key = ftok((const char *)keyfile, (int)getuid())) == -1) // each user gets his own shm
	{
		MAFREE(pmashm);
		masprintf(mastr, MAXLOGLINELEN, L"mashmopen ftok('%ls', 0) failed", keyfile);
		ShowErr(mastr);
		return NULL;
	}
	if((pmashm->shmid = shmget(pmashm->key, pmashm->size,  0666 | IPC_CREAT)) == -1)
	{
		MAFREE(pmashm);
		ShowErr(L"mashmopen shmget failed");
		return NULL;
	}
	if(shmctl(pmashm->shmid, IPC_STAT, &shm) == -1)
	{
		MAFREE(pmashm);
		masprintf(mastr, MAXLOGLINELEN, L"mashmopen shmctl IPC_STAT failed errno=%d", errno);
		ShowErr(mastr);
		return NULL;
	}	
	pmashm->size = shm.shm_segsz;
	pmashm->addr = (char *)shmat(pmashm->shmid, 0, 0);
	if(pmashm->addr == (char *)-1)
	{
		MAFREE(pmashm);
		ShowErr(L"mashmopen shmat failed"); // errno
		return NULL;
	}
	if(maNumProcesses(0) == 1)
	{
		pmashm->alone = 1;
		memset(pmashm->addr, 0, pmashm->size); // we are the only instance
	}
	else
		pmashm->alone = 0;
 #endif

	return pmashm;
	
 } // mashmopen


 int mashmclose(void *pmashm)
 {
	if(!pmashm)
	{   
		ShowErr(L"mashmclose null pmashm");
		return 1;
	}

 #if defined(__MAWIN__) // mashmdelete
	UnmapViewOfFile(((MASHM *)pmashm)->view);
	CloseHandle(((MASHM *)pmashm)->map);
	CloseHandle(((MASHM *)pmashm)->file);
 #endif

 #if defined(__MAOSX__) || defined(__MAUNIX__)
	if(shmdt(((MASHM *)pmashm)->addr) == -1)
	{
		ShowErr(L"mashmclose shmdt failed"); // errno
		return 1;
	}
 #endif
	
	return 0;
 }


 int mashmwrite(char *ptr, size_t size, size_t nmemb, void *pmashm)
 {
	size_t data_len;
	machar mastr[MAXLOGLINELEN];

	if(!pmashm)
	{   
		ShowErr(L"mashmwrite null pointer to shared mem");
		return 0;
	}
	
	if((data_len = size * nmemb) <= 0)
	{   
		ShowErr(L"mashmwrite data length is 0");
		return 0;
	}
		
	if(data_len > (size_t)((MASHM *)pmashm)->size)
	{
		masprintf(mastr, MAXLOGLINELEN, L"mashmwrite data size %d greater than shared mem size %d",
								data_len, ((MASHM *)pmashm)->size);
		ShowErr(mastr);
		return 0;
	}

	memcpy(((MASHM *)pmashm)->addr, ptr, data_len);

	return (int)nmemb;
	
 } // mashmwrite


 int mashmread(char *ptr, size_t size, size_t nmemb, void *pmashm)
 {
	int data_len;

	if(!pmashm)
	{   
		ShowErr(L"mashmread null pointer to shared mem");
		return 0;
	}
	
	if((data_len = (int)(size * nmemb)) <= 0)
	{   
		ShowErr(L"mashmread data length is 0");
		return 0;
	}
		
	if(data_len > (int)((MASHM *)pmashm)->size)
	{
		ShowErr(L"mashmread data size greater than shared mem");
		return 0;
	}

	memcpy(ptr, ((MASHM *)pmashm)->addr, data_len);

	return (int)nmemb;
	
 } // mashmread


 int mashmdelete(void *pmashm)
 {
	if(!pmashm)
	{   
		ShowErr(L"mashmdelete null pmashm");
		return 1;
	}

 #if defined(__MAWIN__)
	UnmapViewOfFile(((MASHM *)pmashm)->view);
	CloseHandle(((MASHM *)pmashm)->map);
	CloseHandle(((MASHM *)pmashm)->file);
 #endif

 #if defined(__MAOSX__) || defined(__MAUNIX__)
	if(shmctl(((MASHM *)pmashm)->shmid, IPC_RMID, NULL))
	{
		ShowErr(L"mashmdelete shmctl IPC_RMID failed");
		return 1;
	}
 #endif

	return 0;
 }
#endif // MAUSESHM


fschar *magetmoreampdir(fschar *dir)
{
#ifdef __MAWIN__
	// moreampdir is the dir holding the MoreAmp.exe binary
	return magetbindir(dir);
#endif

#if defined(__MAOSX__) && !defined(MAOSXHOME)
	// moreampdir is the dir holding MoreAmp.app
	return magetbindir(dir);
#endif

#if defined(__MAUNIX__) || defined(MAOSXHOME)
	// moreampdir is MOREAMP_DIRNAME in the user's home dir
	ssize_t len;
	maenvchar *p;
	char ospath[MAX_PATH];
#if defined(MASETLOCALEWIN)
	fschar fsstr[MAXLOGLINELEN];
#endif
	fschar name[MAX_PATH];
	machar mastr[MAXLOGLINELEN];
	if(!(p = magetenv("HOME")))
	{
		ShowErr(L"magetmoreampdir getenv(HOME) failed");
		return NULL;
	}
	fssprintf(dir, MAX_PATH, fsS, maenv2fs(fsstr, p));
	// resolve any links to home dir
	if((len = readlink(p, ospath, MAX_PATH - 1)) == -1)
	{
		if(errno != EINVAL) // dir is not a symlink
		{
			masprintf(mastr, MAXLOGLINELEN, L"magetmoreampdir readlink '%ls' errno=%d", dir, errno);
			maLog(mastr);
		}
	}
	else
	{
		ospath[len] = '\0';
		fssprintf(dir, MAX_PATH, fsS, os2fs(name, ospath));
	}
	fssprintf(name, MAX_PATH, fsC fsS, FILESEP, MOREAMP_DIRNAME);
	fsstrcat(dir, name);
	return dir;
#endif

} // magetmoreampdir


fschar *magetcwd(fschar *dir, int size)
{
#if defined(__MAOSX__) || defined(__MAUNIX__)
	// cwd is set by the shell
	return (fschar *)getcwd((char *)dir, (size_t)size);
#endif

#if defined(__MAWIN__)	
	return fsgetcwd(dir, size); // 'starting dir' property ?
#endif
} // magetcwd


fschar *magetbindir(fschar *dir)
{
#if defined(__MAWIN__)
	fschar *p, *q, *r;
    fschar path[MAX_PATH];

    if(!GetModuleFileName(NULL, path, MAX_PATH))
    {
        ShowErr(L"magetbindir GetModuleFileName failed");
        return NULL;
    }
	q = fsT('\0');
	r = path;									// 'C:\...\adir\MOREAMP_NAME.EXE'
	while((p = fsstrchr(r, FILESEP)) != 0)		// remove '\MOREAMP_NAME.EXE'
	{
		r = p + 1;
		q = p;
	}
	if(q)
		*q =  fsT('\0');						// "C:\...\adir"
	fsstrcpy(dir, path);
	
	return dir;
#else // not win
	fschar *p, *q, *r;

 #if defined(MAGUIWX)
	ma2fsfn(dir, (machar *)g_argv[0]); // "/.../adir/MoreAmp.app/Contents/MacOS/MOREAMP_NAME" or "/.../adir/moreamp"
 #else
	fsstrcpy(dir, (fschar *)g_argv[0]);	// "/.../adir/MoreAmp.app/Contents/MacOS/MOREAMP_NAME" or "/.../adir/moreamp"
 #endif
 #if defined(__MAOSX__)
	// this gets the dir with the MoreAmp.app bundle
	if((p = fsstrstr(dir, fsT("/Contents/MacOS"))))		// we're in the bundle  
		*p = fsT('\0');									// "/.../adir/MoreAmp.app"
	else
	{
		ShowErr(L"magetbindir unable to determine working dir (bad .app contents ?)");
		return NULL;
	}
 #endif
	q = fsT('\0');							// remove '/MoreAmp.app' or '/moreamp'
	r = &dir[1];
	while((p = fsstrchr(r, FILESEP)))
	{
		r = p + 1;
		q = p;
	}
	if(q)
		*q =  fsT('\0');					// "/.../adir"

	return dir;
#endif

} // magetbindir


/* not used
int masetcwd(fschar *dir)
{
	int err;
	machar mastr[MAXLOGLINELEN];
	
#if defined(__MAOSX__) || defined(__MAUNIX__)
	if((err = chdir(dir)) == 0)
		return 0;
#endif

#if defined(__MAWIN__)
	if(SetCurrentDirectory(dir)) // or _chdir
		return 0;
	err = 0;
#endif

	masprintf(mastr, MAXLOGLINELEN, L"masetcwd SetCurrentDirectory err %d on '%ls'", err, dir);
	maLog(mastr);	
	return 1;
	
} // masetcwd
*/


#if defined(__MAWIN__)
int maIsWin9x()
{
	OSVERSIONINFOA info; // A for ANSI names

	info.dwOSVersionInfoSize = sizeof(info);

	if(GetVersionExA(&info))
		if((info.dwPlatformId == 2 && info.dwMajorVersion == 4) || info.dwMajorVersion >= 5) // NT, WDM
			return 0;

	return 1; // 95, 98

} // maIsWin9x


int maIsWin7() // or later
{
	OSVERSIONINFOA info; // A for ANSI names

	info.dwOSVersionInfoSize = sizeof(info);

	if(GetVersionExA(&info))
		if((info.dwMajorVersion >= 6) && info.dwMinorVersion >= 1)	// windows 7 (or Server 2008 R2) version 6.1
			return 1;												// dwMinorVersion == 0 for Vista version 6.0

	return 0;

} // maIsWin7
#endif


#ifdef __MAWIN__
 // return the number of 'moreamp' processes running
 // or if 'moreamp' process 'pid' is running
 // code from ms pviewer sample
 #define INDEX_STR_LEN 10
 #define PN_PROCESS TEXT("Process")
 #define PN_PROCESS_ID TEXT("ID Process")
 typedef PERF_DATA_BLOCK PERF_DATA, *PPERF_DATA;
 typedef PERF_OBJECT_TYPE PERF_OBJECT, *PPERF_OBJECT;
 typedef PERF_INSTANCE_DEFINITION PERF_INSTANCE, *PPERF_INSTANCE;
 typedef PERF_COUNTER_DEFINITION PERF_COUNTER, *PPERF_COUNTER;
 DWORD GetPerfTitleSz(HKEY hKeyMachine, HKEY hKeyPerf, LPTSTR *TitleBuffer, LPTSTR *TitleSz[], DWORD *TitleLastIdx);
 TCHAR INDEX_PROCTHRD_OBJ[2*INDEX_STR_LEN];

 int maNumProcessesWin(int *pid)
 {
	int i;
	int *p = pid;
	int lpid = 0;
	int ninstances = 0;
	INT InstanceIndex = 0;
	DWORD gPerfDataSize = 50*1024;
	DWORD *pdwProcID;
	DWORD Last;
	DWORD Index;
	DWORD dwR;
	DWORD PX_PROCESS;
	DWORD PX_PROCESS_ID;
	PPERF_OBJECT gpProcessObject;
	PPERF_DATA gpPerfData = NULL;
	PPERF_INSTANCE pInstance;
	PPERF_COUNTER pCounterProcID;
	PERF_COUNTER_BLOCK *pCounterBlock;
	LPTSTR TitleBuffer;
	LPTSTR *Title;
	HKEY ghMachineKey = HKEY_LOCAL_MACHINE;
	HKEY ghPerfKey = HKEY_PERFORMANCE_DATA;
	fschar name[MAX_PATH];
	machar mastr[MAXLOGLINELEN];

	dwR = GetPerfTitleSz(ghMachineKey, ghPerfKey, &TitleBuffer, &Title, &Last);
	if(dwR != ERROR_SUCCESS)
	{
		masprintf(mastr, MAXLOGLINELEN, L"maNumProcessesWin error GetPerfTitleSz %d", dwR);
		ShowErr(mastr);
		return 0;
	}

	for(Index = 0; Index <= Last; Index++)
		if(Title[Index])
			if(!lstrcmpi (Title[Index], PN_PROCESS))
				break;
	if(Index > Last)
	{
		ShowErr(L"maNumProcessesWin cannot find process name index");
		return 0;
	}
	PX_PROCESS = Index;

	for(Index = 0; Index <= Last; Index++)
		if(Title[Index])
			if(!lstrcmpi (Title[Index], PN_PROCESS_ID))
				break;
	if(Index > Last)
	{
		ShowErr(L"maNumProcessesWin cannot find process id index");
		return 0;
	}
	PX_PROCESS_ID = Index;

	DWORD DataSize;
	DWORD Type;

	if(!gpPerfData)
		gpPerfData = (PPERF_DATA)LocalAlloc(LMEM_FIXED, gPerfDataSize);

	do
	{
		DataSize = gPerfDataSize;
		dwR = RegQueryValueEx(ghPerfKey, INDEX_PROCTHRD_OBJ, NULL, &Type, (BYTE *)gpPerfData, &DataSize);

		if(dwR == ERROR_MORE_DATA)
		{
			LocalFree (gpPerfData);
			gPerfDataSize += 1024;
			gpPerfData = (PPERF_DATA)LocalAlloc (LMEM_FIXED, gPerfDataSize);
		}

		if(!gpPerfData)
		{
			LocalFree (gpPerfData);
			ShowErr(L"maNumProcessesWin LocalAlloc failed");
			return 0;
		}

	} while(dwR == ERROR_MORE_DATA);

	if(dwR != ERROR_SUCCESS)
	{
		ShowErr(L"maNumProcessesWin RegQueryValueEx failed");
		return 0;
	}

	if((gpProcessObject = ((PPERF_OBJECT)((PBYTE)gpPerfData + gpPerfData->HeaderLength))) == NULL)
	{
		ShowErr(L"maNumProcessesWin process not found");
		return 0;
	}
	
	i = 0;
	while(i < (int)gpPerfData->NumObjectTypes)
	{
		if(gpProcessObject->ObjectNameTitleIndex == PX_PROCESS)
			break;
		gpProcessObject = ((PPERF_OBJECT)((PBYTE)gpProcessObject + gpProcessObject->TotalByteLength));
		i++;
	}
	if(i >= (int)gpPerfData->NumObjectTypes)
	{
		ShowErr(L"maNumProcessesWin process not found");
		return 0;
	}

	if((pCounterProcID = (PPERF_COUNTER)((PCHAR)gpProcessObject + gpProcessObject->HeaderLength)) == NULL)
	{
		ShowErr(L"maNumProcessesWin process not found");
		return 0;
	}
	
	i = 0;
	while(i < (int)gpProcessObject->NumCounters)
	{
		if(pCounterProcID->CounterNameTitleIndex == PX_PROCESS_ID)
			break;		
		pCounterProcID = (PPERF_COUNTER)((PCHAR)pCounterProcID + pCounterProcID->ByteLength);
		i++;
	}
	if(i >= (int)gpProcessObject->NumCounters)
	{
		ShowErr(L"maNumProcessesWin process not found");
		return 0;
	}

	pInstance = (PPERF_INSTANCE)((PCHAR)gpProcessObject + gpProcessObject->DefinitionLength);

	if(p)
		lpid = *p;
	while(pInstance && InstanceIndex < gpProcessObject->NumInstances)
	{
		fsstrcpy(name,  (const fschar *)(LPTSTR)((PCHAR)pInstance + pInstance->NameOffset));
		if(fsstrstr(name, MOREAMP_NAMEfs) == name)
		{
			PPERF_COUNTER_BLOCK pCounterBlock;

			pCounterBlock = (PPERF_COUNTER_BLOCK)((PCHAR)pInstance + pInstance->ByteLength);
			if((pdwProcID = (DWORD *)(PVOID)((PCHAR)pCounterBlock + pCounterProcID->CounterOffset)) != 0)
			{
				if(*pdwProcID)
				{
					if(lpid == (int)(*pdwProcID))
						return 1;
					*p++ = (int)(*pdwProcID);
					ninstances++;
				}
			}
		}
		pCounterBlock = (PERF_COUNTER_BLOCK *)((PCHAR)pInstance + pInstance->ByteLength);
		pInstance = (PPERF_INSTANCE)((PCHAR)pCounterBlock + pCounterBlock->ByteLength);
		InstanceIndex++;
	}
	
	if(lpid)
		return 0;

	return ninstances;
	
 } // maNumProcessesWin


 DWORD GetPerfTitleSz(HKEY hKeyMachine, HKEY hKeyPerf, LPTSTR *TitleBuffer,
									LPTSTR *TitleSz[], DWORD *TitleLastIdx)
 {
	HKEY hKey1;
	HKEY hKey2;
	DWORD Type;
	DWORD DataSize;
	DWORD dwR;
	DWORD Len;
	DWORD Index;
	DWORD dwTemp;
	BOOL bNT10;
	LPTSTR szCounterValueName;
	LPTSTR szTitle;

	// Initialize
	hKey1 = NULL;
	hKey2 = NULL;
	*TitleBuffer = NULL;
	*TitleSz 	= NULL;

	// Open the perflib key to find out the last counter's index and system version.
	dwR = RegOpenKeyEx(hKeyMachine,
						TEXT("software\\microsoft\\windows nt\\currentversion\\perflib"),
						0, KEY_READ, &hKey1);
	if(dwR != ERROR_SUCCESS)
		goto done;

	// Get the last counter's index so we know how much memory to allocate for TitleSz
	DataSize = sizeof (DWORD);
	dwR = RegQueryValueEx(hKey1, TEXT("Last Counter"), 0, &Type, (LPBYTE)TitleLastIdx, &DataSize);
	if(dwR != ERROR_SUCCESS)
		goto done;

	// Find system version, for system earlier than 1.0a, there's no version value.
	dwR = RegQueryValueEx(hKey1, TEXT("Version"), 0, &Type, (LPBYTE)&dwTemp, &DataSize);

	if(dwR != ERROR_SUCCESS)
		// unable to read the value, assume NT 1.0
		bNT10 = TRUE;
	else
		// found the value, so, NT 1.0a or later
		bNT10 = FALSE;

	// Now, get ready for the counter names and indexes.
	if(bNT10)
	{
		// NT 1.0, so make hKey2 point to ...\perflib\009 and get
		// the counters from value "Counters"
		szCounterValueName = TEXT("Counters");
		dwR = RegOpenKeyEx(hKeyMachine,
						 TEXT("software\\microsoft\\windows nt\\currentversion\\perflib\\009"),
						 0, KEY_READ, &hKey2);
		if(dwR != ERROR_SUCCESS)
			goto done;
	}
	else
	{
		// NT 1.0a or later. Get the counters in key HKEY_PERFORMANCE_KEY
		// and from value "Counter 009"
		szCounterValueName = TEXT("Counter 009");
		hKey2 = hKeyPerf;
	}

	// Find out the size of the data.
	dwR = RegQueryValueEx (hKey2, szCounterValueName, 0, &Type, 0, &DataSize);
	if(dwR != ERROR_SUCCESS)
		goto done;

	// Allocate memory
	*TitleBuffer = (LPTSTR)LocalAlloc (LMEM_FIXED, DataSize);
	if(!*TitleBuffer)
	{
		dwR = ERROR_NOT_ENOUGH_MEMORY;
		goto done;
	}

	*TitleSz = (LPTSTR *)LocalAlloc (LPTR, (*TitleLastIdx+1) * sizeof (LPTSTR));
	if(!*TitleSz)
	{
		dwR = ERROR_NOT_ENOUGH_MEMORY;
		goto done;
	}

	// Query the data
	dwR = RegQueryValueEx (hKey2, szCounterValueName, 0, &Type, (BYTE *)*TitleBuffer, &DataSize);
	if(dwR != ERROR_SUCCESS)
		goto done;

	// Setup the TitleSz array of pointers to point to beginning of each title string.
	// TitleBuffer is type REG_MULTI_SZ.
	szTitle = *TitleBuffer;

	while((Len = lstrlen(szTitle)) != 0)
	{
		Index = maatoi(szTitle);

		szTitle = szTitle + Len + 1;

		if(Index <= *TitleLastIdx)
			(*TitleSz)[Index] = szTitle;

		szTitle = szTitle + lstrlen(szTitle) + 1;
	}

done:

	// Done. Now cleanup!
	if(dwR != ERROR_SUCCESS)
	{
		// There was an error, free the allocated memory
		if(*TitleBuffer) LocalFree (*TitleBuffer);
		if(*TitleSz) LocalFree (*TitleSz);
	}

	// Close the hKeys.
	if(hKey1) RegCloseKey (hKey1);
	if(hKey2 && hKey2 != hKeyPerf) RegCloseKey (hKey2);

	return dwR;

 } // GetPerfTitleSz
#endif


unsigned long maGetLocalTimeMillis()
{
#if defined(__MAWIN__)
    struct timeb tp;
    (void)ftime(&tp);
    return (unsigned long)(tp.time * 1000 + tp.millitm);
#endif
#if defined(__MAOSX__) || defined(__MAUNIX__)
	struct timeval t;
	struct tm dt;
	gettimeofday(&t, NULL);
	memset(&dt, 0, sizeof(dt));
	dt.tm_isdst = -1;
	dt.tm_hour = t.tv_sec / 3600;
	dt.tm_min = t.tv_sec / 60 - dt.tm_hour * 60;
	dt.tm_sec = t.tv_sec - dt.tm_hour * 3600 - dt.tm_min * 60;
	return mktime(&dt) * 1000 + t.tv_usec / 1000;
#endif
}


machar *maGetLocalDateTimeString(machar *retstring)
{
	time_t now;
	
	if(!retstring)
		return NULL;	
	time(&now);
	masprintf(retstring, MAXLOGLINELEN, 
 #if defined(__MAWIN__)
		L"%.24ls", _wctime(&now));
 #else
		L"%.24s", ctime(&now));
 #endif	
	return retstring;
}
	 
	 
#if defined(MAMEMTRACE)
 #include <stdlib.h>
 int g_nmamallocs = 0;
 int g_nmacallocs = 0;
 int g_ndupallocs = 0;
 int g_nmafrees = 0;
 int g_ndupfrees = 0;
 void *mamalloc(size_t a)
 { 
	void *p = NULL;
	if(a && (p = malloc(a)) != NULL)
	{
		machar mastr[MAXLOGLINELEN];
		masprintf(mastr, MAXLOGLINELEN, L"mamalloc %lu 0x%x", a, (int)p);
		maLog(mastr);
		g_nmamallocs++;
	}
	return p;
 }


 void *macalloc(size_t a, size_t b)
 { 
	void *p = NULL;
	if(a && b && (p = macalloc(a, b)) != NULL)
	{
		machar mastr[MAXLOGLINELEN];
		masprintf(mastr, MAXLOGLINELEN, L"mamalloc %lu 0x%x", a, (int)p);
		maLog(mastr);
		g_nmacallocs++;
	}
	return p;
 }


 void mafree(void *a)
 { 
	if(a)
	{
		machar mastr[MAXLOGLINELEN];
		free(a);
		masprintf(mastr, MAXLOGLINELEN, L"mafree 0x%x", (int)a);
		maLog(mastr);
		g_nmafrees++;
	}
 }


 void madupfree(void *a)
 { 
	if(a)
	{
		machar mastr[MAXLOGLINELEN];
		free(a);
		masprintf(mastr, MAXLOGLINELEN, L"madupfree 0x%x", (int)a);
		maLog(mastr);
		g_ndupfrees++;
	}
 }
#endif // MAMEMTRACE


int string2int(machar* digit, int* pval) // values >= 0
{
	machar *p = digit;
	
	*pval = 0;

	while(*p >= '0' && *p <='9')
	{
		*pval = (*pval * 10) + (*p - '0');
		p++;
	}

	if(*pval >= 0)
		return 0;

	return 1;
   
} // string2int


int maIsFullPath(fschar *path)
{
#if defined(__MAOSX__) || defined(__MAUNIX__) || defined(__MACUNIX__)
	if(path[0] == VOLSEP) // (rootdir)/dir/dir/file.
		return 1;
#endif

#if defined(__MAWIN__)
	if(fsstrchr(path, VOLSEP) != NULL) // drive:dir\dir\file.
		return 1;
#endif

	return 0; // must be relative
	
} // maIsFullPath


#if defined(__MAOSX__) // defined(__MAUNIX__)
 // use of unix process timer as one-shot
 int l_matimerbusy = 0;
 void *l_matimer_callback;
 void matimer_callback()
 {
    (*(void(*)())l_matimer_callback)();
	l_matimerbusy = 0;
 }


 int matimer_set_timeout(long ms, void *callback)
 {
	struct itimerval t_r;
	sigset_t sigset;

	if(callback && !l_matimerbusy)
	{
		l_matimerbusy = 1;
		l_matimer_callback = callback;
		signal(SIGALRM, (sig_t)&matimer_callback);
		sigemptyset(&sigset);
		sigaddset(&sigset, SIGALRM);
		sigprocmask(SIG_UNBLOCK, &sigset, NULL);
	}

	t_r.it_value.tv_usec = 1000 * ms;
	t_r.it_value.tv_sec = t_r.it_interval.tv_sec = t_r.it_interval.tv_usec = 0;

	if(setitimer(ITIMER_REAL, &t_r, NULL) != 0)
	{
		machar mastr[MAXLOGLINELEN];
		masprintf(mastr, MAXLOGLINELEN, L"matimer_set_timeout setitimer ms=%ld errno=%d", ms, errno);
		maLog(mastr);
		return 1;
	}	
	
	return 0;
	
 }  // matimer_set_timeout


 /*
 int matimer_unset_timeout()
 {
	struct itimerval no_timeout = {0};
	// no_timeout.it_value.tv_sec = no_timeout.it_value.tv_usec = 0;
	// no_timeout.it_interval.tv_sec = no_timeout.it_interval.tv_usec = 0;
	if(setitimer(ITIMER_REAL, &no_timeout, NULL))
	{
		machar mastr[MAXLOGLINELEN];
		masprintf(mastr, MAXLOGLINELEN, L"matimer_unset_timeout setitimer errno=%d", errno);
		maLog(mastr);
		return 1;
	}	
	
	return 0;
	
 } // matimer_unset_timeout
 */
#endif // defined(__MAOSX__) // defined(__MAUNIX__)


#if defined(__MAWIN__)
  typedef size_t uintptr_t; // not in MAWIN ?
#endif
#define ONEMASK ((size_t)(-1) / 0xFF)
// size_t strlen_utf8(const char *_s) // from Colin Percival http://www.daemonology.net/blog/2008-06-05-faster-utf8-strlen.html
size_t mastrlen_utf8(const char *_s)
{
	const char * s;
	size_t count = 0;
	size_t u;
	unsigned char b;

	/* Handle any initial misaligned bytes. */
	for (s = _s; (uintptr_t)(s) & (sizeof(size_t) - 1); s++)
	{
		b = *s;

		/* Exit if we hit a zero byte. */
		if(b == '\0')
			goto done;

		/* Is this byte NOT the first byte of a character? */
		count += (b >> 7) & ((~b) >> 6);
	}

	/* Handle complete blocks. */
	for (; ; s += sizeof(size_t))
	{
		/* Prefetch 256 bytes ahead. */
#if !defined(__MAWIN__)
		__builtin_prefetch(&s[256], 0, 0); // not in MAWIN ?
#endif
		/* Grab 4 or 8 bytes of UTF-8 data. */
		u = *(size_t *)(s);

		/* Exit the loop if there are any zero bytes. */
		if((u - ONEMASK) & (~u) & (ONEMASK * 0x80))
			break;

		/* Count bytes which are NOT the first byte of a character. */
		u = ((u & (ONEMASK * 0x80)) >> 7) & ((~u) >> 6);
		count += (u * ONEMASK) >> ((sizeof(size_t) - 1) * 8);
	}

	/* Take care of any left-over bytes. */
	for (; ; s++)
	{
		b = *s;

		/* Exit if we hit a zero byte. */
		if(b == '\0')
			break;

		/* Is this byte NOT the first byte of a character? */
		count += (b >> 7) & ((~b) >> 6);
	}

 done:
	return ((s - _s) - count);

} // mastrlen_utf8


machar *os2mafn(machar *mastr2, char *p)
{
	size_t nchars;
	
	if(!p || *p == '\0' || (nchars = mbstowcs(NULL, p, 0)) == (size_t)-1)
		mastr2[0] = L'\0';
	else
	{
		mbstowcs(mastr2, p, nchars); // assumes mastr2 is at least nchars + 1 machars
		mastr2[nchars] = L'\0';
	}
	
	return mastr2;
	
} // os2mafn
 

#if !defined(__MAWIN__)
 int maatoifn(machar *s)
 {
	return wcstoimax(s, NULL, 10);
 }
#endif


#if defined(MASETLOCALEWIN)
 maenvchar *normcharset(maenvchar *name) // returns normalized charset name
 {
	return(name);
 }
 #if defined(__MAOSX__) && !defined(__MACUNIX__)
  static maenvchar *mamaccharset(TextEncoding encoding)
  {
	return NULL;
  }
 #endif
#else
 #define digit(x) ((x) >= '0' && (x) <= '9')
 char l_nc[STRLEN32];
 char *normcharset(char *name) // returns normalized charset name
 {
  char *p;  
  if(!name)
    return NULL;
  
  if(!strcmp(name, "UTF-8"))
    return (char *)"UTF-8";
  if(!strcmp(name, "EUC-JP"))
    return (char *)"EUC-JP";
  if(!strcmp(name, "EUC-KR"))
    return (char *)"EUC-KR";
  if(!strcmp(name, "EUC-TW"))
    return (char *)"EUC-TW";
  if(!strcmp(name, "KOI8-R"))
    return (char *)"KOI8-R";
  if(!strcmp(name, "KOI8-U"))
    return (char *)"KOI8-U";
  if(!strcmp(name, "GBK"))
    return (char *)"GBK";
  if(!strcmp(name, "GB2312"))
    return (char *)"GB2312";
  if(!strcmp(name, "GB18030"))
    return (char *)"GB18030";
  if(!strcmp(name, "VSCII"))
    return (char *)"VSCII";
  
  /* ASCII comes in many names */
  if(!strcmp(name, "ASCII") ||
      !strcmp(name, "US-ASCII") ||
      !strcmp(name, "ANSI_X3.4-1968") ||
      !strcmp(name, "646") ||
      !strcmp(name, "ISO646") ||
      !strcmp(name, "ISO_646.IRV"))
    return (char *)"US-ASCII";

  /* ISO 8859 will be converted to "ISO-8859-x" */
  if((p = strstr(name, "8859-"))) {
    memcpy(l_nc, "ISO-8859-\0\0", 12);
    p += 5;
    if(digit(*p))
	{
      l_nc[9] = *p++;
      if(digit(*p)) l_nc[10] = *p++;
      return l_nc;
    }
  }

  /* Windows code pages will be converted to "WINDOWS-12xx" */
  if((p = strstr(name, "CP12")))
  {
    memcpy(l_nc, "WINDOWS-12\0\0", 13);
    p += 4;
    if(digit(*p))
	{
      l_nc[10] = *p++;
      if(digit(*p)) l_nc[11] = *p++;
      return l_nc;
    }
  }

  /* TIS-620 comes in at least the following two forms */
  if(!strcmp(name, "TIS-620") ||
      !strcmp(name, "TIS620.2533"))
    return (char *)"ISO-8859-11";

  /* For some, uppercase/lowercase might differ */
  if(!strcmp(name, "Big5") || !strcmp(name, "BIG5"))
    return (char *)"Big5";
  if(!strcmp(name, "Big5HKSCS") || !strcmp(name, "BIG5HKSCS"))
    return (char *)"Big5HKSCS";

  /* fallback return the unmodified name. */
  return name;

 } // normcharset


 #if defined(__MAOSX__) && !defined(__MACUNIX__)
  // code extracted from vim src os_macosx.c
  static char *mamaccharset(TextEncoding encoding)
  {
    int i;
    static struct
	{
		const char* name;
		TextEncoding encode;
    } encodetable[] = {
	/* ISO-8859 encodings family */
	{"latin1", kTextEncodingISOLatin1},
	{"latin2", kTextEncodingISOLatin2},
	{"latin3", kTextEncodingISOLatin3},
	{"latin4", kTextEncodingISOLatin4},
	{"latin5", kTextEncodingISOLatin5},
	{"latin6", kTextEncodingISOLatin6},
	{"latin7", kTextEncodingISOLatin7},
	{"latin8", kTextEncodingISOLatin8},
	{"latin9", kTextEncodingISOLatin9},
	{"iso-8859-1", kTextEncodingISOLatin1},
	{"iso-8859-2", kTextEncodingISOLatin2},
	{"iso-8859-3", kTextEncodingISOLatin3},
	{"iso-8859-4", kTextEncodingISOLatin4},
	{"iso-8859-5", kTextEncodingISOLatinCyrillic},
	{"iso-8859-6", kTextEncodingISOLatinArabic},
	{"iso-8859-7", kTextEncodingISOLatinGreek},
	{"iso-8859-8", kTextEncodingISOLatinHebrew},
	{"iso-8859-9", kTextEncodingISOLatin5},
	{"iso-8859-10", kTextEncodingISOLatin6},
	{"iso-8859-15", kTextEncodingISOLatin9},

	/* Unicode encodings. */
	/* TODO: Add other type of unicode */
	{"ucs-2", kTextEncodingMacUnicode},

	/* Japanese encoding aliases */
	{"cp932", kTextEncodingShiftJIS},
	{"shift-jis", kTextEncodingShiftJIS},
	{"euc-jp", kTextEncodingEUC_JP},
	{"iso-2022-jp", kTextEncodingISO_2022_JP},
	{"iso-2022-jp-1", kTextEncodingISO_2022_JP_1},
	{"iso-2022-jp-2", kTextEncodingISO_2022_JP_2},
	{"iso-2022-jp-3", kTextEncodingISO_2022_JP_3},

	/* Other aliases. These aliases in this block are just guessed. */
	/* TODO: Must be verified. */
	{"gb2312", kTextEncodingGB_2312_80},
	{"cp936", kTextEncodingMacChineseSimp},
	{"euc-cn", kTextEncodingEUC_CN},
	{"cp950", kTextEncodingMacChineseTrad},
	{"euc-tw", kTextEncodingEUC_TW},
	{"cp949", kTextEncodingMacKorean},
	{"euc-kr", kTextEncodingEUC_KR},

	/*
	 * All encodings supported by Macintosh.  You can find these values
	 * in a file:
	 *  /System/Library/Frameworks/CoreServices.framework/Versions/A/
	 *    Frameworks/CarbonCore.framework/Versions/A/Headers/TextCommon.h
	 */
	{"MacRoman", kTextEncodingMacRoman},
	{"MacJapanese", kTextEncodingMacJapanese},
	{"MacChineseTrad", kTextEncodingMacChineseTrad},
	{"MacKorean", kTextEncodingMacKorean},
	{"MacArabic", kTextEncodingMacArabic},
	{"MacHebrew", kTextEncodingMacHebrew},
	{"MacGreek", kTextEncodingMacGreek},
	{"MacCyrillic", kTextEncodingMacCyrillic},
	{"MacDevanagari", kTextEncodingMacDevanagari},
	{"MacGurmukhi", kTextEncodingMacGurmukhi},
	{"MacGujarati", kTextEncodingMacGujarati},
	{"MacOriya", kTextEncodingMacOriya},
	{"MacBengali", kTextEncodingMacBengali},
	{"MacTamil", kTextEncodingMacTamil},
	{"MacTelugu", kTextEncodingMacTelugu},
	{"MacKannada", kTextEncodingMacKannada},
	{"MacMalayalam", kTextEncodingMacMalayalam},
	{"MacSinhalese", kTextEncodingMacSinhalese},
	{"MacBurmese", kTextEncodingMacBurmese},
	{"MacKhmer", kTextEncodingMacKhmer},
	{"MacThai", kTextEncodingMacThai},
	{"MacLaotian", kTextEncodingMacLaotian},
	{"MacGeorgian", kTextEncodingMacGeorgian},
	{"MacArmenian", kTextEncodingMacArmenian},
	{"MacChineseSimp", kTextEncodingMacChineseSimp},
	{"MacTibetan", kTextEncodingMacTibetan},
	{"MacMongolian", kTextEncodingMacMongolian},
	{"MacEthiopic", kTextEncodingMacEthiopic},
	{"MacCentralEurRoman", kTextEncodingMacCentralEurRoman},
	{"MacVietnamese", kTextEncodingMacVietnamese},
	{"MacExtArabic", kTextEncodingMacExtArabic},
	{"MacSymbol", kTextEncodingMacSymbol},
	{"MacDingbats", kTextEncodingMacDingbats},
	{"MacTurkish", kTextEncodingMacTurkish},
	{"MacCroatian", kTextEncodingMacCroatian},
	{"MacIcelandic", kTextEncodingMacIcelandic},
	{"MacRomanian", kTextEncodingMacRomanian},
	{"MacCeltic", kTextEncodingMacCeltic},
	{"MacGaelic", kTextEncodingMacGaelic},
	{"MacKeyboardGlyphs", kTextEncodingMacKeyboardGlyphs},
	{"MacTradChinese", kTextEncodingMacTradChinese},
	{"MacRSymbol", kTextEncodingMacRSymbol},
	{"MacSimpChinese", kTextEncodingMacSimpChinese},
	{"MacGeez", kTextEncodingMacGeez},
	{"MacEastEurRoman", kTextEncodingMacEastEurRoman},
	{"MacUninterp", kTextEncodingMacUninterp},
	{"MacUnicode", kTextEncodingMacUnicode},
	{"MacFarsi", kTextEncodingMacFarsi},
	{"MacUkrainian", kTextEncodingMacUkrainian},
	{"MacInuit", kTextEncodingMacInuit},
	{"MacVT100", kTextEncodingMacVT100},
	{"MacHFS", kTextEncodingMacHFS},
	{"UnicodeDefault", kTextEncodingUnicodeDefault},
	{"UnicodeV1_1", kTextEncodingUnicodeV1_1},
	{"ISO10646_1993", kTextEncodingISO10646_1993},
	{"UnicodeV2_0", kTextEncodingUnicodeV2_0},
	{"UnicodeV2_1", kTextEncodingUnicodeV2_1},
	{"UnicodeV3_0", kTextEncodingUnicodeV3_0},
	{"UnicodeV3_1", kTextEncodingUnicodeV3_1},
	{"UnicodeV3_2", kTextEncodingUnicodeV3_2},
	{"ISOLatin1", kTextEncodingISOLatin1},
	{"ISOLatin2", kTextEncodingISOLatin2},
	{"ISOLatin3", kTextEncodingISOLatin3},
	{"ISOLatin4", kTextEncodingISOLatin4},
	{"ISOLatinCyrillic", kTextEncodingISOLatinCyrillic},
	{"ISOLatinArabic", kTextEncodingISOLatinArabic},
	{"ISOLatinGreek", kTextEncodingISOLatinGreek},
	{"ISOLatinHebrew", kTextEncodingISOLatinHebrew},
	{"ISOLatin5", kTextEncodingISOLatin5},
	{"ISOLatin6", kTextEncodingISOLatin6},
	{"ISOLatin7", kTextEncodingISOLatin7},
	{"ISOLatin8", kTextEncodingISOLatin8},
	{"ISOLatin9", kTextEncodingISOLatin9},
	{"DOSLatinUS", kTextEncodingDOSLatinUS},
	{"DOSGreek", kTextEncodingDOSGreek},
	{"DOSBalticRim", kTextEncodingDOSBalticRim},
	{"DOSLatin1", kTextEncodingDOSLatin1},
	{"DOSGreek1", kTextEncodingDOSGreek1},
	{"DOSLatin2", kTextEncodingDOSLatin2},
	{"DOSCyrillic", kTextEncodingDOSCyrillic},
	{"DOSTurkish", kTextEncodingDOSTurkish},
	{"DOSPortuguese", kTextEncodingDOSPortuguese},
	{"DOSIcelandic", kTextEncodingDOSIcelandic},
	{"DOSHebrew", kTextEncodingDOSHebrew},
	{"DOSCanadianFrench", kTextEncodingDOSCanadianFrench},
	{"DOSArabic", kTextEncodingDOSArabic},
	{"DOSNordic", kTextEncodingDOSNordic},
	{"DOSRussian", kTextEncodingDOSRussian},
	{"DOSGreek2", kTextEncodingDOSGreek2},
	{"DOSThai", kTextEncodingDOSThai},
	{"DOSJapanese", kTextEncodingDOSJapanese},
	{"DOSChineseSimplif", kTextEncodingDOSChineseSimplif},
	{"DOSKorean", kTextEncodingDOSKorean},
	{"DOSChineseTrad", kTextEncodingDOSChineseTrad},
	{"WindowsLatin1", kTextEncodingWindowsLatin1},
	{"WindowsANSI", kTextEncodingWindowsANSI},
	{"WindowsLatin2", kTextEncodingWindowsLatin2},
	{"WindowsCyrillic", kTextEncodingWindowsCyrillic},
	{"WindowsGreek", kTextEncodingWindowsGreek},
	{"WindowsLatin5", kTextEncodingWindowsLatin5},
	{"WindowsHebrew", kTextEncodingWindowsHebrew},
	{"WindowsArabic", kTextEncodingWindowsArabic},
	{"WindowsBalticRim", kTextEncodingWindowsBalticRim},
	{"WindowsVietnamese", kTextEncodingWindowsVietnamese},
	{"WindowsKoreanJohab", kTextEncodingWindowsKoreanJohab},
	{"US_ASCII", kTextEncodingUS_ASCII},
	{"JIS_X0201_76", kTextEncodingJIS_X0201_76},
	{"JIS_X0208_83", kTextEncodingJIS_X0208_83},
	{"JIS_X0208_90", kTextEncodingJIS_X0208_90},
	{"JIS_X0212_90", kTextEncodingJIS_X0212_90},
	{"JIS_C6226_78", kTextEncodingJIS_C6226_78},
	{"ShiftJIS_X0213_00", kTextEncodingShiftJIS_X0213_00},
	{"GB_2312_80", kTextEncodingGB_2312_80},
	{"GBK_95", kTextEncodingGBK_95},
	{"GB_18030_2000", kTextEncodingGB_18030_2000},
	{"KSC_5601_87", kTextEncodingKSC_5601_87},
	{"KSC_5601_92_Johab", kTextEncodingKSC_5601_92_Johab},
	{"CNS_11643_92_P1", kTextEncodingCNS_11643_92_P1},
	{"CNS_11643_92_P2", kTextEncodingCNS_11643_92_P2},
	{"CNS_11643_92_P3", kTextEncodingCNS_11643_92_P3},
	{"ISO_2022_JP", kTextEncodingISO_2022_JP},
	{"ISO_2022_JP_2", kTextEncodingISO_2022_JP_2},
	{"ISO_2022_JP_1", kTextEncodingISO_2022_JP_1},
	{"ISO_2022_JP_3", kTextEncodingISO_2022_JP_3},
	{"ISO_2022_CN", kTextEncodingISO_2022_CN},
	{"ISO_2022_CN_EXT", kTextEncodingISO_2022_CN_EXT},
	{"ISO_2022_KR", kTextEncodingISO_2022_KR},
	{"EUC_JP", kTextEncodingEUC_JP},
	{"EUC_CN", kTextEncodingEUC_CN},
	{"EUC_TW", kTextEncodingEUC_TW},
	{"EUC_KR", kTextEncodingEUC_KR},
	{"ShiftJIS", kTextEncodingShiftJIS},
	{"KOI8_R", kTextEncodingKOI8_R},
	{"Big5", kTextEncodingBig5},
	{"MacRomanLatin1", kTextEncodingMacRomanLatin1},
	{"HZ_GB_2312", kTextEncodingHZ_GB_2312},
	{"Big5_HKSCS_1999", kTextEncodingBig5_HKSCS_1999},
	{"NextStepLatin", kTextEncodingNextStepLatin},
	{"EBCDIC_US", kTextEncodingEBCDIC_US},
	{"EBCDIC_CP037", kTextEncodingEBCDIC_CP037},
	{"MultiRun", kTextEncodingMultiRun},

	/* Terminator */
	{NULL, kTextEncodingUnknown},
    };

    for (i = 0; encodetable[i].encode != kTextEncodingUnknown; i++)
		if(encoding == encodetable[i].encode)
			return (char *)encodetable[i].name;
						
    return NULL;

  } // mamaccharset
 #endif
#endif

#if defined(__MAUNIX__) || defined(__MACUNIX__)
 extern const char * locale_charset (void); // #include <localcharset.h>
#endif
maenvchar l_c[STRLEN32];
maenvchar *magetcharset() // suitable as parameter to iconv_open()
{
	maenvchar *c;
	
#if defined(__MAWIN__)
	c = masetlocale(LC_CTYPE, NULL); // this is locale, not charset
	if(c)
	{
		maenvstrcpy(l_c, c);
		return(normcharset(l_c));
	}
#endif

#if defined(__MAOSX__) && !defined(__MACUNIX__)
 #if (1) // carbon
    TextEncoding encoding = CFStringGetSystemEncoding();
 #else
	UpgradeScriptInfoToTextEncoding(kTextScriptDontCare, 4, 5, NULL, &encoding);
    UpgradeScriptInfoToTextEncoding(smSystemScript, kTextLanguageDontCare, kTextRegionDontCare, NULL, &encoding );
	// call GetTextEncodingBase to obtain the base encoding with variants and formats cut off.
 #endif

	// CFStringRef cfstr = CFStringGetNameOfEncoding(encoding);	// useless for iconv, e.g. kCFStringEncodingMacRoman 
	// CFRelease(cfstr);										// gives "Western (Mac OS Roman)"
	if(!(c = mamaccharset(encoding)))
		ShowErr(L"CFStringGetSystemEncoding unexpected char encoding"); // should show the encoding...
	else
	{
		maenvstrcpy(l_c, c);
		return(normcharset(l_c));
	}
#endif

#if defined(__MAUNIX__) || defined(__MACUNIX__)
 #if defined(HAVE_LANGINFO_CODESET)
	c = nl_langinfo(CODESET);
 #else
	c = (char *)locale_charset(); // in libgettextlib, and in old libiconv
 #endif
	if(c)
	{
		if(strstr(c, "ISO8859-1"))
			strcpy(l_c, "ISO8859-1");
		else if(strstr(c, "US-ASCII"))
			strcpy(l_c, "US-ASCII");
		else
			strcpy(l_c, c);		
		return(normcharset(l_c));
	}
#endif

// still nothing.. fallback values
#if defined(__MAWIN__)
	maenvstrcpy(l_c, maenvT("")); 
	ShowErr(L"magetcharset system encoding not found, using fallback '\"\"'");
#else
 #if defined(__MAOSX__) && !defined(__MACUNIX__)
	maenvstrcpy(l_c, maenvT("MACROMAN"));
	ShowErr(L"magetcharset system encoding not found, using fallback 'MACROMAN'");
 #else // not win not mac
	ShowErr(L"magetcharset system encoding not found, using fallback 'UTF-8'");
	maenvstrcpy(l_c, maenvT("UTF-8"));
 #endif
#endif
	// ShowErr(L"This can be changed by setting env variable 'MACHARSET' or 'MACHARSETKB'");

	return(normcharset(l_c));
	
} // magetcharset


#if defined(__MAOSX__) || defined(__MAUNIX__) // HAVE_ICONV
  // charset convert routines from libflac
  /*
   * Copyright (C) 2001 Edmund Grimley Evans <edmundo@rano.org>
   * 
   * Convert data from one encoding to another. Return:
   *
   *  -2 : memory allocation failed
   *  -1 : unknown encoding
   *   0 : data was converted exactly
   *   1 : data was converted inexactly
   *   2 : data was invalid (but still converted)
   *
   * We convert in two steps, via UTF-8, as this is the only
   * reliable way of distinguishing between invalid input
   * and valid input which iconv refuses to transliterate.
   * We convert from UTF-8 twice, because we have no way of
   * knowing whether the conversion was exact if iconv returns
   * E2BIG (due to a bug in the specification of iconv).
   * An alternative approach is to assume that the output of
   * iconv is never more than 4 times as long as the input,
   * but I prefer to avoid that assumption if possible.
  */
  int iconvert(const char *fromcode, const char *tocode,
	     const char *from, size_t fromlen,
	     char **to, size_t *tolen);
 
  /*
   * alloc - Convenience routines for safely allocating memory
   * Copyright (C) 2007  Josh Coalson
  */
  #ifndef FLAC__SHARE__ALLOC_H
   #define FLAC__SHARE__ALLOC_H
    /* WATCHOUT: for c++ you may have to #define __STDC_LIMIT_MACROS 1 real early
     * before #including this file,  otherwise SIZE_MAX might not be defined
     */
   #include <limits.h> /* for SIZE_MAX */
   #if !defined _MSC_VER && !defined __MINGW32__ && !defined __EMX__
	#include <stdint.h> /* for SIZE_MAX in case limits.h didn't get it */
   #endif
   #include <stdlib.h> /* for size_t, malloc(), etc */

   #ifndef FLaC__INLINE
    #define FLaC__INLINE
   #endif

   static FLaC__INLINE void *safe_malloc_(size_t size)
   {
	/* malloc(0) is undefined; FLAC src convention is to always allocate */
	if(!size)
		size++;
	return malloc(size);
   }


   static FLaC__INLINE void *safe_malloc_add_2op_(size_t size1, size_t size2)
   {
	size2 += size1;
	if(size2 < size1)
		return 0;
	return safe_malloc_(size2);
   }


   static FLaC__INLINE void *safe_realloc_add_2op_(void *ptr, size_t size1, size_t size2)
   {
	size2 += size1;
	if(size2 < size1)
		return 0;
	return realloc(ptr, size2);
   }
  #endif // FLAC__SHARE__ALLOC_H


   /*
    * Convert data from one encoding to another. Return:
    *
    *  -2 : memory allocation failed
    *  -1 : unknown encoding
    *   0 : data was converted exactly
    *   1 : data was converted inexactly
    *   2 : data was invalid (but still converted)
    *
    * We convert in two steps, via UTF-8, as this is the only
    * reliable way of distinguishing between invalid input
    * and valid input which iconv refuses to transliterate.
    * We convert from UTF-8 twice, because we have no way of
    * knowing whether the conversion was exact if iconv returns
    * E2BIG (due to a bug in the specification of iconv).
    * An alternative approach is to assume that the output of
    * iconv is never more than 4 times as long as the input,
    * but I prefer to avoid that assumption if possible.
   */
   int iconvert(const char *fromcode, const char *tocode,
	     const char *from, size_t fromlen,
	     char **to, size_t *tolen)
   {
     int ret = 0;
     iconv_t cd1, cd2;
     char *ib;
     char *ob;
     char *utfbuf = 0, *outbuf, *newbuf;
     size_t utflen, outlen, ibl, obl, k;
     char tbuf[2048];

     cd1 = iconv_open("UTF-8", fromcode);
     if(cd1 == (iconv_t)(-1))
       return -1;

     cd2 = (iconv_t)(-1);
     /* Don't use strcasecmp() as it's locale-dependent. */
	 if(!strchr("Uu", tocode[0]) ||
		!strchr("Tt", tocode[1]) ||
		!strchr("Ff", tocode[2]) ||
		tocode[3] != '-' ||
		tocode[4] != '8' ||
		tocode[5] != '\0')
	 {
		/*
		#if 0 / no translit
		char *tocode1;
		/w*
		* Try using this non-standard feature of glibc and libiconv.
		* This is deliberately not a config option as people often
		* change their iconv library without rebuilding applications.
		*w/
		tocode1 = (char *)safe_malloc_add_2op_(strlen(tocode), 11);
		if(!tocode1)
			goto fail;
		strcpy(tocode1, tocode);
		strcat(tocode1, "//TRANSLIT");
		cd2 = iconv_open(tocode1, "UTF-8");
		free(tocode1);		
		#else
		*/
		cd2 = iconv_open(tocode, "UTF-8");
		// #endif
		if(cd2 == (iconv_t)(-1))
			cd2 = iconv_open(tocode, fromcode);
		if(cd2 == (iconv_t)(-1))
		{
			iconv_close(cd1);
			return -1;
		}
	}

    utflen = 1; /*fromlen * 2 + 1; XXX */
    utfbuf = (char *)malloc(utflen);
    if(!utfbuf)
		goto fail;

	/* Convert to UTF-8 */
	ib = (char *)from;
	ibl = fromlen;
	ob = utfbuf;
	obl = utflen;
	for (;;)
	{
		k = iconv(cd1, (ICONV_CONST char **)&ib, &ibl, &ob, &obl);
		/*
		assert((!k && !ibl) ||
	   (k == (size_t)(-1) && errno == E2BIG && ibl && obl < 6) ||
	   (k == (size_t)(-1) && (errno == EILSEQ || errno == EINVAL) && ibl));
		*/
		if(!ibl)
		break;
		if(obl < 6)
		{
			/* Enlarge the buffer */
			if(utflen*2 < utflen) /* overflow check */
				goto fail;
			utflen *= 2;
			newbuf = (char *)realloc(utfbuf, utflen);
			if(!newbuf)
				goto fail;
			ob = (ob - utfbuf) + newbuf;
			obl = utflen - (ob - newbuf);
			utfbuf = newbuf;
		}
		else
		{
			/* Invalid input */
			ib++, ibl--;
			*ob++ = '#', obl--;
			ret = 2;
			iconv(cd1, 0, 0, 0, 0);
		}
   }

   if(cd2 == (iconv_t)(-1))
   {
		/* The target encoding was UTF-8 */
		if(tolen)
			*tolen = ob - utfbuf;
		if(!to) {
			free(utfbuf);
			iconv_close(cd1);
		return ret;
		}
		newbuf = (char *)safe_realloc_add_2op_(utfbuf, (ob - utfbuf), /*+*/1);
		if(!newbuf)
			goto fail;
		ob = (ob - utfbuf) + newbuf;
		*ob = '\0';
		*to = newbuf;
		iconv_close(cd1);
		return ret;
   }

   /* Truncate the buffer to be tidy */
   utflen = ob - utfbuf;
   newbuf = (char *)realloc(utfbuf, utflen);
   if(!newbuf)
		goto fail;
   utfbuf = newbuf;

   /* Convert from UTF-8 to discover how long the output is */
   outlen = 0;
   ib = utfbuf;
   ibl = utflen;
   while(ibl)
   {
		ob = tbuf;
		obl = sizeof(tbuf);
		k = iconv(cd2, (ICONV_CONST char **)&ib, &ibl, &ob, &obl);
		/*
		assert((k != (size_t)(-1) && !ibl) ||
			(k == (size_t)(-1) && errno == E2BIG && ibl) ||
			(k == (size_t)(-1) && errno == EILSEQ && ibl));
		*/
		if(ibl && !(k == (size_t)(-1) && errno == E2BIG)) 
		{
			/* Replace one character */
			char *tb = (char *)"?";
			size_t tbl = 1;

			outlen += ob - tbuf;
			ob = tbuf;
			obl = sizeof(tbuf);
			k = iconv(cd2, (ICONV_CONST char **)&tb, &tbl, &ob, &obl);
			/*
			assert((!k && !tbl) ||
				(k == (size_t)(-1) && errno == EILSEQ && tbl));
			*/
			for (++ib, --ibl; ibl && (*ib & 0x80); ib++, ibl--)
				;
		}
		outlen += ob - tbuf;
    }
    ob = tbuf;
    obl = sizeof(tbuf);
    k = iconv(cd2, 0, 0, &ob, &obl);
    // assert(!k);
    outlen += ob - tbuf;

    /* Convert from UTF-8 for real */
    outbuf = (char *)safe_malloc_add_2op_(outlen, /*+*/1);
    if(!outbuf)
		goto fail;
    ib = utfbuf;
    ibl = utflen;
    ob = outbuf;
    obl = outlen;
    while(ibl)
    {
		k = iconv(cd2, (ICONV_CONST char **)&ib, &ibl, &ob, &obl);
		/*
		assert((k != (size_t)(-1) && !ibl) ||
			(k == (size_t)(-1) && errno == EILSEQ && ibl));
		*/
		if(k && !ret)
			ret = 1;
		if(ibl && !(k == (size_t)(-1) && errno == E2BIG))
		{
			/* Replace one character */
			char *tb = (char *)"?";
			size_t tbl = 1;

			k = iconv(cd2, (ICONV_CONST char **)&tb, &tbl, &ob, &obl);
			/*
			assert((!k && !tbl) ||
				(k == (size_t)(-1) && errno == EILSEQ && tbl));
			*/
			for (++ib, --ibl; ibl && (*ib & 0x80); ib++, ibl--)
				;
		}
    }
    k = iconv(cd2, 0, 0, &ob, &obl);
    /*
    assert(!k);
    assert(!obl);
    */
    *ob = '\0';

    free(utfbuf);
    iconv_close(cd1);
    iconv_close(cd2);
    if(tolen)
		*tolen = outlen;
    if(!to) {
		free(outbuf);
		return ret;
    }
    *to = outbuf;
	return ret;

    fail:
    if(0 != utfbuf)
		free(utfbuf);
    iconv_close(cd1);
    if(cd2 != (iconv_t)(-1))
		iconv_close(cd2);
    return -2;

   } // iconvert


   static int convert_buffer(const char *fromcode, const char *tocode,
			  const char *from, size_t fromlen,
			  char **to, size_t *tolen)
   {
	return iconvert(fromcode, tocode, from, fromlen, to, tolen);
	/*
	-2 : memory allocation failed
	-1 : unknown encoding
	 0 : data was converted exactly
	 1 : data was converted inexactly
	 2 : data was invalid (but still converted)
	*/
	/* int ret = -1;
	ret = iconvert(fromcode, tocode, from, fromlen, to, tolen);
	if(ret != -1)
			return ret;
		#ifndef HAVE_ICONV /w* should be ifdef USE_CHARSET_CONVERT *w/
		 ret = charset_convert(fromcode, tocode, from, fromlen, to, tolen);
		 if(ret != -1)
			return ret;
		#endif
	return ret;
	*/

  } // convert_buffer


  static int convert_string(const char *fromcode, const char *tocode,
			  const char *from, char **to, char replace, size_t *tolen)
  {
	int ret;
	size_t fromlen;
	char *s;

	fromlen = strlen(from);
	ret = convert_buffer(fromcode, tocode, from, fromlen, to, tolen);
	/*
	-2 : memory allocation failed
	-1 : unknown encoding
	 0 : data was converted exactly
	 1 : data was converted inexactly
	 2 : data was invalid (but still converted)
	*/
	if(ret == -2)
		return -1;
	if(ret != -1)
		return ret;

	s = (char *)safe_malloc_add_2op_(fromlen, /*+*/1);
	if(!s)
		return -1;
	strcpy(s, from);
	*to = s;
	if(replace)
		for (; *s; s++)
			if(*s & ~0x7f)
				*s = replace;
	return 3;
  }


int maconv_charset2utf8(const char *from, char **to, size_t *tolen)
{	
	return convert_string(g_charset, "UTF-8", from, to, '?', tolen);
}


/*
int maconv_charsetkb2charset(const char *from, char **to, size_t *tolen)
{	
	return convert_string(g_charsetkb, g_charset, from, to, '?', tolen);
}
*/


#if defined(MAGUICARBON)
 int maconv_charsetkb2utf8(const char *from, char **to, size_t *tolen)
 {
	return convert_string(g_charsetkb, "UTF-8", from, to, '?', tolen);
 }
#endif


#if !defined(MASETLOCALEWIN)
 int maconv_macharset2utf8(const char *from, char **to, size_t *tolen)
 {	
	return convert_string(g_macharset, "UTF-8", from, to, '?', tolen);
 }

 
 #if defined(MAGUICARBON)
  int maconv_macharsetkb2utf8(const char *from, char **to, size_t *tolen)
  {	
	return convert_string(g_macharsetkb, "UTF-8", from, to, '?', tolen);
  }
 #endif
#endif


#if defined(__MAOSX__) || defined(__MACUNIX__)
 int maconv_charset2utf8mac(const char *from, char **to, size_t *tolen)
 {
	return convert_string(g_charset, "UTF-8-MAC", from, to, '?', tolen);
 }


 int maconv_utf8mac2utf8(const char *from, char **to, size_t *tolen)
 {
	return convert_string("UTF-8-MAC", "UTF-8", from, to, '?', tolen);
 }

	
 int maconv_utf82utf8mac(const char *from, char **to, size_t *tolen)
 {
	return convert_string("UTF-8", "UTF-8-MAC", from, to, '?', tolen);
 }
#endif


#if defined(MAGUICARBON)
 /*
 int maconv_utf8mac2charset(const char *from, char **to, size_t *tolen)
 {
	return convert_string("UTF-8-MAC", g_charset, from, to, '?', tolen);
 }
 */
 
 
 int maconv_utf82charset(const char *from, char **to, size_t *tolen)
 {
	return convert_string("UTF-8", g_charset, from, to, '?', tolen);
 }
#endif


int maconv_utf82utf16le(const char *from, char **to, size_t *tolen)
{
	return convert_string("UTF-8", "UTF-16LE", from, to, '?', tolen);
}

 
int maconv_iso885912utf8(const char *from, char **to, size_t *tolen)
{
	return convert_string("ISO8859-1", "UTF-8", from, to, '?', tolen);
}


/*
int maconv_iso885912charset(const char *from, char **to, size_t *tolen)
{
	return convert_string("ISO8859-1", g_charset, from, to, '?', tolen);
}


int maconv_utf82macroman(const char *from, char **to, size_t *tolen)
{
	return convert_string("UTF-8", "MACROMAN", from, to, '?', tolen);
}
*/


size_t StrCHARSETtoUTF8(char *strconvert)
{
	char *converted;
	size_t tolen;
		
	if(strconvert == NULL || strconvert[0] == fsT('\0') || !strcmp(g_charset, "UTF-8"))
		return 0;
		
	tolen = 0;
	if(maconv_charset2utf8(strconvert, &converted, &tolen) < 0)
		return 0;
	
	strcpy(strconvert, converted);

	free(converted);

	return tolen;

} // StrCHARSETtoUTF8 


/*
size_t StrCHARSETKBtoStrCHARSET(char *strconvert)
{
	char *converted;
	size_t tolen;
	
	if(strconvert == NULL || strconvert[0] == fsT('\0') || !strcmp(g_charsetkb, g_charset))
		return 0;
		
	tolen = 0;
	if(maconv_charsetkb2charset(strconvert, &converted, &tolen) < 0)
		return 0;
	
	strcpy(strconvert, converted);

	free(converted);

	return tolen;

} // StrCHARSETKBtoStrCHARSET
*/


#if defined(MAGUICARBON)
 size_t StrCHARSETKBtoUTF8(char *strconvert)
 {
	char *converted;
	size_t tolen;
	
	if(strconvert == NULL || strconvert[0] == fsT('\0') || !strcmp(g_charsetkb, "UTF-8"))
		return 0;
		
	tolen = 0;
	if(maconv_charsetkb2utf8(strconvert, &converted, &tolen) < 0)
		return 0;
	
	strcpy(strconvert, converted);

	free(converted);

	return tolen;

} // StrCHARSETKBtoUTF8 
#endif


#if !defined(MASETLOCALEWIN)
 size_t StrMACHARSETtoUTF8(char *strconvert)
 {
	char *converted;
	size_t tolen;
	
	if(strconvert == NULL || strconvert[0] == fsT('\0') || !strcmp(g_macharset, "UTF-8"))
		return 0;
		
	tolen = 0;
	if(maconv_macharset2utf8(strconvert, &converted, &tolen) < 0)
		return 0;
	
	strcpy(strconvert, converted);

	free(converted);

	return tolen;

 } // StrMACHARSETtoUTF8 


 #if defined(MAGUICARBON)
  size_t StrMACHARSETKBtoUTF8(char *strconvert)
  {
	char *converted;
	size_t tolen;
	
	if(strconvert == NULL || strconvert[0] == fsT('\0') || !strcmp(g_macharsetkb, "UTF-8"))
		return 0;
		
	tolen = 0;
	if(maconv_macharsetkb2utf8(strconvert, &converted, &tolen) < 0)
		return 0;
	
	strcpy(strconvert, converted);

	free(converted);

	return tolen;

  } // StrMACHARSETKBtoUTF8 
 #endif
#endif


#if defined(__MAOSX__) || defined(__MACUNIX__)
 size_t StrCHARSETtoUTF8MAC(fschar *strconvert)
 {
	char *converted;
	size_t tolen;
	
	if(strconvert == NULL || strconvert[0] == fsT('\0') || !strcmp(g_charset, "UTF-8-MAC"))
		return 0;
		
	tolen = 0;
	if(maconv_charset2utf8mac(strconvert, &converted, &tolen) < 0)
		return 0;
	
	strcpy(strconvert, converted);

	free(converted);

	return tolen;
	
 } // StrCHARSETtoUTF8MAC


 size_t StrUTF8MACtoUTF8(fschar *strconvert)
 {
	char *converted = NULL;
	size_t tolen;
	
	if(strconvert == NULL || strconvert[0] == fsT('\0'))
		return 0;
		
	tolen = 0;
	if(maconv_utf8mac2utf8(strconvert, &converted, &tolen) < 0)
		return 0;
	
	if(converted == NULL)
		return 0;
	
	strcpy(strconvert, converted);

	free(converted);

	return tolen;
	
 } // StrUTF8MACtoUTF8


 size_t StrUTF8toUTF8MAC(fschar *strconvert)
 {
	char *converted;
	size_t tolen;
	
	if(strconvert == NULL || strconvert[0] == fsT('\0'))
		return 0;
		
	tolen = 0;
	if(maconv_utf82utf8mac(strconvert, &converted, &tolen) < 0)
		return 0;
	
	strcpy(strconvert, converted);

	free(converted);

	return tolen;

 } // StrUTF8toUTF8MAC
 #endif


 #if defined(MAGUICARBON)
  /*
  size_t StrUTF8MACtoCHARSET(fschar *strconvert)
  {
	char *converted;
	size_t tolen;
	
	if(strconvert == NULL || strconvert[0] == fsT('\0') || !strcmp(g_charset, "UTF-8-MAC"))
		return 0;

	tolen = 0;
	if(maconv_utf8mac2charset(strconvert, &converted, &tolen) < 0)
		return 0;

	memcpy(strconvert, converted, tolen);
	strconvert[tolen] = 0;
	strconvert[tolen + 1] = 0;

	free(converted);

	return tolen;

  } // StrUTF8MACtoCHARSET
  */

  size_t StrUTF8toCHARSET(fschar *strconvert)
  {
	char *converted;
	size_t tolen;
	
	if(strconvert == NULL || strconvert[0] == fsT('\0') || !strcmp(g_charset, "UTF-8"))
		return 0;
		
	tolen = 0;
	if(maconv_utf82charset(strconvert, &converted, &tolen) < 0)
		return 0;
	
	memcpy(strconvert, converted, tolen);
	strconvert[tolen] = 0;
	strconvert[tolen + 1] = 0;

	free(converted);

	return tolen;
	
  } // StrUTF8toCHARSET
 #endif


 size_t StrUTF8toUTF16LE(fschar *strconvert)
 {
	char *converted;
	size_t tolen;
	
	if(strconvert == NULL || strconvert[0] == fsT('\0'))
		return 0;
		
	tolen = 0;
	if(maconv_utf82utf16le(strconvert, &converted, &tolen) < 0)
		return 0;
	
	memcpy(strconvert, converted, tolen);
	strconvert[tolen] = 0;
	strconvert[tolen + 1] = 0;
	
	free(converted);

	return tolen;
	
 } // StrUTF8toUTF16LE


 size_t StrISO88591toUTF8(char *strconvert)
 {
	char *converted;
	size_t tolen;
	
	if(strconvert == NULL || strconvert[0] == fsT('\0'))
		return 0;
		
	tolen = 0;
	if(maconv_iso885912utf8(strconvert, &converted, &tolen) < 0)
			return 0;
	
	strcpy(strconvert, converted);

	free(converted);

	return tolen;

 } // StrISO88591toUTF8


 /*
 size_t StrISO88591toCHARSET(char *strconvert)
 {
	char *converted;
	size_t tolen;
	
	if(strconvert == NULL || strconvert[0] == fsT('\0') || !strcmp(g_charset, "ISO8859-1"))
		return 0;
		
	tolen = 0;
	if(maconv_iso885912charset(strconvert, &converted, &tolen) < 0)
			return 0;
	
	memcpy(strconvert, converted, tolen);
	strconvert[tolen] = 0;
	strconvert[tolen + 1] = 0;

	free(converted);

	return tolen;

 } // StrISO88591toCHARSET


 size_t StrUTF8toMACROMAN(char *strconvert)
 {
	char *converted;
	size_t tolen;
	
	if(strconvert == NULL || strconvert[0] == fsT('\0'))
		return 0;
		
	tolen = 0;
	if(maconv_utf82macroman(strconvert, &converted, &tolen) < 0)
			return 0;
	
	strcpy(strconvert, converted);

	free(converted);

	return tolen;

 } // StrUTF8toMACROMAN
 */
#endif // defined(__MAOSX__) || defined(__MACUNIX__)


#if !defined(__MAWIN__)
 locale_t l_localeUTF8 = NULL;
#endif
#if defined(__MAWIN__)
 size_t utf8_to_wclocal(const char *in, size_t ninbytes, wchar_t *out, size_t noutchars, int flags)
 {
	char *pin;
	size_t nchars;
	size_t len;
	size_t lninbytes = ninbytes;

	if(flags & UTF8_SKIP_BOM)
	{
		if(in[0] == '\xef' && in[1] == '\xbb' && in[2] == '\xbf') // BOM_UTF-8
		{
			if((lninbytes -= 3) <= 3) // nothing but a bom
			{
				out[0] = L'\0';
				return 0;
			}
			pin = (char *)in + 3;
		}
		else
			pin = (char *)in;
	}
	else
		pin = (char *)in;

	if((len = (size_t)MultiByteToWideChar(CP_UTF8, 0, pin, (int)lninbytes, NULL, 0)) == 0) // ninbytes can be -1 if it is null terminated
	{
		out[0] = L'\0';
		nchars = 0;
	}
	else
	{
		if(len > noutchars)
			len = noutchars;
		nchars = (size_t)MultiByteToWideChar(CP_UTF8, 0, pin, (int)lninbytes, out, (int)len);
	}
	
	return nchars;

 } // utf8_to_wclocal win


 size_t wclocal_to_utf8(const wchar_t *in, size_t ninchars, char *out, size_t noutbytes)
 {
	size_t nbytes;
	size_t len;
	
	len = (size_t)WideCharToMultiByte(CP_UTF8, 0, in, (int)ninchars, NULL, 0, NULL, NULL);

	if(len <= 0)
	{
		out[0] = '\0';
		nbytes = 0;
	}
	else
	{
		if(len > noutbytes)
			len = noutbytes;
		nbytes = (size_t)WideCharToMultiByte(CP_UTF8, 0, in, (int)ninchars, out, (int)len, NULL, NULL);
	}
	
	// out[nbytes] = '\0';
	
	return nbytes;

 } // wclocal_to_utf8
#else // not win
 size_t utf8_to_wclocal(const char *in, size_t ninbytes, wchar_t *out, size_t noutchars, int flags)
 {
	size_t nchars;
	
	if(in == NULL || ninbytes == 0 || out == NULL || noutchars == 0)
		return (0);
		
	if((nchars = mbstowcs(NULL, in, 0)) <= 0) // == -1)
		out[0] = L'\0';
	else
	{
		if(nchars > noutchars / sizeof(wchar_t))
			nchars = noutchars / sizeof(wchar_t);
		mbstowcs(out, in, nchars);
		// out[nchars] = L'\0';
	}

	return (nchars);

 } // utf8_to_wclocal

	
 size_t wclocal_to_utf8(const wchar_t *in, size_t ninchars, char *out, size_t noutbytes)
 {
	size_t size;
	
	if(in == NULL || ninchars == 0 || out == NULL || noutbytes == 0)
		return 0;
		
	if((size = wcstombs(NULL, in, 0)) <= 0) // == -1)
		out[0] = '\0';
	else
	{
		if(size > noutbytes)
			size = noutbytes;
		wcstombs(out, in, size);
		//out[size] = '\0';
	}

	return size;

 } // wclocal_to_utf8
#endif
 

#if defined(MAPLAYSBSMS)
 size_t mafilereadraw(void **pmem, fschar *filepath, int sizeonly)
 {
	size_t fsize;
	FILE *fp;
 #if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
 #endif
	machar mastr[MAXLOGLINELEN];

	if(filepath == NULL || (filepath[0] == fsT('\0')) || (!sizeonly && pmem == NULL))
	{
		maLog(L"mafilereadraw bad parameters");
		return 0;
	}

	if((fp = mafopen(filepath, (fschar *)fsT("rb"))) == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"mafilereadraw mafopen failed on '%ls'",
																		fs2ma(mastr2, filepath));
		maLog(mastr);
		return 0;
	}

	fsize = maGetFileSize(fp, 1); // nomem

	if(sizeonly)
	{
		fclose(fp);
		return fsize;
	}

	if((*pmem = MAMALLOC(fsize)) == NULL)
	{
		fclose(fp);
		masprintf(mastr, MAXLOGLINELEN, L"mafilereadraw MAMALLOC failed size=%ld", (long)fsize);
		maLog(mastr);
		return 0;
	}

	if(fread(*pmem, 1, fsize, fp) != fsize)
	{
		MAFREE(pmem);
		fclose(fp);
		masprintf(mastr, MAXLOGLINELEN, L"mafilereadraw fread failed errno %d", errno);
		maLog(mastr);
		return 0;
	}

	fclose(fp);
	
	return fsize;

} // mafilereadraw
#endif // MAPLAYSBSMS


size_t mafileread(machar **pmastr, fschar *filepath)
{
	size_t fsize;
	FILE *fp;
	size_t wlen;
	int bomsize = 0;
	char *p0;
	char p1[16];
#if !defined(__MAWIN__)
	machar mastr2[MAXLOGLINELEN];	
#endif
	machar mastr[MAXLOGLINELEN];

	if(filepath == NULL || (filepath[0] == fsT('\0')) || pmastr == NULL)
		return 0;

	if((fp = mafopen(filepath, (fschar *)fsT("rb"))) == NULL)
	{
		masprintf(mastr, MAXLOGLINELEN, L"mafileread mafopen failed on '%ls'",
																		fs2ma(mastr2, filepath));
		maLog(mastr);
		return 0;
	}	
	fsize = maGetFileSize(fp, 1);

	if(fsize >= 4)
	{
		if(fread(p1, 1, 4, fp) != 4)
		{
			fclose(fp);
			masprintf(mastr, MAXLOGLINELEN, L"mafileread fread failed errno %d", errno);
			maLog(mastr);
			return 0;
		}
		if((p1[0] == '\xff' && p1[1] == '\xfe' && p1[2] == '\x0' && p1[3] == '\x0') ||	// BOM_UTF-32LE
		   (p1[0] == '\x0'  && p1[1] ==  '\x0' && p1[2] == '\xfe' && p1[3] == '\xff'))	// BOM_UTF-32BE
			bomsize = 4;															
		else if(p1[0] == '\xef' && p1[1] == '\xbb' && p1[2] == '\xbf')					// BOM_UTF-8
			bomsize = 3;
		else if((p1[1] == '\xff' && p1[1] == '\xfe') ||									// BOM_UTF-16LE
				(p1[1] == '\xfe' && p1[1] == '\xff'))									// BOM_UTF-16BE
			bomsize = 2; 
	}
	else if(fsize >= 3)
	{
		if(fread(p1, 1, 3, fp) != 3)
		{
			fclose(fp);
			masprintf(mastr, MAXLOGLINELEN, L"mafileread fread failed errno %d", errno);
			maLog(mastr);
			return 0;
		}
		if(p1[0] == '\xef' && p1[1] == '\xbb' && p1[2] == '\xbf')						// BOM_UTF-8
			bomsize = 3;
		else if((p1[1] == '\xff' && p1[1] == '\xfe') ||									// BOM_UTF-16LE
				(p1[1] == '\xfe' && p1[1] == '\xff'))									// BOM_UTF-16BE
			bomsize = 2;
	}
	else if(fsize >= 2)
	{
		if(fread(p1, 1, 2, fp) != 2)
		{
			fclose(fp);
			masprintf(mastr, MAXLOGLINELEN, L"mafileread fread failed errno %d", errno);
			maLog(mastr);
			return 0;
		}
		if((p1[1] == '\xff' && p1[1] == '\xfe') ||										// BOM_UTF-16LE
			(p1[1] == '\xfe' && p1[1] == '\xff'))										// BOM_UTF-16BE
			bomsize = 2; 
	}

	if(bomsize == 0)
		rewind(fp);
	else
	{
		fseek(fp, bomsize, SEEK_SET);
		fsize -= bomsize;
	}
	
#if defined(__MAOSX__) || defined(__MACUNIX__)
	if(bomsize == 0)
		wlen = (fsize + 8) * 4; // max sizeof(UTF-8) for conversion ansi to utf8 below
	else
#endif
		wlen = fsize + 8;
	if((p0 = (char *)MAMALLOC(wlen)) == NULL)
	{
		fclose(fp);
		masprintf(mastr, MAXLOGLINELEN, L"mafileread MAMALLOC failed size=%ld", (long)wlen);
		maLog(mastr);
		return 0;
	}
	
	if(fread(p0, 1, fsize, fp) != fsize)
	{
		MAFREE(p0);
		fclose(fp);
		masprintf(mastr, MAXLOGLINELEN, L"mafileread fread failed errno %d", errno);
		maLog(mastr);
		return 0;
	}
	fclose(fp);
	p0[fsize] = fsT('\0');

	// if the playlist is ansi instead of the expected utf-8
	// ... do nothing
	
	// utf8 to machar
	size_t len;
	wlen = (fsize + 8) * sizeof(wchar_t);
	if((*pmastr = (machar *)MAMALLOC(wlen)) == NULL)
	{
		MAFREE(p0);
		masprintf(mastr, MAXLOGLINELEN, L"mafileread MAMALLOC failed size=%ld", (long)wlen);
		maLog(mastr);
		return 0;
	}
	len = utf8_to_wclocal((char *)p0, fsize, (wchar_t *)*pmastr, wlen, UTF8_IGNORE_ERROR + UTF8_SKIP_BOM);
	if(len >= wlen) // never happens?
		len = wlen - 1;
	(*pmastr)[len] = L'\0';
	MAFREE(p0);
	if(bomsize != 3)
	{
		masprintf(mastr, MAXLOGLINELEN, L"non-UTF-8 encoded file '%ls'", fs2ma(mastr2, filepath));
		maLog(mastr);	
	}	

	return len;

} // mafileread


size_t mafilewrite(size_t nchars, machar *mastr2, FILE *fp)
{
	size_t len;
	size_t size;
	size_t memsize;
	char *p0;
	machar mastr[MAXLOGLINELEN];

	if(fp == NULL)
		return 0;
	if(nchars > 0)
	{
		len = nchars;
		memsize = (len + 8) * 4; // max sizeof(UTF-8)
	}
	else
	{
		len = mastrlen(mastr2);
		memsize = (len + 2) * 6; // max sizeof(UTF-8) // why not 4 like everywhere else?
	}
	if(len <= 0)
		return 0;
	if((p0 = (char *)MAMALLOC(memsize)) == NULL)
	{
		maLog(L"mafilewrite MAMALLOC failed");
		return 0;
	}
	size = wclocal_to_utf8((wchar_t *)mastr2, len, p0, len * 4);
	if(size > 0)
		if(fwrite(p0, 1, size, fp) != size)
		{
			masprintf(mastr, MAXLOGLINELEN, L"mafilewrite fwrite errno=%d", errno);
			maLog(mastr);
			size = 0;
		}
	MAFREE(p0);

	return size;

} //mafilewrite


machar *fs2mafn(machar *mastr2, fschar *fsstr)
{
	if(!mastr2)
		return NULL;
	if(!fsstr || fsstr[0] == fsT('\0'))
	{
		mastr2[0] = L'\0';
		return mastr2;
	}
#if defined(__MAWIN__)
	mastrcpy(mastr2, fsstr); // both are wchar_t
#else // fschar is char, machar is wchar_t
	size_t charsin;
	size_t bytesin;
	size_t charsout; // in machars
	charsin = mastrlen_utf8(fsstr); // in utf8 chars
	bytesin = strlen(fsstr); // in bytes
	// assume mastr2 is big enough
	charsout = utf8_to_wclocal(fsstr, bytesin, mastr2, charsin * sizeof(machar), UTF8_SKIP_BOM); // + UTF8_IGNORE_ERROR);
	mastr2[charsout] = L'\0';
#endif

	return mastr2;

} // fs2mafn


fschar *ma2fsfn(fschar *fsstr, machar *mastr2)
{
	if(!fsstr)
		return NULL;
	if(!mastr2 || mastr2[0] == L'\0')
	{
		fsstr[0] = fsT('\0');
		return fsstr;
	}
#if defined(__MAWIN__)
	fsstrcpy(fsstr, mastr2); // both are wchar_t
#else // fschar is char, machar is wchar_t
	size_t charsin;
	size_t bytesin;
	size_t bytesout; // in bytes

	charsin = mastrlen(mastr2); // in machars
	bytesin = charsin * 4; // max in bytes // * max size(UTF-8)
	// assume fsstr is big enough
	bytesout = wclocal_to_utf8(mastr2, charsin, fsstr, bytesin);
	fsstr[bytesout] = fsT('\0');
#endif

	return fsstr;
	
} // ma2fsfn


#if defined(MAGUIGTK2)
 size_t offset_wchar_to_utf8(wchar_t *mastr2, size_t maoffset)
 {
	size_t maxbytes;
	size_t utf8offset;
	size_t utf8bytes;
	wchar_t c;
	char *utf8str;

	maxbytes = (maoffset + 2) * 4; // max sizeof(UTF-8)
	if(!(utf8str = (char *)MAMALLOC(maxbytes)))
	{
		maLog(L"offset_wchar_to_utf8 MAMALLOC failed");
		return 0;
	}
	
	c = mastr2[maoffset];
	mastr2[maoffset] = L'\0';

	utf8bytes = wclocal_to_utf8(mastr2, maoffset, utf8str, maxbytes);
	utf8str[utf8bytes] = fsT('\0');

	mastr2[maoffset] = c;

	utf8offset = mastrlen_utf8(utf8str);
	MAFREE(utf8str);

	return utf8offset;

 } // offset_wchar_to_utf8
#endif

#if defined(__MAOSX__) || defined(__MAUNIX__)
 fschar *fsstrstrfn(fschar *fsstr, fschar *p)
 {
		size_t len;
		machar *q;
		machar mastr2[MAXLOGLINELEN];
		machar mastr3[MAXLOGLINELEN];
		fschar fsstr2[MAXLOGLINELEN];
 
		fs2mafn(mastr2, fsstr);
		fs2mafn(mastr3, p);
		if((q = wcsstr(mastr2, mastr3)))
		{
			*q = L'\0';
			ma2fs(fsstr2, mastr2);
			len = strlen(fsstr2);
			return (fsstr + len);
		}
		else
			return NULL;

 } // fsstrstrfn
#endif


#if defined(MAGUINONE) && defined(__MACUNIX__)
 // here to avoid gdk_keyval_to_unicode g_unichar_to_utf8 dependency for moreamposxnogui
 // copied from gtk-2 gdkkeyuni.c
 #define MA_N_ELEMENTS(arr) (sizeof (arr) / sizeof ((arr)[0])) // G_N_ELEMENTS(arr) (sizeof (arr) / sizeof ((arr)[0]))
 static const struct {
  unsigned short keysym;
  unsigned short ucs;
 } gdk_keysym_to_unicode_tab[] = {
  { 0x01a1, 0x0104 }, /*                     Aogonek Ą LATIN CAPITAL LETTER A WITH OGONEK */
  { 0x01a2, 0x02d8 }, /*                       breve ˘ BREVE */
  { 0x01a3, 0x0141 }, /*                     Lstroke Ł LATIN CAPITAL LETTER L WITH STROKE */
  { 0x01a5, 0x013d }, /*                      Lcaron Ľ LATIN CAPITAL LETTER L WITH CARON */
  { 0x01a6, 0x015a }, /*                      Sacute Ś LATIN CAPITAL LETTER S WITH ACUTE */
  { 0x01a9, 0x0160 }, /*                      Scaron Š LATIN CAPITAL LETTER S WITH CARON */
  { 0x01aa, 0x015e }, /*                    Scedilla Ş LATIN CAPITAL LETTER S WITH CEDILLA */
  { 0x01ab, 0x0164 }, /*                      Tcaron Ť LATIN CAPITAL LETTER T WITH CARON */
  { 0x01ac, 0x0179 }, /*                      Zacute Ź LATIN CAPITAL LETTER Z WITH ACUTE */
  { 0x01ae, 0x017d }, /*                      Zcaron Ž LATIN CAPITAL LETTER Z WITH CARON */
  { 0x01af, 0x017b }, /*                   Zabovedot Ż LATIN CAPITAL LETTER Z WITH DOT ABOVE */
  { 0x01b1, 0x0105 }, /*                     aogonek ą LATIN SMALL LETTER A WITH OGONEK */
  { 0x01b2, 0x02db }, /*                      ogonek ˛ OGONEK */
  { 0x01b3, 0x0142 }, /*                     lstroke ł LATIN SMALL LETTER L WITH STROKE */
  { 0x01b5, 0x013e }, /*                      lcaron ľ LATIN SMALL LETTER L WITH CARON */
  { 0x01b6, 0x015b }, /*                      sacute ś LATIN SMALL LETTER S WITH ACUTE */
  { 0x01b7, 0x02c7 }, /*                       caron ˇ CARON */
  { 0x01b9, 0x0161 }, /*                      scaron š LATIN SMALL LETTER S WITH CARON */
  { 0x01ba, 0x015f }, /*                    scedilla ş LATIN SMALL LETTER S WITH CEDILLA */
  { 0x01bb, 0x0165 }, /*                      tcaron ť LATIN SMALL LETTER T WITH CARON */
  { 0x01bc, 0x017a }, /*                      zacute ź LATIN SMALL LETTER Z WITH ACUTE */
  { 0x01bd, 0x02dd }, /*                 doubleacute ˝ DOUBLE ACUTE ACCENT */
  { 0x01be, 0x017e }, /*                      zcaron ž LATIN SMALL LETTER Z WITH CARON */
  { 0x01bf, 0x017c }, /*                   zabovedot ż LATIN SMALL LETTER Z WITH DOT ABOVE */
  { 0x01c0, 0x0154 }, /*                      Racute Ŕ LATIN CAPITAL LETTER R WITH ACUTE */
  { 0x01c3, 0x0102 }, /*                      Abreve Ă LATIN CAPITAL LETTER A WITH BREVE */
  { 0x01c5, 0x0139 }, /*                      Lacute Ĺ LATIN CAPITAL LETTER L WITH ACUTE */
  { 0x01c6, 0x0106 }, /*                      Cacute Ć LATIN CAPITAL LETTER C WITH ACUTE */
  { 0x01c8, 0x010c }, /*                      Ccaron Č LATIN CAPITAL LETTER C WITH CARON */
  { 0x01ca, 0x0118 }, /*                     Eogonek Ę LATIN CAPITAL LETTER E WITH OGONEK */
  { 0x01cc, 0x011a }, /*                      Ecaron Ě LATIN CAPITAL LETTER E WITH CARON */
  { 0x01cf, 0x010e }, /*                      Dcaron Ď LATIN CAPITAL LETTER D WITH CARON */
  { 0x01d0, 0x0110 }, /*                     Dstroke Đ LATIN CAPITAL LETTER D WITH STROKE */
  { 0x01d1, 0x0143 }, /*                      Nacute Ń LATIN CAPITAL LETTER N WITH ACUTE */
  { 0x01d2, 0x0147 }, /*                      Ncaron Ň LATIN CAPITAL LETTER N WITH CARON */
  { 0x01d5, 0x0150 }, /*                Odoubleacute Ő LATIN CAPITAL LETTER O WITH DOUBLE ACUTE */
  { 0x01d8, 0x0158 }, /*                      Rcaron Ř LATIN CAPITAL LETTER R WITH CARON */
  { 0x01d9, 0x016e }, /*                       Uring Ů LATIN CAPITAL LETTER U WITH RING ABOVE */
  { 0x01db, 0x0170 }, /*                Udoubleacute Ű LATIN CAPITAL LETTER U WITH DOUBLE ACUTE */
  { 0x01de, 0x0162 }, /*                    Tcedilla Ţ LATIN CAPITAL LETTER T WITH CEDILLA */
  { 0x01e0, 0x0155 }, /*                      racute ŕ LATIN SMALL LETTER R WITH ACUTE */
  { 0x01e3, 0x0103 }, /*                      abreve ă LATIN SMALL LETTER A WITH BREVE */
  { 0x01e5, 0x013a }, /*                      lacute ĺ LATIN SMALL LETTER L WITH ACUTE */
  { 0x01e6, 0x0107 }, /*                      cacute ć LATIN SMALL LETTER C WITH ACUTE */
  { 0x01e8, 0x010d }, /*                      ccaron č LATIN SMALL LETTER C WITH CARON */
  { 0x01ea, 0x0119 }, /*                     eogonek ę LATIN SMALL LETTER E WITH OGONEK */
  { 0x01ec, 0x011b }, /*                      ecaron ě LATIN SMALL LETTER E WITH CARON */
  { 0x01ef, 0x010f }, /*                      dcaron ď LATIN SMALL LETTER D WITH CARON */
  { 0x01f0, 0x0111 }, /*                     dstroke đ LATIN SMALL LETTER D WITH STROKE */
  { 0x01f1, 0x0144 }, /*                      nacute ń LATIN SMALL LETTER N WITH ACUTE */
  { 0x01f2, 0x0148 }, /*                      ncaron ň LATIN SMALL LETTER N WITH CARON */
  { 0x01f5, 0x0151 }, /*                odoubleacute ő LATIN SMALL LETTER O WITH DOUBLE ACUTE */
  { 0x01f8, 0x0159 }, /*                      rcaron ř LATIN SMALL LETTER R WITH CARON */
  { 0x01f9, 0x016f }, /*                       uring ů LATIN SMALL LETTER U WITH RING ABOVE */
  { 0x01fb, 0x0171 }, /*                udoubleacute ű LATIN SMALL LETTER U WITH DOUBLE ACUTE */
  { 0x01fe, 0x0163 }, /*                    tcedilla ţ LATIN SMALL LETTER T WITH CEDILLA */
  { 0x01ff, 0x02d9 }, /*                    abovedot ˙ DOT ABOVE */
  { 0x02a1, 0x0126 }, /*                     Hstroke Ħ LATIN CAPITAL LETTER H WITH STROKE */
  { 0x02a6, 0x0124 }, /*                 Hcircumflex Ĥ LATIN CAPITAL LETTER H WITH CIRCUMFLEX */
  { 0x02a9, 0x0130 }, /*                   Iabovedot İ LATIN CAPITAL LETTER I WITH DOT ABOVE */
  { 0x02ab, 0x011e }, /*                      Gbreve Ğ LATIN CAPITAL LETTER G WITH BREVE */
  { 0x02ac, 0x0134 }, /*                 Jcircumflex Ĵ LATIN CAPITAL LETTER J WITH CIRCUMFLEX */
  { 0x02b1, 0x0127 }, /*                     hstroke ħ LATIN SMALL LETTER H WITH STROKE */
  { 0x02b6, 0x0125 }, /*                 hcircumflex ĥ LATIN SMALL LETTER H WITH CIRCUMFLEX */
  { 0x02b9, 0x0131 }, /*                    idotless ı LATIN SMALL LETTER DOTLESS I */
  { 0x02bb, 0x011f }, /*                      gbreve ğ LATIN SMALL LETTER G WITH BREVE */
  { 0x02bc, 0x0135 }, /*                 jcircumflex ĵ LATIN SMALL LETTER J WITH CIRCUMFLEX */
  { 0x02c5, 0x010a }, /*                   Cabovedot Ċ LATIN CAPITAL LETTER C WITH DOT ABOVE */
  { 0x02c6, 0x0108 }, /*                 Ccircumflex Ĉ LATIN CAPITAL LETTER C WITH CIRCUMFLEX */
  { 0x02d5, 0x0120 }, /*                   Gabovedot Ġ LATIN CAPITAL LETTER G WITH DOT ABOVE */
  { 0x02d8, 0x011c }, /*                 Gcircumflex Ĝ LATIN CAPITAL LETTER G WITH CIRCUMFLEX */
  { 0x02dd, 0x016c }, /*                      Ubreve Ŭ LATIN CAPITAL LETTER U WITH BREVE */
  { 0x02de, 0x015c }, /*                 Scircumflex Ŝ LATIN CAPITAL LETTER S WITH CIRCUMFLEX */
  { 0x02e5, 0x010b }, /*                   cabovedot ċ LATIN SMALL LETTER C WITH DOT ABOVE */
  { 0x02e6, 0x0109 }, /*                 ccircumflex ĉ LATIN SMALL LETTER C WITH CIRCUMFLEX */
  { 0x02f5, 0x0121 }, /*                   gabovedot ġ LATIN SMALL LETTER G WITH DOT ABOVE */
  { 0x02f8, 0x011d }, /*                 gcircumflex ĝ LATIN SMALL LETTER G WITH CIRCUMFLEX */
  { 0x02fd, 0x016d }, /*                      ubreve ŭ LATIN SMALL LETTER U WITH BREVE */
  { 0x02fe, 0x015d }, /*                 scircumflex ŝ LATIN SMALL LETTER S WITH CIRCUMFLEX */
  { 0x03a2, 0x0138 }, /*                         kra ĸ LATIN SMALL LETTER KRA */
  { 0x03a3, 0x0156 }, /*                    Rcedilla Ŗ LATIN CAPITAL LETTER R WITH CEDILLA */
  { 0x03a5, 0x0128 }, /*                      Itilde Ĩ LATIN CAPITAL LETTER I WITH TILDE */
  { 0x03a6, 0x013b }, /*                    Lcedilla Ļ LATIN CAPITAL LETTER L WITH CEDILLA */
  { 0x03aa, 0x0112 }, /*                     Emacron Ē LATIN CAPITAL LETTER E WITH MACRON */
  { 0x03ab, 0x0122 }, /*                    Gcedilla Ģ LATIN CAPITAL LETTER G WITH CEDILLA */
  { 0x03ac, 0x0166 }, /*                      Tslash Ŧ LATIN CAPITAL LETTER T WITH STROKE */
  { 0x03b3, 0x0157 }, /*                    rcedilla ŗ LATIN SMALL LETTER R WITH CEDILLA */
  { 0x03b5, 0x0129 }, /*                      itilde ĩ LATIN SMALL LETTER I WITH TILDE */
  { 0x03b6, 0x013c }, /*                    lcedilla ļ LATIN SMALL LETTER L WITH CEDILLA */
  { 0x03ba, 0x0113 }, /*                     emacron ē LATIN SMALL LETTER E WITH MACRON */
  { 0x03bb, 0x0123 }, /*                    gcedilla ģ LATIN SMALL LETTER G WITH CEDILLA */
  { 0x03bc, 0x0167 }, /*                      tslash ŧ LATIN SMALL LETTER T WITH STROKE */
  { 0x03bd, 0x014a }, /*                         ENG Ŋ LATIN CAPITAL LETTER ENG */
  { 0x03bf, 0x014b }, /*                         eng ŋ LATIN SMALL LETTER ENG */
  { 0x03c0, 0x0100 }, /*                     Amacron Ā LATIN CAPITAL LETTER A WITH MACRON */
  { 0x03c7, 0x012e }, /*                     Iogonek Į LATIN CAPITAL LETTER I WITH OGONEK */
  { 0x03cc, 0x0116 }, /*                   Eabovedot Ė LATIN CAPITAL LETTER E WITH DOT ABOVE */
  { 0x03cf, 0x012a }, /*                     Imacron Ī LATIN CAPITAL LETTER I WITH MACRON */
  { 0x03d1, 0x0145 }, /*                    Ncedilla Ņ LATIN CAPITAL LETTER N WITH CEDILLA */
  { 0x03d2, 0x014c }, /*                     Omacron Ō LATIN CAPITAL LETTER O WITH MACRON */
  { 0x03d3, 0x0136 }, /*                    Kcedilla Ķ LATIN CAPITAL LETTER K WITH CEDILLA */
  { 0x03d9, 0x0172 }, /*                     Uogonek Ų LATIN CAPITAL LETTER U WITH OGONEK */
  { 0x03dd, 0x0168 }, /*                      Utilde Ũ LATIN CAPITAL LETTER U WITH TILDE */
  { 0x03de, 0x016a }, /*                     Umacron Ū LATIN CAPITAL LETTER U WITH MACRON */
  { 0x03e0, 0x0101 }, /*                     amacron ā LATIN SMALL LETTER A WITH MACRON */
  { 0x03e7, 0x012f }, /*                     iogonek į LATIN SMALL LETTER I WITH OGONEK */
  { 0x03ec, 0x0117 }, /*                   eabovedot ė LATIN SMALL LETTER E WITH DOT ABOVE */
  { 0x03ef, 0x012b }, /*                     imacron ī LATIN SMALL LETTER I WITH MACRON */
  { 0x03f1, 0x0146 }, /*                    ncedilla ņ LATIN SMALL LETTER N WITH CEDILLA */
  { 0x03f2, 0x014d }, /*                     omacron ō LATIN SMALL LETTER O WITH MACRON */
  { 0x03f3, 0x0137 }, /*                    kcedilla ķ LATIN SMALL LETTER K WITH CEDILLA */
  { 0x03f9, 0x0173 }, /*                     uogonek ų LATIN SMALL LETTER U WITH OGONEK */
  { 0x03fd, 0x0169 }, /*                      utilde ũ LATIN SMALL LETTER U WITH TILDE */
  { 0x03fe, 0x016b }, /*                     umacron ū LATIN SMALL LETTER U WITH MACRON */
  { 0x047e, 0x203e }, /*                    overline ‾ OVERLINE */
  { 0x04a1, 0x3002 }, /*               kana_fullstop 。 IDEOGRAPHIC FULL STOP */
  { 0x04a2, 0x300c }, /*         kana_openingbracket 「 LEFT CORNER BRACKET */
  { 0x04a3, 0x300d }, /*         kana_closingbracket 」 RIGHT CORNER BRACKET */
  { 0x04a4, 0x3001 }, /*                  kana_comma 、 IDEOGRAPHIC COMMA */
  { 0x04a5, 0x30fb }, /*            kana_conjunctive ・ KATAKANA MIDDLE DOT */
  { 0x04a6, 0x30f2 }, /*                     kana_WO ヲ KATAKANA LETTER WO */
  { 0x04a7, 0x30a1 }, /*                      kana_a ァ KATAKANA LETTER SMALL A */
  { 0x04a8, 0x30a3 }, /*                      kana_i ィ KATAKANA LETTER SMALL I */
  { 0x04a9, 0x30a5 }, /*                      kana_u ゥ KATAKANA LETTER SMALL U */
  { 0x04aa, 0x30a7 }, /*                      kana_e ェ KATAKANA LETTER SMALL E */
  { 0x04ab, 0x30a9 }, /*                      kana_o ォ KATAKANA LETTER SMALL O */
  { 0x04ac, 0x30e3 }, /*                     kana_ya ャ KATAKANA LETTER SMALL YA */
  { 0x04ad, 0x30e5 }, /*                     kana_yu ュ KATAKANA LETTER SMALL YU */
  { 0x04ae, 0x30e7 }, /*                     kana_yo ョ KATAKANA LETTER SMALL YO */
  { 0x04af, 0x30c3 }, /*                    kana_tsu ッ KATAKANA LETTER SMALL TU */
  { 0x04b0, 0x30fc }, /*              prolongedsound ー KATAKANA-HIRAGANA PROLONGED SOUND MARK */
  { 0x04b1, 0x30a2 }, /*                      kana_A ア KATAKANA LETTER A */
  { 0x04b2, 0x30a4 }, /*                      kana_I イ KATAKANA LETTER I */
  { 0x04b3, 0x30a6 }, /*                      kana_U ウ KATAKANA LETTER U */
  { 0x04b4, 0x30a8 }, /*                      kana_E エ KATAKANA LETTER E */
  { 0x04b5, 0x30aa }, /*                      kana_O オ KATAKANA LETTER O */
  { 0x04b6, 0x30ab }, /*                     kana_KA カ KATAKANA LETTER KA */
  { 0x04b7, 0x30ad }, /*                     kana_KI キ KATAKANA LETTER KI */
  { 0x04b8, 0x30af }, /*                     kana_KU ク KATAKANA LETTER KU */
  { 0x04b9, 0x30b1 }, /*                     kana_KE ケ KATAKANA LETTER KE */
  { 0x04ba, 0x30b3 }, /*                     kana_KO コ KATAKANA LETTER KO */
  { 0x04bb, 0x30b5 }, /*                     kana_SA サ KATAKANA LETTER SA */
  { 0x04bc, 0x30b7 }, /*                    kana_SHI シ KATAKANA LETTER SI */
  { 0x04bd, 0x30b9 }, /*                     kana_SU ス KATAKANA LETTER SU */
  { 0x04be, 0x30bb }, /*                     kana_SE セ KATAKANA LETTER SE */
  { 0x04bf, 0x30bd }, /*                     kana_SO ソ KATAKANA LETTER SO */
  { 0x04c0, 0x30bf }, /*                     kana_TA タ KATAKANA LETTER TA */
  { 0x04c1, 0x30c1 }, /*                    kana_CHI チ KATAKANA LETTER TI */
  { 0x04c2, 0x30c4 }, /*                    kana_TSU ツ KATAKANA LETTER TU */
  { 0x04c3, 0x30c6 }, /*                     kana_TE テ KATAKANA LETTER TE */
  { 0x04c4, 0x30c8 }, /*                     kana_TO ト KATAKANA LETTER TO */
  { 0x04c5, 0x30ca }, /*                     kana_NA ナ KATAKANA LETTER NA */
  { 0x04c6, 0x30cb }, /*                     kana_NI ニ KATAKANA LETTER NI */
  { 0x04c7, 0x30cc }, /*                     kana_NU ヌ KATAKANA LETTER NU */
  { 0x04c8, 0x30cd }, /*                     kana_NE ネ KATAKANA LETTER NE */
  { 0x04c9, 0x30ce }, /*                     kana_NO ノ KATAKANA LETTER NO */
  { 0x04ca, 0x30cf }, /*                     kana_HA ハ KATAKANA LETTER HA */
  { 0x04cb, 0x30d2 }, /*                     kana_HI ヒ KATAKANA LETTER HI */
  { 0x04cc, 0x30d5 }, /*                     kana_FU フ KATAKANA LETTER HU */
  { 0x04cd, 0x30d8 }, /*                     kana_HE ヘ KATAKANA LETTER HE */
  { 0x04ce, 0x30db }, /*                     kana_HO ホ KATAKANA LETTER HO */
  { 0x04cf, 0x30de }, /*                     kana_MA マ KATAKANA LETTER MA */
  { 0x04d0, 0x30df }, /*                     kana_MI ミ KATAKANA LETTER MI */
  { 0x04d1, 0x30e0 }, /*                     kana_MU ム KATAKANA LETTER MU */
  { 0x04d2, 0x30e1 }, /*                     kana_ME メ KATAKANA LETTER ME */
  { 0x04d3, 0x30e2 }, /*                     kana_MO モ KATAKANA LETTER MO */
  { 0x04d4, 0x30e4 }, /*                     kana_YA ヤ KATAKANA LETTER YA */
  { 0x04d5, 0x30e6 }, /*                     kana_YU ユ KATAKANA LETTER YU */
  { 0x04d6, 0x30e8 }, /*                     kana_YO ヨ KATAKANA LETTER YO */
  { 0x04d7, 0x30e9 }, /*                     kana_RA ラ KATAKANA LETTER RA */
  { 0x04d8, 0x30ea }, /*                     kana_RI リ KATAKANA LETTER RI */
  { 0x04d9, 0x30eb }, /*                     kana_RU ル KATAKANA LETTER RU */
  { 0x04da, 0x30ec }, /*                     kana_RE レ KATAKANA LETTER RE */
  { 0x04db, 0x30ed }, /*                     kana_RO ロ KATAKANA LETTER RO */
  { 0x04dc, 0x30ef }, /*                     kana_WA ワ KATAKANA LETTER WA */
  { 0x04dd, 0x30f3 }, /*                      kana_N ン KATAKANA LETTER N */
  { 0x04de, 0x309b }, /*                 voicedsound ゛ KATAKANA-HIRAGANA VOICED SOUND MARK */
  { 0x04df, 0x309c }, /*             semivoicedsound ゜ KATAKANA-HIRAGANA SEMI-VOICED SOUND MARK */
  { 0x05ac, 0x060c }, /*                Arabic_comma ، ARABIC COMMA */
  { 0x05bb, 0x061b }, /*            Arabic_semicolon ؛ ARABIC SEMICOLON */
  { 0x05bf, 0x061f }, /*        Arabic_question_mark ؟ ARABIC QUESTION MARK */
  { 0x05c1, 0x0621 }, /*                Arabic_hamza ء ARABIC LETTER HAMZA */
  { 0x05c2, 0x0622 }, /*          Arabic_maddaonalef آ ARABIC LETTER ALEF WITH MADDA ABOVE */
  { 0x05c3, 0x0623 }, /*          Arabic_hamzaonalef أ ARABIC LETTER ALEF WITH HAMZA ABOVE */
  { 0x05c4, 0x0624 }, /*           Arabic_hamzaonwaw ؤ ARABIC LETTER WAW WITH HAMZA ABOVE */
  { 0x05c5, 0x0625 }, /*       Arabic_hamzaunderalef إ ARABIC LETTER ALEF WITH HAMZA BELOW */
  { 0x05c6, 0x0626 }, /*           Arabic_hamzaonyeh ئ ARABIC LETTER YEH WITH HAMZA ABOVE */
  { 0x05c7, 0x0627 }, /*                 Arabic_alef ا ARABIC LETTER ALEF */
  { 0x05c8, 0x0628 }, /*                  Arabic_beh ب ARABIC LETTER BEH */
  { 0x05c9, 0x0629 }, /*           Arabic_tehmarbuta ة ARABIC LETTER TEH MARBUTA */
  { 0x05ca, 0x062a }, /*                  Arabic_teh ت ARABIC LETTER TEH */
  { 0x05cb, 0x062b }, /*                 Arabic_theh ث ARABIC LETTER THEH */
  { 0x05cc, 0x062c }, /*                 Arabic_jeem ج ARABIC LETTER JEEM */
  { 0x05cd, 0x062d }, /*                  Arabic_hah ح ARABIC LETTER HAH */
  { 0x05ce, 0x062e }, /*                 Arabic_khah خ ARABIC LETTER KHAH */
  { 0x05cf, 0x062f }, /*                  Arabic_dal د ARABIC LETTER DAL */
  { 0x05d0, 0x0630 }, /*                 Arabic_thal ذ ARABIC LETTER THAL */
  { 0x05d1, 0x0631 }, /*                   Arabic_ra ر ARABIC LETTER REH */
  { 0x05d2, 0x0632 }, /*                 Arabic_zain ز ARABIC LETTER ZAIN */
  { 0x05d3, 0x0633 }, /*                 Arabic_seen س ARABIC LETTER SEEN */
  { 0x05d4, 0x0634 }, /*                Arabic_sheen ش ARABIC LETTER SHEEN */
  { 0x05d5, 0x0635 }, /*                  Arabic_sad ص ARABIC LETTER SAD */
  { 0x05d6, 0x0636 }, /*                  Arabic_dad ض ARABIC LETTER DAD */
  { 0x05d7, 0x0637 }, /*                  Arabic_tah ط ARABIC LETTER TAH */
  { 0x05d8, 0x0638 }, /*                  Arabic_zah ظ ARABIC LETTER ZAH */
  { 0x05d9, 0x0639 }, /*                  Arabic_ain ع ARABIC LETTER AIN */
  { 0x05da, 0x063a }, /*                Arabic_ghain غ ARABIC LETTER GHAIN */
  { 0x05e0, 0x0640 }, /*              Arabic_tatweel ـ ARABIC TATWEEL */
  { 0x05e1, 0x0641 }, /*                  Arabic_feh ف ARABIC LETTER FEH */
  { 0x05e2, 0x0642 }, /*                  Arabic_qaf ق ARABIC LETTER QAF */
  { 0x05e3, 0x0643 }, /*                  Arabic_kaf ك ARABIC LETTER KAF */
  { 0x05e4, 0x0644 }, /*                  Arabic_lam ل ARABIC LETTER LAM */
  { 0x05e5, 0x0645 }, /*                 Arabic_meem م ARABIC LETTER MEEM */
  { 0x05e6, 0x0646 }, /*                 Arabic_noon ن ARABIC LETTER NOON */
  { 0x05e7, 0x0647 }, /*                   Arabic_ha ه ARABIC LETTER HEH */
  { 0x05e8, 0x0648 }, /*                  Arabic_waw و ARABIC LETTER WAW */
  { 0x05e9, 0x0649 }, /*          Arabic_alefmaksura ى ARABIC LETTER ALEF MAKSURA */
  { 0x05ea, 0x064a }, /*                  Arabic_yeh ي ARABIC LETTER YEH */
  { 0x05eb, 0x064b }, /*             Arabic_fathatan ً ARABIC FATHATAN */
  { 0x05ec, 0x064c }, /*             Arabic_dammatan ٌ ARABIC DAMMATAN */
  { 0x05ed, 0x064d }, /*             Arabic_kasratan ٍ ARABIC KASRATAN */
  { 0x05ee, 0x064e }, /*                Arabic_fatha َ ARABIC FATHA */
  { 0x05ef, 0x064f }, /*                Arabic_damma ُ ARABIC DAMMA */
  { 0x05f0, 0x0650 }, /*                Arabic_kasra ِ ARABIC KASRA */
  { 0x05f1, 0x0651 }, /*               Arabic_shadda ّ ARABIC SHADDA */
  { 0x05f2, 0x0652 }, /*                Arabic_sukun ْ ARABIC SUKUN */
  { 0x06a1, 0x0452 }, /*                 Serbian_dje ђ CYRILLIC SMALL LETTER DJE */
  { 0x06a2, 0x0453 }, /*               Macedonia_gje ѓ CYRILLIC SMALL LETTER GJE */
  { 0x06a3, 0x0451 }, /*                 Cyrillic_io ё CYRILLIC SMALL LETTER IO */
  { 0x06a4, 0x0454 }, /*                Ukrainian_ie є CYRILLIC SMALL LETTER UKRAINIAN IE */
  { 0x06a5, 0x0455 }, /*               Macedonia_dse ѕ CYRILLIC SMALL LETTER DZE */
  { 0x06a6, 0x0456 }, /*                 Ukrainian_i і CYRILLIC SMALL LETTER BYELORUSSIAN-UKRAINIAN I */
  { 0x06a7, 0x0457 }, /*                Ukrainian_yi ї CYRILLIC SMALL LETTER YI */
  { 0x06a8, 0x0458 }, /*                 Cyrillic_je ј CYRILLIC SMALL LETTER JE */
  { 0x06a9, 0x0459 }, /*                Cyrillic_lje љ CYRILLIC SMALL LETTER LJE */
  { 0x06aa, 0x045a }, /*                Cyrillic_nje њ CYRILLIC SMALL LETTER NJE */
  { 0x06ab, 0x045b }, /*                Serbian_tshe ћ CYRILLIC SMALL LETTER TSHE */
  { 0x06ac, 0x045c }, /*               Macedonia_kje ќ CYRILLIC SMALL LETTER KJE */
  { 0x06ad, 0x0491 }, /*   Ukrainian_ghe_with_upturn ґ CYRILLIC SMALL LETTER GHE WITH UPTURN */
  { 0x06ae, 0x045e }, /*         Byelorussian_shortu ў CYRILLIC SMALL LETTER SHORT U */
  { 0x06af, 0x045f }, /*               Cyrillic_dzhe џ CYRILLIC SMALL LETTER DZHE */
  { 0x06b0, 0x2116 }, /*                  numerosign № NUMERO SIGN */
  { 0x06b1, 0x0402 }, /*                 Serbian_DJE Ђ CYRILLIC CAPITAL LETTER DJE */
  { 0x06b2, 0x0403 }, /*               Macedonia_GJE Ѓ CYRILLIC CAPITAL LETTER GJE */
  { 0x06b3, 0x0401 }, /*                 Cyrillic_IO Ё CYRILLIC CAPITAL LETTER IO */
  { 0x06b4, 0x0404 }, /*                Ukrainian_IE Є CYRILLIC CAPITAL LETTER UKRAINIAN IE */
  { 0x06b5, 0x0405 }, /*               Macedonia_DSE Ѕ CYRILLIC CAPITAL LETTER DZE */
  { 0x06b6, 0x0406 }, /*                 Ukrainian_I І CYRILLIC CAPITAL LETTER BYELORUSSIAN-UKRAINIAN I */
  { 0x06b7, 0x0407 }, /*                Ukrainian_YI Ї CYRILLIC CAPITAL LETTER YI */
  { 0x06b8, 0x0408 }, /*                 Cyrillic_JE Ј CYRILLIC CAPITAL LETTER JE */
  { 0x06b9, 0x0409 }, /*                Cyrillic_LJE Љ CYRILLIC CAPITAL LETTER LJE */
  { 0x06ba, 0x040a }, /*                Cyrillic_NJE Њ CYRILLIC CAPITAL LETTER NJE */
  { 0x06bb, 0x040b }, /*                Serbian_TSHE Ћ CYRILLIC CAPITAL LETTER TSHE */
  { 0x06bc, 0x040c }, /*               Macedonia_KJE Ќ CYRILLIC CAPITAL LETTER KJE */
  { 0x06bd, 0x0490 }, /*   Ukrainian_GHE_WITH_UPTURN Ґ CYRILLIC CAPITAL LETTER GHE WITH UPTURN */
  { 0x06be, 0x040e }, /*         Byelorussian_SHORTU Ў CYRILLIC CAPITAL LETTER SHORT U */
  { 0x06bf, 0x040f }, /*               Cyrillic_DZHE Џ CYRILLIC CAPITAL LETTER DZHE */
  { 0x06c0, 0x044e }, /*                 Cyrillic_yu ю CYRILLIC SMALL LETTER YU */
  { 0x06c1, 0x0430 }, /*                  Cyrillic_a а CYRILLIC SMALL LETTER A */
  { 0x06c2, 0x0431 }, /*                 Cyrillic_be б CYRILLIC SMALL LETTER BE */
  { 0x06c3, 0x0446 }, /*                Cyrillic_tse ц CYRILLIC SMALL LETTER TSE */
  { 0x06c4, 0x0434 }, /*                 Cyrillic_de д CYRILLIC SMALL LETTER DE */
  { 0x06c5, 0x0435 }, /*                 Cyrillic_ie е CYRILLIC SMALL LETTER IE */
  { 0x06c6, 0x0444 }, /*                 Cyrillic_ef ф CYRILLIC SMALL LETTER EF */
  { 0x06c7, 0x0433 }, /*                Cyrillic_ghe г CYRILLIC SMALL LETTER GHE */
  { 0x06c8, 0x0445 }, /*                 Cyrillic_ha х CYRILLIC SMALL LETTER HA */
  { 0x06c9, 0x0438 }, /*                  Cyrillic_i и CYRILLIC SMALL LETTER I */
  { 0x06ca, 0x0439 }, /*             Cyrillic_shorti й CYRILLIC SMALL LETTER SHORT I */
  { 0x06cb, 0x043a }, /*                 Cyrillic_ka к CYRILLIC SMALL LETTER KA */
  { 0x06cc, 0x043b }, /*                 Cyrillic_el л CYRILLIC SMALL LETTER EL */
  { 0x06cd, 0x043c }, /*                 Cyrillic_em м CYRILLIC SMALL LETTER EM */
  { 0x06ce, 0x043d }, /*                 Cyrillic_en н CYRILLIC SMALL LETTER EN */
  { 0x06cf, 0x043e }, /*                  Cyrillic_o о CYRILLIC SMALL LETTER O */
  { 0x06d0, 0x043f }, /*                 Cyrillic_pe п CYRILLIC SMALL LETTER PE */
  { 0x06d1, 0x044f }, /*                 Cyrillic_ya я CYRILLIC SMALL LETTER YA */
  { 0x06d2, 0x0440 }, /*                 Cyrillic_er р CYRILLIC SMALL LETTER ER */
  { 0x06d3, 0x0441 }, /*                 Cyrillic_es с CYRILLIC SMALL LETTER ES */
  { 0x06d4, 0x0442 }, /*                 Cyrillic_te т CYRILLIC SMALL LETTER TE */
  { 0x06d5, 0x0443 }, /*                  Cyrillic_u у CYRILLIC SMALL LETTER U */
  { 0x06d6, 0x0436 }, /*                Cyrillic_zhe ж CYRILLIC SMALL LETTER ZHE */
  { 0x06d7, 0x0432 }, /*                 Cyrillic_ve в CYRILLIC SMALL LETTER VE */
  { 0x06d8, 0x044c }, /*           Cyrillic_softsign ь CYRILLIC SMALL LETTER SOFT SIGN */
  { 0x06d9, 0x044b }, /*               Cyrillic_yeru ы CYRILLIC SMALL LETTER YERU */
  { 0x06da, 0x0437 }, /*                 Cyrillic_ze з CYRILLIC SMALL LETTER ZE */
  { 0x06db, 0x0448 }, /*                Cyrillic_sha ш CYRILLIC SMALL LETTER SHA */
  { 0x06dc, 0x044d }, /*                  Cyrillic_e э CYRILLIC SMALL LETTER E */
  { 0x06dd, 0x0449 }, /*              Cyrillic_shcha щ CYRILLIC SMALL LETTER SHCHA */
  { 0x06de, 0x0447 }, /*                Cyrillic_che ч CYRILLIC SMALL LETTER CHE */
  { 0x06df, 0x044a }, /*           Cyrillic_hardsign ъ CYRILLIC SMALL LETTER HARD SIGN */
  { 0x06e0, 0x042e }, /*                 Cyrillic_YU Ю CYRILLIC CAPITAL LETTER YU */
  { 0x06e1, 0x0410 }, /*                  Cyrillic_A А CYRILLIC CAPITAL LETTER A */
  { 0x06e2, 0x0411 }, /*                 Cyrillic_BE Б CYRILLIC CAPITAL LETTER BE */
  { 0x06e3, 0x0426 }, /*                Cyrillic_TSE Ц CYRILLIC CAPITAL LETTER TSE */
  { 0x06e4, 0x0414 }, /*                 Cyrillic_DE Д CYRILLIC CAPITAL LETTER DE */
  { 0x06e5, 0x0415 }, /*                 Cyrillic_IE Е CYRILLIC CAPITAL LETTER IE */
  { 0x06e6, 0x0424 }, /*                 Cyrillic_EF Ф CYRILLIC CAPITAL LETTER EF */
  { 0x06e7, 0x0413 }, /*                Cyrillic_GHE Г CYRILLIC CAPITAL LETTER GHE */
  { 0x06e8, 0x0425 }, /*                 Cyrillic_HA Х CYRILLIC CAPITAL LETTER HA */
  { 0x06e9, 0x0418 }, /*                  Cyrillic_I И CYRILLIC CAPITAL LETTER I */
  { 0x06ea, 0x0419 }, /*             Cyrillic_SHORTI Й CYRILLIC CAPITAL LETTER SHORT I */
  { 0x06eb, 0x041a }, /*                 Cyrillic_KA К CYRILLIC CAPITAL LETTER KA */
  { 0x06ec, 0x041b }, /*                 Cyrillic_EL Л CYRILLIC CAPITAL LETTER EL */
  { 0x06ed, 0x041c }, /*                 Cyrillic_EM М CYRILLIC CAPITAL LETTER EM */
  { 0x06ee, 0x041d }, /*                 Cyrillic_EN Н CYRILLIC CAPITAL LETTER EN */
  { 0x06ef, 0x041e }, /*                  Cyrillic_O О CYRILLIC CAPITAL LETTER O */
  { 0x06f0, 0x041f }, /*                 Cyrillic_PE П CYRILLIC CAPITAL LETTER PE */
  { 0x06f1, 0x042f }, /*                 Cyrillic_YA Я CYRILLIC CAPITAL LETTER YA */
  { 0x06f2, 0x0420 }, /*                 Cyrillic_ER Р CYRILLIC CAPITAL LETTER ER */
  { 0x06f3, 0x0421 }, /*                 Cyrillic_ES С CYRILLIC CAPITAL LETTER ES */
  { 0x06f4, 0x0422 }, /*                 Cyrillic_TE Т CYRILLIC CAPITAL LETTER TE */
  { 0x06f5, 0x0423 }, /*                  Cyrillic_U У CYRILLIC CAPITAL LETTER U */
  { 0x06f6, 0x0416 }, /*                Cyrillic_ZHE Ж CYRILLIC CAPITAL LETTER ZHE */
  { 0x06f7, 0x0412 }, /*                 Cyrillic_VE В CYRILLIC CAPITAL LETTER VE */
  { 0x06f8, 0x042c }, /*           Cyrillic_SOFTSIGN Ь CYRILLIC CAPITAL LETTER SOFT SIGN */
  { 0x06f9, 0x042b }, /*               Cyrillic_YERU Ы CYRILLIC CAPITAL LETTER YERU */
  { 0x06fa, 0x0417 }, /*                 Cyrillic_ZE З CYRILLIC CAPITAL LETTER ZE */
  { 0x06fb, 0x0428 }, /*                Cyrillic_SHA Ш CYRILLIC CAPITAL LETTER SHA */
  { 0x06fc, 0x042d }, /*                  Cyrillic_E Э CYRILLIC CAPITAL LETTER E */
  { 0x06fd, 0x0429 }, /*              Cyrillic_SHCHA Щ CYRILLIC CAPITAL LETTER SHCHA */
  { 0x06fe, 0x0427 }, /*                Cyrillic_CHE Ч CYRILLIC CAPITAL LETTER CHE */
  { 0x06ff, 0x042a }, /*           Cyrillic_HARDSIGN Ъ CYRILLIC CAPITAL LETTER HARD SIGN */
  { 0x07a1, 0x0386 }, /*           Greek_ALPHAaccent Ά GREEK CAPITAL LETTER ALPHA WITH TONOS */
  { 0x07a2, 0x0388 }, /*         Greek_EPSILONaccent Έ GREEK CAPITAL LETTER EPSILON WITH TONOS */
  { 0x07a3, 0x0389 }, /*             Greek_ETAaccent Ή GREEK CAPITAL LETTER ETA WITH TONOS */
  { 0x07a4, 0x038a }, /*            Greek_IOTAaccent Ί GREEK CAPITAL LETTER IOTA WITH TONOS */
  { 0x07a5, 0x03aa }, /*          Greek_IOTAdieresis Ϊ GREEK CAPITAL LETTER IOTA WITH DIALYTIKA */
  { 0x07a7, 0x038c }, /*         Greek_OMICRONaccent Ό GREEK CAPITAL LETTER OMICRON WITH TONOS */
  { 0x07a8, 0x038e }, /*         Greek_UPSILONaccent Ύ GREEK CAPITAL LETTER UPSILON WITH TONOS */
  { 0x07a9, 0x03ab }, /*       Greek_UPSILONdieresis Ϋ GREEK CAPITAL LETTER UPSILON WITH DIALYTIKA */
  { 0x07ab, 0x038f }, /*           Greek_OMEGAaccent Ώ GREEK CAPITAL LETTER OMEGA WITH TONOS */
  { 0x07ae, 0x0385 }, /*        Greek_accentdieresis ΅ GREEK DIALYTIKA TONOS */
  { 0x07af, 0x2015 }, /*              Greek_horizbar ― HORIZONTAL BAR */
  { 0x07b1, 0x03ac }, /*           Greek_alphaaccent ά GREEK SMALL LETTER ALPHA WITH TONOS */
  { 0x07b2, 0x03ad }, /*         Greek_epsilonaccent έ GREEK SMALL LETTER EPSILON WITH TONOS */
  { 0x07b3, 0x03ae }, /*             Greek_etaaccent ή GREEK SMALL LETTER ETA WITH TONOS */
  { 0x07b4, 0x03af }, /*            Greek_iotaaccent ί GREEK SMALL LETTER IOTA WITH TONOS */
  { 0x07b5, 0x03ca }, /*          Greek_iotadieresis ϊ GREEK SMALL LETTER IOTA WITH DIALYTIKA */
  { 0x07b6, 0x0390 }, /*    Greek_iotaaccentdieresis ΐ GREEK SMALL LETTER IOTA WITH DIALYTIKA AND TONOS */
  { 0x07b7, 0x03cc }, /*         Greek_omicronaccent ό GREEK SMALL LETTER OMICRON WITH TONOS */
  { 0x07b8, 0x03cd }, /*         Greek_upsilonaccent ύ GREEK SMALL LETTER UPSILON WITH TONOS */
  { 0x07b9, 0x03cb }, /*       Greek_upsilondieresis ϋ GREEK SMALL LETTER UPSILON WITH DIALYTIKA */
  { 0x07ba, 0x03b0 }, /* Greek_upsilonaccentdieresis ΰ GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND TONOS */
  { 0x07bb, 0x03ce }, /*           Greek_omegaaccent ώ GREEK SMALL LETTER OMEGA WITH TONOS */
  { 0x07c1, 0x0391 }, /*                 Greek_ALPHA Α GREEK CAPITAL LETTER ALPHA */
  { 0x07c2, 0x0392 }, /*                  Greek_BETA Β GREEK CAPITAL LETTER BETA */
  { 0x07c3, 0x0393 }, /*                 Greek_GAMMA Γ GREEK CAPITAL LETTER GAMMA */
  { 0x07c4, 0x0394 }, /*                 Greek_DELTA Δ GREEK CAPITAL LETTER DELTA */
  { 0x07c5, 0x0395 }, /*               Greek_EPSILON Ε GREEK CAPITAL LETTER EPSILON */
  { 0x07c6, 0x0396 }, /*                  Greek_ZETA Ζ GREEK CAPITAL LETTER ZETA */
  { 0x07c7, 0x0397 }, /*                   Greek_ETA Η GREEK CAPITAL LETTER ETA */
  { 0x07c8, 0x0398 }, /*                 Greek_THETA Θ GREEK CAPITAL LETTER THETA */
  { 0x07c9, 0x0399 }, /*                  Greek_IOTA Ι GREEK CAPITAL LETTER IOTA */
  { 0x07ca, 0x039a }, /*                 Greek_KAPPA Κ GREEK CAPITAL LETTER KAPPA */
  { 0x07cb, 0x039b }, /*                Greek_LAMBDA Λ GREEK CAPITAL LETTER LAMDA */
  { 0x07cc, 0x039c }, /*                    Greek_MU Μ GREEK CAPITAL LETTER MU */
  { 0x07cd, 0x039d }, /*                    Greek_NU Ν GREEK CAPITAL LETTER NU */
  { 0x07ce, 0x039e }, /*                    Greek_XI Ξ GREEK CAPITAL LETTER XI */
  { 0x07cf, 0x039f }, /*               Greek_OMICRON Ο GREEK CAPITAL LETTER OMICRON */
  { 0x07d0, 0x03a0 }, /*                    Greek_PI Π GREEK CAPITAL LETTER PI */
  { 0x07d1, 0x03a1 }, /*                   Greek_RHO Ρ GREEK CAPITAL LETTER RHO */
  { 0x07d2, 0x03a3 }, /*                 Greek_SIGMA Σ GREEK CAPITAL LETTER SIGMA */
  { 0x07d4, 0x03a4 }, /*                   Greek_TAU Τ GREEK CAPITAL LETTER TAU */
  { 0x07d5, 0x03a5 }, /*               Greek_UPSILON Υ GREEK CAPITAL LETTER UPSILON */
  { 0x07d6, 0x03a6 }, /*                   Greek_PHI Φ GREEK CAPITAL LETTER PHI */
  { 0x07d7, 0x03a7 }, /*                   Greek_CHI Χ GREEK CAPITAL LETTER CHI */
  { 0x07d8, 0x03a8 }, /*                   Greek_PSI Ψ GREEK CAPITAL LETTER PSI */
  { 0x07d9, 0x03a9 }, /*                 Greek_OMEGA Ω GREEK CAPITAL LETTER OMEGA */
  { 0x07e1, 0x03b1 }, /*                 Greek_alpha α GREEK SMALL LETTER ALPHA */
  { 0x07e2, 0x03b2 }, /*                  Greek_beta β GREEK SMALL LETTER BETA */
  { 0x07e3, 0x03b3 }, /*                 Greek_gamma γ GREEK SMALL LETTER GAMMA */
  { 0x07e4, 0x03b4 }, /*                 Greek_delta δ GREEK SMALL LETTER DELTA */
  { 0x07e5, 0x03b5 }, /*               Greek_epsilon ε GREEK SMALL LETTER EPSILON */
  { 0x07e6, 0x03b6 }, /*                  Greek_zeta ζ GREEK SMALL LETTER ZETA */
  { 0x07e7, 0x03b7 }, /*                   Greek_eta η GREEK SMALL LETTER ETA */
  { 0x07e8, 0x03b8 }, /*                 Greek_theta θ GREEK SMALL LETTER THETA */
  { 0x07e9, 0x03b9 }, /*                  Greek_iota ι GREEK SMALL LETTER IOTA */
  { 0x07ea, 0x03ba }, /*                 Greek_kappa κ GREEK SMALL LETTER KAPPA */
  { 0x07eb, 0x03bb }, /*                Greek_lambda λ GREEK SMALL LETTER LAMDA */
  { 0x07ec, 0x03bc }, /*                    Greek_mu μ GREEK SMALL LETTER MU */
  { 0x07ed, 0x03bd }, /*                    Greek_nu ν GREEK SMALL LETTER NU */
  { 0x07ee, 0x03be }, /*                    Greek_xi ξ GREEK SMALL LETTER XI */
  { 0x07ef, 0x03bf }, /*               Greek_omicron ο GREEK SMALL LETTER OMICRON */
  { 0x07f0, 0x03c0 }, /*                    Greek_pi π GREEK SMALL LETTER PI */
  { 0x07f1, 0x03c1 }, /*                   Greek_rho ρ GREEK SMALL LETTER RHO */
  { 0x07f2, 0x03c3 }, /*                 Greek_sigma σ GREEK SMALL LETTER SIGMA */
  { 0x07f3, 0x03c2 }, /*       Greek_finalsmallsigma ς GREEK SMALL LETTER FINAL SIGMA */
  { 0x07f4, 0x03c4 }, /*                   Greek_tau τ GREEK SMALL LETTER TAU */
  { 0x07f5, 0x03c5 }, /*               Greek_upsilon υ GREEK SMALL LETTER UPSILON */
  { 0x07f6, 0x03c6 }, /*                   Greek_phi φ GREEK SMALL LETTER PHI */
  { 0x07f7, 0x03c7 }, /*                   Greek_chi χ GREEK SMALL LETTER CHI */
  { 0x07f8, 0x03c8 }, /*                   Greek_psi ψ GREEK SMALL LETTER PSI */
  { 0x07f9, 0x03c9 }, /*                 Greek_omega ω GREEK SMALL LETTER OMEGA */
/*  0x08a1                               leftradical ? ??? */
/*  0x08a2                            topleftradical ? ??? */
/*  0x08a3                            horizconnector ? ??? */
  { 0x08a4, 0x2320 }, /*                 topintegral ⌠ TOP HALF INTEGRAL */
  { 0x08a5, 0x2321 }, /*                 botintegral ⌡ BOTTOM HALF INTEGRAL */
  { 0x08a6, 0x2502 }, /*               vertconnector │ BOX DRAWINGS LIGHT VERTICAL */
/*  0x08a7                          topleftsqbracket ? ??? */
/*  0x08a8                          botleftsqbracket ? ??? */
/*  0x08a9                         toprightsqbracket ? ??? */
/*  0x08aa                         botrightsqbracket ? ??? */
/*  0x08ab                             topleftparens ? ??? */
/*  0x08ac                             botleftparens ? ??? */
/*  0x08ad                            toprightparens ? ??? */
/*  0x08ae                            botrightparens ? ??? */
/*  0x08af                      leftmiddlecurlybrace ? ??? */
/*  0x08b0                     rightmiddlecurlybrace ? ??? */
/*  0x08b1                          topleftsummation ? ??? */
/*  0x08b2                          botleftsummation ? ??? */
/*  0x08b3                 topvertsummationconnector ? ??? */
/*  0x08b4                 botvertsummationconnector ? ??? */
/*  0x08b5                         toprightsummation ? ??? */
/*  0x08b6                         botrightsummation ? ??? */
/*  0x08b7                      rightmiddlesummation ? ??? */
  { 0x08bc, 0x2264 }, /*               lessthanequal ≤ LESS-THAN OR EQUAL TO */
  { 0x08bd, 0x2260 }, /*                    notequal ≠ NOT EQUAL TO */
  { 0x08be, 0x2265 }, /*            greaterthanequal ≥ GREATER-THAN OR EQUAL TO */
  { 0x08bf, 0x222b }, /*                    integral ∫ INTEGRAL */
  { 0x08c0, 0x2234 }, /*                   therefore ∴ THEREFORE */
  { 0x08c1, 0x221d }, /*                   variation ∝ PROPORTIONAL TO */
  { 0x08c2, 0x221e }, /*                    infinity ∞ INFINITY */
  { 0x08c5, 0x2207 }, /*                       nabla ∇ NABLA */
  { 0x08c8, 0x2245 }, /*                 approximate ≅ APPROXIMATELY EQUAL TO */
/*  0x08c9                              similarequal ? ??? */
  { 0x08cd, 0x21d4 }, /*                    ifonlyif ⇔ LEFT RIGHT DOUBLE ARROW */
  { 0x08ce, 0x21d2 }, /*                     implies ⇒ RIGHTWARDS DOUBLE ARROW */
  { 0x08cf, 0x2261 }, /*                   identical ≡ IDENTICAL TO */
  { 0x08d6, 0x221a }, /*                     radical √ SQUARE ROOT */
  { 0x08da, 0x2282 }, /*                  includedin ⊂ SUBSET OF */
  { 0x08db, 0x2283 }, /*                    includes ⊃ SUPERSET OF */
  { 0x08dc, 0x2229 }, /*                intersection ∩ INTERSECTION */
  { 0x08dd, 0x222a }, /*                       union ∪ UNION */
  { 0x08de, 0x2227 }, /*                  logicaland ∧ LOGICAL AND */
  { 0x08df, 0x2228 }, /*                   logicalor ∨ LOGICAL OR */
  { 0x08ef, 0x2202 }, /*           partialderivative ∂ PARTIAL DIFFERENTIAL */
  { 0x08f6, 0x0192 }, /*                    function ƒ LATIN SMALL LETTER F WITH HOOK */
  { 0x08fb, 0x2190 }, /*                   leftarrow ← LEFTWARDS ARROW */
  { 0x08fc, 0x2191 }, /*                     uparrow ↑ UPWARDS ARROW */
  { 0x08fd, 0x2192 }, /*                  rightarrow → RIGHTWARDS ARROW */
  { 0x08fe, 0x2193 }, /*                   downarrow ↓ DOWNWARDS ARROW */
  { 0x09df, 0x2422 }, /*                       blank ␢ BLANK SYMBOL */
  { 0x09e0, 0x25c6 }, /*                soliddiamond ◆ BLACK DIAMOND */
  { 0x09e1, 0x2592 }, /*                checkerboard ▒ MEDIUM SHADE */
  { 0x09e2, 0x2409 }, /*                          ht ␉ SYMBOL FOR HORIZONTAL TABULATION */
  { 0x09e3, 0x240c }, /*                          ff ␌ SYMBOL FOR FORM FEED */
  { 0x09e4, 0x240d }, /*                          cr ␍ SYMBOL FOR CARRIAGE RETURN */
  { 0x09e5, 0x240a }, /*                          lf ␊ SYMBOL FOR LINE FEED */
  { 0x09e8, 0x2424 }, /*                          nl ␤ SYMBOL FOR NEWLINE */
  { 0x09e9, 0x240b }, /*                          vt ␋ SYMBOL FOR VERTICAL TABULATION */
  { 0x09ea, 0x2518 }, /*              lowrightcorner ┘ BOX DRAWINGS LIGHT UP AND LEFT */
  { 0x09eb, 0x2510 }, /*               uprightcorner ┐ BOX DRAWINGS LIGHT DOWN AND LEFT */
  { 0x09ec, 0x250c }, /*                upleftcorner ┌ BOX DRAWINGS LIGHT DOWN AND RIGHT */
  { 0x09ed, 0x2514 }, /*               lowleftcorner └ BOX DRAWINGS LIGHT UP AND RIGHT */
  { 0x09ee, 0x253c }, /*               crossinglines ┼ BOX DRAWINGS LIGHT VERTICAL AND HORIZONTAL */
 /*  0x09ef                            horizlinescan1 ? ??? */
 /*  0x09f0                            horizlinescan3 ? ??? */
  { 0x09f1, 0x2500 }, /*              horizlinescan5 ─ BOX DRAWINGS LIGHT HORIZONTAL */
 /*  0x09f2                            horizlinescan7 ? ??? */
 /*  0x09f3                            horizlinescan9 ? ??? */
  { 0x09f4, 0x251c }, /*                       leftt ├ BOX DRAWINGS LIGHT VERTICAL AND RIGHT */
  { 0x09f5, 0x2524 }, /*                      rightt ┤ BOX DRAWINGS LIGHT VERTICAL AND LEFT */
  { 0x09f6, 0x2534 }, /*                        bott ┴ BOX DRAWINGS LIGHT UP AND HORIZONTAL */
  { 0x09f7, 0x252c }, /*                        topt ┬ BOX DRAWINGS LIGHT DOWN AND HORIZONTAL */
  { 0x09f8, 0x2502 }, /*                     vertbar │ BOX DRAWINGS LIGHT VERTICAL */
  { 0x0aa1, 0x2003 }, /*                     emspace   EM SPACE */
  { 0x0aa2, 0x2002 }, /*                     enspace   EN SPACE */
  { 0x0aa3, 0x2004 }, /*                    em3space   THREE-PER-EM SPACE */
  { 0x0aa4, 0x2005 }, /*                    em4space   FOUR-PER-EM SPACE */
  { 0x0aa5, 0x2007 }, /*                  digitspace   FIGURE SPACE */
  { 0x0aa6, 0x2008 }, /*                  punctspace   PUNCTUATION SPACE */
  { 0x0aa7, 0x2009 }, /*                   thinspace   THIN SPACE */
  { 0x0aa8, 0x200a }, /*                   hairspace   HAIR SPACE */
  { 0x0aa9, 0x2014 }, /*                      emdash — EM DASH */
  { 0x0aaa, 0x2013 }, /*                      endash – EN DASH */
/*  0x0aac                               signifblank ? ??? */
  { 0x0aae, 0x2026 }, /*                    ellipsis … HORIZONTAL ELLIPSIS */
/*  0x0aaf                           doubbaselinedot ? ??? */
  { 0x0ab0, 0x2153 }, /*                    onethird ⅓ VULGAR FRACTION ONE THIRD */
  { 0x0ab1, 0x2154 }, /*                   twothirds ⅔ VULGAR FRACTION TWO THIRDS */
  { 0x0ab2, 0x2155 }, /*                    onefifth ⅕ VULGAR FRACTION ONE FIFTH */
  { 0x0ab3, 0x2156 }, /*                   twofifths ⅖ VULGAR FRACTION TWO FIFTHS */
  { 0x0ab4, 0x2157 }, /*                 threefifths ⅗ VULGAR FRACTION THREE FIFTHS */
  { 0x0ab5, 0x2158 }, /*                  fourfifths ⅘ VULGAR FRACTION FOUR FIFTHS */
  { 0x0ab6, 0x2159 }, /*                    onesixth ⅙ VULGAR FRACTION ONE SIXTH */
  { 0x0ab7, 0x215a }, /*                  fivesixths ⅚ VULGAR FRACTION FIVE SIXTHS */
  { 0x0ab8, 0x2105 }, /*                      careof ℅ CARE OF */
  { 0x0abb, 0x2012 }, /*                     figdash ‒ FIGURE DASH */
  { 0x0abc, 0x2329 }, /*            leftanglebracket 〈 LEFT-POINTING ANGLE BRACKET */
  { 0x0abd, 0x002e }, /*                decimalpoint . FULL STOP */
  { 0x0abe, 0x232a }, /*           rightanglebracket 〉 RIGHT-POINTING ANGLE BRACKET */
 /*  0x0abf                                    marker ? ??? */
  { 0x0ac3, 0x215b }, /*                   oneeighth ⅛ VULGAR FRACTION ONE EIGHTH */
  { 0x0ac4, 0x215c }, /*                threeeighths ⅜ VULGAR FRACTION THREE EIGHTHS */
  { 0x0ac5, 0x215d }, /*                 fiveeighths ⅝ VULGAR FRACTION FIVE EIGHTHS */
  { 0x0ac6, 0x215e }, /*                seveneighths ⅞ VULGAR FRACTION SEVEN EIGHTHS */
  { 0x0ac9, 0x2122 }, /*                   trademark ™ TRADE MARK SIGN */
  { 0x0aca, 0x2613 }, /*               signaturemark ☓ SALTIRE */
 /*  0x0acb                         trademarkincircle ? ??? */
  { 0x0acc, 0x25c1 }, /*            leftopentriangle ◁ WHITE LEFT-POINTING TRIANGLE */
  { 0x0acd, 0x25b7 }, /*           rightopentriangle ▷ WHITE RIGHT-POINTING TRIANGLE */
  { 0x0ace, 0x25cb }, /*                emopencircle ○ WHITE CIRCLE */
  { 0x0acf, 0x25a1 }, /*             emopenrectangle □ WHITE SQUARE */
  { 0x0ad0, 0x2018 }, /*         leftsinglequotemark ‘ LEFT SINGLE QUOTATION MARK */
  { 0x0ad1, 0x2019 }, /*        rightsinglequotemark ’ RIGHT SINGLE QUOTATION MARK */
  { 0x0ad2, 0x201c }, /*         leftdoublequotemark “ LEFT DOUBLE QUOTATION MARK */
  { 0x0ad3, 0x201d }, /*        rightdoublequotemark ” RIGHT DOUBLE QUOTATION MARK */
  { 0x0ad4, 0x211e }, /*                prescription ℞ PRESCRIPTION TAKE */
  { 0x0ad6, 0x2032 }, /*                     minutes ′ PRIME */
  { 0x0ad7, 0x2033 }, /*                     seconds ″ DOUBLE PRIME */
  { 0x0ad9, 0x271d }, /*                  latincross ✝ LATIN CROSS */
 /*  0x0ada                                  hexagram ? ??? */
  { 0x0adb, 0x25ac }, /*            filledrectbullet ▬ BLACK RECTANGLE */
  { 0x0adc, 0x25c0 }, /*         filledlefttribullet ◀ BLACK LEFT-POINTING TRIANGLE */
  { 0x0add, 0x25b6 }, /*        filledrighttribullet ▶ BLACK RIGHT-POINTING TRIANGLE */
  { 0x0ade, 0x25cf }, /*              emfilledcircle ● BLACK CIRCLE */
  { 0x0adf, 0x25a0 }, /*                emfilledrect ■ BLACK SQUARE */
  { 0x0ae0, 0x25e6 }, /*            enopencircbullet ◦ WHITE BULLET */
  { 0x0ae1, 0x25ab }, /*          enopensquarebullet ▫ WHITE SMALL SQUARE */
  { 0x0ae2, 0x25ad }, /*              openrectbullet ▭ WHITE RECTANGLE */
  { 0x0ae3, 0x25b3 }, /*             opentribulletup △ WHITE UP-POINTING TRIANGLE */
  { 0x0ae4, 0x25bd }, /*           opentribulletdown ▽ WHITE DOWN-POINTING TRIANGLE */
  { 0x0ae5, 0x2606 }, /*                    openstar ☆ WHITE STAR */
  { 0x0ae6, 0x2022 }, /*          enfilledcircbullet • BULLET */
  { 0x0ae7, 0x25aa }, /*            enfilledsqbullet ▪ BLACK SMALL SQUARE */
  { 0x0ae8, 0x25b2 }, /*           filledtribulletup ▲ BLACK UP-POINTING TRIANGLE */
  { 0x0ae9, 0x25bc }, /*         filledtribulletdown ▼ BLACK DOWN-POINTING TRIANGLE */
  { 0x0aea, 0x261c }, /*                 leftpointer ☜ WHITE LEFT POINTING INDEX */
  { 0x0aeb, 0x261e }, /*                rightpointer ☞ WHITE RIGHT POINTING INDEX */
  { 0x0aec, 0x2663 }, /*                        club ♣ BLACK CLUB SUIT */
  { 0x0aed, 0x2666 }, /*                     diamond ♦ BLACK DIAMOND SUIT */
  { 0x0aee, 0x2665 }, /*                       heart ♥ BLACK HEART SUIT */
  { 0x0af0, 0x2720 }, /*                maltesecross ✠ MALTESE CROSS */
  { 0x0af1, 0x2020 }, /*                      dagger † DAGGER */
  { 0x0af2, 0x2021 }, /*                doubledagger ‡ DOUBLE DAGGER */
  { 0x0af3, 0x2713 }, /*                   checkmark ✓ CHECK MARK */
  { 0x0af4, 0x2717 }, /*                 ballotcross ✗ BALLOT X */
  { 0x0af5, 0x266f }, /*                musicalsharp ♯ MUSIC SHARP SIGN */
  { 0x0af6, 0x266d }, /*                 musicalflat ♭ MUSIC FLAT SIGN */
  { 0x0af7, 0x2642 }, /*                  malesymbol ♂ MALE SIGN */
  { 0x0af8, 0x2640 }, /*                femalesymbol ♀ FEMALE SIGN */
  { 0x0af9, 0x260e }, /*                   telephone ☎ BLACK TELEPHONE */
  { 0x0afa, 0x2315 }, /*           telephonerecorder ⌕ TELEPHONE RECORDER */
  { 0x0afb, 0x2117 }, /*         phonographcopyright ℗ SOUND RECORDING COPYRIGHT */
  { 0x0afc, 0x2038 }, /*                       caret ‸ CARET */
  { 0x0afd, 0x201a }, /*          singlelowquotemark ‚ SINGLE LOW-9 QUOTATION MARK */
  { 0x0afe, 0x201e }, /*          doublelowquotemark „ DOUBLE LOW-9 QUOTATION MARK */
 /*  0x0aff                                    cursor ? ??? */
  { 0x0ba3, 0x003c }, /*                   leftcaret < LESS-THAN SIGN */
  { 0x0ba6, 0x003e }, /*                  rightcaret > GREATER-THAN SIGN */
  { 0x0ba8, 0x2228 }, /*                   downcaret ∨ LOGICAL OR */
  { 0x0ba9, 0x2227 }, /*                     upcaret ∧ LOGICAL AND */
  { 0x0bc0, 0x00af }, /*                     overbar ¯ MACRON */
  { 0x0bc2, 0x22a4 }, /*                    downtack ⊤ DOWN TACK */
  { 0x0bc3, 0x2229 }, /*                      upshoe ∩ INTERSECTION */
  { 0x0bc4, 0x230a }, /*                   downstile ⌊ LEFT FLOOR */
  { 0x0bc6, 0x005f }, /*                    underbar _ LOW LINE */
  { 0x0bca, 0x2218 }, /*                         jot ∘ RING OPERATOR */
  { 0x0bcc, 0x2395 }, /*                        quad ⎕ APL FUNCTIONAL SYMBOL QUAD (Unicode 3.0) */
  { 0x0bce, 0x22a5 }, /*                      uptack ⊥ UP TACK */
  { 0x0bcf, 0x25cb }, /*                      circle ○ WHITE CIRCLE */
  { 0x0bd3, 0x2308 }, /*                     upstile ⌈ LEFT CEILING */
  { 0x0bd6, 0x222a }, /*                    downshoe ∪ UNION */
  { 0x0bd8, 0x2283 }, /*                   rightshoe ⊃ SUPERSET OF */
  { 0x0bda, 0x2282 }, /*                    leftshoe ⊂ SUBSET OF */
  { 0x0bdc, 0x22a3 }, /*                    lefttack ⊣ LEFT TACK */
  { 0x0bfc, 0x22a2 }, /*                   righttack ⊢ RIGHT TACK */
  { 0x0cdf, 0x2017 }, /*        hebrew_doublelowline ‗ DOUBLE LOW LINE */
  { 0x0ce0, 0x05d0 }, /*                hebrew_aleph א HEBREW LETTER ALEF */
  { 0x0ce1, 0x05d1 }, /*                  hebrew_bet ב HEBREW LETTER BET */
  { 0x0ce2, 0x05d2 }, /*                hebrew_gimel ג HEBREW LETTER GIMEL */
  { 0x0ce3, 0x05d3 }, /*                hebrew_dalet ד HEBREW LETTER DALET */
  { 0x0ce4, 0x05d4 }, /*                   hebrew_he ה HEBREW LETTER HE */
  { 0x0ce5, 0x05d5 }, /*                  hebrew_waw ו HEBREW LETTER VAV */
  { 0x0ce6, 0x05d6 }, /*                 hebrew_zain ז HEBREW LETTER ZAYIN */
  { 0x0ce7, 0x05d7 }, /*                 hebrew_chet ח HEBREW LETTER HET */
  { 0x0ce8, 0x05d8 }, /*                  hebrew_tet ט HEBREW LETTER TET */
  { 0x0ce9, 0x05d9 }, /*                  hebrew_yod י HEBREW LETTER YOD */
  { 0x0cea, 0x05da }, /*            hebrew_finalkaph ך HEBREW LETTER FINAL KAF */
  { 0x0ceb, 0x05db }, /*                 hebrew_kaph כ HEBREW LETTER KAF */
  { 0x0cec, 0x05dc }, /*                hebrew_lamed ל HEBREW LETTER LAMED */
  { 0x0ced, 0x05dd }, /*             hebrew_finalmem ם HEBREW LETTER FINAL MEM */
  { 0x0cee, 0x05de }, /*                  hebrew_mem מ HEBREW LETTER MEM */
  { 0x0cef, 0x05df }, /*             hebrew_finalnun ן HEBREW LETTER FINAL NUN */
  { 0x0cf0, 0x05e0 }, /*                  hebrew_nun נ HEBREW LETTER NUN */
  { 0x0cf1, 0x05e1 }, /*               hebrew_samech ס HEBREW LETTER SAMEKH */
  { 0x0cf2, 0x05e2 }, /*                 hebrew_ayin ע HEBREW LETTER AYIN */
  { 0x0cf3, 0x05e3 }, /*              hebrew_finalpe ף HEBREW LETTER FINAL PE */
  { 0x0cf4, 0x05e4 }, /*                   hebrew_pe פ HEBREW LETTER PE */
  { 0x0cf5, 0x05e5 }, /*            hebrew_finalzade ץ HEBREW LETTER FINAL TSADI */
  { 0x0cf6, 0x05e6 }, /*                 hebrew_zade צ HEBREW LETTER TSADI */
  { 0x0cf7, 0x05e7 }, /*                 hebrew_qoph ק HEBREW LETTER QOF */
  { 0x0cf8, 0x05e8 }, /*                 hebrew_resh ר HEBREW LETTER RESH */
  { 0x0cf9, 0x05e9 }, /*                 hebrew_shin ש HEBREW LETTER SHIN */
  { 0x0cfa, 0x05ea }, /*                  hebrew_taw ת HEBREW LETTER TAV */
  { 0x0da1, 0x0e01 }, /*                  Thai_kokai ก THAI CHARACTER KO KAI */
  { 0x0da2, 0x0e02 }, /*                Thai_khokhai ข THAI CHARACTER KHO KHAI */
  { 0x0da3, 0x0e03 }, /*               Thai_khokhuat ฃ THAI CHARACTER KHO KHUAT */
  { 0x0da4, 0x0e04 }, /*               Thai_khokhwai ค THAI CHARACTER KHO KHWAI */
  { 0x0da5, 0x0e05 }, /*                Thai_khokhon ฅ THAI CHARACTER KHO KHON */
  { 0x0da6, 0x0e06 }, /*             Thai_khorakhang ฆ THAI CHARACTER KHO RAKHANG */
  { 0x0da7, 0x0e07 }, /*                 Thai_ngongu ง THAI CHARACTER NGO NGU */
  { 0x0da8, 0x0e08 }, /*                Thai_chochan จ THAI CHARACTER CHO CHAN */
  { 0x0da9, 0x0e09 }, /*               Thai_choching ฉ THAI CHARACTER CHO CHING */
  { 0x0daa, 0x0e0a }, /*               Thai_chochang ช THAI CHARACTER CHO CHANG */
  { 0x0dab, 0x0e0b }, /*                   Thai_soso ซ THAI CHARACTER SO SO */
  { 0x0dac, 0x0e0c }, /*                Thai_chochoe ฌ THAI CHARACTER CHO CHOE */
  { 0x0dad, 0x0e0d }, /*                 Thai_yoying ญ THAI CHARACTER YO YING */
  { 0x0dae, 0x0e0e }, /*                Thai_dochada ฎ THAI CHARACTER DO CHADA */
  { 0x0daf, 0x0e0f }, /*                Thai_topatak ฏ THAI CHARACTER TO PATAK */
  { 0x0db0, 0x0e10 }, /*                Thai_thothan ฐ THAI CHARACTER THO THAN */
  { 0x0db1, 0x0e11 }, /*          Thai_thonangmontho ฑ THAI CHARACTER THO NANGMONTHO */
  { 0x0db2, 0x0e12 }, /*             Thai_thophuthao ฒ THAI CHARACTER THO PHUTHAO */
  { 0x0db3, 0x0e13 }, /*                  Thai_nonen ณ THAI CHARACTER NO NEN */
  { 0x0db4, 0x0e14 }, /*                  Thai_dodek ด THAI CHARACTER DO DEK */
  { 0x0db5, 0x0e15 }, /*                  Thai_totao ต THAI CHARACTER TO TAO */
  { 0x0db6, 0x0e16 }, /*               Thai_thothung ถ THAI CHARACTER THO THUNG */
  { 0x0db7, 0x0e17 }, /*              Thai_thothahan ท THAI CHARACTER THO THAHAN */
  { 0x0db8, 0x0e18 }, /*               Thai_thothong ธ THAI CHARACTER THO THONG */
  { 0x0db9, 0x0e19 }, /*                   Thai_nonu น THAI CHARACTER NO NU */
  { 0x0dba, 0x0e1a }, /*               Thai_bobaimai บ THAI CHARACTER BO BAIMAI */
  { 0x0dbb, 0x0e1b }, /*                  Thai_popla ป THAI CHARACTER PO PLA */
  { 0x0dbc, 0x0e1c }, /*               Thai_phophung ผ THAI CHARACTER PHO PHUNG */
  { 0x0dbd, 0x0e1d }, /*                   Thai_fofa ฝ THAI CHARACTER FO FA */
  { 0x0dbe, 0x0e1e }, /*                Thai_phophan พ THAI CHARACTER PHO PHAN */
  { 0x0dbf, 0x0e1f }, /*                  Thai_fofan ฟ THAI CHARACTER FO FAN */
  { 0x0dc0, 0x0e20 }, /*             Thai_phosamphao ภ THAI CHARACTER PHO SAMPHAO */
  { 0x0dc1, 0x0e21 }, /*                   Thai_moma ม THAI CHARACTER MO MA */
  { 0x0dc2, 0x0e22 }, /*                  Thai_yoyak ย THAI CHARACTER YO YAK */
  { 0x0dc3, 0x0e23 }, /*                  Thai_rorua ร THAI CHARACTER RO RUA */
  { 0x0dc4, 0x0e24 }, /*                     Thai_ru ฤ THAI CHARACTER RU */
  { 0x0dc5, 0x0e25 }, /*                 Thai_loling ล THAI CHARACTER LO LING */
  { 0x0dc6, 0x0e26 }, /*                     Thai_lu ฦ THAI CHARACTER LU */
  { 0x0dc7, 0x0e27 }, /*                 Thai_wowaen ว THAI CHARACTER WO WAEN */
  { 0x0dc8, 0x0e28 }, /*                 Thai_sosala ศ THAI CHARACTER SO SALA */
  { 0x0dc9, 0x0e29 }, /*                 Thai_sorusi ษ THAI CHARACTER SO RUSI */
  { 0x0dca, 0x0e2a }, /*                  Thai_sosua ส THAI CHARACTER SO SUA */
  { 0x0dcb, 0x0e2b }, /*                  Thai_hohip ห THAI CHARACTER HO HIP */
  { 0x0dcc, 0x0e2c }, /*                Thai_lochula ฬ THAI CHARACTER LO CHULA */
  { 0x0dcd, 0x0e2d }, /*                   Thai_oang อ THAI CHARACTER O ANG */
  { 0x0dce, 0x0e2e }, /*               Thai_honokhuk ฮ THAI CHARACTER HO NOKHUK */
  { 0x0dcf, 0x0e2f }, /*              Thai_paiyannoi ฯ THAI CHARACTER PAIYANNOI */
  { 0x0dd0, 0x0e30 }, /*                  Thai_saraa ะ THAI CHARACTER SARA A */
  { 0x0dd1, 0x0e31 }, /*             Thai_maihanakat ั THAI CHARACTER MAI HAN-AKAT */
  { 0x0dd2, 0x0e32 }, /*                 Thai_saraaa า THAI CHARACTER SARA AA */
  { 0x0dd3, 0x0e33 }, /*                 Thai_saraam ำ THAI CHARACTER SARA AM */
  { 0x0dd4, 0x0e34 }, /*                  Thai_sarai ิ THAI CHARACTER SARA I */
  { 0x0dd5, 0x0e35 }, /*                 Thai_saraii ี THAI CHARACTER SARA II */
  { 0x0dd6, 0x0e36 }, /*                 Thai_saraue ึ THAI CHARACTER SARA UE */
  { 0x0dd7, 0x0e37 }, /*                Thai_sarauee ื THAI CHARACTER SARA UEE */
  { 0x0dd8, 0x0e38 }, /*                  Thai_sarau ุ THAI CHARACTER SARA U */
  { 0x0dd9, 0x0e39 }, /*                 Thai_sarauu ู THAI CHARACTER SARA UU */
  { 0x0dda, 0x0e3a }, /*                Thai_phinthu ฺ THAI CHARACTER PHINTHU */
  { 0x0dde, 0x0e3e }, /*      Thai_maihanakat_maitho ฾ ??? */
  { 0x0ddf, 0x0e3f }, /*                   Thai_baht ฿ THAI CURRENCY SYMBOL BAHT */
  { 0x0de0, 0x0e40 }, /*                  Thai_sarae เ THAI CHARACTER SARA E */
  { 0x0de1, 0x0e41 }, /*                 Thai_saraae แ THAI CHARACTER SARA AE */
  { 0x0de2, 0x0e42 }, /*                  Thai_sarao โ THAI CHARACTER SARA O */
  { 0x0de3, 0x0e43 }, /*          Thai_saraaimaimuan ใ THAI CHARACTER SARA AI MAIMUAN */
  { 0x0de4, 0x0e44 }, /*         Thai_saraaimaimalai ไ THAI CHARACTER SARA AI MAIMALAI */
  { 0x0de5, 0x0e45 }, /*            Thai_lakkhangyao ๅ THAI CHARACTER LAKKHANGYAO */
  { 0x0de6, 0x0e46 }, /*               Thai_maiyamok ๆ THAI CHARACTER MAIYAMOK */
  { 0x0de7, 0x0e47 }, /*              Thai_maitaikhu ็ THAI CHARACTER MAITAIKHU */
  { 0x0de8, 0x0e48 }, /*                  Thai_maiek ่ THAI CHARACTER MAI EK */
  { 0x0de9, 0x0e49 }, /*                 Thai_maitho ้ THAI CHARACTER MAI THO */
  { 0x0dea, 0x0e4a }, /*                 Thai_maitri ๊ THAI CHARACTER MAI TRI */
  { 0x0deb, 0x0e4b }, /*            Thai_maichattawa ๋ THAI CHARACTER MAI CHATTAWA */
  { 0x0dec, 0x0e4c }, /*            Thai_thanthakhat ์ THAI CHARACTER THANTHAKHAT */
  { 0x0ded, 0x0e4d }, /*               Thai_nikhahit ํ THAI CHARACTER NIKHAHIT */
  { 0x0df0, 0x0e50 }, /*                 Thai_leksun ๐ THAI DIGIT ZERO */
  { 0x0df1, 0x0e51 }, /*                Thai_leknung ๑ THAI DIGIT ONE */
  { 0x0df2, 0x0e52 }, /*                Thai_leksong ๒ THAI DIGIT TWO */
  { 0x0df3, 0x0e53 }, /*                 Thai_leksam ๓ THAI DIGIT THREE */
  { 0x0df4, 0x0e54 }, /*                  Thai_leksi ๔ THAI DIGIT FOUR */
  { 0x0df5, 0x0e55 }, /*                  Thai_lekha ๕ THAI DIGIT FIVE */
  { 0x0df6, 0x0e56 }, /*                 Thai_lekhok ๖ THAI DIGIT SIX */
  { 0x0df7, 0x0e57 }, /*                Thai_lekchet ๗ THAI DIGIT SEVEN */
  { 0x0df8, 0x0e58 }, /*                Thai_lekpaet ๘ THAI DIGIT EIGHT */
  { 0x0df9, 0x0e59 }, /*                 Thai_lekkao ๙ THAI DIGIT NINE */
  { 0x0ea1, 0x3131 }, /*               Hangul_Kiyeog ㄱ HANGUL LETTER KIYEOK */
  { 0x0ea2, 0x3132 }, /*          Hangul_SsangKiyeog ㄲ HANGUL LETTER SSANGKIYEOK */
  { 0x0ea3, 0x3133 }, /*           Hangul_KiyeogSios ㄳ HANGUL LETTER KIYEOK-SIOS */
  { 0x0ea4, 0x3134 }, /*                Hangul_Nieun ㄴ HANGUL LETTER NIEUN */
  { 0x0ea5, 0x3135 }, /*           Hangul_NieunJieuj ㄵ HANGUL LETTER NIEUN-CIEUC */
  { 0x0ea6, 0x3136 }, /*           Hangul_NieunHieuh ㄶ HANGUL LETTER NIEUN-HIEUH */
  { 0x0ea7, 0x3137 }, /*               Hangul_Dikeud ㄷ HANGUL LETTER TIKEUT */
  { 0x0ea8, 0x3138 }, /*          Hangul_SsangDikeud ㄸ HANGUL LETTER SSANGTIKEUT */
  { 0x0ea9, 0x3139 }, /*                Hangul_Rieul ㄹ HANGUL LETTER RIEUL */
  { 0x0eaa, 0x313a }, /*          Hangul_RieulKiyeog ㄺ HANGUL LETTER RIEUL-KIYEOK */
  { 0x0eab, 0x313b }, /*           Hangul_RieulMieum ㄻ HANGUL LETTER RIEUL-MIEUM */
  { 0x0eac, 0x313c }, /*           Hangul_RieulPieub ㄼ HANGUL LETTER RIEUL-PIEUP */
  { 0x0ead, 0x313d }, /*            Hangul_RieulSios ㄽ HANGUL LETTER RIEUL-SIOS */
  { 0x0eae, 0x313e }, /*           Hangul_RieulTieut ㄾ HANGUL LETTER RIEUL-THIEUTH */
  { 0x0eaf, 0x313f }, /*          Hangul_RieulPhieuf ㄿ HANGUL LETTER RIEUL-PHIEUPH */
  { 0x0eb0, 0x3140 }, /*           Hangul_RieulHieuh ㅀ HANGUL LETTER RIEUL-HIEUH */
  { 0x0eb1, 0x3141 }, /*                Hangul_Mieum ㅁ HANGUL LETTER MIEUM */
  { 0x0eb2, 0x3142 }, /*                Hangul_Pieub ㅂ HANGUL LETTER PIEUP */
  { 0x0eb3, 0x3143 }, /*           Hangul_SsangPieub ㅃ HANGUL LETTER SSANGPIEUP */
  { 0x0eb4, 0x3144 }, /*            Hangul_PieubSios ㅄ HANGUL LETTER PIEUP-SIOS */
  { 0x0eb5, 0x3145 }, /*                 Hangul_Sios ㅅ HANGUL LETTER SIOS */
  { 0x0eb6, 0x3146 }, /*            Hangul_SsangSios ㅆ HANGUL LETTER SSANGSIOS */
  { 0x0eb7, 0x3147 }, /*                Hangul_Ieung ㅇ HANGUL LETTER IEUNG */
  { 0x0eb8, 0x3148 }, /*                Hangul_Jieuj ㅈ HANGUL LETTER CIEUC */
  { 0x0eb9, 0x3149 }, /*           Hangul_SsangJieuj ㅉ HANGUL LETTER SSANGCIEUC */
  { 0x0eba, 0x314a }, /*                Hangul_Cieuc ㅊ HANGUL LETTER CHIEUCH */
  { 0x0ebb, 0x314b }, /*               Hangul_Khieuq ㅋ HANGUL LETTER KHIEUKH */
  { 0x0ebc, 0x314c }, /*                Hangul_Tieut ㅌ HANGUL LETTER THIEUTH */
  { 0x0ebd, 0x314d }, /*               Hangul_Phieuf ㅍ HANGUL LETTER PHIEUPH */
  { 0x0ebe, 0x314e }, /*                Hangul_Hieuh ㅎ HANGUL LETTER HIEUH */
  { 0x0ebf, 0x314f }, /*                    Hangul_A ㅏ HANGUL LETTER A */
  { 0x0ec0, 0x3150 }, /*                   Hangul_AE ㅐ HANGUL LETTER AE */
  { 0x0ec1, 0x3151 }, /*                   Hangul_YA ㅑ HANGUL LETTER YA */
  { 0x0ec2, 0x3152 }, /*                  Hangul_YAE ㅒ HANGUL LETTER YAE */
  { 0x0ec3, 0x3153 }, /*                   Hangul_EO ㅓ HANGUL LETTER EO */
  { 0x0ec4, 0x3154 }, /*                    Hangul_E ㅔ HANGUL LETTER E */
  { 0x0ec5, 0x3155 }, /*                  Hangul_YEO ㅕ HANGUL LETTER YEO */
  { 0x0ec6, 0x3156 }, /*                   Hangul_YE ㅖ HANGUL LETTER YE */
  { 0x0ec7, 0x3157 }, /*                    Hangul_O ㅗ HANGUL LETTER O */
  { 0x0ec8, 0x3158 }, /*                   Hangul_WA ㅘ HANGUL LETTER WA */
  { 0x0ec9, 0x3159 }, /*                  Hangul_WAE ㅙ HANGUL LETTER WAE */
  { 0x0eca, 0x315a }, /*                   Hangul_OE ㅚ HANGUL LETTER OE */
  { 0x0ecb, 0x315b }, /*                   Hangul_YO ㅛ HANGUL LETTER YO */
  { 0x0ecc, 0x315c }, /*                    Hangul_U ㅜ HANGUL LETTER U */
  { 0x0ecd, 0x315d }, /*                  Hangul_WEO ㅝ HANGUL LETTER WEO */
  { 0x0ece, 0x315e }, /*                   Hangul_WE ㅞ HANGUL LETTER WE */
  { 0x0ecf, 0x315f }, /*                   Hangul_WI ㅟ HANGUL LETTER WI */
  { 0x0ed0, 0x3160 }, /*                   Hangul_YU ㅠ HANGUL LETTER YU */
  { 0x0ed1, 0x3161 }, /*                   Hangul_EU ㅡ HANGUL LETTER EU */
  { 0x0ed2, 0x3162 }, /*                   Hangul_YI ㅢ HANGUL LETTER YI */
  { 0x0ed3, 0x3163 }, /*                    Hangul_I ㅣ HANGUL LETTER I */
  { 0x0ed4, 0x11a8 }, /*             Hangul_J_Kiyeog ᆨ HANGUL JONGSEONG KIYEOK */
  { 0x0ed5, 0x11a9 }, /*        Hangul_J_SsangKiyeog ᆩ HANGUL JONGSEONG SSANGKIYEOK */
  { 0x0ed6, 0x11aa }, /*         Hangul_J_KiyeogSios ᆪ HANGUL JONGSEONG KIYEOK-SIOS */
  { 0x0ed7, 0x11ab }, /*              Hangul_J_Nieun ᆫ HANGUL JONGSEONG NIEUN */
  { 0x0ed8, 0x11ac }, /*         Hangul_J_NieunJieuj ᆬ HANGUL JONGSEONG NIEUN-CIEUC */
  { 0x0ed9, 0x11ad }, /*         Hangul_J_NieunHieuh ᆭ HANGUL JONGSEONG NIEUN-HIEUH */
  { 0x0eda, 0x11ae }, /*             Hangul_J_Dikeud ᆮ HANGUL JONGSEONG TIKEUT */
  { 0x0edb, 0x11af }, /*              Hangul_J_Rieul ᆯ HANGUL JONGSEONG RIEUL */
  { 0x0edc, 0x11b0 }, /*        Hangul_J_RieulKiyeog ᆰ HANGUL JONGSEONG RIEUL-KIYEOK */
  { 0x0edd, 0x11b1 }, /*         Hangul_J_RieulMieum ᆱ HANGUL JONGSEONG RIEUL-MIEUM */
  { 0x0ede, 0x11b2 }, /*         Hangul_J_RieulPieub ᆲ HANGUL JONGSEONG RIEUL-PIEUP */
  { 0x0edf, 0x11b3 }, /*          Hangul_J_RieulSios ᆳ HANGUL JONGSEONG RIEUL-SIOS */
  { 0x0ee0, 0x11b4 }, /*         Hangul_J_RieulTieut ᆴ HANGUL JONGSEONG RIEUL-THIEUTH */
  { 0x0ee1, 0x11b5 }, /*        Hangul_J_RieulPhieuf ᆵ HANGUL JONGSEONG RIEUL-PHIEUPH */
  { 0x0ee2, 0x11b6 }, /*         Hangul_J_RieulHieuh ᆶ HANGUL JONGSEONG RIEUL-HIEUH */
  { 0x0ee3, 0x11b7 }, /*              Hangul_J_Mieum ᆷ HANGUL JONGSEONG MIEUM */
  { 0x0ee4, 0x11b8 }, /*              Hangul_J_Pieub ᆸ HANGUL JONGSEONG PIEUP */
  { 0x0ee5, 0x11b9 }, /*          Hangul_J_PieubSios ᆹ HANGUL JONGSEONG PIEUP-SIOS */
  { 0x0ee6, 0x11ba }, /*               Hangul_J_Sios ᆺ HANGUL JONGSEONG SIOS */
  { 0x0ee7, 0x11bb }, /*          Hangul_J_SsangSios ᆻ HANGUL JONGSEONG SSANGSIOS */
  { 0x0ee8, 0x11bc }, /*              Hangul_J_Ieung ᆼ HANGUL JONGSEONG IEUNG */
  { 0x0ee9, 0x11bd }, /*              Hangul_J_Jieuj ᆽ HANGUL JONGSEONG CIEUC */
  { 0x0eea, 0x11be }, /*              Hangul_J_Cieuc ᆾ HANGUL JONGSEONG CHIEUCH */
  { 0x0eeb, 0x11bf }, /*             Hangul_J_Khieuq ᆿ HANGUL JONGSEONG KHIEUKH */
  { 0x0eec, 0x11c0 }, /*              Hangul_J_Tieut ᇀ HANGUL JONGSEONG THIEUTH */
  { 0x0eed, 0x11c1 }, /*             Hangul_J_Phieuf ᇁ HANGUL JONGSEONG PHIEUPH */
  { 0x0eee, 0x11c2 }, /*              Hangul_J_Hieuh ᇂ HANGUL JONGSEONG HIEUH */
  { 0x0eef, 0x316d }, /*     Hangul_RieulYeorinHieuh ㅭ HANGUL LETTER RIEUL-YEORINHIEUH */
  { 0x0ef0, 0x3171 }, /*    Hangul_SunkyeongeumMieum ㅱ HANGUL LETTER KAPYEOUNMIEUM */
  { 0x0ef1, 0x3178 }, /*    Hangul_SunkyeongeumPieub ㅸ HANGUL LETTER KAPYEOUNPIEUP */
  { 0x0ef2, 0x317f }, /*              Hangul_PanSios ㅿ HANGUL LETTER PANSIOS */
 /*  0x0ef3                  Hangul_KkogjiDalrinIeung ? ??? */
  { 0x0ef4, 0x3184 }, /*   Hangul_SunkyeongeumPhieuf ㆄ HANGUL LETTER KAPYEOUNPHIEUPH */
  { 0x0ef5, 0x3186 }, /*          Hangul_YeorinHieuh ㆆ HANGUL LETTER YEORINHIEUH */
  { 0x0ef6, 0x318d }, /*                Hangul_AraeA ㆍ HANGUL LETTER ARAEA */
  { 0x0ef7, 0x318e }, /*               Hangul_AraeAE ㆎ HANGUL LETTER ARAEAE */
  { 0x0ef8, 0x11eb }, /*            Hangul_J_PanSios ᇫ HANGUL JONGSEONG PANSIOS */
 /*  0x0ef9                Hangul_J_KkogjiDalrinIeung ? ??? */
  { 0x0efa, 0x11f9 }, /*        Hangul_J_YeorinHieuh ᇹ HANGUL JONGSEONG YEORINHIEUH */
  { 0x0eff, 0x20a9 }, /*                  Korean_Won ₩ WON SIGN */
  { 0x13bc, 0x0152 }, /*                          OE Œ LATIN CAPITAL LIGATURE OE */
  { 0x13bd, 0x0153 }, /*                          oe œ LATIN SMALL LIGATURE OE */
  { 0x13be, 0x0178 }, /*                  Ydiaeresis Ÿ LATIN CAPITAL LETTER Y WITH DIAERESIS */
  { 0x20a0, 0x20a0 }, /*                     EcuSign ₠ EURO-CURRENCY SIGN */
  { 0x20a1, 0x20a1 }, /*                   ColonSign ₡ COLON SIGN */
  { 0x20a2, 0x20a2 }, /*                CruzeiroSign ₢ CRUZEIRO SIGN */
  { 0x20a3, 0x20a3 }, /*                  FFrancSign ₣ FRENCH FRANC SIGN */
  { 0x20a4, 0x20a4 }, /*                    LiraSign ₤ LIRA SIGN */
  { 0x20a5, 0x20a5 }, /*                    MillSign ₥ MILL SIGN */
  { 0x20a6, 0x20a6 }, /*                   NairaSign ₦ NAIRA SIGN */
  { 0x20a7, 0x20a7 }, /*                  PesetaSign ₧ PESETA SIGN */
  { 0x20a8, 0x20a8 }, /*                   RupeeSign ₨ RUPEE SIGN */
  { 0x20a9, 0x20a9 }, /*                     WonSign ₩ WON SIGN */
  { 0x20aa, 0x20aa }, /*               NewSheqelSign ₪ NEW SHEQEL SIGN */
  { 0x20ab, 0x20ab }, /*                    DongSign ₫ DONG SIGN */
  { 0x20ac, 0x20ac }, /*                    EuroSign € EURO SIGN */


  /* Following items added to GTK, not in the xterm table */

  /* Numeric keypad */
  
  { 0xFF80 /* Space */, ' ' },
  { 0xFFAA /* Multiply */, '*' },
  { 0xFFAB /* Add */, '+' },
  { 0xFFAC /* Separator */, ',' },
  { 0xFFAD /* Subtract */, '-' },
  { 0xFFAE /* Decimal */, '.' },
  { 0xFFAF /* Divide */, '/' },
  { 0xFFB0 /* 0 */, '0' },
  { 0xFFB1 /* 1 */, '1' },
  { 0xFFB2 /* 2 */, '2' },
  { 0xFFB3 /* 3 */, '3' },
  { 0xFFB4 /* 4 */, '4' },
  { 0xFFB5 /* 5 */, '5' },
  { 0xFFB6 /* 6 */, '6' },
  { 0xFFB7 /* 7 */, '7' },
  { 0xFFB8 /* 8 */, '8' },
  { 0xFFB9 /* 9 */, '9' },
  { 0xFFBD /* Equal */, '=' },  

  /* End numeric keypad */
 };


 uint32_t ma_keyval_to_unicode(uint32_t keyval) // guint32 gdk_keyval_to_unicode(guint keyval)
 {
  int min = 0;
  int max = MA_N_ELEMENTS (gdk_keysym_to_unicode_tab) - 1;
  int mid;

  /* First check for Latin-1 characters (1:1 mapping) */
  if ((keyval >= 0x0020 && keyval <= 0x007e) ||
      (keyval >= 0x00a0 && keyval <= 0x00ff))
    return keyval;

  /* Also check for directly encoded 24-bit UCS characters:
   */
  if ((keyval & 0xff000000) == 0x01000000)
    return keyval & 0x00ffffff;

  /* binary search in table */
  while (max >= min) {
    mid = (min + max) / 2;
    if (gdk_keysym_to_unicode_tab[mid].keysym < keyval)
      min = mid + 1;
    else if (gdk_keysym_to_unicode_tab[mid].keysym > keyval)
      max = mid - 1;
    else {
      /* found it */
      return gdk_keysym_to_unicode_tab[mid].ucs;
    }
  }
  
  /* No matching Unicode value found */
  return 0;

 } // ma_keyval_to_unicode


 int ma_unichar_to_utf8(uint32_t c, char *outbuf) // int g_unichar_to_utf8(gunichar c, gchar *outbuf);
 {
  uint32_t len = 0;    
  int first;
  int i;

  if (c < 0x80)
    {
      first = 0;
      len = 1;
    }
  else if (c < 0x800)
    {
      first = 0xc0;
      len = 2;
    }
  else if (c < 0x10000)
    {
      first = 0xe0;
      len = 3;
    }
   else if (c < 0x200000)
    {
      first = 0xf0;
      len = 4;
    }
  else if (c < 0x4000000)
    {
      first = 0xf8;
      len = 5;
    }
  else
    {
      first = 0xfc;
      len = 6;
    }

  if (outbuf)
    {
      for (i = len - 1; i > 0; --i)
	{
	  outbuf[i] = (c & 0x3f) | 0x80;
	  c >>= 6;
	}
      outbuf[0] = c | first;
    }

  return len;

 } // ma_unichar_to_utf8
#endif // defined(MAGUINONE) && defined(__MACUNIX__)


#if defined(__MAWIN__)
  /*
  inline long int malrint (double flt)
  {
	int intgr;

	_asm
	{
		fld flt
		fistp intgr
	};

	return intgr ;
  }
  */


  long int malrintf (float flt)
  {
	int intgr;
	_asm
	{
		fld flt
		fistp intgr
	};

	return intgr ;
  }
#endif


#ifdef __cplusplus
 }
#endif

// eof
